/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "window.h"
#include "graphics.h"
#include "gui.h"

#include "graphics.inl"

namespace magnum { namespace sample {
using magnum::application::Message;
using magnum::freetype::Descriptor;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Gui::Gui(GraphicsView* view,magnum::graphics3d::Context* context) : magnum::gui3d::Root(context) {
	m_Graphics = view;
	
	m_Constraint.setDirection(Vector3<float>(0,0,1),Vector2<float>(0.8f,0.2f));
	m_Constraint.setUp(Vector3<float>(0,1,0),0.4f);

	// Setup controllers
	m_Trackball.attachNext(&m_Fixed);
	//m_Trackball.attachConstraint(&m_Constraint);
	m_Fixed.identity();
	m_Fixed.setTranslation(Vector3<float>(0.0f, 0.0f, 400.0f));
	
	// Construct Navigator
	m_Navigator.attachController(&m_Trackball);
	m_Navigator.update();
	m_MouseDispatcher.insert(&m_Trackball);
	m_MouseDispatcher.fireArea(Rectangle<float>(Point<float>(0.0f,0.0f),getSize()));

	// Create font	
	FontFactory ffactory;
	Descriptor desc;
	desc.setFamily("Accidental Presidency");
	desc.setSize(24);
	m_Font = ffactory.create(desc);

	AutoPointer<Button> button = new Button(Rectangle<float>(10,10,170,40), Message(m_Graphics->getWindow(), MainWindow::LoadModel), "Load model");
	button->setFont(m_Font);
	insert(button);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Gui::~Gui() {
	m_Navigator.detachController();
	m_Trackball.detachNext();
	m_Trackball.detachConstraint();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Gui::onMouseMove(const Point<float>& pt, Mouse::Button but) {
	invalidate();
	m_Mouse = pt;
	if (Root::onMouseMove(pt, but))
		return true;

	m_MouseDispatcher.fireMove(pt,but);
	m_Navigator.update();

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Gui::onMouseButton(const Point<float>& p,Mouse::Button b) {
	invalidate();
	if(Root::onMouseButton(p, b))
		return true;
		
	m_MouseDispatcher.fireButton(p,b);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Gui::onMouseWheel(float v,Mouse::Button ) {
	m_MouseDispatcher.fireWheel(v);
	invalidate();
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Gui::onResize(const Dimension<float>& d) {
	Root::onResize(d);
	m_MouseDispatcher.fireArea(Rectangle<float>(Point<float>(0.0f,0.0f),d));
	return true;
}

}; };
