uniform samplerCube tex0;
varying vec3 transformedNormal;
varying vec4 ecPosition;

vec4 Ambient;
vec4 Diffuse;
vec4 Specular;

void pointLight() {
    float nDotVP;         // normal . light direction
    float nDotHV;         // normal . light half vector
    float pf;                   // power factor
    float d;                    // distance from surface to light source
    vec3  VP;               // direction from surface to light position
    vec3  halfVector;    // direction of maximum highlights
    vec3 ecPosition3;
    vec3 eye;

    ecPosition3 = (vec3 (ecPosition)) / ecPosition.w;
    eye = vec3 (0.0, 0.0, 1.0);

    // Compute vector from surface to light position
    VP = vec3 (gl_LightSource[0].position) - ecPosition3;

    // Normalize the vector from surface to light position
    VP = normalize(VP);

    halfVector = normalize(VP + eye);
    nDotVP = max(0.0, dot(transformedNormal, VP));
    nDotHV = max(0.0, dot(transformedNormal, halfVector));

    if (nDotVP == 0.0)
        pf = 0.0;
    else
        pf = pow(nDotHV, gl_FrontMaterial.shininess);

    Ambient  = gl_LightSource[0].ambient;
    Diffuse  = gl_LightSource[0].diffuse * nDotVP;
    Specular = gl_LightSource[0].specular * pf;
}


void main (void) {
    vec4 color;
    vec4 texture;

    pointLight();

    texture = textureCube(tex0, gl_TexCoord[0].xyz);
    color = gl_FrontLightModelProduct.sceneColor +
                Ambient  * gl_FrontMaterial.ambient +
                Diffuse  * gl_FrontMaterial.diffuse;
    color += Specular * gl_FrontMaterial.specular;
    color.rgb += texture.rgb;
    color = clamp(color, 0.0, 1.0);

    gl_FragColor = color;
}
