/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics.h"
#include "gui.h"


namespace magnum { namespace sample {
using magnum::graphics3d::Context;
using magnum::graphics::Colour;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
GraphicsView::GraphicsView(const Rectangle<float>& rect) : OpenGL(rect) {
	setColourBits(24);
	setAlphaBits(8);
	setDepthBits(24);
	setStencilBits(8);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onAttachWindow() {
	OpenGL::onAttachWindow();

	m_Context = Context::getCurrent();
	m_Context->setCulling(Context::CullBack);

	m_MatrixStack = m_Context->getMatrixStack();

	m_Context->setClearColour(Colour<float>(0,0,0));
	m_Context->setViewport(getBounds());
	
	m_Gui = new Gui(m_Context);
	m_Gui->attachDirectInput();
	
	m_Timer.init(0);
	
	m_Camera.setNearPlane(10);
	m_Camera.setFarPlane(800);
	float w = getSize().getWidth();
	float h = getSize().getHeight();
	float f = max(w,h);
	w = w / f * 10.0f;
	h = h / f * 10.0f;
	m_Camera.setProjectionPlane(10.0f);
	m_Camera.setViewWindow(Rectangle<float>(-0.5f*w, -0.5f*h, w, h));

	return onInit();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onDetachWindow() {
	onExit();
	m_Gui->release();
	return OpenGL::onDetachWindow();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onResize(const Rectangle<float>& r) {
	OpenGL::onResize(r);
	float w = r.getWidth();
	float h = r.getHeight();
	float f = max(w,h);
	w = w / f * 10.0f;
	h = h / f * 10.0f;
	m_Camera.setProjectionPlane(10.0f);
	m_Camera.setViewWindow(Rectangle<float>(-0.5f*w, -0.5f*h, w, h));
	return true;
}

}; };
