/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SAMPLE_GRAPHICS_H
#define __SAMPLE_GRAPHICS_H

#include "magnum.application.h"
#include "magnum.gui.h"
#include "magnum.graphics3d.h"
#include "magnum.graphics.h"
#include "magnum.math.h"
#include "magnum.geometry.h"
#include "magnum.system.h"

namespace magnum { namespace sample {
using namespace magnum::application;
using magnum::gui::Window;
using magnum::gui::Bitmap;
using magnum::graphics3d::Context;
using magnum::graphics3d::FrameBuffer;
using magnum::graphics3d::MatrixStack;
using magnum::graphics3d::LightStack;
using magnum::graphics3d::TextureStack;
using magnum::graphics3d::TextureManager;
using magnum::graphics3d::PerspectiveCamera;
using magnum::graphics::Point;
using magnum::math::Vector3;
using magnum::geometry::Transform3;
using magnum::system::Timer;

class Gui;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class GraphicsView : public magnum::gui::OpenGL {
protected:
	Context*			m_Context;
	MatrixStack*		m_MatrixStack;
	TextureManager*		m_TextureManager;
	Gui*				m_Gui;
	Timer				m_Timer;
	PerspectiveCamera	m_Camera;

public:
	GraphicsView(const Rectangle<float>& rect);
	
	inline void			setTranslation(const Vector3<float>& );
	inline Vector3<float> getTranslation() const;
	inline const Point<float>& getRotation() const;
	inline const Point<float>& getMouse() const;
	inline const Transform3<float>& getTransform() const;
	
	inline Context*		getContext() const;
	inline MatrixStack*	getMatrixStack() const;
	inline Gui*			getGui() const;
	inline const Timer*	getTimer() const;

protected:
	virtual bool		onAttachWindow();
	virtual bool		onDetachWindow();
	virtual bool		onResize(const Rectangle<float>& );
	
	virtual bool		onHandleMessage(const Message& );
	
	virtual void		onRender();
	virtual bool		onInit();
	virtual bool		onExit();
};


}; };

#endif
