/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __SAMPLE_GUI_H
#define __SAMPLE_GUI_H

#include "magnum.gui3d.h"
#include "magnum.input.h"
#include "magnum.input3d.h"
#include "magnum.math.h"
#include "magnum.freetype.h"
#include "magnum.graphics.h"
#include "magnum.geometry.h"

namespace magnum { namespace sample {
using magnum::gui3d::Button;
using magnum::gui3d::FontMap;
using magnum::gui3d::StringView;
using magnum::gui3d::View;
using magnum::input3d::Navigator;
using magnum::input3d::TrackballController;
using magnum::input3d::FlightController;
using magnum::input3d::FixedController;
using magnum::freetype::Font;
using magnum::freetype::FontFactory;
using magnum::graphics::Point;
using magnum::graphics::Dimension;
using magnum::graphics::Rectangle;
using magnum::math::Vector3;
using magnum::math::Vector2;
using magnum::input::Mouse;
using magnum::input::MouseListener;
using magnum::input::MouseDispatcher;
using magnum::input::Keyboard;
using magnum::input::KeyboardListener;
using magnum::input::KeyboardDispatcher;
using magnum::geometry::Transform3;
using magnum::geometry::DirectionBoxConstraint3;
using magnum::geometry::DirectionConeConstraint3;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Gui : public magnum::gui3d::Root {
DeclareClass(Gui, Root);
private:
	Font*				m_Font;
	Button*				m_EnableGuiButton;
	Point<float>		m_Mouse;
	Navigator			m_Navigator;
	TrackballController	m_Trackball;
	FlightController	m_Flyer;
	FixedController		m_Fixed;
	MouseDispatcher		m_MouseDispatcher;
	KeyboardDispatcher	m_KeyboardDispatcher;
	DirectionBoxConstraint3<float> m_Constraint;
	
public:
	Gui(magnum::graphics3d::Context* );
	~Gui();
	
	inline const Point<float>& getMouse() const;
	inline const Transform3<float>& getTransform() const;
	
	inline Vector3<float> getTranslation() const;
	inline void			setTranslation(const Vector3<float>& );
	
	inline float		getMotionSpeed() const;
	inline void			setMotionSpeed(float );
	
	inline Font*		getFont() const;

protected:
	virtual bool		onMouseMove(const Point<float>& ,Mouse::Button );
	virtual bool		onMouseButton(const Point<float>& ,Mouse::Button );
	virtual bool		onResize(const Dimension<float>& );
	virtual bool		onMouseWheel(float ,Mouse::Button );
};

}; };

#endif
