/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __ALGORITHM_FUNCTION_INL
#define __ALGORITHM_FUNCTION_INL

namespace magnum { namespace algorithm {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class C, class F> inline void foreach(C& container, F& function) {
	typename C::Iterator;
	C::Iterator end = container.end();
	C::Iterator iter = container.begin();
	while(iter != end) {
		function(*iter);
		iter++;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class I, class F> inline void foreach(const I& begin, const I& end, F& function) {
	I iter = begin;
	while(iter != end) {
		function(*iter);
		iter++;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class C, class G> inline void generate(C& container, G& generator) {
	typename C::Iterator;
	C::Iterator end = container.end();
	C::Iterator iter = container.begin();
	while(iter != end) {
		*iter = generator();
		iter++;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class I, class G> inline void generate(const I& begin, const I& end, G& generator) {
	I iter = begin;
	while(iter != end) {
		*iter = generator();
		iter++;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class C, class T> inline void count(const C& container, const T& item) {
	typename C::Iterator;
	C::Iterator end = container.end();
	C::Iterator iter = container.begin();
	unsigned int n = 0;
	while(iter != end) {
		if (*iter == item)
			n++;
		iter++;
	}
	return n;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class I, class T> inline unsigned int count(const I& begin, const I& end, const T& item) {
	unsigned int n = 0;
	I iter = begin;
	while(iter != end) {
		if (*iter == item)
			n++;
		iter++;
	}
	return n;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class I, class J> inline J copy(const I& begin, const I& end, const J& result) {
	I in = begin;
	J out = result;
	while(in != end) {
		*out = *in;
		out++;
		in++;
	}
	return out;
}


}; };

#endif

