/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __APPLICATION_APPLICATION_H
#define __APPLICATION_APPLICATION_H

#include "application.messenger.h"
#include "application.messagequeue.h"
#include "system.mutex.h"
#include "system.signal.h"

namespace magnum { namespace application {
using magnum::object::Object;
using magnum::types::String;
using magnum::system::Mutex;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Application : public Messenger {
DeclareClass(Application, Messenger);
public:
	enum {
		QuitMessage		= 0x80000001,
	};
	
public:
	Application();
	~Application();
	
	bool				run();
	bool				quit();
	
	inline static Application* getInstance();
	
	static const char*	getCommandLine();
	static const char*	getEnvironment(const char* );
	static bool			setEnvironment(const char* ,const char* );
	
protected:
	virtual bool		onRun();
	virtual bool		onQuit();
	virtual bool		onHandleMessage(const Message& );

private:
	virtual bool		onSendMessage(Message* );
	virtual bool		onPostMessage(Message* );
	virtual bool		onSynchronize();

private:
	MessageQueue		m_MessageQueue;
	Mutex				m_MessageMutex;
	system::Signal		m_MessageSignal;
	static Application*	g_Application;
};

}; };

#endif
