/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __APPLICATION_LISTENER_H
#define __APPLICATION_LISTENER_H

#include "object.h"
#include "container.list.h"

namespace magnum { namespace application {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::List;

template<class T> class Listener;
template<class T> class Dispatcher;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Listener {
friend class Dispatcher<T>;
private:
	List<Dispatcher<T>*>	m_Lists;

public:
	inline ~Listener();
};



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Dispatcher : public Object, private magnum::object::Noncopyable {
DeclareClass(Dispatcher, Object);
friend class Listener<T>;
private:
	List<T*>			m_Listeners;

public:
	typedef typename magnum::container::List<T*>::Iterator	Iterator;

public:
	~Dispatcher();

	inline void			insert(T *listener);
	inline bool			contains(T* listener) const;
	inline void			remove(T *listener);
	void				clear();
	
	inline unsigned int	getSize() const;
	inline bool			isEmpty() const;

	inline Iterator		begin() const;
	inline Iterator		end() const;

	inline void			fire(void (T::*)()) const;
	template<class P1>
	inline void			fire(void (T::*)(P1 ),const P1& ) const;
	template<class P1, class P2>
	inline void			fire(void (T::*)(P1 ,P2 ),const P1& ,const P2& ) const;
	template<class P1, class P2, class P3>
	inline void			fire(void (T::*)(P1 ,P2 ,P3 ),const P1& ,const P2& ,const P3& ) const;
};

}; };

#endif
