/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __APPLICATION_LISTENER_INL
#define __APPLICATION_LISTENER_INL

namespace magnum { namespace application {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Listener<T>::~Listener() {
	List<Dispatcher<T> *>::Iterator iter = m_Lists.begin();
	List<Dispatcher<T> *>::Iterator end = m_Lists.end();

	while (iter != end) {
		(*iter)->m_Listeners.remove((T*)this);
		++iter;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Dispatcher<T>::clear() {
	Iterator iter;
	for (iter = m_Listeners.begin(); iter != m_Listeners.end(); ++iter)
		((Listener<T>*)(*iter))->m_Lists.remove(this);
		
	m_Listeners.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Dispatcher<T>::~Dispatcher() {
	clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline bool	Dispatcher<T>::contains(T *listener) const {
	return m_Listeners.contains(listener);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void	Dispatcher<T>::insert(T *listener) {
	if (m_Listeners.contains(listener))
		return;
	m_Listeners.insert(listener);
	((Listener<T>*)listener)->m_Lists.insert(this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void	Dispatcher<T>::remove(T *listener) {
	m_Listeners.remove(listener);
	((Listener<T>*)listener)->m_Lists.remove(this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline unsigned int	Dispatcher<T>::getSize() const {
	return m_Listeners.getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline bool Dispatcher<T>::isEmpty() const {
	return m_Listeners.isEmpty();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline typename Dispatcher<T>::Iterator Dispatcher<T>::begin() const {
	return m_Listeners.begin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline typename Dispatcher<T>::Iterator Dispatcher<T>::end() const {
	return m_Listeners.end();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void Dispatcher<T>::fire(void (T::*fn)()) const {
	for (Iterator iter = m_Listeners.begin(); iter != m_Listeners.end(); ++iter) {
		((*iter)->*fn)();
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> template<class P1>
inline void Dispatcher<T>::fire(void (T::*fn)(P1 ),const P1& p1) const {
	for (Iterator iter = m_Listeners.begin(); iter != m_Listeners.end(); ++iter) {
		((*iter)->*fn)(p1);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> template<class P1, class P2>
inline void Dispatcher<T>::fire(void (T::*fn)(P1 ,P2 ),const P1& p1, const P2& p2) const {
	for (Iterator iter = m_Listeners.begin(); iter != m_Listeners.end(); ++iter) {
		((*iter)->*fn)(p1,p2);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> template<class P1, class P2, class P3>
inline void Dispatcher<T>::fire(void (T::*fn)(P1 ,P2 ,P3 ),const P1& p1, const P2& p2, const P3& p3) const {
	for (Iterator iter = m_Listeners.begin(); iter != m_Listeners.end(); ++iter) {
		((*iter)->*fn)(p1,p2,p3);
	}
}

}; };

#endif
