/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __APPLICATION_OBSERVER_INL
#define __APPLICATION_OBSERVER_INL

namespace magnum { namespace application {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Observer<T>::~Observer() {
	for (List<Observable<T>*>::Iterator iter = m_Observables.begin(); iter != m_Observables.end(); ++iter)
		(*iter)->m_Observers.remove((T*)this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Observer<T>::subscribe(T* o) {
	ASSERT(o != NULL);
	if (m_Observables.contains(o))
		return;
	m_Observables.insert(o);
	o->m_Observers.insert(this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Observer<T>::unsubscribe(T* o) {
	ASSERT(o != NULL);
	m_Observables.remove(o);
	o->m_Observers.remove(this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Observer<T>::unsubscribe() {
	for (List<Observable<T>*>::Iterator iter = m_Observables.begin(); iter != m_Observables.end(); ++iter)
		(*iter)->m_Observers.remove((T*)this);
	m_Observables.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Observer<T>::onNotify(T* ) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Observable<T>::~Observable() {
	for (List<Observer<T>*>::Iterator iter = m_Observers.begin(); iter != m_Observers.end(); ++iter)
		(*iter)->m_Observables.remove(this);
}
	

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Observable<T>::notifyObservers() {
	for (List<Observer<T>*>::Iterator iter = m_Observers.begin(); iter != m_Observers.end(); ++iter)
		(*iter)->onNotify((T*)this);
}

}; };

#endif
