/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __APPLICATION_PERFORMER_H
#define __APPLICATION_PERFORMER_H

#include "object.pointer.h"
#include "system.barrier.h"
#include "system.thread.h"
#include "system.mutex.h"
#include "system.signal.h"
#include "container.chain.h"

namespace magnum { namespace application {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::object::Noncopyable;
using magnum::system::Barrier;
using magnum::system::Mutex;
using magnum::system::Thread;
using magnum::types::String;
using magnum::container::Chain;

class Task;
class Performer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class PerformerThread : public Thread {
DeclareClass(PerformerThread, Thread);
friend class Performer;
private:
	explicit inline PerformerThread(Performer* );
	
public:
	inline Performer*	getPerformer() const;
	
protected:
	virtual bool		run();	
	
private:
	Performer*			m_Performer;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Performer : public Object, public Noncopyable {
DeclareClass(Performer, Object);
friend class Task;
public:
	typedef Chain<Task>::Iterator	Iterator;
	
public:
	Performer();
	~Performer();
	
	bool				insert(Task* );
	void				remove(Task* );
	void				clear();
	
	bool				loop();
	bool				start();
	void				stop();
	bool				step();
	void				join();
	void				suspend();
	void				resume();
	
	inline bool			isSuspended() const;
	inline bool			isStarted() const;
	
	inline Iterator		begin() const;
	inline Iterator		at(const Task* ) const;
	inline Iterator		end() const;
	
	inline Thread*		getThread() const;
	
protected:
	virtual void		onStart();
	virtual void		onLoop();
	virtual void		onEnd();	
	
private:
	Chain<Task>			m_Tasks;
	Mutex				m_ChainMutex;

	system::Signal		m_PauseSignal;
	system::Signal		m_StopSignal;
	Barrier				m_SyncBarrier;
	Barrier				m_Barrier;

	PerformerThread		m_PerformerThread;
	Thread*				m_Thread;
};

}; };

#endif
