/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __APPLICATION_SIGNAL_H
#define __APPLICATION_SIGNAL_H

#include "object.softpointer.h"
#include "container.chain.h"

namespace magnum { namespace application {
using magnum::object::Object;
using magnum::object::SoftPointer;
using magnum::types::String;
using magnum::container::Chain;
using magnum::container::Chainable;


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1=void, class A2=void, class A3=void, class A4=void>
class Signal {
public:
	inline Signal();
	inline Signal(const Signal& );
	inline ~Signal();

	template<class T>
	void			connect(T* ,void (T::*)(A1 ,A2, A3, A4 ));
	template<class T>
	void			disconnect(T* );
	void			disconnectAll();
	
	inline void		operator()(A1 ,A2 ,A3 ,A4 );
	void			emit(A1 ,A2 ,A3 ,A4 );
	
	inline bool		operator==(const Signal<A1,A2,A3,A4>& ) const;
	inline bool		operator!=(const Signal<A1,A2,A3,A4>& ) const;
	inline const Signal<A1,A2,A3,A4>& operator=(const Signal<A1,A2,A3,A4>& );
	
private:
	struct ConnectionBase : public Chainable<ConnectionBase> {
		virtual inline	~ConnectionBase();
		virtual void	emit(A1 ,A2, A3, A4 ) const = 0;
		virtual Object*	object() const = 0;
		
		inline void		refer();
		inline void		release();
	};

	template<class T>
	struct Connection : public ConnectionBase {
		inline Connection(T* ,void (T::*)(A1,A2,A3,A4));
		virtual void	emit(A1 ,A2, A3, A4 ) const;
		virtual T*		object() const;
	
		SoftPointer<T>	m_Object;
		void			(T::*m_Method)(A1 ,A2, A3, A4 );
	};

	Chain<ConnectionBase>	m_Connections;
};



/*--------------------------------------------------------------------------*/
/**
 */
template<>
class Signal<void, void, void, void> {
public:
	inline Signal();
	inline Signal(const Signal& );
	inline ~Signal();

	template<class T>
	void			connect(T* ,void (T::*)());
	template<class T>
	void			disconnect(T* );
	void			disconnectAll();
	
	inline void		operator()();
	void			emit();
	
	inline bool		operator==(const Signal<>& ) const;
	inline bool		operator!=(const Signal<>& ) const;
	inline const Signal<>& operator=(const Signal<>& );
	
private:
	struct ConnectionBase : public Chainable<ConnectionBase> {
		virtual inline	~ConnectionBase();
		virtual void	emit() const = 0;
		virtual Object*	object() const = 0;
		
		inline void		refer();
		inline void		release();
	};

	template<class T>
	struct Connection : public ConnectionBase {
		inline Connection(T* ,void (T::*)());
		virtual void	emit() const;
		virtual T*		object() const;
	
		SoftPointer<T>	m_Object;
		void			(T::*m_Method)();
	};

	Chain<ConnectionBase>	m_Connections;
};


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
class Signal<A1, void, void, void> {
public:
	inline Signal();
	inline Signal(const Signal& );
	inline ~Signal();

	template<class T>
	void			connect(T* ,void (T::*)(A1 ));
	template<class T>
	void			disconnect(T* );
	void			disconnectAll();
	
	inline void		operator()(A1 );
	void			emit(A1 );
	
	inline bool		operator==(const Signal<A1>& ) const;
	inline bool		operator!=(const Signal<A1>& ) const;
	inline const Signal<A1>& operator=(const Signal<A1>& );
	
private:
	struct ConnectionBase : public Chainable<ConnectionBase> {
		virtual inline	~ConnectionBase();
		virtual void	emit(A1 ) const = 0;
		virtual Object*	object() const = 0;
		
		inline void		refer();
		inline void		release();
	};

	template<class T>
	struct Connection : public ConnectionBase {
		inline Connection(T* ,void (T::*)(A1));
		virtual void	emit(A1 ) const;
		virtual T*		object() const;
	
		SoftPointer<T>	m_Object;
		void			(T::*m_Method)(A1 );
	};

	Chain<ConnectionBase>	m_Connections;
};


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1, class A2>
class Signal<A1, A2, void, void> {
public:
	inline Signal();
	inline Signal(const Signal& );
	inline ~Signal();

	template<class T>
	void			connect(T* ,void (T::*)(A1 ,A2 ));
	template<class T>
	void			disconnect(T* );
	void			disconnectAll();
	
	inline void		operator()(A1 ,A2 );
	void			emit(A1 ,A2 );
	
	inline bool		operator==(const Signal<A1,A2>& ) const;
	inline bool		operator!=(const Signal<A1,A2>& ) const;
	inline const Signal<A1,A2>& operator=(const Signal<A1,A2>& );
	
private:
	struct ConnectionBase : public Chainable<ConnectionBase> {
		virtual inline	~ConnectionBase();
		virtual void	emit(A1 ,A2 ) const = 0;
		virtual Object*	object() const = 0;
		
		inline void		refer();
		inline void		release();
	};

	template<class T>
	struct Connection : public ConnectionBase {
		inline Connection(T* ,void (T::*)(A1,A2));
		virtual void	emit(A1 ,A2 ) const;
		virtual T*		object() const;
	
		SoftPointer<T>	m_Object;
		void			(T::*m_Method)(A1 ,A2);
	};

	Chain<ConnectionBase>	m_Connections;
};


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1, class A2, class A3>
class Signal<A1, A2, A3, void> {
public:
	inline Signal();
	inline Signal(const Signal& );
	inline ~Signal();

	template<class T>
	void			connect(T* ,void (T::*)(A1 ,A2, A3 ));
	template<class T>
	void			disconnect(T* );
	void			disconnectAll();
	
	inline void		operator()(A1 ,A2 ,A3 );
	void			emit(A1 ,A2 ,A3 );
	
	inline bool		operator==(const Signal<A1,A2,A3>& ) const;
	inline bool		operator!=(const Signal<A1,A2,A3>& ) const;
	inline const Signal<A1,A2,A3>& operator=(const Signal<A1,A2,A3>& );
	
private:
	struct ConnectionBase : public Chainable<ConnectionBase> {
		virtual inline	~ConnectionBase();
		virtual void	emit(A1 ,A2, A3 ) const = 0;
		virtual Object*	object() const = 0;
		
		inline void		refer();
		inline void		release();
	};

	template<class T>
	struct Connection : public ConnectionBase {
		inline Connection(T* ,void (T::*)(A1,A2,A3));
		virtual void	emit(A1 ,A2, A3 ) const;
		virtual T*		object() const;
	
		SoftPointer<T>	m_Object;
		void			(T::*m_Method)(A1 ,A2, A3 );
	};

	Chain<ConnectionBase>	m_Connections;
};


}; };

#endif
