/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __APPLICATION_SIGNAL_INL
#define __APPLICATION_SIGNAL_INL

namespace magnum { namespace application {

/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
inline Signal<A1,A2,A3,A4>::Signal() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
inline Signal<A1,A2,A3,A4>::Signal(const Signal<A1,A2,A3,A4>& ) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
inline Signal<A1,A2,A3,A4>::~Signal() {
	disconnectAll();
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
template<class T>
void Signal<A1,A2,A3,A4>::connect(T* obj,void (T::*method)(A1 ,A2, A3, A4 )) {
	ASSERT(obj != NULL);
	Connection<T>* con = new Connection<T>(obj, method);
	m_Connections.insert(con);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
template<class T>
void Signal<A1,A2,A3,A4>::disconnect(T* t) {
	ASSERT(t != NULL);
	Chain<ConnectionBase>::Iterator iter = m_Connections.begin();
	while (iter != m_Connections.end()) {
		if (iter->object() != t) {
			++iter;
			continue;
		}
		Connection<T>* con = static_cast<Connection<T>*>(&*iter);
		++iter;
		m_Connections.remove(con);
		delete con;
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
void Signal<A1,A2,A3,A4>::disconnectAll() {
	while (m_Connections.begin() != m_Connections.end()) {
		ConnectionBase* con = m_Connections.begin();
		m_Connections.remove(m_Connections.begin());
		delete con;
	}
}



/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
void Signal<A1,A2,A3,A4>::emit(A1 a1,A2 a2,A3 a3,A4 a4) {
	Chain<ConnectionBase>::Iterator iter = m_Connections.begin();
	while( iter != m_Connections.end()) {
		ConnectionBase* con = iter;
		++iter;
		con->emit(a1,a2,a3,a4);
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
inline void Signal<A1,A2,A3,A4>::operator()(A1 a1,A2 a2,A3 a3,A4 a4) {
	emit(a1,a2,a3,a4);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
inline bool Signal<A1,A2,A3,A4>::operator==(const Signal<A1,A2,A3,A4>& ) const {
	return true;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
inline bool Signal<A1,A2,A3,A4>::operator!=(const Signal<A1,A2,A3,A4>& ) const {
	return false;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
inline const Signal<A1,A2,A3,A4>& Signal<A1,A2,A3,A4>::operator=(const Signal<A1,A2,A3,A4>& ) {
	return *this;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
inline void Signal<A1,A2,A3,A4>::ConnectionBase::refer() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
inline Signal<A1,A2,A3,A4>::ConnectionBase::~ConnectionBase() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
inline void Signal<A1,A2,A3,A4>::ConnectionBase::release() {
}

	
/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
template<class T>
inline Signal<A1,A2,A3,A4>::Connection<T>::Connection(T* obj,void (T::*method)(A1,A2,A3,A4))
	: m_Object(obj), m_Method(method) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
template<class T>
void Signal<A1,A2,A3,A4>::Connection<T>::emit(A1 a1,A2 a2,A3 a3,A4 a4) const {
	if (m_Object == NULL) {
		delete this;
		return;
	}
	(m_Object->*m_Method)(a1,a2,a3,a4);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3,class A4>
template<class T>
T* Signal<A1,A2,A3,A4>::Connection<T>::object() const {
	return m_Object;
}






/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
inline Signal<A1,A2,A3,void>::Signal() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
inline Signal<A1,A2,A3,void>::Signal(const Signal<A1,A2,A3,void>& ) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
inline Signal<A1,A2,A3,void>::~Signal() {
	disconnectAll();
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
template<class T>
void Signal<A1,A2,A3,void>::connect(T* obj,void (T::*method)(A1 ,A2, A3 )) {
	ASSERT(obj != NULL);
	Connection<T>* con = new Connection<T>(obj, method);
	m_Connections.insert(con);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
template<class T>
void Signal<A1,A2,A3,void>::disconnect(T* t) {
	ASSERT(t != NULL);
	Chain<ConnectionBase>::Iterator iter = m_Connections.begin();
	while (iter != m_Connections.end()) {
		if (iter->object() != t) {
			++iter;
			continue;
		}
		Connection<T>* con = static_cast<Connection<T>*>(&*iter);
		++iter;
		m_Connections.remove(con);
		delete con;
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
void Signal<A1,A2,A3,void>::disconnectAll() {
	while (m_Connections.begin() != m_Connections.end()) {
		ConnectionBase* con = m_Connections.begin();
		m_Connections.remove(m_Connections.begin());
		delete con;
	}
}



/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
void Signal<A1,A2,A3,void>::emit(A1 a1,A2 a2,A3 a3) {
	Chain<ConnectionBase>::Iterator iter = m_Connections.begin();
	while( iter != m_Connections.end()) {
		ConnectionBase* con = iter;
		++iter;
		con->emit(a1,a2,a3);
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
inline void Signal<A1,A2,A3,void>::operator()(A1 a1,A2 a2,A3 a3) {
	emit(a1,a2,a3);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
inline bool Signal<A1,A2,A3,void>::operator==(const Signal<A1,A2,A3,void>& ) const {
	return true;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
inline bool Signal<A1,A2,A3,void>::operator!=(const Signal<A1,A2,A3,void>& ) const {
	return false;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
inline const Signal<A1,A2,A3,void>& Signal<A1,A2,A3,void>::operator=(const Signal<A1,A2,A3,void>& ) {
	return *this;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
inline void Signal<A1,A2,A3,void>::ConnectionBase::refer() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
inline Signal<A1,A2,A3,void>::ConnectionBase::~ConnectionBase() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
inline void Signal<A1,A2,A3,void>::ConnectionBase::release() {
}

	
/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
template<class T>
inline Signal<A1,A2,A3,void>::Connection<T>::Connection(T* obj,void (T::*method)(A1,A2,A3))
	: m_Object(obj), m_Method(method) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
template<class T>
void Signal<A1,A2,A3,void>::Connection<T>::emit(A1 a1,A2 a2,A3 a3) const {
	if (m_Object == NULL) {
		delete this;
		return;
	}
	(m_Object->*m_Method)(a1,a2,a3);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2,class A3>
template<class T>
T* Signal<A1,A2,A3,void>::Connection<T>::object() const {
	return m_Object;
}










/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
inline Signal<A1,A2,void,void>::Signal() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
inline Signal<A1,A2,void,void>::Signal(const Signal<A1,A2,void,void>& ) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
inline Signal<A1,A2,void,void>::~Signal() {
	disconnectAll();
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
template<class T>
void Signal<A1,A2,void,void>::connect(T* obj,void (T::*method)(A1 ,A2 )) {
	ASSERT(obj != NULL);
	Connection<T>* con = new Connection<T>(obj, method);
	m_Connections.insert(con);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
template<class T>
void Signal<A1,A2,void,void>::disconnect(T* t) {
	ASSERT(t != NULL);
	Chain<ConnectionBase>::Iterator iter = m_Connections.begin();
	while (iter != m_Connections.end()) {
		if (iter->object() != t) {
			++iter;
			continue;
		}
		Connection<T>* con = static_cast<Connection<T>*>(&*iter);
		++iter;
		m_Connections.remove(con);
		delete con;
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
void Signal<A1,A2,void,void>::disconnectAll() {
	while (m_Connections.begin() != m_Connections.end()) {
		ConnectionBase* con = m_Connections.begin();
		m_Connections.remove(m_Connections.begin());
		delete con;
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
void Signal<A1,A2,void,void>::emit(A1 a1,A2 a2) {
	Chain<ConnectionBase>::Iterator iter = m_Connections.begin();
	while( iter != m_Connections.end()) {
		ConnectionBase* con = iter;
		++iter;
		con->emit(a1,a2);
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
inline void Signal<A1,A2,void,void>::operator()(A1 a1,A2 a2) {
	emit(a1,a2);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
inline bool Signal<A1,A2,void,void>::operator==(const Signal<A1,A2,void,void>& ) const {
	return true;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
inline bool Signal<A1,A2,void,void>::operator!=(const Signal<A1,A2,void,void>& ) const {
	return false;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
inline const Signal<A1,A2,void,void>& Signal<A1,A2,void,void>::operator=(const Signal<A1,A2,void,void>& ) {
	return *this;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
inline void Signal<A1,A2,void,void>::ConnectionBase::refer() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
inline Signal<A1,A2,void,void>::ConnectionBase::~ConnectionBase() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
inline void Signal<A1,A2,void,void>::ConnectionBase::release() {
}

	
/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
template<class T>
inline Signal<A1,A2,void,void>::Connection<T>::Connection(T* obj,void (T::*method)(A1,A2))
	: m_Object(obj), m_Method(method) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
template<class T>
void Signal<A1,A2,void,void>::Connection<T>::emit(A1 a1,A2 a2) const {
	if (m_Object == NULL) {
		delete this;
		return;
	}
	(m_Object->*m_Method)(a1,a2);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1,class A2>
template<class T>
T* Signal<A1,A2,void,void>::Connection<T>::object() const {
	return m_Object;
}







/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
inline Signal<A1,void,void,void>::Signal() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
inline Signal<A1,void,void,void>::Signal(const Signal<A1,void,void,void>& ) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
inline Signal<A1,void,void,void>::~Signal() {
	disconnectAll();
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
template<class T>
void Signal<A1,void,void,void>::connect(T* obj,void (T::*method)(A1 )) {
	ASSERT(obj != NULL);
	Connection<T>* con = new Connection<T>(obj, method);
	m_Connections.insert(con);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
template<class T>
void Signal<A1,void,void,void>::disconnect(T* t) {
	ASSERT(t != NULL);
	Chain<ConnectionBase>::Iterator iter = m_Connections.begin();
	while (iter != m_Connections.end()) {
		if (iter->object() != t) {
			++iter;
			continue;
		}
		Connection<T>* con = static_cast<Connection<T>*>(&*iter);
		++iter;
		m_Connections.remove(con);
		delete con;
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
void Signal<A1,void,void,void>::disconnectAll() {
	while (m_Connections.begin() != m_Connections.end()) {
		ConnectionBase* con = m_Connections.begin();
		m_Connections.remove(m_Connections.begin());
		delete con;
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
void Signal<A1,void,void,void>::emit(A1 a1) {
	Chain<ConnectionBase>::Iterator iter = m_Connections.begin();
	while( iter != m_Connections.end()) {
		ConnectionBase* con = iter;
		++iter;
		con->emit(a1);
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
inline void Signal<A1,void,void,void>::operator()(A1 a1) {
	emit(a1);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
inline bool Signal<A1,void,void,void>::operator==(const Signal<A1,void,void,void>& ) const {
	return true;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
inline bool Signal<A1,void,void,void>::operator!=(const Signal<A1,void,void,void>& ) const {
	return false;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
inline const Signal<A1,void,void,void>& Signal<A1,void,void,void>::operator=(const Signal<A1,void,void,void>& ) {
	return *this;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
inline void Signal<A1,void,void,void>::ConnectionBase::refer() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
inline Signal<A1,void,void,void>::ConnectionBase::~ConnectionBase() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
inline void Signal<A1,void,void,void>::ConnectionBase::release() {
}

	
/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
template<class T>
inline Signal<A1,void,void,void>::Connection<T>::Connection(T* obj,void (T::*method)(A1))
	: m_Object(obj), m_Method(method) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
template<class T>
void Signal<A1,void,void,void>::Connection<T>::emit(A1 a1) const {
	if (m_Object == NULL) {
		delete this;
		return;
	}
	(m_Object->*m_Method)(a1);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class A1>
template<class T>
T* Signal<A1,void,void,void>::Connection<T>::object() const {
	return m_Object;
}











/*--------------------------------------------------------------------------*/
/**
 */
inline Signal<void,void,void,void>::Signal() {
}


/*--------------------------------------------------------------------------*/
/**
 */
inline Signal<void,void,void,void>::Signal(const Signal<void,void,void,void>& ) {
}


/*--------------------------------------------------------------------------*/
/**
 */
inline Signal<void,void,void,void>::~Signal() {
	disconnectAll();
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
void Signal<void,void,void,void>::connect(T* obj,void (T::*method)()) {
	ASSERT(obj != NULL);
	Connection<T>* con = new Connection<T>(obj, method);
	m_Connections.insert(con);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
void Signal<void,void,void,void>::disconnect(T* t) {
	ASSERT(t != NULL);
	Chain<ConnectionBase>::Iterator iter = m_Connections.begin();
	while (iter != m_Connections.end()) {
		if (iter->object() != t) {
			++iter;
			continue;
		}
		Connection<T>* con = static_cast<Connection<T>*>(&*iter);
		++iter;
		m_Connections.remove(con);
		delete con;
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
inline void Signal<void,void,void,void>::operator()() {
	emit();
}


/*--------------------------------------------------------------------------*/
/**
 */
inline bool Signal<void,void,void,void>::operator==(const Signal<void,void,void,void>& ) const {
	return true;
}


/*--------------------------------------------------------------------------*/
/**
 */
inline bool Signal<void,void,void,void>::operator!=(const Signal<void,void,void,void>& ) const {
	return false;
}


/*--------------------------------------------------------------------------*/
/**
 */
inline const Signal<void,void,void,void>& Signal<void,void,void,void>::operator=(const Signal<void,void,void,void>& ) {
	return *this;
}


/*--------------------------------------------------------------------------*/
/**
 */
inline void Signal<void,void,void,void>::ConnectionBase::refer() {
}


/*--------------------------------------------------------------------------*/
/**
 */
inline Signal<void,void,void,void>::ConnectionBase::~ConnectionBase() {
}


/*--------------------------------------------------------------------------*/
/**
 */
inline void Signal<void,void,void,void>::ConnectionBase::release() {
}

	
/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
inline Signal<void,void,void,void>::Connection<T>::Connection(T* obj,void (T::*method)())
	: m_Object(obj), m_Method(method) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
void Signal<void,void,void,void>::Connection<T>::emit() const {
	if (m_Object == NULL) {
		delete this;
		return;
	}
	(m_Object->*m_Method)();
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
T* Signal<void,void,void,void>::Connection<T>::object() const {
	return m_Object;
}
	
}; };

#endif

