/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __AUDIO3D_CONTEXT_H
#define __AUDIO3D_CONTEXT_H

#include "object.pointer.h"
#include "container.chain.h"
#include "audio3d.device.h"

namespace magnum { namespace audio3d {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::container::Chain;
using magnum::container::Chainable;

class Listener;
class Device;
class Resource;
class Source;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Context : public Object,private Chainable<Context>, private magnum::object::Noncopyable {
DeclareClass(Context, Object);
friend class Device;
friend class Listener;
friend class Resource;
friend class Source;
private:
	static Context*			g_Current;
	static Chain<Context>	g_Contexts;

	Pointer<Device>			m_Device;
	Pointer<Listener>		m_Listener;
	Chain<Source>			m_Sources;
	ALCcontext*				m_Context;

private:
	explicit Context(ALCcontext* );
	explicit Context(Device* ,ALCcontext* );

public:
	~Context();

	static Context*			createContext();
	inline static Context*	getCurrent();
	inline bool				makeCurrent();

	inline Device*			getDevice() const;
	inline Listener*		getListener() const;

	void					process();
	void					suspend();
	void					stop();

private:
	static Context*			searchCurrent();
};

}; };

#endif
