/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __AUDIO3D_SOURCE_INL
#define __AUDIO3D_SOURCE_INL

namespace magnum { namespace audio3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Source::Source() {
	alGenSources(1, &m_Source);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Source::Source(const Source& s) : Resource(s) {
	alGenSources(1, &m_Source);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Source::~Source() {
	alDeleteSources(1, &m_Source);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::setPosition(const Vector3<float>& v) {
	alSourcefv(m_Source, AL_POSITION, (ALfloat*) (const float*)v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::setPosition(float x,float y,float z) {
	alSource3f(m_Source, AL_POSITION, x, y, z);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> Source::getPosition() const {
	Vector3<float> v;
	alGetSourcefv(m_Source, AL_POSITION, (float*)v);
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::setVelocity(const Vector3<float>& v) {
	alSourcefv(m_Source, AL_VELOCITY, (ALfloat*)(const float*)v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::setVelocity(float x,float y,float z) {
	alSource3f(m_Source, AL_VELOCITY, x, y, z);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> Source::getVelocity() const {
	Vector3<float> v;
	alGetSourcefv(m_Source, AL_VELOCITY, (float*)v);
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::pause() {
	alSourcePause(m_Source);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::play() {
	getContext()->m_Sources.insert(this);
	alSourcePlay(m_Source);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::stop() {
	getContext()->m_Sources.remove(this);
	alSourceStop(m_Source);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::rewind() {
	alSourceRewind(m_Source);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::queue(Buffer* buf) {
	alSourceQueueBuffers(m_Source, 1, &buf->m_Buffer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::unqueue(Buffer* buf) {
	alSourceUnqueueBuffers(m_Source, 1, &buf->m_Buffer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::queue(const BufferGroup& bl) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Source::unqueue(const BufferGroup& bl) {
}

}; };

#endif
