/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __BASE_NUMBER_H
#define __BASE_NUMBER_H

#include "object.h"

namespace magnum { namespace types {
using magnum::object::Object;
using magnum::types::String;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Number : public Object {
DeclareClass(Number, Object);
public:
	virtual unsigned char	byteValue() const = 0;
	virtual short			shortValue() const = 0;
	virtual int				intValue() const = 0;
	virtual long			longValue() const = 0;
	virtual float			floatValue() const = 0;
	virtual double			doubleValue() const = 0;
	virtual bool			boolValue() const = 0;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class TypedNumber : public Number {
DeclareClass(TypedNumber, Numner);
public:
	inline TypedNumber();
	inline TypedNumber(const T& );

	inline unsigned char byteValue() const;
	inline short		shortValue() const;
	inline int			intValue() const;
	inline long			longValue() const;
	inline float		floatValue() const;
	inline double		doubleValue() const;
	inline bool			boolValue() const;

	template<class S>
	inline bool 		operator==(const TypedNumber<S>& ) const;
	template<class S>
	inline bool 		operator!=(const TypedNumber<S>& ) const;
	template<class S>
	inline bool 		operator>(const TypedNumber<S>& ) const;
	template<class S>
	inline bool 		operator<(const TypedNumber<S>& ) const;
	template<class S>
	inline bool 		operator>=(const TypedNumber<S>& ) const;
	template<class S>
	inline bool 		operator<=(const TypedNumber<S>& ) const;

private:
	T					m_Number;	
};


typedef TypedNumber<unsigned char>	Byte;
typedef TypedNumber<unsigned short>	Word;
typedef TypedNumber<short>			Short;
typedef TypedNumber<int>			Int;
typedef TypedNumber<long>			Long;
typedef TypedNumber<float>			Float;
typedef TypedNumber<double>			Double;
typedef TypedNumber<bool>			Bool;

}; };
#endif
