/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __BASE_STRING_H
#define __BASE_STRING_H

#include "object.h"
#include "math.consts.h"

# if !defined(__FORWINDOWS__)
#	include <ctype.h>
# endif

namespace magnum { namespace types {
using magnum::object::Object;
using magnum::math::min;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class String : public Object {
DeclareClass(String, Object);
public:
	inline String();
	String(const String& );
	String(const char* );
	String(char );
	~String();

	inline String		getName() const;
	inline String		toString() const;
	
	inline void			clear();
	void				allocate(unsigned int );
	
	void				sprintf(const char* ,...);
	int					sscanf(const char* ,...);

	inline unsigned int	getLength() const;
	inline const char*	getString() const;

	int					indexOf(const String& , unsigned int from = 0) const;
	int					indexOf(const char* , unsigned int from = 0) const;
	int					indexOf(char , unsigned int from = 0) const;
	inline int			lastIndexOf(const String& , unsigned int from = 0) const;
	int					lastIndexOf(const char* , unsigned int from = 0) const;
	int					lastIndexOf(char , unsigned int from = 0) const;
	bool				startsWith(const String& , unsigned int from = 0) const;
	bool				startsWith(const char* , unsigned int from = 0) const;
	inline String		substring(unsigned int start) const;
	inline String		substring(unsigned int start, unsigned int end) const;
	String				toLower() const;
	String				toUpper() const;
	String&				lower();
	String&				upper();
	
	inline const String& append(const String& );
	inline const String& append(const char* );
	inline const String& append(const Object& );
	inline const String& append(char );

	inline String		trim();
	
	inline bool 		operator==(const String& ) const;
	inline bool 		operator==(const char* ) const;
	inline bool 		operator==(char* ) const;
	inline bool 		operator!=(const String& ) const;
	inline bool 		operator!=(const char* ) const;
	inline bool 		operator!=(char* ) const;
	inline bool 		operator>(const String& ) const;
	inline bool 		operator>(const char* ) const;
	inline bool 		operator>(char* ) const;
	inline bool 		operator<(const String& ) const;
	inline bool 		operator<(const char* ) const;
	inline bool 		operator<(char* ) const;
	inline bool 		operator>=(const String& ) const;
	inline bool 		operator>=(const char* ) const;
	inline bool 		operator>=(char* ) const;
	inline bool 		operator<=(const String& ) const;
	inline bool 		operator<=(const char* ) const;
	inline bool 		operator<=(char* ) const;

	const String& 		operator+=(const String& );
	const String& 		operator+=(const char* );
	const String& 		operator+=(const Object& );
	const String& 		operator+=(char );

	const String& 		operator=(const String& );
	const String& 		operator=(const char* );
	const String& 		operator=(const Object& );
	const String&		operator=(char );

	inline				operator const char*() const;
	inline				operator char*();

	friend String 		operator+(const String& ,const String& );
	friend String 		operator+(const String& ,const char* );
	friend String 		operator+(const char* ,const String& );
	friend String 		operator+(const String& ,const Object& );
	
private:
	void				allocBuffer();

private:
	char*				m_Buffer;
	unsigned int		m_BufferSize;
	unsigned int		m_Length;
};



String operator+(const String& ,const String& );
String operator+(const String& ,const char* );
String operator+(const char* ,const String& );
inline String operator+(const String& ,const Object& );

}; };
#endif
