/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_ALLOCATOR_H
#define __CONTAINER_ALLOCATOR_H

#include "object.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int I=64> class Allocator : public Object {
DeclareClass(Allocator, Object);
public:
	Allocator();
	~Allocator();

	inline T*		alloc();
	inline void		free(T* );
	void			freeAll();
	
	void			clear();

private:
	void			allocate();
	void			deallocate();

private:
	struct Pool {
		Pool*		m_Next;
		unsigned int m_Size;
		T			m_Data[1];
	};

	enum {
		Tsize	= ((sizeof(T) + sizeof(void*) - 1) / sizeof(void*)) * sizeof(void*)
	};

	Pool*			m_First;
	Pool*			m_Last;
	void*			m_Free;
	unsigned int	m_Size;
	unsigned int	m_Increment;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Allocator<T,1> : public Object {
DeclareClass(Allocator, Object);
public:
	inline Allocator();
	inline ~Allocator();

	inline T*		alloc();
	inline void		free(T* );
	inline void		freeAll();
	
	inline void		clear();
};

}; };


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int I> inline void* operator new(size_t, magnum::container::Allocator<T,I>& a) {
	return a.alloc(); 
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int I> inline void operator delete(void* obj, magnum::container::Allocator<T,I>& a) {
	a.free((T*)obj);
}

#endif
