/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_ALLOCATOR_INL
#define __CONTAINER_ALLOCATOR_INL

namespace magnum { namespace container {
using magnum::types::byte;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int I> 
Allocator<T,I>::Allocator() {
	m_Size = I;
	m_Increment = I;
	m_Size = 0;
	m_Last = NULL;
	m_First = NULL;
	m_Free = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int I>
Allocator<T,I>::~Allocator() {
	Pool* pool = m_First;
	while(pool != NULL) {
		Pool* next = pool->m_Next;
		delete[] (byte*)(void*)pool;
		pool = next;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int I> 
inline T* Allocator<T,I>::alloc() {
	if (m_Free == NULL)
		allocate();
	T* ret = (T*)m_Free;
	m_Free = *(void **)m_Free;
	return ret;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int I> 
inline void Allocator<T,I>::free(T* data) {
	*(void **)data = (void *)m_Free;
	m_Free = (void **)data;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int I> 
void Allocator<T,I>::freeAll() {
	Pool* pool = m_First;
	if (pool == NULL)
		return;
		
	while(pool != NULL) {
		void* data = &pool->m_Data[0];
		void* next;
		for (unsigned int i = 0; i < m_Increment-1; i++) {
			next = (void*)((byte *)data + Tsize);
			*(void **)data = next;
			data = next;
		}
		
		pool = pool->m_Next;
		if (pool != NULL)
			*(void **)data = &pool->m_Data[0];
		else
			*(void **)data = NULL;
	}

	m_Free = &m_First->m_Data[0];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int I> 
void Allocator<T,I>::clear() {
	Pool* pool = m_First
	while(pool != NULL) {
		Pool* next = pool->m_Next;
		delete[] (byte*)(void*)pool;
		pool = next;
	}
	m_Size = 0;
	m_Last = NULL;
	m_First = NULL;
	m_Free = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int I> 
void Allocator<T,I>::allocate() {
	Pool* pool = (Pool*)(void *)new byte[sizeof(Pool) + (m_Increment-1)*Tsize];
	pool->m_Size = m_Increment;
	pool->m_Next = NULL;
	if (m_Last != NULL) {
		m_Last->m_Next = pool;
		m_Last = pool;
	} else {
		m_First = pool;
		m_Last = m_First;
	}

	void* data = &pool->m_Data[0];
	void* next;
	for (unsigned int i = 0; i < m_Increment-1; i++) {
		next = (void*)((byte *)data + Tsize);
		*(void **)data = next;
		data = next;
	}

	*(void **)data = m_Free;
	m_Free = &m_Last->m_Data[0];

	m_Size += m_Increment;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int I> 
void Allocator<T,I>::deallocate() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Allocator<T,1>::Allocator() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Allocator<T,1>::~Allocator() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline T* Allocator<T,1>::alloc() {
	return (T*)(void*)new byte[sizeof(T)];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void Allocator<T,1>::free(T* t) {
	delete[] (byte*)(void*)t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void	Allocator<T,1>::freeAll() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void	Allocator<T,1>::clear() {
}


}; };

#endif
