/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_ARRAY_H
#define __CONTAINER_ARRAY_H

#include "object.h"
#include "container.iterator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

#pragma warning(disable:4284)

template<class T> class Array;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class ArrayIterator : public IteratorBase<ArrayIterator<T> > {
friend Array<T>;
public:
	inline ArrayIterator();
	
	inline operator T*();
	inline operator const T*() const;
	inline T&			operator*() const;
	inline T*			operator->() const;
	
	inline ArrayIterator& operator++();
	inline ArrayIterator operator++(int );
	inline ArrayIterator& operator+=(int n);
	inline ArrayIterator operator+(int n) const;
	inline ArrayIterator& operator--();
	inline ArrayIterator operator--(int );
	inline ArrayIterator& operator-=(int n);
	inline ArrayIterator operator-(int n) const;
	inline bool			operator==(const ArrayIterator& iter) const;
	inline bool			operator<(const ArrayIterator& iter) const;
	inline bool			operator<=(const ArrayIterator& iter) const;
	inline bool			operator>(const ArrayIterator& iter) const;
	inline bool			operator>=(const ArrayIterator& iter) const;

private:
	inline ArrayIterator(T* item);

private:
	T*	m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Array : public Object {
DeclareClass(Array, Object);
public:
	typedef ArrayIterator<T>	Iterator;
	
public:
	inline Array();
	inline Array(const Array& );
	inline ~Array();

	void				clear();
	inline unsigned int getSize() const;
	void				setSize(unsigned int n);
	inline bool			isEmpty() const;
	
	
	inline unsigned int	getCapacity() const;
	void				setCapacity(unsigned int n);

	inline const T&		get(unsigned int n) const;
	inline void			set(unsigned int n, const T& elem);

	inline const T*		getArray() const;
	inline T*			getArray();

	inline Iterator		begin() const;
	inline Iterator		at(unsigned int ) const;
	inline Iterator		end() const;

	inline Iterator		rbegin() const;
	inline Iterator		rend() const;

	inline unsigned int	insert(const T& item);
	void				insert(unsigned int n, const T& item);
	void				remove(unsigned int n);
	void				remove(const Iterator& iter);
	bool				contains(const T& item) const;

	void				insert(unsigned int n, const Array& );
	inline unsigned int	append(unsigned int = 1);
	void				append(const Array& );
	void				append(const T* ,unsigned int );

	void				copy(const Array& );
	void				copy(const T* ,unsigned int );

	void				shrink();

	inline T&			operator [](unsigned int nIndex );
	inline const T&		operator [](unsigned int nIndex ) const;

	const Array&		operator=(const Array& );

private:
	unsigned int		m_Size;
	unsigned int		m_Capacity;
	unsigned int		m_MaxGrow;
	T*					m_Array;
};

}; };
#endif
