/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_CACHE_H
#define __CONTAINER_CACHE_H

#include "container.map.h"

namespace magnum {namespace container {

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> class Cache : public Object, private magnum::object::Noncopyable {
DeclareClass(Cache, Object)
public:
	typedef typename Map<Key, Item>::Iterator	Iterator;

private:
	Factory			m_Factory;
	Map<Key, Item>	m_Cache;

public:
	inline ~Cache();
	
	inline bool		contains(const Key& k) const;
	inline void		insert(const Key& key, const Item& item);
	inline bool		remove(const Key& key);
	inline bool		remove(const Iterator& item);
	inline void		clear();

	Item			get(const Key& key);

	inline unsigned int getSize() const;
					
	inline bool		isEmpty() const;

	inline Iterator	begin() const;
	inline Iterator at(const Key& ) const;
	inline Iterator	end() const;

	inline Factory*	getFactory();
	inline const Factory* getFactory() const;
};

}; };

#endif
