/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_CACHE_INL
#define __CONTAINER_CACHE_INL

namespace magnum {namespace container {

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline Cache<Key,Item,Factory>::~Cache() {
	clear();
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline bool Cache<Key,Item,Factory>::contains(const Key& k) const {
	return m_Cache.contains(k);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline void	Cache<Key,Item,Factory>::insert(const Key& key, const Item& item) {
	m_Cache.insert(key, item);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline bool	Cache<Key,Item,Factory>::remove(const Key& key) {
	return m_Cache.remove(key);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline bool	Cache<Key,Item,Factory>::remove(const Iterator& iter) {
	return m_Cache.remove(iter);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline void	Cache<Key,Item,Factory>::clear() {
	Iterator iter;

	while((iter = m_Cache.begin()) != m_Cache.end()) {
		m_Factory.release(iter.item());
		m_Cache.remove(iter);
	}
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
Item Cache<Key,Item,Factory>::get(const Key& key) {
	Iterator iter = m_Cache.at(key);
	if (iter != m_Cache.end())
		return iter.item();

	Item item = m_Factory.create(key);
	m_Cache.insert(key, item);
	return item;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline unsigned int Cache<Key,Item,Factory>::getSize() const {
	return m_Cache.getSize();
}
				

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline bool Cache<Key,Item,Factory>::isEmpty() const {
	return m_Cache.isEmpty();
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline typename Cache<Key,Item,Factory>::Iterator Cache<Key,Item,Factory>::begin() const {
	return m_Cache.begin();
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline typename Cache<Key,Item,Factory>::Iterator Cache<Key,Item,Factory>::at(const Key& key) const {
	return m_Cache.at(key);
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline typename Cache<Key,Item,Factory>::Iterator Cache<Key,Item,Factory>::end() const {
	return m_Cache.end();
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline Factory* Cache<Key,Item,Factory>::getFactory() {
	return &m_Factory;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class Key, class Item, class Factory> 
inline const Factory* Cache<Key,Item,Factory>::getFactory() const {
	return &m_Factory;
}

}; };

#endif
