/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_CHAIN_INL
#define __CONTAINER_CHAIN_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Chainable<T>::Chainable() {
	m_Prev = NULL;
	m_Next = NULL;
	m_Chain = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Chainable<T>::Chainable(const Chainable<T>& ) {
	m_Prev = NULL;
	m_Next = NULL;
	m_Chain = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
Chainable<T>::~Chainable() {
	if (m_Chain == NULL)
		return;

	m_Prev->m_Next = m_Next;
	m_Next->m_Prev = m_Prev;
	m_Chain->m_Size--;

	m_Prev = NULL;
	m_Next = NULL;
	m_Chain = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Chain<T>* Chainable<T>::getChain() const {
	return m_Chain;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Chainable<T>& Chainable<T>::operator=(const Chainable<T>& ) {
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Chain<T>::Chain() {
	m_Root.m_Chain = this;
	m_Root.m_Prev = &m_Root;
	m_Root.m_Next = &m_Root;
	m_Size = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
Chain<T>::Chain(Chain& c) {
	m_Root.m_Next = c.m_Root.m_Next;
	m_Root.m_Prev = c.m_Root.m_Prev;
	m_Root.m_Next->m_Prev = &m_Root;
	m_Root.m_Prev->m_Next = &m_Root;
	m_Size = c.m_Size;

	c.m_Root.m_First = c.m_Root.m_Last = &c.m_Root;
	c.m_Size = 0;

	Chainable<T>* node = m_Root.m_Next;
	while (node != &m_Root) {
		node->m_Chain = this;
		node = node->m_Next;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Chain<T>::~Chain() {
	clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
const Chain<T>& Chain<T>::operator=(Chain& c) {
	clear();

	m_Root.m_Next = c.m_Root.m_Next;
	m_Root.m_Prev = c.m_Root.m_Prev;
	m_Root.m_Next->m_Prev = &m_Root;
	m_Root.m_Prev->m_Next = &m_Root;
	m_Size = c.m_Size;

	c.m_Root.m_First = c.m_Root.m_Last = &c.m_Root;
	c.m_Size = 0;

	Chainable<T>* node = m_Root.m_Next;
	while (node != &m_Root) {
		node->m_Chain = this;
		node = node->m_Next;
	}

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool Chain<T>::contains(const T* item) const {
	return (((Chainable<T>*)item)->m_Chain == this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline unsigned int Chain<T>::getSize() const {
	return m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
typename Chain<T>::Iterator Chain<T>::insert(T* item) {
	ASSERT (((Chainable<T>*)item)->m_Chain == NULL);
	((Chainable<T>*)item)->m_Chain = this;
	((Chainable<T>*)item)->m_Next = &m_Root;
	((Chainable<T>*)item)->m_Prev = m_Root.m_Prev;

	m_Root.m_Prev->m_Next = (Chainable<T>*)item;
	m_Root.m_Prev = (Chainable<T>*)item;
	m_Size++;

	item->refer();
	return Iterator((Chainable<T>*)item);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
typename Chain<T>::Iterator Chain<T>::insertFirst(T* item) {
	ASSERT (((Chainable<T>*)item)->m_Chain == NULL);
	((Chainable<T>*)item)->m_Chain = this;
	((Chainable<T>*)item)->m_Next = m_Root.m_Next;
	((Chainable<T>*)item)->m_Prev = &m_Root;

	m_Root.m_Next->m_Prev = (Chainable<T>*)item;
	m_Root.m_Next = (Chainable<T>*)item;
	m_Size++;

	item->refer();
	return Iterator((Chainable<T>*)item);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
typename Chain<T>::Iterator Chain<T>::insertBefore(T* item, const T* poso) {
	ASSERT (((Chainable<T>*)item)->m_Chain == NULL);
	if (((const Chainable<T>*)poso)->m_Chain != this)
		return insertFirst(item);
	T* pos = const_cast<T*>(poso);
	((Chainable<T>*)item)->m_Chain = this;
	((Chainable<T>*)item)->m_Next = (Chainable<T>*)pos;
	((Chainable<T>*)item)->m_Prev = pos->m_Prev;
	((Chainable<T>*)pos)->m_Prev->m_Next = (Chainable<T>*)item;
	((Chainable<T>*)pos)->m_Prev = (Chainable<T>*)pos;

	m_Size++;
	item->refer();
	return Iterator((Chainable<T>*)item);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
typename Chain<T>::Iterator Chain<T>::insertAfter(T* item, const T* poso) {
	ASSERT (((Chainable<T>*)item)->m_Chain == NULL);
	if (((const Chainable<T>*)poso)->m_Chain != this)
		return insert(item);
	T* pos = const_cast<T*>(poso);
	((Chainable<T>*)item)->m_Chain = this;
	((Chainable<T>*)item)->m_Next = pos->m_Next;
	((Chainable<T>*)item)->m_Prev = (Chainable<T>*)pos;
	((Chainable<T>*)pos)->m_Next->m_Prev = (Chainable<T>*)item;
	((Chainable<T>*)pos)->m_Next = (Chainable<T>*)pos;

	m_Size++;
	item->refer();
	return Iterator((Chainable<T>*)item);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
bool Chain<T>::remove(T* item) {
	if (((Chainable<T>*)item)->m_Chain != this)
		return false;

	((Chainable<T> *)item)->m_Next->m_Prev = ((Chainable<T> *)item)->m_Prev;
	((Chainable<T> *)item)->m_Prev->m_Next = ((Chainable<T> *)item)->m_Next;

	((Chainable<T> *)item)->m_Next = NULL;
	((Chainable<T> *)item)->m_Prev = NULL;
	((Chainable<T> *)item)->m_Chain = NULL;
	item->release();
	m_Size--;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool Chain<T>::remove(const typename Chain<T>::Iterator& iter) {
	return remove((T*)iter.m_Item);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Chain<T>::clear() {
	T* node = (T*)m_Root.m_Next;
	T* next;
	while ((Chainable<T>*)node != &m_Root) {
		next = (T*)((Chainable<T> *)node)->m_Next;
		((Chainable<T> *)node)->m_Next = NULL;
		((Chainable<T> *)node)->m_Prev = NULL;
		((Chainable<T> *)node)->m_Chain = NULL;
		node->release();
		node = next;
	}
	m_Root.m_Next = m_Root.m_Prev = &m_Root;
	m_Size = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool Chain<T>::isEmpty() const {
	return (m_Root.m_Next == &m_Root);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline typename Chain<T>::Iterator Chain<T>::begin() const {
	return Iterator(m_Root.m_Next);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline typename Chain<T>::Iterator Chain<T>::at(const T* item) const {
	return Iterator((Chainable<T>*)item);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline typename Chain<T>::Iterator Chain<T>::end() const {
	return Iterator((Chainable<T>*)&m_Root);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline typename Chain<T>::Iterator Chain<T>::rbegin() const {
	return Iterator(m_Root.m_Prev);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline typename Chain<T>::Iterator Chain<T>::rend() const {
	return Iterator((Chainable<T>*)&m_Root);
}

}; };
#endif
