/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_COMPOUND_INL
#define __CONTAINER_COMPOUND_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline Compound<K,T>::Compound() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline Compound<K,T>::Compound(const K& k, const T& i) {
	m_Key = k;
	m_Item = i;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline T& Compound<K,T>::operator*() {
	return m_Item;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline const T& Compound<K,T>::operator*() const {
	return m_Item;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline T* Compound<K,T>::operator->() {
	return &m_Item;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline const T* Compound<K,T>::operator->() const {
	return &m_Item;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline const K& Compound<K,T>::getKey() const {
	return m_Key;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline void Compound<K,T>::setKey(const K& k) {
	m_Key = k;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline const T& Compound<K,T>::getItem() const {
	return m_Item;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline void Compound<K,T>::setItem(const T& t) {
	m_Item = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator==(const Compound<K,T>& c) const {
	return m_Key == c.m_Key;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator!=(const Compound<K,T>& c) const {
	return m_Key != c.m_Key;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator<=(const Compound<K,T>& c) const {
	return m_Key <= c.m_Key;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator<(const Compound<K,T>& c) const {
	return m_Key < c.m_Key;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator>=(const Compound<K,T>& c) const {
	return m_Key >= c.m_Key;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator>(const Compound<K,T>& c) const {
	return m_Key > c.m_Key;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator==(const K& k) const {
	return m_Key == k;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator!=(const K& k) const {
	return m_Key != k;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator<=(const K& k) const {
	return m_Key <= k;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator<(const K& k) const {
	return m_Key < k;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator>=(const K& k) const {
	return m_Key >= k;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class K, class T> inline bool Compound<K,T>::operator>(const K& k) const {
	return m_Key > k;
}

}; };

#endif
