/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_HEAP_H
#define __CONTAINER_HEAP_H

#include "container.iterator.h"
#include "container.array.h"
#include "container.allocator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Heap : public Object {
DeclareClass(Heap, Object);
private:
	Array<T*>			m_Heap;
	Allocator<T>		m_Allocator;

public:
	class Iterator : public IteratorBase<Iterator> {
	friend class Heap<T>;
	public:
		inline Iterator() {
		};
		inline operator T*() {
			return *m_Item;
		};
		inline operator const T*() const {
			return *m_Item;
		};
		inline const T& operator*() const {
			return **m_Item;
		};
		inline const T* operator->() const {
			return *m_Item;
		};
		inline Iterator& operator++() {
			ASSERT(m_Item != NULL);
			m_Item++;
			return *this;
		};
		inline Iterator operator++(int ) {
			ASSERT(m_Item != NULL);
			T** item = m_Item;
			m_Item++;
			return Iterator(item);
		};
		inline Iterator	operator+(int n) const {
			ASSERT(m_Item != NULL);
			return Iterator(m_Item + n);
		};
		inline Iterator& operator+=(int n) {
			m_Item += n;
			return *this;
		};
		inline Iterator& operator--() {
			ASSERT(m_Item != NULL);
			m_Item--;
			return *this;
		};
		inline Iterator	operator--(int ) {
			ASSERT(m_Item != NULL);
			T** item = m_Item;
			m_Item--;
			return Iterator(item);
		};
		inline Iterator	operator-(int n) const {
			ASSERT(m_Item != NULL);
			return Iterator(m_Item - n);
		};
		inline Iterator& operator-=(int n) {
			m_Item -= n;
			return *this;
		};
		inline bool operator==(const Iterator& iter) const {
			return m_Item == iter.m_Item;
		};
		inline bool operator!=(const Iterator& iter) const {
			return m_Item != iter.m_Item;
		};
		inline bool	operator<(const Iterator& iter) const {
			return m_Item < iter.m_Item;
		};
		inline bool	operator>(const Iterator& iter) const {
			return m_Item > iter.m_Item;
		};
		inline bool	operator<=(const Iterator& iter) const {
			return m_Item <= iter.m_Item;
		};
		inline bool	operator>=(const Iterator& iter) const {
			return m_Item >= iter.m_Item;
		};

	private:
		inline Iterator(T** item) {
			m_Item = item;
		};

		T**	m_Item;
	};

public:
	inline Heap();
	inline ~Heap();

	inline bool		contains(const T& ) const;
	void			insert(const T& );
	void			remove(const Iterator& );
	void			clear();

	inline const T&	peek() const;
	inline T		remove();

	inline bool		isEmpty() const;
	inline unsigned int getSize() const;

	inline Iterator	begin() const;
	inline Iterator at(const T& item) const;
	inline Iterator top() const;
	inline Iterator	end() const;

	bool			operator==(const Heap& ) const;
	bool			operator!=(const Heap& ) const;

private:
	T				remove_idx(unsigned int );
};

}; };

#endif
