/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_HEAP_INL
#define __CONTAINER_HEAP_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class T> 
inline Heap<T>::Heap() {
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class T> 
inline Heap<T>::~Heap() {
	clear();
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class T> 
inline bool Heap<T>::isEmpty() const {
	return (m_Heap.isEmpty());
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class T> 
inline unsigned int Heap<T>::getSize() const {
	return m_Heap.getSize();
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class T> 
void Heap<T>::clear() {
	unsigned int i;
	for(i = 0; i < m_Heap.getSize(); i++) {
		m_Heap[i]->~T();
		operator delete(m_Heap[i], m_Allocator);
	}

	m_Heap.clear();
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class T> 
void Heap<T>::insert(const T& item) {
	T* nitem = new(m_Allocator) T(item);
	m_Heap.insert(nitem);

	unsigned int i = m_Heap.getSize() - 1;
	unsigned int j = (i-1)/2;
	while((i > 0) && (*m_Heap[j] < *nitem)) {
		m_Heap[i] = m_Heap[j];
		i = j;
		j = (i-1)/2;
	}
	m_Heap[i] = nitem;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class T> 
T Heap<T>::remove_idx(unsigned int i) {
	ASSERT(i < m_Heap.getSize());

	T ritem = *m_Heap[i];
	int last = m_Heap.getSize() - 1; 
	m_Heap[i]->~T();
	operator delete(m_Heap[i], m_Allocator);
	T* v = m_Heap[last];
	
	int j;
	for(;;) {
		j = i+i+1;
		if (j > last)
			break;
		if ((j < last) && (*m_Heap[j] < *m_Heap[j+1]))
			j++;
		if (*v >= *m_Heap[j])
			break;
		m_Heap[i] = m_Heap[j];
		i = j;
	}
	m_Heap[i] = v;

	m_Heap.remove(last);
	return ritem;
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class T> 
inline void Heap<T>::remove(const Iterator& item) {
	remove_idx(item.m_Item - m_Heap.getArray());
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class T> 
inline T Heap<T>::remove() {
	return remove_idx(0);
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class T> 
inline const T& Heap<T>::peek() const {
	ASSERT(m_Heap.getSize() > 0);
	return *m_Heap[0];
}


/*----------------------------------------------------------------------------------
Description:
----------------------------------------------------------------------------------*/
template<class T> 
inline bool Heap<T>::contains(const T& item) const {
	for (unsigned int i = 0; i < m_Heap.getSize(); i++)
		if (*m_Heap[i] == item)
			return true;
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline typename Heap<T>::Iterator Heap<T>::begin() const {
	return Iterator((T**)m_Heap.getArray());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline typename Heap<T>::Iterator Heap<T>::end() const {
	return Iterator((T**)m_Heap.getArray() + m_Heap.getSize());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline typename Heap<T>::Iterator Heap<T>::top() const {
	return Iterator((T**)m_Heap.getArray());
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> 
inline typename Heap<T>::Iterator Heap<T>::at(const T& item) const {
	for (unsigned int i = 0; i < m_Heap.getSize(); i++)
		if (*m_Heap[i] == item)
			return Iterator(&m_Heap[i]);
	return Iterator(m_Heap.getArray() + m_Heap.getSize());;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> 
inline bool Heap<T>::operator==(const Heap<T>& heap) const {
	if (m_Size != heap.m_Size)
		return false;
	ASSERT(false);
	return false;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> 
inline bool Heap<T>::operator!=(const Heap<T>& heap) const {
	if (m_Size != set.m_Size)
		return true;
	ASSERT(false);
	return false;
}

}; };

#endif
