/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_LIST_H
#define __CONTAINER_LIST_H

#include "object.h"
#include "container.iterator.h"
#include "container.allocator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

#pragma warning(push)
#pragma warning(disable:4284)


template<class T,int N> class ListAllocator;
template<class T,class A> class List;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class ListItem {
public:
	T				m_Item;
	ListItem*		m_Next;
	ListItem*		m_Prev;
	
public:
	inline ListItem();
	inline ListItem(const T& i);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int N=64> class ListAllocator : public Allocator<ListItem<T>,N > {
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class A>
class ListIterator : public IteratorBase<ListIterator<T,A> > {
friend List<T,A>;
public:
	inline ListIterator();
	
	inline operator T*();
	inline operator const T*() const; 
	
	inline T& operator*() const;
	inline T* operator->() const;
	
	inline ListIterator& operator++();
	inline ListIterator operator++(int );
	inline ListIterator& operator--();
	inline ListIterator	operator--(int );

	inline bool operator==(const ListIterator& iter) const;
	inline bool operator!=(const ListIterator& iter) const;
	inline bool operator<(const ListIterator& iter) const;
	inline bool operator>(const ListIterator& iter) const;
	inline bool operator<=(const ListIterator& iter) const;
	inline bool operator>=(const ListIterator& iter) const;

private:
	inline ListIterator(ListItem<T>* item);

private:
	ListItem<T>*	m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A = ListAllocator<T> > class List : public Object {
DeclareClass(List, Object);
public:
	typedef ListIterator<T,A>	Iterator;

public:
	inline List();
	inline List(const List& );
	inline ~List();

	bool				contains(const T& item) const;
	Iterator			append();
	Iterator			insert(const T& item);
	Iterator			insertFirst(const T& item);
	bool				remove(const T& item);
	bool				remove(T* );
	bool				remove(const Iterator& iter);
	void				clear();

	inline bool			isEmpty() const;
	inline unsigned int getSize() const;

	inline Iterator		begin() const;
	inline Iterator 	at(const T& ) const;
	inline Iterator		end() const;

	inline Iterator		rbegin() const;
	inline Iterator		rend() const;

	const List&			operator=(const List &);

private:
	A				m_Allocator;
	ListItem<T>		m_Root;
	unsigned int	m_Size;
};


#pragma warning(pop)

}; };
#endif
