/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_LIST_INL
#define __CONTAINER_LIST_INL

namespace magnum { namespace container {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline ListItem<T>::ListItem() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline ListItem<T>::ListItem(const T& i) : m_Item(i) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
inline List<T,A>::List() {
	m_Root.m_Next = m_Root.m_Prev = &m_Root;
	m_Size = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
inline List<T,A>::List(const List& l) {
	m_Root.m_Next = m_Root.m_Prev = &m_Root;
	m_Size = 0;
	operator=(l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
inline List<T,A>::~List() {
	clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
const List<T,A>& List<T,A>::operator=(const List& l) {
	clear();

	m_Size = l.m_Size;
	ListItem<T> *node = l.m_Root.m_Next;
	ListItem<T> *last = &m_Root;

	while (node != &l.m_Root) {
		ListItem<T> *newNode = new(m_Allocator) ListItem<T>(*node);
		newNode->m_Prev = last;
		newNode->m_Item = node->m_Item;
		last->m_Next = newNode;
		last = newNode;
		node = node->m_Next;
	}

	last->m_Next = &m_Root;
	m_Root.m_Prev = last;

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
bool List<T,A>::contains(const T& item) const {
	ListItem<T> *node = m_Root.m_Next;
	while (node != &m_Root) {
		if (node->m_Item == item)
			return true;
		node = node->m_Next;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
typename List<T,A>::Iterator List<T,A>::append() {
	ListItem<T> *node = new(m_Allocator) ListItem<T>();
	node->m_Next = &m_Root;
	node->m_Prev = m_Root.m_Prev;
	m_Root.m_Prev->m_Next = node;
	m_Root.m_Prev = node;
	m_Size++;
	return Iterator(node);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
typename List<T,A>::Iterator List<T,A>::insert(const T& item) {
	ListItem<T> *node = new(m_Allocator) ListItem<T>(item);
	node->m_Next = &m_Root;
	node->m_Prev = m_Root.m_Prev;
	m_Root.m_Prev->m_Next = node;
	m_Root.m_Prev = node;
	m_Size++;
	return Iterator(node);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
typename List<T,A>::Iterator List<T,A>::insertFirst(const T& item) {
	ListItem<T> *node = new(m_Allocator) ListItem<T>(item);
	node->m_Prev = &m_Root;
	node->m_Next = m_Root.m_Next;
	m_Root.m_Next->m_Prev = node;
	m_Root.m_Next = node;
	m_Size++;
	return Iterator(node);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
bool List<T,A>::remove(const T& item) {
	ListItem<T> *node = m_Root.m_Next;
	while (node != &m_Root) {
		if (node->m_Item == item) {
			node->m_Prev->m_Next = node->m_Next;
			node->m_Next->m_Prev = node->m_Prev;
			node->~ListItem<T>();
			operator delete(node, m_Allocator);
			m_Size--;
			return true;
		}
		node = node->m_Next;
	}

	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
bool List<T,A>::remove(T* item) {
	ListItem<T> *node = (ListItem<T>*)item;
	node->m_Prev->m_Next = node->m_Next;
	node->m_Next->m_Prev = node->m_Prev;
	node->~ListItem<T>();
	operator delete(node, m_Allocator);
	m_Size--;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
bool List<T,A>::remove(const typename List<T,A>::Iterator& iter) {
	ListItem<T> *node = iter.m_Item;
	node->m_Prev->m_Next = node->m_Next;
	node->m_Next->m_Prev = node->m_Prev;
	node->~ListItem<T>();
	operator delete(node, m_Allocator);
	m_Size--;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
void List<T,A>::clear() {
	ListItem<T> *node;
	while (m_Root.m_Next != &m_Root) {
		node = m_Root.m_Next;
		m_Root.m_Next = node->m_Next;
		node->~ListItem();
		operator delete(node, m_Allocator);
		m_Size--;
	}

	m_Root.m_Next = m_Root.m_Prev = &m_Root;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
inline unsigned int List<T,A>::getSize() const {
	return m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
inline typename List<T,A>::Iterator List<T,A>::begin() const {
	return Iterator(m_Root.m_Next);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
inline typename List<T,A>::Iterator List<T,A>::end() const {
	return Iterator((ListItem<T>*)&m_Root);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
inline typename List<T,A>::Iterator List<T,A>::rbegin() const {
	return Iterator(m_Root.m_Prev);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
inline typename List<T,A>::Iterator List<T,A>::rend() const {
	return Iterator((ListItem<T>*)&m_Root);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A> 
inline typename List<T,A>::Iterator List<T,A>::at(const T& item) const {
	ListItem<T> *node = m_Root.m_Next;

	while (node != &m_Root) {
		if (node->m_Item == item) 
			return Iterator(node);
		node = node->m_Next;
	}

	return Iterator((ListItem<T>*)&m_Root);
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline ListIterator<T,A>::ListIterator() {
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline ListIterator<T,A>::ListIterator(ListItem<T>* item) {
	m_Item = item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline ListIterator<T,A>::operator T*() {
	return &m_Item->m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline ListIterator<T,A>::operator const T*() const {
	return &m_Item->m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline T& ListIterator<T,A>::operator*() const {
	return m_Item->m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline T* ListIterator<T,A>::operator->() const {
	return &m_Item->m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline ListIterator<T,A>& ListIterator<T,A>::operator++() {
	ASSERT(m_Item != NULL);
	m_Item = m_Item->m_Next;
	return *this;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline ListIterator<T,A> ListIterator<T,A>::operator++(int ) {
	ASSERT(m_Item != NULL);
	ListItem<T>* item = m_Item;
	m_Item = m_Item->m_Next;
	return ListIterator(item);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline ListIterator<T,A>& ListIterator<T,A>::operator--() {
	ASSERT(m_Item != NULL);
	m_Item = m_Item->m_Prev;
	return *this;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline ListIterator<T,A>	ListIterator<T,A>::operator--(int ) {
	ASSERT(m_Item != NULL);
	ListItem<T>* item = m_Item;
	m_Item = m_Item->m_Prev;
	return ListIterator(item);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline bool ListIterator<T,A>::operator==(const ListIterator& iter) const {
	return m_Item == iter.m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline bool ListIterator<T,A>::operator!=(const ListIterator& iter) const {
	return m_Item != iter.m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline bool ListIterator<T,A>::operator<(const ListIterator& iter) const {
	return m_Item < iter.m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline bool ListIterator<T,A>::operator>(const ListIterator& iter) const {
	return m_Item > iter.m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline bool ListIterator<T,A>::operator<=(const ListIterator& iter) const {
	return m_Item <= iter.m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class A>
inline bool ListIterator<T,A>::operator>=(const ListIterator& iter) const {
	return m_Item >= iter.m_Item;
};

}; };

#endif
