/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_MAP_H
#define __CONTAINER_MAP_H

#include "container.iterator.h"
#include "container.allocator.h"


namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

template<class K, class I> class Map;
template<class K, class I> class Multimap;

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class K, class I> struct MapNode {
	K				m_Key;
	unsigned int	m_Rand;
	MapNode<K,I>*	m_LSon;
	MapNode<K,I>*	m_RSon;
	MapNode<K,I>*	m_Father;
	I				m_Item;
};


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class K, class I> class MapIterator : public IteratorBase<MapIterator<K,I> > {
friend class Map<K, I>;
friend class Multimap<K, I>;
public:
	inline MapIterator();
	inline				operator I*();
	inline				operator const I*() const;
	
	inline I&			operator*() const;
	inline I*			operator->() const;
	
	inline MapIterator&	operator++();
	inline MapIterator	operator++(int );
	inline MapIterator&	operator--();
	inline MapIterator	operator--(int );
	inline bool 		operator==(const MapIterator& iter) const;
	inline bool 		operator!=(const MapIterator& iter) const;
	inline bool			operator<(const MapIterator& iter) const;
	
	inline const I&		item() const;
	inline const K&		key() const;

private:
	inline MapIterator(MapNode<K,I>* n);

	MapNode<K,I>*		m_Node;
};


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class K, class I> class Map : public Object {
DeclareClass(Map, Object);
friend class Multimap<K, I>;
public:
	typedef MapIterator<K,I>	Iterator;

public:
	inline Map();
	inline ~Map();

	inline bool			contains(const K& k) const;
	Iterator			insert(const K& key, const I& item);
	Iterator			insert(const K& key);
	bool				remove(const K& key);
	bool				remove(const Iterator& item);
	void				clear();
						
	inline const I&		get(const K& key) const;
	inline I&			get(const K& key);
	inline void			set(const K&, const I& elem);
						
	inline unsigned		int getSize() const;
						
	inline bool			isEmpty() const;
						
	inline Iterator		begin() const;
	inline Iterator		at(const K& ) const;
	inline Iterator		end() const;

	inline I&			operator [](const K& );
	inline const I&		operator [](const K& ) const;


private:
	void				removeNode(MapNode<K,I> *node, MapNode<K,I> **father);
	void				repairUp(MapNode<K,I> *node);
	inline unsigned int rand();

private:
	MapNode<K,I>*		m_Root;
	unsigned int		m_Size;
	unsigned int		m_Rand;
	Allocator<MapNode<K,I> >		m_Allocator;
};


}; };

#endif
