/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_MULTIMAP_INL
#define __CONTAINER_MULTIMAP_INL

namespace magnum {namespace container {


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class K, class I> 
typename Multimap<K,I>::Iterator Multimap<K,I>::insert(const K& key, const I& item) {
	MapNode<K,I> *node = m_Root;
	MapNode<K,I> *newNode;
	
	if (node == NULL) {
		m_Root = new(m_Allocator) MapNode<K,I>;
		m_Root->m_LSon =
		m_Root->m_RSon =
		m_Root->m_Father = NULL;
		m_Root->m_Key = key;
		m_Root->m_Item = item;
		m_Root->m_Rand = rand();
		m_Size++;
		return Iterator(m_Root);
	}	

	for (;;) {
		if (node->m_Key < key) {
			if (node->m_RSon == NULL) {
				newNode = node->m_RSon = new(m_Allocator) MapNode<K,I>;
				newNode->m_Father = node;
				newNode->m_Key = key;
				newNode->m_Item = item;
				newNode->m_LSon =
				newNode->m_RSon = NULL;
				newNode->m_Rand = rand();
				repairUp(newNode);
				m_Size++;
				return Iterator(newNode);
			} else {
				node = node->m_RSon;
			}
		} else {
			if (node->m_LSon == NULL) {
				newNode = node->m_LSon = new(m_Allocator) MapNode<K,I>;
				newNode->m_Father = node;
				newNode->m_Key = key;
				newNode->m_Item = item;
				newNode->m_LSon =
				newNode->m_RSon = NULL;
				newNode->m_Rand = rand();
				repairUp(newNode);
				m_Size++;
				return Iterator(newNode);
			} else {
				node = node->m_LSon;
			}
		}
	}
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class K, class I> 
typename Multimap<K,I>::Iterator Multimap<K,I>::insert(const K& key) {
	MapNode<K,I> *node = m_Root;
	MapNode<K,I> *newNode;
	
	if (node == NULL) {
		m_Root = new(m_Allocator) MapNode<K,I>;
		m_Root->m_LSon =
		m_Root->m_RSon =
		m_Root->m_Father = NULL;
		m_Root->m_Key = key;
		m_Root->m_Rand = rand();
		m_Size++;
		return Iterator(m_Root);
	}	

	for (;;) {
		if (node->m_Key < key) {
			if (node->m_RSon == NULL) {
				newNode = node->m_RSon = new(m_Allocator) MapNode<K,I>;
				newNode->m_Father = node;
				newNode->m_Key = key;
				newNode->m_LSon =
				newNode->m_RSon = NULL;
				newNode->m_Rand = rand();
				repairUp(newNode);
				m_Size++;
				return Iterator(newNode);
			} else {
				node = node->m_RSon;
			}
		} else {
			if (node->m_LSon == NULL) {
				newNode = node->m_LSon = new(m_Allocator) MapNode<K,I>;
				newNode->m_Father = node;
				newNode->m_Key = key;
				newNode->m_LSon =
				newNode->m_RSon = NULL;
				newNode->m_Rand = rand();
				repairUp(newNode);
				m_Size++;
				return Iterator(newNode);
			} else {
				node = node->m_LSon;
			}
		}
	}
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class K, class I> 
bool Multimap<K,I>::remove(const K& key) {
	MapNode<K,I> *node = m_Root;
	MapNode<K,I> **father = &m_Root;
	bool removed = false;

	// first search the node
	while (node != NULL) {
		if (node->m_Key == key) {
			removeNode(node, father);
			node = m_Root;
			father = &m_Root;
			removed = true;
			continue;
		}
		if (node->m_Key < key) {
			father = &node->m_RSon;
			node = node->m_RSon;
		} else {
			father = &node->m_LSon;
			node = node->m_LSon;
		}
	}
	
	return removed;
}

}; };

#endif
