/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_PRIORITYHEAP_H
#define __CONTAINER_PRIORITYHEAP_H


namespace magnum { namespace container {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class P = int> class PriorityHeap : public Object {
DeclareClass(PriorityHeap, Object);
private:
	struct Item {
	public:
		T				m_Item;
		P				m_Priority;
		unsigned int	m_Index;

	public:
		inline Item() {};
		inline Item(const T& t, const P& p) {
			m_Item = t;
			m_Priority = p;
		};
	};
	
public:
	class Iterator : public IteratorBase<Iterator> {
	friend PriorityHeap<T,P>;
	private:
		const Array<Item*>*		m_Heap;
		Item*					m_Item;

	public:
		inline Iterator() {
		};
		inline operator T*() {
			return &(m_Item->m_Item);
		};
		inline operator const T*() const {
			return &(m_Item->m_Item);
		};
		inline T& operator*() const {
			return m_Item->m_Item;
		};
		inline T* operator->() const {
			return &(m_Item->m_Item);
		};
		inline const T& item() const {
			return m_Item->m_Item;
		};
		inline const T& priority() const {
			return m_Item->m_Priority;
		};
		inline Iterator& operator++() {
			ASSERT(m_Item != NULL);
			m_Item = (*m_Heap)[m_Item->m_Index + 1];
			return *this;
		};
		inline Iterator operator++(int ) {
			ASSERT(m_Item != NULL);
			Item* item = m_Item;
			m_Item = (*m_Heap)[m_Item->m_Index + 1];
			return Iterator(m_Heap, item);
		};
		inline Iterator& operator+=(int n) {
			m_Item = (*m_Heap)[m_Item->m_Index + n];
			return *this;
		};
		inline Iterator operator+(int n) const {
			return Iterator(m_Heap, (*m_Heap)[m_Item->m_Index + n]);
		};
		inline Iterator& operator--() {
			ASSERT(m_Item != NULL);
			m_Item = (*m_Heap)[m_Item->m_Index - 1];
			return *this;
		};
		inline Iterator	operator--(int ) {
			ASSERT(m_Item != NULL);
			Item* item = m_Item;
			m_Item = (*m_Heap)[m_Item->m_Index - 1];
			return Iterator(m_Heap, item);
		};
		inline Iterator& operator-=(int n) {
			m_Item = (*m_Heap)[m_Item->m_Index - n];
			return *this;
		};
		inline Iterator operator-(int n) const {
			return Iterator(m_Heap, (*m_Heap)[m_Item->m_Index - n]);
		};
		inline bool operator==(const Iterator& iter) const {
			return m_Item == iter.m_Item;
		};
		inline bool	operator<(const Iterator& iter) const {
			return m_Item < iter.m_Item;
		};
		inline bool	operator<=(const Iterator& iter) const {
			return m_Item <= iter.m_Item;
		};
		inline bool	operator>(const Iterator& iter) const {
			return m_Item > iter.m_Item;
		};
		inline bool	operator>=(const Iterator& iter) const {
			return m_Item >= iter.m_Item;
		};

	private:
		inline Iterator(const Array<Item*>* heap, Item* item) {
			m_Heap = heap;
			m_Item = item;
		}
	};


public:
	PriorityHeap();
	~PriorityHeap();

	inline bool			contains(const T& ) const;
	inline Iterator		insert(const T& , const P& );
	inline void			change(const Iterator& ,const P& );
	void				remove(const Iterator& );
	void				clear();

	inline unsigned int	getCapacity() const;
	inline void			setCapacity(unsigned int n);

	inline const T&		peek() const;
	inline T			remove();

	inline bool			isEmpty() const;
	inline unsigned int getSize() const;

	inline Iterator		begin() const;
	inline Iterator 	at(const T& item) const;
	inline Iterator 	top() const;
	inline Iterator		end() const;

	bool				operator==(const PriorityHeap& ) const;
	bool				operator!=(const PriorityHeap& ) const;
	

private:
	T					remove_idx(unsigned int );
	void				repair_up(unsigned int );
	void				repair_down(unsigned int );

private:
	Array<Item*>		m_Heap;
	Allocator<Item>		m_Allocator;
	Item				m_Terminator;
};

}; };

#endif
