/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_QUEUE_H
#define __CONTAINER_QUEUE_H

#include "object.h"
#include "container.iterator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;


#pragma warning(push)
#pragma warning(disable:4284)

template<class T> class Queue;
template<class T> class QueueIterator;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Queueable {
friend class Queue<T>;
friend class QueueIterator<T>;
private:
	Queue<T>*		m_Queue;
	Queueable<T>*	m_Next;
	Queueable<T>*	m_Prev;

public:
	inline Queueable();
	~Queueable();
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class QueueIterator : public IteratorBase<QueueIterator<T> > {
public:
	inline QueueIterator() {
	};
	inline operator T*() {
		return (T*)m_Item;
	};
	inline operator const T*() const {
		return (T*)m_Item;
	};
	inline T& operator*() const {
		return *(T*)m_Item;
	};
	inline T* operator->() const {
		return (T*)m_Item;
	};
	inline QueueIterator& operator++() {
		ASSERT(m_Item != NULL);
		m_Item = m_Item->m_Next;
		return *this;
	};
	inline QueueIterator operator++(int ) {
		ASSERT(m_Item != NULL);
		Queueable<T>* item = m_Item;
		m_Item = m_Item->m_Next;
		return Iterator(item);
	};
	inline QueueIterator& operator--() {
		ASSERT(m_Item != NULL);
		m_Item = m_Item->m_Prev;
		return *this;
	};
	inline QueueIterator	operator--(int ) {
		ASSERT(m_Item != NULL);
		Queueable<T>* item = m_Item;
		m_Item = m_Item->m_Prev;
		return Iterator(item);
	};
	inline bool	operator==(const QueueIterator& iter) {
		return m_Item == iter.m_Item;
	};
	inline bool	operator<(const QueueIterator& iter) {
		return m_Item < iter.m_Item;
	};

private:
	inline QueueIterator(Queueable<T>* item) {
		m_Item = item;
	};

	Queueable<T>*	m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Queue : public Object {
DeclareClass(Queue, Object);
friend class Queueable<T>;
public:
	typedef QueueIterator<T>	Iterator;

public:
	inline Queue();
	inline ~Queue();

	inline bool		contains(const T* item) const;
	void			insert(T* item);
	void			insertHead(T* item);
	bool			remove(T* item);
	void			clear();

	inline T*		removeHead();
	inline T*		peekHead() const;
	inline T*		removeTail();
	inline T*		peekTail() const;

	inline bool		isEmpty() const;
	inline unsigned int getSize() const;

	inline Iterator	begin() const;
	inline Iterator at(T* ) const;
	inline Iterator	end() const;

	inline Iterator	rbegin() const;
	inline Iterator	rend() const;

private:
	Queueable<T>*		m_Head;
	Queueable<T>*		m_Tail;
	unsigned int		m_Size;
};


#pragma warning(pop)

}; };


#endif
