/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_SCHEDULER_H
#define __CONTAINER_SCHEDULER_H

#include "object.h"
#include "object.pointer.h"
#include "object.softpointer.h"
#include "container.iterator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::object::SoftPointer;

template<class T> class Scheduler;
template<class T> class TaskIterator;

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> class Task {
friend class Scheduler<T>;
friend class TaskIterator<T>;
public:
	enum Priority {
		Lowest,
		Low,
		Normal,
		High,
		Highest,
		Dependant = 0x7fffffff,
	};

private:
	SoftPointer<T>		m_Dependency;
	Priority			m_Priority;
	Scheduler<T>*		m_Scheduler;
	Task<T>*			m_Next;
	Task<T>*			m_Prev;
	bool				m_Visited;

public:
	inline Task();
	inline Task(const Task& );
	inline ~Task();
	
	void				setDependency(T* );
	void				clearDependency();
	inline T*			getDependency() const;
	
	void				setPriority(Priority );
	inline Priority		getPriority() const;
	
	inline Scheduler<T>* getScheduler() const;
	
	inline const Task&	operator=(const Task& );
};


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> class TaskIterator : public IteratorBase<TaskIterator<T> > {
friend class Scheduler<T>;
private:
	Task<T>*		m_Task;

private:
	inline TaskIterator(Task<T>* );

public:
	inline TaskIterator();

	inline					operator T*();
	inline					operator const T*() const;
	inline T*				operator->();
	inline const T*			operator->() const;
	inline T&				operator*();
	inline const T&			operator*() const;
	
	inline TaskIterator&	operator++();
	inline TaskIterator		operator++(int );
	inline TaskIterator&	operator--();
	inline TaskIterator		operator--(int );

	inline bool				operator==(const TaskIterator& ) const;
	inline bool				operator!=(const TaskIterator& ) const;
	inline bool				operator<(const TaskIterator& ) const;
	inline bool				operator>(const TaskIterator& ) const;
	inline bool				operator<=(const TaskIterator& ) const;
	inline bool				operator>=(const TaskIterator& ) const;
};


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> class Scheduler : public Object, private magnum::object::Noncopyable {
DeclareClass(Scheduler, Object);
friend class Task<T>;
private:
	Task<T>				m_Root;
	unsigned int		m_Size;
		
public:
	typedef TaskIterator<T>	Iterator;

public:
	Scheduler();
	~Scheduler();

	Iterator			insert(T* );
	bool				remove(T* );
	inline bool			contains(const T* ) const;
	void				clear();
	
	inline unsigned int	getSize() const;
	inline bool			isEmpty() const;
	
	inline Iterator		begin() const;
	inline Iterator		at(const T* ) const;
	inline Iterator		end() const;
};

}; };

#endif
