/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_SET_H
#define __CONTAINER_SET_H

#include "container.iterator.h"
#include "container.allocator.h"


namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

template<class T> class Set;
template<class T> class Multiset;

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> struct SetNode {
	T				m_Item;
	unsigned int	m_Rand;
	SetNode<T>*			m_LSon;
	SetNode<T>*			m_RSon;
	SetNode<T>*			m_Father;
};


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> class SetIterator : public IteratorBase<SetIterator<T> > {
friend class Set<T>;
friend class Multiset<T>;
public:
	inline SetIterator();
	
	inline			operator T*();
	inline			operator const T*() const;
	inline T&		operator*() const;
	inline T*		operator->() const;

	SetIterator&	operator++();
	inline SetIterator operator++(int );
	SetIterator&	operator--();
	inline SetIterator	operator--(int );
	
	inline bool		operator==(const SetIterator& ) const;
	inline bool		operator<(const SetIterator& ) const;

private:
	inline SetIterator(SetNode<T>* );

private:
	SetNode<T>*	m_Node;
};


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
template<class T> class Set : public Object {
DeclareClass(Set, Object);
friend class Multiset<T>;
public:
	typedef SetIterator<T>	Iterator;
	
public:
	inline Set();
	inline ~Set();

	inline bool		contains(const T& item) const;
	Iterator		insert(const T& item);
	bool			remove(const T& item);
	bool			remove(const Iterator& item);
	void			clear();

	inline bool		isEmpty() const;
	inline unsigned int getSize() const;

	inline Iterator	begin() const;
	inline Iterator at(const T& item) const;
	inline Iterator	end() const;

	bool			operator==(const Set& ) const;
	bool			operator!=(const Set& ) const;

private:
	void			removeNode(SetNode<T> *node, SetNode<T> **father);
	void			repairUp(SetNode<T> *node);
	inline unsigned int rand();

private:
	SetNode<T>*			m_Root;
	unsigned int		m_Size;
	unsigned int		m_Rand;
	Allocator<SetNode<T> > m_Allocator;
};


}; };
#endif
