/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_STACK_H
#define __CONTAINER_STACK_H

#include "object.h"
#include "container.iterator.h"

namespace magnum { namespace container {
using magnum::object::Object;
using magnum::types::String;

template<class T,int size> class Stack;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class StackIterator : public IteratorBase<StackIterator<T> > {
template<class T,int size> friend class Stack;
public:
	inline StackIterator();
	inline 					operator T*();
	inline 					operator const T*() const;
	inline T& 				operator*() const;
	inline T* 				operator->() const;
	inline StackIterator&	operator++();
	inline StackIterator	operator++(int );
	inline StackIterator&	operator+=(int n);
	inline StackIterator&	operator--();
	inline StackIterator	operator--(int );
	inline StackIterator&	operator-=(int n);
	inline StackIterator 	operator+(int n) const;
	inline StackIterator 	operator-(int n) const;
	inline bool				operator==(const StackIterator& iter) const;
	inline bool				operator<(const StackIterator& iter) const;

private:
	inline StackIterator(T* item);

private:
	T*	m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int size=64> class Stack : public Object {
DeclareClass(Stack, Object);
public:
	typedef StackIterator<T>	Iterator;
	
public:
	Stack();

	inline void    		push(const T& );
	inline T 			pop(void);
	inline T&			peek(void);
	inline const T&		peek(void) const;
						
	inline void    		clear(void);
	inline bool    		isEmpty(void) const;
	inline bool    		isFull(void) const;
	inline unsigned int	getSize() const;

	inline Iterator		begin() const;
	inline Iterator		at(unsigned int ) const;
	inline Iterator		end() const;

	inline T&			operator[](unsigned int );
	inline const T&		operator[](unsigned int ) const;

private:
	T       	 	m_Stack[size];
	unsigned int	m_Position;
};


}; };
#endif
