/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_STACK_INL
#define __CONTAINER_STACK_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T>::StackIterator() {
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T>::StackIterator(T* item) {
	m_Item = item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T>::operator T*() {
	return m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T>::operator const T*() const {
	return m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline T& StackIterator<T>::operator*() const {
	return *m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline T* StackIterator<T>::operator->() const {
	return m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T>& StackIterator<T>::operator++() {
	ASSERT(m_Item != NULL);
	m_Item++;
	return *this;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T> StackIterator<T>::operator++(int ) {
	ASSERT(m_Item != NULL);
	T* item = m_Item;
	m_Item++;
	return StackIterator(item);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T>& StackIterator<T>::operator+=(int n) {
	m_Item += n;
	return iter;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T>& StackIterator<T>::operator--() {
	ASSERT(m_Item != NULL);
	m_Item--;
	return *this;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T> StackIterator<T>::operator--(int ) {
	ASSERT(m_Item != NULL);
	T* item = m_Item;
	m_Item--;
	return StackIterator(item);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T>& StackIterator<T>::operator-=(int n) {
	m_Item -= n;
	return iter;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T> StackIterator<T>::operator+(int n) const {
	return StackIterator(m_Item + n);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline StackIterator<T> StackIterator<T>::operator-(int n) const {
	return StackIterator(m_Item - n);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool StackIterator<T>::operator==(const StackIterator<T>& iter) const {
	return m_Item == iter.m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool StackIterator<T>::operator<(const StackIterator<T>& iter) const {
	return m_Item < iter.m_Item;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline Stack<T,size>::Stack() {
	m_Position = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline void Stack<T,size>::push(const T& t) {
	ASSERT(m_Position < size);
	m_Stack[m_Position] = t;
	m_Position++;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline T Stack<T,size>::pop() {
	ASSERT(m_Position > 0);
	m_Position--;
	return m_Stack[m_Position];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline T& Stack<T,size>::peek() {
	ASSERT(m_Position > 0);
	return m_Stack[m_Position-1];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline const T& Stack<T,size>::peek() const {
	ASSERT(m_Position > 0);
	return m_Stack[m_Position-1];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline void Stack<T,size>::clear() {
	m_Position = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline bool Stack<T,size>::isEmpty() const {
	return (m_Position == 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline bool Stack<T,size>::isFull() const {
	return (m_Position == size);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline unsigned int Stack<T,size>::getSize() const {
	return size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline typename Stack<T,size>::Iterator Stack<T,size>::begin() const {
	return Iterator(const_cast<T*>(&m_Stack[0]));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline typename Stack<T,size>::Iterator Stack<T,size>::at(unsigned int i) const {
	ASSERT(i < m_Position);
	return Iterator(const_cast<T*>(&m_Stack[i]));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline typename Stack<T,size>::Iterator Stack<T,size>::end() const {
	return Iterator(const_cast<T*>(&m_Stack[m_Position]));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline T& Stack<T,size>::operator[](unsigned int i) {
	ASSERT(i < m_Position);
	return m_Stack[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int size> 
inline const T& Stack<T,size>::operator[](unsigned int i) const {
	ASSERT(i < m_Position);
	return m_Stack[i];
}

}; };
#endif
