/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __FREETYPE_FONTFACTORY_H
#define __FREETYPE_FONTFACTORY_H

#include "object.pointer.h"
#include "io.file.h"
#include "io.filefactory.h"
#include "freetype.library.h"
#include "container.array.h"

namespace magnum { namespace freetype {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::io::File;
using magnum::io::FileFactory;
using magnum::container::Array;

class Font;
class Descriptor;

/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
class FontFactory : public Object {
DeclareClass(FontFactory, Object);
private:
	struct FontInfo {
		String				m_File;
		String				m_Style;
		String				m_Family;
	};

	Pointer<FileFactory>	m_FileFactory;
	Array<FontInfo>			m_FontInfo;
	
public:
	FontFactory();
	FontFactory(FileFactory* );
	
	Font*					create(const Descriptor& );
	Font*					create(const Descriptor& ,File* );	
	void					release(Font* );

	void					setFileFactory(FileFactory* );
	inline FileFactory*		getFileFactory() const;
	
private:
	bool					readFontInfo(const char* );

	static unsigned long	streamRead(FT_Stream ,unsigned long ,unsigned char* ,unsigned long );	
	static void				streamClose(FT_Stream );
	static void				copyGlyph(const FT_Bitmap* ,byte* dst, unsigned int pitch);

	static void				copyGlyphGray1(const FT_Bitmap* ,byte* dst, unsigned int pitch);
	static void				copyGlyphGray2(const FT_Bitmap* ,byte* dst, unsigned int pitch);
	static void				copyGlyphGray4(const FT_Bitmap* ,byte* dst, unsigned int pitch);
	static void				copyGlyphGrayN(const FT_Bitmap* ,byte* dst, unsigned int pitch);
};

}; };

#endif
