/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __FUNCTION_DELEGATE_INL
#define __FUNCTION_DELEGATE_INL

namespace magnum { namespace function {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R> 
inline Delegate<R>::Delegate() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R> 
inline Delegate<R>::Delegate(R (*fn)()) {
	m_Stub = &function_stub<&fn>;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R> template<class T>
inline Delegate<R>::Delegate(T* obj, R (T::*method)()) {
	m_Object = obj;
	m_Stub = &method_stub<&method>;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R>
inline R Delegate<R>::operator()() const {
	return (*m_Stub)(m_Object);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R> template<class T,R (T::*method)()>
R Delegate<R>::method_stub(void* obj) {
    T* p = static_cast<T*>(obj);
    return (p->*method)();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R> template<R (*method)()>
R Delegate<R>::function_stub(void* ) {
   	return (*method)();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1> 
inline Delegate1<R,A1>::Delegate1() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1> 
inline Delegate1<R,A1>::Delegate1(R (*fn)(A1)) {
	m_Stub = &function_stub<&fn>;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1> template<class T>
inline Delegate1<R,A1>::Delegate1(T* obj, R (T::*method)(A1)) {
	m_Object = obj;
	m_Stub = &method_stub<&method>;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1>
inline R Delegate1<R,A1>::operator()(A1& a1) const {
	return (*m_Stub)(m_Object,a1);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1> template<class T,R (T::*method)(A1)>
static R Delegate1<R,A1>::method_stub(void* obj, A1& a1) {
    T* p = static_cast<T*>(obj);
    return (p->*method)(a1);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R,class A1> template<R (*method)(A1)>
R Delegate1<R,A1>::function_stub(void* ,A1& a1) {
   	return (*method)(a1);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1, class A2> 
inline Delegate2<R,A1,A2>::Delegate2() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1, class A2> 
inline Delegate2<R,A1,A2>::Delegate2(R (*fn)(A1,A2)) {
	m_Stub = &function_stub<&fn>;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1, class A2> template<class T>
inline Delegate2<R,A1,A2>::Delegate2(T* obj, R (T::*method)(A1,A2)) {
	m_Object = obj;
	m_Stub = &method_stub<&method>;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1, class A2>
inline R Delegate2<R,A1,A2>::operator()(A1& a1,A2& a2) const {
	return (*m_Stub)(m_Object,a1,a2);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1, class A2> template<class T,R (T::*method)(A1,A2)>
static R Delegate2<R,A1,A2>::method_stub(void* obj, A1& a1, A2& a2) {
    T* p = static_cast<T*>(obj);
    return (p->*method)(a1,a2);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R,class A1, class A2> template<R (*method)(A1,A2)>
R Delegate2<R,A1,A2>::function_stub(void* ,A1& a1, A2& a2) {
   	return (*method)(a1,a2);
}







/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1, class A2, class A3> 
inline Delegate3<R,A1,A2,A3>::Delegate3() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1, class A2, class A3> 
inline Delegate3<R,A1,A2,A3>::Delegate3(R (*fn)(A1,A2,A3)) {
	m_Stub = &function_stub<&fn>;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1, class A2, class A3> template<class T>
inline Delegate3<R,A1,A2,A3>::Delegate3(T* obj, R (T::*method)(A1,A2,A3)) {
	m_Object = obj;
	m_Stub = &method_stub<&method>;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1, class A2, class A3>
inline R Delegate3<R,A1,A2,A3>::operator()(A1& a1,A2& a2, A3& a3) const {
	return (*m_Stub)(m_Object,a1,a2,a3);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R, class A1, class A2, class A3> template<class T,R (T::*method)(A1,A2,A3)>
static R Delegate3<R,A1,A2,A3>::method_stub(void* obj, A1& a1, A2& a2, A3& a3) {
    T* p = static_cast<T*>(obj);
    return (p->*method)(a1,a2,a3);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class R,class A1, class A2, class A3> template<R (*method)(A1,A2,A3)>
R Delegate3<R,A1,A2,A3>::function_stub(void* ,A1& a1, A2& a2, A3& a3) {
   	return (*method)(a1,a2,a3);
}

}; };

#endif
