/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __CONTAINER_SEQUENCE_INL
#define __CONTAINER_SEQUENCE_INL

namespace magnum { namespace container {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Sequence::insert(T* obj,void (T::*method)(void)) {
	m_Sequence.insert((intptr_t)(void*)obj);
	m_Sequence.insert((intptr_t)(void*)&Executor0<T>::execute);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class P1>
void Sequence::insert(T* obj,void (T::*method)(P1),const P1& p1) {
	m_Sequence.insert((intptr_t)(void*)obj);
	m_Sequence.insert((intptr_t)(void*)&Executor1<T,P1>::execute);
	m_Sequence.insert((intptr_t)(void*)*(void**)&p1);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class P1,class P2>
void Sequence::insert(T* obj,void (T::*method)(P1,P2),const P1& p1,const P2& p2) {
	m_Sequence.insert((intptr_t)(void*)obj);
	m_Sequence.insert((intptr_t)(void*)&Executor2<T,P1,P2>::execute);
	m_Sequence.insert((intptr_t)(void*)*(void**)&p1);
	m_Sequence.insert((intptr_t)(void*)*(void**)&p2);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class P1,class P2,class P3>
void Sequence::insert(T* obj,void (T::*method)(P1,P2,P3),const P1& p1,const P2& p2,const P3& p3) {
	m_Sequence.insert((intptr_t)(void*)obj);
	m_Sequence.insert((intptr_t)(void*)&Executor2<T,P1,P2,P3>::execute);
	m_Sequence.insert((intptr_t)(void*)*(void**)&p1);
	m_Sequence.insert((intptr_t)(void*)*(void**)&p2);
	m_Sequence.insert((intptr_t)(void*)*(void**)&p3);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
const intptr_t* Sequence::Executor0<T>::execute(const intptr_t* stream) {
	T* obj = (T*)(void*)*stream++;
	void (T::*method)() = *(void (T::**)())(void**)&*stream++;
	(obj->*method)();
	return stream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class P1>
const intptr_t* Sequence::Executor1<T,P1>::execute(const intptr_t* stream) {
	T* obj = (T*)(void*)*stream++;
	void (T::*method)(P1) = *(void (T::**)(P1))(void**)&*stream++;
	const P1& p1 = *(const P1*)(void *)&*stream++;
	(obj->*method)(p1);
	return stream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class P1,class P2>
const intptr_t* Sequence::Executor2<T,P1,P2>::execute(const intptr_t* stream) {
	T* obj = (T*)(void*)*stream++;
	void (T::*method)(P1,P2) = *(void (T::**)(P1,P2))(void**)&*stream++;
	const P1& p1 = *(const P1*)(void *)&*stream++;
	const P2& p2 = *(const P2*)(void *)&*stream++;
	(obj->*method)(p1,p2);
	return stream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,class P1,class P2,class P3>
const intptr_t* Sequence::Executor3<T,P1,P2,P3>::execute(const intptr_t* stream) {
	T* obj = (T*)(void*)*stream++;
	void (T::*method)(P1,P2,P3) = *(void (T::**)(P1,P2,P3))(void**)&*stream++;
	const P1& p1 = *(const P1*)(void *)&*stream++;
	const P2& p2 = *(const P2*)(void *)&*stream++;
	const P3& p3 = *(const P3*)(void *)&*stream++;
	(obj->*method)(p1,p2,p3);
	return stream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Sequence::clear() {
	m_Sequence.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Sequence::isEmpty() const {
	return m_Sequence.isEmpty();
}

}; };

#endif
