/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_ALIGNEDBOX_H
#define __GEOMETRY_ALIGNEDBOX_H

#include "geometry.body.h"
#include "geometry.intersection.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;

template<class T, int dim> class Sphere;
template<class T, int dim> class Box;
template<class T, int dim> class Cube;
template<class T, int dim> class Line;
template<class T, int dim> class Segment;
template<class T, int dim> class Ray;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class AlignedBox : public Body<T,dim> {
public:
	inline				AlignedBox();
	inline				AlignedBox(const Vector<T,dim>& min, const Vector<T,dim>& max);
	 
	inline void			setMinimum(const Vector<T,dim>& );
	inline const Vector<T,dim>&	getMinimum() const;

	inline void			setMaximum(const Vector<T,dim>& );
	inline const Vector<T,dim>&	getMaximum() const;
	
	inline void			setOrigin(const Vector<T,dim>& );
	inline Vector<T,dim> getOrigin() const;

	inline void			setExtents(const T* );
	inline const T*		getExtents() const;
	inline void			setExtent(unsigned int ,const T& );
	inline T			getExtent(unsigned int ) const;

	inline bool			isEmpty() const;
	inline void			empty();

	inline bool			isInside(const Vector<T,dim>& ) const; 
	inline T			getDistance(const Vector<T,dim>& ) const;
	
	bool				intersect(const Sphere<T,dim>& ) const;
	bool				intersect(const AlignedBox<T,dim>& ) const;
	bool				intersect(const Cube<T,dim>& ) const;
	inline bool			intersect(const Box<T,dim>& ) const;

	bool				intersect(const Line<T,dim>& ,Intersection<T,dim>* = NULL) const;
	bool				intersect(const Segment<T,dim>& ,Intersection<T,dim>* = NULL) const;
	bool				intersect(const Ray<T,dim>& ,Intersection<T,dim>* = NULL) const;

	Box<T,dim>			toBox() const;
	Sphere<T,dim>		toSphere() const;
	Cube<T,dim>			toCube() const;

	inline void			bound(const Vector<T,dim>* ,unsigned int n);
	void				bound(const Vector<T,dim>* ,unsigned int stride,unsigned int n);
	
	void				merge(const AlignedBox<T,dim>& );
	
	void				getVertices(Vector<T,dim>* ) const;
	
private:
	template<class L>
	bool				intersectLinear(const L& ,Intersection<T,dim>* ) const;

private:
	Vector<T,dim>		m_Minimum;
	Vector<T,dim>		m_Maximum;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class AlignedBox2 : public AlignedBox<T,2> {
public:
	inline	AlignedBox2() {};
	inline	AlignedBox2(const AlignedBox<T,2>& box) : AlignedBox<T,2>(box) {};
	inline	AlignedBox2(const Vector<T,2>& min, const Vector<T,2>& max) : AlignedBox<T,2>(min,max) {};
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class AlignedBox3 : public AlignedBox<T,3> {
public:
	inline	AlignedBox3() {};
	inline	AlignedBox3(const AlignedBox<T,3>& box) : AlignedBox<T,3>(box) {};
	inline	AlignedBox3(const Vector<T,3>& min, const Vector<T,3>& max) : AlignedBox<T,3>(min,max) {};
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class AlignedBox4 : public AlignedBox<T,4> {
public:
	inline	AlignedBox4() {};
	inline	AlignedBox4(const AlignedBox<T,4>& box) : AlignedBox<T,4>(box) {};
	inline	AlignedBox4(const Vector<T,4>& min, const Vector<T,4>& max) : AlignedBox<T,4>(min,max) {};
};

}; };

#endif
