/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_ATTRIBUTEGENERATOR_H
#define __GEOMETRY_ATTRIBUTEGENERATOR_H

#include "object.h"
#include "math.vector3.h"

namespace magnum { namespace geometry {
using magnum::math::Vector3;
using magnum::object::Object;
using magnum::types::String;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I> class AttributeGenerator : public Object {
DeclareClass(AttributeGenerator, Object);
public:
	enum Primitive {
		TriangleList,
		TriangleStrip,
		TriangleFan,
	};
	
public:
	AttributeGenerator();
	
	inline Primitive	getPrimitive() const;
	inline void			setPrimitive(Primitive );
	
	inline unsigned int getVertexStride() const;
	inline const Vector3<T>* getVertices() const;
	inline void			setVertices(const Vector3<T>* ,unsigned int = sizeof(Vector3<T>));

	inline unsigned int	getIndexSize() const;
	inline const I*		getIndices() const;
	inline void			setIndices(const I* ,unsigned int);
	
	void				generate();
	
protected:
	inline const Vector3<T>& getVertex(unsigned int );
	unsigned int		getPrimitiveCount() const;
	void				getTriangle(unsigned int ,unsigned int* ,unsigned int* ,unsigned int* ) const;

private:
	const Vector3<T>*	m_Vertices;
	unsigned int		m_VertexStride;
	const I*			m_Indices;
	unsigned int		m_Size;
	Primitive			m_Primitive;
};

}; };

#endif
