/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_ATTRIBUTEGENERATOR_INL
#define __GEOMETRY_ATTRIBUTEGENERATOR_INL

namespace magnum { namespace geometry {
using magnum::container::Array;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
AttributeGenerator<T,I>::AttributeGenerator() {
	m_Vertices = NULL;
	m_VertexStride = 0;
	m_Indices = NULL;
	m_Size = 0;
	m_Primitive = TriangleList;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline typename AttributeGenerator<T,I>::Primitive AttributeGenerator<T,I>::getPrimitive() const {
	return m_Primitive;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline void AttributeGenerator<T,I>::setPrimitive(typename AttributeGenerator<T,I>::Primitive p) {
	m_Primitive = p;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline unsigned int AttributeGenerator<T,I>::getVertexStride() const {
	return m_VertexStride;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline const Vector3<T>* AttributeGenerator<T,I>::getVertices() const {
	return m_Vertices;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline void AttributeGenerator<T,I>::setVertices(const Vector3<T>* v,unsigned int s) {
	m_Vertices = v;
	m_VertexStride = s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline const I* AttributeGenerator<T,I>::getIndices() const {
	return m_Indices;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline unsigned int	AttributeGenerator<T,I>::getIndexSize() const {
	return m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline void AttributeGenerator<T,I>::setIndices(const I* i,unsigned int c) {
	m_Indices = i;
	m_Size = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline const Vector3<T>& AttributeGenerator<T,I>::getVertex(unsigned int i) {
	return *reinterpret_cast<const Vector3<T>*>(reinterpret_cast<const byte*>(m_Vertices)+i*m_VertexStride);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
unsigned int AttributeGenerator<T,I>::getPrimitiveCount() const {
	switch (m_Primitive) {
		case TriangleList:
			return m_Size / 3;
		case TriangleStrip:
			return m_Size - 2;
		case TriangleFan:
			return m_Size - 2;
		default:
			return 0;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
void AttributeGenerator<T,I>::getTriangle(unsigned int i,unsigned int* vi1,unsigned int* vi2,unsigned int* vi3) const {
	switch (m_Primitive) {
		case TriangleList:
			*vi1 = m_Indices[3*i];
			*vi2 = m_Indices[3*i+1];
			*vi3 = m_Indices[3*i+2];
			return;
		case TriangleStrip:
			if (i & 1) {
				*vi1 = m_Indices[i];
				*vi2 = m_Indices[i+2];
				*vi3 = m_Indices[i+1];
			} else {
				*vi1 = m_Indices[i];
				*vi2 = m_Indices[i+1];
				*vi3 = m_Indices[i+2];
			}
			return;
		case TriangleFan:
			*vi1 = m_Indices[0];
			*vi2 = m_Indices[i+1];
			*vi3 = m_Indices[i+2];
			return;
		default:
			return;
	}
}

}; };

#endif
