/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_BOUNDS_H
#define __GEOMETRY_BOUNDS_H

#include "math.matrix.h"
#include "geometry.sphere.h"
#include "geometry.box.h"
#include "geometry.cube.h"
#include "geometry.frustum.h"
#include "geometry.alignedbox.h"
#include "geometry.transform.h"

namespace magnum { namespace geometry {
using magnum::math::Matrix;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Bounds : public Box<T,dim> {
public:
	inline Bounds();
	inline Bounds(const Sphere<T,dim>& );
	inline Bounds(const Box<T,dim>& );
	inline Bounds(const AlignedBox<T,dim>& );
	inline Bounds(const Cube<T,dim>& );
	
	inline bool				isEmpty() const;
	inline bool				isInfinite() const;
	
	inline void				empty();
	inline void				infinite();
	
	inline void				setBounds(const Bounds<T,dim>& );
	inline void				setBounds(const Cube<T,dim>& );
	inline void				setBounds(const Sphere<T,dim>& );
	inline void				setBounds(const Box<T,dim>& );
	inline void				setBounds(const AlignedBox<T,dim>& );

	inline void				merge(const Cube<T,dim>& );
	inline void				merge(const Sphere<T,dim>& );
	inline void				merge(const AlignedBox<T,dim>& );
	inline void				merge(const Box<T,dim>& );
	
	inline Bounds<T,dim>& 	translate(const Vector<T,dim>& );
	Bounds<T,dim>&			rotate(const Matrix<T,dim>& );
	Bounds<T,dim>& 			transform(const Matrix<T,dim+1>& );
	inline Bounds<T,dim>&	transform(const Transform<T,dim>& );

	inline const Bounds<T,dim>& operator=(const Bounds<T,dim>& );
	inline const Bounds<T,dim>& operator=(const Cube<T,dim>& );
	inline const Bounds<T,dim>& operator=(const Sphere<T,dim>& );
	inline const Bounds<T,dim>& operator=(const Box<T,dim>& );
	inline const Bounds<T,dim>& operator=(const AlignedBox<T,dim>& );
};

// Some Transformations for some common objects
template<class T, int dim> inline Bounds<T,dim> operator*(const Transform<T,dim>& ,const Bounds<T,dim>& );


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Bounds2 : public Bounds<T,2> {
//DeclareClass(Bounds2, Bounds);
public:
	inline Bounds2() {};
	inline Bounds2(const Sphere<T,2>& s) : Bounds<T,2>(s) {};
	inline Bounds2(const Box<T,2>& b) : Bounds<T,2>(b) {};
	inline Bounds2(const AlignedBox<T,2>& b) : Bounds<T,2>(b) {};
	inline Bounds2(const Cube<T,2>& b) : Bounds<T,2>(b) {};
	inline Bounds2(const Bounds<T,2>& b) : Bounds<T,2>(b) {};
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Bounds3 : public Bounds<T,3> {
//DeclareClass(Bounds3, Bounds);
public:
	inline Bounds3() {};
	inline Bounds3(const Sphere<T,3>& s) : Bounds<T,3>(s) {};
	inline Bounds3(const Box<T,3>& b) : Bounds<T,3>(b) {};
	inline Bounds3(const AlignedBox<T,3>& b) : Bounds<T,3>(b) {};
	inline Bounds3(const Cube<T,3>& b) : Bounds<T,3>(b) {};
	inline Bounds3(const Bounds<T,3>& b) : Bounds<T,3>(b) {};
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Bounds4 : public Bounds<T,4> {
//DeclareClass(Bounds4, Bounds);
public:
	inline Bounds4() {};
	inline Bounds4(const Sphere<T,4>& s) : Bounds<T,4>(s) {};
	inline Bounds4(const Box<T,4>& b) : Bounds<T,4>(b) {};
	inline Bounds4(const AlignedBox<T,4>& b) : Bounds<T,4>(b) {};
	inline Bounds4(const Cube<T,4>& b) : Bounds<T,4>(b) {};
	inline Bounds4(const Bounds<T,4>& b) : Bounds<T,4>(b) {};
};

}; };

#endif
