/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_BOUNDS_INL
#define __GEOMETRY_BOUNDS_INL

namespace magnum { namespace geometry {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Bounds<T,dim>::Bounds() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Bounds<T,dim>::Bounds(const Sphere<T,dim>& s) : Box<T,dim>(s.toBox()) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Bounds<T,dim>::Bounds(const AlignedBox<T,dim>& b) : Box<T,dim>(b.toBox()) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Bounds<T,dim>::Bounds(const Cube<T,dim>& b) : Box<T,dim>(b.toBox()) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Bounds<T,dim>::Bounds(const Box<T,dim>& b) : Box<T,dim>(b) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void Bounds<T,dim>::empty() {
	Box<T,dim>::empty();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void Bounds<T,dim>::infinite() {
	ASSERT(false);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Bounds<T,dim>::setBounds(const Bounds<T,dim>& bounds) {
	*this = bounds;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Bounds<T,dim>::setBounds(const Sphere<T,dim>& sphere) {
	*(Box<T,dim>*)this = sphere.toBox();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Bounds<T,dim>::setBounds(const Box<T,dim>& box) {
	*(Box<T,dim>*)this = box;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Bounds<T,dim>::setBounds(const AlignedBox<T,dim>& box) {
	*(Box<T,dim>*)this = box.toBox();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Bounds<T,dim>::isEmpty() const {
	return Box<T,dim>::isEmpty();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Bounds<T,dim>::isInfinite() const {
	ASSERT(false);
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Bounds<T,dim>& Bounds<T,dim>::translate(const Vector<T,dim>& v) {
	setOrigin(getOrigin() + v);
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
Bounds<T,dim>& Bounds<T,dim>::rotate(const Matrix<T,dim>& mtx) {
	setOrigin(mtx * getOrigin());
	for (unsigned int i = 0; i < dim; i++)
		setAxis(i, mtx * getAxis(i));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
Bounds<T,dim>& Bounds<T,dim>::transform(const Matrix<T,dim+1>& mtx) {
	// Transform origin
	Vector<T,dim> result;
	Vector<T,dim> v = getOrigin();
	if (dim == 2) {
		result[0] = mtx[0][0]*v[0] + mtx[0][1]*v[1] + mtx[0][2];
		result[1] = mtx[1][0]*v[0] + mtx[1][1]*v[1] + mtx[1][2];
	} else if (dim == 3) {
		result[0] = mtx[0][0]*v[0] + mtx[0][1]*v[1] + mtx[0][2]*v[2] + mtx[0][3];
		result[1] = mtx[1][0]*v[0] + mtx[1][1]*v[1] + mtx[1][2]*v[2] + mtx[1][3];
		result[2] = mtx[2][0]*v[0] + mtx[2][1]*v[1] + mtx[2][2]*v[2] + mtx[2][3];
	} else {
		for (unsigned int i = 0; i < dim; i++) {
			T tmp = mtx[i][dim];
			for (unsigned int j = 0; j < dim; j++) 
				tmp += mtx[i][j]*v[j];
			result[i] = tmp;	
		}
	}
	setOrigin(result);

	// Transform axes
	for (unsigned int k = 0; k < dim; k++) {
		Vector<T,dim> v = getAxis(k);
		if (dim == 2) {
			result[0] = mtx[0][0]*v[0] + mtx[0][1]*v[1];
			result[1] = mtx[1][0]*v[0] + mtx[1][1]*v[1];
		} else if (dim == 3) {
			result[0] = mtx[0][0]*v[0] + mtx[0][1]*v[1] + mtx[0][2]*v[2];
			result[1] = mtx[1][0]*v[0] + mtx[1][1]*v[1] + mtx[1][2]*v[2];
			result[2] = mtx[2][0]*v[0] + mtx[2][1]*v[1] + mtx[2][2]*v[2];
		} else {
			for (unsigned int i = 0; i < dim; i++) {
				T tmp = mtx[i][0]*v[0];
				for (unsigned int j = 1; j < dim; j++) 
					tmp += mtx[i][j]*v[j];
				result[i] = tmp;	
			}
		}
		setAxis(k,result);
	}
	
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Bounds<T,dim>& Bounds<T,dim>::transform(const Transform<T,dim>& t) {
	return transform(t.getTransform());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Bounds<T,dim>::merge(const Sphere<T,dim>& s) {
	Box<T,dim>::merge(s.toBox());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Bounds<T,dim>::merge(const AlignedBox<T,dim>& b) {
	Box<T,dim>::merge(b.toBox());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Bounds<T,dim>::merge(const Box<T,dim>& b) {
	Box<T,dim>::merge(b);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Bounds<T,dim>&	Bounds<T,dim>::operator=(const Bounds<T,dim>& b) {
	*(Box<T,dim>*)this = (Box<T,dim>)b;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Bounds<T,dim>&	Bounds<T,dim>::operator=(const Sphere<T,dim>& s) {
	*(Box<T,dim>*)this = s.toBox();
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Bounds<T,dim>&	Bounds<T,dim>::operator=(const Box<T,dim>& b) {
	*(Box<T,dim>*)this = b;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Bounds<T,dim>&	Bounds<T,dim>::operator=(const AlignedBox<T,dim>& b) {
	*(Box<T,dim>*)this = b.toBox();
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Bounds<T,dim> operator*(const Transform<T,dim>& t, const Bounds<T,dim>& b) {
	Bounds<T,dim> r(b);
	r.transform(t);
	return r;
}

}; };

#endif
