/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_BOX_H
#define __GEOMETRY_BOX_H

#include "math.limits.h"
#include "geometry.body.h"
#include "geometry.intersection.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;
using magnum::math::Limits;

template<class T, int dim> class Sphere;
template<class T, int dim> class AlignedBox;
template<class T, int dim> class Cube;
template<class T, int dim> class Line;
template<class T, int dim> class Ray;
template<class T, int dim> class Segment;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Box : public Body<T,dim> {
public:
	inline Box();
	inline Box(const Vector<T,dim>& ,const Vector<T,dim>* );

	inline void			setOrigin(const Vector<T,dim>& );
	inline const Vector<T,dim>& getOrigin() const;

	inline void			setAxes(const Vector<T,dim>* );
	inline const Vector<T,dim>* getAxes() const;
	inline void			setAxis(unsigned int ,const Vector<T,dim>& );
	inline const Vector<T,dim>& getAxis(unsigned int ) const;
	inline const T&		getAxis2(unsigned int ) const;

	inline bool			isEmpty() const; 
	inline void			empty(); 

	inline bool			isInside(const Vector<T,dim>& ) const; 
	T					getDistance(const Vector<T,dim>& ) const;
	
	bool				intersect(const Sphere<T,dim>& ) const;
	bool				intersect(const AlignedBox<T,dim>& ) const;
	bool				intersect(const Box<T,dim>& ) const;
	bool				intersect(const Cube<T,dim>& ) const;

	bool				intersect(const Line<T,dim>& ,Intersection<T,dim>* = NULL) const;
	bool				intersect(const Segment<T,dim>& ,Intersection<T,dim>* = NULL) const;
	bool				intersect(const Ray<T,dim>& ,Intersection<T,dim>* = NULL) const;

	Cube<T,dim>			toCube() const;
	void				toCube(Cube<T,dim>* ) const;
	Sphere<T,dim>		toSphere() const;
	void				toSphere(Sphere<T,dim>* ) const;
	AlignedBox<T,dim>	toAlignedBox() const;
	void				toAlignedBox(AlignedBox<T,dim>* ) const;

	void				bound(const Vector<T,dim>* ,unsigned int n);
	void				bound(const Vector<T,dim>* ,unsigned int stride,unsigned int n);
	
	void				merge(const Cube<T,dim>& );
	void				merge(const AlignedBox<T,dim>& );
	void				merge(const Box<T,dim>& );

	void				getVertices(Vector<T,dim>* ) const;

private:
	template<class L>
	bool				intersectLinear(const L& ,Intersection<T,dim>* ) const;

private:
	Vector<T,dim>		m_Origin;
	Vector<T,dim>		m_Axes[dim];
	T					m_Axes2[dim];
};


template<class T> class Box2 : public Box<T,2> {};
template<class T> class Box3 : public Box<T,3> {};
template<class T> class Box4 : public Box<T,4> {};


}; };

#endif
