/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_CAPSULE_INL
#define __GEOMETRY_CAPSULE_INL

namespace magnum { namespace geometry {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline Capsule<T,dim>::Capsule() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline Capsule<T,dim>::Capsule(const Vector<T,dim>& o,const Vector<T,dim>& d, const T& r) {
	m_Segment.setOrigin(o);
	m_Segment.setDirection(d);
	m_Radius = r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void Capsule<T,dim>::setOrigin(const Vector<T,dim>& o) {
	m_Segment.setOrigin(o);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline const Vector<T,dim>& Capsule<T,dim>::getOrigin() const {
	return m_Segment.getOrigin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void Capsule<T,dim>::setDirection(const Vector<T,dim>& d) {
	m_Segment.setDirection(d);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline const Vector<T,dim>& Capsule<T,dim>::getDirection() const {
	return m_Segment.getDirection();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void Capsule<T,dim>::setRadius(const T& r) {
	m_Radius = r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline T Capsule<T,dim>::getRadius() const {
	return m_Radius;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline void Capsule<T,dim>::setLength(const T& r) {
	m_Segment.setLength(r);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline T Capsule<T,dim>::getLength() const {
	return m_Segment.getLength();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline bool Capsule<T,dim>::isEmpty() const {
	return (m_Radius == T(0.0));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline bool Capsule<T,dim>::isInside(const Vector<T,dim>& v) const {
	return (m_Segment.getDistance2(v) <= m_Radius * m_Radius);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline T Capsule<T,dim>::getDistance(const Vector<T,dim>& v) const {
	return m_Segment.getDistance(v) - m_Radius;
}


}; };

#endif
