/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_CYLINDER_H
#define __GEOMETRY_CYLINDER_H

#include "math.vector3.h"
#include "geometry.body.h"

namespace magnum { namespace geometry {
using magnum::math::Vector3;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Cylinder3 : public Body<T,3> {
private:
	Vector3<T>			m_Origin;
	Vector3<T>			m_Direction;
	T					m_Radius;

public:
	inline Cylinder3();
	inline Cylinder3(const Vector3<T>& o, const Vector3<T>& d, const T& r);

	inline void			setOrigin(const Vector3<T>& );
	inline const Vector3<T>& getOrigin() const;

	inline void			setDirection(const Vector3<T>& );
	inline const Vector3<T>& getDirection() const;

	inline void			setRadius(const T& );
	inline T			getRadius() const;
	
	inline void			setLength(const T& );
	inline T			getLength() const;

	inline bool			isEmpty() const;
						
	inline bool			isInside(const Vector3<T>& ) const; 
	inline T			getDistance(const Vector3<T>& ) const;
						
	void				bound(const Vector3<T>* ,unsigned int n);
	void				bound(const Vector3<T>* ,unsigned int stride,unsigned int n);
};

}; };

#endif
