/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_FRUSTUM_H
#define __GEOMETRY_FRUSTUM_H

#include "math.vector.h"
#include "math.vector2.h"
#include "math.vector3.h"
#include "geometry.plane.h"
#include "geometry.body.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;
using magnum::math::Vector3;
using magnum::math::Vector2;

template<class T, int dim> class Sphere;
template<class T, int dim> class Box;
template<class T, int dim> class AlignedBox;
template<class T, int dim> class Cube;
template<class T, int dim> class Bounds;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Frustum3 : public Body<T,3> {
public:
	inline Frustum3();
	
	void				setOrigin(const Vector3<T>& );
	inline const Vector3<T>& getOrigin() const;

	void				setAxes(const Vector3<T>* );
	inline const Vector3<T>* getAxes() const;
	void				setAxis(unsigned int ,const Vector3<T>& );
	inline const Vector3<T>& getAxis(unsigned int ) const;
	
	void				setDirection(const Vector3<T>& );
	inline const Vector3<T>& getDirection() const;

	void				setOffset(const Vector2<T>& );
	inline const Vector2<T>& getOffset() const;
	
	void				setNear(const T& );
	inline const T&		getNear() const;

	void				setFar(const T& );
	inline const T&		getFar() const;

	bool				isEmpty() const;
	
	void				getVertices(Vector3<T>* ) const;
						
	bool				isInside(const Vector3<T>& ) const; 
	T					getDistance(const Vector3<T>& ) const;

	inline bool			intersect(const Sphere<T,3>& ) const;
	bool				intersect(const Cube<T,3>& ) const;
	bool				intersect(const Box<T,3>& ) const;
	bool				intersect(const AlignedBox<T,3>& ) const;
	inline bool			intersect(const Bounds<T,3>& ) const;
	
	Containment			cull(const Cube<T,3>& ) const;
	Containment			cull(const Bounds<T,3>& ) const;

private:
	Vector3<T>			m_Origin;
	Vector3<T>			m_Direction;
	Vector2<T>			m_Offset;
	T					m_Near;
	T					m_Far;

	// Derived values
	Vector3<T>			m_NearOrigin;
	Vector3<T>			m_FarOrigin;
	Vector3<T>			m_Axes[2];
	T					m_Axes2[2];
	Vector3<T>			m_OffsetAxes[2];
	T					m_OffsetAxes2[2];
	Vector3<T>			m_NearOffsetOrigin;
	Vector3<T>			m_FarOffsetOrigin;
	T					m_Direction2;
	T					m_NearDirection2;
	T					m_FarDirection2;
	T					m_FarByNear;
	Vector3<T>			m_PlaneNormals[4];
};

}; };

#endif
