/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_FRUSTUM_INL
#define __GEOMETRY_FRUSTUM_INL

namespace magnum { namespace geometry {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Frustum3<T>::Frustum3() {
	m_Offset.null();
	m_Origin.null();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Frustum3<T>::setOrigin(const Vector3<T>& v) {
	m_Origin = v;
	m_NearOrigin = m_Origin + m_Near*m_Direction;
	m_FarOrigin = m_Origin + m_Far*m_Direction;
	m_NearOffsetOrigin = m_NearOrigin + m_OffsetAxes[0] + m_OffsetAxes[1];
	m_FarOffsetOrigin = m_FarOrigin  + m_FarByNear*(m_OffsetAxes[0] + m_OffsetAxes[1]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Vector3<T>& Frustum3<T>::getOrigin() const {
	return m_Origin;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Frustum3<T>::setAxes(const Vector3<T>* e) {
	m_Axes[0] = e[0];
	m_Axes[1] = e[1];
	m_Axes2[0] = e[0] * e[0];
	m_Axes2[1] = e[1] * e[1];
	m_OffsetAxes[0] = m_Offset[0] * m_Axes[0];
	m_OffsetAxes2[0] = m_Offset[0] * m_Axes2[0];
	m_OffsetAxes[1] = m_Offset[1] * m_Axes[1];
	m_OffsetAxes2[1] = m_Offset[1] * m_Axes2[1];
	m_NearOffsetOrigin = m_NearOrigin + m_OffsetAxes[0] + m_OffsetAxes[1];
	m_FarOffsetOrigin = m_FarOrigin  + m_FarByNear*(m_OffsetAxes[0] + m_OffsetAxes[1]);
	
	m_PlaneNormals[0] = m_NearDirection2*m_Axes[0] - (m_Offset[0]+1)*m_Axes2[0]*m_Direction;
	m_PlaneNormals[1] = -m_NearDirection2*m_Axes[0] + (m_Offset[0]-1)*m_Axes2[0]*m_Direction;
	m_PlaneNormals[2] = m_NearDirection2*m_Axes[1] - (m_Offset[1]+1)*m_Axes2[1]*m_Direction;
	m_PlaneNormals[3] = -m_NearDirection2*m_Axes[1] + (m_Offset[1]-1)*m_Axes2[1]*m_Direction;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const Vector3<T>* Frustum3<T>::getAxes() const {
	return m_Axes;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Frustum3<T>::setAxis(unsigned int i, const Vector3<T>& a) {
	ASSERT(i < 2);
	m_Axes[i] = a;
	m_Axes2[i] = a*a;
	m_OffsetAxes[i] = m_Offset[i] * m_Axes[i];
	m_OffsetAxes2[i] = m_Offset[i] * m_Axes2[i];
	m_NearOffsetOrigin = m_NearOrigin + m_OffsetAxes[0] + m_OffsetAxes[1];
	m_FarOffsetOrigin = m_FarOrigin  + m_FarByNear*(m_OffsetAxes[0] + m_OffsetAxes[1]);
	m_PlaneNormals[2*i+0] = m_NearDirection2*m_Axes[i] - (m_Offset[i]+1)*m_Axes2[i]*m_Direction;
	m_PlaneNormals[2*i+1] = -m_NearDirection2*m_Axes[i] + (m_Offset[i]-1)*m_Axes2[i]*m_Direction;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const Vector3<T>& Frustum3<T>::getAxis(unsigned int i) const {
	ASSERT(i < 2);
	return m_Axes[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Frustum3<T>::setDirection(const Vector3<T>& v) {
	m_Direction = v;
	m_Direction2 = v*v;
	m_NearDirection2 = m_Direction2 * m_Near;
	m_FarDirection2 = m_Direction2 * m_Far;
	m_NearOrigin = m_Origin + m_Near*m_Direction;
	m_FarOrigin = m_Origin + m_Far*m_Direction;
	m_NearOffsetOrigin = m_NearOrigin + m_OffsetAxes[0] + m_OffsetAxes[1];
	m_FarOffsetOrigin = m_FarOrigin  + m_FarByNear*(m_OffsetAxes[0] + m_OffsetAxes[1]);

	m_PlaneNormals[0] = m_NearDirection2*m_Axes[0] - (m_Offset[0]+1)*m_Axes2[0]*m_Direction;
	m_PlaneNormals[1] = -m_NearDirection2*m_Axes[0] + (m_Offset[0]-1)*m_Axes2[0]*m_Direction;
	m_PlaneNormals[2] = m_NearDirection2*m_Axes[1] - (m_Offset[1]+1)*m_Axes2[1]*m_Direction;
	m_PlaneNormals[3] = -m_NearDirection2*m_Axes[1] + (m_Offset[1]-1)*m_Axes2[1]*m_Direction;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Vector3<T>& Frustum3<T>::getDirection() const {
	return m_Direction;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Frustum3<T>::setOffset(const Vector2<T>& v) {
	m_Offset = v;
	m_OffsetAxes[0] = m_Offset[0] * m_Axes[0];
	m_OffsetAxes2[0] = m_Offset[0] * m_Axes2[0];
	m_OffsetAxes[1] = m_Offset[1] * m_Axes[1];
	m_OffsetAxes2[1] = m_Offset[1] * m_Axes2[1];
	m_NearOffsetOrigin = m_NearOrigin + m_OffsetAxes[0] + m_OffsetAxes[1];
	m_FarOffsetOrigin = m_FarOrigin  + m_FarByNear*(m_OffsetAxes[0] + m_OffsetAxes[1]);

	m_PlaneNormals[0] = m_NearDirection2*m_Axes[0] - (m_Offset[0]+1)*m_Axes2[0]*m_Direction;
	m_PlaneNormals[1] = -m_NearDirection2*m_Axes[0] + (m_Offset[0]-1)*m_Axes2[0]*m_Direction;
	m_PlaneNormals[2] = m_NearDirection2*m_Axes[1] - (m_Offset[1]+1)*m_Axes2[1]*m_Direction;
	m_PlaneNormals[3] = -m_NearDirection2*m_Axes[1] + (m_Offset[1]-1)*m_Axes2[1]*m_Direction;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Vector2<T>& Frustum3<T>::getOffset() const {
	return m_Offset;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Frustum3<T>::setNear(const T& n) {
	m_Near = n;
	m_FarByNear = m_Far / m_Near;
	m_NearDirection2 = m_Direction2 * m_Near;
	m_NearOrigin = m_Origin + m_Near*m_Direction;
	m_NearOffsetOrigin = m_NearOrigin + m_OffsetAxes[0] + m_OffsetAxes[1];

	m_PlaneNormals[0] = m_NearDirection2*m_Axes[0] - (m_Offset[0]+1)*m_Axes2[0]*m_Direction;
	m_PlaneNormals[1] = -m_NearDirection2*m_Axes[0] + (m_Offset[0]-1)*m_Axes2[0]*m_Direction;
	m_PlaneNormals[2] = m_NearDirection2*m_Axes[1] - (m_Offset[1]+1)*m_Axes2[1]*m_Direction;
	m_PlaneNormals[3] = -m_NearDirection2*m_Axes[1] + (m_Offset[1]-1)*m_Axes2[1]*m_Direction;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const T& Frustum3<T>::getNear() const {
	return m_Near;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Frustum3<T>::setFar(const T& f) {
	m_Far = f;
	m_FarByNear = m_Far / m_Near;
	m_FarDirection2 = m_Direction2 * m_Far;
	m_FarOrigin = m_Origin + m_Far*m_Direction;
	m_FarOffsetOrigin = m_FarOrigin  + m_FarByNear*(m_OffsetAxes[0] + m_OffsetAxes[1]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const T& Frustum3<T>::getFar() const {
	return m_Far;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
bool Frustum3<T>::isEmpty() const {
	if (m_Near == m_Far)
		return true;
	if ((m_Axes2[0] <= T(0)) || (m_Axes2[0] <= T(0)))
		return true;

	return false;
}
					

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Frustum3<T>::getVertices(Vector3<T>* v) const {
	unsigned int i,j;
	Vector3<T> center;

	// Get near plane vertices	
	center = m_NearOrigin + m_OffsetAxes[0] + m_OffsetAxes[1];
	for (i = 0; i < (1 << 2); i++) {
		*v = center;
		for (j = 0; j < 2; j++) {
			if (i & (1 << j))
				*v += m_Axes[j];
			else
				*v -= m_Axes[j];
		}
		v++
	}

	// Get far plane vertices
	center = m_FarOrigin + m_FarByNear * (m_OffsetAxes[0] + m_OffsetAxes[1]);
	for (i = 0; i < (1 << 2); i++) {
		*v = center;
		for (j = 0; j < 2; j++) {
			if (i & (1 << j))
				*v += m_FarByNear*m_Axes[j];
			else
				*v -= m_FarByNear*m_Axes[j];
		}
		v++
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
bool Frustum3<T>::isInside(const Vector3<T>& v) const {
	// Check against near and far plane
	T t = (v - m_Origin) * m_Direction;
	if ((t <= m_NearDirection2) || (t >= m_FarDirection2))
		return false;
		
	// Project v onto projection hyperplane
	Vector3<T> p = (m_NearDirection2 / t) * (vec - m_Origin);
	T l;
	l = m_Axes[0]*p - m_OffsetAxes2[0];
	if ((l < -m_Axes2[0]) || (l > m_Axes2[0]))
		return false;
	l = m_Axes[1]*p - m_OffsetAxes2[1];
	if ((l < -m_Axes2[1]) || (l > m_Axes2[1]))
		return false;

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
T Frustum3<T>::getDistance(const Vector3<T>& v) const {
	// Project v onto projection hyperplane
	T t = (v - m_Origin) * m_Direction;
	Vector3<T> p = (m_NearDirection2 / abs(t)) * (v - m_Origin);

	// Find nearest point on hyperplane
	Vector3<T> best;
	best.null();
	for (unsigned int i = 0; i < 2; i++) {
		T l = (p*m_Axes[i])/m_Axes2[i] - m_Offset[i]; 
		if (l < T(-1)) 
			l = T(-1);
		else if (l > T(1))
			l = T(1);
		best += (l + m_Offset[i]) * m_Axes[i];
	}

	// Check distances
	if (t < m_NearDirection2) {
		best += m_NearOrigin;
	} else if (t > m_FarDirection2) {
		// TODO: Bug, this is not really correct. We should
		// find a point perpendicular to the closest plane
		best = m_FarOrigin + m_FarByNear * best;
	} else {
		t /= m_Direction2;
		best = m_Origin + t/m_Near*best + t * m_Direction;
	}
	
	return (best - v).length();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool Frustum3<T>::intersect(const Sphere<T,3>& sphere) const {
	return getDistance(sphere.getOrigin()) <= sphere.getRadius();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
bool Frustum3<T>::intersect(const Cube<T,3>& cube) const {
	unsigned int i;
	Vector3<T> p = cube.getOrigin();
	T radius = cube.getRadius();
	T t,l,m;
	T mi, ma;

	// Project Frustum onto cube axes
	for (i = 0; i < 3; i++) {
		// Project Frustum onto ith axes of cube
		t = abs(m_Axes[0][i]) + abs(m_Axes[1][i]);

		l = m_FarByNear * t;
		mi = min(m_NearOffsetOrigin[i] - t, m_FarOffsetOrigin[i] - l);
		ma = max(m_NearOffsetOrigin[i] + t, m_FarOffsetOrigin[i] + l);
		
		if ((ma <= p[i] - radius) || (mi >= p[i] + radius))
			return false;
	}

	// Project Cube onto Direction Vector
	p -= m_Origin;
	t = m_Direction * p;
	m = m_Direction.sumabs() * radius;
	if ((t - m >= m_FarDirection2) || (t + m <= m_NearDirection2))
		return false;
		
	// Project Cube onto plane normals
	if (p*m_PlaneNormals[0] >= m_PlaneNormals[0].sumabs()*radius)
		return false;
	if (p*m_PlaneNormals[1] >= m_PlaneNormals[1].sumabs()*radius)
		return false;
	if (p*m_PlaneNormals[2] >= m_PlaneNormals[2].sumabs()*radius)
		return false;
	if (p*m_PlaneNormals[3] >= m_PlaneNormals[3].sumabs()*radius)
		return false;
	
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
bool Frustum3<T>::intersect(const AlignedBox<T,3>& box) const {
	unsigned int i;
	Vector3<T> bmin = box.getMinimum();
	Vector3<T> bmax = box.getMaximum();
	T t,l,m;
	T mi, ma;

	// Project Frustum onto aligned box axes
	for (i = 0; i < 3; i++) {
		// Project Frustum onto ith axes of box
		t = abs(m_Axes[0][i]) + abs(m_Axes[1][i]);

		l = m_FarByNear * t;
		mi = min(m_NearOffsetOrigin[i] - t, m_FarOffsetOrigin[i] - l);
		ma = max(m_NearOffsetOrigin[i] + t, m_FarOffsetOrigin[i] + l);
		
		if ((ma <= bmin[i]) || (mi >= bmax[i]))
			return false;
	}

	// Project box onto Direction Vector
	Vector3<T> bdiff = bmax - bmin;
	Vector3<T> bcent = bmax + bmin - T(2)*m_Origin;
	t = m_Direction * bcent;
	m = abs(m_Direction[0] * bdiff[0]) + abs(m_Direction[1] * bdiff[1]) + abs(m_Direction[2] * bdiff[2]);
	if ((t - m >= 2*m_FarDirection2) || (t + m <= 2*m_NearDirection2))
		return false;
		
	// Project Cube onto plane normals
	for (i = 0; i < 4; i++) {
		Vector3<T> pnorm = m_PlaneNormals[i];
		m = abs(pnorm[0] * bdiff[0]) + abs(pnorm[1] * bdiff[1]) + abs(pnorm[2] * bdiff[2]);
		if (bcent*pnorm >= m)
			return false;
	}
	
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
bool Frustum3<T>::intersect(const Box<T,3>& box) const {
	unsigned int i;
	T t,l,m;
	T mi,ma;

	// Project Frustum onto box axes
	for (i = 0; i < 3; i++) {
		// Project Frustum onto ith axes of box
		t = abs(m_Axes[0]*box.getAxis(i)) + abs(m_Axes[1]*box.getAxis(i));

		l = m_FarByNear * t;
		T nearO = (m_NearOffsetOrigin - box.getOrigin()) * box.getAxis(i);
		T farO = (m_FarOffsetOrigin - box.getOrigin()) * box.getAxis(i);
		mi = min(nearO - t, farO - l);
		ma = max(nearO + t, farO + l);
		
		if ((ma <= -box.getAxis2(i)) || (mi >= box.getAxis2(i)))
			return false;
	}

	// Project box onto Direction Vector
	Vector3<T> bcent = box.getOrigin() - m_Origin;
	t = m_Direction * bcent;
	m = abs(m_Direction * box.getAxis(0)) + abs(m_Direction * box.getAxis(1)) + abs(m_Direction * box.getAxis(2));
	if ((t - m >= m_FarDirection2) || (t + m <= m_NearDirection2))
		return false;
		
	// Project Cube onto plane normals
	for (i = 0; i < 4; i++) {
		Vector3<T> pnorm = m_PlaneNormals[i];
		m = abs(pnorm * box.getAxis(0)) + abs(pnorm * box.getAxis(1)) + abs(pnorm * box.getAxis(2));
		if (bcent*pnorm >= m)
			return false;
	}

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool Frustum3<T>::intersect(const Bounds<T,3>& b) const {
	return intersect(*(const Box<T,3>*)&b);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
Containment Frustum3<T>::cull(const Cube<T,3>& cube) const {
	unsigned int i;
	Containment cont = Subset;
	Vector3<T> p = cube.getOrigin();
	T radius = cube.getRadius();
	T t,l,m;
	T mi, ma;

	// Project Frustum onto cube axes
	for (i = 0; i < 3; i++) {
		// Project Frustum onto ith axes of cube
		t = abs(m_Axes[0][i]) + abs(m_Axes[1][i]);

		l = m_FarByNear * t;
		mi = min(m_NearOffsetOrigin[i] - t, m_FarOffsetOrigin[i] - l);
		ma = max(m_NearOffsetOrigin[i] + t, m_FarOffsetOrigin[i] + l);
		
		if ((ma <= p[i] - radius) || (mi >= p[i] + radius))
			return Independent;
	}

	// Project Cube onto Direction Vector
	p -= m_Origin;
	t = m_Direction * p;
	m = m_Direction.sumabs() * radius;
	if ((t - m >= m_FarDirection2) || (t + m <= m_NearDirection2))
		return Independent;
	if ((t - m < m_NearDirection2) || (t+m > m_FarDirection2))
		cont = Section;
		
	// Project Cube onto plane normals
	for (i = 0; i < 4; i++) {
		// Project Cube onto plane normals
		t = p*m_PlaneNormals[i];
		m = m_PlaneNormals[i].sumabs()*radius;
		if (t >= m)
			return Independent;
		if (t >= -m)
			cont = Section;
	}
	
	return cont;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
Containment Frustum3<T>::cull(const Bounds<T,3>& b) const {
	if (intersect(b))
		return Section;
	else
		return Independent;
}


}; };

#endif
