/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_LINEAR_H
#define __GEOMETRY_LINEAR_H

#include "math.limits.h"
#include "math.vector.h"
#include "geometry.transform.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;
using magnum::math::Limits;
using magnum::math::Matrix;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Linear {
public:
	typedef T			Scalar;
	static const int	Dimension = dim;
	
public:
	inline Linear();
	inline Linear(const Vector<T,dim>& ,const Vector<T,dim>& );

	inline void			setOrigin(const Vector<T,dim>& );
	inline const Vector<T,dim>& getOrigin() const;

	inline void			setDirection(const Vector<T,dim>& );
	inline const Vector<T,dim>& getDirection() const;
	
	inline Vector<T,dim> operator()(const T& ) const;

	inline T			getDistance2(const Vector<T,dim>& ) const;
	inline T			getDistance(const Vector<T,dim>& ) const;
	
	Linear<T,dim>&		transform(const Matrix<T,dim+1>& );
	inline Linear<T,dim>& transform(const Transform<T,dim>& );

protected:
	Vector<T,dim>		m_Origin;
	Vector<T,dim>		m_Direction;
};


}; };

#endif
