/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_MESHCOLLIDER_H
#define __GEOMETRY_MESHCOLLIDER_H

#include "math.limits.h"
#include "math.vector.h"
#include "math.vector3.h"
#include "container.array.h"
#include "geometry.collider.h"
#include "geometry.alignedbox.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;
using magnum::math::Vector3;
using magnum::math::Limits;
using magnum::container::Array;

template<class T,int dim> class TriangleMesh;

/*--------------------------------------------------------------------------*/
/**
 */
template<class T> class MeshCollider3 : public Collider<T,3> {
//DeclareClass(MeshCollider3, Collider<T,3>);
public:
	enum Primitive {
		Unknown = 0,
		TriangleList = 0x0004,			// Compatible with OpenGL definitions
		TriangleStrip = 0x0005,
		TriangleFan = 0x0006,
		QuadStrip = 0x0008,
		QuadList = 0x0007,
		Polygon = 0x0009,
	};

public:
	inline MeshCollider3();
	inline MeshCollider3(const TriangleMesh<T,3>& t);
	
	inline void				setPrimitive(Primitive );
	inline Primitive		getPrimitive() const;
	
	void					setMesh(const TriangleMesh<T,3>& );

	inline void				setIndices(unsigned int );
	void					setIndices(const unsigned int* ,unsigned int );
	inline const Array<unsigned int>& getIndices() const;
	inline void				setIndex(unsigned int ,unsigned int );
	inline unsigned int		getIndex(unsigned int ) const;
	inline void				clearIndices();
	
	inline void				setVertices(unsigned int);
	void					setVertices(const Vector3<T>* ,unsigned int );
	void					setVertices(const Vector<T,3>* ,unsigned int );
	inline const Array<Vector3<T> >& getVertices() const;
	inline void				setVertex(unsigned int, const Vector3<T>& );
	inline void				setVertex(unsigned int, const Vector<T,3>& );
	inline const Vector3<T>& getVertex(unsigned int ) const;
	inline void				clearVertices();

	void					clear();
	
protected:
	virtual bool			onIntersect(const Line<T,3>& ,Intersection<T,3>* ) const;
	virtual bool			onIntersect(const Segment<T,3>& ,Intersection<T,3>* ) const;
	virtual bool			onIntersect(const Ray<T,3>& ,Intersection<T,3>* ) const;

private:
	template<class L>
	bool					doIntersect(const L& ,Intersection<T,3>* ) const;
	template<class L,int P>
	bool					doIntersectIndexed(const L& ,Intersection<T,3>* ) const;
	template<class L,int P>
	bool					doIntersectDirect(const L& ,Intersection<T,3>* ) const;
	
private:	
	Array<Vector3<T> >		m_Vertices;
	Array<unsigned int>		m_Indices;
	AlignedBox3<T>			m_Bounds;
	Primitive				m_Primitive;
};

}; };

#endif
