/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@3ajix.de)
*/

#ifndef __GEOMETRY_MESHCOLLIDER_INL
#define __GEOMETRY_MESHCOLLIDER_INL

#include "geometry.ray.h"
#include "geometry.line.h"
#include "geometry.segment.h"

namespace magnum { namespace geometry {

/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline MeshCollider3<T>::MeshCollider3() : m_Primitive(MeshCollider3<T>::TriangleList) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline MeshCollider3<T>::MeshCollider3(const TriangleMesh<T,3>& t) {
	setMesh(t);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline void MeshCollider3<T>::setPrimitive(typename MeshCollider3<T>::Primitive p) {
	m_Primitive = p;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline typename MeshCollider3<T>::Primitive MeshCollider3<T>::getPrimitive() const {
	return m_Primitive;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
void MeshCollider3<T>::setMesh(const TriangleMesh<T,3>& mesh) {
	setIndices(&mesh.getIndices()[0], mesh.getIndices().getSize());
	setVertices(&mesh.getVertices()[0], mesh.getVertices().getSize());
	setPrimitive(TriangleList);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline void MeshCollider3<T>::setIndices(unsigned int i) {
	m_Indices.setSize(i);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
void MeshCollider3<T>::setIndices(const unsigned int* idx,unsigned int i) {
	m_Indices.setSize(i);
	memcpy(&m_Indices[0], idx, i*sizeof(unsigned int));
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline void MeshCollider3<T>::clearIndices() {
	m_Indices.clear();
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline const Array<unsigned int>& MeshCollider3<T>::getIndices() const {
	return m_Indices;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline void MeshCollider3<T>::setIndex(unsigned int i,unsigned int idx) {
	m_Indices[i] = idx;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline unsigned int MeshCollider3<T>::getIndex(unsigned int i) const {
	return m_Indices[i];
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline void MeshCollider3<T>::setVertices(unsigned int count) {
	m_Vertices.setSize(count);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
void MeshCollider3<T>::setVertices(const Vector3<T>* v,unsigned int count) {
	m_Vertices.setSize(count);
	Vector3<T>* dst = &m_Vertices[0];
	Vector3<T>* end = dst+count;
	while (dst < end)
		*dst++ = *v++;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
void MeshCollider3<T>::setVertices(const Vector<T,3>* v,unsigned int count) {
	m_Vertices.setSize(count);
	Vector3<T>* dst = &m_Vertices[0];
	Vector3<T>* end = dst+count;
	while (dst < end)
		*dst++ = *v++;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline const Array<Vector3<T> >& MeshCollider3<T>::getVertices() const {
	return m_Vertices;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline void MeshCollider3<T>::setVertex(unsigned int i,const Vector<T,3>& v) {
	m_Vertices[i] = v;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline void MeshCollider3<T>::setVertex(unsigned int i,const Vector3<T>& v) {
	m_Vertices[i] = v;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline const Vector3<T>& MeshCollider3<T>::getVertex(unsigned int i) const {
	return m_Vertices[i];
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
inline void MeshCollider3<T>::clearVertices() {
	m_Vertices.clear();
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
void MeshCollider3<T>::clear() {
	m_Vertices.clear();
	m_Indices.clear();
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
bool MeshCollider3<T>::onIntersect(const Line<T,3>& l,Intersection<T,3>* is) const {
	return doIntersect(l,is);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
bool MeshCollider3<T>::onIntersect(const Segment<T,3>& s,Intersection<T,3>* is) const {
	return doIntersect(s,is);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
bool MeshCollider3<T>::onIntersect(const Ray<T,3>& r,Intersection<T,3>* is) const {
	return doIntersect(r,is);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
template<class L>
bool MeshCollider3<T>::doIntersect(const L& l,Intersection<T,3>* is) const {
	switch(m_Primitive) {
		case TriangleList:
			if (m_Indices.getSize() > 0)
				return doIntersectIndexed<L,TriangleList>(l,is);
			else
				return doIntersectDirect<L,TriangleList>(l,is);
		case TriangleStrip:
			if (m_Indices.getSize() > 0)
				return doIntersectIndexed<L,TriangleStrip>(l,is);
			else
				return doIntersectDirect<L,TriangleStrip>(l,is);
		case TriangleFan:
			if (m_Indices.getSize() > 0)
				return doIntersectIndexed<L,TriangleFan>(l,is);
			else
				return doIntersectDirect<L,TriangleFan>(l,is);
		case QuadStrip:
			if (m_Indices.getSize() > 0)
				return doIntersectIndexed<L,QuadStrip>(l,is);
			else
				return doIntersectDirect<L,QuadStrip>(l,is);
		case QuadList:
			if (m_Indices.getSize() > 0)
				return doIntersectIndexed<L,QuadList>(l,is);
			else
				return doIntersectDirect<L,QuadList>(l,is);
		default:
			return false;
	}
}


/*--------------------------------------------------------------------------*/
/* The following #define is the core triangle intersection routine
 * used by the following indexed / direct intersection loops. A
 * macro is used instead of a function in order to avoid an argument
 * push/pop orgy taking place on the stack.
 * This #define will be removed from the namespace ASAP.
 */
#define _geometry_mesh_doIntersectTriangle() \
	Vector3<T> e1 = v1 - v0;										\
	Vector3<T> e2 = v2 - v0;										\
	Vector3<T> pvec = dir % e2;										\
	T det = e1 * pvec;												\
	if (det > -Limits<T>::epsilon() && det < Limits<T>::epsilon())	\
		continue;													\
	T inv_det = T(1) / det;											\
	Vector3<T> tvec = orig - v0;									\
	T u = (tvec * pvec) * inv_det;									\
	if (u < T(0) || u > T(1))										\
		continue;													\
	Vector3<T> qvec = tvec % e1;									\
	T v = (dir * qvec) * inv_det;									\
	if (v < T(0) || (u+v) > T(1))									\
		continue;													\
	T t = (e2 * qvec) * inv_det;									\
	if (t < mint)													\
		continue;													\
	if (t > maxt)													\
		continue;													\
	maxt = t;														\
	if (is == NULL)													\
		return true;												\
	is->setNormal(e1 % e2);											\
	intersect = true;												\
	

/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
template<class L,int primitive>
bool MeshCollider3<T>::doIntersectIndexed(const L& l,Intersection<T,3>* is) const {
	if (m_Indices.getSize() < 3)
		return false;
		
	const unsigned int* idx = &m_Indices[0];
	const unsigned int* end = &m_Indices[m_Indices.getSize()-2];
	
	T mint = L::MinParam;
	T maxt = L::MaxParam;
	bool intersect = false;
	Vector3<T> dir = l.getDirection();
	Vector3<T> orig = l.getOrigin();
	
	while(idx < end) {
		Vector3<T> v0, v1, v2;
		if (primitive == TriangleList) {
			v0 = m_Vertices[*idx++];
			v1 = m_Vertices[*idx++];
			v2 = m_Vertices[*idx++];
		} else if (primitive == TriangleStrip) {
			v0 = m_Vertices[idx[0]];
			v1 = m_Vertices[idx[1]];
			v2 = m_Vertices[idx[2]];
			idx++;
		}
		
		_geometry_mesh_doIntersectTriangle();
	}
	
	if (!intersect)
		return false;
	is->setPosition(orig + maxt*dir);
	is->setParameter(maxt);
	return true;	
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
template<class L,int primitive>
bool MeshCollider3<T>::doIntersectDirect(const L& l,Intersection<T,3>* is) const {
	if (m_Vertices.getSize() < 3)
		return false;

	const Vector3<T>* vtx = &m_Vertices[0];
	const Vector3<T>* end = &m_Vertices[m_Vertices.getSize()-2];
	
	T mint = L::MinParam;
	T maxt = L::MaxParam;
	bool intersect = false;
	Vector3<T> dir = l.getDirection();
	Vector3<T> orig = l.getOrigin();
	
	while(vtx < end) {
		Vector3<T> v0, v1, v2;
		if (primitive == TriangleList) {
			v0 = *vtx++;
			v1 = *vtx++;
			v2 = *vtx++;
		} else if (primitive == TriangleStrip) {
			v0 = vtx[0];
			v1 = vtx[1];
			v2 = vtx[2];
			vtx++;
		}
		
		_geometry_mesh_doIntersectTriangle();
	}
	
	if (!intersect)
		return false;
	is->setPosition(orig + maxt*dir);
	is->setParameter(maxt);
	return true;	
}

// Remove that ugly define
#undef _geometry_mesh_doIntersectTriangle

}; };

#endif
