/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_NORMALGENERATOR_INL
#define __GEOMETRY_NORMALGENERATOR_INL

namespace magnum { namespace geometry {
using magnum::container::Array;
using magnum::math::min;
using magnum::math::max;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
NormalGenerator<T,I>::NormalGenerator() {
	m_Normals = NULL;
	m_NormalStride = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline unsigned int NormalGenerator<T,I>::getNormalStride() const {
	return m_NormalStride;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline Vector3<T>* NormalGenerator<T,I>::getNormals() const {
	return m_Normals;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline void NormalGenerator<T,I>::setNormals(Vector3<T>* n,unsigned int s) {
	m_Normals = n;
	m_NormalStride = s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
inline Vector3<T>& NormalGenerator<T,I>::getNormal(unsigned int i) {
	return *reinterpret_cast<Vector3<T>*>(reinterpret_cast<byte*>(m_Normals)+i*m_NormalStride);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class I>
void NormalGenerator<T,I>::generate() {
	if (getIndexSize() == 0)
		return;
	if (getIndices() == NULL)
		return;
	if (getVertices() == NULL)
		return;
	if (m_Normals == NULL)
		return;
		
	unsigned int minIndex = ~0;
	unsigned int maxIndex = 0;
	
	unsigned int count = getPrimitiveCount();
	
	// Collect triangle normals
	Array<Vector3<float> > normals;
	normals.setSize(count);
	for (unsigned int i = 0; i < count; i++) {
		unsigned int vi1,vi2,vi3;
		getTriangle(i, &vi1, &vi2, &vi3);
		
		minIndex = min(vi1, min(vi2, min(vi3, minIndex)));
		maxIndex = max(vi1, max(vi2, max(vi3, maxIndex)));

		Vector3<T> v1 = getVertex(vi1);
		Vector3<T> v2 = getVertex(vi2);
		Vector3<T> v3 = getVertex(vi3);
		normals[i] = (v2-v1) % (v3-v1);
	}
	
	// Clear all vertex normals
	for (unsigned int i = minIndex; i <= maxIndex; i++)
		getNormal(i) = Vector3<T>::Null;
		
	// Add triangle normals to vertex normals
	for (unsigned int i = 0; i < count; i++) {
		unsigned int vi1,vi2,vi3;
		getTriangle(i, &vi1, &vi2, &vi3);
		
		getNormal(vi1) += normals[i];
		getNormal(vi2) += normals[i];
		getNormal(vi3) += normals[i];
	}

	// Normalise vertex normals
	for (unsigned int i = minIndex; i <= maxIndex; i++)
		getNormal(i).normalise();
}

}; };

#endif
