/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_PLANE_INL
#define __GEOMETRY_PLANE_INL

namespace magnum { namespace geometry {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Plane<T,dim>::Plane() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Plane<T,dim>::Plane(const Vector<T,dim>& o,const Vector<T,dim>& n) {
	m_Origin = o;
	m_Normal = n;
	m_OriginNormal = o*n;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Plane<T,dim>::set(const Vector<T,dim>& o,const Vector<T,dim>& n) {
	m_Origin = o;
	m_Normal = n;
	m_OriginNormal = o*n;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Plane<T,dim>::setOrigin(const Vector<T,dim>& o) {
	m_Origin = o;
	m_OriginNormal = o*m_Normal;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Plane<T,dim>::getOrigin() const {
	return m_Origin;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Plane<T,dim>::setNormal(const Vector<T,dim>& n) {
	m_Normal = n;
	m_OriginNormal = m_Origin * n;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Plane<T,dim>::getNormal() const {
	return m_Normal;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Plane<T,dim>::getOffset() const {
	return m_OriginNormal;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Plane<T,dim>::isInside(const Vector<T,dim>& v) const {
	return (m_Normal * v <= m_OriginNormal);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Plane<T,dim>::getDistance2(const Vector<T,dim>& v) const {
	return (m_Normal*v - m_OriginNormal) / m_Normal.length();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Plane<T,dim>::getDistance(const Vector<T,dim>& v) const {
	return sqrt((m_Normal*v - m_OriginNormal) / m_Normal.length());
}

}; };

#endif
