/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_RAY_INL
#define __GEOMETRY_RAY_INL

namespace magnum { namespace geometry {

template<class T, int dim>
const T	Ray<T,dim>::MinParam = T(0);
template<class T, int dim>
const T	Ray<T,dim>::MaxParam = Limits<T>::max();


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline Ray<T,dim>::Ray() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline Ray<T,dim>::Ray(const Vector<T,dim>& o,const Vector<T,dim>& d) :Linear<T,dim>(o,d) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline T Ray<T,dim>::getDistance2(const Vector<T,dim>& v) const {
    Vector<T,dim> diff = v - m_Origin;
    T t = diff * m_Direction;

    if ( t <= T(0.0)) {
        t = T(0.0); 
    } else {
        t /= m_Direction.length2();
        diff -= t * m_Direction;
    }

    return diff.length2();	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline T Ray<T,dim>::getDistance(const Vector<T,dim>& v) const {
    Vector<T,dim> diff = v - m_Origin;
    T t = diff * m_Direction;

    if ( t <= T(0.0)) {
        t = T(0.0); 
    } else {
        t /= m_Direction.length2();
        diff -= t * m_Direction;
    }

    return diff.length();	
}

}; };

#endif
