/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_SPHERE_H
#define __GEOMETRY_SPHERE_H

#include "math.vector2.h"
#include "math.vector3.h"
#include "math.vector4.h"
#include "container.tuple.h"
#include "geometry.body.h"
#include "geometry.intersection.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;
using magnum::math::Vector2;
using magnum::math::Vector3;
using magnum::math::Vector4;

template<class T, int dim> class AlignedBox;
template<class T, int dim> class Box;
template<class T, int dim> class Cube;
template<class T, int dim> class Line;
template<class T, int dim> class Ray;
template<class T, int dim> class Segment;

template<class T> class Sphere2;
template<class T> class Sphere3;
template<class T> class Sphere4;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Sphere : public Body<T,dim> {
friend class Sphere2<T>;
friend class Sphere3<T>;
friend class Sphere4<T>;
private:
	Vector<T,dim>	m_Origin;
	T				m_Radius;

public:
	inline Sphere();
	inline Sphere(const Vector<T,dim>& ,const T& );

	inline void		setOrigin(const Vector<T,dim>& );
	inline const Vector<T,dim>& getOrigin() const;

	inline void		setRadius(const T& );
	inline const T&	getRadius() const;

	inline bool		isEmpty() const;
	inline void		empty();

	inline bool		isInside(const Vector<T,dim>& ) const; 
	inline T		getDistance(const Vector<T,dim>& ) const;

	inline bool		intersect(const Sphere<T,dim>& ) const;
	inline bool		intersect(const AlignedBox<T,dim>& ) const;
	inline bool		intersect(const Box<T,dim>& ) const;
	inline bool		intersect(const Cube<T,dim>& ) const;
	
	bool			intersect(const Line<T,dim>& ,Intersection<T,dim>* = NULL) const;
	bool			intersect(const Segment<T,dim>& ,Intersection<T,dim>* = NULL) const;
	bool			intersect(const Ray<T,dim>& ,Intersection<T,dim>* = NULL) const;

	Cube<T,dim>		toCube() const;
	Box<T,dim>		toBox() const;
	AlignedBox<T,dim> toAlignedBox() const;

	inline bool		bound(const Vector<T,dim>* ,unsigned int n);
	bool			bound(const Vector<T,dim>* ,unsigned int stride,unsigned int n);

	void			merge(const Sphere<T,dim>& );

	bool			construct(const Vector<T,dim>& );
	bool			construct(const Vector<T,dim>& ,const Vector<T,dim>& );
	bool			construct(const Vector<T,dim>& ,const Vector<T,dim>& ,const Vector<T,dim>& );
	bool			construct(const Vector<T,dim>& ,const Vector<T,dim>& ,const Vector<T,dim>& ,const Vector<T,dim>& );
	bool			construct(const Vector<T,dim>* ,unsigned int n);
};



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Sphere2 : public Sphere<T,2> {
public:	
	inline Sphere2();
	inline Sphere2(const Vector2<T>& ,const T& );
	inline Sphere2(const Vector<T,2>& ,const T& );
	inline Sphere2(const Sphere<T,2>& );

	inline bool		bound(const Vector2<T>* ,unsigned int n);
	inline bool		bound(const Vector2<T>* ,unsigned int stride,unsigned int n);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Sphere3 : public Sphere<T,3> { 
public:	
	inline Sphere3();
	inline Sphere3(const Vector3<T>& ,const T& );
	inline Sphere3(const Vector<T,3>& ,const T& );
	inline Sphere3(const Sphere<T,3>& );

	inline bool		bound(const Vector3<T>* ,unsigned int n);
	inline bool		bound(const Vector3<T>* ,unsigned int stride,unsigned int n);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Sphere4 : public Sphere<T,4> { 
public:	
	inline Sphere4();
	inline Sphere4(const Vector4<T>& ,const T& );
	inline Sphere4(const Vector<T,4>& ,const T& );
	inline Sphere4(const Sphere<T,4>& );

	inline bool		bound(const Vector4<T>* ,unsigned int n);
	inline bool		bound(const Vector4<T>* ,unsigned int stride,unsigned int n);
};


}; };

#endif
