/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_TRANSLATIONCONSTRAINTS_INL
#define __GEOMETRY_TRANSLATIONCONSTRAINTS_INL

#include "geometry.transformconstraints.h"

namespace magnum { namespace geometry {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int dim>
inline TranslationConstraint<T,dim>::TranslationConstraint() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int dim>
inline TranslationConstraint<T,dim>::TranslationConstraint(const Vector<T,dim>& mi,const Vector<T,dim>& ma) {
	m_Min = mi;
	m_Max = ma;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int dim>
inline const Vector<T,dim>&	TranslationConstraint<T,dim>::getMinimum() const {
	return m_Min;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int dim>
inline const Vector<T,dim>&	TranslationConstraint<T,dim>::getMaximum() const {
	return m_Max;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int dim>
inline void TranslationConstraint<T,dim>::setMinimum(const Vector<T,dim>& m) {
	m_Min = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int dim>
inline void TranslationConstraint<T,dim>::setMaximum(const Vector<T,dim>& m) {
	m_Max = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int dim>
inline void TranslationConstraint<T,dim>::setRange(const Vector<T,dim>& mi,const Vector<T,dim>& ma) {
	m_Min = mi;
	m_Max = ma;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int dim>
bool TranslationConstraint<T,dim>::onConstrain(Transform<T,dim>& t) {
	Vector<T,dim> v = t.getTranslation();
	bool dirty = false;
	// First process generic case
	if (dim > 4) {
		for (unsigned int i = 0; i < dim; i++) {
			if (v[i] < m_Min[i]) {
				v[i] = m_Min[i];
				dirty = true;
			} else if (v[i] > m_Max[i]) {
				v[i] = m_Max[i];
				dirty = true;
			}
		}
		if (dirty)
			t.setTranslation(v);
		return dirty;
	}
	
	// Process 2-dim case
	if (v[0] < m_Min[0]) {
		v[0] = m_Min[0];
		dirty = true;
	} else if (v[0] > m_Max[0]) {
		v[0] = m_Max[0];
		dirty = true;
	}
	if (v[1] < m_Min[1]) {
		v[1] = m_Min[1];
		dirty = true;
	} else if (v[1] > m_Max[1]) {
		v[1] = m_Max[1];
		dirty = true;
	}
	// Process 3-dim case
	if (dim >= 3) {
		if (v[2] < m_Min[2]) {
			v[2] = m_Min[2];
			dirty = true;
		} else if (v[2] > m_Max[2]) {
			v[2] = m_Max[2];
			dirty = true;
		}
	}
	// Process 4-dim case
	if (dim == 4) {
		if (v[3] < m_Min[3]) {
			v[3] = m_Min[3];
			dirty = true;
		} else if (v[3] > m_Max[3]) {
			v[3] = m_Max[3];
			dirty = true;
		}
	}

	if (dirty)
		t.setTranslation(v);
	return dirty;
}

}; };

#endif
