/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_TRIANGLE_H
#define __GEOMETRY_TRIANGLE_H

#include "math.limits.h"
#include "math.vector.h"
#include "math.vector3.h"

namespace magnum { namespace geometry {
using magnum::math::Vector;
using magnum::math::Vector3;
using magnum::math::Limits;

template<class T,int dim> class Line;
template<class T,int dim> class Segment;
template<class T,int dim> class Ray;
template<class T,int dim> class Triangle;

/*--------------------------------------------------------------------------*/
/** Proxy class for intersection routines with a dimension specific
 * partial template specialisation.
 */
template<class T, int dim> class TriangleIntersector {
public:
	static bool				intersect(const Triangle<T,dim>& ,const Line<T,dim>& );
	static bool				intersect(const Triangle<T,dim>& ,const Segment<T,dim>& );
	static bool				intersect(const Triangle<T,dim>& ,const Ray<T,dim>& );
};


/*--------------------------------------------------------------------------*/
/** Proxy class for intersection routines for dimension 3
 */
template<class T> class TriangleIntersector<T,3> {
public:
	static bool				intersect(const Triangle<T,3>& ,const Line<T,3>& );
	static bool				intersect(const Triangle<T,3>& ,const Segment<T,3>& );
	static bool				intersect(const Triangle<T,3>& ,const Ray<T,3>& );
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Triangle {
public:
	Vector<T,dim>			a,b,c;

public:
	inline Triangle();
	inline Triangle(const Vector<T,dim>& , const Vector<T,dim>& , const Vector<T,dim>& );

	inline Vector<T,dim>&	operator [](unsigned int );
	inline Vector<T,dim>	operator [](unsigned int ) const;

	inline bool				intersect(const Line<T,dim>& ) const;
	inline bool				intersect(const Segment<T,dim>& ) const;
	inline bool				intersect(const Ray<T,dim>& ) const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Triangle2 : public Triangle<T,2> {
public:
	inline Triangle2() {};
	inline Triangle2(const Triangle<T,2>& t) : Triangle(t) {};
	inline Triangle2(const Vector<T,2>& a, const Vector<T,2>& b, const Vector<T,2>& c) : Triangle(a,b,c) {};
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Triangle3 : public Triangle<T,3> {
public:
	inline Triangle3() {};
	inline Triangle3(const Triangle<T,3>& t) : Triangle(t) {};
	inline Triangle3(const Vector<T,3>& a, const Vector<T,3>& b, const Vector<T,3>& c) : Triangle(a,b,c) {};
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Triangle4 : public Triangle<T,4> {
public:
	inline Triangle4() {};
	inline Triangle4(const Triangle<T,4>& t) : Triangle(t) {};
	inline Triangle4(const Vector<T,4>& a, const Vector<T,4>& b, const Vector<T,4>& c) : Triangle(a,b,c) {};
};

}; };

#endif
