/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GEOMETRY_TRIANGLE_INL
#define __GEOMETRY_TRIANGLE_INL


namespace magnum { namespace geometry {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Triangle<T,dim>::Triangle() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Triangle<T,dim>::Triangle(const Vector<T,dim>& v1, const Vector<T,dim>& v2, const Vector<T,dim>& v3) {
	a = v1;
	b = v2;
	c = v3;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>& Triangle<T,dim>::operator [](unsigned int i) {
	return *(&a + i);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Triangle<T,dim>::operator [](unsigned int i) const {
	return *(&a + i);
}



/*--------------------------------------------------------------------------*/
/**
 */
template<class T, int dim> 
inline bool Triangle<T,dim>::intersect(const Line<T,dim>& l) const {
	return TriangleIntersector<T,dim>::intersect(*this,l);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T, int dim> 
inline bool Triangle<T,dim>::intersect(const Segment<T,dim>& s) const {
	return TriangleIntersector<T,dim>::intersect(*this,s);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T, int dim> 
inline bool Triangle<T,dim>::intersect(const Ray<T,dim>& r) const {
	return TriangleIntersector<T,dim>::intersect(*this,r);
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T, int dim> 
bool TriangleIntersector<T,dim>::intersect(const Triangle<T,dim>& ,const Line<T,dim>& ) {
	ASSERT(false);
	return false;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T, int dim> 
bool TriangleIntersector<T,dim>::intersect(const Triangle<T,dim>& ,const Ray<T,dim>& ) {
	ASSERT(false);
	return false;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T, int dim> 
bool TriangleIntersector<T,dim>::intersect(const Triangle<T,dim>& ,const Segment<T,dim>& ) {
	ASSERT(false);
	return false;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
bool TriangleIntersector<T,3>::intersect(const Triangle<T,3>& tri,const Line<T,3>& line) {
	Vector3<T> dir = line.getDirection();
	Vector3<T> orig = line.getOrigin();
	Vector3<T> e1 = tri.b - tri.a;
	Vector3<T> e2 = tri.c - tri.a;
	Vector3<T> pvec = dir % e2;
	T det = e1 * pvec;
	if (det > -Limits<T>::epsilon() && det < Limits<T>::epsilon())
		return false;	
	T inv_det = T(1) / det;
	Vector3<T> tvec = orig - tri.a;
	T u = (tvec * pvec) * inv_det;
	if (u < T(0) || u > T(1))
		return false;
	Vector3<T> qvec = tvec % e1;
	T v = (dir * qvec) * inv_det;
	if (v < T(0) || (u+v) > T(1))
		return false;
	return true;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
bool TriangleIntersector<T,3>::intersect(const Triangle<T,3>& tri,const Segment<T,3>& seg) {
	Vector3<T> dir = seg.getDirection();
	Vector3<T> orig = seg.getOrigin();
	Vector3<T> e1 = tri.b - tri.a;
	Vector3<T> e2 = tri.c - tri.a;
	Vector3<T> pvec = dir % e2;
	T det = e1 * pvec;
	if (det > -Limits<T>::epsilon() && det < Limits<T>::epsilon())
		return false;	
	T inv_det = T(1) / det;
	Vector3<T> tvec = orig - tri.a;
	T u = (tvec * pvec) * inv_det;
	if (u < T(0) || u > T(1))
		return false;
	Vector3<T> qvec = tvec % e1;
	T v = (dir * qvec) * inv_det;
	if (v < T(0) || (u+v) > T(1))
		return false;
	T t = (e2 * qvec) * inv_det;
	return ((t >= T(0)) && (t <= T(1)));
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> 
bool TriangleIntersector<T,3>::intersect(const Triangle<T,3>& tri,const Ray<T,3>& ray) {
	Vector3<T> dir = ray.getDirection();
	Vector3<T> orig = ray.getOrigin();
	Vector3<T> e1 = tri.b - tri.a;
	Vector3<T> e2 = tri.c - tri.a;
	Vector3<T> pvec = dir % e2;
	T det = e1 * pvec;
	if (det > -Limits<T>::epsilon() && det < Limits<T>::epsilon())
		return false;	
	T inv_det = T(1) / det;
	Vector3<T> tvec = orig - tri.a;
	T u = (tvec * pvec) * inv_det;
	if (u < T(0) || u > T(1))
		return false;
	Vector3<T> qvec = tvec % e1;
	T v = (dir * qvec) * inv_det;
	if (v < T(0) || (u+v) > T(1))
		return false;
	T t = (e2 * qvec) * inv_det;
	return (t >= T(0));
}

}; };

#endif
