/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPH_GRAPH_H
#define __GRAPH_GRAPH_H

#include "container.allocator.h"
#include "container.array.h"
#include "graph.edge.h"
#include "graph.node.h"

namespace magnum { namespace graph {
using magnum::container::Array;
using magnum::container::Allocator;
using magnum::object::Object;

	
/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E> class Graph : public Object {
DeclareClass(Graph, Object);
friend Node<N,E>;
public:
	typedef	magnum::graph::Edge<N,E> Edge;
	typedef	magnum::graph::Node<N,E> Node;
	typedef typename magnum::container::Array<Node>::Iterator Iterator;

public:
	inline Graph();
	inline explicit Graph(unsigned int );

	void				setSize(unsigned int );
	inline unsigned int	getSize() const;
	
	inline Node&		operator[](unsigned int );
	inline const Node&	operator[](unsigned int ) const;
	
	inline Iterator		begin() const;
	inline Iterator		end() const;
	inline Iterator		at(unsigned int ) const;

	inline Iterator		rbegin() const;
	inline Iterator		rend() const;
	
	void				clear();
	inline void			insert(unsigned int start, unsigned int end);
	inline void			insert(unsigned int start, unsigned int end, const E& elem);
	inline void			insert(const Iterator& start, const Iterator& end);
	inline void			insert(const Iterator& start, const Iterator& end, const E& elem);
	inline void			remove(unsigned int start, unsigned int end);
	inline void			remove(const Iterator& start, const Iterator& end);
	inline bool			contains(unsigned int start, unsigned int end);
	inline bool			contains(const Iterator& start, const Iterator& end);
	
	bool				isEmpty() const;

private:
	Array<Node>			m_Nodes;
	Allocator<Edge>		m_Allocator;
};

}; };

#endif
