/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPH_GRAPH_INL
#define __GRAPH_GRAPH_INL

namespace magnum { namespace graph {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline Graph<N,E>::Graph() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline Graph<N,E>::Graph(unsigned int i) {
	setSize(i);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
void Graph<N,E>::setSize(unsigned int s) {
	m_Nodes.setSize(s);
	for (unsigned int i = 0; i < s; i++) {
		m_Nodes[i].m_Index = i;
		m_Nodes[i].m_Graph = this;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline unsigned int	Graph<N,E>::getSize() const {
	return m_Nodes.getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline typename Graph<N,E>::Node& Graph<N, E>::operator[](unsigned int i) {
	return m_Nodes[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline const typename Graph<N,E>::Node& Graph<N,E>::operator[](unsigned int i) const {
	return m_Nodes[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline typename Graph<N,E>::Iterator Graph<N,E>::begin() const {
	return m_Nodes.begin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline typename Graph<N,E>::Iterator Graph<N,E>::end() const {
	return m_Nodes.end();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline typename Graph<N,E>::Iterator Graph<N,E>::at(unsigned int i) const {
	return m_Nodes.at(i);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline typename Graph<N,E>::Iterator Graph<N,E>::rbegin() const {
	return m_Nodes.rbegin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline typename Graph<N,E>::Iterator Graph<N,E>::rend() const {
	return m_Nodes.rend();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
void Graph<N,E>::clear() {
	for (unsigned int i = 0; i < m_Nodes.getSize(); i++) {
		m_Nodes[i].clear();
		m_Nodes[i].unmark();
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline void	Graph<N,E>::insert(unsigned int start, unsigned int end) {
	ASSERT(start < m_Nodes.getSize());
	ASSERT(end < m_Nodes.getSize());
	m_Nodes[start].insert(end);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline void	Graph<N,E>::insert(unsigned int start, unsigned int end, const E& elem) {
	ASSERT(start < m_Nodes.getSize());
	ASSERT(end < m_Nodes.getSize());
	m_Nodes[start].insert(end, elem);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline void	Graph<N,E>::insert(const typename Graph<N,E>::Iterator& start, const typename Graph<N,E>::Iterator& end) {
	start->insert(&(*end) - m_Nodes.getArray());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline void	Graph<N,E>::insert(const typename Graph<N,E>::Iterator& start, const typename Graph<N,E>::Iterator& end, const E& elem) {
	start->insert(&(*end) - m_Nodes.getArray(), elem);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline void	Graph<N,E>::remove(unsigned int start, unsigned int end) {
	ASSERT(start < m_Nodes.getSize());
	ASSERT(end < m_Nodes.getSize());
	m_Nodes[start].remove(end);	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline void	Graph<N,E>::remove(const typename Graph<N,E>::Iterator& start, const typename Graph<N,E>::Iterator& end) {
	start->remove(&(*end) - m_Nodes.getArray());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline bool	Graph<N,E>::contains(unsigned int start, unsigned int end) {
	ASSERT(start < m_Nodes.getSize());
	ASSERT(end < m_Nodes.getSize());
	return m_Nodes[start].contains(end);	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline bool	Graph<N,E>::contains(const typename Graph<N,E>::Iterator& start, const typename Graph<N,E>::Iterator& end) {
	return start->contains(&(*end) - m_Nodes.getArray());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
bool Graph<N,E>::isEmpty() const {
	for (unsigned int i = 0; i < m_Nodes.getSize(); i++) {
		if (!m_Nodes[i].m_Edges.isEmpty())
			return false;
	}

	return true;
}


}; };

#endif
