/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPH_NODE_H
#define __GRAPH_NODE_H

#include "container.iterator.h"
#include "graph.edge.h"

namespace magnum { namespace graph {
using magnum::container::IteratorBase;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>  class NodeIterator : IteratorBase<NodeIterator<N,E> > {
friend Node<N,E>;
public:
	inline NodeIterator();
	
	inline operator Edge<N,E>*();
	inline operator const Edge<N,E>*() const; 
	
	inline Edge<N,E>&		operator*() const;
	inline Edge<N,E>*		operator->() const;
	
	inline NodeIterator&	operator++();
	inline NodeIterator		operator++(int );
	inline NodeIterator&	operator--();
	inline NodeIterator		operator--(int );

	inline bool 			operator==(const NodeIterator& iter) const;
	inline bool 			operator!=(const NodeIterator& iter) const;
	inline bool 			operator<(const NodeIterator& iter) const;
	inline bool 			operator>(const NodeIterator& iter) const;
	inline bool 			operator<=(const NodeIterator& iter) const;
	inline bool 			operator>=(const NodeIterator& iter) const;

private:
	inline NodeIterator(Edge<N,E>* );

private:
	Edge<N,E>*				m_Edge;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E> class Node {
friend Graph<N,E>;
public:
	typedef NodeIterator<N,E>	Iterator;
	
public:
	inline Node();
	inline N*				operator->();
	inline const N*			operator->() const;
	inline N&				operator*();
	inline const N&			operator*() const;

	inline Iterator			begin() const;
	inline Iterator			end() const;

	inline Iterator			rbegin() const;
	inline Iterator			rend() const;
							
	inline void				mark();
	inline void				unmark();
	inline bool				isMarked() const;
							
	void					clear();
	void					insert(unsigned int target);
	void					insert(unsigned int target, const E& e);
	void					remove(unsigned int target);
	void					remove(const Iterator& iter);
	bool					contains(unsigned int target) const;
	inline bool				isEmpty() const;
	inline unsigned int		getSize() const;

private:
	Graph<N,E>*				m_Graph;
	N						m_Node;
	bool					m_Marker;
	unsigned int			m_Index;
	unsigned int			m_Size;
	Edge<N,E>				m_Root;
};

}; };

#endif
