/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPH_NODE_INL
#define __GRAPH_NODE_INL

namespace magnum { namespace graph {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline Node<N,E>::Node() {
	m_Marker = false;
	m_Size = 0;
	m_Root.m_Next = m_Root.m_Prev = &m_Root;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline N* Node<N,E>::operator->() {
	return &m_Node;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline const N* Node<N,E>::operator->() const {
	return &m_Node;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline N& Node<N,E>::operator*() {
	return m_Node;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline const N& Node<N,E>::operator*() const {
	return m_Node;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline NodeIterator<N,E> Node<N,E>::begin() const {
	return Iterator(m_Root.m_Next);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline NodeIterator<N,E> Node<N,E>::end() const {
	return Iterator((Edge<N,E>*)&m_Root);
};
						

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline NodeIterator<N,E> Node<N,E>::rbegin() const {
	return Iterator(m_Root.m_Prev);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline NodeIterator<N,E> Node<N,E>::rend() const {
	return Iterator((Edge<N,E>*)&m_Root);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline void Node<N,E>::mark() {
	m_Marker = true;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline void Node<N,E>::unmark() {
	m_Marker = false;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline bool Node<N,E>::isMarked() const {
	return m_Marker;
};
						

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
void Node<N,E>::clear() {
	while(m_Root.m_Next != &m_Root) {
		Edge<N,E>* e = m_Root.m_Next;
		m_Root.m_Next = e->m_Next;
		m_Graph->m_Allocator.free(e);
	}
	m_Root.m_Prev = &m_Root;
	m_Size = 0;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
void Node<N,E>::insert(unsigned int target) {
	Edge<N,E>* e = new(m_Graph->m_Allocator) Edge<N,E>(m_Index, target);
	e->m_Next = &m_Root;
	e->m_Prev = m_Root.m_Prev;
	m_Root.m_Prev->m_Next = e;
	m_Root.m_Prev = e;
	m_Size++;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
void Node<N,E>::insert(unsigned int target, const E& i) {
	Edge<N,E>* e = new(m_Graph->m_Allocator) Edge<N,E>(m_Index, target, i);
	e->m_Next = &m_Root;
	e->m_Prev = m_Root.m_Prev;
	m_Root.m_Prev->m_Next = e;
	m_Root.m_Prev = e;
	m_Size++;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
void Node<N,E>::remove(unsigned int target) {
	Edge<N,E>* edge = m_Root.m_Next;
	while(edge != &m_Root) {
		if (edge->m_End == target) {
			edge->m_Prev->m_Next = edge->m_Next;
			edge->m_Next->m_Prev = edge->m_Prev;
			m_Graph->m_Allocator.free(edge);
			m_Size--;
			return;
		}
		edge = edge->m_Next;
	}
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
void Node<N,E>::remove(typename const Node<N,E>::Iterator& iter) {
	Edge<N,E>* edge = iter.m_Edge;
	edge->m_Prev->m_Next = edge->m_Next;
	edge->m_Next->m_Prev = edge->m_Prev;
	m_Graph->m_Allocator.free(edge);
	m_Size--;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
bool Node<N,E>::contains(unsigned int target) const {
	Edge<N,E>* edge = m_Root.m_Next;
	while(edge != &m_Root) {
		if (edge->m_End == target) {
			return true;
		}
		edge = edge->m_Next;
	}
	return false;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline bool Node<N,E>::isEmpty() const {
	return (m_Size == 0);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N, class E>
inline unsigned int Node<N,E>::getSize() const {
	return m_Size;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline NodeIterator<N,E>::NodeIterator() {
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline NodeIterator<N,E>::NodeIterator(Edge<N,E>* edge) {
	m_Edge = edge;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline NodeIterator<N,E>::operator Edge<N,E>*() {
	return m_Edge;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline NodeIterator<N,E>::operator const Edge<N,E>*() const {
	return m_Edge;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline Edge<N,E>& NodeIterator<N,E>::operator*() const {
	return *m_Edge;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline Edge<N,E>* NodeIterator<N,E>::operator->() const {
	return m_Edge;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline NodeIterator<N,E>& NodeIterator<N,E>::operator++() {
	ASSERT(m_Edge != NULL);
	m_Edge = m_Edge->m_Next;
	return *this;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline NodeIterator<N,E> NodeIterator<N,E>::operator++(int ) {
	ASSERT(m_Edge != NULL);
	Edge<N,E>* edge = m_Edge;
	m_Edge = m_Edge->m_Next;
	return NodeIterator(edge);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline NodeIterator<N,E>& NodeIterator<N,E>::operator--() {
	ASSERT(m_Edge != NULL);
	m_Edge = m_Edge->m_Prev;
	return *this;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline NodeIterator<N,E> NodeIterator<N,E>::operator--(int ) {
	ASSERT(m_Edge != NULL);
	Edge<N,E>* edge = m_Edge;
	m_Edge = m_Edge->m_Prev;
	return NodeIterator(edge);
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline bool NodeIterator<N,E>::operator==(const NodeIterator<N,E>& iter) const {
	return m_Edge == iter.m_Edge;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline bool NodeIterator<N,E>::operator!=(const NodeIterator<N,E>& iter) const {
	return m_Edge != iter.m_Edge;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline bool NodeIterator<N,E>::operator<(const NodeIterator<N,E>& iter) const {
	return m_Edge < iter.m_Edge;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline bool NodeIterator<N,E>::operator>(const NodeIterator<N,E>& iter) const {
	return m_Edge > iter.m_Edge;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline bool NodeIterator<N,E>::operator<=(const NodeIterator<N,E>& iter) const {
	return m_Edge <= iter.m_Edge;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class N,class E>
inline bool NodeIterator<N,E>::operator>=(const NodeIterator<N,E>& iter) const {
	return m_Edge >= iter.m_Edge;
};

}; };

#endif
