/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS_COLOUR_INL
#define __GRAPHICS_COLOUR_INL

namespace magnum { namespace graphics {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Colour<T>::Colour() {
	R = G = B = A = T(0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Colour<T>::Colour(T r,T g,T b, T alpha) {
	R = r;
	G = g;
	B = b;
	A = alpha;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Colour<T> Colour<T>::operator+ (const Colour &c) const {
	return Colour<T>(R + c.R, G + c.G, B + c.B, A + c.A);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Colour<T> Colour<T>::operator- (const Colour &c) const {
	return Colour<T>(R + c.R, G + c.G, B + c.B, A + c.A);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Colour<T>& Colour<T>::operator+=(const Colour &c) {
	R += c.R;
	G += c.G;
	B += c.B;
	A += c.A;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Colour<T>& Colour<T>::operator-=(const Colour &c) {
	R -= c.R;
	G -= c.G;
	B -= c.B;
	A -= c.A;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Colour<T>::operator==(const Colour &c) const {
	return ((R == c.R) && (G == c.G) && (B == c.B) && (A == c.A));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Colour<T>::operator!=(const Colour &c) const  {
	return ((R != c.R) || (G != c.G) || (B != c.B) || (A != c.A));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Colour<T>::operator const T*() const {
	return &R;
}


#if defined(__FORX11__)
/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int colourToX11(const Colour<byte>& col) {
	return ((unsigned int)col.R << 16) + ((unsigned int)col.G << 8) + (unsigned int)col.B;
}
#endif


}; };
#endif
