/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS_DIMENSION_INL
#define __GRAPHICS_DIMENSION_INL

namespace magnum { namespace graphics {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Dimension<T>::Dimension() {
	width = height = T(0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Dimension<T>::Dimension(const T& w,const T& h) {
	width = w;
	height = h;
}
	

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Dimension<T>::setWidth(const T& w) {
	width = w;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Dimension<T>::setHeight(const T& h) {
	height = h;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T Dimension<T>::getWidth() const {
	return width;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline T Dimension<T>::getHeight() const {
	return height;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Dimension<T>& Dimension<T>::operator=(const Dimension<T>& d) {
	width = d.width;
	height = d.height;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Dimension<T>::operator==(const Dimension<T>& d) const {
	return ((width == d.width) && (height == d.height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Dimension<T>::operator!=(const Dimension<T>& d) const {
	return ((width != d.width) || (height != d.height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Dimension<T>::operator>=(const Dimension<T>& d) const {
	return ((width >= d.width) && (height >= d.height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Dimension<T>::operator<=(const Dimension<T>& d) const {
	return ((width <= d.width) && (height <= d.height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Dimension<T>::operator>(const Dimension<T>& d) const {
	return ((width > d.width) && (height > d.height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline bool Dimension<T>::operator<(const Dimension<T>& d) const {
	return ((width < d.width) && (height < d.height));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Dimension<T> Dimension<T>::operator+(const Dimension<T>& d) const {
	return Dimension<T>(width + d.width, height + d.height);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Dimension<T> Dimension<T>::operator-(const Dimension<T>& d) const {
	return Dimension<T>(width - d.width, height - d.height);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Dimension<T>& Dimension<T>::operator+=(const Dimension<T>& d) {
	width += d.width;
	height += d.height;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Dimension<T>& Dimension<T>::operator-=(const Dimension<T>& d) {
	width -= d.width;
	height -= d.height;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Dimension<T>& Dimension<T>::operator*=(const T& d) {
	width *= d;
	height *= d;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline const Dimension<T>& Dimension<T>::operator/=(const T& d) {
	width /= d;
	height /= d;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> template<class S> 
inline Dimension<T>::operator Dimension<S>() const {
	return Dimension<S>((S)width, (S)height);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Dimension<T> operator*(const T& t,const Dimension<T>& d) {
	return Dimension<T>(t * d.width, t * d.height);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Dimension<T> operator*(const Dimension<T>& d,const T& t) {
	return Dimension<T>(d.width * t, d.height * t);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Dimension<T> operator/(const Dimension<T>& d,const T& t) {
	return Dimension<T>(d.width / t, d.height / t);
}	

}; };
#endif
