/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS_IMAGE_INL
#define __GRAPHICS_IMAGE_INL

namespace magnum { namespace graphics {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String Image::getName() const {
	return m_Name;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Image::setName(const char* n) {
	m_Name = n;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Image::attachNext(Image* img) {
	img->refer();
	if (m_Next != NULL)
		m_Next->release();
	m_Next = img;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Image::detachNext() {
	if (m_Next != NULL) {
		m_Next->release();
		m_Next = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Image* Image::getNext() {
	return m_Next;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Image* Image::getNext() const {
	return m_Next;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Dimension<int>& Image::getSize() const {
	return m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Image::getWidth() const {
	return m_Size.getWidth();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Image::getHeight() const {
	return m_Size.getHeight();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Image::Type Image::getType() const {
	return m_Type;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Image::getLinesize() const {
	return m_Linesize;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Image::getPixelsize() const {
	return (m_Type >> 16) & 0xff;;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Image::isIndexed() const {
	return (m_Type == Index8);
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* Image::lock() {
	ASSERT(!m_Locked);
	void* ret = onLock();
	if (ret == NULL)
		return NULL;
	m_Locked = true;
	return ret;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Image::unlock() {
	ASSERT(m_Locked);
	onUnlock();
	m_Locked = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Image::isLocked() const {
	return m_Locked;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Image::getPalette(Palette<byte>* pal) const {
	return onGetPalette(pal);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Image::setPalette(const Palette<byte>& pal) {
	return onSetPalette(pal);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Image::setLinesize(int ls) {
	m_Linesize = ls;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Image::setType(Image::Type t) {
	m_Type = t;
}


}; };

#endif
