/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS_PALETTE_INL
#define __GRAPHICS_PALETTE_INL

namespace magnum { namespace graphics {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> Palette<T> Palette<T>::operator+ (const Colour<T> &col) const {
	Palette pal;
	int i;
	for (i=0 ;i < 256; i++)
		pal.m_Colour[i] = m_Colour[i] + col;

	return pal;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> Palette<T> Palette<T>::operator- (const Colour<T> &col) const {
	Palette pal;
	int i;
	for (i=0 ;i < 256; i++)
		pal.m_Colour[i] = m_Colour[i] - col;

	return pal;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> const Palette<T>& Palette<T>::operator+=(const Colour<T> &col) {
	int i;
	for (i=0 ;i < 256; i++)
		m_Colour[i] += col;

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> const Palette<T>& Palette<T>::operator-=(const Colour<T> &col) {
	int i;
	for (i=0 ;i < 256; i++)
		m_Colour[i] -= col;

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> const Palette<T>& Palette<T>::operator= (const Palette<T> &pal) {
	int i;
	for (i = 0; i < 256; i++)
		m_Colour[i] = pal.m_Colour[i];

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> bool Palette<T>::operator==(const Palette<T> &pal) const {
	int i;
	for (i = 0; i < 256; i++) {
		if (m_Colour[i] != pal.m_Colour[i])
			return false;
	}
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> bool Palette<T>::operator!=(const Palette<T> &) const {
	int i;
	for (i = 0; i < 256; i++) {
		if (m_Colour[i] != pal.m_Colour[i])
			return true;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Palette<T>::setColours(const Palette<T>& pal,int i,int count) {
	ASSERT((i >= 0) && (i < 256) && (i+count >= 0) && (i + count < 256));
	memcpy(&m_Colour[i],pal.m_Colour[i],count*sizeof(Colour<T>));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline void Palette<T>::setColour(const Colour<T>& col,int i) {
	ASSERT((i >= 0) && (i < 256));
	m_Colour[i] = col;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Colour<T> Palette<T>::getColour(int i) const {
	ASSERT((i >= 0) && (i < 256));
	return m_Colour[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Colour<T>& Palette<T>::operator [](int i) {
	ASSERT((i >= 0) && (i < 256));
	return m_Colour[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> inline Colour<T> Palette<T>::operator [](int i) const {
	ASSERT((i >= 0) && (i < 256));
	return m_Colour[i];
}

}; };

#endif
