/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS_POINT_H
#define __GRAPHICS_POINT_H

#include "math.vector.h"
#include "math.vector2.h"

namespace magnum { namespace graphics {
using magnum::math::Vector;
using magnum::math::Vector2;

template<class T> class Rectangle;
template<class T> class Dimension;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Point  {
friend class Rectangle<T>;
public:
	T					x;
	T					y;

public:
	inline Point();
	inline Point(const T& ,const T& );

	inline void 		setX(const T& );
	inline void 		setY(const T& );

	inline T 			getX() const;
	inline T 			getY() const;

	inline const Point& operator=(const Point& );

	inline bool 		operator==(const Point<T>& ) const;
	inline bool 		operator!=(const Point<T>& ) const;

	inline Point<T>		operator+(const Point<T>& ) const;
	inline Point<T>		operator-(const Point<T>& ) const;
	inline Point<T>		operator+(const Dimension<T>& ) const;
	inline Point<T>		operator-(const Dimension<T>& ) const;

	inline const Point& operator+=(const Point<T>& );
	inline const Point& operator-=(const Point<T>& );
	inline const Point& operator+=(const Dimension<T>& );
	inline const Point& operator-=(const Dimension<T>& );
	inline const Point& operator*=(const T& );
	inline const Point& operator/=(const T& );
	
	inline T&			operator[](unsigned int );
	inline const T&		operator[](unsigned int ) const;

	template<class S>
	inline operator Point<S>();
	
	inline operator Vector<T,2>();
	inline operator Vector2<T>();
};

template<class T> inline Point<T> operator*(const T& ,const Point<T>& );
template<class T> inline Point<T> operator*(const Point<T>& ,const T& );
template<class T> inline Point<T> operator/(const Point<T>& ,const T& );

}; };
#endif
