/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS_POINT_INL
#define __GRAPHICS_POINT_INL

namespace magnum { namespace graphics {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Point<T>::Point() {
	x = y = T(0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Point<T>::Point(const T& nx,const T& ny) {
	x = nx;
	y = ny;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void Point<T>::setX(const T& nx) {
	x = nx;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void Point<T>::setY(const T& ny) {
	y = ny;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline T Point<T>::getX() const {
	return x;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline T Point<T>::getY() const {
	return y;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Point<T>& Point<T>::operator=(const Point<T>& p) {
	x = p.x;
	y = p.y;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool Point<T>::operator==(const Point<T>& p) const {
	return ((x == p.x) && (y == p.y));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool Point<T>::operator!=(const Point<T>& p) const {
	return ((x != p.x) || (y != p.y));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Point<T> Point<T>::operator+(const Point<T>& p) const {
	return Point<T>(x + p.x, y + p.y);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Point<T> Point<T>::operator-(const Point<T>& p) const {
	return Point<T>(x - p.x, y - p.y);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Point<T> Point<T>::operator+(const Dimension<T>& d) const {
	return Point<T>(x + d.width, y + d.height);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Point<T> Point<T>::operator-(const Dimension<T>& d) const {
	return Point<T>(x - d.width, y - d.height);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Point<T>& Point<T>::operator+=(const Point<T>& p) {
	x += p.x;
	y += p.y;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Point<T>& Point<T>::operator-=(const Point<T>& p) {
	x -= p.x;
	y -= p.y;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Point<T>& Point<T>::operator+=(const Dimension<T>& d) {
	x += d.width;
	y += d.height;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Point<T>& Point<T>::operator-=(const Dimension<T>& d) {
	x -= d.width;
	y -= d.height;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Point<T>& Point<T>::operator*=(const T& t) {
	x *= t;
	y *= t;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Point<T>& Point<T>::operator/=(const T& d) {
	x /= d;
	y /= d;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline T& Point<T>::operator[](unsigned int i) {
	ASSERT(i < 2);
	if (i==0)
		return x;
	else
		return y;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const T& Point<T>::operator[](unsigned int i) const {
	ASSERT(i < 2);
	if (i==0)
		return x;
	else
		return y;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> template<class S>
inline Point<T>::operator Point<S>() {
	return Point<S>((S)x, (S)y);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Point<T>::operator Vector<T,2>() {
	Vector<T,2> v;
	v[0] = x;
	v[1] = y;
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Point<T>::operator Vector2<T>() {
	return Vector2<T>(x,y);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Point<T> operator*(const Point<T>& p,const T& t) {
	return Point<T>(p.x * t, p.y * t);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Point<T> operator*(const T& t, const Point<T>& p) {
	return Point<T>(t * p.x, t * p.y);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Point<T> operator/(const Point<T>& p,const T& t) {
	return Point<T>(p.x / t, p.y / t);
}

}; };
#endif
