/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS_TRANSFORM_H
#define __GRAPHICS_TRANSFORM_H

#include "math.h"
#include "graphics.point.h"
#include "graphics.rectangle.h"
#include "graphics.dimension.h"


namespace magnum { namespace graphics {
using magnum::object::Object;
using magnum::types::String;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Transform : public Object {
DeclareClass(Transform, Object);
private:
	T						m_Matrix[2][3];

public:
	inline void				setIdentity();

	inline Point<T>			getTransformX() const;
	inline void				setTransformX(const Point<T>& );
	inline void				setTransformX(T ,T );

	inline Point<T>			getTransformY() const;
	inline void				setTransformY(const Point<T>& );
	inline void				setTransformY(T ,T );

	inline Point<T>			getTranslation() const;
	inline void				setTranslation(const Point<T>& );
	inline void				setTranslation(T ,T );

	inline Point<T>			getScale() const;
	inline void				setScale(const Point<T>& );
	inline void				setScale(T , T );

	inline Transform		operator+(const Point<int>& ) const;
	inline Transform		operator+(const Point<T>& ) const;
	inline Transform		operator-(const Point<int>& ) const;
	inline Transform		operator-(const Point<T>& ) const;
	inline const Transform&	operator+=(const Point<int>& );
	inline const Transform&	operator+=(const Point<T>& );
	inline const Transform&	operator-=(const Point<int>& );
	inline const Transform&	operator-=(const Point<T>& );

	inline Transform		operator*(const Transform& ) const;
	inline const Transform&	operator*=(const Transform& );

	inline Point<T>			operator*(const Point<T>& ) const;
	inline Point<T>			operator%(const Point<T>& ) const;
	inline Rectangle<T>		operator*(const Rectangle<T>& ) const;
	inline Rectangle<T>		operator%(const Rectangle<T>& ) const;
	inline Dimension<T>		operator*(const Dimension<T>& ) const;
	inline Dimension<T>		operator%(const Dimension<T>& ) const;

	inline bool				operator==(const Transform& ) const;
	inline bool				operator!=(const Transform& ) const;
	
	inline const T*			operator[](unsigned int ) const;
	inline T*				operator[](unsigned int );
};

}; };

#endif
