/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS_TRANSLATOR_DDS_H
#define __GRAPHICS_TRANSLATOR_DDS_H

#include "graphics.image.h"
#include "io.translator.h"


namespace magnum { namespace graphics {
using magnum::object::Object;
using magnum::types::String;
using magnum::types::dword;
using magnum::io::File;
using magnum::io::InputStream;
using magnum::io::OutputStream;
using magnum::io::Translator;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TranslatorDDS : public Translator<Image> {
DeclareClass(TranslatorDDS, Translator<Image>);
private:
	struct DdsHeader {
		dword	ddsIdentifier;
		dword	headerSize;
		dword	flags;
		dword	height;
		dword	width;
		dword	linesize;
		dword	depth;
		dword	m_MipMaps;
		
		dword	reserved1[12];
		dword	pixelFlags;
		dword	pixelFourCC;
		dword	pixelBpp;
		dword	reserved2[4];
		dword	caps1;
		dword	caps2;
		dword	reserved3[3];
	};

	Image::Type		m_Type;
	unsigned int	m_Width;
	unsigned int	m_Height;
	unsigned int	m_Depth;
	unsigned int	m_MipMaps;
	unsigned int	m_CubeMaps;
	DdsHeader		m_Header;
	InputStream*	m_Input;
	
protected:
	Image*			onDecode(InputStream* ,size_t size = Autosize);
	size_t			onEncode(OutputStream* ,const Image* );
	bool			onIdentify(File* );
	
private:
	Image*			readTexture2D();
	Image*			readCubeTexture();
	Image*			readTexture3D();	
};


}; };

#endif
