/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS_TRANSLATOR_PSD_H
#define __GRAPHICS_TRANSLATOR_PSD_H

#include "graphics.image.h"
#include "io.translator.h"


namespace magnum { namespace graphics {
using magnum::object::Object;
using magnum::types::String;
using magnum::types::uint16;
using magnum::types::uint32;
using magnum::types::uint8;
using magnum::io::File;
using magnum::io::InputStream;
using magnum::io::OutputStream;
using magnum::io::Translator;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TranslatorPSD : public Translator<Image> {
DeclareClass(TranslatorPSD, Translator<Image>);
public:
	TranslatorPSD();
	~TranslatorPSD();
	
protected:
	Image*		onDecode(InputStream* ,size_t size = Autosize);
	size_t		onEncode(OutputStream* ,const Image* );
	bool		onIdentify(File* );

private:	
	struct Header {
		uint8	m_Signature[4];	// Always equal to 8BPS.
		uint16	m_Version;		// Always equal to 1
		uint8	m_Reserved[6];	// Must be zero
		uint16	m_Channels;		// Number of any channels inc. alphas
		uint16	m_Height2;
		uint16	m_Height;		// Rows Height of image in pixel
		uint16	m_Width2;
		uint16	m_Width;		// Colums Width of image in pixel
		uint16	m_Depth;		// Bits/channel
		uint16	m_Mode;			// Color mode of the file (Bitmap/Grayscale..)
	};
	
private:
	void		readRaw(InputStream *file, Image* img);
	void		readRLE(InputStream *file, Image* img);
	int			getShift(int channelNr);
private:
	Header		m_Header;

};


}; };

#endif
