/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_ACCELERATOR_H
#define __GRAPHICS3D_ACCELERATOR_H

#include "math.matrix.h"
#include "graphics3d.texture.h"
#include "graphics3d.light.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::object::Noncopyable;
using magnum::math::Matrix;

class Context;
class Appearance;
class Shader;
class Material;
class FrameBuffer;
class Mesh;
class ParameterBuffer;
class Model;
class Camera;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Accelerator : public Object, private Noncopyable {
DeclareClass(Accelerator, Object);
public:
	Accelerator(Context* );
	~Accelerator();
	
	inline Context*		getContext() const;
	
	inline void			select(FrameBuffer* );
	inline void			deselectFrameBuffer();
	inline void			select(const Appearance* );
	inline void			deselectAppearance();
	inline void			select(const Shader* );
	inline void			deselectShader();
	inline void			select(const Texture* ,unsigned int );
	inline void			deselectTexture(unsigned int );
	inline void			select(const Material* );
	inline void			deselectMaterial();
	inline void			select(const Light* ,unsigned int );
	inline void			deselectLight(unsigned int );
	inline void			select(const Camera* );

	void				render(const Mesh* ,const void* ,void (*)(Mesh* ,const void* ));
	void				renderSorted(const Mesh* ,const void* ,void (*)(Mesh* ,const void* ),float key);
	inline void			render(const Model* ,const ParameterBuffer* );
	
	void				flush();
	void				clear();
	
private:
	struct RenderState {
		Shader*			m_Shader;
		Material*		m_Material;
		Texture*		m_Texture[Texture::MaxTextures];
		Light*			m_Light[Light::MaxLights];
	};
	struct DeferedMesh {
		Mesh*			m_Mesh;
		float			m_Key;
	};
	
	Context*			m_Context;
};

}; };

#endif
