/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_ANIMATION_H
#define __GRAPHICS3D_ANIMATION_H

#include "container.array.h"
#include "graphics3d.resource.h"
#include "graphics3d.keyframe.h"
#include "graphics3d.parameterformat.h"

namespace magnum { namespace graphics3d {
using magnum::container::Array;

class ParameterBuffer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Animation : public Object {
DeclareClass(Animation, Object);
public:
	enum Direction {
		DefaultDirection = 0,
		Forward = 1,
		Backward = 2,
	};
	
	enum Repeat {
		DefaultRepeat = 0,
		Single = 1,
		Loop = 2,
		PingPong = 3,
		PingPongLoop = 4,
	};
	
	enum Interpolation {
		DefaultInterpolation = 0,
		Linear = 1,
		Stepped = 2,
		Spline = 3,
	};
	
	
protected:
	Animation();
	explicit Animation(const char* );

public:
	inline String		getName() const;
	inline void			setName(const char* );

	inline void			setStart(float );
	inline float		getStart() const;
	
	inline void			setEnd(float );
	inline float		getEnd() const;
	
	inline float		getLength() const;
	
	inline void			setDirection(Direction );
	inline Direction	getDirection() const;

	inline void			setRepeat(Repeat );
	inline Repeat		getRepeat() const;

	inline void			setInterpolation(Interpolation );
	inline Interpolation getInterpolation() const;
	
	inline String		getLocation() const;
	inline void			setLocation(const char* );

	inline const ParameterFormat& getParameters() const;
	
	inline void			animate(float t,void* ) const;
	inline void			animate(float t,ParameterBuffer* ) const;
	inline void			animate(float t,ParameterBuffer* ,ParameterFormat::Location ) const;
	
protected:
	virtual void		onAnimate(float t,void* ) const;

	inline void			publishParameters(const ParameterFormat& );

	float				normaliseTime(float ) const;
	float				stepTime(float ntime, float step) const;
	template<class T>
	const T*			getKeyframe(const Array<T>& ,float ) const;
	template<class T>
	const T*			getKeyframe(const Array<T>& ,float ,const T* ) const;
	template<class T>
	void				sortKeyframes(Array<T>& );
	
private:
	String				m_Name;
	String				m_Location;
	float				m_Start;
	float				m_End;
	float				m_Length;
	Interpolation		m_Interpolation;
	Direction			m_Direction;	
	Repeat				m_Repeat;
	ParameterFormat		m_Parameters;
};

typedef Array<Animation*>	AnimationList;

}; };

#endif
