/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_APPEARANCE_H
#define __GRAPHICS3D_APPEARANCE_H

#include "object.pointer.h"
#include "graphics3d.texture.h"

namespace magnum { namespace graphics3d {
using magnum::object::Pointer;

class Shader;
class Material;
class BlendMode;
class StencilMode;
class DepthMode;
class PolygonMode;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Appearance : public Resource {
DeclareClass(Appearance, Resource);
public:
	Appearance();
	
	void				select() const;
	void				deselect() const;
	void				forceSelect() const;
	static void			selectNone();
	static void			selectDefault();

	inline void			attachTexture(Texture*, unsigned int stage=0);
	inline void			detachTexture(unsigned int stage=0);
	inline Texture*		getTexture(unsigned int stage=0) const;
	
	inline Material*	getMaterial() const;
	inline void			attachMaterial(Material* );
	inline void			defaultMaterial();
	inline void			detachMaterial();

	inline Shader*		getShader() const;
	inline void			attachShader(Shader* );
	inline void			defaultShader();
	inline void			detachShader();

	inline BlendMode*	getBlendMode() const;
	inline void			attachBlendMode(BlendMode* );
	inline void			defaultBlendMode();
	inline void			detachBlendMode();

	inline StencilMode*	getStencilMode() const;
	inline void			attachStencilMode(StencilMode* );
	inline void			defaultStencilMode();
	inline void			detachStencilMode();

	inline DepthMode*	getDepthMode() const;
	inline void			attachDepthMode(DepthMode* );
	inline void			defaultDepthMode();
	inline void			detachDepthMode();

	inline PolygonMode*	getPolygonMode() const;
	inline void			attachPolygonMode(PolygonMode* );
	inline void			defaultPolygonMode();
	inline void			detachPolygonMode();
	
	inline bool			isTransparent() const;

private:
	Pointer<Material>	m_Material;
	Pointer<Shader>		m_Shader;
	Pointer<BlendMode>	m_BlendMode;
	Pointer<StencilMode> m_StencilMode;
	Pointer<DepthMode>	m_DepthMode;
	Pointer<PolygonMode> m_PolygonMode;
	Pointer<Texture>	m_Texture[Texture::MaxTextures];
};


}; };

#endif
