/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_APPEARANCE_INL
#define __GRAPHICS3D_APPEARANCE_INL

#include "graphics3d.shader.h"
#include "graphics3d.material.h"
#include "graphics3d.blendmode.h"
#include "graphics3d.depthmode.h"
#include "graphics3d.stencilmode.h"
#include "graphics3d.polygonmode.h"

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Material* Appearance::getMaterial() const {
	return m_Material;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::attachMaterial(Material* mat) {
	ASSERT(mat != NULL);
	m_Material = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::defaultMaterial() {
	m_Material = Material::getDefault();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::detachMaterial() {
	m_Material = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Shader* Appearance::getShader() const {
	return m_Shader;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::attachShader(Shader* s) {
	ASSERT(s != NULL);
	m_Shader = s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::defaultShader() {
	m_Shader = Shader::getDefault();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::detachShader() {
	m_Shader = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline BlendMode* Appearance::getBlendMode() const {
	return m_BlendMode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::attachBlendMode(BlendMode* s) {
	ASSERT(s != NULL);
	m_BlendMode = s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::defaultBlendMode() {
	m_BlendMode = BlendMode::getDefault();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::detachBlendMode() {
	m_BlendMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Appearance::isTransparent() const {
	if (m_BlendMode == NULL)
		return false;
	return m_BlendMode->isTransparent();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline StencilMode* Appearance::getStencilMode() const {
	return m_StencilMode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::attachStencilMode(StencilMode* s) {
	ASSERT(s != NULL);
	m_StencilMode = s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::defaultStencilMode() {
	m_StencilMode = StencilMode::getDefault();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::detachStencilMode() {
	m_StencilMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DepthMode* Appearance::getDepthMode() const {
	return m_DepthMode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::attachDepthMode(DepthMode* s) {
	ASSERT(s != NULL);
	m_DepthMode = s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::defaultDepthMode() {
	m_DepthMode = DepthMode::getDefault();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::detachDepthMode() {
	m_DepthMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline PolygonMode* Appearance::getPolygonMode() const {
	return m_PolygonMode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::attachPolygonMode(PolygonMode* s) {
	ASSERT(s != NULL);
	m_PolygonMode = s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::defaultPolygonMode() {
	m_PolygonMode = PolygonMode::getDefault();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::detachPolygonMode() {
	m_PolygonMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::attachTexture(Texture* t, unsigned int stage) {
	ASSERT((stage >= 0) && (stage < Texture::MaxTextures));
	ASSERT(t != NULL);
	m_Texture[stage] = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Appearance::detachTexture(unsigned int stage) {
	ASSERT((stage >= 0) && (stage < Texture::MaxTextures));
	m_Texture[stage] = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture*	Appearance::getTexture(unsigned int stage) const {
	ASSERT((stage >= 0) && (stage < Texture::MaxTextures));
	return m_Texture[stage];
}


}; };

#endif
