/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_BLENDMODE_H
#define __GRAPHICS3D_BLENDMODE_H

#include "system.assembler.h"
#include "graphics3d.opengl.h"
#include "graphics3d.resource.h"


namespace magnum { namespace graphics3d {
using magnum::system::Assembler;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class BlendMode : public Resource {
DeclareClass(BlendMode, Resource);
public:
	enum Equation {
		Add = GL_FUNC_ADD,
		Subtract = GL_FUNC_SUBTRACT,
		ReverseSubtract = GL_FUNC_REVERSE_SUBTRACT,
		Min = GL_MIN,
		Max = GL_MAX
	};

	enum BlendFunction {
		Zero = GL_ZERO,
		One = GL_ONE,
		DstColour = GL_DST_COLOR,
		DstColourInv = GL_ONE_MINUS_DST_COLOR, 
		DstAlpha = GL_DST_ALPHA, 
		DstAlphaInv = GL_ONE_MINUS_DST_ALPHA,
		SrcColour = GL_SRC_COLOR,
		SrcColourInv = GL_ONE_MINUS_SRC_COLOR, 
		SrcAlpha = GL_SRC_ALPHA, 
		SrcAlphaInv = GL_ONE_MINUS_SRC_ALPHA,
		SrcAlphaSaturate = GL_SRC_ALPHA_SATURATE,
		ConstAlpha = GL_CONSTANT_ALPHA,
		ConstAlphaInv = GL_ONE_MINUS_CONSTANT_ALPHA,
		ConstColour = GL_CONSTANT_COLOR,
		ConstColourInv = GL_ONE_MINUS_CONSTANT_COLOR,
	};

	enum TestFunction {
		Never = GL_NEVER,
		Less = GL_LESS,
		Equal = GL_EQUAL,
		LessEqual = GL_LEQUAL,
		Greater = GL_GREATER,
		GreaterEqual = GL_GEQUAL,
		NotEqual = GL_NOTEQUAL,
		Always = GL_ALWAYS,
		Disable = -1,
	};

public:
	BlendMode();
	~BlendMode();

	inline void			select() const;
	inline void			deselect() const;
	inline void			forceSelect() const;
	static inline void	selectNone();
	static inline void	selectDefault();
	static inline BlendMode* getDefault();

	void				compileSelect(Assembler& ) const;
	void				compileDeselect(Assembler& ) const;
	
	// Blending
	inline void			setFunction(BlendFunction ,BlendFunction );
	inline void			setEquation(Equation );
	inline Equation		getEquation() const;
	inline void			setColour(const Colour<float>& );
	inline Colour<float> getColour() const;
	
	// Alphatest				
	inline void			setAlphaTest(TestFunction ,float ref);
	inline void			setAlphaTest(TestFunction );
	inline void			setAlphaReference(float );
	inline TestFunction	getAlphaTest() const;
	inline float		getAlphaReference() const;
	
	inline bool			isTransparent() const;

protected:
	void				onSelect() const;
	bool				onCompileSelect(Assembler& ) const;
	
private:
	BlendFunction		m_Source;
	BlendFunction		m_Destination;
	Equation			m_Equation;
	Colour<float>		m_Colour;
	TestFunction		m_AlphaTest;
	float				m_AlphaRef;
};

}; };

#endif
