/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_BLENDMODE_INL
#define __GRAPHICS3D_BLENDMODE_INL

namespace magnum { namespace graphics3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline BlendMode* BlendMode::getDefault() {
	return &getContext()->m_DefaultBlendMode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void BlendMode::select() const {
	Context* context = getContext();
	if (context->m_BlendMode == this)
		return;
	context->m_BlendMode = this;
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	BlendMode::forceSelect() const {
	Context* context = getContext();
	context->m_BlendMode = this;
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	BlendMode::deselect() const {
	Context* context = getContext();
	if (context->m_BlendMode == this)
		context->m_BlendMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	BlendMode::selectNone() {
	Context* context = getContext();
	context->m_BlendMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	BlendMode::selectDefault() {
	Context* context = getContext();
	context->m_BlendMode = &context->m_DefaultBlendMode;
	context->m_BlendMode->onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool BlendMode::isTransparent() const {
	return ((m_Source != One) || (m_Destination != Zero));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void BlendMode::setFunction(BlendMode::BlendFunction src,BlendMode::BlendFunction dst) {
	m_Source = src;
	m_Destination = dst;
}
					

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	BlendMode::setEquation(BlendMode::Equation e) {
	m_Equation = e;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline BlendMode::Equation BlendMode::getEquation() const {
	return m_Equation;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	BlendMode::setColour(const Colour<float>& c) {
	m_Colour = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Colour<float> BlendMode::getColour() const {
	return m_Colour;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	BlendMode::setAlphaTest(BlendMode::TestFunction t,float ref) {
	m_AlphaTest = t;
	m_AlphaRef = ref;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	BlendMode::setAlphaTest(BlendMode::TestFunction t) {
	m_AlphaTest = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline BlendMode::TestFunction BlendMode::getAlphaTest() const {
	return m_AlphaTest;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void BlendMode::setAlphaReference(float r) {
	m_AlphaRef = r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float BlendMode::getAlphaReference() const {
	return m_AlphaRef;
}

}; };

#endif
