/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_BONEANIMATION_H
#define __GRAPHICS3D_BONEANIMATION_H

#include "object.pointer.h"
#include "math.quaternion.h"
#include "container.array.h"
#include "graphics3d.animation.h"
#include "graphics3d.keyframe.h"

namespace magnum { namespace graphics3d {
using magnum::container::Array;
using magnum::object::Pointer;
using magnum::math::Quaternion;
using magnum::math::Vector3;

class Skeleton;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
struct BoneTransform {
	Quaternion<float> rotation;
	Vector3<float>	offset;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class BoneAnimation : public Animation {
DeclareClass(BoneAnimation, Animation);
public:
	typedef Keyframe<BoneTransform,0> Frame;
	typedef Array<Frame>::Iterator Iterator;

public:
	inline BoneAnimation(Skeleton* );
	inline explicit BoneAnimation(Skeleton* ,const char* );
	
	inline Skeleton*	getSkeleton() const;

	inline void			insert(const Frame& );
	inline void			remove(const Iterator& );
	inline void			remove(const Frame& );
	void				remove(float );
	
	inline unsigned int getSize() const;
	inline const Frame& operator[](unsigned int ) const;
	inline Frame&		operator[](unsigned int );

	void				clear();
	void				finish();

	inline Iterator		begin() const;
	inline Iterator		at(unsigned int ) const;
	inline Iterator 	end() const;

protected:
	virtual void		onAnimate(float t,void* ) const;

private:	
	Array<Frame>		m_Frames;
	Pointer<Skeleton>	m_Skeleton;
};

}; };

#endif
