/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_BONEMODEL_H
#define __GRAPHICS3D_BONEMODEL_H

#include "graphics3d.model.h"
#include "graphics3d.mesh.h"
#include "graphics3d.boneanimation.h"

namespace magnum { namespace graphics3d {

class WeightBuffer;
class VertexBuffer;
class Skeleton;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class BoneModel : public Model {
DeclareClass(BoneModel, Model);
public:
	inline BoneModel();
	inline ~BoneModel();
	
	inline void				insertMesh(Mesh* );
	inline void				removeMesh(Mesh* );
	
	inline void				attachSkeleton(Skeleton* );
	inline void				detachSkeleton();
	inline Skeleton*		getSkeleton() const;
	
	inline void				attachWeightBuffer(WeightBuffer* );
	inline void				detachWeightBuffer();
	inline WeightBuffer*	getWeightBuffer() const;				

	inline void				insertAnimation(BoneAnimation* );
	inline void				removeAnimation(BoneAnimation* );

	void					finish();

protected:
	virtual void			onRender(const ParameterBuffer* ) const;
	virtual void			onTransform(const ParameterBuffer* ,Matrix4<float>* ,Joint::Handle ) const;
	virtual bool			onOptimize();
	virtual void			onClear();

private:
	Pointer<WeightBuffer>	m_Weights;
	Pointer<VertexBuffer>	m_Vertices;
	Pointer<Skeleton>		m_Skeleton;
};

}; };

#endif
