/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_CAMERA_H
#define __GRAPHICS3D_CAMERA_H

#include "math.vector2.h"
#include "math.vector3.h"
#include "geometry.ray.h"
#include "graphics.point.h"
#include "graphics3d.resource.h"


namespace magnum { namespace graphics3d {
using magnum::math::Vector2;
using magnum::math::Vector3;
using magnum::geometry::Ray;
using magnum::graphics::Point;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Camera : public Resource {
DeclareClass(Camera, Resource);
public:
	inline Camera();

	inline void			select() const;

	inline void			setNearPlane(float );
	inline float		getNearPlane() const;
	inline void			setFarPlane(float );
	inline float		getFarPlane() const;
	inline void			setProjectionPlane(float );
	inline float		getProjectionPlane() const;
	
	inline Ray<float,3>	createRay(const Vector2<float>& ) const;
	inline Ray<float,3>	createRay(const Point<float>& ) const;
	inline Ray<float,3>	createRay(float ,float ) const;

protected:
	virtual void		onSelect() const = 0;
	virtual Ray<float,3> onCreateRay(float ,float ) const = 0;

private:
	float				m_NearPlane;
	float				m_FarPlane;
	float				m_ProjectionPlane;
};

}; };

#endif
