/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_CAMERA_INL
#define __GRAPHICS3D_CAMERA_INL


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Camera::Camera() {
	m_NearPlane = 10.0f;
	m_ProjectionPlane = 10.0f;
	m_FarPlane = 100.0f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Camera::setNearPlane(float f) {
	m_NearPlane = f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Camera::getNearPlane() const {
	return m_NearPlane;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Camera::setFarPlane(float f) {
	m_FarPlane = f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Camera::getFarPlane() const {
	return m_FarPlane;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Camera::setProjectionPlane(float f) {
	m_ProjectionPlane = f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Camera::getProjectionPlane() const {
	return m_ProjectionPlane;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Camera::select() const {
	onSelect();
}


/*--------------------------------------------------------------------------*/
/** Creates a ray that starts in the origin and passes through a point
 * on the view window. The coordinates for x and y should go from -1 to
 * 1, where (-1,-1) is the upper right corner and (1,1) is the lower left
 * corner.
 */
inline Ray<float,3>	Camera::createRay(const Vector2<float>& p) const {
	return onCreateRay(p.x, p.y);
}


/*--------------------------------------------------------------------------*/
/** Creates a ray that starts in the origin and passes through a point
 * on the view window. The coordinates for x and y should go from -1 to
 * 1, where (-1,-1) is the upper right corner and (1,1) is the lower left
 * corner.
 */
inline Ray<float,3>	Camera::createRay(const Point<float>& p) const {
	return onCreateRay(p.x, p.y);
}


/*--------------------------------------------------------------------------*/
/** Creates a ray that starts in the origin and passes through a point
 * on the view window. The coordinates for x and y should go from -1 to
 * 1, where (-1,-1) is the upper right corner and (1,1) is the lower left
 * corner.
 */
inline Ray<float,3>	Camera::createRay(float x,float y) const {
	return onCreateRay(x,y);
}


}; };

#endif
