/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_CONSTANT_H
#define __GRAPHICS3D_CONSTANT_H

#include "math.vector.h"
#include "math.vector2.h"
#include "math.vector3.h"
#include "math.vector4.h"
#include "math.matrix.h"
#include "graphics.colour.h"
#include "graphics3d.types.h"
#include "container.array.h"
#include "system.assembler.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Array;
using magnum::math::Vector;
using magnum::math::Vector2;
using magnum::math::Vector3;
using magnum::math::Vector4;
using magnum::math::Matrix;
using magnum::math::Matrix2;
using magnum::math::Matrix3;
using magnum::math::Matrix4;
using magnum::graphics::Colour;
using magnum::system::Assembler;

class ConstantList;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Constant : public Object {
DeclareClass(Constant, Object);
friend class ConstantList;
private:
	char			m_Name[64];
	char			m_Description[256];
	Types::Type		m_Type;

public:
	Constant(Types::Type, const char* ,const char* );

	inline String			getName() const;
	inline String			getDescription() const;
	inline Types::Type		getType() const;

	inline void				setFloat(float );
	inline void				setFloat(const float* );
	inline void				setDouble(double );
	inline void				setDouble(const double* );
	inline void				setInteger(int );
	inline void				setInteger(const int* );
	inline void				setVector(const Vector3<float>& );
	inline void				setVector(const Vector4<float>& );
	inline void				setVector(const Vector3<double>& );
	inline void				setVector(const Vector4<double>& );
	inline void				setMatrix(const Matrix3<float>& );
	inline void				setMatrix(const Matrix4<float>& );
	inline void				setMatrix(const Matrix3<double>& );
	inline void				setMatrix(const Matrix4<double>& );

	inline void				compileFloat(Assembler& ,float );
	inline void				compileFloat(Assembler& ,const float* );
	inline void				compileDouble(Assembler& ,double );
	inline void				compileDouble(Assembler& ,const double* );
	inline void				compileInteger(Assembler& ,int );
	inline void				compileInteger(Assembler& ,const int* );
	inline void				compileVector(Assembler& ,const Vector3<float>& );
	inline void				compileVector(Assembler& ,const Vector4<float>& );
	inline void				compileVector(Assembler& ,const Vector3<double>& );
	inline void				compileVector(Assembler& ,const Vector4<double>& );
	inline void				compileMatrix(Assembler& ,const Matrix3<float>& );
	inline void				compileMatrix(Assembler& ,const Matrix4<float>& );
	inline void				compileMatrix(Assembler& ,const Matrix3<double>& );
	inline void				compileMatrix(Assembler& ,const Matrix4<double>& );

protected:
	inline void				setName(const char* );
	inline void				setDescription(const char* );

	virtual void			onSetFloat(float );
	virtual void			onSetFloat(const float* );
	virtual void			onSetDouble(double );
	virtual void			onSetDouble(const double* );
	virtual void			onSetInteger(int );
	virtual void			onSetInteger(const int* );
	virtual void			onSetVector(const Vector3<float>& );
	virtual void			onSetVector(const Vector4<float>& );
	virtual void			onSetVector(const Vector3<double>& );
	virtual void			onSetVector(const Vector4<double>& );
	virtual void			onSetMatrix(const Matrix3<float>& );
	virtual void			onSetMatrix(const Matrix4<float>& );
	virtual void			onSetMatrix(const Matrix3<double>& );
	virtual void			onSetMatrix(const Matrix4<double>& );

	virtual void			onCompileFloat(Assembler& ,float );
	virtual void			onCompileFloat(Assembler& ,const float* );
	virtual void			onCompileDouble(Assembler& ,double );
	virtual void			onCompileDouble(Assembler& ,const double* );
	virtual void			onCompileInteger(Assembler& ,int );
	virtual void			onCompileInteger(Assembler& ,const int* );
	virtual void			onCompileVector(Assembler& ,const Vector3<float>& );
	virtual void			onCompileVector(Assembler& ,const Vector4<float>& );
	virtual void			onCompileVector(Assembler& ,const Vector3<double>& );
	virtual void			onCompileVector(Assembler& ,const Vector4<double>& );
	virtual void			onCompileMatrix(Assembler& ,const Matrix3<float>& );
	virtual void			onCompileMatrix(Assembler& ,const Matrix4<float>& );
	virtual void			onCompileMatrix(Assembler& ,const Matrix3<double>& );
	virtual void			onCompileMatrix(Assembler& ,const Matrix4<double>& );
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ConstantList : public Object {
DeclareClass(ConstantList, Object);
public:
	typedef magnum::container::Array<Constant*>::Iterator	Iterator;
	
private:
	Array<Constant*>		m_Constants;
	
public:
	inline ConstantList();

	inline void				clear();
	inline void				add(Constant* );
	void					remove(const char* );
	void					remove(Constant* );
	inline void				shrink();
	bool					contains(const char* ) const;
	inline bool				contains(Constant* ) const;
	inline void				append(const ConstantList& );
							
	inline unsigned int		getSize() const;
							
	Constant* 				operator[](const char* name) const;
							
	inline Iterator 		begin() const;
	Iterator				at(const char* name) const;
	inline Iterator 		end() const;
	
	inline const ConstantList& operator=(const ConstantList&);
	inline bool				operator==(const ConstantList& ) const;
	inline bool				operator!=(const ConstantList& ) const;
};

}; };

#endif
