/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_CUBEMAPGENERATOR_H
#define __GRAPHICS3D_CUBEMAPGENERATOR_H

#include "graphics3d.perspectivecamera.h"
#include "graphics3d.cubetexture.h"
#include "graphics3d.framebuffer.h"

namespace magnum { namespace graphics3d {
using magnum::math::Matrix;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class CubemapGenerator : public Resource {
DeclareClass(CubemapGenerator, Resource);
public:
	enum Mode {
		Compatible	= 1,
		Fast		= 2,
		Auto		= 3
	};

public:
	CubemapGenerator();
	explicit CubemapGenerator(CubeTexture* ,Mode = Auto);
	~CubemapGenerator();

	void					attachTexture(CubeTexture* ,Mode = Auto);
	void					detachTexture();
	inline CubeTexture*		getTexture() const;
	
	inline FrameBuffer*		getFrameBuffer(CubeTexture::Face ) const;
	inline const Matrix4<float>& getTransform(CubeTexture::Face ) const;
	
	inline void				setNearPlane(float );
	inline float			getNearPlane() const;

	inline void				setFarPlane(float );
	inline float			getFarPlane() const;
	
	inline Mode				getMode() const;
	
	template<class F>
	void					render(F& ,const Matrix4<float>& );
	void					render(const Matrix4<float>& );
	
protected:
	virtual void			onRender(const Matrix4<float>& ,CubeTexture::Face );
	
private:
	void					initTransforms();

private:
	Matrix4<float>			m_Transform[6];
	PerspectiveCamera		m_Camera;
	Pointer<CubeTexture>	m_Texture;
	FrameBuffer*			m_FrameBuffer[6];
	Mode					m_Mode;
};

}; };

#endif
