/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_CUBEMAPGENERATOR_INL
#define __GRAPHICS3D_CUBEMAPGENERATOR_INL

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline CubeTexture* CubemapGenerator::getTexture() const {
	return m_Texture;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline FrameBuffer*	CubemapGenerator::getFrameBuffer(CubeTexture::Face f) const {
	return m_FrameBuffer[f - CubeTexture::PositiveX];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Matrix4<float>& CubemapGenerator::getTransform(CubeTexture::Face f) const {
	return m_Transform[f - CubeTexture::PositiveX];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void CubemapGenerator::setNearPlane(float n) {
	m_Camera.setNearPlane(n);
	m_Camera.setViewWindow(Rectangle<float>(-n,-n,2*n,2*n));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float CubemapGenerator::getNearPlane() const {
	return m_Camera.getNearPlane();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void CubemapGenerator::setFarPlane(float f) {
	m_Camera.setFarPlane(f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float CubemapGenerator::getFarPlane() const {
	return m_Camera.getFarPlane();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline CubemapGenerator::Mode CubemapGenerator::getMode() const {
	return m_Mode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class F>
void CubemapGenerator::render(F& fn,const Matrix4<float>& mtx) {
	m_Camera.select();
	if (m_Mode == Compatible) {
		Point<float> p(0,0);
		Rectangle<float> r(0,0,(float)m_Texture->getWidth(),(float)m_Texture->getHeight());
		Context* c = getContext();
		Rectangle<float> oldviewport = c->getViewport();
		c->setViewport(r);
		fn(m_Transform[0]*mtx, CubeTexture::PositiveX);
		c->copyTo(m_Texture, CubeTexture::PositiveX, 0, p, r);
		fn(m_Transform[1]*mtx, CubeTexture::NegativeX);
		c->copyTo(m_Texture, CubeTexture::NegativeX, 0, p, r);
		fn(m_Transform[2]*mtx, CubeTexture::PositiveY);
		c->copyTo(m_Texture, CubeTexture::PositiveY, 0, p, r);
		fn(m_Transform[3]*mtx, CubeTexture::NegativeY);
		c->copyTo(m_Texture, CubeTexture::NegativeY, 0, p, r);
		fn(m_Transform[4]*mtx, CubeTexture::PositiveZ);
		c->copyTo(m_Texture, CubeTexture::PositiveZ, 0, p, r);
		fn(m_Transform[5]*mtx, CubeTexture::NegativeZ);
		c->copyTo(m_Texture, CubeTexture::NegativeZ, 0, p, r);
		c->setViewport(oldviewport);
	} else {
		FrameBuffer* oldfb = getContext()->getFrameBuffer();
		m_FrameBuffer[0]->select();
		fn(m_Transform[0]*mtx, CubeTexture::PositiveX);
		m_FrameBuffer[1]->select();
		fn(m_Transform[1]*mtx, CubeTexture::NegativeX);
		m_FrameBuffer[2]->select();
		fn(m_Transform[2]*mtx, CubeTexture::PositiveY);
		m_FrameBuffer[3]->select();
		fn(m_Transform[3]*mtx, CubeTexture::NegativeY);
		m_FrameBuffer[4]->select();
		fn(m_Transform[4]*mtx, CubeTexture::PositiveZ);
		m_FrameBuffer[5]->select();
		fn(m_Transform[5]*mtx, CubeTexture::NegativeZ);
		oldfb->select();
	}
}

}; };

#endif
