/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_CUBETEXTURE_H
#define __GRAPHICS3D_CUBETEXTURE_H

#include "graphics3d.texture.h"
#include "graphics.dimension.h"
#include "graphics.rectangle.h"
#include "graphics.palette.h"
#include "graphics.image.h"


namespace magnum { namespace graphics3d {
using magnum::graphics::Image;
using magnum::graphics::Dimension;
using magnum::graphics::Point;
using magnum::graphics::Rectangle;
using magnum::graphics::Palette;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class CubeTexture : public Texture {
DeclareClass(CubeTexture, Texture);
private:
	Dimension<int>		m_Size;
	bool				m_Locked;
	static GLenum		g_Faces[6];
	
public:
	enum Face {
		PositiveX = GL_TEXTURE_CUBE_MAP_POSITIVE_X,
		NegativeX = GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
		PositiveY = GL_TEXTURE_CUBE_MAP_POSITIVE_Y,
		NegativeY = GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
		PositiveZ = GL_TEXTURE_CUBE_MAP_POSITIVE_Z,
		NegativeZ = GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
	};
	

protected:
	CubeTexture(unsigned int tex);

public:
	inline static Dimension<int> getMinSize();
	inline static Dimension<int> getMaxSize();

	static CubeTexture*	create(Type ,Dimension<int>& d, int levels = GenerateAll);
	static CubeTexture*	create(Type ,unsigned int w, unsigned int h, int levels = GenerateAll);
	static CubeTexture*	create(const Image* ,int levels = GenerateAll);

	~CubeTexture();

	bool				update(const Image* ,int level = 0);
	bool				update(const Image* ,Face ,int level = 0);

	bool				copyTo(Image* img, const Point<int>& ,const Rectangle<int>& ) const;
	bool				copyFrom(const Image* img, const Point<int>& ,const Rectangle<int>& );
	bool				copyFrom(const Image* img, const Point<int>& );

	void*				lock(Face ,int level, const Rectangle<int>& , LockFlag );
	void				unlock();
	inline bool			isLocked() const;

	inline Dimension<int> getSize() const;
	inline unsigned int	getWidth() const;
	inline unsigned int	getHeight() const;
	inline unsigned int	getLevels() const;
};

}; };

#endif
