/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_DEFEREDMESH_H
#define __GRAPHICS3D_DEFEREDMESH_H

#include "math.matrix.h"
#include "geometry.bounds.h"
#include "graphics3d.texture.h"

namespace magnum { namespace graphics3d {
using magnum::math::Matrix4;
using magnum::math::Matrix;
using magnum::geometry::Bounds3;

class Mesh;
class BlendMode;
class PolygonMode;
class StencilMode;
class DepthMode;
class Material;
class Shader;
class Context;

/*--------------------------------------------------------------------------*/
/**
 */
class DeferedMesh {
public:
	DeferedMesh(const Context* ,const Mesh* ,const Matrix<float,4>& );
	
	inline const Mesh*			getMesh() const;
	inline const BlendMode*		getBlendMode() const;
	inline const PolygonMode*	getPolygoneMode() const;
	inline const StencilMode*	getStencilMode() const;
	inline const DepthMode*		getDepthMode() const;
	inline const Shader*		getShader() const;
	inline const Material*		getMaterial() const;
	inline const Texture*		getTexture(unsigned int = 0) const;

	inline const Bounds3<float>& getBounds() const;
	inline const Matrix4<float>& getMatrix() const;
	inline float				getKey() const;

	void						render() const;
	
private:
	const Mesh*			m_Mesh;
	const BlendMode*	m_BlendMode;
	const PolygonMode*	m_PolygonMode;
	const StencilMode*	m_StencilMode;
	const DepthMode*	m_DepthMode;
	const Shader*		m_Shader;
	const Material*		m_Material;
	const Texture*		m_Texture[Texture::MaxTextures];
	Matrix4<float>		m_ModelView;
	Bounds3<float>		m_Bounds;
	float				m_Key;
};

}; };

#endif
