/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_DEFERQUEUE_H
#define __GRAPHICS3D_DEFERQUEUE_H

#include "math.matrix.h"
#include "container.allocator.h"
#include "container.array.h"
#include "container.pair.h"
#include "graphics3d.deferedmesh.h"

namespace magnum { namespace graphics3d {
using magnum::container::Allocator;
using magnum::container::Array;
using magnum::container::Pair;
using magnum::object::Noncopyable;
using magnum::math::Matrix;

/*--------------------------------------------------------------------------*/
/**
 * This class implements a queue for defered rendering as needed for
 * transparent objects
 */
class DeferQueue : public Object, private Noncopyable {
DeclareClass(DeferQueue, Object);
public:
	DeferQueue(Context* );
	
	inline void					insert(const Mesh* ,const Matrix<float,4>& );
	void						clear();
	void						render();
	
private:
	typedef Pair<DeferedMesh*,float> MeshPtr;
	Context*					m_Context;
	Allocator<DeferedMesh, 64>	m_Allocator;
	Array<MeshPtr>				m_Meshes;
};

}; };

#endif
