/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_DEPTHMODE_H
#define __GRAPHICS3D_DEPTHMODE_H

#include "system.assembler.h"
#include "math.range.h"
#include "graphics3d.opengl.h"
#include "graphics3d.resource.h"

namespace magnum { namespace graphics3d {
using magnum::system::Assembler;
using magnum::math::Range;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class DepthMode : public Resource {
DeclareClass(DepthMode, Resource);
public:
	enum TestFunction {
		Never = GL_NEVER,
		Less = GL_LESS,
		Equal = GL_EQUAL,
		LessEqual = GL_LEQUAL,
		Greater = GL_GREATER,
		GreaterEqual = GL_GEQUAL,
		NotEqual = GL_NOTEQUAL,
		Always = GL_ALWAYS,
		Disable = -1,
	};

public:
	DepthMode();
	~DepthMode();

	inline void			select() const;
	inline void			deselect() const;
	inline void			forceSelect() const;
	static inline void	selectNone();
	static inline void	selectDefault();
	static inline DepthMode* getDefault();

	void				compileSelect(Assembler& ) const;
	void				compileDeselect(Assembler& ) const;

	// Depth Buffer
	inline void			setRange(float , float);
	inline void			setRange(const Range<float>& );
	inline Range<float>	getRange() const;
	inline void			setFunction(TestFunction  );
	inline TestFunction	getFunction() const;
	inline void			setWrite(bool );
	inline bool			getWrite() const;

protected:
	void				onSelect() const;
	void				onDeselect() const;
	bool				onCompileSelect(Assembler& ) const;

private:	
	Range<float>		m_Range;
	TestFunction		m_Function;
	bool				m_Write;
};

}; };

#endif
