/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_DEPTHNMODE_INL
#define __GRAPHICS3D_DEPTHNMODE_INL

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DepthMode* DepthMode::getDefault() {
	return &getContext()->m_DefaultDepthMode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DepthMode::select() const {
	Context* context = getContext();
	if (context->m_DepthMode == this)
		return;
	context->m_DepthMode = this;
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	DepthMode::forceSelect() const {
	Context* context = getContext();
	context->m_DepthMode = this;
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	DepthMode::deselect() const {
	Context* context = getContext();
	if (context->m_DepthMode == this)
		context->m_DepthMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	DepthMode::selectNone() {
	Context* context = getContext();
	context->m_DepthMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	DepthMode::selectDefault() {
	Context* context = getContext();
	context->m_DepthMode = &context->m_DefaultDepthMode;
	context->m_DepthMode->onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DepthMode::setRange(const Range<float>& r) {
	m_Range = r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DepthMode::setRange(float min, float max) {
	m_Range.setStart(min);
	m_Range.setEnd(max);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Range<float> DepthMode::getRange() const {
	return m_Range;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	DepthMode::setFunction(TestFunction  t) {
	m_Function = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DepthMode::TestFunction DepthMode::getFunction() const {
	return m_Function;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	DepthMode::setWrite(bool b) {
	m_Write = b;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool	DepthMode::getWrite() const {
	return m_Write;
}


}; };

#endif
