/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_FRAMEBUFFER_H
#define __GRAPHICS3D_FRAMEBUFFER_H

#include "graphics3d.rendertarget.h"
#include "graphics.colour.h"
#include "graphics.rectangle.h"
#include "graphics.dimension.h"
#include "graphics.point.h"
#include "graphics.image.h"
#include "container.map.h"


namespace magnum { namespace graphics3d {
using magnum::graphics::Colour;
using magnum::graphics::Rectangle;
using magnum::graphics::Dimension;
using magnum::graphics::Point;
using magnum::graphics::Image;
using magnum::container::Map;

class Context;
class Texture1D;
class Texture2D;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class FrameBuffer : public Resource {
DeclareClass(FrameBuffer, Resource);
friend Context;
friend RenderTarget;
public:
	class Binding {
	friend FrameBuffer;
	public:
		inline Binding();
		inline Binding(RenderTarget* ,RenderTarget::Attachment );
		inline RenderTarget*	getTarget() const;
		inline RenderTarget::Attachment getAttachment() const;

	private:
		RenderTarget*			m_Target;
		RenderTarget::Attachment m_Attachment;
	};

	typedef magnum::container::Array<Binding>::Iterator Iterator;

private:
	FrameBuffer(GLuint );

public:
	~FrameBuffer();

	static bool				isSupported();

	static FrameBuffer*		create();

	inline Dimension<int>	getSize() const;
	inline unsigned int		getWidth() const;
	inline unsigned int		getHeight() const;

	// Validation
	bool					isValid() const;

	// RenderTarget stuff
	void					attach(RenderTarget* ,RenderTarget::Attachment );
	void					detach(RenderTarget* );
	void					detachAll();
	inline Iterator			begin() const;
	inline Iterator			end() const;
	
	// Select / Deselect
	void					forceSelect() const;
	void					select() const;
	void					deselect() const;
	static void				selectDefault();
	static inline FrameBuffer* getDefault();

private:
	Array<Binding>			m_Bindings;
	Dimension<int>			m_Size;
	Rectangle<int>			m_Viewport;
	GLuint					m_OglFramebuffer;
};

}; };

#endif
