/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_FRAMEMODEL_H
#define __GRAPHICS3D_FRAMEMODEL_H

#include "object.pointer.h"
#include "container.list.h"
#include "graphics3d.model.h"
#include "graphics3d.mesh.h"
#include "graphics3d.animation.h"


namespace magnum { namespace graphics3d {
using magnum::container::Array;
using magnum::container::List;
using magnum::object::Pointer;

class RenderStream;
class VertexBuffer;
class MeshGroup;
class FrameAnimation;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class FrameModel : public Model {
DeclareClass(FrameModel, Model);
public:
	FrameModel();
	~FrameModel();

	void					insertGroup(MeshGroup* );
	void					removeGroup(MeshGroup* );				

	void					insertFrame(MeshGroup* ,VertexBuffer* ,unsigned int frame );
	void					removeFrame(MeshGroup* ,unsigned int );
	void					removeFrame(MeshGroup* ,VertexBuffer* );
	
	inline void				insertAnimation(FrameAnimation* );
	inline void				removeAnimation(FrameAnimation* );

	void					finish();

	Array<MeshGroup*>		getGroups() const;
	Array<VertexBuffer*>	getFrames(const MeshGroup* ) const;

protected:
	virtual void			onRender(const ParameterBuffer* ) const;
	virtual bool			onOptimize();
	virtual void			onClear();
	virtual bool			onUpdateBounds(bool = true);

private:
	struct Group {
		Array<Pointer<VertexBuffer> > m_Frames;
		Pointer<MeshGroup>		m_MeshGroup;
		Pointer<VertexBuffer>	m_Vertices;
	};

	void					interpolate(Group* ,unsigned int ,unsigned int ,float ) const;

	List<Group>				m_Groups;
};

}; };

#endif
