/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_GEOMETRYRENDERER_H
#define __GRAPHICS3D_GEOMETRYRENDERER_H

#include "math.matrix.h"
#include "math.vector2.h"
#include "geometry.line.h"
#include "geometry.ray.h"
#include "geometry.segment.h"
#include "geometry.sphere.h"
#include "geometry.cube.h"
#include "geometry.alignedbox.h"
#include "geometry.box.h"
#include "geometry.frustum.h"
#include "graphics3d.resource.h"


namespace magnum { namespace graphics3d {
using magnum::math::Matrix;
using magnum::math::Vector2;
using magnum::geometry::Line;
using magnum::geometry::Ray;
using magnum::geometry::Segment;
using magnum::geometry::Sphere;
using magnum::geometry::Cube;
using magnum::geometry::AlignedBox;
using magnum::geometry::Box;
using magnum::geometry::Frustum3;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class GeometryRenderer : public Object, private magnum::object::Noncopyable {
DeclareClass(GeometryRenderer, Object);
public:
	static void		renderAxes(const Matrix<float,4>& );
	static void		renderAxes(const Matrix<float,4>& ,const Vector2<float>& );
	static void		renderAxes(const Matrix<float,3>& ,const Vector2<float>& );

	static void		render(const Line<float, 3>& );
	static void		render(const Segment<float, 3>& );
	static void		render(const Ray<float, 3>& );

	static void		render(const Sphere<float, 3>& ,unsigned int tes=20);
	static void		render(const Cube<float, 3>& );
	static void		render(const AlignedBox<float, 3>& );
	static void		render(const Box<float, 3>& );
	static void		render(const Frustum3<float>& );
};

}; };

#endif
