/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_INDEXMESH_H
#define __GRAPHICS3D_INDEXMESH_H

#include "object.pointer.h"
#include "container.map.h"
#include "geometry.meshcollider.h"
#include "graphics3d.vertexmesh.h"


namespace magnum { namespace graphics3d {
using magnum::object::Pointer;
using magnum::container::Map;
using magnum::geometry::MeshCollider3;

class IndexBuffer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class IndexMesh : public VertexMesh {
DeclareClass(IndexMesh, VertexMesh);
public:
	inline IndexBuffer*		getIndexBuffer() const;
	inline void				attachIndexBuffer(IndexBuffer* );
	inline void				detachIndexBuffer();

protected:
	inline void				selectIndexBuffer() const;

	virtual void			onRender() const;
	virtual TriangleMesh3<float>* onTriangulate() const;
	virtual bool			onUpdateBounds();
	virtual bool			onUpdateCollider();

private:
	template<class T> 
	static void				triangulateTriangleStrip(TriangleMesh3<float>* ,const T* ,unsigned int );
	template<class T> 
	static void				triangulateTriangleList(TriangleMesh3<float>* ,const T* ,unsigned int );
	template<class T> 
	static void				triangulateTriangleFan(TriangleMesh3<float>* ,const T* ,unsigned int );
	template<class T> 
	static void				triangulateQuadList(TriangleMesh3<float>* ,const T* ,unsigned int );

	template<class T> 
	bool					copyIndices(MeshCollider3<float>* ,Map<unsigned int,unsigned int>& ) const;
	template<class T> 
	bool					copyVertices(MeshCollider3<float>* ,Map<unsigned int,unsigned int>& ) const;
	bool					hasRange() const;

private:
	Pointer<IndexBuffer>	m_IndexBuffer;
};

}; };

#endif
