/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_KEYFRAME_H
#define __GRAPHICS3D_KEYFRAME_H

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class KeyframeBase {
public:
	inline KeyframeBase();
	inline explicit KeyframeBase(float );
	
	inline float		getTime() const;
	inline void			setTime(float );

	inline float		getDuration() const;
	inline void			setDuration(float );

	inline bool			isBefore(const KeyframeBase& ) const;
	inline bool			isAfter(const KeyframeBase& ) const;

	inline bool			operator==(const KeyframeBase& ) const;
	inline bool			operator!=(const KeyframeBase& ) const;
	inline bool			operator<=(const KeyframeBase& ) const;
	inline bool			operator>=(const KeyframeBase& ) const;
	inline bool			operator<(const KeyframeBase& ) const;
	inline bool			operator>(const KeyframeBase& ) const;

private:
	float				m_Time;
	float				m_Duration;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size = 1> class Keyframe : public KeyframeBase {
public:
	inline Keyframe();
	inline explicit Keyframe(float );
	inline Keyframe(float ,const T* );
	
	inline const T&		getKey(unsigned int ) const;
	inline void			setKey(unsigned int ,const T& );

	inline const T*		getKeys() const;
	inline void			setKeys(const T* );
	
	inline const T&		operator[](unsigned int ) const;
	inline T&			operator[](unsigned int );

	bool				operator==(const Keyframe& ) const;
	bool				operator!=(const Keyframe& ) const;

private:
	T					m_Keys[size];
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Keyframe<T,1> : public KeyframeBase {
public:
	inline Keyframe();
	inline Keyframe(float );
	inline Keyframe(float ,const T& );
	
	inline const T&		getKey() const;
	inline void			setKey(const T& );

	inline const T&		operator[](unsigned int ) const;
	inline T&			operator[](unsigned int );

	inline bool			operator==(const Keyframe& ) const;
	inline bool			operator!=(const Keyframe& ) const;
	
private:
	T					m_Key;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Keyframe<T,0> : public KeyframeBase {
public:
	inline Keyframe();
	inline explicit Keyframe(float );
	Keyframe(const Keyframe& );
	explicit Keyframe(unsigned int );
	Keyframe(float ,unsigned int );
	Keyframe(float ,unsigned int ,const T* );
	~Keyframe();
	
	inline unsigned int	getSize() const;
	void				setSize(unsigned int );

	inline const T&		getKey(unsigned int ) const;
	inline void			setKey(unsigned int ,const T& );

	inline const T*		getKeys() const;
	void				setKeys(const T* );
	
	inline const T&		operator[](unsigned int ) const;
	inline T&			operator[](unsigned int );

	bool				operator==(const Keyframe& ) const;
	bool				operator!=(const Keyframe& ) const;
	
	const Keyframe&		operator=(const Keyframe& );
	
private:
	unsigned int		m_Size;
	T*					m_Keys;
};


}; };

#endif
