/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_KEYFRAME_INL
#define __GRAPHICS3D_KEYFRAME_INL

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
Keyframe Base
----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline KeyframeBase::KeyframeBase() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline KeyframeBase::KeyframeBase(float t) : m_Time(t) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float KeyframeBase::getTime() const {
	return m_Time;
}	


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void KeyframeBase::setTime(float t) {
	m_Time = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float KeyframeBase::getDuration() const {
	return m_Duration;
}	


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void KeyframeBase::setDuration(float t) {
	m_Duration = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool KeyframeBase::isBefore(const KeyframeBase& k) const {
	return (m_Time < k.m_Time);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool KeyframeBase::isAfter(const KeyframeBase& k) const {
	return (m_Time > k.m_Time);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool KeyframeBase::operator==(const KeyframeBase& k) const {
	return (m_Time == k.m_Time);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool KeyframeBase::operator!=(const KeyframeBase& k) const {
	return (m_Time != k.m_Time);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool KeyframeBase::operator<=(const KeyframeBase& k) const {
	return (m_Time <= k.m_Time);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool KeyframeBase::operator>=(const KeyframeBase& k) const {
	return (m_Time >= k.m_Time);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool KeyframeBase::operator<(const KeyframeBase& k) const {
	return (m_Time < k.m_Time);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool KeyframeBase::operator>(const KeyframeBase& k) const {
	return (m_Time > k.m_Time);
}



/*----------------------------------------------------------------------------
Keyframe Specialisation for one element
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Keyframe<T,1>::Keyframe() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Keyframe<T,1>::Keyframe(float t) : KeyframeBase(t) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Keyframe<T,1>::Keyframe(float t, const T& d) : KeyframeBase(t), m_Key(d) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const T& Keyframe<T,1>::getKey() const {
	return m_Key;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void Keyframe<T,1>::setKey(const T& k) {
	m_Key = k;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const T& Keyframe<T,1>::operator[](unsigned int i) const {
	ASSERT(i == 0);
	return m_Key;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline T& Keyframe<T,1>::operator[](unsigned int i) {
	ASSERT(i == 0);
	return m_Key;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline bool Keyframe<T,1>::operator==(const Keyframe<T,1>& k) const {
	return ((*(KeyframeBase*)this == k) && (m_Key == k.m_Key)) ;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline bool Keyframe<T,1>::operator!=(const Keyframe<T,1>& k) const {
	return ((*(KeyframeBase*)this != k) || (m_Key != k.m_Key)) ;
}



/*----------------------------------------------------------------------------
Keyframe Specialisation for fixed number of elements
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size>
inline Keyframe<T,size>::Keyframe() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size>
inline Keyframe<T,size>::Keyframe(float t) : KeyframeBase(t) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size>
inline Keyframe<T,size>::Keyframe(float t, const T* d) : KeyframeBase(t) {
	for (unsigned int i = 0; i < size; i++)
		m_Keys[i] = d[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size>
inline const T& Keyframe<T,size>::getKey(unsigned int i) const {
	ASSERT(i < size);
	return m_Keys[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size>
inline void Keyframe<T,size>::setKey(unsigned int i,const T& k) {
	ASSERT(i < size);
	m_Keys[i] = k;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size>
inline const T* Keyframe<T,size>::getKeys() const {
	return m_Keys;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size>
inline void Keyframe<T,size>::setKeys(const T* k) {
	for (unsigned int i = 0; i < size; i++)
		m_Keys[i] = k[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size>
inline const T& Keyframe<T,size>::operator[](unsigned int i) const {
	ASSERT(i < size);
	return m_Keys[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size>
inline T& Keyframe<T,size>::operator[](unsigned int i) {
	ASSERT(i < size);
	return m_Keys[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size>
bool Keyframe<T,size>::operator==(const Keyframe<T,size>& k) const {
	return ((*(KeyframeBase*)this == k) && (!memcmp(m_Keys, k.m_Keys, sizeof(m_Keys)))) ;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,unsigned int size>
bool Keyframe<T,size>::operator!=(const Keyframe<T,size>& k) const {
	return ((*(KeyframeBase*)this != k) || (memcmp(m_Keys, k.m_Keys, sizeof(m_Keys))));
}



/*----------------------------------------------------------------------------
Keyframe Specialisation for dynamic number of elements
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Keyframe<T,0>::Keyframe() {
	m_Keys = NULL;
	m_Size = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline Keyframe<T,0>::Keyframe(float t) : KeyframeBase(t) {
	m_Keys = NULL;
	m_Size = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Keyframe<T,0>::Keyframe(const Keyframe<T,0>& kf) : KeyframeBase(kf) {
	m_Size = kf.m_Size;
	m_Keys = new T[m_Size];
	for (unsigned int i = 0; i < m_Size; i++)
		m_Keys[i] = kf.m_Keys[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Keyframe<T,0>::Keyframe(unsigned int s) {
	m_Size = s;
	m_Keys = new T[s];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Keyframe<T,0>::Keyframe(float t,unsigned int s) : KeyframeBase(t) {
	m_Size = s;
	m_Keys = new T[s];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Keyframe<T,0>::Keyframe(float t, unsigned int s, const T* d) : KeyframeBase(t) {
	m_Size = s;
	m_Keys = new T[s];
	for (unsigned int i = 0; i < size; i++)
		m_Keys[i] = d[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Keyframe<T,0>::~Keyframe() {
	if (m_Keys != NULL)
		delete[] m_Keys;
	m_Keys = NULL;
	m_Size = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline unsigned int	Keyframe<T,0>::getSize() const {
	return m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Keyframe<T,0>::setSize(unsigned int size) {
	// The easiest special case - nothing to do
	if (size == m_Size)
		return;
		
	// New size is zero - this is also an easy one	
	if (size == 0) {
		delete[] m_Keys;
		m_Keys = NULL;
		m_Size = 0;
		return;
	}
	
	// Old size is zero?
	if (m_Size == 0) {
		ASSERT(m_Keys == NULL);
		m_Keys = new T[size];
		m_Size = size;
		return;
	}
	
	// Different sizes
	ASSERT(m_Keys != NULL);
	T* newkeys = new T[size];
	unsigned int count = min(size, m_Size);
	for (unsigned int i = 0; i < count; i++)
		newkeys[i] = m_Keys[i];
	delete[] m_Keys;
	m_Keys = newkeys;
	m_Size = size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const T& Keyframe<T,0>::getKey(unsigned int i) const {
	ASSERT(i < m_Size);
	return m_Keys[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void Keyframe<T,0>::setKey(unsigned int i,const T& k) {
	ASSERT(i < m_Size);
	m_Keys[i] = k;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const T* Keyframe<T,0>::getKeys() const {
	return m_Keys;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Keyframe<T,0>::setKeys(const T* k) {
	for (unsigned int i = 0; i < m_Size; i++)
		m_Keys[i] = k[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const T& Keyframe<T,0>::operator[](unsigned int i) const {
	ASSERT(i < m_Size);
	return m_Keys[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline T& Keyframe<T,0>::operator[](unsigned int i) {
	ASSERT(i < m_Size);
	return m_Keys[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
bool Keyframe<T,0>::operator==(const Keyframe<T,0>& k) const {
	return ((*(KeyframeBase*)this == k) && (!memcmp(m_Keys, k.m_Keys, m_Size*sizeof(T)))) ;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
bool Keyframe<T,0>::operator!=(const Keyframe<T,0>& k) const {
	return ((*(KeyframeBase*)this != k) || (memcmp(m_Keys, k.m_Keys, m_Size*sizeof(T))));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
const Keyframe<T,0>& Keyframe<T,0>::operator=(const Keyframe<T,0>& kf) {
	KeyframeBase::operator=(kf);
	if (m_Size != kf.m_Size) {
		if (m_Keys != NULL)
			delete[] m_Keys;
		m_Size = kf.m_Size;
		m_Keys = new T[m_Size];
	}
	
	for (unsigned int i = 0; i < m_Size; i++)
		m_Keys[i] = kf.m_Keys[i];
		
	return *this;
}

}; };

#endif
