/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_LOCATION_H
#define __GRAPHICS3D_LOCATION_H

#include "graphics3d.types.h"
#include "container.array.h"

namespace magnum { namespace graphics3d {
using magnum::container::Array;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class L> class LocationArray : public Object {
public:
	class Location {
	friend LocationArray;
	private:
		char			m_Name[64];
		char			m_Description[64];
		L				m_Location;
		Types::Type		m_Type;
						
	public:				
		const char*		getName() const;
		const char*		getDescription() const;
		Types::Type		getType() const;
		L		getLocation() const;
	};
	
	typedef magnum::container::Array<Location>::Iterator	Iterator;
	
private:
	Array<Location>			m_Locations;
	unsigned int			m_Length;
	
public:
	inline LocationArray();

	inline void				clear();
	void					add(const char* ,const char* ,Types::Type ,L );
	void					remove(const char* );
	void					shrink();
	bool					contains(const char* ) const;
	bool					contains(L ) const;
	void					append(const LocationArray& );
							
	inline unsigned int		getSize() const;
	inline unsigned int		getLength() const;
							
	inline const char*		getDescription(const char* ) const;
	inline Types::Type		getType(const char* ) const;
	inline unsigned int		getSize(const char* ) const;
	inline L				getLocation(const char* ) const;
							
	const Location& 		at(const char* name) const;
	const Location& 		at(L ) const;
	const Location& 		operator[](const char* name) const;
	const Location& 		operator[](L ) const;
							
	inline Iterator 		begin() const;
	inline Iterator 		end() const;
	
	inline const LocationArray& operator=(const LocationArray&);
	inline bool				operator==(const LocationArray& ) const;
	inline bool				operator!=(const LocationArray& ) const;
};

}; };

#endif
