/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_MATRIXSTACK_H
#define __GRAPHICS3D_MATRIXSTACK_H

#include "object.h"
#include "math.matrix.h"
#include "math.vector3.h"
#include "graphics.rectangle.h"
#include "graphics3d.opengl.h"

namespace magnum {namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::graphics::Rectangle;
using magnum::math::Matrix;
using magnum::math::Matrix4;
using magnum::math::Vector3;

class Context;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class MatrixStack : public Object, private magnum::object::Noncopyable {
DeclareClass(MatrixStack, Object);
friend Context;
public:
	enum Mode {
		Projection = GL_PROJECTION,
		ModelView = GL_MODELVIEW,
		Texture = GL_TEXTURE0,
		Texture0 = GL_TEXTURE0,
		Texture1 = GL_TEXTURE1,
		Texture2 = GL_TEXTURE2,
		Texture3 = GL_TEXTURE3,
		Texture4 = GL_TEXTURE4,
		Texture5 = GL_TEXTURE5,
		Texture6 = GL_TEXTURE6,
		Texture7 = GL_TEXTURE7,
	};

private:
	Mode			m_Mode;

private:
	inline MatrixStack();

public:
	inline void		setProjection(const Matrix<float,4>& );
	inline Matrix4<float> getProjection() const;
	inline void		identityProjection();
	inline void		orthoProjection(float ,float, float ,float ,float ,float );
	inline void		orthoProjection(const Rectangle<float>& ,float ,float );
	inline void		pushProjection();
	inline void		popProjection();

	inline void		setModel(const Matrix<float,4>& );
	inline Matrix4<float> getModel() const;
	inline void		pushModel();
	inline void		popModel();
	inline void		multModel(const Matrix<float,4>& );
	inline void		rotateModel(float , const Vector3<float>& );
	inline void		translateModel(const Vector3<float>& );
	inline void		scaleModel(const Vector3<float>& );
	inline void		identityModel();
					
	inline void		setTexture(const Matrix<float,4>& );
	inline void		pushTexture();
	inline void		popTexture();
	inline void		multTexture(const Matrix<float,4>& );
	inline void		rotateTexture(float , const Vector3<float>& );
	inline void		translateTexture(const Matrix<float,4>& );
	inline void		scaleTexture(const Matrix<float,4>& );
	inline void		identityTexture();
					
	inline void		selectMode(Mode );
	void			forceSelectMode(Mode );
	inline Mode		getMode() const;

	inline void		set(const Matrix<float,4>& );
	inline Matrix4<float> get() const;
	inline void		push();
	inline void		pop();
	inline void		mult(const Matrix<float,4>& );
	inline void		translate(const Vector3<float>& );
	inline void		scale(const Vector3<float>& );
	inline void		rotate(float , const Vector3<float>& );
	inline void		identity();
	inline void		ortho(float ,float, float ,float ,float ,float );
	inline void		ortho(const Rectangle<float>& ,float ,float );
};


}; };

#endif
