/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_MESH_H
#define __GRAPHICS3D_MESH_H

#include "object.h"
#include "object.pointer.h"
#include "container.array.h"
#include "geometry.trianglemesh.h"
#include "geometry.bounds.h"
#include "geometry.collider.h"
#include "graphics3d.opengl.h"
#include "graphics3d.resource.h"


namespace magnum { namespace graphics3d {
using magnum::container::Array;
using magnum::geometry::Collider;
using magnum::geometry::TriangleMesh;
using magnum::geometry::TriangleMesh3;
using magnum::geometry::Bounds3;
using magnum::object::Pointer;

class Appearance;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Mesh : public Resource {
DeclareClass(Mesh, Resource);
public:
	enum Primitive {
		Unknown = 0,
		PointList = GL_POINTS,
		LineStrip = GL_LINE_STRIP,
		LineLoop = GL_LINE_LOOP,
		LineList = GL_LINES,
		TriangleList = GL_TRIANGLES,
		TriangleStrip = GL_TRIANGLE_STRIP,
		TriangleFan = GL_TRIANGLE_FAN,
		QuadStrip = GL_QUAD_STRIP,
		QuadList = GL_QUADS,
		Polygon = GL_POLYGON,
	};

protected:
	Mesh();

public:
	inline void				render() const;
	inline TriangleMesh3<float>* triangulate() const;
	
	inline void				setBounds(const Bounds3<float>& );
	inline const Bounds3<float>& getBounds() const;
	inline bool				updateBounds();
	
	inline void				attachCollider(Collider<float,3>* );
	inline void				detachCollider();
	inline Collider<float,3>* getCollider() const;
	inline bool				updateCollider();

	inline Appearance*		getAppearance() const;
	inline void				attachAppearance(Appearance* );
	inline void				detachAppearance();

	inline Primitive		getPrimitive() const;
	inline void				setPrimitive(Primitive p);

protected:
	virtual void			onRender() const = 0;
	virtual TriangleMesh3<float>* onTriangulate() const = 0;
	virtual bool			onUpdateBounds();
	virtual bool			onUpdateCollider();

protected:
	static const GLenum		g_OglType[5];

private:
	Pointer<Collider<float,3> > m_Collider;
	Pointer<Appearance>		m_Appearance;
	Primitive				m_Primitive;
	Bounds3<float>			m_Bounds;
};

typedef Array<Mesh *>		MeshList;

}; };

#endif
