/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_MESH_INL
#define __GRAPHICS3D_MESH_INL

#include "graphics3d.appearance.h"

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Mesh::render() const {
	onRender();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline TriangleMesh3<float>* Mesh::triangulate() const {
	return onTriangulate();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mesh::setBounds(const Bounds3<float>& b) {
	m_Bounds = b;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Bounds3<float>& Mesh::getBounds() const {
	return m_Bounds;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Mesh::updateBounds() {
	return onUpdateBounds();
}


/*--------------------------------------------------------------------------*/
/**
 */
inline void Mesh::attachCollider(Collider<float,3>* c) {
	ASSERT(c != NULL);
	m_Collider = c;
}


/*--------------------------------------------------------------------------*/
/**
 */
inline void Mesh::detachCollider() {
	m_Collider.setNull();
}


/*--------------------------------------------------------------------------*/
/**
 */
inline Collider<float,3>* Mesh::getCollider() const {
	return m_Collider;
}


/*--------------------------------------------------------------------------*/
/**
 */
inline bool Mesh::updateCollider() {
	m_Collider.setNull();
	return onUpdateCollider();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Appearance* Mesh::getAppearance() const {
	return m_Appearance;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mesh::attachAppearance(Appearance* app) {
	ASSERT(app != NULL);
	m_Appearance = app;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mesh::detachAppearance() {
	m_Appearance = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Mesh::Primitive Mesh::getPrimitive() const {
	return m_Primitive;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Mesh::setPrimitive(Mesh::Primitive p) {
	m_Primitive = p;
}

}; };

#endif
