/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_MESHCONVERTER_H
#define __GRAPHICS3D_MESHCONVERTER_H

#include "io.filter.h"
#include "graphics3d.indexmesh.h"

namespace magnum {namespace graphics3d {
using magnum::io::Filter;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class MeshConverter : public Filter<Mesh> {
DeclareClass(MeshConverter, Filter<Mesh>);
public:
	MeshConverter(Mesh::Primitive = Mesh::TriangleList);

	inline void			setPrimitive(Mesh::Primitive );
	inline Mesh::Primitive getPrimitive() const;

protected:
	Mesh*				onProcess(const Mesh* );

private:
	// TL = TriangleList
	// TS = TriangleStrip
	// QL = QuadList
	// QS = QuadStrip
	template<class T>
	IndexMesh*			convert(const IndexMesh* );

	template<class T>
	IndexMesh*			convertTL2TS(const IndexMesh* );
	template<class T>
	IndexMesh*			convertTS2TL(const IndexMesh* );
	template<class T>
	IndexMesh*			convertQL2TS(const IndexMesh* );
	template<class T>
	IndexMesh*			convertQL2TL(const IndexMesh* );
	template<class T>
	IndexMesh*			convertQS2TS(const IndexMesh* );
	template<class T>
	IndexMesh*			convertQS2TL(const IndexMesh* );
	
	IndexMesh*			createIndexMesh(const VertexMesh* );
	
	IndexMesh*			cloneMesh(const IndexMesh* );

private:
	Mesh::Primitive		m_Primitive;
};

}; };

#endif


