/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_MESHSTRIPPER_H
#define __GRAPHICS3D_MESHSTRIPPER_H

#include "graph.graph.h"
#include "object.pointer.h"
#include "io.filter.h"
#include "container.array.h"
#include "container.tuple.h"
#include "container.pair.h"
#include "container.compound.h"
#include "container.priorityheap.h"
#include "algorithm.sort.h"
#include "algorithm.find.h"

namespace magnum { namespace graphics3d {
using namespace magnum::container;
using namespace magnum::math;
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::io::Filter;
using magnum::container::Array;
using magnum::container::Tuple;
using magnum::container::Pair;
using magnum::container::Compound;
using magnum::container::PriorityHeap;
using magnum::algorithm::sort;
using magnum::algorithm::find;
using magnum::algorithm::findfirst;
using magnum::graph::Graph;

class IndexMesh;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class TriangleStripper : public Object, private magnum::object::Noncopyable {
DeclareClass(TriangleStripper, Object);
public:
	void						clear();
	void						insert(const T* ,unsigned int numIndices);
	void						finish();

	Array<T>*					strip();						

	inline void					setCacheSize(unsigned int );
	inline unsigned int 		getCacheSize() const;

private:
	typedef Pair<T,T>			Edge;
	typedef Compound<Edge, int>	TriangleEdge;
	typedef Array<T>			Strip;
	typedef Pointer<Strip>		StripPointer;

	struct Triangle {
		T						v0, v1, v2;
		PriorityHeap<int>::Iterator	m_Degree;
	};

	inline bool					findCache(unsigned int );
	inline void					clearCache();
	inline void					insertCache(unsigned int );

	void						connect(const TriangleEdge& edge);
	void						removeTriangle(unsigned int );
	T							getConnection(unsigned int tri, const Edge& edge) const;

	Array<T>*					findStrip();
	int							getStripValue(unsigned int first, const typename Graph<Triangle, Edge>::Edge& );
	unsigned int				countSingletons(unsigned int triangle) const;
	Array<T>*					buildStrip(unsigned int first, const typename Graph<Triangle, Edge>::Edge& );
	Array<T>*					buildSingleton(unsigned int first);

private:
	Graph<Triangle, Edge>		m_Mesh;
	Array<TriangleEdge>			m_Edges;
	Array<unsigned int>			m_MarkedTriangles;
	Array<Pair<int,int> >		m_RemovedTriangles;
	PriorityHeap<int>			m_Degrees;
	Array<StripPointer>			m_Strips;
	unsigned int				m_StripSize;
	Array<unsigned int>			m_Cache;
	unsigned int				m_CacheSize;
	T							m_Index[2];
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class MeshStripper : public Filter<Mesh> {
DeclareClass(MeshStripper, Filter<Mesh>);
public:

protected:
	Mesh*				onProcess(const Mesh* );
};

}; };

#endif
