/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_MODEL_H
#define __GRAPHICS3D_MODEL_H

#include "math.matrix.h"
#include "geometry.bounds.h"
#include "geometry.ray.h"
#include "graphics3d.joint.h"
#include "graphics3d.animation.h"
#include "graphics3d.parameterformat.h"
#include "graphics3d.mesh.h"

namespace magnum { namespace graphics3d {
using magnum::math::Matrix;
using magnum::math::Matrix4;
using magnum::geometry::Bounds3;
using magnum::geometry::Ray;

class ParameterBuffer;
class Intersection;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Model : public Resource {
DeclareClass(Model, Resource);
protected:				
	inline				Model();
						
public:					
	inline void			render(const ParameterBuffer* ) const;
	inline void			transform(const ParameterBuffer* ,Matrix4<float>* ,Joint::Handle ) const;
	inline void			transform(const ParameterBuffer* ,Matrix4<float>* ,const Joint& ) const;
	
	inline bool			intersect(const ParameterBuffer* ,const Ray<float,3>& ,Intersection* = NULL) const;

	inline bool			optimise();
	inline bool			isOptimised() const;
	
	inline void			clear();
	
	inline void			setBounds(const Bounds3<float>& );
	inline void			clearBounds();
	inline bool			updateBounds(bool deep=true);
	inline const Bounds3<float>& getBounds() const;
	
	inline bool			updateCollider(bool deep=true);

	inline const ParameterFormat& getParameters() const;
	inline const JointList& getJoints() const;
	inline const AnimationList& getAnimations() const;
	inline const MeshList& getMeshes() const;

protected:
	virtual void		onRender(const ParameterBuffer* ) const = 0;
	virtual void		onTransform(const ParameterBuffer* ,Matrix4<float>* ,Joint::Handle ) const;
	virtual bool		onIntersect(const ParameterBuffer* ,const Ray<float,3>& ,Intersection* ) const;
	virtual bool		onUpdateBounds(bool = true);
	virtual bool		onUpdateCollider(bool = true);
	virtual bool		onOptimise();
	virtual void		onClear();
	
	void				publishParameters(const ParameterFormat& );
	void				clearParameters();

	inline void			publishMesh(Mesh* );
	void				unpublishMesh(Mesh* );
	void				clearMeshes();

	inline void			publishAnimation(Animation* );
	void				unpublishAnimation(Animation* );
	void				clearAnimations();

	inline const Joint&	publishJoint(const char* ,Joint::Handle );
	void				unpublishJoint(const Joint& );
	void				unpublishJoint(const char* );
	void				unpublishJoint(Joint::Handle );
	inline void			clearJoints();

private:
	bool				m_Optimised;
	ParameterFormat		m_Parameters;
	ParameterFormat::Location m_ModelMatrix;
	JointList			m_Joints;
	MeshList			m_Meshes;
	AnimationList		m_Animations;
	Bounds3<float>		m_Bounds;
};

}; };

#endif
