/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_MODEL_INL
#define __GRAPHICS3D_MODEL_INL


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Model::Model() {
	m_Optimised = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::render(const ParameterBuffer* pbuf) const {
	ASSERT(pbuf != NULL);
	onRender(pbuf);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::transform(const ParameterBuffer* pbuf,Matrix4<float>* m, Joint::Handle hnd) const {
	ASSERT(pbuf != NULL);
	ASSERT(m != NULL);
	onTransform(pbuf, m, hnd);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::transform(const ParameterBuffer* pbuf,Matrix4<float>* m, const Joint& j) const {
	ASSERT(pbuf != NULL);
	ASSERT(m != NULL);
	onTransform(pbuf, m, j.getHandle());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Model::intersect(const ParameterBuffer* pbuf,const Ray<float,3>& ray,Intersection* is) const {
	ASSERT(pbuf != NULL);
	return onIntersect(pbuf, ray, is);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Model::isOptimised() const {
	return m_Optimised;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Model::optimise() {
	m_Optimised = onOptimise();
	return m_Optimised;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::clear() {
	onClear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const ParameterFormat& Model::getParameters() const {
	return m_Parameters;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const AnimationList& Model::getAnimations() const {
	return m_Animations;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::publishAnimation(Animation* m) {
	m->refer();
	m_Animations.insert(m);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const MeshList& Model::getMeshes() const {
	return m_Meshes;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::publishMesh(Mesh* m) {
	m->refer();
	m_Meshes.insert(m);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const JointList& Model::getJoints() const {
	return m_Joints;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Joint&	Model::publishJoint(const char*n, Joint::Handle hnd) {
	return *m_Joints.insert(Joint(n,hnd));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Model::clearJoints() {
	m_Joints.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Bounds3<float>& Model::getBounds() const {
	return m_Bounds;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Model::setBounds(const Bounds3<float>& b) {
	m_Bounds = b;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Model::clearBounds() {
	m_Bounds.empty();
}


/*--------------------------------------------------------------------------*/
/** Updates the bounding volume information of this model. This volume
 * is needed for culling etc.
 * \param deep If using a "deep" update, all bounding volumes of all
 *			meshes will be recalculated, too.
 */
inline bool Model::updateBounds(bool deep) {
	return onUpdateBounds(deep);
}


/*--------------------------------------------------------------------------*/
/** Updates all collision information of this model. This information
 * is needed for intersection tests etc.
 * \param deep if using a "deep" update, all colliders of all meshes
 *			will be updated, too. This means that even a mesh already
 *			contains a collider, it will be recalculated.
 */
inline bool Model::updateCollider(bool deep) {
	return onUpdateCollider(deep);
}

}; };

#endif
