/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_OPTIMISER_INL
#define __GRAPHICS3D_OPTIMISER_INL

namespace magnum {namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimiser::setStrategy(Optimiser::Strategy s) {
	m_Strategy = s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Optimiser::Strategy Optimiser::getStrategy() const {
	return m_Strategy;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimiser::selectVertexBuffer(VertexBuffer* vb, unsigned int attribs) {
	ASSERT(vb != NULL);
	attribs &= vb->getAttributes();
	for (unsigned int i = 0; i < Attributes::MaxAttributes; i++) {
		if (attribs & (1 << i))
			m_CurrentVertexBuffers[i] = vb;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimiser::selectIndexBuffer(IndexBuffer* ib) {
	ASSERT(ib != NULL)
	m_CurrentIndexBuffer = ib;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::selectMaterial(Material* mat) {
	ASSERT(mat != NULL)
	m_CurrentMaterial = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::deselectMaterial() {
	m_CurrentMaterial = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::selectStencilMode(StencilMode* mat) {
	ASSERT(mat != NULL)
	m_CurrentStencilMode = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::deselectStencilMode() {
	m_CurrentStencilMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::selectBlendMode(BlendMode* mat) {
	ASSERT(mat != NULL)
	m_CurrentBlendMode = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::deselectBlendMode() {
	m_CurrentBlendMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::selectDepthMode(DepthMode* mat) {
	ASSERT(mat != NULL)
	m_CurrentDepthMode = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::deselectDepthMode() {
	m_CurrentDepthMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::selectPolygonMode(PolygonMode* mat) {
	ASSERT(mat != NULL)
	m_CurrentPolygonMode = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::deselectPolygonMode() {
	m_CurrentPolygonMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimiser::selectTexture(Texture* tex, unsigned int i) {
	ASSERT(tex != NULL);
	ASSERT(i < Texture::MaxTextures);
	m_CurrentTextures[i] = tex;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimiser::deselectTexture(unsigned int i) {
	ASSERT(i < Texture::MaxTextures);
	m_CurrentTextures[i] = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::selectShader(Shader* shader) {
	ASSERT(shader != NULL)
	m_CurrentShader = shader;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::deselectShader() {
	m_CurrentShader = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimiser::preselectVertexBuffer(VertexBuffer* vb, unsigned int attribs) {
	ASSERT(vb != NULL);
	attribs &= vb->getAttributes();
	for (unsigned int i = 0; i < Attributes::MaxAttributes; i++) {
		if (attribs & (1 << i))
			m_PreselectedVertexBuffers[i] = vb;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimiser::preselectIndexBuffer(IndexBuffer* ib) {
	ASSERT(ib != NULL)
	m_PreselectedIndexBuffer = ib;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::preselectMaterial(Material* mat) {
	ASSERT(mat != NULL)
	m_PreselectedMaterial = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::preselectStencilMode(StencilMode* mat) {
	ASSERT(mat != NULL)
	m_PreselectedStencilMode = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::preselectBlendMode(BlendMode* mat) {
	ASSERT(mat != NULL)
	m_PreselectedBlendMode = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::preselectPolygonMode(PolygonMode* mat) {
	ASSERT(mat != NULL)
	m_PreselectedPolygonMode = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::preselectDepthMode(DepthMode* mat) {
	ASSERT(mat != NULL)
	m_PreselectedDepthMode = mat;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Optimiser::preselectTexture(Texture* tex, unsigned int i) {
	ASSERT(tex != NULL);
	ASSERT(i < Texture::MaxTextures);
	m_PreselectedTextures[i] = tex;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Optimiser::preselectShader(Shader* shader) {
	ASSERT(shader != NULL)
	m_PreselectedShader = shader;
}


}; };

#endif
