/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_PARAMETERBUFFER_H
#define __GRAPHICS3D_PARAMETERBUFFER_H

#include "object.h"
#include "math.vector.h"
#include "math.vector2.h"
#include "math.vector3.h"
#include "math.vector4.h"
#include "math.matrix.h"
#include "graphics.colour.h"
#include "graphics3d.parameterformat.h"

namespace magnum {namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::types::byte;
using magnum::math::Vector;
using magnum::math::Vector2;
using magnum::math::Vector3;
using magnum::math::Vector4;
using magnum::math::Matrix;
using magnum::math::Matrix2;
using magnum::math::Matrix3;
using magnum::math::Matrix4;
using magnum::graphics::Colour;

class ParameterFormat;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ParameterBuffer : public Object, private magnum::object::Noncopyable {
DeclareClass(ParameterBuffer, Object);
private:
	const ParameterFormat*	m_Format;
	byte					m_Parameters[1];

private:	
	inline ParameterBuffer();

public:
	static ParameterBuffer* create(const ParameterFormat& );
	
	inline unsigned int		getSize() const;
	inline const ParameterFormat* getFormat() const;
	inline void*			getPointer(ParameterFormat::Location );
	inline const void*		getPointer(ParameterFormat::Location ) const;
	inline void*			getBuffer();
	inline const void*		getBuffer() const;

	inline void				setVector(ParameterFormat::Location l, const Vector3<float>& );
	inline void				setVector(ParameterFormat::Location l, const Vector4<float>& );
	inline void				setVector(ParameterFormat::Location l, const Vector3<double>& );
	inline void				setVector(ParameterFormat::Location l, const Vector4<double>& );
	inline void				setMatrix(ParameterFormat::Location l, const Matrix3<float>& );
	inline void				setMatrix(ParameterFormat::Location l, const Matrix4<float>& );
	inline void				setMatrix(ParameterFormat::Location l, const Matrix3<double>& );
	inline void				setMatrix(ParameterFormat::Location l, const Matrix4<double>& );
	inline void				setColour(ParameterFormat::Location l, const Colour<float>& );
	inline void				setColour(ParameterFormat::Location l, const Colour<byte>& );
	inline void				setInteger(ParameterFormat::Location l, int );
	inline int				getInteger(ParameterFormat::Location l) const;
	inline void				setFloat(ParameterFormat::Location l, float );
	inline float			getFloat(ParameterFormat::Location l) const;
	inline void				setDouble(ParameterFormat::Location l, double );
	inline double			getDouble(ParameterFormat::Location l) const;
};


}; };

#endif
