/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_PARAMETERBUFFER_INL
#define __GRAPHICS3D_PARAMETERBUFFER_INL

namespace magnum {namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline ParameterBuffer::ParameterBuffer() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int	ParameterBuffer::getSize() const {
	return m_Format->getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* ParameterBuffer::getPointer(ParameterFormat::Location l) {
	ASSERT(m_Format->contains(l));
	return (void*)((byte *)m_Parameters + l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const void* ParameterBuffer::getPointer(ParameterFormat::Location l) const {
	ASSERT(m_Format->contains(l));
	return (void*)((byte *)m_Parameters + l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* ParameterBuffer::getBuffer() {
	return m_Parameters;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const void* ParameterBuffer::getBuffer() const {
	return m_Parameters;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const ParameterFormat* ParameterBuffer::getFormat() const {
	return m_Format;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setInteger(ParameterFormat::Location l,int val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Integer));
	*(int *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int ParameterBuffer::getInteger(ParameterFormat::Location l) const {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Integer));
	return *(int *)((byte *)m_Parameters + l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setFloat(ParameterFormat::Location l,float val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Float));
	*(float *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float ParameterBuffer::getFloat(ParameterFormat::Location l) const {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Float));
	return *(float *)((byte *)m_Parameters + l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setDouble(ParameterFormat::Location l,double val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Double));
	*(double *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline double ParameterBuffer::getDouble(ParameterFormat::Location l) const {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Double));
	return *(double *)((byte *)m_Parameters + l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setVector(ParameterFormat::Location l,const Vector3<float>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Vector3f));
	*(Vector3<float> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setVector(ParameterFormat::Location l,const Vector4<float>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Vector4f));
	*(Vector4<float> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setVector(ParameterFormat::Location l,const Vector3<double>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Vector3d));
	*(Vector3<double> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setVector(ParameterFormat::Location l,const Vector4<double>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Vector4d));
	*(Vector4<double> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setMatrix(ParameterFormat::Location l,const Matrix3<float>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Matrix3f));
	*(Matrix3<float> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setMatrix(ParameterFormat::Location l,const Matrix4<float>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Matrix4f));
	*(Matrix4<float> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setMatrix(ParameterFormat::Location l,const Matrix3<double>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Matrix3d));
	*(Matrix3<double> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setMatrix(ParameterFormat::Location l,const Matrix4<double>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Matrix4d));
	*(Matrix4<double> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setColour(ParameterFormat::Location l,const Colour<float>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Colour4f));
	*(Colour<float> *)(void *)((byte *)m_Parameters + l) = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ParameterBuffer::setColour(ParameterFormat::Location l,const Colour<byte>& val) {
	ASSERT(m_Format->contains(l));
	ASSERT(m_Format->verifyType(l,Types::Colour4b));
	*(Colour<byte> *)(void *)((byte *)m_Parameters + l) = val;
}

}; };

#endif
