/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_PARAMETERFORMAT_H
#define __GRAPHICS3D_PARAMETERFORMAT_H

#include "graphics3d.types.h"
#include "container.array.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Array;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ParameterFormat : public Object {
public:
	typedef int				Location;

	class Component {
	friend ParameterFormat;
	private:
		char				m_Name[64];
		char				m_Description[64];
		Location			m_Location;
		Types::Type			m_Type;
						
	public:				
		inline const char*	getName() const;
		inline const char*	getDescription() const;
		inline Types::Type	getType() const;
		inline Location		getLocation() const;
	};
	
	typedef magnum::container::Array<Component>::Iterator Iterator;
	
public:
	inline ParameterFormat();

	inline void				clear();
	void					insert(const char* ,const char* ,Types::Type );
	void					insert(const char* ,Types::Type );
	void					remove(const char* );
	void					shrink();
	bool					contains(const char* ) const;
	bool					contains(Location ) const;
	void					append(const ParameterFormat& );
	
	bool					verifyType(Location ,Types::Type ) const;
							
	inline unsigned int		getSize() const;
	inline unsigned int		getLength() const;
							
	inline const char*		getDescription(const char* ) const;
	inline Types::Type		getType(const char* ) const;
	inline unsigned int		getSize(const char* ) const;
	inline Location			getLocation(const char* ) const;
							
	const Component& 		operator[](const char* name) const;
	const Component& 		operator[](Location ) const;
							
	inline Iterator 		begin() const;
	Iterator		 		at(const char* name) const;
	Iterator 				at(Location ) const;
	inline Iterator 		end() const;
	
	inline const ParameterFormat& operator=(const ParameterFormat&);
	inline bool				operator==(const ParameterFormat& ) const;
	inline bool				operator!=(const ParameterFormat& ) const;

private:
	Array<Component>		m_Components;
	unsigned int			m_Length;
};

}; };

#endif
