/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_POLYGONMODE_H
#define __GRAPHICS3D_POLYGONMODE_H

#include "system.assembler.h"
#include "graphics3d.opengl.h"
#include "graphics3d.resource.h"

namespace magnum { namespace graphics3d {
using magnum::system::Assembler;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class PolygonMode : public Resource {
DeclareClass(PolygonMode, Resource);
public:
	enum FillMode {
		Point		= GL_POINT,
		Line		= GL_LINE,
		Filled		= GL_FILL,
	};
	enum CullMode {
		CullFront = 1,
		CullBack = 2,
		CullNone = 3
	};

	
public:
	PolygonMode();
	~PolygonMode();

	inline void			select() const;
	inline void			deselect() const;
	inline void			forceSelect() const;
	static inline void	selectNone();
	static inline void	selectDefault();
	static inline PolygonMode* getDefault();

	void				compileSelect(Assembler& ) const;
	void				compileDeselect(Assembler& ) const;
	
	inline void			setMode(FillMode );
	inline void			setFrontMode(FillMode );
	inline void			setBackMode(FillMode );
	inline FillMode		getFrontMode() const;
	inline FillMode		getBackMode() const;
	
	inline void			setOffset(float );
	inline float		getOffset() const;
	inline void			setOffsetFactor(float );
	inline float		getOffsetFactor() const;
	
	inline void			setCulling(CullMode );
	inline CullMode		getCulling() const;

protected:
	void				onSelect() const;
	bool				onCompileSelect(Assembler& ) const;
	
private:
	FillMode			m_FrontMode;
	FillMode			m_BackMode;	
	CullMode			m_Culling;
	float				m_Offset;
	float				m_OffsetFactor;
};

}; };

#endif
