/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_POLYGONMODE_INL
#define __GRAPHICS3D_POLYGONMODE_INL

namespace magnum { namespace graphics3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline PolygonMode* PolygonMode::getDefault() {
	return &getContext()->m_DefaultPolygonMode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void PolygonMode::select() const {
	Context* context = getContext();
	if (context->m_PolygonMode == this)
		return;
	context->m_PolygonMode = this;
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	PolygonMode::forceSelect() const {
	Context* context = getContext();
	context->m_PolygonMode = this;
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	PolygonMode::deselect() const {
	Context* context = getContext();
	if (context->m_PolygonMode == this)
		context->m_PolygonMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	PolygonMode::selectNone() {
	Context* context = getContext();
	context->m_PolygonMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	PolygonMode::selectDefault() {
	Context* context = getContext();
	context->m_PolygonMode = &context->m_DefaultPolygonMode;
	context->m_PolygonMode->onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void PolygonMode::setMode(PolygonMode::FillMode m) {
	m_FrontMode = m_BackMode = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void PolygonMode::setFrontMode(PolygonMode::FillMode m) {
	m_FrontMode = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void PolygonMode::setBackMode(PolygonMode::FillMode m) {
	m_BackMode = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline PolygonMode::FillMode PolygonMode::getFrontMode() const {
	return m_FrontMode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline PolygonMode::FillMode PolygonMode::getBackMode() const {
	return m_BackMode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void PolygonMode::setOffset(float o) {
	m_Offset = o;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float PolygonMode::getOffset() const {
	return m_Offset;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void PolygonMode::setOffsetFactor(float f) {
	m_OffsetFactor = f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float PolygonMode::getOffsetFactor() const {
	return m_OffsetFactor;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void PolygonMode::setCulling(PolygonMode::CullMode m) {
	m_Culling = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline PolygonMode::CullMode PolygonMode::getCulling() const {
	return m_Culling;
}

}; };

#endif
