/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_PROGRAMSHADER_H
#define __GRAPHICS3D_PROGRAMSHADER_H

#include "graphics3d.shader.h"
#include "container.array.h"


namespace magnum { namespace graphics3d {
using magnum::object::Pointer;
using magnum::container::Array;

class ShaderProgram;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ProgrammableShader : public Shader {
DeclareClass(ProgrammableShader, Shader);
private:
	Array<Pointer<ShaderProgram> >	m_Programs;
	unsigned int			m_BaseAttributes;
	unsigned int			m_OglProgramHandle;
	
public:
	typedef Array<Pointer<ShaderProgram> >::Iterator Iterator;

protected:
	ProgrammableShader();
	~ProgrammableShader();
	
public:
	static ProgrammableShader* create();

    bool					attach(ShaderProgram* );
    bool					detach(ShaderProgram* );
    void					clear();
    
    inline bool				contains(ShaderProgram* ) const;
    inline Iterator			begin() const;
    inline Iterator			end() const;
    inline unsigned int		getSize() const;
    inline bool				isEmpty() const;
    
    bool					isValid() const;
    
    void					link();
    inline bool				isLinked() const;
    
    inline String			getLog() const;

protected:
	virtual void			onSelect() const;
	virtual void			onDeselect() const;
	
	virtual void			onCompileSelect(Assembler& ) const;
	virtual void			onCompileDeselect(Assembler& ) const;
	
	static unsigned int		mapAttribute(const char* );
};

}; };

#endif
