/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_RENDERTARGET_H
#define __GRAPHICS3D_RENDERTARGET_H

#include "object.pointer.h"
#include "graphics.dimension.h"
#include "graphics3d.resource.h"
#include "graphics3d.cubetexture.h"


namespace magnum { namespace graphics3d {
using magnum::graphics::Dimension;
using magnum::object::Pointer;

class FrameBuffer;
class Texture2D;
class CubeTexture;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class RenderTarget : public Resource {
DeclareClass(RenderTarget, Resource);
friend FrameBuffer;
public:
	enum Type {
		RGB8		= GL_RGB8,
		RGB16		= GL_RGB16,
		RGBA8		= GL_RGBA8,
		RGBA16		= GL_RGBA16,
		Depth		= GL_DEPTH_COMPONENT,
		Depth16		= GL_DEPTH_COMPONENT16,
		Depth24		= GL_DEPTH_COMPONENT24,
		Stencil		= GL_STENCIL_INDEX,
		Stencil1	= GL_STENCIL_INDEX1_EXT,
		Stencil4	= GL_STENCIL_INDEX4_EXT,
		Stencil8	= GL_STENCIL_INDEX8_EXT,
		Stencil16	= GL_STENCIL_INDEX16_EXT
	};
	
	enum Attachment {
		DepthBuffer			= GL_DEPTH_ATTACHMENT_EXT,
		StencilBuffer		= GL_STENCIL_ATTACHMENT_EXT,
		ColourBuffer		= GL_COLOR_ATTACHMENT0_EXT,
		ColourBuffer0		= GL_COLOR_ATTACHMENT0_EXT,
		ColourBuffer1		= GL_COLOR_ATTACHMENT1_EXT,
		ColourBuffer2		= GL_COLOR_ATTACHMENT2_EXT,
		ColourBuffer3		= GL_COLOR_ATTACHMENT3_EXT,
		ColourBuffer4		= GL_COLOR_ATTACHMENT4_EXT,
		ColourBuffer5		= GL_COLOR_ATTACHMENT5_EXT,
		ColourBuffer6		= GL_COLOR_ATTACHMENT6_EXT,
		ColourBuffer7		= GL_COLOR_ATTACHMENT7_EXT,
	};
	

private:
	Dimension<int>			m_Size;
	Type					m_Type;
	Pointer<Texture>		m_Texture;
	int						m_ZOffset;
	GLuint					m_OglRenderbuffer;
	GLuint					m_OglTexture;

protected:
	RenderTarget(GLuint );
	RenderTarget(Texture* ,unsigned int = 0);
	
public:
	~RenderTarget();
	
	static bool				isSupported();
	
	static RenderTarget*	create(Texture2D* );
	static RenderTarget*	create(CubeTexture* ,CubeTexture::Face );
	static RenderTarget*	create(Type ,const Dimension<int>& );
	static RenderTarget*	create(Type ,unsigned int ,unsigned int );

	inline Dimension<int>	getSize() const;
	inline unsigned int		getWidth() const;
	inline unsigned int		getHeight() const;
	
	inline Type				getType() const;
	
	inline Texture*			getTexture() const;
	
	inline bool				isBuffer() const;
	inline bool				isTexture() const;
	
protected:
	void					onAttach(FrameBuffer* ,Attachment );
	void					onDetach(FrameBuffer* ,Attachment );
	
private:
	static Type				mapType(Image::Type );
};

}; };

#endif
