/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_SIMPLEANIMATION_H
#define __GRAPHICS3D_SIMPLEANIMATION_H

#include "container.array.h"
#include "graphics3d.animation.h"
#include "graphics3d.keyframe.h"

namespace magnum { namespace graphics3d {
using magnum::container::Array;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class SimpleAnimation : public Animation {
DeclareClass(SimpleAnimation, Animation);
public:
	typedef typename Keyframe<T>				Frame;
	typedef typename Array<Frame>::Iterator	Iterator;
	
public:
	inline SimpleAnimation();
	inline explicit SimpleAnimation(const char* );
	
	inline void			insert(float ,const T& );
	inline void			insert(const Frame& );
	inline void			remove(const Iterator& );
	inline void			remove(const Frame& );
	void				remove(float );
	
	inline unsigned int getSize() const;
	inline const Frame& operator[](unsigned int ) const;
	inline Frame&		operator[](unsigned int );
	
	void				clear();
	void				finish();
	
	inline Iterator		begin() const;
	inline Iterator		at(unsigned int ) const;
	inline Iterator 	end() const;
	
protected:
	virtual void		onAnimate(float t,void* ) const;

private:	
	Array<Frame>		m_Frames;
};


typedef SimpleAnimation<Vector3<float> >	Vector3Animation;
typedef SimpleAnimation<Vector4<float> >	Vector4Animation;
typedef SimpleAnimation<Matrix3<float> >	Matrix3Animation;
typedef SimpleAnimation<Matrix4<float> >	Matrix4Animation;

}; };

#endif
