/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_SIMPLEANIMATION_INL
#define __GRAPHICS3D_SIMPLEANIMATION_INL

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline SimpleAnimation<T>::SimpleAnimation() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline SimpleAnimation<T>::SimpleAnimation(const char* n) : Animation(n) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void SimpleAnimation<T>::clear() {
	m_Frames.clear();
	setStart(0.0f);
	setEnd(0.0f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void SimpleAnimation<T>::insert(float t,const T& k) {
	m_Frames.insert(Frame(t,k));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void SimpleAnimation<T>::insert(const Frame& f) {
	m_Frames.insert(f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void SimpleAnimation<T>::remove(const Frame& f) {
	remove(f.getTime());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline void SimpleAnimation<T>::remove(const Iterator& i) {
	m_Frames.remove(i);
}




/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void SimpleAnimation<T>::remove(float f) {
	for (Iterator iter = m_Frames.begin(); iter != m_Frames.end(); ++iter) {
		if (iter->getTime() == f) {
			m_Frames.remove(iter);
			return;
		}
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void SimpleAnimation<T>::finish() {
	sortKeyframes(m_Frames);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline unsigned int SimpleAnimation<T>::getSize() const {
	return m_Frames.getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const typename SimpleAnimation<T>::Frame& SimpleAnimation<T>::operator[](unsigned int i) const {
	return m_Frames[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline typename SimpleAnimation<T>::Frame& SimpleAnimation<T>::operator[](unsigned int i) {
	return m_Frames[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline typename SimpleAnimation<T>::Iterator SimpleAnimation<T>::begin() const {
	return m_Frames.begin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline typename SimpleAnimation<T>::Iterator SimpleAnimation<T>::at(unsigned int f) const {
	return m_Frames.at(f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline typename SimpleAnimation<T>::Iterator SimpleAnimation<T>::end() const {
	return m_Frames.end();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void SimpleAnimation<T>::onAnimate(float t,void* data) const {
	// Normalise time
	t = normaliseTime(t);
	
	// Get keyframe
	const Frame* frame0 = getKeyframe(m_Frames, t);
	const Frame* frame1 = frame0+1;
	ASSERT(frame0 != NULL);
	ASSERT(t >= frame0->getTime());
	ASSERT(t - frame0->getTime() <= frame0->getDuration());
	
	// Get weight from keyframe
	t = 1.0f - (t - frame0->getTime()) / frame0->getDuration();
	if (t == 1.0f)
		*(T*)data = frame0->getKey();
	else
		*(T*)data = t*(frame0->getKey() - frame1->getKey()) + frame1->getKey();
}

}; };

#endif
