/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_SKELETON_H
#define __GRAPHICS3D_SKELETON_H

#include "container.list.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::object::Noncopyable;
using magnum::types::String;
using magnum::container::List;

class Bone;
class Skeleton;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Bone : public Object, private Noncopyable {
DeclareClass(Bone, Object);
friend class Skeleton;
public:
	Bone();

	inline void			setName(const char* );
	inline String		getName() const;
	
	inline void			setIndex(unsigned int );
	inline unsigned int	getIndex() const;

	inline void			setTorsoWeight(float );	
	inline float		getTorsoWeight() const;
	
	inline void			setDistance(float );
	inline float		getDistance() const;

	inline Bone*		getParent() const;
	inline Skeleton*	getSkeleton() const;
	
private:
	char				m_Name[64];	
	Skeleton*			m_Skeleton;
	Bone*				m_Parent;
	float				m_TorsoWeight;
	float				m_Distance;
	unsigned int		m_Index;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Skeleton : public Object, private Noncopyable {
DeclareClass(Skeleton, Object);
public:
	typedef List<Bone>::Iterator	Iterator;
	
public:
	inline Skeleton();
	inline Skeleton(const char* );
	
	inline void			setName(const char* );
	inline String		getName() const;

	Bone*				insert(const char* name, Bone* parent);
	Bone*				insert(const char* name);
	void				remove(Bone* );
	
	inline unsigned int	getSize() const;
	
	void				clear();

	inline Iterator		begin() const;
	Iterator			at(const char* ) const;
	inline Iterator		end() const;

private:
	void				removeChilds(Bone* );
	
private:
	List<Bone>			m_Bones;
	String				m_Name;
};

}; };

#endif
