/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_SKINMODEL_H
#define __GRAPHICS3D_SKINMODEL_H

#include "object.pointer.h"
#include "container.list.h"
#include "graphics3d.model.h"
#include "graphics3d.mesh.h"
#include "graphics3d.animation.h"


namespace magnum { namespace graphics3d {
using magnum::container::Array;
using magnum::container::List;
using magnum::object::Pointer;
using magnum::math::Matrix4;

class RenderStream;
class VertexBuffer;
class MeshGroup;
class FrameAnimation;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class SkinModel : public Model {
DeclareClass(SkinModel, Model);
public:
	typedef Keyframe<Matrix4<float>,0> Frame;
	
public:
	inline					SkinModel();

	void					insertGroup(MeshGroup* );
	void					removeGroup(MeshGroup* );				

	void					insertFrame(MeshGroup* ,Frame* ,unsigned int frame );
	void					removeFrame(MeshGroup* ,unsigned int );
	void					removeFrame(MeshGroup* ,Frame* );
	
	void					insertAnimation(FrameAnimation* );
	void					removeAnimation(FrameAnimation* );

	void					clear();
	void					finish();

	Array<MeshGroup*>		getGroups() const;
	MeshList				getMeshes() const;
	Array<Frame*>			getFrames(const MeshGroup* ) const;

protected:
	virtual void			onRender(const ParameterBuffer* ) const;
	virtual bool			onOptimize();

private:
	struct Group {
		Array<Pointer<VertexBuffer> > m_Frames;
		Pointer<MeshGroup>		m_MeshGroup;
		Pointer<VertexBuffer>	m_Vertices;
	};

	void					interpolate(Group* ,unsigned int ,unsigned int ,float ) const;

	List<Group>				m_Groups;
	AnimationList			m_Animations;
};

}; };

#endif
