/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/
#ifndef __GRAPHICS3D_SKYMODEL_H
#define __GRAPHICS3D_SKYMODEL_H

#include "graphics3d.model.h"
#include "graphics3d.plainshader.h"
#include "graphics3d.appearance.h"
#include "graphics3d.indexmesh.h"
#include "graphics3d.colourmaterial.h"

namespace magnum { namespace graphics3d {
class IndexBuffer;
class VertexBuffer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class SkyModel : public Model {
DeclareClass(SkyModel, Model);
private:
	PlainShader				m_Shader;
	Appearance				m_Appearance;
	ColourMaterial			m_Material;
	IndexMesh				m_Mesh;
	Pointer<IndexBuffer>	m_IndexBuffer;
	Pointer<VertexBuffer>	m_VertexBuffer;

public:
	enum Type {
		Sphere,
		Hemisphere,
		Box,
	};

public:
	SkyModel(Type ,float ,unsigned int );
	~SkyModel();
	
	inline void				attachAppearance(Appearance* );
	inline void				detachAppearance();
	inline Appearance*		getAppearance() const;

protected:
	virtual void			onRender(const ParameterBuffer* ) const;
	
private:
	void					createSphere(float ,unsigned int );
	void					createBox(float ,unsigned int );
	void					createHemisphere(float ,unsigned int );
};

}; };

#endif
