/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_SPRINGMODEL_H
#define __GRAPHICS3D_SPRINGMODEL_H

#include "graphics3d.dynamicmodel.h"
#include "graphics3d.plainshader.h"
#include "graphics3d.colourmaterial.h"
#include "physics.relaxator.h"

namespace magnum { namespace graphics3d {
using magnum::physics::Relaxator;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class SpringModel : public DynamicModel {
DeclareClass(SpringModel, DynamicModel);
private:
	PlainShader						m_Shader;
	Appearance						m_Appearance;
	ColourMaterial					m_Material;
	IndexMesh						m_Mesh;
	Pointer<Relaxator<float,3> >	m_Springs;
	Array<Vector3<float> >			m_Vertices;
	Array<unsigned int>				m_Indices;
	
public:
	enum Type {
		Empty = 0,
		Rectangle = 1,
	};	
	
public:
	SpringModel(Type = Empty,unsigned int = 10,float scale = 10.0f);
	~SpringModel();

	inline void				relax(float dt);
	
	void					setGeometry(unsigned int* ,unsigned int);
	
	inline void				attachSprings(Relaxator<float,3>* );
	inline void				detachSprings();
	inline Relaxator<float,3>* getSprings() const;

	inline void				attachAppearance(Appearance* );
	inline void				detachAppearance();
	inline Appearance*		getAppearance() const;

protected:
	virtual void			onUpdate();
	virtual void			onRender(const ParameterBuffer* ) const;

private:
	void					init();
	void					createRectangle(unsigned int ,unsigned int ,float );	
};

}; };

#endif
