/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_STENCILMODE_H
#define __GRAPHICS3D_STENCILMODE_H

#include "system.assembler.h"
#include "graphics3d.opengl.h"
#include "graphics3d.resource.h"

namespace magnum { namespace graphics3d {
using magnum::system::Assembler;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class StencilMode : public Resource {
DeclareClass(StencilMode, Resource);
public:
	enum TestFunction {
		Never = GL_NEVER,
		Less = GL_LESS,
		Equal = GL_EQUAL,
		LessEqual = GL_LEQUAL,
		Greater = GL_GREATER,
		GreaterEqual = GL_GEQUAL,
		NotEqual = GL_NOTEQUAL,
		Always = GL_ALWAYS,
		Disable = -1,
	};

	enum Action {
		Keep = GL_KEEP,
		Zero = GL_ZERO,
		Replace = GL_REPLACE,
		Increment = GL_INCR,
		Decrement = GL_DECR,
		IncrementWrap = GL_INCR_WRAP_EXT,
		DecrementWrap = GL_DECR_WRAP_EXT,
		Invert = GL_INVERT,
	};

public:
	StencilMode();
	~StencilMode();

	inline void			select() const;
	inline void			deselect() const;
	inline void			forceSelect() const;
	static inline void	selectNone();
	static inline void	selectDefault();
	static inline StencilMode* getDefault();

	void				compileSelect(Assembler& ) const;
	void				compileDeselect(Assembler& ) const;

	// Stencil Buffer							
	inline void			setAction(Action ,Action ,Action );
	inline void			setFunction(TestFunction );
	inline void			setFunction(TestFunction , unsigned int ref, unsigned int mask);
	inline TestFunction	getFunction() const;
	inline void			setReference(unsigned int );
	inline unsigned int	getReference() const;
	inline void			setReadMask(unsigned int );
	inline unsigned int	getReadMask() const;
	inline void			setWriteMask(unsigned int mask = 0xffffffff);
	inline unsigned int	getWriteMask() const;

protected:
	void				onSelect() const;
	bool				onCompileSelect(Assembler& ) const;

private:	
	Action				m_Fail;
	Action				m_ZFail;
	Action				m_Pass;
	TestFunction		m_Function;
	unsigned int		m_Reference;
	unsigned int		m_ReadMask;
	unsigned int		m_WriteMask;
};

}; };

#endif
