/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_STENCILMODE_INL
#define __GRAPHICS3D_STENCILMODE_INL

namespace magnum { namespace graphics3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline StencilMode* StencilMode::getDefault() {
	return &getContext()->m_DefaultStencilMode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void StencilMode::select() const {
	Context* context = getContext();
	if (context->m_StencilMode == this)
		return;
	context->m_StencilMode = this;
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	StencilMode::forceSelect() const {
	Context* context = getContext();
	context->m_StencilMode = this;
	onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	StencilMode::deselect() const {
	Context* context = getContext();
	if (context->m_StencilMode == this)
		context->m_StencilMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	StencilMode::selectNone() {
	Context* context = getContext();
	context->m_StencilMode = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	StencilMode::selectDefault() {
	Context* context = getContext();
	context->m_StencilMode = &context->m_DefaultStencilMode;
	context->m_StencilMode->onSelect();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void StencilMode::setAction(StencilMode::Action fail,StencilMode::Action zfail,StencilMode::Action pass) {
	m_Fail = fail;
	m_ZFail = zfail;
	m_Pass = pass;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void StencilMode::setFunction(StencilMode::TestFunction t, unsigned int ref, unsigned int mask) {
	m_Function = t;
	m_Reference = ref;
	m_ReadMask = mask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void StencilMode::setFunction(StencilMode::TestFunction t) {
	m_Function = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline StencilMode::TestFunction StencilMode::getFunction() const {
	return m_Function;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	StencilMode::setReference(unsigned int ref) {
	m_Reference = ref;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int	StencilMode::getReference() const {
	return m_Reference;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	StencilMode::setReadMask(unsigned int mask) {
	m_ReadMask = mask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int	StencilMode::getReadMask() const {
	return m_ReadMask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	StencilMode::setWriteMask(unsigned int mask) {
	m_WriteMask = mask;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int	StencilMode::getWriteMask() const {
	return m_WriteMask;
}


}; };

#endif
