/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TEXTURE_H
#define __GRAPHICS3D_TEXTURE_H

#include "system.assembler.h"
#include "container.list.h"
#include "graphics.palette.h"
#include "graphics.image.h"
#include "graphics3d.resource.h"
#include "graphics3d.opengl.h"


namespace magnum { namespace graphics3d {
using magnum::graphics::Palette;
using magnum::types::byte;
using magnum::types::String;
using magnum::container::List;
using magnum::system::Assembler;

class Texture1D;
class Texture2D;
class Texture3D;
class DepthTexture;
class CubeTexture;
class Context;
class TextureStack;
class RenderTarget;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Texture : public Resource, private magnum::object::Noncopyable  {
DeclareClass(Texture, Resource);
friend Texture1D;
friend Texture2D;
friend Texture3D;
friend DepthTexture;
friend CubeTexture;
friend RenderTarget;
friend Context;
public:
	enum {
		MaxTextures = 8,
	};

	enum LockFlag {
		ReadOnly = 1,
		WriteOnly = 2,
		ReadWrite = 3,
		Discard = 4
	};

	enum Filter {
		Nearest = GL_NEAREST,
		Linear = GL_LINEAR,
		NearestMipmapNearest = GL_NEAREST_MIPMAP_NEAREST,
		LinearMipmapNearest = GL_LINEAR_MIPMAP_NEAREST,
		NearestMipmapLinear = GL_NEAREST_MIPMAP_LINEAR,
		LinearMipmapLinear = GL_LINEAR_MIPMAP_LINEAR
	};

	enum Wrapping {
		Clamp = GL_CLAMP,
		ClampEdge = GL_CLAMP_TO_EDGE,
		Repeat = GL_REPEAT,
		Mirror = GL_MIRRORED_REPEAT
	};

	enum MipMaps {
		CopyAll	= -1,
		GenerateAll = -2
	};


	typedef magnum::graphics::Image::Type Type;

private:
	unsigned int	m_Levels;
	Type			m_Type;
	Filter			m_MagFilter;
	Filter			m_MinFilter;
	float			m_MaxLod;
	float			m_MinLod;
	float			m_LodBias;
	Wrapping		m_WrapS;
	Wrapping		m_WrapT;
	Wrapping		m_WrapU;
	GLuint			m_OglTexture;
	GLuint			m_OglTarget;
	GLuint			m_OglBinding;


protected:
	Texture(unsigned int tex, unsigned int target);

public:
	~Texture();

	inline void			select(unsigned int stage = 0) const;
	inline void			deselect(unsigned int stage = 0) const;
	void				forceSelect(unsigned int stage = 0) const;
	static inline void	selectNone(unsigned int stage = 0);

	void				compileSelect(Assembler& ,unsigned int stage = 0) const;
	void				compileDeselect(Assembler& ,unsigned int stage = 0) const;

	inline bool			isIndexed() const;
	inline Type			getType() const;
	inline unsigned int	getTarget() const;
	inline unsigned int	getLevels() const;
	
	void				generateMipmaps();

	void				setFilter(Filter min, Filter max);
	inline Filter		getMagFilter() const;
	inline Filter		getMinFilter() const;
	
	void				setLod(float ,float );
	inline float		getMinLod() const;
	inline float		getMaxLod() const;
	
	void				setLodBias(float );
	inline float		getLodBias() const;
						
	void				setWrapping(Wrapping , Wrapping = Clamp, Wrapping = Clamp);
	inline Wrapping		getWrappingS() const;
	inline Wrapping		getWrappingT() const;
	inline Wrapping		getWrappingU() const;

	bool				getPalette(Palette<byte>& pal) const;
	bool				setPalette(const Palette<byte>& pal);
	
protected:
	static int			mapOutputFormat(Type );
	static int			mapInputFormat(Type );
	static int			mapInputType(Type );
	static void			setupPalette(const Palette<byte>& );
};

typedef List<Texture*>	TextureList;

}; };

#endif
