/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TEXTURE_INL
#define __GRAPHICS3D_TEXTURE_INL

namespace magnum { namespace graphics3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Texture::select(unsigned int idx) const {
	ASSERT(idx < MaxTextures);
	Context* c = getContext();
	if (c->m_Texture[idx] == this)
		return;
	c->m_Texture[idx] = this;
	c->m_OglTextureTarget[idx] = m_OglTarget;
	glActiveTexture(GL_TEXTURE0 + idx);
	glBindTexture(m_OglTarget, m_OglTexture);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Texture::deselect(unsigned int idx) const {
	ASSERT(idx < MaxTextures);
	Context* c = getContext();
	if (c->m_Texture[idx] != this)
		return;
	c->m_Texture[idx] = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Texture::selectNone(unsigned int idx) {
	ASSERT(idx < MaxTextures);
	Context* c = getContext();
	c->m_Texture[idx] = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Type Texture::getType() const {
	return m_Type;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Texture::getTarget() const {
	return m_OglTarget;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Filter Texture::getMagFilter() const {
	return m_MagFilter;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Filter Texture::getMinFilter() const {
	return m_MinFilter;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Texture::getMaxLod() const {
	return m_MaxLod;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Texture::getMinLod() const {
	return m_MinLod;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Texture::getLodBias() const {
	return m_LodBias;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Wrapping Texture::getWrappingS() const {
	return m_WrapS;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Wrapping Texture::getWrappingT() const {
	return m_WrapT;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Texture::Wrapping Texture::getWrappingU() const {
	return m_WrapU;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Texture::getLevels() const {
	return m_Levels;
}


}; };

#endif
