/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TRANSLATOR_3DS_H
#define __GRAPHICS3D_TRANSLATOR_3DS_H

#include "object.pointer.h"
#include "io.translator.h"
#include "container.map.h"
#include "container.stack.h"
#include "container.array.h"
#include "container.list.h"
#include "graphics.colour.h"
#include "graphics3d.translator.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::math::Matrix4;
using magnum::graphics::Colour;
using magnum::container::Map;
using magnum::container::Stack;
using magnum::container::Array;
using magnum::container::List;

class StaticModel;
class IndexMesh;
class Shader;
class Appearance;
class VertexBuffer;
class IndexBuffer;
class ColourMaterial;
class BlendMode;
class DepthMode;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Translator3DS : public ModelTranslator {
DeclareClass(Translator3DS, ModelTranslator);
public:
	Translator3DS();
	~Translator3DS();
	
protected:
	Model*			onDecode(InputStream* ,size_t size = Autosize);
	size_t			onEncode(OutputStream* ,const Model* );
	bool			onIdentify(File* );


private:
	void			ascendChunk();
	bool			nextChunk();
	void			descendChunk();

	inline size_t	getChunkSize();
	inline word		getChunkId();

	inline size_t	readChunk(void* dst, size_t len);

	Colour<float>	readColour();
	String			readString();
	float			readScalar();

	void			readMaterial();
	void			readMaterialTexture(Appearance* ,unsigned int = 0);
	void			readVertexList();
	void			readFaceList();
	void			readFaceMaterial();
	void			readMapList();
	void			readTraMatrix();
	void			readTriMesh();
	void			readObjectBlock();
	void			readMeshWorld();

	void			createVertexBuffer();
	void			createIndexBuffer();
	void			createMeshes();

private:
	Shader*				m_PlainShader;
	Shader*				m_TextureShader;
	BlendMode*			m_TransparentBlending;
	DepthMode*			m_TransparentDepthMode;
	InputStream*		m_In;
	StaticModel*		m_Model;
	VertexBuffer*		m_VertexBuffer;
	IndexBuffer*		m_IndexBuffer;

	Matrix4<float>		m_Matrix;
	Array<float32>		m_VertexList;
	Array<float32>		m_MapList;
	Array<word>			m_FaceList;
	unsigned int		m_VertexCount;
	unsigned int		m_FaceCount;

	struct FaceMaterial {
		Appearance*		m_Appearance;
		int				m_FaceCount;
		Array<word>		m_FaceList;
	};

	List<FaceMaterial>					m_FaceMaterials;
	Map<String, Pointer<Appearance> >	m_Materials;

#pragma pack(push,1)
	struct Chunk {
		word			m_Id;
		dword			m_Len;
	};					
#pragma pack(pop)		
						
	Stack<Chunk>		m_ChunkStack;
	size_t				m_ChunkLen;
	word				m_ChunkId;
	size_t				m_ChunkLeft;
};

}; };

#endif

