/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TRANSLATOR_IFS_H
#define __GRAPHICS3D_TRANSLATOR_IFS_H

#include "math.vector3.h"
#include "io.translator.h"
#include "container.map.h"
#include "container.array.h"
#include "graphics.colour.h"
#include "graphics3d.translator.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::math::Matrix4;
using magnum::math::Vector3;
using magnum::graphics::Colour;
using magnum::container::Map;
using magnum::container::Array;

class StaticModel;
class IndexMesh;
class Shader;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TranslatorIFS : public ModelTranslator {
DeclareClass(TranslatorIFS, ModelTranslator);
public:
	TranslatorIFS();

protected:
	Model*			onDecode(InputStream* ,size_t size = Autosize);
	size_t			onEncode(OutputStream* ,const Model* );
	bool			onIdentify(File* );
	
private:
	void			readHeader();
	void			readVertices();
	void			readIndices();
	void			createNormals();
	String			readString();

private:
	typedef Vector3<float> vec3;
	
	Array<vec3>			m_Normals;
	Array<vec3>			m_Vertices;
	Array<unsigned int>	m_Indices;
	
	unsigned int		m_VertexCount;
	unsigned int		m_FaceCount;
	
	String				m_Name;
	InputStream*		m_Input;
}; 


}; };

#endif