/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TRANSLATOR_LWO_H
#define __GRAPHICS3D_TRANSLATOR_LWO_H

#include "object.pointer.h"
#include "io.translator.h"
#include "container.map.h"
#include "container.array.h"
#include "math.vector2.h"
#include "math.vector3.h"
#include "graphics.colour.h"
#include "graphics3d.translator.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::math::Matrix4;
using magnum::math::Vector2;
using magnum::math::Vector3;
using magnum::graphics::Colour;
using magnum::container::Map;
using magnum::container::Array;

class StaticModel;
class IndexMesh;
class Mesh;
class Shader;
class Appearance;
class VertexBuffer;
class IndexBuffer;
class ColourMaterial;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TranslatorLWO : public ModelTranslator {
DeclareClass(TranslatorLWO, ModelTranslator);
public:
	TranslatorLWO();
	~TranslatorLWO();
	
protected:
	Model*			onDecode(InputStream* ,size_t size = Autosize);
	size_t			onEncode(OutputStream* ,const Model* );
	bool			onIdentify(File* );


private:
	inline int		readShort();
	inline int		readInt();
	inline float	readFloat();
	inline int		readString(String& );
	inline void		skipChunk(int size);
	
	void			read_LWO2(int size);
	void			read_LAYR(int size);			// layer
	void			read_PNTS(int size);			// point list
	void			read_VMAP(int size);		// vertex mapping
	void			read_POLS(int size);			// polygon list
	void			read_PTAG(int size);		// polygon tag mapping
	void			read_VMAD(int size);	// discontinuous vertex mapping
	void			read_SURF(int size);			// surface
	void			read_BLOK(int size);
	
	struct Layer;
	struct Surface;

	Model*			createModel();
	Mesh*			createMesh(Layer* ,Surface* );
	void			createSurfaces(Layer* );

	void			applyMapping(Layer *layer,Surface *surface);
	void			calculateNormals(Layer *layer,Surface *surface);

private:
	enum {
		NUM_VERTEX = 64,
	};
	
	struct VTexCoord {
		Array<Vector2<float> >	m_Coords;
	};
	
	struct PTexCoord {
		Array<int>				m_Polygons;
		Array<int>				m_Vertices;
		Array<Vector2<float> >	m_Coords;
	};
	
	struct Polygon {
		unsigned int		m_Size;
		unsigned int		m_Vertex[NUM_VERTEX];
		Vector3<float>		m_Normals[NUM_VERTEX];
		Vector2<float>		m_TexCoords[NUM_VERTEX];
		Vector3<float>		m_Normal;
	};
	
	struct Surface {
		String				m_Name;
		String				m_Source;
		String				m_Texture;
		Array<int>			m_Polygons;
		float				m_Smoothing;
		Appearance*			m_Appearance;
	};
	
	struct Layer {
		String				m_Name;
		Array<Vector3<float> > m_Vertices;
		Map<String,VTexCoord> m_VTexCoords;
		Map<String,PTexCoord> m_PTexCoords;
		Array<Polygon>		m_Polygons;
		Array<int>			m_PolygonTags;
		Array<Surface>		m_Surfaces;
	};
	
	struct Vertex {
		Vector3<float>		m_Vertex;
		Vector3<float>		m_Normal;
		Vector2<float>		m_Texture;
	};

	Array<Layer>		m_Layers;
	Layer*				m_CurrentLayer;
	Surface*			m_CurrentSurface;
	InputStream*		m_In;

	Shader*				m_PlainShader;
	Shader*				m_TextureShader;
	StaticModel*		m_Model;
	ColourMaterial*		m_Material;
	VertexBuffer*		m_VertexBuffer;
	IndexBuffer*		m_IndexBuffer;
};

}; };

#endif

