/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TRANSLATOR_MD2_H
#define __GRAPHICS3D_TRANSLATOR_MD2_H

#include "io.translator.h"
#include "container.map.h"
#include "container.array.h"
#include "container.pair.h"
#include "graphics.colour.h"
#include "graphics3d.translator.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::graphics::Colour;
using magnum::container::Map;
using magnum::container::Array;
using magnum::container::Pair;

class FrameModel;
class IndexMesh;
class Shader;
class MeshGroup;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TranslatorMD2 : public ModelTranslator {
DeclareClass(TranslatorMD2, ModelTranslator);
public:
	TranslatorMD2();

protected:
	Model*			onDecode(InputStream* ,size_t size = Autosize);
	size_t			onEncode(OutputStream* ,const Model* );
	bool			onIdentify(File* );

private:
	size_t			readSkins(unsigned int );	
	size_t			readTexCoords(unsigned int );
	size_t			readTriangles(unsigned int );
	size_t			readFrames(unsigned int );
	size_t			readGlCommands(unsigned int );

	void			createIndexMapping();
	void			createTextureCoords(IndexMesh* mesh);
	void			createFrames(FrameModel* model, MeshGroup* );
	void			createIndices(IndexMesh* mesh);
	void			createAnimations(FrameModel* );

private:
	struct Header { 
		int		m_Magic; 
		int		m_Version; 
		int		m_SkinWidth; 
		int		m_SkinHeight; 
		int		m_FrameSize; 
		int		m_NumSkins; 
		int		m_NumVertices; 
		int		m_NumTexCoords; 
		int		m_NumTriangles; 
		int		m_NumGlCommands; 
		int		m_NumFrames; 
		int		m_OffsetSkins; 
		int		m_OffsetTexCoords; 
		int		m_OffsetTriangles; 
		int		m_OffsetFrames; 
		int		m_OffsetGlCommands; 
		int		m_OffsetEnd; 
	};

	struct TriangleVertex {
		byte	m_Vertex[3];
		byte	m_LightNormalIndex;
	};

	struct Frame {
		float	m_Scale[3];
		float	m_Translate[3];
		char	m_Name[16];
		TriangleVertex vertices[1];
	};

	struct Triangle {
		word	m_VertexIndices[3];
		word	m_TextureIndices[3];
	};

	struct TextureCoordinate {
		word	s, t;
	};
	
	struct AnimationInfo {
		unsigned int	m_First;
		unsigned int	m_Last;
		unsigned int	m_Fps;
		bool			m_Loop;
		const char*		m_Name;
	};

	typedef Pair<int,int> IPair;


	enum {
		Magic = 0x32504449,
		MaxTriangles = 4096,
		MaxVertices = 2048,
		MaxTextureCoordinates = 2048,
		MaxFrames = 512,
		MaxSkins = 32 
	};

	InputStream*		m_In;
						
	Triangle*			m_Triangles;
	TriangleVertex*		m_Vertices;
	Array<Frame*>		m_Frames;
	TextureCoordinate*	m_TexCoords;
	Texture2D*			m_Skin;
	Array<int>			m_VertexSource;
	Array<int>			m_TextureSource;
	Map<IPair,int>		m_VertexMapping;
	unsigned int		m_TriangleCount;
	unsigned int		m_VertexCount;
	unsigned int		m_SkinWidth;
	unsigned int		m_SkinHeight;
	unsigned int		m_FrameSize;
	static const AnimationInfo m_Animations[];
}; 


}; };

#endif