/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TRANSLATOR_MD3_H
#define __GRAPHICS3D_TRANSLATOR_MD3_H

#include "io.translator.h"
#include "container.array.h"
#include "graphics.colour.h"
#include "graphics3d.translator.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::math::Matrix4;
using magnum::graphics::Colour;
using magnum::container::Map;
using magnum::container::Array;

class StaticModel;
class IndexMesh;
class Shader;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TranslatorMD3 : public ModelTranslator {
DeclareClass(TranslatorMD3, ModelTranslator);
public:
	TranslatorMD3();

protected:
	Model*				onDecode(InputStream* ,size_t size = Autosize);
	size_t				onEncode(OutputStream* ,const Model* );
	bool				onIdentify(File* );


private:
	struct Header { 
		unsigned int	m_Magic; 
		unsigned int	m_Version; 
		char			m_Name[64];
		unsigned int	m_Flags;
		unsigned int	m_NumFrames;
		unsigned int	m_NumTags;
		unsigned int	m_NumSurfaces;
		unsigned int	m_NumSkins;
		unsigned int	m_OffsetFrames;
		unsigned int	m_OffsetTags;
		unsigned int	m_OffsetSurfaces;
		unsigned int	m_OffsetEnd;
	};
	
	struct Frame {
		Vector3<float>	m_Minimum;
		Vector3<float>	m_Maximum;
		Vector3<float>	m_Origin;
		float			m_Radius;
		char			m_Name[16];
	};
	
	struct Tag {
		char			m_Name[64];
		Vector3<float>	m_Origin;
		Vector3<float>	m_Orientation[3];
	};
	
	struct Surface {
		unsigned int	m_Magic;
		char			m_Name[64];
		unsigned int	m_Flags;
		unsigned int	m_NumFrames;
		unsigned int	m_NumShaders;
		unsigned int	m_NumVertices;
		unsigned int	m_NumTriangles;
		unsigned int	m_OffsetTriangles;
		unsigned int	m_OffsetShaders;
		unsigned int	m_OffsetTextures;
		unsigned int	m_OffsetVertices;
		unsigned int	m_OffsetEnd;
	};
	
	struct Shader {
		char			m_Name[64];
		unsigned int	m_Index;
	};
	
	struct Vertex {
		int16			m_Vertex[3];
		int16			m_Normal;
	};

	enum {
		Magic1 = 0x51806873,
		Magic2 = 0x33504449,
		MaxTriangles = 4096,
		MaxVertices = 2048,
		MaxTextureCoordinates = 2048,
		MaxFrames = 512,
		MaxSkins = 32 
	};

private:
	size_t				readFrames(unsigned int );
	size_t				readTags(unsigned int );
	size_t				readSurfaces(MeshGroup* ,unsigned int );
	size_t				readShaders(unsigned int );	
	size_t				readTextures(Vector2<float>* ,unsigned int );
	size_t				readTriangles(IndexMesh* ,unsigned int ,unsigned int );
	size_t				readVertices(Vertex* ,unsigned int );

	void				createTextureCoords(IndexMesh* mesh);
	void				createFrames(FrameModel* model, MeshGroup* );
	void				createAnimations(FrameModel* );
	void				createJoints(FrameModel* );

	InputStream*		m_In;
	List<Frame>			m_Frames;
	List<Surface>		m_Surfaces;
	List<Tag>			m_Tags;
	Array<Vertex>		m_Vertices;
	Array<Vector2<float> > m_Textures;
}; 


}; };

#endif
