/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TRANSLATOR_MD4_H
#define __GRAPHICS3D_TRANSLATOR_MD4_H

#include "io.translator.h"
#include "container.map.h"
#include "container.array.h"
#include "graphics.colour.h"
#include "graphics3d.translator.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::math::Matrix4;
using magnum::graphics::Colour;
using magnum::container::Map;
using magnum::container::Array;

class StaticModel;
class IndexMesh;
class Shader;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TranslatorMD4 : public ModelTranslator {
DeclareClass(TranslatorMD4, ModelTranslator);
public:
	TranslatorMD4();

protected:
	Model*				onDecode(InputStream* ,size_t size = Autosize);
	size_t				onEncode(OutputStream* ,const Model* );
	bool				onIdentify(File* );

private:
	struct Header { 
		unsigned int	m_Magic; 
		unsigned int	m_Version; 
		char			m_Name[64];
		unsigned int	m_NumFrames;
		unsigned int	m_NumBones;
		unsigned int	m_OffsetFrames;
		unsigned int	m_NumLODs;
		unsigned int	m_OffsetLODs;
		unsigned int	m_OffsetEnd;
	};

	struct Frame {
		Vector3<float>	m_Minimum;
		Vector3<float>	m_Maximum;
		Vector3<float>	m_Origin;
		float			m_Radius;
		char			m_Name[16];
	};
	
	struct Bone {
		Vector3<float>	m_Location;
		Vector3<float>	m_Axis[3];
	};
	
	struct LOD {
		unsigned int	m_NumSurfaces;
		unsigned int	m_OffsetSurfaces;
		unsigned int	m_OffsetEnd;
	};

	struct Surface {
		unsigned int	m_Magic;
		char			m_Name[64];
		char			m_Shader[64];
		unsigned int	m_ShaderIndex;
		unsigned int	m_OffsetHeader;
		unsigned int	m_NumVertices;
		unsigned int	m_OffsetVertices;
		unsigned int	m_NumTriangles;
		unsigned int	m_OffsetTriangles;
		unsigned int	m_NumBoneReferences;
		unsigned int	m_OffsetBoneReferences;
		unsigned int	m_OffsetEnd;
	};

	struct Vertex {
		Vector3<float>	m_Vertex;
		Vector3<float>	m_Normal;
		Vector2<float>	m_Texture;
		unsigned int	m_NumWeights;
	};
	
	struct Weight {
		unsigned int	m_BoneIndex;
		unsigned int	m_BoneWeight;
	};
	
	struct FrameExt : public Frame {
		List<Bone>		m_Bones;
	};
	
	struct LODExt : public LOD {
		List<Surface>	m_Surfaces;
	};

	static const int	Magic = 0x52806873;

private:
	size_t				readFrames(unsigned int ,unsigned int);
	size_t				readBones(FrameExt* ,unsigned int );
	size_t				readLODs(unsigned int );
	size_t				readSurfaces(LODExt* ,unsigned int );
	size_t				readVertices(unsigned int );
	size_t				readTriangles(unsigned int );
	size_t				readBoneReferences(unsigned int );

	InputStream*		m_In;
	List<FrameExt>		m_Frames;
	List<LODExt>		m_LODs;
}; 


}; };

#endif