/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TRANSLATOR_OBJ_H
#define __GRAPHICS3D_TRANSLATOR_OBJ_H

#include "object.pointer.h"
#include "io.translator.h"
#include "io.datainput.h"
#include "container.array.h"
#include "container.list.h"
#include "container.map.h"
#include "math.vector2.h"
#include "math.vector3.h"
#include "graphics.colour.h"
#include "graphics3d.translator.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::math::Vector2;
using magnum::math::Vector3;
using magnum::graphics::Colour;
using magnum::container::Array;
using magnum::container::List;
using magnum::container::Map;
using magnum::io::DataInput;

class StaticModel;
class IndexMesh;
class Mesh;
class Shader;
class Appearance;
class VertexBuffer;
class IndexBuffer;
class ColourMaterial;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TranslatorOBJ : public ModelTranslator {
DeclareClass(TranslatorOBJ, ModelTranslator);
public:

protected:
	Model*			onDecode(InputStream* ,size_t size = Autosize);
	size_t			onEncode(OutputStream* ,const Model* );
	bool			onIdentify(File* );
	
private:
	typedef Vector3<unsigned int> Vertex;
	
	void			readMain(DataInput& );
	void			readVertex(const String& );
	void			readFace(const String& );
	void			readUV(const String& );
	void			readNormal(const String& );
	void			readObject(const String& );
	void			readGroup(const String& );
	void			readMaterial(const String& );
	void			readMaterialLibrary(const String& );
	
	void			mtlReadMain(DataInput& );
	void			mtlReadMaterial(const String& );
	void			mtlReadIllum(const String& );	
	void			mtlReadDiffuse(const String& );	
	void			mtlReadSpecular(const String& );	
	void			mtlReadAmbient(const String& );	
	void			mtlReadDiffuseMap(const String& );	
	void			mtlReadHighlight(const String& );	

	StaticModel*	createModel();
	void			createNormals();
	void			createVertexMapping(Map<Vertex,unsigned int>& , Map<Vertex,unsigned int>& );
	void			createIndices(Map<Appearance*,Array<unsigned int> >& ,const Map<Vertex,unsigned int>& ,Map<Appearance*,Array<unsigned int> >& ,const Map<Vertex,unsigned int>& );
	VertexBuffer*	createSimpVertexBuffer(const Map<Vertex,unsigned int>& );
	VertexBuffer*	createTexVertexBuffer(const Map<Vertex,unsigned int>& );
	
	inline void		adjustVertex(int& ) const;
	inline void		adjustTexture(int& ) const;
	inline void		adjustNormal(int& ) const;
	
	
private:
	struct Triangle {
		int					m_v1;
		int					m_t1;
		int					m_n1;

		int					m_v2;
		int					m_t2;
		int					m_n2;

		int					m_v3;
		int					m_t3;
		int					m_n3;
		
		Appearance*			m_Appearance;
	};
	struct Object {
		String				m_Name;
		Array<Triangle>		m_Triangles;
	};
	
	Array<Vector3<float> >	m_Normals;
	Array<Vector3<float> >	m_Vertices;
	Array<Vector3<float> >	m_Textures;
	List<Object>			m_Objects;
	Map<String,Appearance*> m_Materials;

	Object*					m_Object;
	Array<Triangle>*		m_Triangles;
	Appearance*				m_Appearance;
	ColourMaterial*			m_Material;
}; 

}; };

#endif
