/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TRANSLATOR_OFF_H
#define __GRAPHICS3D_TRANSLATOR_OFF_H

#include "object.pointer.h"
#include "io.translator.h"
#include "math.vector2.h"
#include "math.vector3.h"
#include "graphics.colour.h"
#include "graphics3d.translator.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::math::Matrix4;
using magnum::math::Vector2;
using magnum::math::Vector3;
using magnum::graphics::Colour;

class StaticModel;
class IndexMesh;
class Mesh;
class Shader;
class Appearance;
class VertexBuffer;
class IndexBuffer;
class ColourMaterial;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TranslatorOFF : public ModelTranslator {
DeclareClass(TranslatorOFF, ModelTranslator);
public:
	TranslatorOFF();
	
protected:
	Model*				onDecode(InputStream* ,size_t size = Autosize);
	size_t				onEncode(OutputStream* ,const Model* );
	bool				onIdentify(File* );

private:
	void				readHeader();
	void				readVertices();
	void				readIndices();
	void				createNormals();
	const char*			readToken();
	void				finishLine();

private:
	typedef Vector3<float> vec3;
	typedef Colour<float> col;
	
	Array<vec3>			m_Normals;
	Array<vec3>			m_Vertices;
	Array<col>			m_Colours;
	Array<unsigned int>	m_Indices;
	
	unsigned int		m_VertexCount;
	unsigned int		m_FaceCount;
	bool				m_ReadNormals;
	bool				m_ReadColours;
	
	InputStream*		m_Input;
	char				m_Token[256];
	bool				m_LineFinished;
};

}; };

#endif

