/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TRANSLATOR_PLY_H
#define __GRAPHICS3D_TRANSLATOR_PLY_H

#include "object.pointer.h"
#include "io.translator.h"
#include "io.inputbuffer.h"
#include "container.array.h"
#include "container.list.h"
#include "math.vector2.h"
#include "math.vector3.h"
#include "graphics.colour.h"
#include "graphics3d.translator.h"

namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::math::Matrix4;
using magnum::math::Vector2;
using magnum::math::Vector3;
using magnum::graphics::Colour;
using magnum::container::Array;
using magnum::container::List;
using magnum::io::InputBuffer;
using magnum::types::byte;

class StaticModel;
class IndexMesh;
class Mesh;
class Shader;
class Appearance;
class VertexBuffer;
class IndexBuffer;
class ColourMaterial;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TranslatorPLY : public ModelTranslator {
DeclareClass(TranslatorPLY, ModelTranslator);
public:

protected:
	Model*			onDecode(InputStream* ,size_t size = Autosize);
	size_t			onEncode(OutputStream* ,const Model* );
	bool			onIdentify(File* );

private:
	enum FileType {
		ASCII = 1,
		BinaryBE = 2,
		BinaryLE = 3,
	};
	enum DataType {
		TypeNone = 0,
		TypeChar = 1,
		TypeShort = 2,
		TypeInt = 3,
		TypeUChar = 4,
		TypeUShort = 5,
		TypeUInt = 6,
		TypeFloat = 7,
		TypeDouble = 8,
		TypeEnd = 9
	};

	typedef void (TranslatorPLY::*PropertyHandler)(const String& ,DataType );
	typedef void (TranslatorPLY::*ListHandler)(unsigned int );
	typedef void (TranslatorPLY::*ElementHandler)();

	struct Property {
		String			m_Name;
		DataType		m_Type;
		DataType		m_ListType;
		PropertyHandler	m_Handler;
		ListHandler		m_Lister;
	};
	struct Element {
		String			m_Name;
		Array<Property>	m_Properties;
		unsigned int	m_Size;
		ElementHandler	m_Handler;
	};

private:	
	void				readHeader();
	void				readData();
	void				installHandlers();
	void				createNormals();
	VertexBuffer*		createVertexBuffer();
	IndexBuffer*		createIndexBuffer();

	void				readVertex(const String& ,DataType );
	void				readColour(const String& ,DataType );
	void				readFace(unsigned int );
	void				readFaceIndex(const String& ,DataType );

	void				addElementHandler(const char* elem, ElementHandler );
	void				addPropertyHandler(const char* elem, const char* prop, PropertyHandler );
	void				addListHandler(const char* elem, const char* prop, ListHandler );

	Element*			findElement(const char* ) const;
	Property*			findProperty(const char* ,const char* ) const;
	
	Element*			addElement(const Array<String>& );
	Property*			addProperty(Element* ,const Array<String>& );
	
	void				getWords(const char* ,Array<String>& );
	template<class T>
	inline T			getItem(const String& ,DataType );
	template<class T>
	T					getItemASCII(const String& ,DataType );
	template<class T>
	T					getItemBinary(DataType );
	void				getElementASCII(const Element* , const Array<String>& );
	void				getElementBinary(const Element*  );

	DataType			getPropertyType(const String& );
	
private:
	FileType			m_Type;
	float				m_Version;
	InputBuffer			m_In;
	List<Element>		m_Elements;
	List<String>		m_Comments;
	List<String>		m_Infos;
	String				m_Dummy;
	Array<Vector3<float> > m_Normals;
	Array<Vector3<float> > m_Vertices;
	Array<byte>			m_Colours;
	Array<unsigned int>	m_Indices;
	float*				m_CurrentVertex;
	unsigned int		m_CurrentIndex;
	byte*				m_CurrentColour;
	unsigned int		m_FirstIndex;
	unsigned int		m_LastIndex;

	static const char*	m_TypeNames[];
	static const unsigned int m_TypeSizes[];
}; 

}; };

#endif
