/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_TYPES_H
#define __GRAPHICS3D_TYPES_H

#include "object.h"

namespace magnum { namespace graphics3d {

class Types {
public:
	enum {
		TypeShift	= 0,
		TypeMask	= 0xff,
		CountShift  = 8,
		CountMask   = 0xf,
		SizeShift   = 16,
		SizeMask    = 0xfff,
	};

	enum Type {
		// Each member has the following format
		// Bits 0..3  : Size per Component
		// Bit  4     : Unsigned
		// Bit  5     : Float
		// Bit  6	  : Struct
		// Bits 8..15 : Number of Components
		// Bits 16..27: Total Size in Bytes
		Struct		= 0x00000040,
		Byte		= 0x00010101,
		Byte1		= 0x00010101,
		Byte2		= 0x00020201,
		Byte3		= 0x00030301,
		Byte4		= 0x00040401,
		UByte		= 0x00010111,
		UByte1		= 0x00010111,
		UByte2		= 0x00020211,
		UByte3		= 0x00030311,
		UByte4		= 0x00040411,
		Short		= 0x00020102,
		Short1		= 0x00020102,
		Short2		= 0x00040202,
		Short3		= 0x00060302,
		Short4		= 0x00080402,
		UShort		= 0x00020112,
		UShort1		= 0x00020112,
		UShort2		= 0x00040212,
		UShort3		= 0x00060312,
		UShort4		= 0x00080412,
		Int			= 0x00040104,
		Integer		= 0x00040104,
		Int1		= 0x00040104,
		Integer1	= 0x00040104,
		Int2		= 0x00080204,
		Integer2	= 0x00080204,
		Int3		= 0x000c0304,
		Integer3	= 0x000c0304,
		Int4		= 0x00100404,
		Integer4	= 0x00100404,
		UInt		= 0x00040114,
		UInt1		= 0x00040114,
		UInt2		= 0x00080214,
		UInt3		= 0x000c0314,
		UInt4		= 0x00100414,
		Float		= 0x00040124,
		Float1		= 0x00040124,
		Float2		= 0x00080224,
		Float3		= 0x000c0324,
		Float4		= 0x00100424,
		Double		= 0x00080128,
		Double1		= 0x00080128,
		Double2		= 0x00100228,
		Double3		= 0x00180328,
		Double4		= 0x00200428,

		Vector2f	= 0x00080224,
		Vector3f	= 0x000c0324,
		Vector4f	= 0x00100424,
		Vector2d	= 0x00100228,
		Vector3d	= 0x00180328,
		Vector4d	= 0x00200428,
		Matrix2f	= 0x00100424,	
		Matrix2d	= 0x00200428,
		Matrix3f	= 0x00240924,
		Matrix3d    = 0x00480928,
		Matrix4f	= 0x00401024,
		Matrix4d	= 0x00801028,

		Colour4b	= 0x00040411,
		Colour3b	= 0x00030311,
		Colour3f	= 0x000c0324,
		Colour4f	= 0x00100424,
	};
};


class Attributes {
public:
	enum Attribute {
		Vertex		= 0x00000001,
		Vertex1		= 0x00000001,
		Vertex2		= 0x00000002,
		Normal		= 0x00000004,
		Normal1		= 0x00000004,
		Normal2		= 0x00000008,
		Colour		= 0x00000010,
		Colour1		= 0x00000010,
		Colour2		= 0x00000020,
		Weights		= 0x00000040,
		Fog			= 0x00000080,
		Edge		= 0x00000100,
		Tangent		= 0x00000200,
		Binormal	= 0x00000400,
		Texture		= 0x00001000,
		Texture0	= 0x00001000,
		Texture1	= 0x00002000,
		Texture2	= 0x00004000,
		Texture3	= 0x00008000,
		Texture4	= 0x00010000,
		Texture5	= 0x00020000,
		Texture6	= 0x00040000,
		Texture7	= 0x00080000,
		TextureAll	= 0x000ff000,
		MaxAttributes = 20
	};

	enum AttributeI {
		VertexBit	= 0,
		Vertex1Bit	= 0,
		Vertex2Bit	= 1,
		NormalBit	= 2,
		Normal1Bit	= 2,
		Normal2Bit	= 3,
		ColourBit	= 4,
		Colour1Bit	= 4,
		Colour2Bit	= 5,
		WeightsBit	= 6,
		FogBit		= 7,
		EdgeBit		= 8,
		TangentBit	= 9,
		BinormalBit = 10,
		TextureBit	= 12,
		Texture0Bit	= 12,
		Texture1Bit	= 13,
		Texture2Bit	= 14,
		Texture3Bit	= 15,
		Texture4Bit	= 16,
		Texture5Bit	= 17,
		Texture6Bit	= 18,
		Texture7Bit	= 19,
	};
};


}; }; 

#endif
