/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_VERTEXBUFFER_H
#define __GRAPHICS3D_VERTEXBUFFER_H

#include "system.assembler.h"
#include "graphics3d.resource.h"
#include "graphics3d.opengl.h"
#include "graphics3d.vertexformat.h"


namespace magnum { namespace graphics3d {
using magnum::system::Assembler;
class VertexMesh;
class Material;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class VertexBuffer : public Resource, private magnum::object::Noncopyable  {
DeclareClass(VertexBuffer, Resource);
public:
	enum { 
		MaxBuffers = 16,
	};

	enum LockFlag {
		ReadOnly = 1,
		WriteOnly = 2,
		ReadWrite = 3,
		Discard = 4
	};

	enum Usage {
		Static = 1,
		Dynamic = 2,
		Stream = 3,
		System = 4
	};


protected:
	VertexBuffer(const VertexFormat& );

public:
	static VertexBuffer* create(const VertexFormat& , unsigned int count, Usage = Dynamic);

	~VertexBuffer();

	inline void*		lock(LockFlag = ReadWrite);
	inline void			unlock();
	inline bool			isLocked() const;

	void				setBuffer(unsigned int offset, const void* data, unsigned int size);
	void				getBuffer(unsigned int offset, void* data, unsigned int size) const;

	inline void			selectVertices(unsigned int ) const;
	inline void			selectNormals(unsigned int ) const;
	inline void			selectColours(unsigned int ) const;
	inline void			selectEdges(unsigned int ) const;
	inline void			selectTexturCoords(unsigned int , unsigned int = 0) const;
	inline void			selectFogCoords(unsigned int ) const;

	inline void			selectVertices() const;
	inline void			selectNormals() const;
	inline void			selectColours() const;
	inline void			selectEdges() const;
	inline void			selectTexturCoords(unsigned int stage = 0) const;
	inline void			selectFogCoords() const;

	void				select(unsigned int flags = 0xffffffff) const;
	void				compileSelect(Assembler& as, unsigned int flags = 0xffffffff) const;

	inline const VertexFormat& getFormat() const;
	inline unsigned int	getSize() const;
	inline unsigned int	getStride() const;
	inline Usage		getUsage() const;

	inline unsigned int	getOffset(unsigned int i) const;
	inline unsigned int	getSize(unsigned int i) const;
	inline unsigned int	getCount(unsigned int i) const;
	inline VertexFormat::Type getType(unsigned int i) const;
	inline VertexFormat::Attribute getAttribute(unsigned int i) const;
	inline unsigned int	getAttributes() const;

private:
	inline void*		bindBuffer() const;

private:
	struct Mapping {
		unsigned int	m_OglType;
		unsigned int	m_OglSize;
		unsigned int	m_Offset;
	};

	Mapping				m_Mapping[Attributes::MaxAttributes];
	VertexFormat		m_Format;
	void*				m_Buffer;
	unsigned int		m_Size;
	bool				m_Locked;
	unsigned int		m_Attributes;
	unsigned int		m_OglUsage;
	unsigned int		m_OglBuffer;
	Usage				m_Usage;

	static const int 	g_OglAcces[];
	static const int 	g_OglUsage[];
};



}; };

#endif
