/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_VERTEXFORMAT_H
#define __GRAPHICS3D_VERTEXFORMAT_H

#include "graphics3d.types.h"


namespace magnum { namespace graphics3d {
using magnum::object::Object;
using magnum::types::String;

class VertexBuffer;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class VertexFormat : public Object {
DeclareClass(VertexFormat, Object);
public:
	typedef Types::Type				Type;
	typedef Attributes::Attribute	Attribute;

	enum {
		MaxComponents = 16,
	};

	struct Component {
	friend VertexFormat;
	friend VertexBuffer;
	private:
		Type				m_Type;
		Attribute			m_Attribute;
		unsigned int		m_Offset;
		unsigned int		m_OglSize;
		unsigned int		m_OglType;
		
	public:
		inline Type			getType() const;
		inline Attribute	getAttribute() const;
		inline unsigned int	getOffset() const;
		inline unsigned int	getSize() const;
		inline unsigned int getCount() const;
	};

	typedef Component*		Iterator;

private:
	Component				m_Component[MaxComponents];
	unsigned int			m_Stride;
	unsigned int			m_Size;

public:
	inline VertexFormat();
	inline VertexFormat(const VertexFormat& );

	inline void				clear();
	unsigned int			insert(Type t, Attribute m);
	void					set(unsigned int i, Type t, Attribute m);
	void					remove(unsigned int i);
	unsigned int			shrink();
	void					append(const VertexFormat& );

	inline const Component& at(unsigned int i) const;
	inline const Component& operator[](unsigned int ) const;

	inline unsigned int		getSize() const;
	inline unsigned int		getStride() const;

	inline unsigned int		getOffset(unsigned int i) const;
	inline Attribute		getAttribute(unsigned int i) const;
	inline Type				getType(unsigned int i) const;
	inline unsigned int		getSize(unsigned int i) const;
	inline unsigned int		getCount(unsigned int i) const;

	inline Iterator			begin() const;
	inline Iterator			end() const;

	inline const VertexFormat& operator=(const VertexFormat& );

	bool					operator==(const VertexFormat& ) const;
	bool					operator!=(const VertexFormat& ) const;
};



}; };

#endif
