/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_VERTEXMESH_H
#define __GRAPHICS3D_VERTEXMESH_H

#include "math.range.h"
#include "graphics3d.mesh.h"
#include "graphics3d.vertexbuffer.h"

namespace magnum { namespace graphics3d {
using magnum::math::Range;
using magnum::object::Pointer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class VertexMesh : public Mesh {
DeclareClass(VertexMesh, Mesh);
public:
	VertexMesh();

	inline VertexBuffer*	getVertexBuffer(unsigned int i=0) const;
	void					attachVertexBuffer(VertexBuffer* ,unsigned int i=0);
	void					detachVertexBuffer(unsigned int i=0);
							
	inline Range<unsigned int> getRange() const;
	inline void				setRange(const Range<unsigned int>& );
	inline void				setRange(unsigned int start, unsigned int size);
							
protected:					
	inline void				selectVertexBuffers() const;
	void					releaseVertexBuffers();
	bool					searchVertices(VertexBuffer** ,unsigned int* offset, unsigned int* count, unsigned int* type) const;
							
	virtual void			onRender() const;
	virtual TriangleMesh3<float>* onTriangulate() const;
	virtual bool			onUpdateBounds();
	virtual bool			onUpdateCollider();

private:
	void					updateBufferInfo();

private:
	Pointer<VertexBuffer>	m_VertexBuffer[VertexBuffer::MaxBuffers];
	unsigned int			m_BufferCount;
	unsigned int			m_VertexCount;
	unsigned int			m_Attributes;
	Range<unsigned int>		m_Range;
};

}; };

#endif
