/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRAPHICS3D_WEIGHTBUFFER_H
#define __GRAPHICS3D_WEIGHTBUFFER_H

#include "container.array.h"
#include "math.vector3.h"
#include "math.matrix.h"
#include "graphics3d.resource.h"

namespace magnum { namespace graphics3d {
using magnum::container::Array;
using magnum::types::int32;
using magnum::types::uint32;
using magnum::math::Vector3;
using magnum::math::Matrix4;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
struct Weight {
	uint32				bone;
	float				weight;
	Vector3<float>		position;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class WeightBuffer : public Resource {
DeclareClass(WeightBuffer, Resource);
public:
	WeightBuffer();
	~WeightBuffer();
	
	void				insert(unsigned int ,const Vector3<float>& ,const Weight* );
	void				transform(Vector3<float>* ,const Matrix4<float>* );

	void				clear();
	void				finish();
	
	inline const void*	lock();
	inline void			unlock();

	inline unsigned int	getSize() const;
	inline unsigned int	getLength() const;

private:
	unsigned int		m_Size;
	Array<int32>		m_Buffer;
};

}; };

#endif
