/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRID_FILTER3D_SOBEL_INL
#define __GRID_FILTER3D_SOBEL_INL

namespace magnum { namespace grid {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Grid3d<Vector3<T> >* FilterSobel3d<T>::onProcess(const Grid3d<T>* gs) {
    static const T Mx[3][3][3] = {{{-2, -3, -2}, {-3, -6, -3}, {-2, -3, -2}},
                            {{ 0,  0,  0}, { 0,  0,  0}, { 0,  0,  0}},
                            {{ 2,  3,  2}, { 3,  6,  3}, { 2,  3,  2}}};
    static const T  My[3][3][3] = {{{-2, -3, -2}, { 0,  0,  0}, { 2,  3,  2}},
                            {{-3, -6, -3}, { 0,  0,  0}, { 3,  6,  3}},
                            {{-2, -3, -2}, { 0,  0,  0}, { 2,  3,  2}}};
    static const T  Mz[3][3][3] = {{{-2,  0,  2}, {-3,  0,  3}, {-2,  0,  2}},
                            {{-3,  0,  3}, {-6,  0,  6}, {-3,  0,  3}},
                            {{-2,  0,  2}, {-3,  0,  3}, {-2,  0,  2}}};

	Grid3d<Vector3<T> >* gd = new Grid3d<Vector3<T> >;
	gd->setSize(gs->getWidth(), gs->getHeight(), gs->getDepth());

    float maxmag = 0;
    for (int z = 0; z < gs->getDepth(); z++) {
		for (int y = 0; y < gs->getHeight(); y++) {
			for (int x = 0; x < gs->getWidth(); x++) {
                gx[index] = 0;
                gy[index] = 0;
                gz[index] = 0;
                for (int a=-1; a<=1; a++) {
                    for (int b=-1; b<=1; b++) {
                        for (int c=-1; c<=1; c++) {
                            float val;
                        
                            int ii = i+a;
                            int jj = j+b;
                            int kk = k+c;

                            ii = MAX(0, MIN(nx-1, ii));
                            jj = MAX(0, MIN(ny-1, jj));
                            kk = MAX(0, MIN(nz-1, kk));
                            val = opac->GetTuple1(CalculateIndex(grid,ii,jj,kk));

                            gx[index] += Mx[a+1][b+1][c+1] * val;
                            gy[index] += My[a+1][b+1][c+1] * val;
                            gz[index] += Mz[a+1][b+1][c+1] * val;
                        }
                    }
                }

                // Normalize the computed gradient
                float mag = sqrt(gx[index]*gx[index] + gy[index]*gy[index] + gz[index]*gz[index]);
                gm[index] = mag;
                gmn[index] = mag;
                if (mag>0) {
                    gx[index] /= mag;
                    gy[index] /= mag;
                    gz[index] /= mag;
                }

                if (mag > maxmag)
                    maxmag = mag;
            }
        }
    }
    if (maxmag > 0) {
        for (int n=0; n<nels; n++)
            gmn[n] /= maxmag;
    }
	
	return gd;
}


}; };

#endif
