/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GRID_GRID2D_INL
#define __GRID_GRID2D_INL

namespace magnum { namespace grid {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Grid2d<T>::Grid2d() {
	m_Width = m_Height = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Grid2d<T>::Grid2d(const T* data,unsigned int w,unsigned int h) {
	m_Data.copy(data,w*h);
	m_Width = w;
	m_Height = h;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Grid2d<T>::setSize(unsigned int w,unsigned int h) {
	m_Data.setSize(w*h);
	m_Width = w;
	m_Height = h;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline unsigned int	Grid2d<T>::getWidth() const {
	return m_Width;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline unsigned int	Grid2d<T>::getHeight() const {
	return m_Height;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline T* Grid2d<T>::operator[](unsigned int i) {
	return &m_Data[i*m_Width];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const T* Grid2d<T>::operator[](unsigned int i) const {
	return &m_Data[i*m_Width];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline T& Grid2d<T>::at(unsigned int i,unsigned int j) {
	ASSERT((i < m_Width) && (j < m_Height));
	return m_Data[j*m_Width+i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const T& Grid2d<T>::at(unsigned int i,unsigned int j) const {
	ASSERT((i < m_Width) && (j < m_Height));
	return m_Data[j*m_Width+i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> template<class S>
Grid2d<S>* Grid2d<T>::convert() const {
	Grid2d<S>* grid = new Grid2d<S>;
	grid->setSize(m_Width, m_Height);
	T* src = &m_Data[0];
	S* dst = &grid->m_Data[0];
	T* end = src + m_Height*m_Width;
	while(src < end)
		*dst++ = (S)*src++;
		
	return grid;
}


}; };

#endif
