/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/


#ifndef __GRID_GRID3D_INL
#define __GRID_GRID3D_INL

namespace magnum { namespace grid {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Grid3d<T>::Grid3d() {
	m_Width = m_Height = m_Depth = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Grid3d<T>::Grid3d(const T* data,unsigned int w,unsigned int h,unsigned int d) {
	m_Data.copy(data,w*h*d);
	m_Width = w;
	m_Height = h;
	m_Depth = d
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
void Grid3d<T>::setSize(unsigned int w,unsigned int h,unsigned int d) {
	m_Data.setSize(w*h*d);
	m_Width = w;
	m_Height = h;
	m_Depth = d;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline unsigned int	Grid3d<T>::getWidth() const {
	return m_Width;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline unsigned int	Grid3d<T>::getHeight() const {
	return m_Height;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline unsigned int	Grid3d<T>::getDepth() const {
	return m_Depth;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline T& Grid3d<T>::at(unsigned int x,unsigned int y,unsigned int z) {
	ASSERT((x < m_Width) && (y < m_Height) && (z < m_Depth));
	return m_Data[z*m_Width*m_Height+y*m_Width+x];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const T& Grid3d<T>::at(unsigned int x,unsigned int y,unsigned int z) const {
	ASSERT((x < m_Width) && (y < m_Height) && (z < m_Depth));
	return m_Data[z*m_Width*m_Height+y*m_Width+x];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> template<class S>
Grid3d<S>* Grid3d<T>::convert() const {
	Grid3d<S>* grid = new Grid3d<S>;
	grid->setSize(m_Width, m_Height, m_Depth);
	T* src = &m_Data[0];
	S* dst = &grid->m_Data[0];
	T* end = src + m_Height*m_Width*m_Depth;
	while(src < end)
		*dst++ = (S)*src++;
		
	return grid;
}


}; };

#endif
