/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GUI_CONTEXT_INL
#define __GUI_CONTEXT_INL

namespace magnum { namespace gui {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Colour<byte>& Context::getFillColour() const {
	return m_FillColour;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Colour<byte>& Context::getLineColour() const {
	return m_LineColour;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Context::getLineWidth() const {
	return m_LineWidth;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::fillRectangle(const Rectangle<float>& rect) {
#if defined(__FORWINDOWS__)
	::Rectangle(m_DeviceHandle, (int)rect.getX(), (int)rect.getY(), (int)rect.getX() + (int)rect.getWidth() - 1, (int)rect.getY() + (int)rect.getHeight() - 1);
#endif	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::drawRectangle(const Rectangle<float>& rect) {
#if defined(__FORWINDOWS__)
	POINT p[5];
	p[0].x = (int)rect.getX();
	p[0].y = (int)rect.getY();
	p[1].x = (int)rect.getX() + (int)rect.getWidth() - 1;
	p[1].y = (int)rect.getY();
	p[2].x = (int)rect.getX() + (int)rect.getWidth() - 1;
	p[2].y = (int)rect.getY() + (int)rect.getHeight() - 1;
	p[3].x = (int)rect.getX();
	p[3].y = (int)rect.getY() + (int)rect.getHeight() - 1;
	p[4].x = (int)rect.getX();
	p[4].y = (int)rect.getY();
	Polyline(m_DeviceHandle, p, 5);
#endif
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::drawPoint(const Point<float>& p) {
#if defined(__FORWINDOWS__)
	::SetPixel(m_DeviceHandle, (int)p.getX(), (int)p.getY(), m_LineColour.R);
#endif	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::drawLine(const Point<float>& p1, const Point<float>& p2) {
#if defined(__FORWINDOWS__)
	POINT p[2];
	p[0].x = (long)p1.getX();
	p[0].y = (long)p1.getY();
	p[1].x = (long)p2.getX();
	p[1].y = (long)p2.getY();
	::Polyline(m_DeviceHandle, p, 2);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::drawPolygon(const Polygon<float>& poly) {

}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::drawBitmap(const Point<float>& pos, const Bitmap* img) {
#if defined(__FORWINDOWS__)
	BitBlt(m_DeviceHandle, 
			(int)pos.getX(), (int)pos.getY(), img->getWidth(), img->getHeight(), 
			((Bitmap *)img)->m_Device, 0, 0, SRCCOPY);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::drawBitmap(const Point<float>& p, const Bitmap* img,const Rectangle<float>& r) {
#if defined(__FORWINDOWS__)
	BitBlt(m_DeviceHandle, 
			(int)p.getX(), (int)p.getY(), (int)r.getWidth(), (int)r.getHeight(), 
			((Bitmap *)img)->m_Device, (int)r.getX(), (int)r.getY(), SRCCOPY);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::drawBitmap(const Rectangle<float>& r, const Bitmap* img) {
#if defined(__FORWINDOWS__)
	BitBlt(m_DeviceHandle, 
			(int)r.getX(), (int)r.getY(), (int)r.getWidth(), (int)r.getHeight(), 
			((Bitmap *)img)->m_Device, 0, 0, SRCCOPY);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::drawBitmap(const Rectangle<float>& r, const Bitmap* img, const Point<float>& p) {
#if defined(__FORWINDOWS__)
	BitBlt(m_DeviceHandle, 
			(int)r.getX(), (int)r.getY(), (int)r.getWidth(), (int)r.getHeight(), 
			((Bitmap *)img)->m_Device, (int)p.getX(), (int)p.getY(), SRCCOPY);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Context::stretchBitmap(const Rectangle<float>& , const Bitmap* ,const Rectangle<float>& ) {

}


#if defined(__FORWINDOWS__)

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline HWND	Context::getWindowHandle() const {
	return m_WindowHandle;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline HDC	Context::getDeviceHandle() const {
	return m_DeviceHandle;
}


#elif defined(__FORX11__)

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Window Context::getWindowHandle() const {
	return m_WindowHandle;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Display* Context::getDisplayHandle() const {
	return m_Display;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline GC Context::getDeviceHandle() const {
	return m_DeviceHandle;
}


#endif


}; };
#endif
