/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GUI_VIEW_INL
#define __GUI_VIEW_INL

namespace magnum { namespace gui {
using namespace magnum::application;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Context* View::getContext() {
	return &m_Context;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Window* View::getWindow() const {
	return m_Window;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Container* View::getParent() const {
	return m_Parent;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void View::setName(const char* s) {
	m_Name = s;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String View::getName(void) const {
	return m_Name;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int View::getAlignement() const {
	return m_Alignement;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void View::setAlignement(int r) {
	m_Alignement = r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Rectangle<float> View::getBounds() const {
	return m_Bounds;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void View::setFrame(const Rectangle<float>& Frame) {
	m_Frame = Frame;
	updateFrame();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Rectangle<float> View::getFrame() const {
	return m_Frame;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void View::setSize(const Dimension<float>& size) {
	m_Frame.setSize(size);
	updateFrame();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> View::getSize() const {
	return m_Frame.getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void View::setLocation(const Point<float>& size) {
	m_Frame.setOrigin(size);
	updateFrame();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::getLocation() const {
	return m_Frame.getOrigin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline	Point<float> View::getScale() const {
	return m_Transform.getScale();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline	Point<float> View::getOrigin() const {
	return m_Transform.getTranslation();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::convertToParent(const Point<float>& p) const {
	return m_Transform * p + m_Frame.getOrigin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> View::convertToParent(const Dimension<float>& d) const {
	return m_Transform * d;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline  Rectangle<float> View::convertToParent(const Rectangle<float>& r) const {
	return (m_Transform * r) + m_Frame.getOrigin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::convertFromParent(const Point<float>& localPoint) const {
	return m_Transform % (localPoint - m_Frame.getOrigin());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> View::convertFromParent(const Dimension<float>& localDimension) const {
	return m_Transform % localDimension;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Rectangle<float> View::convertFromParent(const Rectangle<float>& localRect) const {
	return m_Transform % (localRect - m_Frame.getOrigin());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::convertToWindow(const Point<float>& p) const {
	return m_AbsTransform * p;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> View::convertToWindow(const Dimension<float>& localDimension) const {
	return m_AbsTransform * localDimension;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline  Rectangle<float> View::convertToWindow(const Rectangle<float>& localRect) const {
	return m_AbsTransform * localRect;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> View::convertFromWindow(const Point<float>& localPoint) const {
	return m_AbsTransform % localPoint;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> View::convertFromWindow(const Dimension<float>& localDimension) const {
	return m_AbsTransform % localDimension;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Rectangle<float> View::convertFromWindow(const Rectangle<float>& localRect) const {
	return m_AbsTransform % localRect;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool View::getFocus() {
	if (m_Window == NULL)
		return false;

	return m_Window->setFocus(this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool View::hasFocus() const {
	if (m_Window == NULL)
		return false;

	return (m_Window->getFocus() == this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool View::releaseFocus() {
	if (m_Window == NULL)
		return false;

	return m_Window->releaseFocus(this);
}



#if defined(__FORWINDOWS__)
/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline HWND View::getWindowHandle() const {
	return m_WindowHandle;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline HDC View::getDeviceHandle() const {
	return m_DeviceHandle;
}
#endif

}; };
#endif
