/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GUI_WINDOW_INL
#define __GUI_WINDOW_INL

namespace magnum { namespace gui {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Window::isOpen() const {
	return m_Open;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> Window::getSize(void) const {
	return m_Frame.getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Point<float> Window::getLocation(void) const {
	return m_Frame.getOrigin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String Window::getName(void) const {
	return m_Title;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Window::setMinimumSize(const Dimension<float>& min) {
	m_MinSize = min;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> Window::getMinimumSize(void) const {
	return m_MinSize;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Window::setMaximumSize(const Dimension<float>& max) {
	m_MaxSize = max;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Dimension<float> Window::getMaximumSize(void) const {
	return m_MaxSize;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline View* Window::getFocus() const {
	return m_Focus;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Window::remove(const Window::Iterator& iter) {
	return remove(&(*iter));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Window::Iterator Window::begin() const {
	return m_Views.begin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Window::Iterator Window::end() const {
	return m_Views.end();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Window::Iterator Window::at(View* v) const {
	return m_Views.at(v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Window::insertHandler(Handler* hnd) {
	return Looper::insert(hnd);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Window::removeHandler(Handler* hnd) {
	return Looper::remove(hnd);
}


#if defined(__FORWINDOWS__)
/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline HWND Window::getWindowHandle() const {
	return m_WindowHandle;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline HDC Window::getDeviceHandle() const {
	return m_DeviceHandle;
}
#endif

}; };
#endif
