/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GUI3D_BUTTON_H
#define __GUI3D_BUTTON_H

#include "gui3d.control.h"
#include "freetype.font.h"
#include "object.pointer.h"
#include "graphics.colour.h"

namespace magnum { namespace gui3d {
using magnum::freetype::Font;
using magnum::object::Pointer;
using magnum::graphics::Colour;

class FontMap;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Button : public Control {
DeclareClass(Button, Control);
public:
	Button(const Rectangle<float>& ,const Message& );
	Button(const Rectangle<float>& ,const Message& ,const char* );

	inline void			setText(const char* );
	inline const String& getText() const;
	
	inline void			setFont(const Font* font);
	inline const Font*	getFont();
	
	inline void			setTextColour(const Colour<float> col);
	inline const Colour<float>& getTextColour() const;
	
	inline bool			isPressed() const;
	
protected:
	virtual bool		onPaint();
	virtual bool		onMouseEnter(const Point<float>&, Mouse::Button );
	virtual bool		onMouseMove(const Point<float>&, Mouse::Button );
	virtual bool		onMouseButton(const Point<float>&, Mouse::Button );
	virtual bool		onMouseLeave();
	
	// Enable/Disable
	virtual bool		onEnable();
	virtual bool		onDisable();

	// Button specific callbacks
	virtual bool		onPush();
	virtual bool		onRelease();
	
private:
	void				init();

private:
	enum State {
		NormalState,
		HoveringState,
		PressedState,
		DisabledState
	};
	
	State				m_State;
	String				m_Text;
	Pointer<Font>		m_Font;
	Pointer<FontMap>	m_FontMap;
	Rectangle<float>	m_TextExtents;
	Colour<float>		m_TextColour;
	Colour<float>		m_BackColour;
	Colour<float>		m_BorderColour;
	Colour<float>		m_HoverTextColour;
	Colour<float>		m_HoverBackColour;
	Colour<float>		m_HoverBorderColour;
	Colour<float>		m_PressedTextColour;
	Colour<float>		m_PressedBackColour;
	Colour<float>		m_PressedBorderColour;
	Colour<float>		m_DisabledTextColour;
	Colour<float>		m_DisabledBackColour;
	Colour<float>		m_DisabledBorderColour;};

}; };

#endif
