/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GUI3D_CONTAINER_H
#define __GUI3D_CONTAINER_H

#include "gui3d.view.h"

namespace magnum { namespace gui3d {
using magnum::container::Chain;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Container : public View {
DeclareClass(Container, View);
public:
	typedef magnum::container::Chain<View>::Iterator	Iterator;
	
public:
	Container(const Rectangle<float>& );
	
	bool				insert(View* view);
	bool				remove(View *view);
	bool				clear();

	inline Iterator		begin() const;
	inline Iterator		at(View* ) const;
	inline Iterator		end() const;

protected:				
	virtual bool		onPaint();
						
	// Window management related methods
	virtual bool		onAttach();
	virtual bool		onDetach();
	virtual bool		onActivate();
	virtual bool		onDeactivate();

	// Enable/Disable
	virtual bool		onEnable();
	virtual bool		onDisable();

	// Size related methods
	virtual Dimension<float> onGetPreferredSize() const;
	virtual bool			onSetPreferredSize(void);
	virtual bool		onResize(const Rectangle<float>& );
						
	// Mouse related methods
	virtual bool		onMouseMove(const Point<float>& ,Mouse::Button );
	virtual bool		onMouseButton(const Point<float>& ,Mouse::Button );
	virtual bool		onMouseEnter(const Point<float>& ,Mouse::Button );
	virtual bool		onMouseLeave();
	
private:
	virtual void		updateFrame();
	virtual void		updateTransform();

private:
	Chain<View>			m_Views;
	View*				m_MouseView;
	Rectangle<float>	m_OldFrame;
};

}; };

#endif
