/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GUI3D_ROOT_H
#define __GUI3D_ROOT_H

#include "gui3d.context.h"
#include "graphics.dimension.h"
#include "graphics.point.h"
#include "input.keyboard.h"
#include "input.mouse.h"
#include "input.character.h"
#include "application.messenger.h"
#include "application.messagequeue.h"
#include "system.mutex.h"

namespace magnum { namespace gui3d {
using magnum::object::Object;
using magnum::types::String;
using magnum::system::Mutex;
using magnum::container::Chain;
using magnum::graphics::Point;
using magnum::graphics::Dimension;
using magnum::application::Messenger;
using magnum::application::Message;
using magnum::application::MessageQueue;
using magnum::application::Handler;
using magnum::input::Mouse;
using magnum::input::Keyboard;
using magnum::input::Character;

class View;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Root : public Messenger {
DeclareClass(Root, Messenger);
friend class View;
private:
	String					m_Name;
	Dimension<float>		m_Size;
	Dimension<float>		m_OldSize;
	Context					m_Context;
	Chain<View>				m_Views;
	View*					m_Focus;
	View*					m_MouseView;
	int						m_KeyboardModifiers;
	MessageQueue			m_MessageQueue;
	Mutex					m_MessageMutex;
#if defined(__FORWINDOWS__)
	HWND					m_WindowHandle;
	HDC						m_DeviceContext;
	WNDPROC					m_WindowProc;
	bool					m_MouseTracking;
#endif
	
	
public:
	typedef magnum::container::Chain<View>::Iterator	Iterator;

public:
	Root(magnum::graphics3d::Context* ,bool di = false);
	~Root();
	
    inline void         	setName(const char* );
    inline String       	getName(void) const;

	bool					insert(View* view);
	bool					remove(View *view);
	bool					remove(const Iterator& iter);
	bool					clear();

	inline Iterator			begin() const;
	inline Iterator			at(View* view) const;
	inline Iterator			end() const;
	
	inline bool				insertHandler(Handler* );
	inline bool				removeHandler(Handler* );
	
	void					attachDirectInput();
	void					detachDirectInput();
	bool					getDirectInput() const;

	void					setSize(const Dimension<float>& dim);
	inline Dimension<float> getSize(void) const;

	void					process();
	void					paint();
	
	inline void				invalidate();
	
	inline View*			getFocus() const;
	bool					setFocus(View* focus);
	bool					releaseFocus(View* focus);
	bool					releaseFocus();
	
	
protected:
	// Window management related callbacks
	virtual bool			onResize(const Dimension<float>& );
    virtual bool			onActivate(void);
	virtual bool			onDeactivate(void);

	// Leyboard specific callbacks
	virtual bool			onKeyDown(Keyboard::Key );
	virtual bool			onKeyUp(Keyboard::Key );
	virtual bool			onChar(Character::Code ,Character::Modifier );

	// Mouse specific callbacks
	virtual bool			onMouseMove(const Point<float>& ,Mouse::Button );
	virtual bool			onMouseButton(const Point<float>& ,Mouse::Button );
	virtual bool			onMouseWheel(float ,Mouse::Button );
	virtual bool			onMouseEnter(const Point<float>& ,Mouse::Button );
	virtual bool			onMouseLeave();

   
private:
	virtual bool			onSendMessage(Message* );
	virtual bool			onPostMessage(Message* );
	virtual bool			onSynchronize();
	
#if defined(__FORWINDOWS__)
	static LRESULT CALLBACK stdWindowProc(HWND hwnd, UINT Message,WPARAM wParam, LPARAM lParam);
	static LRESULT CALLBACK inputWindowProc(HWND hwnd, UINT Message,WPARAM wParam, LPARAM lParam);
#endif
};

}; };

#endif
