/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __GUI3D_SLIDER_H
#define __GUI3D_SLIDER_H

#include "gui3d.control.h"
#include "graphics.colour.h"
#include "math.range.h"

namespace magnum { namespace gui3d {
using magnum::graphics::Colour;
using magnum::math::Range;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Slider : public Control {
DeclareClass(Slider, Control);
private:
	enum State {
		NormalState,
		HoveringState,
		PressedState,
		DisabledState
	};
	
	Range<float>		m_Range;
	float				m_Step;
	float				m_Value;
	State				m_State;
	Colour<float>		m_BorderColour;
	Colour<float>		m_SliderColour;
	Colour<float>		m_HoverBorderColour;
	Colour<float>		m_HoverSliderColour;
	Colour<float>		m_PressedBorderColour;
	Colour<float>		m_PressedSliderColour;
	Colour<float>		m_DisabledBorderColour;
	Colour<float>		m_DisabledSliderColour;

public:
	Slider(const Rectangle<float>& ,const Message& );
	Slider(const Rectangle<float>& ,const Message& ,const Range<float>& );

	inline void			setRange(const Range<float>& );
	inline void			setRange(float mi, float ma);
	inline const Range<float>& getRange() const;
	
	inline void			setStepsize(float );
	inline float		getStepsize() const;
	
	inline void			setValue(float );
	inline float		getValue() const;

	inline bool			isPressed() const;
	
protected:
	virtual bool		onPaint();
	virtual bool		onMouseEnter(const Point<float>&, Mouse::Button );
	virtual bool		onMouseMove(const Point<float>&, Mouse::Button );
	virtual bool		onMouseButton(const Point<float>&, Mouse::Button );
	virtual bool		onMouseLeave();
	
	// Enable/Disable
	virtual bool		onEnable();
	virtual bool		onDisable();

	// Slider specific callbacks
	virtual bool		onPush();
	virtual bool		onRelease();
	
private:
	void				init();
};

}; };

#endif
