/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT_GLOVE_H
#define __INPUT_GLOVE_H

#include "application.listener.h"
#include "input.body.h"


namespace magnum { namespace input {
using magnum::application::Listener;
using magnum::application::Dispatcher;

class GloveListener;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Glove : public Body, private Chainable<Glove> {
DeclareClass(FlightStick, Body);
public:
	enum Button {
		Button0Down		=	0x00000001,
		Button1Down		=	0x00000002,
		Button2Down		=	0x00000004,
		Button3Down		=	0x00000008,
		Button4Down		=	0x00000010,
		Button5Down		=	0x00000020,
		Button6Down		=	0x00000040,
		Button7Down		=	0x00000080,
		Button8Down		=	0x00000100,
		Button9Down		=	0x00000200,
		Button10Down	=	0x00000400,
		Button11Down	=	0x00000800,
		Button12Down	=	0x00001000,
		Button13Down	=	0x00002000,
		Button14Down	=	0x00004000,
		Button15Down	=	0x00008000,
		Button0Up		=	0x00010000,
		Button1Up		=	0x00020000,
		Button2Up		=	0x00040000,
		Button3Up		=	0x00080000,
		Button4Up		=	0x00100000,
		Button5Up		=	0x00200000,
		Button6Up		=	0x00400000,
		Button7Up		=	0x00800000,
		Button8Up		=	0x01000000,
		Button9Up		=	0x02000000,
		Button10Up		=	0x04000000,
		Button11Up		=	0x08000000,
		Button12Up		=	0x10000000,
		Button13Up		=	0x20000000,
		Button14Up		=	0x40000000,
		Button15Up		=	0x80000000,
	};
	
	enum Finger {
		Thumb			= 0,
		Index			= 1,
		Middle			= 2,
		Ring			= 3,
		Pinky			= 4
	};

protected:
	inline Glove();
	inline explicit Glove(const char* );
	inline explicit Glove(const String& );
	explicit Glove(Space* );
	Glove(Space* ,const char* );
	Glove(Space* ,const String& );
	
public:
	inline void		insertListener(GloveListener* );
	inline void		removeListener(GloveListener* );
	inline void		clearListeners();
	
	inline float	getFinger(Finger ) const;

protected:
	void			fireConnect();
	void			fireDisconnect();
	void			fireMove(const Matrix4<float>& ,Glove::Button );
	void			fireButton(const Matrix4<float>& ,Glove::Button );
	
	inline void		updateFinger(Finger ,float );
	
private:
	Dispatcher<GloveListener> m_Dispatcher;
	float			m_Fingers[5];
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class GloveListener : public Listener<GloveListener> {
friend class Glove;
protected:
	virtual void	onGloveConnect();
	virtual void	onGloveDisconnect();
	virtual void	onGloveMove(const Matrix4<float>& ,Glove::Button );
	virtual void	onGloveButton(const Matrix4<float>& ,Glove::Button );
};

}; };

#endif

