/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT_KEYBOARD_H
#define __INPUT_KEYBOARD_H

#include "application.listener.h"
#include "input.character.h"
#include "input.device.h"

namespace magnum { namespace input {
using magnum::application::Listener;
using magnum::application::Dispatcher;

class KeyboardListener;
class KeyboardDispatcher;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Keyboard : public Device {
DeclareClass(Keyboard, Device);
public:
#if defined(__FORWINDOWS__)
	enum Key {
		Back		=		VK_BACK,
		Tab			=		VK_TAB,
		Clear		=		VK_CLEAR,
		Return		=		VK_RETURN,
					
		Pause		=		VK_PAUSE,
		Escape 		=		VK_ESCAPE,
		Space		=		VK_SPACE,
					
		Prior 		= 		VK_PRIOR,
		Next		=		VK_NEXT,
		End			=		VK_END,
		Home		=		VK_HOME,
		Left		=		VK_LEFT,
		Up			=		VK_UP,
		Right		=		VK_RIGHT,
		Down		=		VK_DOWN,
					
		Select		= 		VK_SELECT,
		Print		=		VK_PRINT,
		Snapshot	=		VK_SNAPSHOT,
		Process		=		VK_PROCESSKEY,
		Execute		=		VK_EXECUTE,
		Cancel		=		VK_CANCEL,
					
		Insert		= 		VK_INSERT,
		Delete		= 		VK_DELETE,
		Help		=		VK_HELP,
		Numpad0		=		VK_NUMPAD0,
		Numpad1		=		VK_NUMPAD1,
		Numpad2		=		VK_NUMPAD2,
		Numpad3		=		VK_NUMPAD3,
		Numpad4		=		VK_NUMPAD4,
		Numpad5		=		VK_NUMPAD5,
		Numpad6		=		VK_NUMPAD6,
		Numpad7		=		VK_NUMPAD7,
		Numpad8		=		VK_NUMPAD8,
		Numpad9		=		VK_NUMPAD9,
		Multiply	=		VK_MULTIPLY,
		Add			=		VK_ADD,
		Separator	=		VK_SEPARATOR,
		Subtract	=		VK_SUBTRACT,
		Decimal		=		VK_DECIMAL,
		Divide		=		VK_DIVIDE,
					
		F1			= 		VK_F1,
		F2			= 		VK_F2,
		F3			= 		VK_F3,
		F4			= 		VK_F4,
		F5			= 		VK_F5,
		F6			= 		VK_F6,
		F7			= 		VK_F7,
		F8			= 		VK_F8,
		F9			= 		VK_F9,
		F10			=		VK_F10,
		F11			=		VK_F11,
		F12			=		VK_F12,
		F13			=		VK_F13,
		F14			=		VK_F14,
		F15			=		VK_F15,
		F16			=		VK_F16,
		F17			=		VK_F17,
		F18			=		VK_F18,
		F19			=		VK_F19,
		F20			=		VK_F20,
		F21			=		VK_F21,
		F22			=		VK_F22,
		F23			=		VK_F23,
		F24			=		VK_F24,

		NumLock		=		VK_NUMLOCK,
		ScrollLock	= 		VK_SCROLL,
		CapsLock	=		VK_CAPITAL,
		ShiftLock	=		VK_CAPITAL,
		Shift		=		VK_SHIFT,
		ShiftLeft	= 		VK_LSHIFT,
		ShiftRight	= 		VK_RSHIFT,
		Control		=		VK_CONTROL,
		ControlLeft	=		VK_LCONTROL,
		ControlRight=		VK_RCONTROL,
		Alt			=		VK_MENU,
		AltLeft 	=		VK_LMENU,
		AltRight    =     	VK_RMENU,
		Meta		=		VK_APPS,
		MetaLeft	=		VK_LWIN,
		MetaRight	=		VK_RWIN
	};
#elif defined(__FORX11__)
	enum Key {
		Back		=		XK_BackSpace,
		Tab			=		XK_Tab,
		Clear		=		XK_Clear,
		Return		=		XK_Return,
					
		Pause		=		XK_Pause,
		Escape 		=		XK_Escape,
		Space		=		XK_space,
		Prior 		= 		XK_Prior,
		Next		=		XK_Next,
		End			=		XK_End,
		Home		=		XK_Home,
		Left		=		XK_Left,
		Up			=		XK_Up,
		Right		=		XK_Right,
		Down		=		XK_Down,
					
		Select		= 		XK_Select,
		Print		=		XK_Print,
//		Snapshot	=		VK_SNAPSHOT,
		Process		=		XK_Sys_Req,
		Execute		=		XK_Execute,
		Cancel		=		XK_Cancel,
					
		Insert		= 		XK_Insert,
		Delete		= 		XK_Delete,
		Help		=		XK_Help,
					
		Numpad0		=		XK_KP_0,
		Numpad1		=		XK_KP_1,
		Numpad2		=		XK_KP_2,
		Numpad3		=		XK_KP_3,
		Numpad4		=		XK_KP_4,
		Numpad5		=		XK_KP_5,
		Numpad6		=		XK_KP_6,
		Numpad7		=		XK_KP_7,
		Numpad8		=		XK_KP_8,
		Numpad9		=		XK_KP_9,
		Multiply	=		XK_KP_Multiply,
		Add			=		XK_KP_Add,
		Separator	=		XK_KP_Separator,
		Subtract	=		XK_KP_Subtract,
		Decimal		=		XK_KP_Decimal,
		Divide		=		XK_KP_Divide,
					
		F1			= 		XK_F1,
		F2			= 		XK_F2,
		F3			= 		XK_F3,
		F4			= 		XK_F4,
		F5			= 		XK_F5,
		F6			= 		XK_F6,
		F7			= 		XK_F7,
		F8			= 		XK_F8,
		F9			= 		XK_F9,
		F10			=		XK_F10,
		F11			=		XK_F11,
		F12			=		XK_F12,
		F13			=		XK_F13,
		F14			=		XK_F14,
		F15			=		XK_F15,
		F16			=		XK_F16,
		F17			=		XK_F17,
		F18			=		XK_F18,
		F19			=		XK_F19,
		F20			=		XK_F20,
		F21			=		XK_F21,
		F22			=		XK_F22,
		F23			=		XK_F23,
		F24			=		XK_F24,

		NumLock		=		XK_Num_Lock,
		ScrollLock	= 		XK_Scroll_Lock,
		CapsLock	=		XK_Caps_Lock,
		ShiftLock	=		XK_Shift_Lock,
//		Shift		=		VK_SHIFT,
		ShiftLeft	= 		XK_Shift_L,
		ShiftRight	= 		XK_Shift_R,
//		Control		=		VK_CONTROL,
		ControlLeft	=		XK_Control_L,
		ControlRight=		XK_Control_R,
//		Alt			=		VK_MENU,
		AltLeft 	=		XK_Alt_L,
		AltRight    =     	XK_Alt_R,
//		Meta		=		XK_Meta,
		MetaLeft	=		XK_Meta_L,
		MetaRight	=		XK_Meta_R
	};
#endif

public:
	inline void		insertListener(KeyboardListener* );
	inline void		removeListener(KeyboardListener* );
	inline void		clear();

protected:
	inline Dispatcher<KeyboardListener>& getDispatcher();
	inline const Dispatcher<KeyboardListener>& getDispatcher() const;

private:
	Dispatcher<KeyboardListener> m_Dispatcher;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class KeyboardListener : public Listener<KeyboardListener> {
friend class KeyboardDispatcher;
protected:
	virtual void	onKeyDown(Keyboard::Key );
	virtual void	onKeyUp(Keyboard::Key );
	virtual void	onChar(Character::Code ,Character::Modifier );
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class KeyboardDispatcher : public Dispatcher<KeyboardListener> {
DeclareClass(KeyboardDispatcher, Dispatcher<KeyboardListener>);
public:
	void			fireKeyDown(Keyboard::Key );
	void			fireKeyUp(Keyboard::Key );
	void			fireChar(Character::Code ,Character::Modifier );
};

}; };

#endif
