/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT_MOUSE_H
#define __INPUT_MOUSE_H

#include "application.listener.h"
#include "graphics.point.h"
#include "graphics.rectangle.h"
#include "input.device.h"

namespace magnum { namespace input {
using magnum::application::Listener;
using magnum::application::Dispatcher;
using magnum::graphics::Point;
using magnum::graphics::Rectangle;

class MouseListener;
class MouseDispatcher;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Mouse : public Device {
DeclareClass(Mouse, Device);
public:
	enum Modifier {
		Shift			=   0x00003,
		ShiftLeft		= 	0x00001,
		ShiftRight		= 	0x00002,
		Control     	=   0x0000c,
		ControlLeft		=	0x00004,
		ControlRight	=	0x00008,
		Alt				=	0x00030,
		AltLeft 		=	0x00010,
		AltRight    	=   0x00020,
		Meta			=	0x000c0,
		MetaLeft		=	0x00040,
		MetaRight		=	0x00080
	};
	enum Button {
		LeftDown		=	0x01000,
		MiddleDown		=	0x02000,
		RightDown		=	0x04000,
		AnyDown			=	0x07000,
		LeftUp			=	0x10000,
		MiddleUp		=	0x20000,
		RightUp			=	0x40000
	};
	
public:
	inline void		insertListener(MouseListener* );
	inline void		removeListener(MouseListener* );
	inline void		clear();

protected:
	inline Dispatcher<MouseListener>& getDispatcher();
	inline const Dispatcher<MouseListener>& getDispatcher() const;
	
private:
	Dispatcher<MouseListener> m_Dispatcher;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class MouseListener : public Listener<MouseListener> {
friend class MouseDispatcher;
protected:
	virtual void	onMouseArea(const Rectangle<float>& );
	virtual void	onMouseMove(const Point<float>& ,Mouse::Button );
	virtual void	onMouseButton(const Point<float>& ,Mouse::Button );
	virtual void	onMouseWheel(float );
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class MouseDispatcher : public Dispatcher<MouseListener> {
DeclareClass(MouseDispatcher, Dispatcher<MouseListener>);
public:
	void			fireArea(const Rectangle<float>& );
	void			fireMove(const Point<float>& ,Mouse::Button );
	void			fireButton(const Point<float>& ,Mouse::Button );
	void			fireWheel(float );
};

}; };

#endif
