/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT_SPACE_H
#define __INPUT_SPACE_H

#include "math.matrix.h"
#include "math.vector3.h"
#include "container.chain.h"

namespace magnum { namespace input {
using magnum::object::Object;
using magnum::types::String;
using magnum::math::Matrix;
using magnum::math::Matrix4;
using magnum::math::Vector3;
using magnum::container::Chain;

class Marker;
class Body;
class Glove;

/*--------------------------------------------------------------------------*/
/**
 */
class Space : public Object, private magnum::object::Noncopyable {
DeclareClass(Space, Object);
friend Marker;
friend Body;
friend Glove;
public:
	Space();
	
	inline void		setCalibration(const Matrix<float,4>& );
	inline const Matrix4<float>& getCalibration() const;

	inline const Vector3<float>& getMinimum() const;
	inline const Vector3<float>& getMaximum() const;
	
	inline const Chain<Marker>& getMarkers() const;
	inline const Chain<Body>& getBodies() const;
	inline const Chain<Glove>& getGloves() const;

protected:
	inline void		setMinimum(const Vector3<float>& );
	inline void		setMaximum(const Vector3<float>& );
	
private:
	Matrix4<float>	m_Calibration;
	Vector3<float>	m_Minimum;
	Vector3<float>	m_Maximum;
	
	Chain<Marker>	m_Markers;
	Chain<Body>		m_Bodies;
	Chain<Glove>	m_Gloves;
};

}; };

#endif
