/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT3D_CONTROLLER_H
#define __INPUT3D_CONTROLLER_H

#include "object.h"
#include "object.pointer.h"
#include "geometry.transform.h"
#include "geometry.transformconstraints.h"

namespace magnum { namespace input3d {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::math::Matrix;
using magnum::math::Vector3;
using magnum::geometry::Transform;
using magnum::geometry::Transform3;
using magnum::geometry::TransformConstraint;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Controller : public Object {
DeclareClass(Controller, Object);
private:
	Pointer<Controller>		m_Next;
	Pointer<TransformConstraint<float,3> > m_Constraint;
	Transform3<float>		m_Transform;

protected:
	inline Controller();
	
public:
	inline const Transform3<float>&	getTransform() const;
	inline void				setTransform(const Transform<float,3>& );

	inline void				attachConstraint(TransformConstraint<float,3>* );
	inline void				detachConstraint();
	inline TransformConstraint<float,3>* getConstraint() const;

	inline void				attachNext(Controller* );
	inline void				detachNext();
	inline Controller*		getNext() const;
	
	void					update(float timestep = 1.0f);
	void					reset();
	
protected:
	inline Transform3<float>&	get();
	inline const Transform3<float>&	get() const;
	inline void				set(const Transform<float,3>& );
	inline void				set(const Matrix<float,4>& );
	inline void				mult(const Transform<float,3>& );
	inline void				mult(const Matrix<float,4>& );
	inline void				rmult(const Transform<float,3>& );
	inline void				rmult(const Matrix<float,4>& );
	inline void				identity();
	inline void				rotate(float ,const Vector3<float>& );
	inline void				translate(const Vector3<float>& );
	inline void				rrotate(float ,const Vector3<float>& );
	inline void				rtranslate(const Vector3<float>& );
	inline void				scale(const Vector3<float>& );
	inline void				scale(float );
	inline Vector3<float>	getAxis(unsigned int );
	inline void				setAxis(unsigned int ,const Vector3<float>& );

	virtual void			onReset();
	virtual void			onUpdate(float );
	virtual void			onModify();
};

}; };

#endif
