/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT3D_CONTROLLER_INL
#define __INPUT3D_CONTROLLER_INL

namespace magnum { namespace input3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Controller::Controller() {
	m_Transform.identity();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::attachConstraint(TransformConstraint<float,3>* tc) {
	m_Constraint = tc;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::detachConstraint() {
	m_Constraint.setNull();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline TransformConstraint<float,3>* Controller::getConstraint() const {
	return m_Constraint;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::attachNext(Controller* c) {
	ASSERT(c != NULL);
	m_Next = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::detachNext() {
	m_Next.setNull();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Controller* Controller::getNext() const {
	return m_Next;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Transform3<float>& Controller::getTransform() const {
	return m_Transform;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::setTransform(const Transform<float,3>& t) {
	m_Transform = t;
	onModify();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Transform3<float>& Controller::get() {
	return m_Transform;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Transform3<float>& Controller::get() const {
	return m_Transform;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::set(const Transform<float,3>& t) {
	m_Transform = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::set(const Matrix<float,4>& t) {
	m_Transform = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::mult(const Transform<float,3>& t) {
	m_Transform.mult(t);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::mult(const Matrix<float,4>& m) {
	m_Transform.mult(m);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::rmult(const Transform<float,3>& t) {
	m_Transform.rmult(t);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::rmult(const Matrix<float,4>& m) {
	m_Transform.rmult(m);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::identity() {
	m_Transform.identity();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::rotate(float a,const Vector3<float>& v) {
	m_Transform.rotate(a,v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::translate(const Vector3<float>& v) {
	m_Transform.translate(v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::rrotate(float a,const Vector3<float>& v) {
	m_Transform.rrotate(a,v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::rtranslate(const Vector3<float>& v) {
	m_Transform.rtranslate(v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::scale(const Vector3<float>& v) {
	m_Transform.scale(v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::scale(float s) {
	m_Transform.scale(s);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> Controller::getAxis(unsigned int i) {
	return m_Transform.getAxis(i);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Controller::setAxis(unsigned int i,const Vector3<float>& v) {
	m_Transform.setAxis(i,v);
}

}; };

#endif
