/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT3D_FIXEDCONTROLLER_H
#define __INPUT3D_FIXEDCONTROLLER_H

#include "input3d.controller.h"
#include "math.vector3.h"

namespace magnum { namespace input3d {
using magnum::math::Matrix;
using magnum::math::Vector3;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class FixedController : public Controller {
DeclareClass(FixedController, Controller);
private:
	Transform3<float>		m_Transform;
	
public:
	inline FixedController();
	inline FixedController(const Transform<float,3>& );
	inline FixedController(const Matrix<float,4>& );
	
	inline void				setTransform(const Transform<float,3>& );
	inline void				setTransform(const Matrix<float,4>& );
	inline const Transform3<float>&	getTransform() const;

	inline void				setTranslation(const Vector3<float>& );
	inline Vector3<float>	getTranslation() const;
	
	inline void				setRotation(const Matrix<float,3>& );
	inline Matrix<float,3>	getRotation() const;

	inline void				set(const Vector3<float>& from, const Vector3<float>& at, const Vector3<float>& up);
	inline void				setFrom(const Vector3<float>& );
	inline void				setAt(const Vector3<float>& );
	inline void				setDirection(const Vector3<float>& );
	inline void				setUp(const Vector3<float>& );
	
	inline Vector3<float> 	getFrom() const;
	inline Vector3<float> 	getAt() const;
	inline Vector3<float> 	getDirection() const;
	inline Vector3<float> 	getUp() const;
	
	inline void				identity();
	
	inline void				translate(const Vector3<float>& );
	inline void				rotate(float ,const Vector3<float>& );
	inline void				scale(const Vector3<float>& );
	inline void				scale(float );
	inline void				mult(const Transform<float,3>& );
	inline void				mult(const Matrix<float,4>& );
	inline void				mult(const Matrix<float,3>& );
	
protected:
	virtual void			onUpdate(float );
	virtual void			onModify();
	virtual void			onReset();
};

}; };

#endif
