/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT3D_FIXEDCONTROLLER_INL
#define __INPUT3D_FIXEDCONTROLLER_INL

namespace magnum { namespace input3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline FixedController::FixedController() {
	m_Transform.identity();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline FixedController::FixedController(const Transform<float,3>& t) : m_Transform(t) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline FixedController::FixedController(const Matrix<float,4>& m) : m_Transform(m) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::identity() {
	m_Transform.identity();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::setTranslation(const Vector3<float>& v) {
	m_Transform.setTranslation(v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> FixedController::getTranslation() const {
	return Vector3<float>(m_Transform.getTranslation());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::setRotation(const Matrix<float,3>& m) {
	m_Transform.setRotation(m);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Matrix<float,3> FixedController::getRotation() const {
	return m_Transform.getRotation();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> FixedController::getFrom() const {
	return m_Transform.getTranslation();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> FixedController::getDirection() const {
	return -m_Transform.getAxis(2);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> FixedController::getAt() const {
	return m_Transform.getTranslation() - m_Transform.getAxis(2);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> FixedController::getUp() const {
	return m_Transform.getAxis(1);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::setFrom(const Vector3<float>& from) {
	Vector3<float> n = from - getAt();
	n.normalise();
	m_Transform.setAxis(2,n);
	m_Transform.setAxis(0,getUp() % n);
	m_Transform.setTranslation(from);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::setAt(const Vector3<float>& at) {
	Vector3<float> n = getFrom() - at;
	n.normalise();
	m_Transform.setAxis(2,n);
	m_Transform.setAxis(0,(Vector3<float>)m_Transform.getAxis(1) % n);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::setUp(const Vector3<float>& up) {
	m_Transform.setAxis(1,up);
	m_Transform.setAxis(0,up % (Vector3<float>)m_Transform.getAxis(2));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::setDirection(const Vector3<float>& dir) {
	m_Transform.setAxis(2,-dir);
	m_Transform.setAxis(0,(Vector3<float>)m_Transform.getAxis(1) % (-dir));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::set(const Vector3<float>& from, const Vector3<float>& at, const Vector3<float>& up) {
	Vector3<float> n = from - at;
	n.normalise();
	m_Transform.setAxis(2,n);
	m_Transform.setAxis(1,up);
	m_Transform.setAxis(0,up % n);
	m_Transform.setTranslation(from);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::setTransform(const Transform<float,3>& t) {
	m_Transform = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::setTransform(const Matrix<float,4>& m) {
	m_Transform = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Transform3<float>& FixedController::getTransform() const {
	return m_Transform;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::translate(const Vector3<float>& v) {
	m_Transform.translate(v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::rotate(float a,const Vector3<float>& v) {
	m_Transform.rotate(a,v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::scale(const Vector3<float>& v) {
	m_Transform.scale(v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::scale(float s) {
	m_Transform.scale(s);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::mult(const Transform<float,3>& t) {
	m_Transform *= t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::mult(const Matrix<float,4>& m) {
	m_Transform *= m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void FixedController::mult(const Matrix<float,3>& m) {
	m_Transform *= m;
}

}; };

#endif
