/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT3D_FLIGHTCONTROLLER_H
#define __INPUT3D_FLIGHTCONTROLLER_H

#include "input.mouse.h"
#include "input.keyboard.h"
#include "input3d.controller.h"

namespace magnum { namespace input3d {
using magnum::graphics::Point;
using magnum::graphics::Dimension;
using magnum::graphics::Rectangle;
using magnum::input::Mouse;
using magnum::input::MouseListener;
using magnum::input::Keyboard;
using magnum::input::KeyboardListener;
using magnum::input::Character;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class FlightController : public Controller, public KeyboardListener, public MouseListener {
DeclareClass(FlightController, Controller);
private:
    Point<float>		m_Position;
    Point<float>		m_Center;
    Dimension<float>	m_Size;

	float				m_Speed;
	Vector3<float>		m_Up;
	Vector3<float>		m_Direction;
	Vector3<float>		m_Right;
	float				m_Motion;	

public:
	FlightController();
	
    inline void			setSpeed(float );
    inline float		getSpeed() const;
    
    inline void			setUp(const Vector3<float>& );
    inline const Vector3<float>& getUp() const;

    inline void			setDirection(const Vector3<float>& );
    inline const Vector3<float>& getDirection() const;

protected:
	virtual void		onMouseArea(const Rectangle<float>& );
	virtual void		onMouseMove(const Point<float>& ,Mouse::Button );
	virtual void		onMouseButton(const Point<float>& ,Mouse::Button );
	virtual void		onMouseWheel(float );

	virtual void		onChar(Character::Code ,Character::Modifier );
	virtual void		onKeyDown(Keyboard::Key );
	virtual void		onKeyUp(Keyboard::Key );

	virtual void		onReset();
	virtual void		onUpdate(float );
	virtual void		onModify();
	
private:
    void				rotate(float deltaX, float deltaY);
    void				forward(float step);
    void				right(float step);
};

}; };

#endif
