/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT3D_NAVIGATOR_H
#define __INPUT3D_NAVIGATOR_H

#include "object.h"
#include "object.pointer.h"
#include "geometry.transform.h"
#include "geometry.transformconstraints.h"
#include "math.vector3.h"

namespace magnum { namespace input3d {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::geometry::Transform3;
using magnum::geometry::Transform;
using magnum::geometry::TransformConstraint;
using magnum::math::Matrix;
using magnum::math::Vector3;

class Controller;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Navigator : public Object {
DeclareClass(Navigator, Object);
private:
	Transform3<float>	m_Transform;
	Pointer<Controller>	m_Controller;
	Pointer<TransformConstraint<float,3> > m_Constraint;
	
public:
	Navigator();
	
	inline void			set(const Vector3<float>& from, const Vector3<float>& at, const Vector3<float>& up);
	inline void			setFrom(const Vector3<float>& );
	inline void			setAt(const Vector3<float>& );
	inline void			setDirection(const Vector3<float>& );
	inline void			setUp(const Vector3<float>& );
	
	inline Vector3<float> getFrom() const;
	inline Vector3<float> getAt() const;
	inline Vector3<float> getDirection() const;
	inline Vector3<float> getUp() const;
	
	inline void			setTransform(const Matrix<float,4>& );
	inline void			setTransform(const Transform<float,3>& );
	inline const Transform3<float>& getTransform() const;

	inline void			attachConstraint(TransformConstraint<float,3>* );
	inline void			detachConstraint();
	inline TransformConstraint<float,3>* getConstraint() const;

	inline void			attachController(Controller* );
	inline void			detachController();
	inline Controller*	getController() const;

	void				update(float timestep = 1.0f);
	void				reset();
};

}; };

#endif
