/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT3D_NAVIGATOR_INL
#define __INPUT3D_NAVIGATOR_INL

namespace magnum { namespace input3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> Navigator::getFrom() const {
	return m_Transform.getTranslation();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> Navigator::getDirection() const {
	return -m_Transform.getAxis(2);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> Navigator::getAt() const {
	return m_Transform.getTranslation() - m_Transform.getAxis(2);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Vector3<float> Navigator::getUp() const {
	return m_Transform.getAxis(1);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Navigator::setFrom(const Vector3<float>& from) {
	Vector3<float> n = from - getAt();
	n.normalise();
	m_Transform.setAxis(2,n);
	m_Transform.setAxis(0,getUp() % n);
	m_Transform.setTranslation(from);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Navigator::setAt(const Vector3<float>& at) {
	Vector3<float> n = getFrom() - at;
	n.normalise();
	m_Transform.setAxis(2,n);
	m_Transform.setAxis(0,(Vector3<float>)m_Transform.getAxis(1) % n);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Navigator::setUp(const Vector3<float>& up) {
	m_Transform.setAxis(1,up);
	m_Transform.setAxis(0,up % (Vector3<float>)m_Transform.getAxis(2));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Navigator::setDirection(const Vector3<float>& dir) {
	m_Transform.setAxis(2,-dir);
	m_Transform.setAxis(0,(Vector3<float>)m_Transform.getAxis(1) % (-dir));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Navigator::set(const Vector3<float>& from, const Vector3<float>& at, const Vector3<float>& up) {
	Vector3<float> n = from - at;
	n.normalise();
	m_Transform.setAxis(2,n);
	m_Transform.setAxis(1,up);
	m_Transform.setAxis(0,up % n);
	m_Transform.setTranslation(from);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Navigator::attachConstraint(TransformConstraint<float,3>* c) {
	ASSERT(c != NULL);
	m_Constraint = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Navigator::detachConstraint() {
	m_Constraint.setNull();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline TransformConstraint<float,3>* Navigator::getConstraint() const {
	return m_Constraint;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Navigator::setTransform(const Matrix<float,4>& m) {
	m_Transform = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Navigator::setTransform(const Transform<float,3>& t) {
	m_Transform = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Transform3<float>& Navigator::getTransform() const {
	return m_Transform;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Navigator::attachController(Controller* c) {
	ASSERT(c != NULL);
	m_Controller = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Navigator::detachController() {
	m_Controller.setNull();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Controller* Navigator::getController() const {
	return m_Controller;
}

}; };

#endif
