/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __INPUT3D_TRACKBALLCONTROLLER_H
#define __INPUT3D_TRACKBALLCONTROLLER_H

#include "input.mouse.h"
#include "input3d.controller.h"

namespace magnum { namespace input3d {
using magnum::graphics::Point;
using magnum::graphics::Dimension;
using magnum::graphics::Rectangle;
using magnum::input::MouseListener;
using magnum::input::Mouse;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class TrackballController : public Controller, public MouseListener {
DeclareClass(TrackballController, Controller);
private:
	Vector3<float>		m_Center;
	Vector3<float>		m_Translation;
	Transform3<float>	m_Rotation;
	Dimension<float>	m_MouseSize;
	Point<float>		m_MouseCenter;
	Point<float>		m_Position;
	float				m_Radius;
	float				m_Radius2;
	float				m_Speed;
	bool				m_AutoCenter;
	
public:
	TrackballController();
	
	inline void			setRadius(float );
	inline float		getRadius() const;

	inline void			setSpeed(float );
	inline float		getSpeed() const;
	
	inline void			setCenter(const Vector3<float>& );
	inline const Vector3<float>& getCenter() const;
	
	void				setAutoCenter(bool );
	inline bool			getAutoCenter() const;
	
    void rotate(const Point<float>& src, const Point<float>& dst);
    void translateXY(const Point<float>& src, const Point<float>& dst);
    void translateZ(const Point<float>& src, const Point<float>& dst);

protected:
	virtual void		onMouseArea(const Rectangle<float>& );
	virtual void		onMouseMove(const Point<float>& ,Mouse::Button );
	virtual void		onMouseButton(const Point<float>& ,Mouse::Button );
	virtual void		onMouseWheel(float );
	
	virtual void		onUpdate(float );
	virtual void		onReset();
	virtual void		onModify();
	
private:
	inline float		projectToSphere(const Point<float>& p) const;
};

}; };

#endif
