/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_ARCHIVE_H
#define __IO_ARCHIVE_H

#include "object.pointer.h"
#include "io.filefactory.h"
#include "io.directory.h"
#include "system.mutex.h"

namespace magnum { namespace io {
using magnum::system::Mutex;
using magnum::object::Pointer;

class Archive;
class ArchiveFile;
class ArchiveDirectory;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ArchiveDirectory : public Directory {
DeclareClass(ArchiveDirectory, Directory);
protected:
	ArchiveDirectory(Archive* );

public:
	inline Archive*		getArchive() const;	
	
protected:
	virtual void		onRefresh() = 0;
	virtual void		onOpen(const char* ) = 0;

	inline size_t 		rawWrite(const void *buf, size_t size);
	inline size_t 		rawRead(void *buf, size_t size);
	inline int64 		rawGetPosition();
	inline int64 		rawSetPosition(int64 where);
	inline int64 		rawGetSize();
	
	inline size_t 		rawWrite(const void *buf, int64 where, size_t size);
	inline size_t 		rawRead(void *buf, int64 where, size_t size);
	
private:
	Archive*			m_Archive;
	int64				m_Position;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ArchiveFile : public File {
DeclareClass(ArchiveFile, File);
protected:
	ArchiveFile(Archive* );
	
public:
	inline Archive*		getArchive() const;
	
protected:
	inline size_t 		rawWrite(const void *buf, size_t size);
	inline size_t 		rawRead(void *buf, size_t size);
	inline int64 		rawGetPosition();
	inline int64 		rawSetPosition(int64 where);
	inline int64 		rawGetSize();

	inline size_t 		rawWrite(const void *buf, int64 where, size_t size);
	inline size_t 		rawRead(void *buf, int64 where, size_t size);
	
private:
	Pointer<Archive>	m_Archive;
	int64				m_Position;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Archive : public FileFactory {
DeclareClass(Archive, FileFactory);
friend class ArchiveFile;
friend class ArchiveDirectory;
protected:
	Archive(File* );
	
public:
	inline Directory*	list();
	
protected:
	virtual bool		onExists(const char* ) = 0;
	virtual ArchiveFile* onCreate(const char* ) = 0;
	virtual ArchiveFile* onOpen(const char* ,File::OpenFlags ) = 0;
	virtual ArchiveDirectory* onList() = 0;

	size_t 				rawWrite(const void *buf, int64 where, size_t size);
	size_t 				rawRead(void *buf, int64 where, size_t size);
	int64				rawGetSize();

private:
	Pointer<File>		m_File;
	int64				m_Position;
	Mutex				m_Mutex;
};

}; };

#endif
