/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_ARCHIVE_PAK_H
#define __IO_ARCHIVE_PAK_H

#include "container.array.h"
#include "io.archive.h"

namespace magnum { namespace io {
using magnum::container::Array;

class FilePAK;
class DirectoryPAK;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ArchivePAK : public Archive {
DeclareClass(ArchivePAK, Archive);
friend class FilePAK;
friend class DirectoryPAK;
public:
	ArchivePAK(File* );

protected:
	virtual bool		onExists(const char* );
	virtual ArchiveFile* onCreate(const char* );
	virtual ArchiveFile* onOpen(const char* ,File::OpenFlags );
	virtual ArchiveDirectory* onList();
	
private:
	struct Entry {
		char			m_Name[56];
		unsigned int	m_Position;
		unsigned int	m_Size;
	};
	
	Array<Entry>		m_Entries;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class DirectoryPAK : public ArchiveDirectory {
DeclareClass(DirectoryPAK, ArchiveDirectory);
friend class ArchivePAK;
private:
	DirectoryPAK(ArchivePAK* );
	
protected:
	virtual void		onRefresh();
	virtual void		onOpen(const char* );
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class FilePAK : public ArchiveFile {
DeclareClass(FilePAK, ArchiveFile);
friend class ArchivePAK;
private:
	FilePAK(ArchivePAK* ,const ArchivePAK::Entry* );
	
protected:
	virtual size_t 		onWrite(const void *buf, size_t size);
	virtual size_t 		onRead(void *buf, size_t size);

	virtual int64 		onSetPosition(int64 ,PositionMode = Absolute);
	virtual int64 		onGetPosition() const;
	virtual bool		onRewind();

	virtual bool		onFlush();

	virtual int64 		onGetSize() const;
	virtual int64 		onSetSize(int64 newSize);
	
private:
	int64				m_Size;
	int64				m_Position;
	int64				m_Start;	
};

}; };

#endif
