/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_DATAINPUT_H
#define __IO_DATAINPUT_H

#include "object.h"

namespace magnum { namespace io {
using magnum::object::Object;
using magnum::types::byte;
using magnum::types::word;
using magnum::types::dword;
using magnum::types::String;
using magnum::types::int64;

class InputStream;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class DataInput : public Object {
DeclareClass(DataInput, Object);
public:
	enum Mode {
		Unknown = 0,
		CR,
		LF,
		CRLF
	};
	
public:
	explicit DataInput(InputStream* );
	explicit DataInput(InputStream& );
	~DataInput();
	
	inline InputStream*	getStream() const;
	inline int64 getBytesRead() const;
	
	size_t				skip(size_t );
	size_t 				read(void *buffer, size_t size);
	inline int64 		available();
	inline void			close();
	
	inline void			setMode(Mode );
	inline Mode			getMode() const;

	inline bool			readBoolean();
	inline float		readFloat();
	inline double		readDouble();
	inline char			readChar();
	inline byte			readByte();
	inline word			readWord();
	inline dword		readDword();

	String				readLine();

private:
	InputStream*		m_Input;
	int64				m_BytesRead;
	char				m_LastByte;
	Mode				m_Mode;
};

}; };

#endif
