/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_DATAOUTPUT_INL
#define __IO_DATAOUTPUT_INL

namespace magnum { namespace io {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream* DataOutput::getStream() const {
	return m_Output;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int64 DataOutput::getBytesWritten() const {
	return m_BytesWritten;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline size_t DataOutput::write(const void *buffer, size_t size) {
	ASSERT(m_Output != NULL);
	size_t w = m_Output->write(buffer,size);
	m_BytesWritten += w;
	return w;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataOutput::flush() {
	ASSERT(m_Output != NULL);
	m_Output->flush();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataOutput::close() {
	if (m_Output != NULL) {
		m_Output->release();
		m_Output = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataOutput::writeBoolean(bool b) {
	ASSERT(m_Output != NULL);
	m_BytesWritten += m_Output->write(&b,sizeof(b));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataOutput::writeFloat(float f) {
	ASSERT(m_Output != NULL);
	m_BytesWritten += m_Output->write(&f, sizeof(f));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataOutput::writeDouble(double d) {
	ASSERT(m_Output != NULL);
	m_BytesWritten += m_Output->write(&d, sizeof(d));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataOutput::writeChar(char b) {
	ASSERT(m_Output != NULL);
	m_BytesWritten += m_Output->write(&b, sizeof(b));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataOutput::writeByte(byte b) {
	ASSERT(m_Output != NULL);
	m_BytesWritten += m_Output->write(&b, sizeof(b));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataOutput::writeWord(word w) {
	ASSERT(m_Output != NULL);
	m_BytesWritten += m_Output->write(&w, sizeof(w));
}	


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataOutput::writeDword(dword d) {
	ASSERT(m_Output != NULL);
	m_BytesWritten += m_Output->write(&d, sizeof(d));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataOutput::writeLine(const String& str) {
	ASSERT(m_Output != NULL);
	m_BytesWritten += m_Output->write(str.getString(), str.getLength()+1);
}

}; };

#endif
