/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_DIRECTORY_H
#define __IO_DIRECTORY_H

#include "container.list.h"
#include "io.fileinfo.h"

namespace magnum { namespace io {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::List;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Directory : public Object {
DeclareClass(Directory, Object);
public:
	typedef List<FileInfo>::Iterator Iterator;
	
protected:
	inline Directory();
	inline Directory(const char* );
	
public:
	inline String		getName() const;
	inline const char*	getDirectory() const;
	
	void				refresh();
	void				open(const char* );
	void				clear();
	
	inline unsigned int	getSize() const;
	
	inline Iterator		begin() const;
	inline Iterator		end() const;
	
protected:
	virtual void		onRefresh() = 0;
	virtual void		onOpen(const char* ) = 0;

	inline FileInfo&	insert(const char* ,int64, unsigned int = FileInfo::Readable | FileInfo::Writeable);
	inline FileInfo&	insert(const char* ,int64, const Timestamp& ,const Timestamp& ,const Timestamp& ,unsigned int = FileInfo::Readable | FileInfo::Writeable);

private:
	String				m_Directory;
	List<FileInfo>		m_Files;
};

}; };

#endif
