/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_FILE_H
#define __IO_FILE_H

#include "system.timestamp.h"
#include "io.filestream.h"

namespace magnum { namespace io {
using magnum::object::Object;
using magnum::object::Noncopyable;
using magnum::types::String;
using magnum::types::int64;
using magnum::system::Timestamp;


/*--------------------------------------------------------------------------*/
/**
 * This class is the basic I/O object for reading or writing files, memory 
 * blocks or database objects. This is a pure virtual implementation, as
 *	the real implementation is very system-specific
 */
class File : public StreamFactory, private Noncopyable {
DeclareClass(File, StreamFactory);
public:
	enum {
		EndOfFile = -1
	};

	enum PositionMode {
		Absolute = 1,
		Current = 2,
		End = 3
	};

	enum OpenFlags {
		ReadOnly = 1,
		Readable = 1,
		WriteOnly = 2,
		Writeable = 2,
		ReadWrite = 3
	};

protected:
	inline File(unsigned int = Readable | Writeable);
	inline ~File();

public:
	inline InputStream*		getInputStream();
	inline OutputStream*	getOutputStream();

	template<class T>
	inline size_t 			write(const T *buf);
	template<class T>
	inline size_t 			read(T *buf);
	
	inline size_t 			write(const void *buf, size_t size);
	inline size_t 			read(void *buf, size_t size);

	inline int64 			setPosition(int64 ,PositionMode = Absolute);
	inline int64 			getPosition() const;
	inline bool				rewind();

	inline bool				flush();

	inline int64 			getSize() const;
	inline int64 			setSize(int64 newSize);

	virtual Timestamp		getCreationTime() const;
	virtual Timestamp		getModificationTime() const;
	virtual Timestamp		getAccessTime() const;

	inline bool				isWriteable() const;
	inline bool 			isReadable() const;

protected:
	virtual size_t 			onWrite(const void *buf, size_t size) = 0;
	virtual size_t 			onRead(void *buf, size_t size) = 0;

	virtual int64 			onSetPosition(int64 ,PositionMode = Absolute) = 0;
	virtual int64 			onGetPosition() const = 0;
	virtual bool			onRewind() = 0;

	virtual bool			onFlush() = 0;

	virtual int64 			onGetSize() const = 0;
	virtual int64 			onSetSize(int64 newSize) = 0;

private:
	Timestamp				m_Timestamp;
	unsigned int			m_Flags;
	FileInputStream			m_InputStream;
	FileOutputStream		m_OutputStream;
};

}; };

#endif
