/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_FILE_INL
#define __IO_FILE_INL

namespace magnum { namespace io {
#pragma warning( disable : 4355)

/*--------------------------------------------------------------------------*/
/**
 * Constructs a new File object and initializes the input- and output-stream
 * objects.
 */
inline File::File(unsigned int flags) : m_InputStream(this), m_OutputStream(this) {
	m_Flags = flags;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline File::~File() {
	m_InputStream.close();
	m_OutputStream.close();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline InputStream* File::getInputStream() {
	return &m_InputStream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream* File::getOutputStream() {
	return &m_OutputStream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool File::isWriteable() const {
	return ((m_Flags & Writeable) != 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool File::isReadable() const {
	return ((m_Flags & Writeable) != 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline size_t File::write(const void *buf, size_t size) {
	ASSERT((buf!=NULL) || (size == 0));
	if (!(m_Flags & Writeable))
		return 0;
	
	return onWrite(buf, size);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline size_t File::read(void *buf, size_t size) {
	ASSERT((buf!=NULL) || (size == 0));
	if (!(m_Flags & Readable))
		return 0;
	
	return onRead(buf, size);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline size_t File::write(const T *buf) {
	return write(buf, sizeof(T));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline size_t File::read(T *buf) {
	return read(bug, sizeof(T));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int64 File::setPosition(int64 where,PositionMode mode) {
	return onSetPosition(where, mode);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int64 File::getPosition() const {
	return onGetPosition();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool File::rewind() {
	return onRewind();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool File::flush() {
	return onFlush();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int64 File::getSize() const {
	return onGetSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int64 File::setSize(int64 newSize) {
	if (!(m_Flags & Writeable))	
		return 0;
	return onSetSize(newSize);
}


#pragma warning( default : 4355)
}; };

#endif
