/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_FILEINFO_H
#define __IO_FILEINFO_H

#include "object.h"
#include "system.timestamp.h"

namespace magnum { namespace io {
using magnum::object::Object;
using magnum::types::String;
using magnum::types::uint64;
using magnum::types::int64;
using magnum::system::Timestamp;

class Directory;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class FileInfo : public Object {
DeclareClass(FileInfo, Object);
friend class Directory;
public:
	enum Flags {
		Readable =		0x00000001,
		Writeable =		0x00000002,
		Directory =		0x00000004,
		Executable =	0x00000008,
		Hidden =		0x00000010,
		Link =			0x00000020,
	};
	
public:
	inline FileInfo();
	inline FileInfo(const char* ,int64 = 0,unsigned int = Readable | Writeable);
	inline FileInfo(const char* ,int64, const Timestamp& ,const Timestamp& ,const Timestamp& ,unsigned int = Readable | Writeable);
	
	inline String			getName() const;
	inline int64			getSize() const;
	inline Timestamp		getCreationTime() const;
	inline Timestamp		getModificationTime() const;
	inline Timestamp		getAccessTime() const;
	
	inline bool				isDirectory() const;
	inline bool				isReadable() const;
	inline bool				isWriteable() const;
	inline bool				isHidden() const;
	inline bool 			isExecutable() const;
	inline bool 			isLink() const;

	inline bool 			operator==(const FileInfo& iter) const;
	inline bool 			operator!=(const FileInfo& iter) const;
	inline bool 			operator>=(const FileInfo& iter) const;
	inline bool 			operator<=(const FileInfo& iter) const;
	inline bool 			operator<(const FileInfo& iter) const;
	inline bool 			operator>(const FileInfo& iter) const;

private:
	char					m_Name[256];
	int64					m_Size;
	uint64					m_CreationTime;
	uint64					m_ModificationTime;
	uint64					m_AccessTime;
	unsigned int			m_Flags;
};

}; };

#endif
