/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_FILEINFO_INL
#define __IO_FILEINFO_INL

namespace magnum { namespace io {
using magnum::types::uint64;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline FileInfo::FileInfo() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline FileInfo::FileInfo(const char* n,int64 s,unsigned int f) {
	m_Size = s;
	m_Flags = f;
	strncpy(m_Name, n, sizeof(m_Name));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline FileInfo:: FileInfo(const char* n,int64 s, const Timestamp& ct,const Timestamp& mt,const Timestamp& at,unsigned int f) {
	m_Size = s;
	m_Flags = f;
	m_CreationTime = ct.getTime();
	m_ModificationTime = mt.getTime();
	m_AccessTime = at.getTime();
	strncpy(m_Name, n, sizeof(m_Name));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String FileInfo::getName() const {
	return m_Name;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int64 FileInfo::getSize() const {
	return m_Size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Timestamp FileInfo::getCreationTime() const {
	return Timestamp(m_CreationTime);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Timestamp FileInfo::getModificationTime() const {
	return Timestamp(m_ModificationTime);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Timestamp FileInfo::getAccessTime() const {
	return Timestamp(m_AccessTime);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool FileInfo::isLink() const {
	return ((m_Flags & Link) != 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool FileInfo::isDirectory() const {
	return ((m_Flags & Directory) != 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool FileInfo::isReadable() const {
	return ((m_Flags & Readable) != 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool FileInfo::isWriteable() const {
	return ((m_Flags & Writeable) != 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool	FileInfo::isHidden() const {
	return ((m_Flags & Hidden) != 0);
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool FileInfo::operator==(const FileInfo& iter) const {
	return strncmp(m_Name, iter.m_Name, sizeof(m_Name)) == 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool FileInfo::operator!=(const FileInfo& iter) const {
	return strncmp(m_Name, iter.m_Name, sizeof(m_Name)) != 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool FileInfo::operator>=(const FileInfo& iter) const {
	return strncmp(m_Name, iter.m_Name, sizeof(m_Name)) >= 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool FileInfo::operator<=(const FileInfo& iter) const {
	return strncmp(m_Name, iter.m_Name, sizeof(m_Name)) <= 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool FileInfo::operator<(const FileInfo& iter) const {
	return strncmp(m_Name, iter.m_Name, sizeof(m_Name)) < 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool FileInfo::operator>(const FileInfo& iter) const {
	return strncmp(m_Name, iter.m_Name, sizeof(m_Name)) > 0;
}

}; };

#endif

