/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_FILTER_INL
#define __IO_FILTER_INL

namespace magnum {namespace io {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S,class T> 
inline T* Filter<S,T>::process(const S* in) {
	return onProcess(in);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline FilterChain<T>::FilterChain() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline FilterChain<T>::FilterChain(Filter<T>& flt) {
	m_Filters.insert(&flt);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline FilterChain<T>::FilterChain(Filter<T>& f1, Filter<T>& f2) {
	m_Filters.insert(&f1);
	m_Filters.insert(&f2);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const FilterChain<T>& FilterChain<T>::operator=(FilterChain<T>& fc) {
	m_Filters = fc.m_Filters;
	return *this;	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void FilterChain<T>::append(Filter<T>& f) {
	m_Filters.insert(&f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
T* FilterChain<T>::onProcess(const T* in0) {
	ASSERT(in0 != NULL);
	T* in = (T*)in0;
	T* result = NULL;
	for (Chain<Filter<T> >::Iterator iter = m_Filters.begin(); iter != m_Filters.end(); ++iter) {
		result = iter->process(in);
		if (in != in0)
			in->release();
		if (result == NULL)
			return NULL;
		in = result;
	}
	return result;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline FilterChain<T> operator<<(Filter<T>& f1,Filter<T>& f2) {
	return FilterChain(f2, f1);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline T* operator<<(Filter<T>& f,const T* in) {
	return f.process(in);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline FilterChain<T>& operator<<(FilterChain<T>& c,Filter<T>& f) {
	c.append(f);
	return c;
}

}; };


#endif
