/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_OUTPUTSTREAM_INL
#define __IO_OUTPUTSTREAM_INL

#include <stdlib.h>
#include <stdio.h>

namespace magnum { namespace io {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream::OutputStream() {
	m_FloatDigits = 10;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline size_t OutputStream::write(const T *buf) {
	return write(buf, sizeof(T));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline size_t OutputStream::write(const void *buffer, size_t size) {
	return onWrite(buffer, size);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool	OutputStream::flush() {
	return onFlush();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool	OutputStream::close() {
	return onClose();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& operator<<(OutputStream& stream, const String& str) {
	stream.write(str.getString(), str.getLength());
	return stream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream* operator<<(OutputStream* stream, const String& str) {
	stream->write(str.getString(), str.getLength());
	return stream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& operator<<(OutputStream& stream, const Object& obj) {
	String str = obj.toString();
	stream.write(str.getString(), str.getLength());
	return stream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream* operator<<(OutputStream* stream, const Object& obj) {
	String str = obj.toString();
	stream->write(str.getString(), str.getLength());
	return stream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& OutputStream::operator<<(const char* str) {
	write(str, (unsigned long)strlen(str));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& OutputStream::operator<<(signed char val) {
	char str[10];
#if defined(__FORWINDOWS__)
	itoa((int)val, str, 10);
#else
	sprintf(str,"%d",(int)val);
#endif
	write(str, (unsigned long)strlen(str));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& OutputStream::operator<<(unsigned char val) {
	char str[10];
#if defined(__FORWINDOWS__)
	itoa((int)val, str, 10);
#else
	sprintf(str,"%u",(unsigned int)val);
#endif
	write(str, (unsigned long)strlen(str));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& OutputStream::operator<<(signed short val) {
	char str[10];
#if defined(__FORWINDOWS__)
	itoa((int)val, str, 10);
#else
	sprintf(str,"%d",(int)val);
#endif
	write(str, (unsigned long)strlen(str));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& OutputStream::operator<<(unsigned short val) {
	char str[10];
#if defined(__FORWINDOWS__)
	itoa((int)val, str, 10);
#else
	sprintf(str,"%u",(unsigned int)val);
#endif
	write(str, (unsigned long)strlen(str));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& OutputStream::operator<<(signed int val) {
	char str[10];
#if defined(__FORWINDOWS__)
	itoa((int)val, str, 10);
#else
	sprintf(str,"%d",(int)val);
#endif
	write(str, (unsigned long)strlen(str));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& OutputStream::operator<<(unsigned int val) {
	char str[10];
#if defined(__FORWINDOWS__)
	itoa((int)val, str, 10);
#else
	sprintf(str,"%u",(unsigned int)val);
#endif
	write(str, (unsigned long)strlen(str));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& OutputStream::operator<<(signed long val) {
	char str[10];
#if defined(__FORWINDOWS__)
	ltoa(val, str, 10);
#else
	sprintf(str,"%d",(int)val);
#endif
	write(str, (unsigned long)strlen(str));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& OutputStream::operator<<(unsigned long val) {
	char str[10];
#if defined(__FORWINDOWS__)
	ltoa(val, str, 10);
#else
	sprintf(str,"%u",(unsigned int)val);
#endif
	write(str, (unsigned long)strlen(str));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& OutputStream::operator<<(float val) {
	char str[20];
	gcvt(val, m_FloatDigits, str);
	write(str, (unsigned long)strlen(str));
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream& OutputStream::operator<<(double val) {
	char str[20];
	gcvt(val, m_FloatDigits, str);
	write(str, (unsigned long)strlen(str));
	return *this;
}

}; };
#endif
