/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_STREAMTOKENISER_H
#define __IO_STREAMTOKENISER_H

#include "object.softpointer.h"
#include "io.inputstream.h"

namespace magnum { namespace io {
using magnum::object::SoftPointer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class StreamTokeniser : public Object {
DeclareClass(StreamTokeniser, Object);
public:
	enum Token {
		EndOfFile	= -1,
		Unknown		= 0,
		EndOfLine	= 1,
		Word		= 2,
		Number		= 3,
		Comment		= 4,
		Ordinary	= 5,
		Whitespace	= 6,
		Quote		= 7,
	};
public:
	StreamTokeniser();
	StreamTokeniser(InputStream* );
	
	inline InputStream*		getStream() const;
	
	Token					nextToken();
	inline Token			getType() const;
	inline double			getNumber() const;
	inline const String&	getString() const;
	
	inline void				setWhitespaces(const char* );
	inline const char*		getWhitespaces() const;

	inline void				setOrdinaryChars(const char* );
	inline const char*		getOrdinaryChars() const;

	inline void				setQuoteChars(const char* );
	inline const char*		getQuoteChars() const;

	inline void				setCommentChars(const char* );
	inline const char*		getCommentChars() const;

	inline void				setWordChars(const char* );
	inline const char*		getWordChars() const;
	
	void					resetSyntax();
	
	inline void				enableSlashSlashComments(bool );
	inline void				enableSlashStarComments(bool );
	
protected:
	Token					readQuote(char c);
	Token					readComment();
	Token					readSlashSlashComment();
	Token					readSlashStarComment();
	
	void					mapChar(char ,Token );
	void					mapChars(const String& tokens, Token type);	
	
private:
	SoftPointer<InputStream> m_Input;
	String					m_String;
	double					m_Number;
	Token					m_Type;
	unsigned char			m_LastByte;
	
	String					m_Whitespaces;
	String					m_Ordinaries;
	String					m_Quotes;
	String					m_Comments;
	String					m_Words;
	bool					m_SlashSlashComments;
	bool					m_SlashStarComments;
	
	uint8					m_TokenTable[256];
};

}; };

#endif
