/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_SYSFILE_H
#define __IO_SYSFILE_H

#include "io.file.h"

# if defined (__FORWINDOWS__)
# else
#	include <stdio.h>
# endif

namespace magnum { namespace io {
using magnum::object::Object;
using magnum::types::String;

/*----------------------------------------------------------------------------
This class adds support for real physical files on harddiscs, CDs or 
similar mediums. The implementation itself may be highly dependant on
the operating system used. There are three static methods for opening,
creating and closing files. All filenames are not abstract and highly
platform specific.
----------------------------------------------------------------------------*/
class SystemFile : public File { 
DeclareClass(SystemFile, File);
protected:
	SystemFile(unsigned int flags);

public:
	~SystemFile();

	static SystemFile* open(const char *fname, OpenFlags flags = ReadOnly);
	static SystemFile* create(const char *fname);
	static bool		remove(const char *fname);
	static bool		exists(const char *fname);

	Timestamp		getCreationTime() const;
	Timestamp		getModificationTime() const;
	Timestamp		getAccessTime() const;

protected:
	size_t			onWrite(const void *buf, size_t size);
	size_t			onRead(void *buf, size_t size);
	bool			onFlush();

	int64			onSetPosition(int64 where,PositionMode mode);
	int64			onGetPosition() const;
	bool			onRewind();

	int64			onGetSize() const;
	int64			onSetSize(int64 newSize);
	
private:
	static void		throwException();

private:
#if defined (__FORWINDOWS__)
	HANDLE	m_FileHandle;
#else
	FILE*	m_FileHandle;
#endif
};

}; };

#endif

