/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __IO_TRANSLATOR_H
#define __IO_TRANSLATOR_H

#include "container.chain.h"
#include "io.file.h"
#include "object.property.h"

namespace magnum { namespace io {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Chain;
using magnum::container::Chainable;
using magnum::object::Property;
using magnum::object::PropertyList;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Translator : public Object, private Chainable<Translator<T> > {
DeclareClass(Translator, Object);
public:
	enum {
		Autosize = -1,
	};

public:
	inline Translator();
	inline ~Translator();

	inline T*				decode(File* ,size_t size = Autosize);
	inline T*				decode(InputStream* ,size_t size = Autosize);
							
	inline size_t			encode(File* ,const T* );
	inline size_t			encode(OutputStream* ,const T* );
							
	inline bool				identify(File* );

	inline PropertyList*	createProperties() const;
	inline void				setProperties(const PropertyList* );

protected:
	virtual T*				onDecode(InputStream* ,size_t size = Autosize) = 0;
	virtual size_t			onEncode(OutputStream* ,const T* ) = 0;
	virtual bool			onIdentify(File* ) = 0;

	virtual PropertyList*	onCreateProperties() const;
	virtual void			onSetProperties(const PropertyList* );
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class TranslatorRoster : public Object {
DeclareClass(TranslatorRoster, Object);
private:
	typedef Translator<T>	TranslatorT;
	Chain<TranslatorT>		m_Translators;

public:
	typedef typename magnum::container::Chain<TranslatorT>::Iterator Iterator;

public:
	Translator<T>*			createTranslator();
	Translator<T>*			identify(File* );
							
	inline void				insert(Translator<T>* );
	inline void				remove(Translator<T>* );
	inline void				clear();
							
	inline Iterator			begin();
	inline Iterator			end();

	inline const Chain<TranslatorT>&	getTranslators() const;
};

/*
template<class T> inline T* operator>>(InputStream&, Translator<T>& );
template<class T> inline T* operator>>(InputStream*, Translator<T>& );
template<class T> inline T* operator>>(InputStream&, Translator<T>* );
template<class T> inline T* operator>>(InputStream*, Translator<T>* );

inline Image* operator<<(OutputStream&, Translator& );
inline Image* operator<<(OutputStream*, Translator& );
*/

}; };

#endif
