/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/
#ifndef __MATH_BILINEARINTERPOLATOR_INL
#define __MATH_BILINEARINTERPOLATOR_INL

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline BilinearInterpolator<S,T>::BilinearInterpolator() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline BilinearInterpolator<S,T>::BilinearInterpolator(const T* data,unsigned int w,unsigned int h) 
: Interpolator2d(data, w, h) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline BilinearInterpolator<S,T>::BilinearInterpolator(const T* data,unsigned int w,unsigned int h,const Vector<S,2>& step) 
: Interpolator2d(data, w, h, step) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline T BilinearInterpolator<S,T>::operator()(const Vector<S,2>& v) const {
	return operator()(v[0], v[2]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline T BilinearInterpolator<S,T>::operator()(const S& sx,const S& sy) const {
	S tx = (sx - m_Origin[0]) / m_Step[0];
	S ty = (sy - m_Origin[1]) / m_Step[1];
	S txi = floor(tx);
	S tyi = floor(ty);
	int ix = (int)txi;
	int iy= (int)tyi;

	// Clip x-coordinate
	if (ix < 0) {
		ix = 0;
		tx = S(0);
	} else if (ix > getWidth()-2) {
		ix = getWidth()-2;
		tx = S(1);
	} else {
		tx -= txi;
	}

	// Clip y-coordinate
	if (iy < 0) {
		iy = 0;
		ty = S(0);
	} else if (iy > getHeight()-2) {
		iy = getHeight()-2;
		ty = S(1);
	} else {
		ty -= tyi;
	}
	
	return ty*(tx*at(ix+1,iy+1) + (S(1)-tx)*at(ix,iy+1))
		+ (S(1)-ty)*(tx*at(ix+1,iy) + (S(1)-tx)*at(ix,iy));
}


}; };

#endif
