/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/
#ifndef __MATH_CUBICINTERPOLATOR_INL
#define __MATH_CUBICINTERPOLATOR_INL

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline CubicInterpolator<S,T>::CubicInterpolator() {
}	


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline CubicInterpolator<S,T>::CubicInterpolator(const T* data,unsigned int count,S step) 
: Interpolator1d(data, count, step) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline T CubicInterpolator<S,T>::operator()(const S& s) const {
	S tx = (s - m_Origin) / m_Step;
	S txi = floor(t - S(1));
	int ix = (int)txi;

	// Clip x-coordinate
	if (ix < 0) {
		ix = 0;
		tx = max(tx,S(0));
	} else if (ix > getSize()-4) {
		ix = getSize()-4;
		tx = min(tx-txi, S(3));
	} else {
		tx -= txi;
	}
	
	T r1 = S(3)  *at(ix+1) -       at(ix) +      at(ix+3) -  S(3)*at(ix+2);
	T r2 = S(-15)*at(ix+1) +  S(6)*at(ix) - S(3)*at(ix+3) + S(12)*at(ix+2);
	T r3 = S(18) *at(ix+1) - S(11)*at(ix) + S(2)*at(ix+3) -  S(9)*at(ix+2);

	return  S(1.0/6.0)*((r1*tx + r2)*tx + r3)*tx) + at(ix);
}


}; };

#endif
