/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MATH_EULERANGLE_H
#define __MATH_EULERANGLE_H

#include "math.vector.h"
#include "math.vector3.h"

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class EulerAngle {
public:
	T						x,y,z;
	typedef T				Scalar;

public:
	inline EulerAngle();
	inline EulerAngle(const T& ,const T& ,const T& );

	inline EulerAngle		operator-() const;
	
	inline EulerAngle		operator+(const EulerAngle& ) const;
	inline EulerAngle		operator-(const EulerAngle& ) const;
	inline EulerAngle		operator*(const T& ) const;
	inline EulerAngle		operator/(const T& ) const;

	inline EulerAngle&		operator+=(const EulerAngle& );
	inline EulerAngle&		operator-=(const EulerAngle& );
	inline EulerAngle&		operator*=(const T& );
	inline EulerAngle&		operator/=(const T& );

	inline bool				operator==(const EulerAngle& ) const;
	inline bool				operator!=(const EulerAngle& ) const;
	
	inline T&				operator[](unsigned int );
	inline const T&			operator[](unsigned int ) const;
	
	inline Vector<T,3>		rotate(const Vector<T,3>& ) const;
	inline Vector3<T>		rotate(const Vector3<T>& ) const;
};

template<class T> inline EulerAngle<T> operator*(const T& ,const EulerAngle<T>& );

}; };

#endif
