/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MATH_EULERANGLE_INL
#define __MATH_EULERANGLE_INL

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T>::EulerAngle() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T>::EulerAngle(const T& _x, const T& _y, const T& _z) {
	x = _x;
	y = _y;
	z = _z;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T> EulerAngle<T>::operator-() const {
	return EulerAngle<T>(-x, -y, -z);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T> EulerAngle<T>::operator+(const EulerAngle<T>& a) const {
	return EulerAngle<T>(x + a.x, y + a.y, z + a.z);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T> EulerAngle<T>::operator-(const EulerAngle<T>& a) const{
	return EulerAngle<T>(x - a.x, y - a.y, z - a.z);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T> EulerAngle<T>::operator*(const T& t) const {
	return EulerAngle<T>(x * t, y * t, z * t);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T> operator*(const T& t,const EulerAngle<T>& a) {
	return EulerAngle<T>(t * a.x, t * a.y, t * a.z);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T> EulerAngle<T>::operator/(const T& t) const{
	return EulerAngle<T>(x / t, y / t, z / t);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T>& EulerAngle<T>::operator+=(const EulerAngle<T>& a) {
	x += a.x;
	y += a.y;
	z += a.z;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T>& EulerAngle<T>::operator-=(const EulerAngle<T>& a) {
	x -= a.x;
	y -= a.y;
	z -= a.z;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T>& EulerAngle<T>::operator*=(const T& t) {
	x *= t;
	y *= t;
	z *= t;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline EulerAngle<T>& EulerAngle<T>::operator/=(const T& t) {
	x /= t;
	y /= t;
	z /= t;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline bool EulerAngle<T>::operator==(const EulerAngle<T>& a) const {
	return ((x == a.x) && (y == a.y) && (z == a.z));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline bool EulerAngle<T>::operator!=(const EulerAngle<T>& a) const{
	return ((x != a.x) || (y != a.y) || (z != a.z));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline T& EulerAngle<T>::operator[](unsigned int i) {
	ASSERT(i < 3);
	return (&x)[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
inline const T& EulerAngle<T>::operator[](unsigned int i) const {
	ASSERT(i < 3);
	return (&x)[i];
}
	
}; };

#endif
