// Magic Software, Inc.
// http://www.magic-software.com
// http://www.wild-magic.com
// Copyright (c) 2004.  All Rights Reserved
//
// The Wild Magic Library (WML) source code is supplied under the terms of
// the license agreement http://www.wild-magic.com/License/WildMagic.pdf and
// may not be copied or disclosed except in accordance with the terms of that
// agreement.

#ifndef __MATH_INTEGER_H
#define __MATH_INTEGER_H

namespace magnum { namespace math {

template<int N> class Rational;

// N is the number of 32-bit words you want per Integer.
/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> class Integer {
friend class Rational<N>;
public:
    enum {
        Size = 2*N,
        Bytes = Size*sizeof(short),
        Last = Size-1
    };

private:
    short m_Buffer[Size];

public:
    // construction and destruction
    inline Integer();
    inline Integer(int i);
    inline Integer(const Integer& rkI);

    // assignment
    inline Integer& operator=(const Integer& rkI);

    // comparison
    inline bool operator==(const Integer& rkI) const;
    inline bool operator!=(const Integer& rkI) const;
    inline bool operator< (const Integer& rkI) const;
    inline bool operator<=(const Integer& rkI) const;
    inline bool operator> (const Integer& rkI) const;
    inline bool operator>=(const Integer& rkI) const;

    // arithmetic operations
    Integer operator-() const;
    Integer operator+(const Integer& rkI) const;
    inline Integer operator-(const Integer& rkI) const;
    Integer operator*(const Integer& rkI) const;
    inline Integer operator/(const Integer& rkI) const;
    inline Integer operator%(const Integer& rkI) const;

    // arithmetic updates
    inline const Integer& operator+=(const Integer& rkI);
    inline const Integer& operator-=(const Integer& rkI);
    inline const Integer& operator*=(const Integer& rkI);
    inline const Integer& operator/=(const Integer& rkI);

    // shift operations
    Integer operator<<(int iShift) const;
    Integer operator>>(int iShift) const;

    // shift updates
    const Integer& operator<<=(int iShift);
    const Integer& operator>>=(int iShift);

private:
    // Support for comparisons.  The return value of Compare is -1 if I0 < I1,
    // is 0 if I0 == I1, or is +1 if I0 > I1.
    static int compare(const Integer& rkI0, const Integer& rkI1);
    int getSign() const;

    // support for division and modulo
    static bool getDivMod(const Integer& rkNumer, const Integer& rkDenom,
        Integer& rkQuotient, Integer& rkRemainder);

    static void divSingle(const Integer& rkNumer, short usDenom,
        Integer& rkQuo, Integer& rkRem);

    static void divMultiple(const Integer& rkNumer, const Integer& rkDenom,
        Integer& rkQuo, Integer& rkRem);

    // miscellaneous utilities
    inline int getLeadingBlock() const;
    inline int getTrailingBlock() const;
    int getLeadingBit(int i) const;  // of m_asBuffer[i]
    int getTrailingBit(int i) const;  // of m_asBuffer[i]
    inline int getLeadingBit() const;  // of entire number
    inline int getTrailingBit() const;  // of entire number
    inline void setBit(int i, bool bOn);
    inline bool getBit(int i) const;
    inline unsigned int toUnsignedInt(int i) const;
    inline void fromUnsignedInt(int i, unsigned int uiValue);
    inline unsigned int toUnsignedInt(int iLo, int iHi) const;
    inline int toInt(int i) const;
};

template<int N> inline Integer<N> operator*(int i, const Integer<N>& rkI);


}; };

#endif
