// Magic Software, Inc.
// http://www.magic-software.com
// http://www.wild-magic.com
// Copyright(c) 2004.  All Rights Reserved
//
// The Wild Magic Library(WML) source code is supplied under the terms of
// the license agreement http://www.wild-magic.com/License/WildMagic.pdf and
// may not be copied or disclosed except in accordance with the terms of that
// agreement.

#ifndef __MATH_INTEGER_INL
#define __MATH_INTEGER_INL

namespace magnum { namespace math {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline Integer<N>::Integer() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline Integer<N>::Integer(int i) {
    if( i >= 0 )
        memset(m_Buffer,0,Bytes);
    else
        memset(m_Buffer,0xFF,Bytes);
	*(int *)(void *)m_Buffer = i;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline Integer<N>::Integer(const Integer& in) {
    memcpy(m_Buffer, in.m_Buffer, Bytes);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline Integer<N>& Integer<N>::operator=(const Integer& in) {
    memcpy(m_Buffer, in.m_Buffer, Bytes);
    return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline int Integer<N>::getSign() const {
    return(m_Buffer[Last] & 0x8000) ? -1 : +1;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline bool Integer<N>::operator==(const Integer& in) const {
    return compare(*this,in) == 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline bool Integer<N>::operator!=(const Integer& in) const {
    return compare(*this,in) != 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline bool Integer<N>::operator<(const Integer& in) const {
    int iS0 = getSign(), iS1 = in.getSign(); 
    if( iS0 > 0 ) {
        if( iS1 > 0 )
            return compare(*this,in) < 0;
        else
            return false;
    } else {
        if( iS1 > 0 )
            return true;
        else
            return compare(*this,in) < 0;
    }
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline bool Integer<N>::operator<=(const Integer& in) const {
    int iS0 = getSign(), iS1 = in.getSign();
    if( iS0 > 0 ) {
        if( iS1 > 0 )
            return compare(*this,in) <= 0;
        else
            return false;
    } else {
        if( iS1 > 0 )
            return true;
        else
            return compare(*this,in) <= 0;
    }
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline bool Integer<N>::operator>(const Integer& in) const {
    int iS0 = getSign(), iS1 = in.getSign();
    if( iS0 > 0 ) {
        if( iS1 > 0 )
            return compare(*this,in) > 0;
        else
            return true;
    } else {
        if( iS1 > 0 )
            return false;
        else
            return compare(*this,in) > 0;
    }
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline bool Integer<N>::operator>=(const Integer& in) const {
    int iS0 = getSign(), iS1 = in.getSign();
    if( iS0 > 0 ) {
        if( iS1 > 0 )
            return compare(*this,in) >= 0;
        else
            return true;
    } else {
        if( iS1 > 0 )
            return false;
        else
            return compare(*this,in) >= 0;
    }
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> int Integer<N>::compare(const Integer<N>& in0, const Integer<N>& in1) {
    for(int i = Last; i >= 0; i--) {
        unsigned int uiValue0 =(unsigned int)in0.m_Buffer[i];
        unsigned int uiValue1 =(unsigned int)in1.m_Buffer[i];
        if( uiValue0 < uiValue1 )
            return -1;
        else if( uiValue0 > uiValue1 )
            return +1;
    }
    return 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> Integer<N> Integer<N>::operator-() const {
    Integer res = *this;

    // negate the bits
    int i;
    for(i = 0; i < Size; i++)
        res.m_Buffer[i] = ~res.m_Buffer[i];

    // add 1(place in carry bit and add zero to 'result')
    unsigned int uiCarry = 1;
    for(i = 0; i < Size; i++) {
        unsigned int uiB1 = res.toUnsignedInt(i);
        unsigned int uiSum = uiB1 + uiCarry;
        res.fromUnsignedInt(i,uiSum);
        uiCarry =(uiSum & 0x00010000) ? 1 : 0;
    }

    // test for overflow
    if( res.getSign() == getSign() ) {
        ASSERT( res == 0 );
    }

    return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> Integer<N> Integer<N>::operator+(const Integer& in) const {
    Integer res;

    unsigned int uiCarry = 0;
    for(int i = 0; i < Size; i++) {
        unsigned int uiB0 = toUnsignedInt(i);
        unsigned int uiB1 = in.toUnsignedInt(i);
        unsigned int uiSum = uiB0 + uiB1 + uiCarry;
        res.fromUnsignedInt(i,uiSum);
        uiCarry =(uiSum & 0x00010000) ? 1 : 0;
    }

    // test for overflow
    if( getSign() == in.getSign() )
    {
        ASSERT( res.getSign() == getSign() );
    }

    return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline Integer<N> Integer<N>::operator-(const Integer& in) const {
    return *this +(-in);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> Integer<N> Integer<N>::operator*(const Integer& in) const {
    int iS0 = getSign(), iS1 = in.getSign(), iSProduct = iS0*iS1;
    Integer kOp0 =( iS0 > 0 ? *this : -*this );
    Integer kOp1 =( iS1 > 0 ? in : -in );

    // product of single-digit number with multiple-digit number
    unsigned short ausProduct[2*Size];
    unsigned short* pusPCurrent = ausProduct;

    // product of the two multiple-digit operands
    unsigned short ausResult[2*Size];
    unsigned short* pusRCurrent = ausResult;
    memset(ausResult,0,2*Bytes);

    for(int i0 = 0, iSize = 2*Size; i0 < Size; i0++, iSize--)
    {
        unsigned int uiB0 = kOp0.toUnsignedInt(i0);
        if( uiB0 > 0 )
        {
            unsigned short* pusPBuffer = pusPCurrent;
            unsigned int uiCarry = 0;
            int i1;
            for(i1 = 0; i1 < Size; i1++)
            {
                unsigned int uiB1 = kOp1.toUnsignedInt(i1);
                unsigned int uiProd = uiB0*uiB1 + uiCarry;
                *pusPBuffer++ =(unsigned short)(uiProd & 0x0000FFFF);
                uiCarry =(uiProd & 0xFFFF0000) >> 16;
            }
            *pusPBuffer =(unsigned short)uiCarry;

            unsigned short* pusRBuffer = pusRCurrent;
            pusPBuffer = pusPCurrent;
            uiCarry = 0;
            unsigned int uiSum, uiTerm0, uiTerm1;
            for(i1 = 0; i1 <= Size; i1++)
            {
                uiTerm0 =(unsigned int)(*pusPBuffer++);
                uiTerm1 =(unsigned int)(*pusRBuffer);
                uiSum = uiTerm0 + uiTerm1 + uiCarry;
                *pusRBuffer++ =(unsigned short)(uiSum & 0x0000FFFF);
                uiCarry =(uiSum & 0x00010000) ? 1 : 0;
            }

            for(/**/; uiCarry > 0 && i1 < iSize; i1++)
            {
                uiTerm0 =(unsigned int)(*pusRBuffer);
                uiSum = uiTerm0 + uiCarry;
                *pusRBuffer++ =(unsigned short)(uiSum & 0x0000FFFF);
                uiCarry =(uiSum & 0x00010000) ? 1 : 0;
            }
        }

        pusPCurrent++;
        pusRCurrent++;
    }

    // Test for overflow.  You can test earlier inside the previous loop, but
    // testing here allows you to get an idea of how much overflow there is.
    // This information might be useful for an application to decide how large
    // to choose the integer size.
    for(int i = 2*Size-1; i >= Size; i--) {
        ASSERT( ausResult[i] == 0 );
    }
    ASSERT((ausResult[Last] & 0x8000) == 0 );

    Integer res;
    memcpy(res.m_Buffer,ausResult,Bytes);
    if( iSProduct < 0 )
        res = -res;

    return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline Integer<N> operator*(int i, const Integer<N>& in) {
    return in*i;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline Integer<N> Integer<N>::operator/(const Integer& in) const {
    // TO DO.  On division by zero, return INVALID or signed INFINITY?
    Integer kQ, kR;
    return( getDivMod(*this,in,kQ,kR) ? kQ : 0 );
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline Integer<N> Integer<N>::operator%(const Integer& in) const {
    // TO DO.  On division by zero, return INVALID or signed INFINITY?
    Integer kQ, kR;
    return( getDivMod(*this,in,kQ,kR) ? kR : 0 );
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline const Integer<N>& Integer<N>::operator+=(const Integer& in) {
    *this = *this + in;
    return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline const Integer<N>& Integer<N>::operator-=(const Integer& in) {
    *this = *this - in;
    return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline const Integer<N>& Integer<N>::operator*=(const Integer& in) {
    *this = *this * in;
    return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline const Integer<N>& Integer<N>::operator/=(const Integer& in) {
    *this = *this / in;
    return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> Integer<N> Integer<N>::operator<<(int iShift) const {
    if( iShift < 0 )
        return 0;
    if( iShift == 0 )
        return *this;

    // number of 16-bit blocks to shift
    Integer res;
    int iBlocks = iShift / 16;
    if( iBlocks > Last )
        return 0;

    int i;
    if( iBlocks > 0 ) {
        int j = Last-iBlocks;
        for(i = Last; j >= 0; i--, j--)
            res.m_Buffer[i] = m_Buffer[j];

        for(/**/; i >= 0; i--)
            res.m_Buffer[i] = 0;
    }

    // number of left-over bits to shift
    int iBits = iShift % 16;
    if( iBits > 0 ) {
        unsigned int uiLo, uiHi, uiValue;
        int iM1;
        for(i = Last, iM1 = i-1; iM1 >= 0; i--, iM1--) {
            uiLo = toUnsignedInt(iM1);
            uiHi = toUnsignedInt(i);
            uiValue =(uiLo |(uiHi << 16));
            uiValue <<= iBits;
            res.fromUnsignedInt(i,((0xFFFF0000 & uiValue) >> 16));
        }

        uiValue = toUnsignedInt(0);
        uiValue <<= iBits;
        res.fromUnsignedInt(0,(0x0000FFFF & uiValue));
    }

    return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> Integer<N> Integer<N>::operator>>(int iShift) const {
    if( iShift < 0 )
        return 0;
    if( iShift == 0 )
        return *this;

    // number of 16-bit blocks to shift
    Integer res;
    int iBlocks = iShift / 16;
    if( iBlocks > Last )
        return 0;

    int i;
    if( iBlocks > 0 ) {
        int j = iBlocks;
        for(i = 0; j <= Last; i++, j++)
            res.m_Buffer[i] = m_Buffer[j];

        if( getSign() > 0 )
        {
            for(/**/; i <= Last; i++)
                res.m_Buffer[i] = 0;
        }
        else
        {
            for(/**/; i <= Last; i++)
                res.m_Buffer[i] =(short)(0x0000FFFF);
        }
    }

    // number of left-over bits to shift
    int iBits = iShift % 16;
    if( iBits > 0 )
    {
        unsigned int uiValue;
        int iP1;
        for(i = 0, iP1 = 1; iP1 <= Last; i++, iP1++)
        {
            uiValue = toUnsignedInt(i,iP1);
            uiValue >>= iBits;
            res.fromUnsignedInt(i,uiValue);
        }

        uiValue = toUnsignedInt(Last);
        if( getSign() < 0 )
            uiValue |= 0xFFFF0000;  // sign extension
        uiValue >>= iBits;
        res.fromUnsignedInt(Last,uiValue);
    }

    return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> const Integer<N>& Integer<N>::operator<<=(int iShift) {
    if( iShift <= 0 )
        return *this;

    // number of 16-bit blocks to shift
    Integer res;
    int iBlocks = iShift / 16;
    if( iBlocks > Last )
        return *this;

    int i;
    if( iBlocks > 0 )
    {
        int j = Last-iBlocks;
        for(i = Last; j >= 0; i--, j--)
            m_Buffer[i] = m_Buffer[j];

        for(/**/; i >= 0; i--)
            m_Buffer[i] = 0;
    }

    // number of left-over bits to shift
    int iBits = iShift % 16;
    if( iBits > 0 )
    {
        unsigned int uiValue;
        int iM1;
        for(i = Last, iM1 = i-1; iM1 >= 0; i--, iM1--)
        {
            uiValue = toUnsignedInt(iM1,i);
            uiValue <<= iBits;
            fromUnsignedInt(i,((0xFFFF0000 & uiValue) >> 16));
        }

        uiValue = toUnsignedInt(0);
        uiValue <<= iBits;
        fromUnsignedInt(0,(0x0000FFFF & uiValue));
    }

    return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> const Integer<N>& Integer<N>::operator>>=(int iShift) {
    if( iShift <= 0 )
        return *this;

    // number of 16-bit blocks to shift
    int iBlocks = iShift / 16;
    if( iBlocks > Last )
        return *this;

    int i;
    if( iBlocks > 0 ) {
        int j = iBlocks;
        for(i = 0, j = iBlocks; j <= Last; i++, j++)
            m_Buffer[i] = m_Buffer[j];

        if( getSign() > 0 ) {
            for(/**/; i <= Last; i++)
                m_Buffer[i] = 0;
        } else {
            for(/**/; i <= Last; i++)
                m_Buffer[i] =(short)(0x0000FFFF);
        }
    }

    // number of left-over bits to shift
    int iBits = iShift % 16;
    if( iBits > 0 ) {
        unsigned int uiValue;
        int iP1;
        for(i = 0, iP1 = 1; iP1 <= Last; i++, iP1++) {
            uiValue = toUnsignedInt(i,iP1);
            uiValue >>= iBits;
            fromUnsignedInt(i,uiValue);
        }

        uiValue = toUnsignedInt(Last);
        if( getSign() < 0 )
            uiValue |= 0xFFFF0000;  // sign extension
        uiValue >>= iBits;
        fromUnsignedInt(Last,uiValue);
    }

    return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> bool Integer<N>::getDivMod(const Integer& rkNumer, const Integer& rkDenom, Integer& rkQuo, Integer& rkRem) {
    if( rkDenom == 0 ) {
        ASSERT( false );
        rkQuo = 0;
        rkRem = 0;
        return false;
    }

    if( rkNumer == 0 ) {
        rkQuo = 0;
        rkRem = 0;
        return true;
    }

    // work with the absolute values of the numerator and denominator
    int iS0 = rkNumer.getSign(), iS1 = rkDenom.getSign();
    Integer kAbsNumer = iS0*rkNumer;
    Integer kAbsDenom = iS1*rkDenom;

    int icompare = compare(kAbsNumer,kAbsDenom);
    if( icompare < 0 ) {
        // numerator < denominator:  numerator = 0*denominator + numerator
        rkQuo = 0;
        rkRem = rkNumer;
        return true;
    }

    if( icompare == 0 ) {
        // numerator == denominator:  numerator = 1*denominator + 0
        rkQuo = 1;
        rkRem = 0;
        return true;
    }

    // numerator > denominator, do the division to find quotient and remainder
    if( kAbsDenom > 0x0000FFFF )
        divMultiple(kAbsNumer,kAbsDenom,rkQuo,rkRem);
    else
        divSingle(kAbsNumer,kAbsDenom.m_Buffer[0],rkQuo,rkRem);

    // apply the original signs of numerator and denominator
    rkQuo *= iS0*iS1;
    rkRem *= iS0;

#ifdef _DEBUG
    Integer kTest = rkNumer - rkDenom*rkQuo - rkRem;
    ASSERT( kTest == 0 );
#endif
    return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> void Integer<N>::divSingle(const Integer& rkNumer, short sDenom, Integer& rkQuo, Integer& rkRem) {
    // denominator is a single "digit"
    unsigned int uiDenom = 0x0000FFFF &(unsigned int)sDenom;

    // numerator
    int iNStart = rkNumer.getLeadingBlock();
    const short* psNumer = &rkNumer.m_Buffer[iNStart];
    unsigned int uiDigit1 = 0;

    // quotient
    short* psQuo = &rkQuo.m_Buffer[iNStart];
    rkQuo = 0;
    int iLastNonZero = -1;
    for(int i = iNStart; i >= 0; i--, psNumer--, psQuo--)
    {
        unsigned int uiDigitB = uiDigit1;
        uiDigit1 = 0x0000FFFF &(unsigned int)(*psNumer);
        unsigned int uiNumer =(uiDigitB << 16) | uiDigit1;
        unsigned int uiQuo = uiNumer/uiDenom;
        uiDigit1 = uiNumer - uiQuo*uiDenom;
        *psQuo =(short)(uiQuo & 0x0000FFFF);
        if( iLastNonZero == -1 && uiQuo > 0 )
            iLastNonZero = i;
    }
    ASSERT( iLastNonZero >= 0 );

    // remainder
    int iRDigits =( uiDigit1 & 0xFFFF0000 ? 2 : 1 );
    rkRem = 0;
    memcpy(rkRem.m_Buffer,&uiDigit1,iRDigits*sizeof(short));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> void Integer<N>::divMultiple(const Integer& rkNumer, const Integer& rkDenom, Integer& rkQuo, Integer& rkRem) {
    rkQuo = 0;
    rkRem = 0;

    // Normalization to allow good estimate of quotient.  TO DO:  It is
    // possible that the numerator is large enough that normalization causes
    // overflow when computing the product iAdjust*rkNumer; an ASSERTion will
    // fire in this case.  Ideally the overflow would be allowed and the
    // digit in the overflow position becomes the first digit of the numerator
    // in the division algorithm.  This will require a mixture of Integer<N>
    // and Integer<N+1>, though.
    int iDInit = rkDenom.getLeadingBlock();
    int iLeadingDigit = rkDenom.ToInt(iDInit);
    int iAdjust = 0x10000/(iLeadingDigit+1);
    Integer kANum = iAdjust*rkNumer;
    Integer kADen = iAdjust*rkDenom;
    ASSERT( kADen.getLeadingBlock() == iDInit );

    // get first two "digits" of denominator
    unsigned int uiD1 = kADen.toUnsignedInt(iDInit);
    unsigned int uiD2 = kADen.toUnsignedInt(iDInit-1);

    // determine the maximum necessary division steps
    int iNInit = kANum.getLeadingBlock();
    ASSERT( iNInit >= iDInit );
    int iQInit;
    unsigned int uiRHat;
    if( iNInit != iDInit ) {
        iQInit = iNInit-iDInit-1;
        uiRHat = 1;
    } else {
        iQInit = 0;
        uiRHat = 0;
    }

    for(/**/; iQInit >= 0; iQInit--) {
        // get first three indices of remainder
        unsigned int uiN0, uiN1, uiN2;
        if( uiRHat > 0 ) {
            uiN0 = kANum.toUnsignedInt(iNInit--);
            uiN1 = kANum.toUnsignedInt(iNInit--);
            uiN2 = kANum.toUnsignedInt(iNInit);
        } else {
            uiN0 = 0;
            uiN1 = kANum.toUnsignedInt(iNInit--);
            uiN2 = kANum.toUnsignedInt(iNInit);
        }

        // estimate the quotient
        unsigned int uiTmp =(uiN0 << 16) | uiN1;
        unsigned int uiQHat =( uiN0 != uiD1 ? uiTmp/uiD1 : 0x0000FFFF );
        unsigned int uiProd = uiQHat*uiD1;
        ASSERT( uiTmp >= uiProd );
        uiRHat = uiTmp - uiProd;
        if( uiD2*uiQHat > 0x10000*uiRHat + uiN2 ) {
            uiQHat--;
            uiRHat += uiD1;
            if( uiD2*uiQHat > 0x10000*uiRHat + uiN2 ) {
                // If this block is entered, we have exactly the quotient for
                // the division.  The adjustment block of code later cannot
                // happen.
                uiQHat--;
                uiRHat += uiD1;
            }
        }

        // compute the quotient for this step of the division
        Integer kLocalQuo;
        kLocalQuo.fromUnsignedInt(iQInit,uiQHat);

        // compute the remainder
        Integer kProduct = kLocalQuo*kADen;
        kANum -= kProduct;
        if( kANum < 0 ) {
            uiQHat--;
            kANum += kADen;
            ASSERT( kANum >= 0 );
        }

        // set quotient digit
        rkQuo.fromUnsignedInt(iQInit,uiQHat);

        if( kANum >= kADen ) {
            // prepare to do another division step
            iNInit = kANum.getLeadingBlock();
        } else {
            // remainder is smaller than divisor, finished dividing
            break;
        }
    }

    // unnormalize the remainder
    if( kANum > 0 ) {
        short sDivisor =(short)(iAdjust & 0x0000FFFF);
        Integer kShouldBeZero;
        divSingle(kANum,sDivisor,rkRem,kShouldBeZero);
    } else {
        rkRem = 0;
    }
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline int Integer<N>::getLeadingBlock() const {
    for(int i = Last; i >= 0; i--) {
        if( m_Buffer[i] != 0 )
            return i;
    }
    return -1;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline int Integer<N>::getTrailingBlock() const {
    for(int i = 0; i <= Last; i++) {
        if( m_Buffer[i] != 0 )
            return i;
    }
    return -1;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> int Integer<N>::getLeadingBit(int i) const {
    ASSERT( 0 <= i && i <= Last);
    if( i < 0 || i > Last )
        return -1;

    // This is a binary search for the high-order bit of m_Buffer[i].  The
    // return value is the index into the bits(0 <= index < 16).
    int iValue =(int)m_Buffer[i];
    if((iValue & 0xFF00) != 0 )
    {
        if((iValue & 0xF000) != 0 )
        {
            if((iValue & 0xC000) != 0 )
            {
                if((iValue & 0x8000) != 0 )
                    return 15;
                else //(iValue & 0x4000) != 0
                    return 14;
            }
            else  //(iValue & 0x3000) != 0
            {
                if((iValue & 0x2000) != 0 )
                    return 13;
                else  //(iValue & 0x1000) != 0
                    return 12;
            }
        }
        else  //(iValue & 0x0F00) != 0
        {
            if((iValue & 0x0C00) != 0 )
            {
                if((iValue & 0x0800) != 0 )
                    return 11;
                else  //(iValue & 0x0400) != 0
                    return 10;
            }
            else  //(iValue & 0x0300) != 0
            {
                if((iValue & 0x0200) != 0 )
                    return 9;
                else  //(iValue & 0x0100) != 0
                    return 8;
            }
        }
    }
    else  //(iValue & 0x00FF)
    {
        if((iValue & 0x00F0) != 0 )
        {
            if((iValue & 0x00C0) != 0 )
            {
                if((iValue & 0x0080) != 0 )
                    return 7;
                else  //(iValue & 0x0040) != 0
                    return 6;
            }
            else  //(iValue & 0x0030) != 0
            {
                if((iValue & 0x0020) != 0 )
                    return 5;
                else  //(iValue & 0x0010) != 0
                    return 4;
            }
        }
        else  //(iValue & 0x000F) != 0
        {
            if((iValue & 0x000C) != 0 )
            {
                if((iValue & 0x0008) != 0 )
                    return 3;
                else  //(iValue & 0x0004) != 0
                    return 2;
            }
            else  //(iValue & 0x0003) != 0
            {
                if((iValue & 0x0002) != 0 )
                    return 1;
                else  //(iValue & 0x0001) != 0
                    return 0;
            }
        }
    }

    return -1;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> int Integer<N>::getTrailingBit(int i) const {
    ASSERT( 0 <= i && i <= Last);
    if( i < 0 || i > Last )
        return -1;

    // This is a binary search for the low-order bit of m_Buffer[i].  The
    // return value is the index into the bits(0 <= index < 16).
    int iValue =(int)m_Buffer[i];
    if((iValue & 0x00FF) != 0 )
    {
        if((iValue & 0x000F) != 0 )
        {
            if((iValue & 0x0003) != 0 )
            {
                if((iValue & 0x0001) != 0 )
                    return 0;
                else //(iValue & 0x0002) != 0
                    return 1;
            }
            else  //(iValue & 0x000C) != 0
            {
                if((iValue & 0x0004) != 0 )
                    return 2;
                else  //(iValue & 0x0080) != 0
                    return 3;
            }
        }
        else  //(iValue & 0x00F0) != 0
        {
            if((iValue & 0x0030) != 0 )
            {
                if((iValue & 0x0010) != 0 )
                    return 4;
                else  //(iValue & 0x0020) != 0
                    return 5;
            }
            else  //(iValue & 0x00C0) != 0
            {
                if((iValue & 0x0040) != 0 )
                    return 6;
                else  //(iValue & 0x0080) != 0
                    return 7;
            }
        }
    }
    else  //(iValue & 0xFF00)
    {
        if((iValue & 0x0F00) != 0 )
        {
            if((iValue & 0x0300) != 0 )
            {
                if((iValue & 0x0100) != 0 )
                    return 8;
                else  //(iValue & 0x0200) != 0
                    return 9;
            }
            else  //(iValue & 0x0C00) != 0
            {
                if((iValue & 0x0400) != 0 )
                    return 10;
                else  //(iValue & 0x0800) != 0
                    return 11;
            }
        }
        else  //(iValue & 0xF000) != 0
        {
            if((iValue & 0x3000) != 0 )
            {
                if((iValue & 0x1000) != 0 )
                    return 12;
                else  //(iValue & 0x2000) != 0
                    return 13;
            }
            else  //(iValue & 0xC000) != 0
            {
                if((iValue & 0x4000) != 0 )
                    return 14;
                else  //(iValue & 0x8000) != 0
                    return 15;
            }
        }
    }

    return -1;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline int Integer<N>::getLeadingBit() const {
    int iBlock = getLeadingBlock();
    if( iBlock >= 0 ) {
        int iBit = getLeadingBit(iBlock);
        if( iBit >= 0 )
            return iBit+16*iBlock;
    }

    return -1;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline int Integer<N>::getTrailingBit() const {
    int iBlock = getTrailingBlock();
    if( iBlock >= 0 )
    {
        int iBit = getTrailingBit(iBlock);
        if( iBit >= 0 )
            return iBit+16*iBlock;
    }

    return -1;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline void Integer<N>::setBit(int i, bool bOn) {
    // ASSERT( 0 <= i && i <= Last );
    int iBlock = i / 16;
    int iBit = i % 16;
    if( bOn )
        m_Buffer[iBlock] |=(1 << iBit);
    else
        m_Buffer[iBlock] &= ~(1 << iBit);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline bool Integer<N>::getBit(int i) const {
    // ASSERT( 0 <= i && i <= Last );
    int iBlock = i / 16;
    int iBit = i % 16;
    return(m_Buffer[iBlock] &(1 << iBit)) != 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline unsigned int Integer<N>::toUnsignedInt(int i) const {
    // ASSERT( 0 <= i && i <= Last );
    return 0x0000FFFF &(unsigned int)m_Buffer[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline void Integer<N>::fromUnsignedInt(int i, unsigned int uiValue) {
    // ASSERT( 0 <= i && i <= Last);
    m_Buffer[i] =(short)(uiValue & 0x0000FFFF);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline unsigned int Integer<N>::toUnsignedInt(int iLo, int iHi) const {
    unsigned int uiLo = toUnsignedInt(iLo);
    unsigned int uiHi = toUnsignedInt(iHi);
    return(uiLo |(uiHi << 16));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> inline int Integer<N>::toInt(int i) const {
    // ASSERT( 0 <= i && i <= Last);
    return(int)(0x0000FFFF &(unsigned int)m_Buffer[i]);
}


}; };

#endif
