/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/
#ifndef __MATH_INTERPOLATOR2D_INL
#define __MATH_INTERPOLATOR2D_INL

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
Interpolator2d<S,T>::Interpolator2d() {
	m_Origin.null();
	m_Step[0] = m_Step[1] = S(1);
	m_Width = m_Height = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
Interpolator2d<S,T>::Interpolator2d(const T* data,unsigned int w,unsigned int h) {
	m_Data.copy(data,w*h);
	m_Origin.null();
	m_Step[0] = m_Step[1] = S(1);
	m_Width = w;
	m_Height = h;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
Interpolator2d<S,T>::Interpolator2d(const T* data,unsigned int w,unsigned int h,const Vector<S,2>& step) {
	m_Data.copy(data,w*h);
	m_Origin.null();
	m_Step = step;
	m_Width = w;
	m_Height = h;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
void Interpolator2d<S,T>::setSize(unsigned int w,unsigned int h) {
	m_Data.setSize(w*h);
	m_Width = w;
	m_Height = h;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline unsigned int	Interpolator2d<S,T>::getWidth() const {
	return m_Width;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline unsigned int	Interpolator2d<S,T>::getHeight() const {
	return m_Height;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline const Vector<S,2>& Interpolator2d<S,T>::getStep() const {
	return m_Step;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline void Interpolator2d<S,T>::setStep(const S& w,const S& h) {
	m_Step[0] = w;
	m_Step[1] = h;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline void Interpolator2d<S,T>::setStep(const Vector<S,2>& step) {
	m_Step = step;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline const Vector<S,2>& Interpolator2d<S,T>::getOrigin() const {
	return m_Origin;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline void Interpolator2d<S,T>::setOrigin(const S& x,const S& y) {
	m_Origin[0] = x;
	m_Origin[1] = y;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline void Interpolator2d<S,T>::setOrigin(const Vector<T,2>& o) {
	m_Origin = o;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline T* Interpolator2d<S,T>::operator[](unsigned int i) {
	return &m_Data[i*m_Width];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline const T* Interpolator2d<S,T>::operator[](unsigned int i) const {
	return &m_Data[i*m_Width];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline T& Interpolator2d<S,T>::at(unsigned int i,unsigned int j) {
	ASSERT((i < m_Width) && (j < m_Height));
	return m_Data[j*m_Width+i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline const T& Interpolator2d<S,T>::at(unsigned int i,unsigned int j) const {
	ASSERT((i < m_Width) && (j < m_Height));
	return m_Data[j*m_Width+i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline T Interpolator2d<S,T>::operator()(const Vector<S,2>& v) const {
	return operator()(v[0], v[2]);
}

}; };

#endif
