/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/
#ifndef __MATH_INTERPOLATOR3D_INL
#define __MATH_INTERPOLATOR3D_INL

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
Interpolator3d<S,T>::Interpolator3d() {
	m_Origin.null();
	m_Step[0] = m_Step[1] = m_Step[2] = S(1);
	m_Width = m_Height = m_Depth = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
Interpolator3d<S,T>::Interpolator3d(const T* data,unsigned int w,unsigned int h,unsigned int d) {
	m_Data.copy(data,w*h*d);
	m_Origin.null();
	m_Step[0] = m_Step[1] = m_Step[2] = S(1);
	m_Width = w;
	m_Height = h;
	m_Depth = d
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
Interpolator3d<S,T>::Interpolator3d(const T* data,unsigned int w,unsigned int h,unsigned int d,const Vector<S,3>& step) {
	m_Data.copy(data,w*h*d);
	m_Origin.null();
	m_Step = step;
	m_Width = w;
	m_Height = h;
	m_Depth = d
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
void Interpolator3d<S,T>::setSize(unsigned int w,unsigned int h,unsigned int d) {
	m_Data.setSize(w*h*d);
	m_Width = w;
	m_Height = h;
	m_Depth = d;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline unsigned int	Interpolator3d<S,T>::getWidth() const {
	return m_Width;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline unsigned int	Interpolator3d<S,T>::getHeight() const {
	return m_Height;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline unsigned int	Interpolator3d<S,T>::getDepth() const {
	return m_Depth;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline const Vector<S,3>& Interpolator3d<S,T>::getStep() const {
	return m_Step;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline void Interpolator3d<S,T>::setStep(const S& w,const S& h,const S& d) {
	m_Step[0] = w;
	m_Step[1] = h;
	m_Step[2] = d;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline void Interpolator3d<S,T>::setStep(const Vector<S,3>& step) {
	m_Step = step;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline const Vector<S,3>& Interpolator3d<S,T>::getOrigin() const {
	return m_Origin;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline void Interpolator3d<S,T>::setOrigin(const S& x,const S& y, const S& z) {
	m_Origin[0] = x;
	m_Origin[1] = y;
	m_Origin[2] = z;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline void Interpolator3d<S,T>::setOrigin(const Vector<T,3>& o) {
	m_Origin = o;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline T& Interpolator3d<S,T>::at(unsigned int i,unsigned int j,unsigned int k) {
	ASSERT((i < m_Width) && (j < m_Height) && (k < m_Depth));
	return m_Data[k*m_Width*m_Height+j*m_Width+i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T>
inline const T& Interpolator3d<S,T>::at(unsigned int i,unsigned int j,unsigned int k) const {
	ASSERT((i < m_Width) && (j < m_Height) && (k < m_Depth));
	return m_Data[k*m_Width*m_Height+j*m_Width+i];
}


}; };

#endif
