/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/
#ifndef __MATH_LINEARFUNCTION_H
#define __MATH_LINEARFUNCTION_H

#include "math.function.h"
#include "math.vector.h"

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim> 
class LinearFunction :  public Function<Vector<T,srcdim>, Vector<T,dstdim> > {
DeclareClass(LinearFunction, Function);
private:
	T							m_Matrix[dstdim][srcdim];
	
public:
	inline Vector<T,dstdim>		operator()(const Vector<T,srcdim>& ) const;

	inline T*					operator[](unsigned int );
	inline const T*				operator[](unsigned int ) const;

	inline T&					operator()(unsigned int ,unsigned int );
	inline const T&				operator()(unsigned int ,unsigned int ) const;
	
	inline bool					operator==(const LinearFunction& ) const;
	inline bool					operator!=(const LinearFunction& ) const;

	inline LinearFunction		operator-() const;

	inline LinearFunction		operator+(const LinearFunction& ) const;
	inline LinearFunction		operator-(const LinearFunction& ) const;

	inline const LinearFunction& operator/=(const T& );
	inline const LinearFunction& operator*=(const T& );
	inline const LinearFunction& operator+=(const LinearFunction& );
	inline const LinearFunction& operator-=(const LinearFunction& );

	inline LinearFunction& 		identity();
	inline LinearFunction& 		null();
	
	// Some friends
	friend LinearFunction<T,srcdim,dstdim> operator*(const T& ,const LinearFunction<T,srcdim,dstdim>& );
	friend LinearFunction<T,srcdim,dstdim> operator*(const LinearFunction<T,srcdim,dstdim>& ,const T& );
	friend LinearFunction<T,srcdim,dstdim> operator/(const LinearFunction<T,srcdim,dstdim>& ,const T& );
};

template<class T,int srcdim, int dstdim>
inline LinearFunction<T,srcdim,dstdim> operator*(const T& ,const LinearFunction<T,srcdim,dstdim>& );
template<class T,int srcdim, int dstdim>
inline LinearFunction<T,srcdim,dstdim> operator*(const LinearFunction<T,srcdim,dstdim>& ,const T& );
template<class T,int srcdim, int dstdim>
inline LinearFunction<T,srcdim,dstdim> operator/(const LinearFunction<T,srcdim,dstdim>& ,const T& );


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim> 
class LinearFunction<T,srcdim,1> :  public Function<Vector<T,srcdim>, T> {
DeclareClass(LinearFunction, Function);
private:
	T							m_Matrix[srcdim];
	
public:
	inline T					operator()(const Vector<T,srcdim>& ) const;

	inline T&					operator[](unsigned int );
	inline const T&				operator[](unsigned int ) const;

	inline bool					operator==(const LinearFunction& ) const;
	inline bool					operator!=(const LinearFunction& ) const;

	inline LinearFunction		operator-() const;

	inline LinearFunction		operator+(const LinearFunction& ) const;
	inline LinearFunction		operator-(const LinearFunction& ) const;

	inline const LinearFunction& operator/=(const T& );
	inline const LinearFunction& operator*=(const T& );
	inline const LinearFunction& operator+=(const LinearFunction& );
	inline const LinearFunction& operator-=(const LinearFunction& );

	inline LinearFunction& 		null();
	
	// Some friends
	friend LinearFunction<T,srcdim,1> operator*(const T& ,const LinearFunction<T,srcdim,1>& );
	friend LinearFunction<T,srcdim,1> operator*(const LinearFunction<T,srcdim,1>& ,const T& );
	friend LinearFunction<T,srcdim,1> operator/(const LinearFunction<T,srcdim,1>& ,const T& );
};

template<class T,int srcdim>
inline LinearFunction<T,srcdim,1> operator*(const T& ,const LinearFunction<T,srcdim,1>& );
template<class T,int srcdim>
inline LinearFunction<T,srcdim,1> operator*(const LinearFunction<T,srcdim,1>& ,const T& );
template<class T,int srcdim>
inline LinearFunction<T,srcdim,1> operator/(const LinearFunction<T,srcdim,1>& ,const T& );

}; };

#endif
