/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/
#ifndef __MATH_LINEARFUNCTION_INL
#define __MATH_LINEARFUNCTION_INL

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline Vector<T,dstdim> LinearFunction<T,srcdim,dstdim>::operator()(const Vector<T,srcdim>& v) const {
	Vector<T,dstdim> r;
	for (unsigned int i = 0; i < dstdim; i++) {
		T t = m_Matrix[i][0] * v[0];
		for (unsigned int j = 1; j < srcdim; j++)
			t += m_Matrix[i][j] * v[j];
		r[i] = t;
	}
	return r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline T* LinearFunction<T,srcdim,dstdim>::operator[](unsigned int i) {
	ASSERT(i < dstdim);
	return m_Matrix[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline const T* LinearFunction<T,srcdim,dstdim>::operator[](unsigned int i) const {
	ASSERT(i < dstdim);
	return m_Matrix[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline T& LinearFunction<T,srcdim,dstdim>::operator()(unsigned int i ,unsigned int j) {
	ASSERT((i < dstdim) && (j < srcdim));
	return m_Matrix[i][j];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline const T& LinearFunction<T,srcdim,dstdim>::operator()(unsigned int i,unsigned int j) const {
	ASSERT((i < dstdim) && (j < srcdim));
	return m_Matrix[i][j];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline bool LinearFunction<T,srcdim,dstdim>::operator==(const LinearFunction<T,srcdim,dstdim>& l) const {
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			if (m_Matrix[i][j] != l.m_Matrix[i][j])
				return false;
		}
	}
	return true;		
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline bool LinearFunction<T,srcdim,dstdim>::operator!=(const LinearFunction<T,srcdim,dstdim>& l) const {
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			if (m_Matrix[i][j] != l.m_Matrix[i][j])
				return true;
		}
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline LinearFunction<T,srcdim,dstdim> LinearFunction<T,srcdim,dstdim>::operator-() const {
	LinearFunction<T,srcdim,dstdim> l;
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			l.m_Matrix[i][j] = -m_Matrix[i][j];
		}
	}
	return l;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline LinearFunction<T,srcdim,dstdim> LinearFunction<T,srcdim,dstdim>::operator+(const LinearFunction<T,srcdim,dstdim>& l) const {
	LinearFunction<T,srcdim,dstdim> r;
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			r.m_Matrix[i][j] = m_Matrix[i][j] + l.m_Matrix[i][j];
		}
	}
	return r;
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline LinearFunction<T,srcdim,dstdim> LinearFunction<T,srcdim,dstdim>::operator-(const LinearFunction<T,srcdim,dstdim>& l) const {
	LinearFunction<T,srcdim,dstdim> r;
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			r.m_Matrix[i][j] = m_Matrix[i][j] - l.m_Matrix[i][j];
		}
	}
	return r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline const LinearFunction<T,srcdim,dstdim>& LinearFunction<T,srcdim,dstdim>::operator/=(const T& t) {
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			m_Matrix[i][j] /= t;
		}
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline const LinearFunction<T,srcdim,dstdim>& LinearFunction<T,srcdim,dstdim>::operator*=(const T& t) {
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			m_Matrix[i][j] *= t;
		}
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline const LinearFunction<T,srcdim,dstdim>& LinearFunction<T,srcdim,dstdim>::operator+=(const LinearFunction<T,srcdim,dstdim>& l) {
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			m_Matrix[i][j] += l.m_Matrix[i][j];
		}
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline const LinearFunction<T,srcdim,dstdim>& LinearFunction<T,srcdim,dstdim>::operator-=(const LinearFunction<T,srcdim,dstdim>& l) {
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			m_Matrix[i][j] -= l.m_Matrix[i][j];
		}
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline LinearFunction<T,srcdim,dstdim>& LinearFunction<T,srcdim,dstdim>::identity() {
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			m_Matrix[i][j] = T(0);
		}
		m_Matrix[i][i] = T(1);
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline LinearFunction<T,srcdim,dstdim>& LinearFunction<T,srcdim,dstdim>::null() {
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			m_Matrix[i][j] = T(0);
		}
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline LinearFunction<T,srcdim,dstdim> operator*(const T& t,const LinearFunction<T,srcdim,dstdim>& l) {
	LinearFunction<T,srcdim,dstdim> r;
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			r.m_Matrix[i][j] = t * l.m_Matrix[i][j];
		}
	}
	return r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline LinearFunction<T,srcdim,dstdim> operator*(const LinearFunction<T,srcdim,dstdim>& l,const T& t) {
	LinearFunction<T,srcdim,dstdim> r;
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			r.m_Matrix[i][j] = l.m_Matrix[i][j] * t;
		}
	}
	return r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim, int dstdim>
inline LinearFunction<T,srcdim,dstdim> operator/(const LinearFunction<T,srcdim,dstdim>& l,const T& t) {
	LinearFunction<T,srcdim,dstdim> r;
	for (unsigned int i = 0; i < dstdim; i++) {
		for (unsigned int j = 0; j < srcdim; j++) {
			r.m_Matrix[i][j] = l.m_Matrix[i][j] / t;
		}
	}
	return r;
}



/*----------------------------------------------------------------------------

Partial template specialisation for dstdim = 1

----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline T LinearFunction<T,srcdim,1>::operator()(const Vector<T,srcdim>& v) const {
	T r = m_Matrix[0] * v[0];
	for (unsigned int j = 1; j < srcdim; j++)
		r += m_Matrix[j] * v[j];
	return r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline T& LinearFunction<T,srcdim,1>::operator[](unsigned int i) {
	ASSERT(i < dstdim);
	return m_Matrix[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline const T& LinearFunction<T,srcdim,1>::operator[](unsigned int i) const {
	ASSERT(i < dstdim);
	return m_Matrix[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline bool LinearFunction<T,srcdim,1>::operator==(const LinearFunction<T,srcdim,1>& l) const {
	for (unsigned int j = 0; j < srcdim; j++) {
		if (m_Matrix[j] != l.m_Matrix[j])
			return false;
	}
	return true;		
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline bool LinearFunction<T,srcdim,1>::operator!=(const LinearFunction<T,srcdim,1>& l) const {
	for (unsigned int j = 0; j < srcdim; j++) {
		if (m_Matrix[j] != l.m_Matrix[j])
			return true;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline LinearFunction<T,srcdim,1> LinearFunction<T,srcdim,1>::operator-() const {
	LinearFunction<T,srcdim,1> l;
	for (unsigned int j = 0; j < srcdim; j++) {
		l.m_Matrix[j] = -m_Matrix[j];
	}
	return l;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline LinearFunction<T,srcdim,1> LinearFunction<T,srcdim,1>::operator+(const LinearFunction<T,srcdim,1>& l) const {
	LinearFunction<T,srcdim,1> r;
	for (unsigned int j = 0; j < srcdim; j++) {
		r.m_Matrix[j] = m_Matrix[j] + l.m_Matrix[j];
	}
	return r;
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline LinearFunction<T,srcdim,1> LinearFunction<T,srcdim,1>::operator-(const LinearFunction<T,srcdim,1>& l) const {
	LinearFunction<T,srcdim,1> r;
	for (unsigned int j = 0; j < srcdim; j++) {
		r.m_Matrix[j] = m_Matrix[j] - l.m_Matrix[j];
	}
	return r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline const LinearFunction<T,srcdim,1>& LinearFunction<T,srcdim,1>::operator/=(const T& t) {
	for (unsigned int j = 0; j < srcdim; j++) {
		m_Matrix[j] /= t;
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline const LinearFunction<T,srcdim,1>& LinearFunction<T,srcdim,1>::operator*=(const T& t) {
	for (unsigned int j = 0; j < srcdim; j++) {
		m_Matrix[j] *= t;
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline const LinearFunction<T,srcdim,1>& LinearFunction<T,srcdim,1>::operator+=(const LinearFunction<T,srcdim,1>& l) {
	for (unsigned int j = 0; j < srcdim; j++) {
		m_Matrix[j] += l.m_Matrix[j];
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline const LinearFunction<T,srcdim,1>& LinearFunction<T,srcdim,1>::operator-=(const LinearFunction<T,srcdim,1>& l) {
	for (unsigned int j = 0; j < srcdim; j++) {
		m_Matrix[j] -= l.m_Matrix[j];
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline LinearFunction<T,srcdim,1>& LinearFunction<T,srcdim,1>::null() {
	for (unsigned int j = 0; j < srcdim; j++) {
		m_Matrix[j] = T(0);
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline LinearFunction<T,srcdim,1> operator*(const T& t,const LinearFunction<T,srcdim,1>& l) {
	LinearFunction<T,srcdim,1> r;
	for (unsigned int j = 0; j < srcdim; j++) {
		r.m_Matrix[j] = t * l.m_Matrix[j];
	}
	return r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline LinearFunction<T,srcdim,1> operator*(const LinearFunction<T,srcdim,1>& l,const T& t) {
	LinearFunction<T,srcdim,1> r;
	for (unsigned int j = 0; j < srcdim; j++) {
		r.m_Matrix[j] = l.m_Matrix[j] * t;
	}
	return r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int srcdim>
inline LinearFunction<T,srcdim,1> operator/(const LinearFunction<T,srcdim,1>& l,const T& t) {
	LinearFunction<T,srcdim,1> r;
	for (unsigned int j = 0; j < srcdim; j++) {
		r.m_Matrix[j] = l.m_Matrix[j] / t;
	}
	return r;
}


}; };

#endif
