/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MATH_LINEARSOLVER_H
#define __MATH_LINEARSOLVER_H

#include "object.h"
#include "math.matrix.h"
#include "math.vector2.h"
#include "math.vector3.h"
#include "math.vector4.h"

namespace magnum { namespace math {
using magnum::object::Object;
using magnum::types::String;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class LinearSolver : public Object {
DeclareClass(LinearSolver, Object);
public:
	template<int dim> static inline bool solve(const Matrix<T,dim>& ,const Vector<T,dim>& ,Vector<T,dim>* );
	template<int dim> static inline bool solve(const Matrix<T,dim>& ,Vector<T,dim>* );
	template<int dim> static inline bool invert(const Matrix<T,dim>& mtx,Matrix<T,dim>* inv);

	static inline bool 		solve(const Matrix2<T>& ,const Vector2<T>& ,Vector2<T>* );
	static inline bool 		solve(const Matrix2<T>& ,Vector2<T>* );
	static inline bool 		solve(const Matrix3<T>& ,const Vector3<T>& ,Vector3<T>* );
	static inline bool 		solve(const Matrix3<T>& ,Vector3<T>* );
	static inline bool		solve(const Matrix4<T>& ,const Vector4<T>& ,Vector4<T>* );
	static inline bool		solve(const Matrix4<T>& ,Vector4<T>* );

	static inline bool		solve(unsigned int dim, const T* mtx, const T* b, T* x);
	static inline bool		solve2(const T* mtx, const T* b, T* x);
	static bool				solve3(const T* mtx, const T* b, T* x);
	static bool				solve4(const T* mtx, const T* b, T* x);

	static bool				solve(unsigned int dim, const T* mtx, T* b);
	static inline bool		solve2(const T* mtx, T* b);
	static bool				solve3(const T* mtx, T* b);
	static bool				solve4(const T* mtx, T* b);

	static inline bool		invert(unsigned int dim, const T* mtx, T* inv);
	static inline bool		invert2(const T* mtx, T* inv);
	static bool				invert3(const T* mtx, T* inv);
	static bool				invert4(const T* mtx, T* inv);

private:
	static int				getPivot(unsigned int dim,const T* mtx,unsigned int where);
};



}; };

#endif
