/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MATH_MATRIX_INL
#define __MATH_MATRIX_INL

#include "math.linearsolver.h"

#include "math.linearsolver.inl"

namespace magnum {namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim>::Matrix() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim>::Matrix(const T* d) {
	T* dst = &m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++)
		*dst++ = *d++;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim> Matrix<T,dim>::operator-() const {
	Matrix m;
	const T* src = &m_Matrix[0][0];
	T* dst = &m.m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++)
		*dst++ = - *src++;

	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim> Matrix<T,dim>::operator*(const T& v) const {
	Matrix m;
	const T* src = &m_Matrix[0][0];
	T* dst = &m.m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++)
		*dst++ = *src++ * v;

	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim> Matrix<T,dim>::operator/(const T& v) const {
	Matrix m;
	const T* src = &m_Matrix[0][0];
	T* dst = &m.m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++)
		*dst++ = *src++ / v;

	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim> Matrix<T,dim>::operator+(const Matrix& m2) const {
	Matrix m;
	const T* src1 = &m_Matrix[0][0];
	const T* src2 = &m2.m_Matrix[0][0];
	T* dst = &m.m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++)
		*dst++ = *src1++ + *src2++;

	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim> Matrix<T,dim>::operator-(const Matrix& m2) const {
	Matrix m;
	const T* src1 = &m_Matrix[0][0];
	const T* src2 = &m2.m_Matrix[0][0];
	T* dst = &m.m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++)
		*dst++ = *src1++ - *src2++;

	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim> Matrix<T,dim>::operator*(const Matrix& mtx) const {
	Matrix<T,dim> res;

	if (dim == 2) {
		res.m_Matrix[0][0] = m_Matrix[0][0]*mtx.m_Matrix[0][0] + m_Matrix[0][1]*mtx.m_Matrix[1][0];
		res.m_Matrix[0][1] = m_Matrix[0][0]*mtx.m_Matrix[0][1] + m_Matrix[0][1]*mtx.m_Matrix[1][1];
		res.m_Matrix[1][0] = m_Matrix[1][0]*mtx.m_Matrix[0][0] + m_Matrix[1][1]*mtx.m_Matrix[1][0];
		res.m_Matrix[1][1] = m_Matrix[1][0]*mtx.m_Matrix[0][1] + m_Matrix[1][1]*mtx.m_Matrix[1][1];
		return res;
	}
	if (dim == 3) {
		res.m_Matrix[0][0] = m_Matrix[0][0]*mtx.m_Matrix[0][0] + m_Matrix[0][1]*mtx.m_Matrix[1][0] + m_Matrix[0][2]*mtx.m_Matrix[2][0];
		res.m_Matrix[0][1] = m_Matrix[0][0]*mtx.m_Matrix[0][1] + m_Matrix[0][1]*mtx.m_Matrix[1][1] + m_Matrix[0][2]*mtx.m_Matrix[2][1];
		res.m_Matrix[0][2] = m_Matrix[0][0]*mtx.m_Matrix[0][2] + m_Matrix[0][1]*mtx.m_Matrix[1][2] + m_Matrix[0][2]*mtx.m_Matrix[2][2];

		res.m_Matrix[1][0] = m_Matrix[1][0]*mtx.m_Matrix[0][0] + m_Matrix[1][1]*mtx.m_Matrix[1][0] + m_Matrix[1][2]*mtx.m_Matrix[2][0];
		res.m_Matrix[1][1] = m_Matrix[1][0]*mtx.m_Matrix[0][1] + m_Matrix[1][1]*mtx.m_Matrix[1][1] + m_Matrix[1][2]*mtx.m_Matrix[2][1];
		res.m_Matrix[1][2] = m_Matrix[1][0]*mtx.m_Matrix[0][2] + m_Matrix[1][1]*mtx.m_Matrix[1][2] + m_Matrix[1][2]*mtx.m_Matrix[2][2];

		res.m_Matrix[2][0] = m_Matrix[2][0]*mtx.m_Matrix[0][0] + m_Matrix[2][1]*mtx.m_Matrix[1][0] + m_Matrix[2][2]*mtx.m_Matrix[2][0];
		res.m_Matrix[2][1] = m_Matrix[2][0]*mtx.m_Matrix[0][1] + m_Matrix[2][1]*mtx.m_Matrix[1][1] + m_Matrix[2][2]*mtx.m_Matrix[2][1];
		res.m_Matrix[2][2] = m_Matrix[2][0]*mtx.m_Matrix[0][2] + m_Matrix[2][1]*mtx.m_Matrix[1][2] + m_Matrix[2][2]*mtx.m_Matrix[2][2];
		return res;
	}

	for (int i = 0; i < dim; i++) {
		if (dim == 4) {
			res.m_Matrix[i][0] = m_Matrix[i][0]*mtx.m_Matrix[0][0] + m_Matrix[i][1]*mtx.m_Matrix[1][0] + m_Matrix[i][2]*mtx.m_Matrix[2][0] + m_Matrix[i][3]*mtx.m_Matrix[3][0];
			res.m_Matrix[i][1] = m_Matrix[i][0]*mtx.m_Matrix[0][1] + m_Matrix[i][1]*mtx.m_Matrix[1][1] + m_Matrix[i][2]*mtx.m_Matrix[2][1] + m_Matrix[i][3]*mtx.m_Matrix[3][1];
			res.m_Matrix[i][2] = m_Matrix[i][0]*mtx.m_Matrix[0][2] + m_Matrix[i][1]*mtx.m_Matrix[1][2] + m_Matrix[i][2]*mtx.m_Matrix[2][2] + m_Matrix[i][3]*mtx.m_Matrix[3][2];
			res.m_Matrix[i][3] = m_Matrix[i][0]*mtx.m_Matrix[0][3] + m_Matrix[i][1]*mtx.m_Matrix[1][3] + m_Matrix[i][2]*mtx.m_Matrix[2][3] + m_Matrix[i][3]*mtx.m_Matrix[3][3];
		} else {
			for (int j = 0; j < dim; j++) {
				T tmp = m_Matrix[i][0] * mtx.m_Matrix[0][j];
				for (int k = 1; k < dim; k++) 
					tmp += m_Matrix[i][k] * mtx.m_Matrix[k][j];
				res.m_Matrix[i][j] = tmp;
			}
		}
	}

	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Matrix<T,dim>& Matrix<T,dim>::operator*=(const T& v) {
	T* dst = &m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++)
		*dst++ *= v;

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Matrix<T,dim>& Matrix<T,dim>::operator/=(const T& v) {
	T* dst = &m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++)
		*dst++ /= v;

	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Matrix<T,dim>& Matrix<T,dim>::operator+=(const Matrix& mtx) {
	T* dst = &m_Matrix[0][0];
	const T* src = &mtx.m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++)
		*dst++ += *src++

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Matrix<T,dim>& Matrix<T,dim>::operator-=(const Matrix& mtx) {
	T* dst = &m_Matrix[0][0];
	const T* src = &mtx.m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++)
		*dst++ -= *src++

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Matrix<T,dim>& Matrix<T,dim>::operator*=(const Matrix& mtx) {
	return rmult(mtx);
}	
	

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Matrix<T,dim>& Matrix<T,dim>::rmult(const Matrix& mtx) {
	if (dim == 2) {
		T res[4];
		res[0] = m_Matrix[0][0]*mtx.m_Matrix[0][0] + m_Matrix[0][1]*mtx.m_Matrix[1][0];
		res[1] = m_Matrix[0][0]*mtx.m_Matrix[0][1] + m_Matrix[0][1]*mtx.m_Matrix[1][1];
		res[2] = m_Matrix[1][0]*mtx.m_Matrix[0][0] + m_Matrix[1][1]*mtx.m_Matrix[1][0];
		res[3] = m_Matrix[1][0]*mtx.m_Matrix[0][1] + m_Matrix[1][1]*mtx.m_Matrix[1][1];
		m_Matrix[0][0] = res[0];
		m_Matrix[0][1] = res[1]; 
		m_Matrix[1][0] = res[2];
		m_Matrix[1][1] = res[3];
		return *this;
	}
	if (dim == 3) {
		T res[3];
		res[0] = m_Matrix[0][0]*mtx.m_Matrix[0][0] + m_Matrix[0][1]*mtx.m_Matrix[1][0] + m_Matrix[0][2]*mtx.m_Matrix[2][0];
		res[1] = m_Matrix[0][0]*mtx.m_Matrix[0][1] + m_Matrix[0][1]*mtx.m_Matrix[1][1] + m_Matrix[0][2]*mtx.m_Matrix[2][1];
		res[2] = m_Matrix[0][0]*mtx.m_Matrix[0][2] + m_Matrix[0][1]*mtx.m_Matrix[1][2] + m_Matrix[0][2]*mtx.m_Matrix[2][2];
		m_Matrix[0][0] = res[0];
		m_Matrix[0][1] = res[1]; 
		m_Matrix[0][2] = res[2];

		res[0] = m_Matrix[1][0]*mtx.m_Matrix[0][0] + m_Matrix[1][1]*mtx.m_Matrix[1][0] + m_Matrix[1][2]*mtx.m_Matrix[2][0];
		res[1] = m_Matrix[1][0]*mtx.m_Matrix[0][1] + m_Matrix[1][1]*mtx.m_Matrix[1][1] + m_Matrix[1][2]*mtx.m_Matrix[2][1];
		res[2] = m_Matrix[1][0]*mtx.m_Matrix[0][2] + m_Matrix[1][1]*mtx.m_Matrix[1][2] + m_Matrix[1][2]*mtx.m_Matrix[2][2];
		m_Matrix[1][0] = res[0];
		m_Matrix[1][1] = res[1]; 
		m_Matrix[1][2] = res[2];

		res[0] = m_Matrix[2][0]*mtx.m_Matrix[0][0] + m_Matrix[2][1]*mtx.m_Matrix[1][0] + m_Matrix[2][2]*mtx.m_Matrix[2][0];
		res[1] = m_Matrix[2][0]*mtx.m_Matrix[0][1] + m_Matrix[2][1]*mtx.m_Matrix[1][1] + m_Matrix[2][2]*mtx.m_Matrix[2][1];
		res[2] = m_Matrix[2][0]*mtx.m_Matrix[0][2] + m_Matrix[2][1]*mtx.m_Matrix[1][2] + m_Matrix[2][2]*mtx.m_Matrix[2][2];
		m_Matrix[2][0] = res[0];
		m_Matrix[2][1] = res[1]; 
		m_Matrix[2][2] = res[2];

		return *this;
	}

	T res[dim];
	for (int i = 0; i < dim; i++) {
		if (dim == 4) {
			res[0] = m_Matrix[i][0]*mtx.m_Matrix[0][0] + m_Matrix[i][1]*mtx.m_Matrix[1][0] + m_Matrix[i][2]*mtx.m_Matrix[2][0] + m_Matrix[i][3]*mtx.m_Matrix[3][0];
			res[1] = m_Matrix[i][0]*mtx.m_Matrix[0][1] + m_Matrix[i][1]*mtx.m_Matrix[1][1] + m_Matrix[i][2]*mtx.m_Matrix[2][1] + m_Matrix[i][3]*mtx.m_Matrix[3][1];
			res[2] = m_Matrix[i][0]*mtx.m_Matrix[0][2] + m_Matrix[i][1]*mtx.m_Matrix[1][2] + m_Matrix[i][2]*mtx.m_Matrix[2][2] + m_Matrix[i][3]*mtx.m_Matrix[3][2];
			res[3] = m_Matrix[i][0]*mtx.m_Matrix[0][3] + m_Matrix[i][1]*mtx.m_Matrix[1][3] + m_Matrix[i][2]*mtx.m_Matrix[2][3] + m_Matrix[i][3]*mtx.m_Matrix[3][3];
			m_Matrix[i][0] = res[0];
			m_Matrix[i][1] = res[1];
			m_Matrix[i][2] = res[2];
			m_Matrix[i][3] = res[3];
		} else {
			for (int j = 0; j < dim; j++) {
				T tmp = (T)0;
				for (int k = 0; k < dim; k++) 
					tmp += m_Matrix[i][k] * mtx.m_Matrix[k][j];
				res[j] = tmp;
			}
			for (int j = 0; j < dim; j++)
				m_Matrix[i][j] = res[j];
		}
	}

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Matrix<T,dim>& Matrix<T,dim>::lmult(const Matrix& mtx) {
	// NOT READY YET!!!
	if (dim == 2) {
		T res[4];
		res[0] = mtx.m_Matrix[0][0]*m_Matrix[0][0] + mtx.m_Matrix[0][1]*m_Matrix[1][0];
		res[1] = mtx.m_Matrix[0][0]*m_Matrix[0][1] + mtx.m_Matrix[0][1]*m_Matrix[1][1];
		res[2] = mtx.m_Matrix[1][0]*m_Matrix[0][0] + mtx.m_Matrix[1][1]*m_Matrix[1][0];
		res[3] = mtx.m_Matrix[1][0]*m_Matrix[0][1] + mtx.m_Matrix[1][1]*m_Matrix[1][1];
		m_Matrix[0][0] = res[0];
		m_Matrix[0][1] = res[1]; 
		m_Matrix[1][0] = res[2];
		m_Matrix[1][1] = res[3];
		return *this;
	}
	if (dim == 3) {
		T res[3];
		res[0] = mtx.m_Matrix[0][0]*m_Matrix[0][0] + mtx.m_Matrix[0][1]*m_Matrix[1][0] + mtx.m_Matrix[0][2]*m_Matrix[2][0];
		res[1] = mtx.m_Matrix[1][0]*m_Matrix[0][0] + mtx.m_Matrix[1][1]*m_Matrix[1][0] + mtx.m_Matrix[1][2]*m_Matrix[2][0];
		res[2] = mtx.m_Matrix[2][0]*m_Matrix[0][0] + mtx.m_Matrix[2][1]*m_Matrix[1][0] + mtx.m_Matrix[2][2]*m_Matrix[2][0];
		m_Matrix[0][0] = res[0];
		m_Matrix[1][0] = res[1]; 
		m_Matrix[2][0] = res[2];

		res[0] = mtx.m_Matrix[0][0]*m_Matrix[0][1] + mtx.m_Matrix[0][1]*m_Matrix[1][1] + mtx.m_Matrix[0][2]*m_Matrix[2][1];
		res[1] = mtx.m_Matrix[1][0]*m_Matrix[0][1] + mtx.m_Matrix[1][1]*m_Matrix[1][1] + mtx.m_Matrix[1][2]*m_Matrix[2][1];
		res[2] = mtx.m_Matrix[2][0]*m_Matrix[0][1] + mtx.m_Matrix[2][1]*m_Matrix[1][1] + mtx.m_Matrix[2][2]*m_Matrix[2][1];
		m_Matrix[0][1] = res[0];
		m_Matrix[1][1] = res[1]; 
		m_Matrix[2][1] = res[2];

		res[0] = mtx.m_Matrix[0][0]*m_Matrix[0][2] + mtx.m_Matrix[0][1]*m_Matrix[1][2] + mtx.m_Matrix[0][2]*m_Matrix[2][2];
		res[1] = mtx.m_Matrix[1][0]*m_Matrix[0][2] + mtx.m_Matrix[1][1]*m_Matrix[1][2] + mtx.m_Matrix[1][2]*m_Matrix[2][2];
		res[2] = mtx.m_Matrix[2][0]*m_Matrix[0][2] + mtx.m_Matrix[2][1]*m_Matrix[1][2] + mtx.m_Matrix[2][2]*m_Matrix[2][2];
		m_Matrix[0][2] = res[0];
		m_Matrix[1][2] = res[1]; 
		m_Matrix[2][2] = res[2];

		return *this;
	}

	T res[dim];
	for (int i = 0; i < dim; i++) {
		if (dim == 4) {
			res[0] = mtx.m_Matrix[0][0]*m_Matrix[0][i] + mtx.m_Matrix[0][1]*m_Matrix[1][i] + mtx.m_Matrix[0][2]*m_Matrix[2][i] + mtx.m_Matrix[0][3]*m_Matrix[3][i];
			res[1] = mtx.m_Matrix[1][0]*m_Matrix[0][i] + mtx.m_Matrix[1][1]*m_Matrix[1][i] + mtx.m_Matrix[1][2]*m_Matrix[2][i] + mtx.m_Matrix[1][3]*m_Matrix[3][i];
			res[2] = mtx.m_Matrix[2][0]*m_Matrix[0][i] + mtx.m_Matrix[2][1]*m_Matrix[1][i] + mtx.m_Matrix[2][2]*m_Matrix[2][i] + mtx.m_Matrix[2][3]*m_Matrix[3][i];
			res[3] = mtx.m_Matrix[3][0]*m_Matrix[0][i] + mtx.m_Matrix[3][1]*m_Matrix[1][i] + mtx.m_Matrix[3][2]*m_Matrix[2][i] + mtx.m_Matrix[3][3]*m_Matrix[3][i];
			m_Matrix[0][i] = res[0];
			m_Matrix[1][i] = res[1];
			m_Matrix[2][i] = res[2];
			m_Matrix[3][i] = res[3];
		} else {
			for (int j = 0; j < dim; j++) {
				T m_ = mtx.m_Matrix[j][0] * m_Matrix[0][i];
				for (int k = 1; k < dim; k++) 
					m_ += mtx.m_Matrix[j][k] * m_Matrix[k][i];
				res[j] = m_;
			}
			for (int j = 0; j < dim; j++)
				m_Matrix[j][i] = res[j];
		}
	}

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Matrix<T,dim>::operator==(const Matrix& ) const {
	const T* dst = &m_Matrix[0][0];
	const T* src = &mtx.m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++) {
		if (*src != *dst)
			return false;
		*src++
		*dst++;
	}

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Matrix<T,dim>::operator!=(const Matrix& ) const {
	const T* dst = &m_Matrix[0][0];
	const T* src = &mtx.m_Matrix[0][0];

	for (int i = 0; i < dim*dim; i++) {
		if (*src != *dst)
			return true;
		*src++
		*dst++;
	}

	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T* Matrix<T,dim>::operator[](unsigned int i) {
	ASSERT(i < dim);
	return m_Matrix[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const T* Matrix<T,dim>::operator[](unsigned int i) const {
	ASSERT(i < dim);
	return m_Matrix[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T& Matrix<T,dim>::operator()(unsigned int i, unsigned int j) {
	ASSERT((i < dim) && (j < dim));
	return m_Matrix[i][j];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const T&	Matrix<T,dim>::operator()(unsigned int i, unsigned int j) const {
	ASSERT((i < dim) && (j < dim));
	return m_Matrix[i][j];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Matrix<T,dim>::getRow(unsigned int i) const {
	ASSERT(i < dim);
	return Vector<T,dim>(m_Matrix[i]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Matrix<T,dim>::getColumn(unsigned int i) const {
	ASSERT(i < dim);
	Vector<T,dim> vec;
	for (int j = 0; j < dim; j++)
		vec[j] = m_Matrix[j][i];
	return vec;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Matrix<T,dim>::setRow(unsigned int i, const Vector<T,dim>& v) {
	ASSERT(i < dim);
	for (int j = 0; j < dim; j++)
		m_Matrix[i][j] = vec[j];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void	Matrix<T,dim>::setColumn(unsigned int i, const Vector<T,dim>& v) {
	ASSERT(i < dim);
	for (int j = 0; j < dim; j++)
		m_Matrix[j][i] = vec[j];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim> Matrix<T,dim>::getTransposed() const {
	Matrix m;
	T *dst = &m.m_Matrix[0][0];

	for (int i = 0; i < dim; i++)
		for (int j = 0; j < dim; j++) 
			*dst++ = m_Matrix[j][i];

	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim> Matrix<T,dim>::getInverse() const {
	return doGetInverse<dim>();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim>& Matrix<T,dim>::transpose() {
	T tmp;
	unsigned int i,j;
	
	for (i = 0; i < dim; i++) {
		for (j = i+1; j < dim; j++) {
			tmp = m_Matrx[i][j];
			m_Matrix[i][j] = m_Matrix[j][i];
			m_Matrix[j][i] = tmp;
		}	
	}

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim>& Matrix<T,dim>::invert() {
	return doInvert<dim>();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim>& Matrix<T,dim>::identity() {
	if (dim == 2) {
		m_Matrix[0][0] = m_Matrix[1][1] = T(1);
		m_Matrix[0][1] = m_Matrix[1][0] = T(0);
		return *this;
	}
	if (dim == 3) {
		m_Matrix[0][0] = m_Matrix[1][1] = m_Matrix[2][2] = T(1);
		m_Matrix[0][1] = m_Matrix[0][2] = 
		m_Matrix[1][0] = m_Matrix[1][2] = 
		m_Matrix[2][0] = m_Matrix[2][1] = T(0);
		return *this;
	}
	for (int i = 0; i < dim; i++) {
		for (int j = 0; j < dim; j++) 
			m_Matrix[i][j] = T(0);
		m_Matrix[i][i] = T(1);
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim>& Matrix<T,dim>::null() {
	T* ptr = &m_Matrix[0][0];
	for (int i = 0; i < dim*dim; i++) {
		*ptr ++ = (T)0;
	}
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Matrix<T,dim> Matrix<T,dim>::doGetInverseN() const {
	Matrix m;
	LinearSolver<T>::invert(dim, &m_Matrix[0][0], &m.m_Matrix[0][0]);
	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim>
inline Matrix<T,dim> Matrix<T,dim>::doGetInverse2() const {
    T det = m_Matrix[0][0]*m_Matrix[1][1] - m_Matrix[0][1]*m_Matrix[1][0];
    if (det > T(0)) {
		Matrix<T,dim> inverse;
        T invdet = T(1)/det;
        inverse[0][0] =  m_Matrix[1][1]*invdet;
        inverse[0][1] = -m_Matrix[0][1]*invdet;
        inverse[1][0] = -m_Matrix[1][0]*invdet;
        inverse[1][1] =  m_Matrix[0][0]*invdet;
        return inverse;
    } else {
		return Matrix<T,dim>::Null;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
Matrix<T,dim>& Matrix<T,dim>::doInvert2() {
    T det = m_Matrix[0][0]*m_Matrix[1][1] - m_Matrix[0][1]*m_Matrix[1][0];
    if (abs(det) < Limits<T>::epsilon()) {
		null();
		return *this;
	}

    T invdet = T(1)/det;
    T tmp = m_Matrix[0][0];
    m_Matrix[0][0] =  m_Matrix[1][1]*invdet;
    m_Matrix[1][1] =  tmp*invdet;
    m_Matrix[0][1] = -m_Matrix[0][1]*invdet;
    m_Matrix[1][0] = -m_Matrix[1][0]*invdet;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
Matrix<T,dim>& Matrix<T,dim>::doInvert3() {
	T dst[9];
    dst[0] = m_Matrix[1][1]*m_Matrix[2][2] -
        m_Matrix[1][2]*m_Matrix[2][1];
    dst[1] = m_Matrix[0][2]*m_Matrix[2][1] -
        m_Matrix[0][1]*m_Matrix[2][2];
    dst[2] = m_Matrix[0][1]*m_Matrix[1][2] -
        m_Matrix[0][2]*m_Matrix[1][1];
    dst[3] = m_Matrix[1][2]*m_Matrix[2][0] -
        m_Matrix[1][0]*m_Matrix[2][2];
    dst[4] = m_Matrix[0][0]*m_Matrix[2][2] -
        m_Matrix[0][2]*m_Matrix[2][0];
    dst[5] = m_Matrix[0][2]*m_Matrix[1][0] -
        m_Matrix[0][0]*m_Matrix[1][2];
    dst[6] = m_Matrix[1][0]*m_Matrix[2][1] -
        m_Matrix[1][1]*m_Matrix[2][0];
    dst[7] = m_Matrix[0][1]*m_Matrix[2][0] -
        m_Matrix[0][0]*m_Matrix[2][1];
    dst[8] = m_Matrix[0][0]*m_Matrix[1][1] -
        m_Matrix[0][1]*m_Matrix[1][0];

    T det =
        m_Matrix[0][0] * dst[0] +
        m_Matrix[0][1] * dst[3] +
        m_Matrix[0][2] * dst[6];
    if (abs(det) < Limits<T>::epsilon()) {
		null();
		return *this;
	}

    det = T(1.0)/det;
    for (int i = 0; i < 3; i++)  {
		m_Matrix[i][0] = dst[i*3] * det;
		m_Matrix[i][1] = dst[i*3 + 1] * det;
		m_Matrix[i][2] = dst[i*3 + 2] * det;
    }

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
Matrix<T,dim>& Matrix<T,dim>::doInvert4() {
	T tmp[12]; /* temp array for pairs */
	T dst[16];
	T det; /* determinant */

	/* calculate pairs for first 8 elements (cofactors) */
	tmp[0] = m_Matrix[2][2] * m_Matrix[3][3];
	tmp[1] = m_Matrix[3][2] * m_Matrix[2][3];
	tmp[2] = m_Matrix[1][2] * m_Matrix[3][3];
	tmp[3] = m_Matrix[3][2] * m_Matrix[1][3];
	tmp[4] = m_Matrix[1][2] * m_Matrix[2][3];
	tmp[5] = m_Matrix[2][2] * m_Matrix[1][3];
	tmp[6] = m_Matrix[0][2] * m_Matrix[3][3];
	tmp[7] = m_Matrix[3][2] * m_Matrix[0][3];
	tmp[8] = m_Matrix[0][2] * m_Matrix[2][3];
	tmp[9] = m_Matrix[2][2] * m_Matrix[0][3];
	tmp[10] = m_Matrix[0][2] * m_Matrix[1][3];
	tmp[11] = m_Matrix[1][2] * m_Matrix[0][3];

	/* calculate first 8 elements (cofactors) */
	dst[0] = tmp[0]*m_Matrix[1][1] + tmp[3]*m_Matrix[2][1] + tmp[4]*m_Matrix[3][1];
	dst[0] -= tmp[1]*m_Matrix[1][1] + tmp[2]*m_Matrix[2][1] + tmp[5]*m_Matrix[3][1];
	dst[1] = tmp[1]*m_Matrix[0][1] + tmp[6]*m_Matrix[2][1] + tmp[9]*m_Matrix[3][1];
	dst[1] -= tmp[0]*m_Matrix[0][1] + tmp[7]*m_Matrix[2][1] + tmp[8]*m_Matrix[3][1];
	dst[2] = tmp[2]*m_Matrix[0][1] + tmp[7]*m_Matrix[1][1] + tmp[10]*m_Matrix[3][1];
	dst[2] -= tmp[3]*m_Matrix[0][1] + tmp[6]*m_Matrix[1][1] + tmp[11]*m_Matrix[3][1];
	dst[3] = tmp[5]*m_Matrix[0][1] + tmp[8]*m_Matrix[1][1] + tmp[11]*m_Matrix[2][1];
	dst[3] -= tmp[4]*m_Matrix[0][1] + tmp[9]*m_Matrix[1][1] + tmp[10]*m_Matrix[2][1];
	dst[4] = tmp[1]*m_Matrix[1][0] + tmp[2]*m_Matrix[2][0] + tmp[5]*m_Matrix[3][0];
	dst[4] -= tmp[0]*m_Matrix[1][0] + tmp[3]*m_Matrix[2][0] + tmp[4]*m_Matrix[3][0];
	dst[5] = tmp[0]*m_Matrix[0][0] + tmp[7]*m_Matrix[2][0] + tmp[8]*m_Matrix[3][0];
	dst[5] -= tmp[1]*m_Matrix[0][0] + tmp[6]*m_Matrix[2][0] + tmp[9]*m_Matrix[3][0];
	dst[6] = tmp[3]*m_Matrix[0][0] + tmp[6]*m_Matrix[1][0] + tmp[11]*m_Matrix[3][0];
	dst[6] -= tmp[2]*m_Matrix[0][0] + tmp[7]*m_Matrix[1][0] + tmp[10]*m_Matrix[3][0];
	dst[7] = tmp[4]*m_Matrix[0][0] + tmp[9]*m_Matrix[1][0] + tmp[10]*m_Matrix[2][0];
	dst[7] -= tmp[5]*m_Matrix[0][0] + tmp[8]*m_Matrix[1][0] + tmp[11]*m_Matrix[2][0];

	/* calculate pairs for second 8 elements (cofactors) */
	tmp[0] = m_Matrix[2][0]*m_Matrix[3][1];
	tmp[1] = m_Matrix[3][0]*m_Matrix[2][1];
	tmp[2] = m_Matrix[1][0]*m_Matrix[3][1];
	tmp[3] = m_Matrix[3][0]*m_Matrix[1][1];
	tmp[4] = m_Matrix[1][0]*m_Matrix[2][1];
	tmp[5] = m_Matrix[2][0]*m_Matrix[1][1];
	tmp[6] = m_Matrix[0][0]*m_Matrix[3][1];
	tmp[7] = m_Matrix[3][0]*m_Matrix[0][1];
	tmp[8] = m_Matrix[0][0]*m_Matrix[2][1];
	tmp[9] = m_Matrix[2][0]*m_Matrix[0][1];
	tmp[10] = m_Matrix[0][0]*m_Matrix[1][1];
	tmp[11] = m_Matrix[1][0]*m_Matrix[0][1];

	/* calculate second 8 elements (cofactors) */
	dst[8] = tmp[0]*m_Matrix[1][3] + tmp[3]*m_Matrix[2][3] + tmp[4]*m_Matrix[3][3];
	dst[8] -= tmp[1]*m_Matrix[1][3] + tmp[2]*m_Matrix[2][3] + tmp[5]*m_Matrix[3][3];
	dst[9] = tmp[1]*m_Matrix[0][3] + tmp[6]*m_Matrix[2][3] + tmp[9]*m_Matrix[3][3];
	dst[9] -= tmp[0]*m_Matrix[0][3] + tmp[7]*m_Matrix[2][3] + tmp[8]*m_Matrix[3][3];
	dst[10] = tmp[2]*m_Matrix[0][3] + tmp[7]*m_Matrix[1][3] + tmp[10]*m_Matrix[3][3];
	dst[10]-= tmp[3]*m_Matrix[0][3] + tmp[6]*m_Matrix[1][3] + tmp[11]*m_Matrix[3][3];
	dst[11] = tmp[5]*m_Matrix[0][3] + tmp[8]*m_Matrix[1][3] + tmp[11]*m_Matrix[2][3];
	dst[11]-= tmp[4]*m_Matrix[0][3] + tmp[9]*m_Matrix[1][3] + tmp[10]*m_Matrix[2][3];
	dst[12] = tmp[2]*m_Matrix[2][2] + tmp[5]*m_Matrix[3][2] + tmp[1]*m_Matrix[1][2];
	dst[12]-= tmp[4]*m_Matrix[3][2] + tmp[0]*m_Matrix[1][2] + tmp[3]*m_Matrix[2][2];
	dst[13] = tmp[8]*m_Matrix[3][2] + tmp[0]*m_Matrix[0][2] + tmp[7]*m_Matrix[2][2];
	dst[13]-= tmp[6]*m_Matrix[2][2] + tmp[9]*m_Matrix[3][2] + tmp[1]*m_Matrix[0][2];
	dst[14] = tmp[6]*m_Matrix[1][2] + tmp[11]*m_Matrix[3][2] + tmp[3]*m_Matrix[0][2];
	dst[14]-= tmp[10]*m_Matrix[3][2] + tmp[2]*m_Matrix[0][2] + tmp[7]*m_Matrix[1][2];
	dst[15] = tmp[10]*m_Matrix[2][2] + tmp[4]*m_Matrix[0][2] + tmp[9]*m_Matrix[1][2];
	dst[15]-= tmp[8]*m_Matrix[1][2] + tmp[11]*m_Matrix[2][2] + tmp[5]*m_Matrix[0][2];

	/* calculate determinant */
	det=m_Matrix[0][0]*dst[0]+m_Matrix[1][0]*dst[1]+m_Matrix[2][0]*dst[2]+m_Matrix[3][0]*dst[3];
    if (abs(det) < Limits<T>::epsilon()) {
		null();
		return *this;
	}

	/* calculate matrix inverse */
	det = 1/det;
	for ( int i = 0; i < 4; i++) {
		m_Matrix[i][0] = dst[i*4]*det;
		m_Matrix[i][1] = dst[i*4 + 1]*det;
		m_Matrix[i][2] = dst[i*4 + 2]*det;
		m_Matrix[i][3] = dst[i*4 + 3]*det;
	}

	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> const Matrix<T,dim> Matrix<T,dim>::Identity = _MatrixIdentity<T,dim>();
template<class T, int dim> const Matrix<T,dim> Matrix<T,dim>::Null = _MatrixNull<T,dim>();


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> _MatrixIdentity<T,dim>::_MatrixIdentity() {
	identity();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> _MatrixNull<T,dim>::_MatrixNull() {
	null();
}


}; };

#endif
