/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/
#ifndef __MATH_QUADRATICFUNCTION_H
#define __MATH_QUADRATICFUNCTION_H

#include "math.function.h"
#include "math.vector.h"
#include "math.matrix.h"

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
class QuadraticFunction : public Function<Vector<T,dim>, T> {
DeclareClass(QuadraticFunction, Function);
private:
	Matrix<T,dim>			m_Matrix;
	
public:
	inline QuadraticFunction();
	inline QuadraticFunction(const Matrix<T,dim>& );
	
	inline T						operator()(const Vector<T,dim>& ) const;

	inline T*						operator[](unsigned int );
	inline const T*					operator[](unsigned int ) const;

	inline T&						operator()(unsigned int ,unsigned int );
	inline const T&					operator()(unsigned int ,unsigned int ) const;
	
	inline bool						operator==(const QuadraticFunction<T,dim>& ) const;
	inline bool						operator!=(const QuadraticFunction<T,dim>& ) const;

	inline QuadraticFunction<T,dim>	operator-() const;

	inline QuadraticFunction<T,dim> operator+(const QuadraticFunction<T,dim>& ) const;
	inline QuadraticFunction<T,dim> operator-(const QuadraticFunction<T,dim>& ) const;

	inline const QuadraticFunction<T,dim>& operator/=(const T& );
	inline const QuadraticFunction<T,dim>& operator*=(const T& );
	inline const QuadraticFunction<T,dim>& operator+=(const QuadraticFunction<T,dim>& );
	inline const QuadraticFunction<T,dim>& operator-=(const QuadraticFunction<T,dim>& );

	inline QuadraticFunction<T,dim>& identity();
	inline QuadraticFunction<T,dim>& null();

	// Some friend declarations
	friend QuadraticFunction<T,dim> operator*(const T& ,const QuadraticFunction<T,dim>& );
	friend QuadraticFunction<T,dim> operator*(const QuadraticFunction<T,dim>& ,const T& );
	friend QuadraticFunction<T,dim> operator/(const QuadraticFunction<T,dim>& ,const T& );
};

template<class T,int dim>
inline QuadraticFunction<T,dim> operator*(const T& ,const QuadraticFunction<T,dim>& );
template<class T,int dim>
inline QuadraticFunction<T,dim> operator*(const QuadraticFunction<T,dim>& ,const T& );
template<class T,int dim>
inline QuadraticFunction<T,dim> operator/(const QuadraticFunction<T,dim>& ,const T& );

}; };

#endif
