/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/
#ifndef __MATH_QUADRATICFUNCTION_INL
#define __MATH_QUADRATICFUNCTION_INL

namespace magnum { namespace math {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T QuadraticFunction<T,dim>::operator()(const Vector<T,dim>& v) const {
	unsigned int i,j,k;
	T val = T(0);
	for (i = 0; i < dim; i++) {
		T tmp = m_Matrix[i][0] * v[0];
		for (j = 1; j < dim; j++)
			tmp += m_Matrix[i][j] * v[j];
		val += v[i]*tmp;
	}
	return val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline QuadraticFunction<T,dim>::QuadraticFunction() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline QuadraticFunction<T,dim>::QuadraticFunction(const Matrix<T,dim>& m) : m_Matrix(m) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T* QuadraticFunction<T,dim>::operator[](unsigned int i) {
	return m_Matrix[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const T* QuadraticFunction<T,dim>::operator[](unsigned int i) const {
	return m_Matrix[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T& QuadraticFunction<T,dim>::operator()(unsigned int i, unsigned int j) {
	return m_Matrix(i,j);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const T& QuadraticFunction<T,dim>::operator()(unsigned int i, unsigned int j) const {
	return m_Matrix(i,j);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool	QuadraticFunction<T,dim>::operator==(const QuadraticFunction<T,dim>& q) const {
	return m_Matrix == q.m_Matrix;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool QuadraticFunction<T,dim>::operator!=(const QuadraticFunction<T,dim>& q) const {
	return m_Matrix != q.m_Matrix;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline QuadraticFunction<T,dim>	QuadraticFunction<T,dim>::operator-() const {
	return QuadraticFunction(-m_Matrix);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline QuadraticFunction<T,dim> QuadraticFunction<T,dim>::operator+(const QuadraticFunction<T,dim>& q) const {
	return QuadraticFunction(m_Matrix + q.m_Matrix);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline QuadraticFunction<T,dim> QuadraticFunction<T,dim>::operator-(const QuadraticFunction<T,dim>& q) const {
	return QuadraticFunction(m_Matrix - q.m_Matrix);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const QuadraticFunction<T,dim>& QuadraticFunction<T,dim>::operator/=(const T& t) {
	m_Matrix /= t;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const QuadraticFunction<T,dim>& QuadraticFunction<T,dim>::operator*=(const T& t) {
	m_Matrix *= t;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const QuadraticFunction<T,dim>& QuadraticFunction<T,dim>::operator+=(const QuadraticFunction<T,dim>& q) {
	m_Matrix += q.m_Matrix;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const QuadraticFunction<T,dim>& QuadraticFunction<T,dim>::operator-=(const QuadraticFunction<T,dim>& q) {
	m_Matrix -= q.m_Matrix;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline QuadraticFunction<T,dim>& QuadraticFunction<T,dim>::identity() {
	m_Matrix.identity();
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline QuadraticFunction<T,dim>& QuadraticFunction<T,dim>::null() {
	m_Matrix.null();
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int dim>
inline QuadraticFunction<T,dim> operator*(const T& t,const QuadraticFunction<T,dim>& q) {
	return QuadraticFunction<T,dim>(t * q.m_Matrix);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int dim>
inline QuadraticFunction<T,dim> operator*(const QuadraticFunction<T,dim>& q,const T& t) {
	return QuadraticFunction<T,dim>(t * q.m_Matrix);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T,int dim>
inline QuadraticFunction<T,dim> operator/(const QuadraticFunction<T,dim>& q,const T& t) {
	return QuadraticFunction<T,dim>(q.m_Matrix / t);
}

}; };

#endif
