/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MATH_QUATERNION_H
#define __MATH_QUATERNION_H

#include "math.vector.h"
#include "math.vector3.h"

namespace magnum {namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Quaternion {
public:
	T		w,x,y,z;

public:
	static Quaternion		Identity;
	static Quaternion		Zero;
	typedef T				Scalar;

public:
	inline Quaternion();
	inline Quaternion(const T&, const T&, const T&, const T& );

    inline Quaternion		operator+(const Quaternion& ) const;
    inline Quaternion		operator-(const Quaternion& ) const;
    inline Quaternion		operator*(const Quaternion& ) const;
    inline Quaternion		operator*(const T& ) const;
    inline Quaternion		operator/(const T& ) const;
    inline Quaternion		operator-() const;

    inline const Quaternion& operator= (const Quaternion& );
    inline const Quaternion& operator+=(const Quaternion& );
    inline const Quaternion& operator-=(const Quaternion& );
    inline const Quaternion& operator*=(const Quaternion& );
    inline const Quaternion& operator*=(const T& );
    inline const Quaternion& operator/=(const T& );

	inline bool				operator==(const Quaternion& ) const;
	inline bool				operator!=(const Quaternion& ) const;
	inline bool				operator<(const Quaternion& ) const;
	inline bool				operator>(const Quaternion& ) const;
	inline bool				operator<=(const Quaternion& ) const;
	inline bool				operator>=(const Quaternion& ) const;

	inline T				dot(const Quaternion& ) const;
	inline T				length2() const;
	inline T				length() const;
	inline Quaternion&		normalise();

	inline Quaternion		getInverse() const;
	inline Quaternion		getConjugate() const;
	inline Quaternion&		invert();
	inline Quaternion&		conjugate();
	inline void				toAxisAngle(Vector3<T>* ,T* ) const;
	inline Matrix<T,3>		toMatrix() const;
	inline void				toMatrix(Matrix<T,3>* ) const;
	
	inline void				fromAxisAngle(const Vector3<T>& , T );	
	void					fromMatrix(const Matrix<T,3>& );

	inline					operator const T*() const;
	inline					operator T*();

	template<class S> inline Quaternion(const Quaternion<S>& q) {
		w = T(q.w);
		x = T(q.x);
		y = T(q.y);
		z = T(q.z);
	};

	template<class S> inline operator Quaternion<S>() const {
		return Quaternion<S>(S(w), S(x), S(y), S(z));
	};
}; 


template<class T> inline Vector<T,3> operator*(const Quaternion<T>& ,const Vector<T,3>& );
template<class T> inline Vector3<T> operator*(const Quaternion<T>& ,const Vector3<T>& );
template<class T> inline Quaternion<T> operator*(const T& ,const Quaternion<T>& );

template<class T> inline Quaternion<T> slerp(const T& ,const Quaternion<T>& ,const Quaternion<T>& );
template<class T> inline Quaternion<T> nlerp(const T& ,const Quaternion<T>& ,const Quaternion<T>& );
template<class T> inline Quaternion<T> squad(const T& ,const Quaternion<T>& ,const Quaternion<T>& ,const Quaternion<T>& ,const Quaternion<T>& );

}; };

#endif
