/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MATH_RATIONAL_H
#define __MATH_RATIONAL_H

#include "math.integer.h"

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<int N> class Rational {
private:
	Integer<N>		m_Numer;
	Integer<N>		m_Denom;

public:
    // construction
    Rational();  // default rational is 0/1
    Rational(const Rational& R);
    Rational(const Integer<N>& Numer);
    Rational(const Integer<N>& Numer, const Integer<N>& Denom);

    // construction converters
    Rational(int iNumer);
    Rational(int iNumer, int iDenom);
    Rational(float fValue);
    Rational(double dValue);

    // member access
    Integer<N>& numer();
    Integer<N>& denom();
    const Integer<N>& numer() const;
    const Integer<N>& denom() const;

    // assignment
    Rational& operator=(const Rational& R);

    // comparison
    bool operator==(const Rational& R) const;
    bool operator!=(const Rational& R) const;
    bool operator<=(const Rational& R) const;
    bool operator< (const Rational& R) const;
    bool operator>=(const Rational& R) const;
    bool operator> (const Rational& R) const;

    // arithmetic operations
    Rational operator+(const Rational& R) const;
    Rational operator-(const Rational& R) const;
    Rational operator*(const Rational& R) const;
    Rational operator/(const Rational& R) const;
    Rational operator-() const;

    // arithmetic updates
    Rational& operator+=(const Rational& R);
    Rational& operator-=(const Rational& R);
    Rational& operator*=(const Rational& R);
    Rational& operator/=(const Rational& R);

    // conversions to float and double(approximately)
    float toFloat() const;
    double toDouble() const;

    // compute the absolute value of the rational number
    Rational abs() const;
};


template <int N> Rational<N> operator+(const Integer<N>& rkI, const Rational<N>& rkR);
template <int N> Rational<N> operator-(const Integer<N>& rkI, const Rational<N>& rkR);
template <int N> Rational<N> operator*(const Integer<N>& rkI, const Rational<N>& rkR);
template <int N> Rational<N> operator/(const Integer<N>& rkI, const Rational<N>& rkR);

}; };

#endif
