/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MATH_TRIGONOMETRICPOLYNOMIAL_H
#define __MATH_TRIGONOMETRICPOLYNOMIAL_H

#include "math.function.h"

namespace magnum { namespace math {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class S, class T> class TrigonometricPolynomial : public Function<S,T> {
private:
	unsigned int			m_Degree;
	T*						m_Coefficients;

public:
	inline TrigonometricPolynomial(unsigned int d = 0);
	inline TrigonometricPolynomial(const TrigonometricPolynomial& );
	inline ~TrigonometricPolynomial();

	inline unsigned int				getDegree() const;
	inline void						setDegree(unsigned int );
	unsigned int					getEffectiveDegree() const;

    // assignment
    inline TrigonometricPolynomial&	operator= (const TrigonometricPolynomial& );

    // evaluation
    inline T						operator() (const S& ) const;

    // arithmetic operations
    inline TrigonometricPolynomial	operator+ (const TrigonometricPolynomial& ) const;
    inline TrigonometricPolynomial	operator- (const TrigonometricPolynomial& ) const;
    inline TrigonometricPolynomial	operator+ (const T&) const; 
    inline TrigonometricPolynomial	operator- (const T&) const; 
    inline TrigonometricPolynomial	operator* (const S&) const;
    inline TrigonometricPolynomial	operator/ (const S&) const;
    inline TrigonometricPolynomial	operator- () const;

    // arithmetic updates
    TrigonometricPolynomial&		operator+= (const TrigonometricPolynomial& );
    TrigonometricPolynomial&		operator-= (const TrigonometricPolynomial& );
    inline TrigonometricPolynomial&	operator+= (const T&);
	inline TrigonometricPolynomial&	operator-= (const T&);
    TrigonometricPolynomial&		operator*= (const S&);
    TrigonometricPolynomial&		operator/= (const S&);

    // derivation
    TrigonometricPolynomial			derive() const;

	inline operator					const T*() const;
	inline operator					T*();

	// Compare operator
	bool							operator==(const TrigonometricPolynomial& ) const;
	bool							operator!=(const TrigonometricPolynomial& ) const;
};

template<class S,class T> inline TrigonometricPolynomial<S,T> operator*(S ,const TrigonometricPolynomial<S,T>& );

}; };

#endif
