/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MATH_VECTOR_H
#define __MATH_VECTOR_H

#include "base.h"

#ifdef max
#undef max
#undef min
#endif

namespace magnum {namespace math {

template<class T, int dim> class Matrix;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Vector {
public:
	T		vec[dim];

public:
	static const Vector		Null;
	typedef T				Scalar;

public:
	inline Vector();
	inline explicit Vector(const T& );
	inline explicit Vector(const T* );

	inline Vector			operator+(const Vector &u) const;
	inline Vector			operator+(const T&) const;
	inline Vector			operator-(const Vector &u) const;
	inline Vector			operator-(const T& s) const;
	inline Vector			operator-() const;
	inline T				operator*(const Vector& u) const;
	inline Vector			operator*(const T& s) const;
	inline Vector			operator/(const T& s) const;
	
	inline Vector<T,dim+1>	operator,(const T& ) const;

	inline const Vector&	operator= (const T& s);
	inline const Vector&	operator+= (const T& s);
	inline const Vector&	operator+= (const Vector& v);
	inline const Vector&	operator-= (const T& s);
	inline const Vector&	operator-= (const Vector& v);
	inline const Vector&	operator*= (const T& s);
	inline const Vector&	operator/= (const T& );

	inline bool				operator==(const Vector& ) const;
	inline bool				operator!=(const Vector& ) const;

	// The following operators are intended to enable the usage of containers
	inline bool				operator<(const Vector& ) const;
	inline bool				operator>(const Vector& ) const;
	inline bool				operator<=(const Vector& ) const;
	inline bool				operator>=(const Vector& ) const;

	inline					operator const T*() const;
	inline					operator T*();

	inline T&				operator()(unsigned int );
	inline const T&			operator()(unsigned int ) const;
	
	// Type conversion routines
	template<class S> inline operator Vector<S,dim>() const;
	template<class S> inline Vector<T,dim>(const Vector<S,dim>& v);

	inline Vector&			null();

	inline Vector&			normalise();

	inline T				length() const;
	inline T				length2() const;
	
	inline Vector			abs() const;

	inline T				max() const;
	inline T				min() const;
	inline T				maxabs() const;
	inline T				minabs() const;
	
	inline T				sum() const;
	inline T				sumabs() const;
};

template<class T, int dim> inline Vector<T,dim> operator*(const T& ,const Vector<T,dim>& );
template<class T, int dim> inline Vector<T,dim>	operator*(const Matrix<T,dim>& ,const Vector<T,dim>& );
template<class T, int dim> inline Vector<T,dim>	operator*(const Matrix<T,dim+1>& ,const Vector<T,dim>& );
template<class T> inline Vector<T,2> operator*(const Matrix<T,2>& ,const Vector<T,2>& );
template<class T> inline Vector<T,2> operator*(const Matrix<T,3>& ,const Vector<T,2>& );
template<class T> inline Vector<T,3> operator*(const Matrix<T,3>& ,const Vector<T,3>& );
template<class T> inline Vector<T,3> operator*(const Matrix<T,4>& ,const Vector<T,3>& );
template<class T> inline Vector<T,4> operator*(const Matrix<T,4>& ,const Vector<T,4>& );
template<class T> inline Vector<T,4> operator*(const Matrix<T,5>& ,const Vector<T,4>& );
template<class T, int dim> inline Vector<T,dim> operator*(const Vector<T,dim>& ,const Matrix<T,dim>& );
template<class T, int dim> inline Vector<T,dim> operator*(const Vector<T,dim>& ,const Matrix<T,dim+1>& );
template<class T> inline Vector<T,2> operator*(const Vector<T,2>& ,const Matrix<T,2>& );
template<class T> inline Vector<T,2> operator*(const Vector<T,2>& ,const Matrix<T,3>& );
template<class T> inline Vector<T,3> operator*(const Vector<T,3>& ,const Matrix<T,3>& );
template<class T> inline Vector<T,3> operator*(const Vector<T,3>& ,const Matrix<T,4>& );
template<class T> inline Vector<T,4> operator*(const Vector<T,4>& ,const Matrix<T,4>& );
template<class T> inline Vector<T,4> operator*(const Vector<T,4>& ,const Matrix<T,5>& );

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class _VectorNull : public Vector<T,dim> {
public:
	_VectorNull();
};


}; };

#endif
