/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MATH_VECTOR_INL
#define __MATH_VECTOR_INL

namespace magnum {namespace math {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>::Vector() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>::Vector(const T& t) {
	for (int i = 0; i < dim; i++)
		vec[i] = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>::Vector(const T* s) {
	for (int i = 0; i < dim; i++)
		vec[i] = s[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> template<class S> 
inline Vector<T,dim>::operator Vector<S,dim>() const {
	Vector<S,dim> v;
	for (unsigned int i = 0; i < dim; i++)
		v.vec[i] = (S)vec[i];
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> template<class S> 
inline Vector<T,dim>::Vector(const Vector<S,dim>& v) {
	for (unsigned int i = 0; i < dim; i++)
		vec[i] = (T)v.vec[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator+(const Vector<T,dim> &u) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] + u.vec[i];
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator+(const T& c) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] + c;
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator-(const Vector<T,dim> &u) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] - u.vec[i];
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator-(const T& c) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] - c;
	return v;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator-() const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = -vec[i];
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator*(const T& s) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] * s;
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> operator*(const T& s,const Vector<T,dim>& in) {
	Vector<T,dim> v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = s * in.vec[i];
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> operator*(const Matrix<T,dim>& mtx,const Vector<T,dim>& vec) {
	Vector<T,dim> res;
	for (int i = 0; i < dim; i++) {
		const T* row = mtx[i];
		T tmp = row[0] * vec[0];
		for (int j = 1; j < dim; j++)
			tmp += row[j] * vec[j];
		res[i] = tmp;
	}

	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> operator*(const Matrix<T,dim+1>& mtx,const Vector<T,dim>& vec) {
	Vector<T,dim> res;
	for (int i = 0; i < dim; i++) {
		const T* row = mtx[i];
		T tmp = row[dim];
		for (int j = 0; j < dim; j++)
			tmp += row[j] * vec[j];
		res[i] = tmp;
	}

	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,2> operator*(const Matrix<T,2>& mtx,const Vector<T,2>& vec) {
	Vector<T,2> res;
	res[0] = mtx[0][0]*vec[0] + mtx[0][1]*vec[1];
	res[1] = mtx[1][0]*vec[0] + mtx[1][1]*vec[1];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,2> operator*(const Matrix<T,3>& mtx,const Vector<T,2>& vec) {
	Vector<T,2> res;
	res[0] = mtx[0][0]*vec[0] + mtx[0][1]*vec[1] + mtx[0][2];
	res[1] = mtx[1][0]*vec[0] + mtx[1][1]*vec[1] + mtx[1][2];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,3> operator*(const Matrix<T,3>& mtx,const Vector<T,3>& vec) {
	Vector<T,3> res;
	res[0] = mtx[0][0]*vec[0] + mtx[0][1]*vec[1] + mtx[0][2]*vec[2];
	res[1] = mtx[1][0]*vec[0] + mtx[1][1]*vec[1] + mtx[1][2]*vec[2];
	res[2] = mtx[2][0]*vec[0] + mtx[2][1]*vec[1] + mtx[2][2]*vec[2];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,3> operator*(const Matrix<T,4>& mtx,const Vector<T,3>& vec) {
	Vector<T,3> res;
	res[0] = mtx[0][0]*vec[0] + mtx[0][1]*vec[1] + mtx[0][2]*vec[2] + mtx[0][3];
	res[1] = mtx[1][0]*vec[0] + mtx[1][1]*vec[1] + mtx[1][2]*vec[2] + mtx[1][3];
	res[2] = mtx[2][0]*vec[0] + mtx[2][1]*vec[1] + mtx[2][2]*vec[2] + mtx[2][3];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,4> operator*(const Matrix<T,4>& mtx,const Vector<T,4>& vec) {
	Vector<T,4> res;
	res[0] = mtx[0][0]*vec[0] + mtx[0][1]*vec[1] + mtx[0][2]*vec[2] + mtx[0][3]*vec[3];
	res[1] = mtx[1][0]*vec[0] + mtx[1][1]*vec[1] + mtx[1][2]*vec[2] + mtx[1][3]*vec[3];
	res[2] = mtx[2][0]*vec[0] + mtx[2][1]*vec[1] + mtx[2][2]*vec[2] + mtx[2][3]*vec[3];
	res[3] = mtx[3][0]*vec[0] + mtx[3][1]*vec[1] + mtx[3][2]*vec[2] + mtx[3][3]*vec[3];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,4> operator*(const Matrix<T,5>& mtx,const Vector<T,4>& vec) {
	Vector<T,4> res;
	res[0] = mtx[0][0]*vec[0] + mtx[0][1]*vec[1] + mtx[0][2]*vec[2] + mtx[0][3]*vec[3] + mtx[0][4];
	res[1] = mtx[1][0]*vec[0] + mtx[1][1]*vec[1] + mtx[1][2]*vec[2] + mtx[1][3]*vec[3] + mtx[1][4];
	res[2] = mtx[2][0]*vec[0] + mtx[2][1]*vec[1] + mtx[2][2]*vec[2] + mtx[2][3]*vec[3] + mtx[2][4];
	res[3] = mtx[3][0]*vec[0] + mtx[3][1]*vec[1] + mtx[3][2]*vec[2] + mtx[3][3]*vec[3] + mtx[3][4];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> operator*(const Vector<T,dim>& vec,const Matrix<T,dim>& mtx) {
	Vector<T,dim> res;
	for (int i = 0; i < dim; i++) {
		T tmp = mtx[0][i] * vec[j];
		for (int j = 1; j < dim; j++)
			tmp += mtx[j][i] * vec[j];
		res[i] = tmp;
	}

	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> operator*(const Vector<T,dim>& vec,const Matrix<T,dim+1>& mtx) {
	Vector<T,dim> res;
	for (int i = 0; i < dim; i++) {
		T tmp = mtx[dim][i];
		for (int j = 0; j < dim; j++)
			tmp += mtx[j][i] * vec[j];
		res[i] = tmp;
	}

	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,2> operator*(const Vector<T,2>& vec,const Matrix<T,2>& mtx) {
	Vector<T,2> res;
	res[0] = mtx[0][0]*vec[0] + mtx[1][0]*vec[1];
	res[1] = mtx[0][1]*vec[0] + mtx[1][1]*vec[1];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,2> operator*(const Vector<T,2>& vec,const Matrix<T,3>& mtx) {
	Vector<T,2> res;
	res[0] = mtx[0][0]*vec[0] + mtx[1][0]*vec[1] + mtx[2][0];
	res[1] = mtx[0][1]*vec[0] + mtx[1][1]*vec[1] + mtx[2][1];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,3> operator*(const Vector<T,3>& vec,const Matrix<T,3>& mtx) {
	Vector<T,3> res;
	res[0] = mtx[0][0]*vec[0] + mtx[1][0]*vec[1] + mtx[2][0]*vec[2];
	res[1] = mtx[0][1]*vec[0] + mtx[1][1]*vec[1] + mtx[2][1]*vec[2];
	res[2] = mtx[0][2]*vec[0] + mtx[1][2]*vec[1] + mtx[2][2]*vec[2];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,3> operator*(const Vector<T,3>& vec,const Matrix<T,4>& mtx) {
	Vector<T,3> res;
	res[0] = mtx[0][0]*vec[0] + mtx[1][0]*vec[1] + mtx[2][0]*vec[2] + mtx[3][0];
	res[1] = mtx[0][1]*vec[0] + mtx[1][1]*vec[1] + mtx[2][1]*vec[2] + mtx[3][1];
	res[2] = mtx[0][2]*vec[0] + mtx[1][2]*vec[1] + mtx[2][2]*vec[2] + mtx[3][2];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,4> operator*(const Vector<T,4>& vec,const Matrix<T,4>& mtx) {
	Vector<T,4> res;
	res[0] = mtx[0][0]*vec[0] + mtx[1][0]*vec[1] + mtx[2][0]*vec[2] + mtx[3][0]*vec[3];
	res[1] = mtx[0][1]*vec[0] + mtx[1][1]*vec[1] + mtx[2][1]*vec[2] + mtx[3][1]*vec[3];
	res[2] = mtx[0][2]*vec[0] + mtx[1][2]*vec[1] + mtx[2][2]*vec[2] + mtx[3][2]*vec[3];
	res[3] = mtx[0][3]*vec[0] + mtx[1][3]*vec[1] + mtx[2][3]*vec[2] + mtx[3][3]*vec[3];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Vector<T,4> operator*(const Vector<T,4>& vec,const Matrix<T,5>& mtx) {
	Vector<T,4> res;
	res[0] = mtx[0][0]*vec[0] + mtx[1][0]*vec[1] + mtx[2][0]*vec[2] + mtx[3][0]*vec[3] + mtx[4][0];
	res[1] = mtx[0][1]*vec[0] + mtx[1][1]*vec[1] + mtx[2][1]*vec[2] + mtx[3][1]*vec[3] + mtx[4][1];
	res[2] = mtx[0][2]*vec[0] + mtx[1][2]*vec[1] + mtx[2][2]*vec[2] + mtx[3][2]*vec[3] + mtx[4][2];
	res[3] = mtx[0][3]*vec[0] + mtx[1][3]*vec[1] + mtx[2][3]*vec[2] + mtx[3][3]*vec[3] + mtx[4][3];
	return res;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::operator*(const Vector<T,dim>& v) const {
	float r = vec[0] * v.vec[0];
	for (int i = 1; i < dim; i++)
		r += vec[i] * v.vec[i];
	return r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::operator/(const T& s) const {
	Vector v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i] / s;
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim+1> Vector<T,dim>::operator,(const T& s) const {
	Vector<T,dim+1> v;
	for (int i = 0; i < dim; i++)
		v.vec[i] = vec[i];
	v.vec[i] = s;
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator+= (const T& s) {
	for (int i = 0; i < dim; i++)
		vec[i] += s;
	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator+= (const Vector<T,dim>& v) {
	for (int i = 0; i < dim; i++)
		vec[i] += v.vec[i];
	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator-= (const T& s) {
	for (int i = 0; i < dim; i++)
		vec[i] -= s;
	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator-= (const Vector<T,dim>& v) {
	for (int i = 0; i < dim; i++)
		vec[i] -= v.vec[i];
	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator*= (const T& s) {
	for (int i = 0; i < dim; i++)
		vec[i] *= s;
	return *this;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Vector<T,dim>::operator/= (const T& s) {
	for (int i = 0; i < dim; i++)
		vec[i] /= s;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Vector<T,dim>::operator==(const Vector<T,dim>& v) const {
	for (int i = 0; i < dim; i++) {
		if (vec[i] != v.vec[i])
			return false;
	}
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Vector<T,dim>::operator!=(const Vector<T,dim>& v) const {
	for (int i = 0; i < dim; i++) {
		if (vec[i] != v.vec[i])
			return true;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Vector<T,dim>::operator<(const Vector<T,dim>& v) const {
	for (int i = 0; i < dim; i++) {
		if (vec[i] < v.vec[i])
			return true;
		if (vec[i] > v.vec[i])
			return false;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Vector<T,dim>::operator>(const Vector<T,dim>& v) const{
	for (int i = 0; i < dim; i++) {
		if (vec[i] > v.vec[i])
			return true;
		if (vec[i] < v.vec[i])
			return false;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Vector<T,dim>::operator<=(const Vector<T,dim>& v) const {
	for (int i = 0; i < dim-1; i++) {
		if (vec[i] < v.vec[i])
			return true;
		if (vec[i] > v.vec[i])
			return false;
	}
	return (vec[i] <= v.vec[i]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool	Vector<T,dim>::operator>=(const Vector<T,dim>& v) const {
	for (int i = 0; i < dim-1; i++) {
		if (vec[i] > v.vec[i])
			return true;
		if (vec[i] < v.vec[i])
			return false;
	}
	return (vec[i] >= v.vec[i]);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>::operator const T*() const {
	return vec;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>::operator T*() {
	return vec;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T& Vector<T,dim>::operator()(unsigned int i) {
	ASSERT(i < dim);
	return vec[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const T&	Vector<T,dim>::operator()(unsigned int i) const {
	ASSERT(i < dim);
	return vec[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::length() const {
	T l = vec[0]*vec[0];
	for(unsigned int i = 1; i < dim; i++)
		l += vec[i]*vec[i];
	return T(sqrt(l));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::length2() const {
	T l = vec[0]*vec[0];
	for(unsigned int i = 1; i < dim; i++)
		l += vec[i]*vec[i];
	return l;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim> Vector<T,dim>::abs() const {
	Vector<T,dim> v;
	for (unsigned int i = 0; i < dim; i++)
		v.vec[i] = abs(vec[i]);
	return v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::max() const {
	T m = vec[0];
	for (unsigned int i = 1; i < dim; i++) {
		if (m < vec[i])
			m = vec[i];
	}
	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::min() const {
	T m = vec[0];
	for (unsigned int i = 1; i < dim; i++) {
		if (m > vec[i])
			m = vec[i];
	}
	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::maxabs() const {
	T m = magnum::math::abs(vec[0]);
	T a;
	for (unsigned int i = 1; i < dim; i++) {
		a = magnum::math::abs(vec[i]);
		if (m < a)
			m = a;
	}
	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::minabs() const {
	T m = magnum::math::abs(vec[0]);
	T a;
	for (unsigned int i = 1; i < dim; i++) {
		a = magnum::math::abs(vec[i]);
		if (m > a)
			m = a;
	}
	return m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::sum() const {
	if (dim == 2) {
		return vec[0] + vec[1];
	} else if (dim == 3) {
		return vec[0] + vec[1] + vec[2];
	} else if (dim == 4) {
		return vec[0] + vec[1] + vec[2] + vec[3];
	} else {
		T m = vec[0];
		for (unsigned int i = 1; i < dim; i++) {
			m += vec[i];
		}
		return m;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Vector<T,dim>::sumabs() const {
	if (dim == 2) {
		return magnum::math::abs(vec[0]) + magnum::math::abs(vec[1]);
	} else if (dim == 3) {
		return magnum::math::abs(vec[0]) + magnum::math::abs(vec[1]) + magnum::math::abs(vec[2]);
	} else if (dim == 4) {
		return magnum::math::abs(vec[0]) + magnum::math::abs(vec[1]) + magnum::math::abs(vec[2]) + magnum::math::abs(vec[3]);
	} else {
		T m = magnum::math::abs(vec[0]);
		for (unsigned int i = 1; i < dim; i++) {
			m += magnum::math::abs(vec[i]);
		}
		return m;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>& Vector<T,dim>::null() {
	for (unsigned int i = 0; i < dim; i++)
		vec[i] = T(0);
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Vector<T,dim>& Vector<T,dim>::normalise() {
	T l = length();
	if (l > T(0)) {
		l = 1/l;
		for (unsigned int i = 0; i < dim; i++)
			vec[i] *= l;
	}
	
	return *this
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> const Vector<T,dim> Vector<T,dim>::Null = _VectorNull<T,dim>();


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> _VectorNull<T,dim>::_VectorNull() {
	null();
}


}; };

#endif
