/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MATH_VECTOR2_H
#define __MATH_VECTOR2_H

#include "math.vector.h"

namespace magnum {namespace math {

template<class T> class Matrix2;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Vector2 {
public:
	T			x,y;

public:
	static Vector2			Null;
	typedef T				Scalar;

public:
	inline Vector2();
	inline explicit Vector2(const T* );
	inline Vector2(const T& ,const T& );	
	inline Vector2(const Vector<T,2>& );

	inline Vector2			operator+(const Vector2 &u) const;
	inline Vector2			operator+(const T&) const;
	inline Vector2			operator-(const Vector2 &u) const;
	inline Vector2			operator-(const T& s) const;
	inline Vector2			operator-() const;
	inline T				operator*(const Vector2& s) const;
	inline Vector2			operator*(const T& s) const;
	inline Vector2			operator/(const T& s) const;

	inline const Vector2&	operator+= (const T& s);
	inline const Vector2&	operator+= (const Vector2& v);
	inline const Vector2&	operator-= (const T& s);
	inline const Vector2&	operator-= (const Vector2& v);
	inline const Vector2&	operator*= (const T& s);
	inline const Vector2&	operator/= (const T& );
							
	inline const Vector2&	operator=(const Vector<T,2>& );

	inline bool				operator==(const Vector2& v) const;
	inline bool				operator!=(const Vector2& v) const;

	// The following operators are intended to enable the usage of containers
	inline bool				operator<(const Vector2& ) const;
	inline bool				operator>(const Vector2& ) const;
	inline bool				operator<=(const Vector2& ) const;
	inline bool				operator>=(const Vector2& ) const;

	inline					operator Vector<T,2>() const;
	inline					operator const T*() const;
	inline					operator T*();

	// Type conversion routines
	template<class S> inline operator Vector2<S>() const {
		return Vector2<S>(S(x), S(y));
	};
	template<class S> inline Vector2<T>(const Vector2<S>& v) {
		x = T(v.x);
		y = T(v.y);
	};

	inline Vector2&			null();
	
	inline Vector2&			normalise();

	inline T				length() const;
	inline T				length2() const;

	inline T				max() const;
	inline T				min() const;
	inline T				maxabs() const;
	inline T				minabs() const;
	
	inline T				sum() const;
	inline T				sumabs() const;
};

template<class T> inline Vector2<T> operator*(const T& ,const Vector2<T>& );
template<class T> inline Vector2<T> operator*(const Matrix<T,3>& ,const Vector2<T>& );
template<class T> inline Vector2<T> operator*(const Vector2<T>& ,const Matrix<T,3>& );
template<class T> inline Vector2<T> operator*(const Matrix<T,2>& ,const Vector2<T>& );
template<class T> inline Vector2<T> operator*(const Vector2<T>& ,const Matrix<T,2>& );


}; };

#endif
