/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MATH_VECTOR4_H
#define __MATH_VECTOR4_H

#include "math.vector.h"
#include "math.vector2.h"
#include "math.vector3.h"


namespace magnum {namespace math {

template<class T> class Matrix4;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Vector4 {
public:
	T			x,y,z,w;

public:
	static Vector4			Null;
	typedef T				Scalar;

public:
	inline Vector4();
	inline explicit Vector4(const T* );
	inline Vector4(const T& ,const T& , const T& , const T& );
	inline Vector4(const Vector3<T>& );
	inline Vector4(const Vector2<T>& );
	inline Vector4(const Vector<T,4>& );

	inline Vector4			operator+(const Vector4 &u) const;
	inline Vector4			operator+(const T&) const;
	inline Vector4			operator-(const Vector4 &u) const;
	inline Vector4			operator-(const T& s) const;
	inline Vector4			operator-() const;
	inline T				operator*(const Vector4& s) const;
	inline Vector4			operator*(const T& s) const;
	inline Vector4			operator/(const T& s) const;

	inline const Vector4&	operator+= (const T& s);
	inline const Vector4&	operator+= (const Vector4& v);
	inline const Vector4&	operator-= (const T& s);
	inline const Vector4&	operator-= (const Vector4& v);
	inline const Vector4&	operator*= (const T& s);
	inline const Vector4&	operator/= (const T& );
							
	inline const Vector4&	operator=(const Vector<T,4>& );
	inline const Vector4&	operator=(const Vector<T,3>& );
	inline const Vector4&	operator=(const Vector<T,2>& );
	inline const Vector4&	operator=(const Vector3<T>& );
	inline const Vector4&	operator=(const Vector2<T>& );

	inline bool				operator==(const Vector4& v) const;
	inline bool				operator!=(const Vector4& v) const;

	// The following operators are intended to enable the usage of containers
	inline bool				operator<(const Vector4& ) const;
	inline bool				operator>(const Vector4& ) const;
	inline bool				operator<=(const Vector4& ) const;
	inline bool				operator>=(const Vector4& ) const;

	inline					operator Vector<T,4>() const;
	inline					operator const T*() const;
	inline					operator T*();

	// Type conversion routines
	template<class S> inline operator Vector4<S>() const {
		return Vector4<S>(S(x), S(y), S(z), S(w));
	};
	template<class S> inline Vector4<T>(const Vector4<S>& v) {
		x = T(v.x);
		y = T(v.y);
		z = T(v.z);
		w = T(v.w);
	};

	inline Vector2<T>		xy() const;
	inline Vector2<T>		xz() const;
	inline Vector2<T>		yz() const;
	inline Vector2<T>		xw() const;
	inline Vector2<T>		yw() const;
	inline Vector2<T>		zw() const;

	inline Vector3<T>		xyz() const;
	inline Vector3<T>		xyw() const;
	inline Vector3<T>		yzw() const;
							
	inline Vector4&			null();

	inline Vector4&			normalise();

	inline T				length() const;
	inline T				length2() const;

	inline T				max() const;
	inline T				min() const;
	inline T				maxabs() const;
	inline T				minabs() const;

	inline T				sum() const;
	inline T				sumabs() const;
};

template<class T> inline Vector4<T> operator*(const T& ,const Vector4<T>& );
template<class T> inline Vector4<T> operator*(const Matrix<T,4>& ,const Vector4<T>& );
template<class T> inline Vector4<T> operator*(const Vector4<T>& ,const Matrix<T,4>& );


}; };

#endif
