/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MEDIA_BUFFER_H
#define __MEDIA_BUFFER_H

#include "object.h"
#include "system.mutex.h"


namespace magnum { namespace media {
using magnum::object::Object;
using magnum::types::String;
using magnum::object::Object;
using magnum::system::Mutex;

class Node;
class Pin;
class InputPin;
class OutputPin;
class BufferFactory;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Buffer : public Object, private magnum::object::Noncopyable {
DeclareClass(Buffer, Object);
friend class Node;
public:
	explicit Buffer(Node* owner, unsigned int size);
	Buffer(Node* owner, void* data, unsigned int size);
	~Buffer();
	
	inline void		setSize(unsigned int );
	inline unsigned int getSize() const;
	inline unsigned int getMaxSize() const;

	inline void*	getBuffer() const;
	bool			getBuffer(void* ,unsigned int offset=0,unsigned int size=-1) const; 
	bool			setBuffer(const void* ,unsigned int offset=0,unsigned int size=-1); 

	inline InputPin* getConsumer() const;
	inline OutputPin* getProducer() const;
	inline Node*	getOwner() const;
	inline void*	getOwnerData() const;
	inline void		resetOwner();
	inline void		setUserData(void* );
	inline void*	getUserData() const;

private:
	InputPin*		m_Consumer;
	OutputPin*		m_Producer;
	Node*			m_Owner;
	void*			m_OwnerData;
	void*			m_UserData;
	void*			m_Buffer;
	unsigned int	m_Size;
	unsigned int	m_MaxSize;
};

}; };

#endif
