/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MEDIA_DEVICE_H
#define __MEDIA_DEVICE_H

#include "object.h"
#include "media.node.h"
#include "container.list.h"

namespace magnum { namespace media {
using namespace magnum::container;
using magnum::types::String;
using magnum::object::Object;
using magnum::system::Thread;

class Node;
class Filter;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Device : public Object, private magnum::object::Noncopyable {
DeclareClass(Device, Object);
public:
	static const char DefaultAudio[];
	static const char DefaultVideo[];

	typedef Node* (*NodeCreator)(Device* dev, const char* name);
	typedef Device* (*DeviceCreator)(const char* name);

	enum Type {
		Audio,
		Video,
		Mixed
	};

public:
	static Device*			createDevice(const char *which, bool exculsive = false);
	Node*					createNode(const char* which = NULL);
	Filter*					createFilter(const char* which = NULL);

protected:
	static bool				publishDevice(const char* which, Device::DeviceCreator );
	bool					publishNode(const char *nodeName, Node::Type t, Device::NodeCreator c);

private:
	struct DeviceInfo {
		String				m_Name;
		Type				m_Type;
		DeviceCreator		m_Creator;
	};
	struct NodeInfo  {
		String				m_Name;
		Node::Type			m_Type;
		NodeCreator			m_Creator;
	};

	static List<DeviceInfo> m_DeviceList;
	List<NodeInfo>			m_NodeList;
};


}; };

#endif
