/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MEDIA_FORMAT_H
#define __MEDIA_FORMAT_H

#include "object.h"

namespace magnum { namespace media {
using magnum::object::Object;
using magnum::types::String;

/*--------------------------------------------------------------------------*/
/**
 */
class AudioFormat {
public:
	enum Format {
		Unsigned8	= 1,
		Signed8		= 2,
		Unsigned16	= 3,
		Signed16	= 4,
		Unsigned24	= 5,
		Signed24	= 6,
	};
	
public:
	inline void			setChannels(unsigned int );
	inline unsigned int	getChannels() const;
	inline void			setSamplerate(unsigned int );
	inline unsigned int	getSamplerate() const;
	inline void			setFormat(Format );
	inline Format		getFormat() const; 
	
private:
	unsigned int		m_Channels;
	unsigned int		m_Samplerate;
	Format				m_Format;	
};


/*--------------------------------------------------------------------------*/
/**
 */
class VideoFormat {
public:
	enum Format {
		RG15		= 1,
		RGB16		= 2,
		RGB24		= 3,
		RGBA32		= 4,
		YUV			= 5
	};
	
public:
	inline void			setWidth(unsigned int );
	inline unsigned int	getWidth() const;
	inline void			setHeight(unsigned int );
	inline unsigned int	getHeight() const;
	inline void			setFormat(Format );
	inline Format		getFormat() const;
	inline void			setAspect(float );
	inline float		getAspect() const;
	inline void			setSpeed(float );
	inline float		getSpeed() const;
	
private:
	unsigned int		m_Width;
	unsigned int		m_Height;
	Format				m_Format;
	float				m_Aspect;	
	float				m_Speed;
};


/*--------------------------------------------------------------------------*/
/**
 */
class Format : public Object {
DeclareClass(Format, Object);
public:
	enum Type {
		Audio,
		Video,
		Text,
		Mixed,
		Unknown
	};

public:
	inline Format();
	Format(Type );
	
	void				setType(Type );
	inline Type			getType() const;
	
	inline void			setBufferSize(unsigned int );
	inline unsigned int getBufferSize() const;
	inline void			setBitrate(unsigned int );
	inline unsigned int	getBitrate() const;
	inline void			setCodec(const char* );
	inline const String& getCodec() const;
	
	inline AudioFormat&	audio();
	inline const AudioFormat& audio() const;
	inline VideoFormat&	video();
	inline const VideoFormat& video() const;

private:
	Type				m_Type;
	unsigned int		m_BufferSize;
	unsigned int		m_Bitrate;
	String				m_Codec;
	union {
		AudioFormat		m_Audio;
		VideoFormat		m_Video;
	};
};


}; };

#endif
