/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MEDIA_GRAPH_H
#define __MEDIA_GRAPH_H

#include "application.listener.h"
#include "container.chain.h"
#include "media.node.h"
#include "media.looper.h"

namespace magnum { namespace media {
using magnum::container::Chain;
using magnum::object::Object;
using magnum::types::String;
using magnum::application::Dispatcher;

class Buffer;
class Graph;
class Node;
class OutputPin;
class InputPin;
class MediaListener;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Graph : public Looper {
DeclareClass(Graph, Looper);
public:
	typedef Node::State State;
	typedef Chain<Node>::Iterator Iterator;
	
public:
	Graph();
	~Graph();
	
	inline void		insertListener(MediaListener* );
	inline void		removeListener(MediaListener* );
	inline bool		containsListener(MediaListener* ) const;
	inline void		clearListeners();

	bool			insert(Node* node);
	inline bool		contains(Node* node) const;
	bool			remove(Node* node);
	bool			clear();
	
	inline Iterator	begin() const;
	inline Iterator	end() const;
	inline bool		isEmpty() const;
	inline unsigned int getSize() const;

	bool			connect(OutputPin* src, InputPin* dst);
	inline bool		connect(InputPin* dst, OutputPin* src);
	bool			connect(OutputPin* src, InputPin* dst, const Format& fmt);
	inline bool		connect(InputPin* src, OutputPin* dst, const Format& fmt);
	bool			disconnect(OutputPin* src, InputPin* dst);
	inline bool		disconnect(InputPin* dst, OutputPin* src);

	bool			start(bool blocking = true);
	bool			stop(bool blocking = true);
	bool			pause();
	bool			resume();
	bool			reset();

	inline State	getState() const;

protected:
	// Looper methods
	virtual bool	onLoop();
	virtual void	onHandleMessage(const Message& );

	// Start/Stop/Pause - this is system and device specific
	bool			onStart();
	bool			onStop();
	bool			onPause();
	bool			onResume();
	bool			onReset();

	void			onEndOfInput(InputPin* );
	void			onEndOfOutput(OutputPin* );

private:
	bool			negotiateFormat(OutputPin* src, InputPin* dst);
	bool			waitForState(State );
	static bool		threadFunc(void* );
	
private:
	Chain<Node>		m_Nodes;
	State			m_State;
	Thread			m_Thread;
	Dispatcher<MediaListener> m_Dispatcher;
};

}; };

#endif
