/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MEDIA_LOOPER_H
#define __MEDIA_LOOPER_H

#include "object.h"
#include "system.thread.h"
#include "system.mutex.h"
#include "system.signal.h"
#include "container.list.h"
#include "media.message.h"

namespace magnum { namespace media {
using magnum::object::Object;
using magnum::system::Mutex;
using magnum::system::Thread;
using magnum::system::Signal;
using magnum::container::List;
using magnum::types::String;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Looper : public Object, private magnum::object::Noncopyable {
DeclareClass(Looper, Object);
public:
	Looper(Thread* );
	~Looper();

	inline Thread*		getThread() const;

	inline bool			loop();
	
	void				sendMessage(const Message& );
	void				sendMessage(unsigned int );	
	void				sendMessage(unsigned int ,Pin* );	
	void				sendMessage(unsigned int ,Pin* ,void* );	

protected:
	virtual void		onHandleMessage(const Message& );
	virtual void		onRemoveMessage(const Message& );
	virtual bool		onLoop();

	inline void			waitMessage();
	inline void			waitMessage(unsigned int );
	bool				processMessages();
	void				clearMessages();
	bool				getMessage(Message* );
	bool				peekMessage(Message* );				
	
private:
	List<Message>		m_Messages;
	Mutex				m_MessageMutex;
	Signal				m_MessageSignal;
	Thread*				m_Thread;
};


}; };

#endif
