/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MEDIA_MESSAGE_H
#define __MEDIA_MESSAGE_H

namespace magnum { namespace media {

class Pin;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Message {
public:
	enum Command {
		Connect = 1,
		Disconnect = 2,
		Start = 3,
		Stop = 4,
		Pause = 5,
		Resume = 6,
		Reset = 7,
		RequestBuffer = 8,
		ReceiveBuffer = 9,
		EndOfInput = 10,
		EndOfOutput = 11,
		PrivateNode = 0x1000,
		Quit = -1,
	};
	
public:
	inline Message();
	inline explicit Message(unsigned int );
	inline Message(unsigned int ,Pin* );
	inline Message(unsigned int ,Pin* ,void* );
	
	inline void		setCommand(unsigned int );
	inline unsigned int getCommand() const;
	
	inline void		setPin(Pin* );
	inline Pin*		getPin() const;
	
	inline void		setPointer(void* );
	inline void*	getPointer() const;

	inline void		setFloat(float );
	inline float	getFloat() const;

	inline void		setInteger(int );
	inline int		getInteger() const;
		
private:		
	unsigned int	m_Command;
	Pin*			m_Pin;
	union {
		void*		m_Pointer;
		float		m_Float;
		int			m_Integer;
	};
};

}; };

#endif
