/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MEDIA_NODE_INL
#define __MEDIA_NODE_INL

#include "media.buffer.h"


namespace magnum { namespace media {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Node::requestBuffer(InputPin* pin) {
	ASSERT(pin != NULL);
	ASSERT(pin->getNode() == this);
	OutputPin* tpin = pin->getConnection();
	if (tpin == NULL)
		return;
	Node* target = tpin->getNode();
	ASSERT(target != NULL);
	target->sendMessage(Message::RequestBuffer, tpin, NULL);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Node::requestBuffer(InputPin* pin, Buffer* buf) {
	ASSERT(pin != NULL);
	ASSERT(pin->getNode() == this);
	OutputPin* tpin = pin->getConnection();
	if (tpin == NULL) {
		if (buf != NULL)
			buf->release();
		return;
	}
	Node* target = tpin->getNode();
	ASSERT(target != NULL);
	target->sendMessage(Message::RequestBuffer, tpin, buf);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Node::requestBuffer(OutputPin* pin) {
	ASSERT(pin != NULL);
	ASSERT(pin->getNode() == this);
	sendMessage(Message::RequestBuffer, pin, NULL);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Node::requestBuffer(OutputPin* pin, Buffer* buf) {
	ASSERT(pin != NULL);
	ASSERT(pin->getNode() == this);
	sendMessage(Message::RequestBuffer, pin, buf);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Device* Node::getDevice() const {
	return m_Device;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Graph* Node::getGraph() const {
	return m_Graph;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Node::State Node::getState() const {
	return m_State;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String Node::getName() const {
	return m_Name;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const InputPinList& Node::getInputPins() const {
	return m_InputPins;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const OutputPinList& Node::getOutputPins() const {
	return m_OutputPins;
}

}; };

#endif
