/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MEDIA_PIN_H
#define __MEDIA_PIN_H

#include "container.chain.h"
#include "media.format.h"

namespace magnum { namespace media {
using magnum::object::Object;
using magnum::types::String;
using magnum::container::Chainable;
using magnum::container::Chain;

class Node;
class Graph;
class InputPin;
class OutputPin;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Pin : public Object {
DeclareClass(Pin, Object);
friend class Graph;
friend class Node;
public:
	static const char DefaultInput[];
	static const char DefaultOutput[];

public:
	inline Pin(Node* node);

	bool			requestFormat(const Format& );
	bool			adjustFormat(Format& );
	bool			setFormat(const Format& );
	inline const Format& getFormat() const;

	inline Graph*	getGraph() const;
	inline Pin*		getConnection() const;
	inline Node*	getNode() const;

	inline bool		isConnected() const;

private:
	Node*			m_Node;
	Format			m_Format;
	String			m_Name;
	Pin*			m_Connection;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class InputPin : public Pin, private Chainable<InputPin> {
DeclareClass(InputPin, Pin);
friend class Node;
public:
	InputPin(Node* node, bool def = true);
	~InputPin();

	inline OutputPin*	getConnection() const;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class OutputPin : public Pin, private Chainable<OutputPin> {
DeclareClass(OutputPin, Pin);
friend class Node;
public:
	OutputPin(Node* node, bool def = true);
	~OutputPin();
	
	inline InputPin*	getConnection() const;
};


typedef Chain<InputPin>		InputPinList;
typedef Chain<OutputPin>	OutputPinList;

}; };

#endif
