/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MEDIA_READER_H
#define __MEDIA_READER_H

#include "object.pointer.h"
#include "io.file.h"
#include "media.node.h"
#include "media.pin.h"

namespace magnum { namespace media {
using magnum::object::Pointer;
using magnum::io::File;
using magnum::io::InputStream;

/*--------------------------------------------------------------------------*/
/**
 */
class Reader : public Node {
DeclareClass(Reader, Node);
protected:
	Reader();
	Reader(File* );
	Reader(InputStream* );
	
public:
	~Reader();
	
	inline OutputPin*		getPin();
	inline const OutputPin*	getPin() const;
	inline File*			getFile() const;
	inline InputStream*		getStream() const;
	
protected:
	inline bool				setFormat(const Format& );
	inline bool				adjustFormat(Format& );
	inline bool				requestFormat(const Format& );

	// Format methods
	virtual bool			onSetFormat(Pin* ,const Format& );
	virtual bool			onAdjustFormat(Pin* ,Format& );

	// Buffer routines
	virtual bool			onRequestBuffer(OutputPin* ,Buffer* );
	
	// Routines for the decoder
	virtual bool			onSetFormat(const Format& );
	virtual bool			onAdjustFormat(Format& );
	virtual bool			onRead(Buffer* );

private:
	OutputPin				m_Pin;
	Pointer<File>			m_File;
	Pointer<InputStream>	m_Stream;
};

}; };

#endif
