/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __MEDIA_WAVEOUTPUT_H
#define __MEDIA_WAVEOUTPUT_H

#include "system.atom.h"
#include "container.list.h"
#include "media.audio.h"

#if defined(__FORWINDOWS__)
#	include <MMSystem.h>
#endif


namespace magnum { namespace media { 
using magnum::object::Object;
using magnum::types::String;
using magnum::container::List;
using magnum::system::Atom;

class WaveDevice;
class WaveBuffer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class WaveOutput : public AudioConsumer {
DeclareClass(WaveOutput, AudioConsumer);
friend class WaveDevice;
public:
	WaveOutput(WaveDevice* device);
	~WaveOutput();

	inline unsigned int	getBufferSize() const;
	inline unsigned int	getBufferCount() const;

protected:
	// Message methods
	virtual bool		onLoop();
	virtual void		onHandleMessage(const Message& );

	// Buffer methods
	virtual void		onDestroyBuffer(Buffer* );
	virtual bool		onReceiveBuffer(InputPin* ,Buffer* );
	virtual void		onEndOfInput(InputPin* );

	// Format methods
	virtual bool		onAdjustFormat(Pin* ,Format& );
	virtual bool		onSetFormat(Pin* ,const Format& );

	// Start/Stop/Pause - this is system and device specific
	virtual bool		onStart();
	virtual bool		onStop();
	virtual bool		onPause();
	virtual bool		onResume();
	virtual bool		onReset();

private:
	Buffer*				createBuffer();
	bool				releaseBuffer(Buffer* buf);
	void				insertPreBuffer(Buffer* );
	void				playPreBuffers();
	void				releasePreBuffers();

	static void CALLBACK callback(HWAVEIN hwi, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2);

private:
	enum Command {
		ReleaseBuffer	= Message::PrivateNode
	};
	
	InputPin			m_InputPin;
	unsigned int		m_BufferSize;
	unsigned int		m_BufferCount;
	unsigned int		m_BufferLength;
	Atom				m_BufferQueue;
	HWAVEOUT			m_WaveHandle;
	WAVEFORMATEX		m_Format;
	List<WAVEHDR>		m_Buffers;
	Mutex				m_BufferMutex;
	bool				m_NoRequests;
	List<Buffer*>		m_PreBuffers;
};

}; }; 

#endif
