/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __NET_ADDRESS_H
#define __NET_ADDRESS_H

#include "object.h"
#include "base.string.h"

# if defined(__FORWINDOWS__)
#	include <winsock2.h>
#	include <ws2tcpip.h>
#else
#	include <unistd.h>
#	include <sys/socket.h>
#	include <netinet/in.h>
#	include <netdb.h>
# endif


namespace magnum { namespace net {
using magnum::object::Object;
using magnum::types::String;
using magnum::types::uint32;

class Socket;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Address : public Object {
DeclareClass(Address, Object);
public:
	Address();
	Address(const char* hostname);
	Address(uint32 address);
	
	inline void			setHostname(const char* hostname);
	inline String		getHostname() const;

	inline void			setAddress(uint32 address);
	inline uint32		getAddress() const;

	inline uint32		getAddressNO() const;
	inline uint32		getAddressHO() const;

	inline void			setAddressNO(uint32 address);
	inline void			setAddressHO(uint32 address);

	inline void			clear();

	inline bool			isValid() const;
	String				toString() const;

	bool 				operator==(const Address& adr) const;
	bool 				operator!=(const Address& adr) const;

	static Address		getByName(const char* hostname);
	static Address		getByAddress(uint32 address);
	static Address		getLocalHost();

protected:
	bool				resolveHostname();
	bool				resolveAddress();

private:
	String				m_Hostname;
	bool				m_HostnameValid;
	uint32				m_Address;
	bool				m_AddressValid;
};

}; };
#endif
