/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __NET_ADDRESS_INL
#define __NET_ADDRESS_INL

namespace magnum { namespace net {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String Address::getHostname() const {
	if (!m_HostnameValid)
		if (!((Address *)this)->resolveHostname())
			return String();
	return m_Hostname;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline uint32 Address::getAddress() const {
	return getAddressHO();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline uint32 Address::getAddressNO() const {
	if (!m_AddressValid)
		if (!((Address *)this)->resolveAddress())
			return 0;
	return m_Address;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline uint32 Address::getAddressHO() const {
	if (!m_AddressValid)
		if (!((Address *)this)->resolveAddress())
			return 0;
	return ntohl(m_Address);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Address::setHostname(const char* hostname) {
	m_Hostname = hostname;
	m_HostnameValid = true;
	m_AddressValid = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Address::setAddress(uint32 address) {
	setAddressHO(address);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Address::setAddressHO(uint32 address) {
	m_Address = htonl(address);
	m_AddressValid = true;
	m_HostnameValid = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Address::setAddressNO(uint32 address) {
	m_Address = address;
	m_AddressValid = true;
	m_HostnameValid = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Address Address::getByName(const char* hostname) {
	return Address(hostname);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Address Address::getByAddress(uint32 address) {
	return Address(address);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Address::clear()  {
	m_HostnameValid = false;
	m_AddressValid = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Address::isValid() const {
	return (m_HostnameValid || m_AddressValid);
}

}; };
#endif
