/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __NET_DATAGRAMSOCKET_H
#define __NET_DATAGRAMSOCKET_H

#include "net.socket.h"

namespace magnum { namespace net {
using magnum::object::Object;
using magnum::types::String;

class DatagramPacket;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class DatagramSocket : public Socket {
DeclareClass(DatagramSocket, Socket);
public:
	inline DatagramSocket();
	inline DatagramSocket(unsigned int port);
	inline DatagramSocket(const Address& localAdr, unsigned int port);
	inline DatagramSocket(const String& localAdr, unsigned int port);

	int						receive(DatagramPacket &p);
	int						receiveFrom(DatagramPacket &p);
	int						send(DatagramPacket &p);
	int						sendTo(DatagramPacket &p);

	inline unsigned int		getMaximumMessageSize();
};

typedef DatagramSocket GUDPSocket;

/*
 int getSoTimeout() 
          Retrive setting for SO_TIMEOUT. 
 void setSoTimeout(int timeout) 
          Enable/disable SO_TIMEOUT with the specified timeout, in milliseconds. 
*/  


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class DatagramPacket : public Object {
DeclareClass(DatagramPacket, Object);
friend class DatagramSocket;
private:
	Address	m_Address;
	unsigned short	m_Port;

#if defined(__FORWINDOWS__)
	WSABUF			m_Buffer;
#else
	void*			m_Buffer;
	unsigned int	m_Length;
#endif
	unsigned int	m_BufferSize;

public:
	inline DatagramPacket();
	inline ~DatagramPacket();

	inline const Address&		getAddress() const;
	inline void					setAddress(const Address& adr);
	inline void					setAddress(const String& adr);
	inline void					clearAddress();

	inline unsigned int			getPort() const;
	inline void					setPort(unsigned int port);

	inline unsigned int			getSize() const;
	inline void					setSize(unsigned int len);

	inline void*				getData();
	inline void					setData(const void* src, unsigned int len);
	inline void					setData(const String& str);

	inline String				toString() const;
};

}; };

#endif
