/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __NET_HTTPINPUTSTREAM_H
#define __NET_HTTPINPUTSTREAM_H

#include "io.inputstream.h"

namespace magnum { namespace net {
using magnum::object::Object;
using magnum::types::String;
using magnum::types::int64;
using magnum::types::byte;
using magnum::io::InputStream;

class HttpClientConnection;

/*--------------------------------------------------------------------------*/
/** A specialised InputStream class that can be used to read data from 
 * a HttpClientConnection. The stream is a buffered InputStream, so the 
 * application need not perform its own buffering.
 */
class HttpInputStream : public InputStream {
DeclareClass(HttpInputStream, InputStream);
friend class HttpClientConnection;
public:
	static const int	BufferSize = 4096;

private:
	HttpInputStream(HttpClientConnection* );

public:
	~HttpInputStream();
	
	inline HttpClientConnection* getConnection() const;
	inline size_t		available() const;
	
protected:
	virtual size_t 		onRead(void *buffer, size_t size);
	virtual size_t 		onSkip(size_t size);
	virtual int64 		onAvailable();
	virtual bool		onClose();
	
private:
	void				detach();
	bool				fillBuffer();
	
private:
	InputStream*		m_Stream;
	HttpClientConnection* m_Connection;
	size_t				m_StreamLeft;
	size_t				m_BufferLeft;
	byte*				m_BufferCurrent;
	byte				m_Buffer[BufferSize];
};

}; };

#endif
