/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __NET_HTTPREQUEST_INL
#define __NET_HTTPREQUEST_INL

namespace magnum { namespace net {

/*--------------------------------------------------------------------------*/
/** Returns the timeout value (in milliseconds) used when establishing a new 
 * TCP/IP connection for this HttpRequest.
 */
inline unsigned int HttpRequest::getConnectionTimeout() const {
	return m_ConnectionTimeout;
}


/*--------------------------------------------------------------------------*/
/** Returns the timeout value (in milliseconds) used for TCP/IP network send 
 * and receive operations for this HttpRequest.
 */
inline unsigned int HttpRequest::getTimeout() const {
	return m_Timeout;
}


/*--------------------------------------------------------------------------*/
/** Sets the timeout value (in milliseconds) used when establishing a new 
 * TCP/IP connection for this HttpRequest.
 */
inline void HttpRequest::setConnectionTimeout(unsigned int t) {
	m_ConnectionTimeout = t;
}


/*--------------------------------------------------------------------------*/
/** Sets the timeout value (in milliseconds) used for TCP/IP network send and 
 * receive operations for this HttpRequest. When a TCP/IP connection is 
 * allocated to a HttpRequest, the SO_TIMEOUT value of the underlying socket 
 * is configured to use the value set by this method.
 */
inline void HttpRequest::setTimeout(unsigned int t) {
	m_Timeout = t;
}


/*--------------------------------------------------------------------------*/
/** Returns the URL of the resource that will be accessed via this 
 * HttpRequest.
 */
inline const URL& HttpRequest::getURL() const {
	return m_Url;
}


/*--------------------------------------------------------------------------*/
/** Sets the URL of the resource that will be accessed via this HttpRequest. 
 * The URL is used when sending a HTTP request to a server, and is used to
 * identify a resource on an origin server.
 */
inline void HttpRequest::setURL(const URL& url) {
	m_Url = url;
}


/*--------------------------------------------------------------------------*/
/** Returns The HTTP method (e.g. GET or POST) for this HTTP request.
 */
inline const String& HttpRequest::getMethod() const {
	return m_Method;
}


/*--------------------------------------------------------------------------*/
/** Sets the HTTP method to be used when sending this HttpRequest to a 
 * server. The default method is GET, but other methods may be configured 
 * by calling this member function before the request is sent.
 */
inline void HttpRequest::setMethod(const char* m) {
	m_Method = m;
}


/*--------------------------------------------------------------------------*/
/** Returns the sequence of MIME-type headers associated with this 
 * HttpRequest. When a HttpRequest is first constructed, its MIME header 
 * sequence is empty.
 */
inline MimeHeaders& HttpRequest::getHeaders() {
	return m_Headers;
}


/*--------------------------------------------------------------------------*/
/** Returns the sequence of MIME-type headers associated with this 
 * HttpRequest. When a HttpRequest is first constructed, its MIME header 
 * sequence is empty.
 */
inline const MimeHeaders& HttpRequest::getHeaders() const {
	return m_Headers;
}


/*--------------------------------------------------------------------------*/
/** 
 */
inline void HttpRequest::setHeaders(const MimeHeaders& h) {
	m_Headers = h;
}


/*--------------------------------------------------------------------------*/
/** 
 */
inline const HttpVersion& HttpRequest::getVersion() const {
	return m_Version;
}


/*--------------------------------------------------------------------------*/
/** 
 */
inline void HttpRequest::setVersion(const HttpVersion& v) {
	m_Version = v;
}

}; };

#endif
