/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __NET_HTTPRESPONSE_H
#define __NET_HTTPRESPONSE_H

#include "object.pointer.h"
#include "io.inputstream.h"
#include "net.url.h"
#include "net.mimeheaders.h"
#include "net.httpversion.h"


namespace magnum { namespace net {
using magnum::io::InputStream;
using magnum::object::Pointer;

class HttpRequest;

/*--------------------------------------------------------------------------*/
/**
 */
class HttpResponse : public Object {
DeclareClass(HttpResponse, Object);
public:
	enum ResponseCode { 	
		HttpOk = 200,  	 
		HttpCreated = 201,  	 
		HttpAccepted = 202,  	 
		HttpNotAuthoritative = 203,  	 
		HttpNoContent = 204,  	 
		HttpReset = 205,  	 
		HttpPartial = 206,  	 

		HttpMultChoice = 300,  	 
		HttpMovedPerm = 301,  	 
		HttpMovedTemp = 302,  	 
		HttpSeeOther = 303, 	 
		HttpNotModified = 304,  	 
		HttpUseProxy = 305,  	 

		HttpBadRequest = 400,  	 
		HttpUnauthorized = 401,	 
		HttpPaymentRequired = 402,  
		HttpForbidden = 403,  	 
		HttpNotFound = 404,  	 
		HttpBadMethod = 405,  	 
		HttpNotAcceptable = 406,  	 
		HttpProxyAuth = 407,  	 
		HttpClientTimeout = 408,  	 
		HttpConflict = 409,  	 
		HttpGone = 410,  	 
		HttpLengthRequired = 411,  	 
		HttpPreconFailed = 412, 	 
		HttpEntityTooLarge = 413,  	 
		HttpReqTooLong = 414,
		HttpUnsupportedType = 415,  	 

		HttpInternalError = 500,  	 
		HttpNotImplemented = 501,  	 
		HttpBadGateway = 502,  	 
		HttpUnavailable = 503,  	 
		HttpGatewayTimeout = 504,  	 
		HttpUnsupportedVersion = 505
	};
  	
public:
	HttpResponse();
	HttpResponse(const HttpRequest& );
	
	inline const URL&		getURL() const;
	inline void				setURL(const URL& );
	
	inline const HttpVersion& getVersion() const;
	inline void				setVersion(const HttpVersion& );

	inline const String&	getMethod() const;
	inline void				setMethod(const char* );
	
	inline void				setHeaders(const MimeHeaders& );
	inline MimeHeaders&		getHeaders();
	inline const MimeHeaders& getHeaders() const;

	inline void				setCode(ResponseCode );
	inline ResponseCode		getCode() const;

	inline void				setInputStream(InputStream* );	
	inline InputStream*		getInputStream();
	void					discard();
	
private:
	URL						m_Url;
	MimeHeaders				m_Headers;
	String					m_Method;
	HttpVersion				m_Version;
	Pointer<InputStream>	m_InputStream;
	ResponseCode			m_Response;
};

}; };

#endif
