/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __NET_MULTICASTSOCKET_INL
#define __NET_MULTICASTSOCKET_INL

namespace magnum { namespace net {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline MulticastSocket::MulticastSocket() : DatagramSocket() {
	in_addr adr;
	int optlen = sizeof(adr);
	getsockopt(m_Socket, IPPROTO_IP, IP_MULTICAST_IF, (char *)&adr, (int *)&optlen);
#if defined(__FORWINDOWS__)
	m_Interface.setAddressNO(adr.S_un.S_addr);
#else
	m_Interface.setAddressNO(adr.s_addr);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline MulticastSocket::MulticastSocket(unsigned int port) : DatagramSocket(port) {
	in_addr adr;
	int optlen = sizeof(adr);
	getsockopt(m_Socket, IPPROTO_IP, IP_MULTICAST_IF, (char *)&adr, (int *)&optlen);
#if defined(__FORWINDOWS__)
	m_Interface.setAddressNO(adr.S_un.S_addr);
#else
	m_Interface.setAddressNO(adr.s_addr);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int MulticastSocket::getTimeToLive() const {
	unsigned int size;
	int optlen = sizeof(size);
	getsockopt(m_Socket, IPPROTO_IP, IP_MULTICAST_TTL, (char *)&size, (int *)&optlen);
	return size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MulticastSocket::setTimeToLive(unsigned int ttl) {
	setsockopt(m_Socket, IPPROTO_IP, IP_MULTICAST_TTL, (char *)&ttl, sizeof(ttl));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool MulticastSocket::getLoopback() const {
	bool b;
	int optlen = sizeof(b);
	getsockopt(m_Socket, IPPROTO_IP, IP_MULTICAST_LOOP , (char *)&b, (int *)&optlen);
	return (b == TRUE);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void MulticastSocket::setLoopback(bool b) {
	bool val = b;
	setsockopt(m_Socket, IPPROTO_IP, IP_MULTICAST_LOOP, (char *)&val, sizeof(val));
}


}; };
#endif
