/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __NET_SERVER_H
#define __NET_SERVER_H

#include "io.inputstream.h"
#include "io.outputstream.h"
#include "net.serversocket.h"
#include "system.thread.h"


namespace magnum { namespace net {
using magnum::object::Object;
using magnum::types::String;
using magnum::io::InputStream;
using magnum::io::OutputStream;
using magnum::system::Thread;

class ServerThread;
class StreamSocket;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Server : public Thread {
DeclareClass(Server, Thread);
protected:
	ServerSocket			m_Socket;

public:
	inline bool				bind(const Address& lcoalAdr, unsigned int localPort);
	inline bool				bind(const String& lcoalAdr, unsigned int localPort);
	inline bool				bind(unsigned int localPort);

	inline unsigned int		getPort() const;

protected:
	inline bool				listen(unsigned int logback);
	ServerThread*			accept();
	bool					run();

	virtual ServerThread*	createThread() = 0;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class ServerThread : public Thread {
DeclareClass(ServerThread, Thread);
friend class Server;
private:
	StreamSocket*			m_Socket;
	Server*					m_Server;

protected:
	bool					run();
	inline bool				disconnect();

	inline InputStream*		getInputStream();
	inline void				shutdownInput();
	inline OutputStream*	getOutputStream();
	inline void				shutdownOutput();

	inline unsigned int		getSendBufferSize() const;
	inline void				setSendBufferSize(unsigned int size);

	inline unsigned int		getReceiveBufferSize() const;
	inline void				setReceiveBufferSize(unsigned int size);

	inline const Address&	getAddress() const;
	inline unsigned int		getPort() const;

	inline const Address&	getLocalAddress() const;
	inline unsigned int		getLocalPort() const;

	inline bool				wait(Socket::WaitMode mode);
	inline bool				wait(Socket::WaitMode mode, unsigned int millis);
};

}; };
#endif
