/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __NET_SERVERSOCKET_INL
#define __NET_SERVERSOCKET_INL

namespace magnum { namespace net {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Address& ServerSocket::getAddress() const {
	return m_Address;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int ServerSocket::getPort() const {
	return ntohs((unsigned short)m_Port);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool ServerSocket::listen(unsigned int backlog) {
	return (::listen(m_Socket, backlog) == 0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int ServerSocket::getSendBufferSize() const {
	unsigned int size;
	int optlen = sizeof(size);
	getsockopt(m_Socket, SOL_SOCKET, SO_SNDBUF, (char *)&size, (int *)&optlen);
	return size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ServerSocket::setSendBufferSize(unsigned int size) {
	setsockopt(m_Socket, SOL_SOCKET, SO_SNDBUF, (char *)&size, sizeof(size));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int ServerSocket::getReceiveBufferSize() const {
	unsigned int size;
	int optlen = sizeof(size);
	getsockopt(m_Socket, SOL_SOCKET, SO_RCVBUF, (char *)&size, (int *)&optlen);
	return size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ServerSocket::setReceiveBufferSize(unsigned int size) {
	setsockopt(m_Socket, SOL_SOCKET, SO_RCVBUF, (char *)&size, sizeof(size));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool	ServerSocket::getKeepAlive() const {
	bool b;
	int len = sizeof(b);
	getsockopt(m_Socket, SOL_SOCKET, SO_KEEPALIVE, (char *)&b, &len);
	return (b == TRUE);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ServerSocket::setKeepAlive(bool val) {
	bool b = val;
	setsockopt(m_Socket, SOL_SOCKET, SO_KEEPALIVE, (char *)&b, sizeof(b));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool ServerSocket::getNoDelay() const {
	bool b;
	int len = sizeof(b);
	getsockopt(m_Socket, IPPROTO_TCP, TCP_NODELAY, (char *)&b, &len);
	return (b == TRUE);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	ServerSocket::setNoDelay(bool val) {
	bool b = val;
	setsockopt(m_Socket, IPPROTO_TCP, TCP_NODELAY, (char *)&b, sizeof(b));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int ServerSocket::getSendTimeout() const {
	unsigned int to;
	int len = sizeof(to);
	getsockopt(m_Socket, SOL_SOCKET, SO_SNDTIMEO, (char *)&to, &len);
	return to;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ServerSocket::setSendTimeout(unsigned int to) {
	setsockopt(m_Socket, SOL_SOCKET, SO_SNDTIMEO, (char *)&to, sizeof(to));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int ServerSocket::getReceiveTimeout() const {
	unsigned int to;
	int len = sizeof(to);
	getsockopt(m_Socket, SOL_SOCKET, SO_RCVTIMEO, (char *)&to, &len);
	return to;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void ServerSocket::setReceiveTimeout(unsigned int to) {
	setsockopt(m_Socket, SOL_SOCKET, SO_RCVTIMEO, (char *)&to, sizeof(to));
}


}; };

#endif
