/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __NET_SOCKET_INL
#define __NET_SOCKET_INL

namespace magnum { namespace net {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Socket::isConnected() const {
	return m_Connected;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Socket::isBound() const {
	return m_Bound;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Socket::connect(const char* remoteAdr, unsigned int port) {
	return connect(Address(remoteAdr), port);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Address& Socket::getAddress() const {
	return m_RemoteAddress;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Socket::getPort() const {
	return ntohs((unsigned short)m_RemotePort);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Address& Socket::getLocalAddress() const {
	return m_LocalAddress;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Socket::getLocalPort() const {
	return ntohs((unsigned short)m_LocalPort);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline InputStream* Socket::getInputStream() {
	return &m_InputStream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline OutputStream* Socket::getOutputStream() {
	return &m_OutputStream;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Socket::getSendBufferSize() const {
	unsigned int size;
	int optlen = sizeof(size);
	getsockopt(m_Socket, SOL_SOCKET, SO_SNDBUF, (char *)&size, (int *)&optlen);
	return size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Socket::setSendBufferSize(unsigned int size) {
	setsockopt(m_Socket, SOL_SOCKET, SO_SNDBUF, (char *)&size, sizeof(size));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Socket::getReceiveBufferSize() const {
	unsigned int size;
	int optlen = sizeof(size);
	getsockopt(m_Socket, SOL_SOCKET, SO_RCVBUF, (char *)&size, (int *)&optlen);
	return size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Socket::setReceiveBufferSize(unsigned int size) {
	setsockopt(m_Socket, SOL_SOCKET, SO_RCVBUF, (char *)&size, sizeof(size));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Socket::getTimeToLive() const {
	unsigned int ttl;
	int optlen = sizeof(ttl);
	getsockopt(m_Socket, IPPROTO_IP, IP_TTL, (char *)&ttl, (int *)&optlen);
	return ttl;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Socket::setTimeToLive(unsigned int ttl) {
	setsockopt(m_Socket, IPPROTO_IP, IP_TTL, (char *)&ttl, sizeof(ttl));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Socket::getSendTimeout() const {
	unsigned int to;
	int len = sizeof(to);
	getsockopt(m_Socket, SOL_SOCKET, SO_SNDTIMEO, (char *)&to, &len);
	return to;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Socket::setSendTimeout(unsigned int to) {
	setsockopt(m_Socket, SOL_SOCKET, SO_SNDTIMEO, (char *)&to, sizeof(to));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int Socket::getReceiveTimeout() const {
	unsigned int to;
	int len = sizeof(to);
	getsockopt(m_Socket, SOL_SOCKET, SO_RCVTIMEO, (char *)&to, &len);
	return to;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Socket::setReceiveTimeout(unsigned int to) {
	setsockopt(m_Socket, SOL_SOCKET, SO_RCVTIMEO, (char *)&to, sizeof(to));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Socket::throwSocketException() {
#if defined(__FORWINDOWS__)
	throwSocketException(WSAGetLastError());
#else
	throwSocketException(errno);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Socket::throwBindException() {
#if defined(__FORWINDOWS__)
	throwBindException(WSAGetLastError());
#else
	throwBindException(errno);
#endif
}


};
};
#endif
