/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __OBJECT_ATTACHMENT_H
#define __OBJECT_ATTACHMENT_H

#include "container.chain.h"

namespace magnum { namespace object {
using magnum::types::String;
using magnum::container::Chain;
using magnum::container::Chainable;


template<class T> class Attachment;
class AttachmentList;

/*--------------------------------------------------------------------------*/
/**
 */
class AttachmentBase : private Chainable<AttachmentBase>, private Noncopyable {
friend Chain<AttachmentBase>;
friend AttachmentList;
protected:
	inline AttachmentBase(const void* );
	inline AttachmentBase(const void* ,const char* );
	
public:
	inline const char*		getName() const;
	inline void				setName(const char* );
	
	inline const void*		getType() const;
	
private:
	inline void				refer();	
	inline void				release();
	
private:
	char					m_Name[64];
	const void*				m_Type;
};


/*--------------------------------------------------------------------------*/
/**
 */
template<class T> class Attachment : public AttachmentBase {
public:
	inline Attachment();
	inline Attachment(const char* );
	inline Attachment(const char* ,const T& );
	
	inline T&				get();
	inline const T&			get() const;
	inline void				set(const T& );
	
	inline					operator T&();
	inline					operator const T&() const;
	
	static inline Attachment* dcast(AttachmentBase* );
	static inline const void* getType();
	
private:
	T						m_Data;
	static int				m_TypeId;
};


/*--------------------------------------------------------------------------*/
/**
 */
class AttachmentList : public Object {
DeclareClass(AttachmentList, Object);
public:
	typedef Chain<AttachmentBase>::Iterator Iterator;
	
public:
	AttachmentList();
	~AttachmentList();
	
	template<class T>
	Attachment<T>*			insert(const char* ,const T& );
	bool					remove(const char* );
	void					remove(AttachmentBase* );
	void					clear();
	
	template<class T>
	bool					get(const char* ,T* ) const;
	
	AttachmentBase*			find(const char* );
	const AttachmentBase*	find(const char* ) const;
	
	inline Iterator			begin() const;
	inline Iterator			end() const;
	inline Iterator			at(AttachmentBase* ) const;
	
	inline AttachmentBase*	operator[](const char* );
	inline const AttachmentBase* operator[](const char* ) const;
	
private:
	Chain<AttachmentBase>	m_Attachments;
};

}; };

#endif
