/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __OBJECT_ATTACHMENT_INL
#define __OBJECT_ATTACHMENT_INL

namespace magnum { namespace object {

/*--------------------------------------------------------------------------*/
/**
 */
inline AttachmentBase::AttachmentBase(const void* t) : m_Type(t) {
	m_Name[0] = 0;
}


/*--------------------------------------------------------------------------*/
/**
 */
inline AttachmentBase::AttachmentBase(const void* t,const char* n) : m_Type(t) {
	strncpy(m_Name, n, sizeof(m_Name));
}


/*--------------------------------------------------------------------------*/
/**
 */
inline const void* AttachmentBase::getType() const {
	return m_Type;
}


/*--------------------------------------------------------------------------*/
/**
 */
inline const char* AttachmentBase::getName() const {
	return m_Name;
}


/*--------------------------------------------------------------------------*/
/**
 */
inline void AttachmentBase::setName(const char* n) {
	strncpy(m_Name, n, sizeof(m_Name));
}


/*--------------------------------------------------------------------------*/
/**
 */
inline void AttachmentBase::refer() {
}


/*--------------------------------------------------------------------------*/
/**
 */
inline void AttachmentBase::release() {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
inline Attachment<T>::Attachment() : AttachmentBase(&m_TypeId) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
inline Attachment<T>::Attachment(const char* n) : AttachmentBase(&m_TypeId, n) {
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
inline Attachment<T>::Attachment(const char* n,const T& t) : AttachmentBase(&m_TypeId, n), m_Data(t) {
}



/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
inline T& Attachment<T>::get() {
	return m_Data;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
inline const T& Attachment<T>::get() const {
	return m_Data;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
inline void Attachment<T>::set(const T& t) {
	m_Data = t;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
inline Attachment<T>::operator T&() {
	return m_Data;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
inline Attachment<T>::operator const T&() const {
	return m_Data;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
inline const void* Attachment<T>::getType() {
	return &m_TypeId;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
inline Attachment<T>* Attachment<T>::dcast(AttachmentBase* a) {
	if (a->getType() == &m_TypeId) 
		return static_cast<Attachment<T>*>(a);
	return NULL;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
int Attachment<T>::m_TypeId;


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
Attachment<T>* AttachmentList::insert(const char* n,const T& t) {
	AttachmentBase* a = find(n);
	if (a != NULL) {
		Attachment<T>* att = Attachment<T>::dcast(a);
		if (att != NULL) {
			att->set(t);
			return att;	
		}
		m_Attachments.remove(a);
		delete a;
	}
	
	Attachment<T>* att = new Attachment<T>(n, t);
	m_Attachments.insert(att);
	return att;
}


/*--------------------------------------------------------------------------*/
/**
 */
template<class T>
bool AttachmentList::get(const char* n,T* t) const {
	ASSERT(n != NULL);
	ASSERT(t != NULL);
	const AttachmentBase* a = find(n);
	if (a == NULL)
		return false;
	if (Attachment<T>::getType() != a->getType())
		return false;
	*t = a->get();
	return true;
}


/*--------------------------------------------------------------------------*/
/**
 */
inline AttachmentList::Iterator AttachmentList::begin() const {
	return m_Attachments.begin();
}


/*--------------------------------------------------------------------------*/
/**
 */
inline AttachmentList::Iterator AttachmentList::end() const {
	return m_Attachments.end();
}


/*--------------------------------------------------------------------------*/
/**
 */
inline AttachmentList::Iterator AttachmentList::at(AttachmentBase* a) const {
	return m_Attachments.at(a);
}


/*--------------------------------------------------------------------------*/
/**
 */
inline AttachmentBase* AttachmentList::operator[](const char* n) {
	return find(n);
}


/*--------------------------------------------------------------------------*/
/**
 */
inline const AttachmentBase* AttachmentList::operator[](const char* n) const {
	return find(n);
}


}; };

#endif
