/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __OBJECT_AUTOPOINTER_H
#define __OBJECT_AUTOPOINTER_H

#include "base.h"

namespace magnum { namespace object {
template<class T> class Pointer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class AutoPointer {
public:
	inline AutoPointer();
	inline AutoPointer(T* ptr);
	inline AutoPointer(const Pointer<T>& );
	inline AutoPointer(const AutoPointer<T>& );
	inline ~AutoPointer();

	inline T*			operator->();
	inline const T*		operator->() const;
	inline				operator T*() const;
	
	AutoPointer&		operator=(T* );
	AutoPointer&		operator=(const Pointer<T>& );
	AutoPointer&		operator=(const AutoPointer<T>& );

	inline bool			isNull() const;
	inline void			setNull();
	inline void			release();

	inline bool			operator==(const AutoPointer<T>& lhs) const;
	inline bool			operator!=(const AutoPointer<T>& lhs) const;
	inline bool			operator<(const AutoPointer<T>& lhs) const;
	inline bool			operator>(const AutoPointer<T>& lhs) const;
	inline bool			operator<=(const AutoPointer<T>& lhs) const;
	inline bool			operator>=(const AutoPointer<T>& lhs) const;
	inline bool			operator==(const T* lhs) const;
	inline bool			operator==(T* lhs) const;
	inline bool			operator==(intptr_t lhs) const;
	inline bool			operator!=(const T* lhs) const;
	inline bool			operator!=(T* lhs) const;
	inline bool			operator!=(intptr_t lhs) const;

private:
	T*			m_Pointer;
};

}; };

#include "object.pointer.h"

#endif
