/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __OBJECT_AUTOPOINTER_INL
#define __OBJECT_AUTOPOINTER_INL

namespace magnum { namespace object {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline AutoPointer<T>::AutoPointer() {
	m_Pointer = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline AutoPointer<T>::AutoPointer(const Pointer<T>& ptr) {
	m_Pointer = (T*)ptr;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline AutoPointer<T>::AutoPointer(const AutoPointer<T>& ptr) {
	m_Pointer = ptr.m_Pointer;
	if (m_Pointer != NULL)
		m_Pointer->refer();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline AutoPointer<T>::AutoPointer(T* ptr) {
	m_Pointer = ptr;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline AutoPointer<T>::~AutoPointer() {
	if (m_Pointer != NULL)
		m_Pointer->release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
AutoPointer<T>& AutoPointer<T>::operator=(T* ptr) {
	if (m_Pointer != NULL)
		m_Pointer->release();
	m_Pointer = ptr;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
AutoPointer<T>& AutoPointer<T>::operator=(const Pointer<T>& ptr) {
	if (m_Pointer != NULL)
		m_Pointer->release();
	m_Pointer = (T*)ptr;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
AutoPointer<T>& AutoPointer<T>::operator=(const AutoPointer<T>& ptr) {
	if (m_Pointer != NULL)
		m_Pointer->release();
	m_Pointer = ptr.m_Pointer;
	if (m_Pointer != NULL)
		m_Pointer->refer();
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline T* AutoPointer<T>::operator->() {
	return m_Pointer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const T* AutoPointer<T>::operator->() const {
	return m_Pointer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline AutoPointer<T>::operator T*() const {
	return m_Pointer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::isNull() const {
	return (m_Pointer == NULL);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void AutoPointer<T>::setNull() {
	if (m_Pointer != NULL) {
		m_Pointer->release();
		m_Pointer = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void AutoPointer<T>::release() {
	if (m_Pointer != NULL) {
		m_Pointer->release();
		m_Pointer = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator==(const AutoPointer<T>& lhs) const {
	return (lhs.m_Pointer == m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator!=(const AutoPointer<T>& lhs) const {
	return (lhs.m_Pointer != m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator<(const AutoPointer<T>& lhs) const {
	return (lhs.m_Pointer > m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator>(const AutoPointer<T>& lhs) const {
	return (lhs.m_Pointer < m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator<=(const AutoPointer<T>& lhs) const {
	return (lhs.m_Pointer >= m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator>=(const AutoPointer<T>& lhs) const {
	return (lhs.m_Pointer <= m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator==(const T* lhs) const {
	return (m_Pointer == lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator!=(const T* lhs) const {
	return (m_Pointer != lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator==(T* lhs) const {
	return (m_Pointer == lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator!=(T* lhs) const {
	return (m_Pointer != lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator==(intptr_t lhs) const {
	return (m_Pointer == (T*)(void*)lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool AutoPointer<T>::operator!=(intptr_t lhs) const {
	return (m_Pointer != (T*)(void*)lhs);
}

}; };

#endif
