/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __OBJECT_OBJECT_H
#define __OBJECT_OBJECT_H

#include "base.h"
#include "base.intrinsics.h"


namespace magnum { namespace system {
class Thread;
}; };


namespace magnum { namespace types {
class String;
}; };


namespace magnum {  namespace object {
using namespace magnum::system;
using namespace magnum::types;

class Class;
class SoftPointerBase;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Object {
friend class SoftPointerBase;
friend class Class;
public:
	inline				Object();
	inline				Object(const Object& obj);
	virtual				~Object();

	inline void			refer();
	inline void			release();
	inline int			getReference() const;

	inline Class		getClass() const;

	virtual String		getName() const;
	virtual String		toString() const;

	virtual bool		isValid() const;

	void				wait();
	void				notify();
	void				notifyAll();

	inline Object&		operator=(const Object& obj);

protected:
	inline String		getClassNameRec() const;

private:
	virtual String		getClassName() const;

private:
	typedef struct WaitingThread {
		Thread*			m_Thread;
		WaitingThread*	m_Next;
	} WaitingThread;

	WaitingThread*		m_WaitingThreads;
	SoftPointerBase*	m_SoftPointer;
#if defined(__FORWINDOWS__)
	LONG				m_Reference;
#else	
	int					m_Reference;
#endif
};

}; };

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
#define DeclareClass(cname, base)										\
	protected:															\
		inline String getClassNameRec() const {							\
			return base::getClassNameRec() + ":"  #cname; };			\
	private:															\
		virtual String getClassName() const {							\
			return getClassNameRec(); };								\


#include "base.string.h"
#include "object.class.h"
#include "object.traits.h"

#endif
