/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __OBJECT_POINTER_H
#define __OBJECT_POINTER_H

#include "base.h"

namespace magnum { namespace object {

template<class T> class AutoPointer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Pointer {
public:
	inline	Pointer();
	inline	Pointer(T* ptr);
	inline	Pointer(const Pointer& ptr);
	inline	Pointer(const AutoPointer<T>& ptr);
	inline	~Pointer();

	inline const Pointer<T>& operator=(T* ptr);
	inline const Pointer<T>& operator=(const Pointer<T>& ptr);
	inline const Pointer<T>& operator=(const AutoPointer<T>& ptr);
	inline T*			operator->() const;
	inline				operator T*() const;

	inline bool			isNull() const;
	inline void			setNull();

	inline bool			operator==(const Pointer<T>& lhs) const;
	inline bool			operator!=(const Pointer<T>& lhs) const;
	inline bool			operator<(const Pointer<T>& lhs) const;
	inline bool			operator>(const Pointer<T>& lhs) const;
	inline bool			operator<=(const Pointer<T>& lhs) const;
	inline bool			operator>=(const Pointer<T>& lhs) const;
	inline bool			operator==(const T* lhs) const;
	inline bool			operator==(T* lhs) const;
	inline bool			operator==(intptr_t lhs) const;
	inline bool			operator!=(const T* lhs) const;
	inline bool			operator!=(T* lhs) const;
	inline bool			operator!=(intptr_t lhs) const;

private:
	T*	m_Pointer;
};

}; };

#include "object.autopointer.h"

#endif
