/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __OBJECT_PROPERTY_INL
#define __OBJECT_PROPERTY_INL

namespace magnum { namespace object {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline  Property::Property() {
	m_Type = Unknown;
	m_Default.m_String =
	m_Value.m_String = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Property::Type Property::getType() const {
	return m_Type;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String Property::getName() const {
	return m_Name;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline String Property::getDescription() const {
	return m_Description;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Property::getFloat() const {
	ASSERT(m_Type == Float);
	return m_Value.m_Float;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline double Property::getDouble() const {
	ASSERT(m_Type == Double);
	return m_Value.m_Double;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Property::getInteger() const {
	ASSERT(m_Type == Integer);
	return m_Value.m_Integer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* Property::getPointer() const {
	ASSERT(m_Type == Pointer);
	return m_Value.m_Pointer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const char* Property::getString() const {
	static const char empty[] = "";
	ASSERT(m_Type == String);
	return (m_Value.m_String ? m_Value.m_String : empty);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Property::getDefaultFloat() const {
	ASSERT(m_Type == Float);
	return m_Default.m_Float;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline double Property::getDefaultDouble() const {
	ASSERT(m_Type == Double);
	return m_Default.m_Double;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Property::getDefaultInteger() const {
	ASSERT(m_Type == Integer);
	return m_Default.m_Integer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* Property::getDefaultPointer() const {
	ASSERT(m_Type == Pointer);
	return m_Default.m_Pointer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const char* Property::getDefaultString() const {
	static const char empty[] = "";
	ASSERT(m_Type == String);
	return (m_Default.m_String ? m_Default.m_String : empty);
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Property::setFloat(float val) {
	ASSERT(m_Type == Float);
	m_Value.m_Float = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Property::setDouble(double val) {
	ASSERT(m_Type == Double);
	m_Value.m_Double = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Property::setInteger(int val) {
	ASSERT(m_Type == Integer);
	m_Value.m_Integer = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Property::setPointer(void* val) {
	ASSERT(m_Type == Pointer);
	m_Value.m_Pointer = val;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline PropertyList::Iterator PropertyList::begin() const {
	return m_Propertys.begin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline PropertyList::Iterator PropertyList::end() const {
	return m_Propertys.end();
}

}; };

#endif
