/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __OBJECT_SOFTPOINTER_H
#define __OBJECT_SOFTPOINTER_H

#include "object.h"

namespace magnum { namespace object {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class SoftPointerBase {
friend class Object;
protected:
	inline SoftPointerBase();
	SoftPointerBase(Object* );
	SoftPointerBase(const SoftPointerBase& );
	~SoftPointerBase();
	
	inline Object*	getPointer() const;
	void			setPointer(Object* );
	void			setNull();

	inline const SoftPointerBase& operator=(const SoftPointerBase& ptr);

private:
	SoftPointerBase* m_Next;
	SoftPointerBase* m_Prev;
	Object*			m_Pointer;
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class SoftPointer : public SoftPointerBase {
public:
	inline SoftPointer();
	inline SoftPointer(T* ptr);
	inline SoftPointer(const SoftPointer& ptr);

	inline const SoftPointer<T>& operator=(const SoftPointer<T>& ptr);
	inline const SoftPointer<T>& operator=(T* ptr);
	inline T*			operator->() const;
	inline				operator T*() const;

	inline bool			isNull() const;
	inline void			setNull();

	inline bool			operator==(const SoftPointer<T>& lhs) const;
	inline bool			operator!=(const SoftPointer<T>& lhs) const;
	inline bool			operator<(const SoftPointer<T>& lhs) const;
	inline bool			operator>(const SoftPointer<T>& lhs) const;
	inline bool			operator<=(const SoftPointer<T>& lhs) const;
	inline bool			operator>=(const SoftPointer<T>& lhs) const;
	inline bool			operator==(const T* lhs) const;
	inline bool			operator==(T* lhs) const;
	inline bool			operator==(intptr_t lhs) const;
	inline bool			operator!=(const T* lhs) const;
	inline bool			operator!=(T* lhs) const;
	inline bool			operator!=(intptr_t lhs) const;
};

}; };

#endif
