/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __OBJECT_SOFTPOINTER_INL
#define __OBJECT_SOFTPOINTER_INL

namespace magnum { namespace object {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline SoftPointerBase::SoftPointerBase() {
	m_Pointer = NULL;
	m_Prev = m_Next = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Object* SoftPointerBase::getPointer() const {
	return m_Pointer;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const SoftPointerBase& SoftPointerBase::operator=(const SoftPointerBase& ptr) {
	setPointer(ptr.m_Pointer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline SoftPointer<T>::SoftPointer() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline SoftPointer<T>::SoftPointer(T* ptr) : SoftPointerBase(ptr) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline SoftPointer<T>::SoftPointer(const SoftPointer& ptr) : SoftPointerBase(ptr) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const SoftPointer<T>& SoftPointer<T>::operator=(const SoftPointer<T>& ptr) {
	setPointer(ptr.getPointer());
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const SoftPointer<T>& SoftPointer<T>::operator=(T* ptr) {
	setPointer(ptr);
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline T* SoftPointer<T>::operator->() const {
	return (T*)getPointer();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline SoftPointer<T>::operator T*() const {
	return (T*)getPointer();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::isNull() const {
	return (getPointer() == NULL);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void SoftPointer<T>::setNull() {
	SoftPointerBase::setNull();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator==(const SoftPointer<T>& lhs) const {
	return (lhs.getPointer() == getPointer());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator!=(const SoftPointer<T>& lhs) const {
	return (lhs.getPointer() != getPointer());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator<(const SoftPointer<T>& lhs) const {
	return (lhs.getPointer() > getPointer());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator>(const SoftPointer<T>& lhs) const {
	return (lhs.getPointer() < getPointer());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator<=(const SoftPointer<T>& lhs) const {
	return (lhs.getPointer() >= getPointer());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator>=(const SoftPointer<T>& lhs) const {
	return (lhs.getPointer() <= getPointer());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator==(const T* lhs) const {
	return (getPointer() == lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator!=(const T* lhs) const {
	return (getPointer() != lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator==(T* lhs) const {
	return (getPointer() == lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator!=(T* lhs) const {
	return (getPointer() != lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator==(intptr_t lhs) const {
	return (getPointer() == (T*)(void*)lhs);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline bool SoftPointer<T>::operator!=(intptr_t lhs) const {
	return (getPointer() != (T*)(void*)lhs);
}


}; };

#endif
