/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __PHYSICS_BODY_H
#define __PHYSICS_BODY_H

#include "ode.h"

#include "math.vector3.h"
#include "math.matrix.h"
#include "container.chain.h"

namespace magnum { namespace physics {
using magnum::object::Object;
using magnum::container::Chainable;
using magnum::math::Vector3;
using magnum::math::Matrix3;
using magnum::math::Matrix4;

class Domain;
class Mass;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Body : public Object, private Chainable<Body> {
DeclareClass(Body, Object);
private:
	dBodyID					m_Body;	

public:
	inline ~Body();
	inline explicit Body(Domain* );

	inline void				setPosition(float ,float ,float );
	inline void				setPosition(const Vector3<float>& );
	inline Vector3<float>	getPosition() const;

	inline void				setRotation(const Matrix3<float>& );
	inline Matrix3<float>	getRotation() const;

	inline void				setTransform(const Matrix4<float>& );
	inline Matrix4<float>	getTransform() const;

	inline void				setLinearVelocity(float ,float ,float );
	inline void				setLinearVelocity(const Vector3<float>& );
	inline Vector3<float>	getLinearVelocity() const;

	inline void				setAngularVelocity(float ,float ,float );
	inline void				setAngularVelocity(const Vector3<float>& );
	inline Vector3<float>	getAngularVelocity() const;

	inline void				setMass(const Mass& );
	inline Mass				getMass() const;

	inline void				setForce(float ,float ,float );
	inline void				setForce(const Vector3<float>& );
	inline void				addForce(float ,float ,float );
	inline void				addForce(const Vector3<float>& );
	inline Vector3<float>	getForce() const;

	inline void				setTorque(float ,float ,float );
	inline void				setTorque(const Vector3<float>& );
	inline void				addTorque(float ,float ,float );
	inline void				addTorque(const Vector3<float>& );
	inline Vector3<float>	getTorque() const;

	inline void				enable();
	inline void				disable();
	inline bool				isEnabled() const;

	inline void				setGravity(bool );
	inline bool				getGravity() const;
};

}; };

#endif
