/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __PHYSICS_SPRING_H
#define __PHYSICS_SPRING_H

#include "object.h"
#include "math.vector.h"

namespace magnum { namespace physics {
using magnum::object::Object;
using magnum::types::String;
using magnum::math::Vector;

template<class T, int dim> class Relaxator;
template<class T, int dim> class Weight;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Spring : public Object {
DeclareClass(Spring, Object);
friend class Relaxator<T,dim>;
private:
	T						m_Constant;
	T						m_NaturalLength;
	T						m_Length;
	Vector<T,dim>			m_Force;
	Weight<T,dim>*			m_Left;
	Spring<T,dim>*			m_LeftNext;
	Weight<T,dim>*			m_Right;
	Spring<T,dim>*			m_RightNext;
	
public:
	inline Spring();
	
	inline void				setConstant(T );
	inline T				getConstant() const;
	
	inline void				setNaturalLength(T );
	inline T				getNaturalLength() const;

	inline Weight<T,dim>*	getLeft() const;
	inline Weight<T,dim>*	getRight() const;
	
	inline T				getLength() const;
	inline const Vector<T,dim>& getForce() const;
};



}; };

#endif
