/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __PHYSICS_WEIGHT_H
#define __PHYSICS_WEIGHT_H

#include "object.h"
#include "math.vector.h"
#include "container.iterator.h"

namespace magnum { namespace physics {
using magnum::object::Object;
using magnum::types::String;
using magnum::math::Vector;
using magnum::container::IteratorBase;

template<class T, int dim> class Relaxator;
template<class T, int dim> class Spring;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> class Weight : public Object {
DeclareClass(Weight, Object);
friend class Relaxator<T,dim>;
private:
	Vector<T,dim>			m_Position;
	Vector<T,dim>			m_Velocity;
	T						m_Mass;
	T						m_InvMass;
	Spring<T,dim>*			m_Springs;
	bool					m_Fixed;

public:
	class Iterator : public IteratorBase<Iterator> {
	private:
		Spring<T,dim>*			m_Spring;
		Weight<T,dim>*			m_Weight;
	
	private:
		inline Iterator(Spring<T,dim>* ,Weight<T,dim>* );
		
	public:
		inline Iterator();
		
		inline Iterator&		operator++();
		inline Iterator			operator++(int );
		
		inline bool				operator==(const Iterator& ) const;
		inline bool				operator!=(const Iterator& ) const;

        inline 					operator Spring<T,dim>*();
        inline 					operator const Spring<T,dim>*() const;
		
		inline const Spring&	operator*() const;
		inline Spring&			operator*();
		inline const Spring*	operator->() const;
		inline Spring*			operator->();
	};

	
public:
	inline Weight();
	
	inline void				setPosition(const Vector<T,dim>& );
	inline const Vector<T,dim>& getPosition() const;

	inline void				setVelocity(const Vector<T,dim>& );
	inline const Vector<T,dim>& getVelocity() const;
	
	inline void				applyForce(const Vector<T,dim>& );
	
	inline void				setFixed(bool );
	inline bool				isFixed() const;
	
	inline void				setMass(T );
	inline T				getMass() const;
	
	inline Iterator			begin() const;
	inline Iterator			at(const Spring<T,dim>& ) const;
	inline Iterator			end() const;
	
	unsigned int			getSize() const;
	inline bool				isEmpty() const;
	bool					isConnected(const Weight<T,dim>& );
};



}; };

#endif
