/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __PHYSICS_WEIGHT_INL
#define __PHYSICS_WEIGHT_INL


namespace magnum { namespace physics {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Weight<T,dim>::Iterator::Iterator(Spring<T,dim>* s, Weight<T,dim>* b) {
	m_Spring = s;
	m_Weight = b;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Weight<T,dim>::Iterator::Iterator() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline typename Weight<T,dim>::Iterator& Weight<T,dim>::Iterator::operator++() {
	if (m_Spring->m_Left == m_Weight)
		m_Spring = m_Spring->m_LeftNext;
	else
		m_Spring = m_Spring->m_RightNext;
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline typename Weight<T,dim>::Iterator Weight<T,dim>::Iterator::operator++(int ) {
	Spring<T,dim>* s = m_Spring;
	if (m_Spring->m_Left == m_Weight)
		m_Spring = m_Spring->m_LeftNext;
	else
		m_Spring = m_Spring->m_RightNext;
	return Iterator(s, m_Weight);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Weight<T,dim>::Iterator::operator==(typename const Weight<T,dim>::Iterator& iter) const {
	return iter.m_Spring == m_Spring;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Weight<T,dim>::Iterator::operator!=(typename const Weight<T,dim>::Iterator& iter) const {
	return iter.m_Spring != m_Spring;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Weight<T,dim>::Iterator::operator Spring<T,dim>*() {
	return m_Spring;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Weight<T,dim>::Iterator::operator const Spring<T,dim>*() const {
	return m_Spring;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Spring<T,dim>& Weight<T,dim>::Iterator::operator*() const {
	return *m_Spring;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Spring<T,dim>& Weight<T,dim>::Iterator::operator*() {
	return *m_Spring;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Spring<T,dim>* Weight<T,dim>::Iterator::operator->() const {
	return m_Spring;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Spring<T,dim>* Weight<T,dim>::Iterator::operator->() {
	return m_Spring;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline Weight<T,dim>::Weight() {
	m_Springs = NULL;
	m_Velocity.null();
	m_Position.null();
	m_Mass = m_InvMass = T(1);
	m_Fixed = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Weight<T,dim>::setPosition(const Vector<T,dim>& p) {
	m_Position = p;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Weight<T,dim>::getPosition() const {
	return m_Position;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Weight<T,dim>::setVelocity(const Vector<T,dim>& v) {
	m_Velocity = v;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline const Vector<T,dim>& Weight<T,dim>::getVelocity() const {
	return m_Velocity;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Weight<T,dim>::applyForce(const Vector<T,dim>& f) {
	m_Velocity += m_InvMass * f;

}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Weight<T,dim>::setFixed(bool f) {
	m_Fixed = f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Weight<T,dim>::isFixed() const {
	return m_Fixed;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline void Weight<T,dim>::setMass(T m) {
	m_Mass = m;
	m_InvMass = T(1) / m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline T Weight<T,dim>::getMass() const {
	return m_Mass;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline typename Weight<T,dim>::Iterator Weight<T,dim>::begin() const {
	return Iterator(m_Springs, m_Weight);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline typename Weight<T,dim>::Iterator Weight<T,dim>::at(const Spring<T,dim>& s) const {
	return Iterator(&s, m_Weight);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline typename Weight<T,dim>::Iterator Weight<T,dim>::end() const {
	return Iteratir(NULL, m_Weight);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
unsigned int Weight<T,dim>::getSize() const {
	unsigned int i = 0;
	Spring<T,dim>* s = m_Springs;
	while (s != NULL) {
		i++;
		if (s->m_Left == this)
			s = s->m_LeftNext;
		else
			s = s->m_RightNext;
	}
	return i;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
inline bool Weight<T,dim>::isEmpty() const {
	return m_Springs == NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, int dim> 
bool Weight<T,dim>::isConnected(const Weight<T,dim>& w) {
	Spring<T,dim>* s = m_Springs;
	while (s != NULL) {
		if (s->m_Left == this) {
			if (s->m_Right == &w)
				return true;			
			s = s->m_LeftNext;
		} else {
			if (s->m_Left == &w)
				return true;			
			s = s->m_RightNext;
		}
	}
	return false;
}

}; };

#endif
