/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __PLOTS3D_SURFACEPLOT3D_H
#define __PLOTS3D_SURFACEPLOT3D_H

#include "container.array.h"
#include "math.function.h"
#include "math.vector2.h"
#include "math.vector3.h"
#include "graphics3d.indexmesh.h"
#include "plots3d.plot3d.h"

namespace magnum { namespace plots3d {
using magnum::container::Array;
using magnum::math::Function;
using magnum::math::Vector2;
using magnum::math::Vector3;
using magnum::graphics3d::IndexMesh;
using magnum::graphics3d::IndexBuffer;
using magnum::graphics3d::VertexBuffer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class SurfacePlot3d : public Plot3d {
DeclareClass(SurfacePlot3d, Plot3d);
public:
	struct Vertex {
		Vector3<float>	position;
		Vector3<float>	normal;
	};
	
public:
	SurfacePlot3d();
	
	inline void			setResolution(const Vector2<unsigned int>& );
	inline void			setResolution(unsigned int );
	inline void			setResolution(unsigned int ,unsigned int );
	inline const Vector2<unsigned int>& getResolution() const;

protected:
	virtual void		onRender(const Matrix4<float>& ) const;
	virtual void		onClear();
	
	void				beginPlot();	
	void				finishPlot();
	
	inline Vertex*		getVertices();
	
private:
	Vector2<unsigned int>	m_Resolution;
	Vector2<unsigned int>	m_OldResolution;
	IndexMesh				m_Mesh;
	Array<Vertex>			m_Vertices;
	Vector3<float>			m_Minimum;
	Vector3<float>			m_Maximum;
};

}; };

#endif
