/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_BEHAVIOUR_H
#define __SCENE3D_BEHAVIOUR_H

#include "container.scheduler.h"
#include "system.timer.h"
#include "scene3d.leaf.h"

namespace magnum { namespace scene3d {
using magnum::container::Scheduler;
using magnum::object::Pointer;
using magnum::system::Timer;

class BehaviourScheduler;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Behaviour : public Leaf<Behaviour>, public magnum::container::Task<Behaviour> {
DeclareClass(Behaviour, Leaf<Behaviour>);
friend BehaviourScheduler;
public:
	enum Type {
		ControllerType	= 0x02000000,
		AllBehaviours	= 0xff000000
	};

	typedef magnum::container::Task<Behaviour>::Priority Priority;
	
protected:
	inline Behaviour(const Behaviour& );
	inline explicit Behaviour(Type t);
	inline Behaviour();
	
public:
	inline void			process();
	
	inline bool			isActive() const;
	inline bool			isScheduled() const;

protected:
	virtual Behaviour*	onClone() const;
	virtual void		onAttachLocale(Locale* );
	virtual void		onDetachLocale(Locale* );
	virtual void		onEnable();
	virtual void		onDisable();

	virtual void		onProcess(const Timer& );
	virtual void		onActivate();
	virtual void		onDeactivate();

private:
	unsigned int		m_Active:1;
	unsigned int		m_WasActive:1;
};

}; };

#endif
