/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_ENUMERATOR_H
#define __SCENE3D_ENUMERATOR_H

#include "scene3d.node.h"
#include "scene3d.nodepath.h"
#include "scene3d.universe.h"
#include "scene3d.locale.h"
#include "geometry.transform.h"

namespace magnum { namespace scene3d {
using magnum::geometry::Transform3;

class Root;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Enumerator : public Object {
DeclareClass(Enumerator, Object);
private:
	const Universe*		m_Universe;
	const Locale*		m_Locale;
	Node::Type			m_Types;

	Transform3<float>	m_BaseTransform;
	Vector3<double>		m_BasePosition;

	Universe::Iterator	m_LocaleIterator;
	Locale::Iterator	m_RootIterator;
	Locale*				m_CurrentLocale;
	Root*				m_CurrentRoot;
	Node*				m_CurrentNode;

	NodePath			m_Path;

public:
	explicit Enumerator(unsigned int = Node::AllTypes);
	Enumerator(const Universe* ,unsigned int = Node::AllTypes);
	Enumerator(const Locale* ,unsigned int = Node::AllTypes);

	inline void				setType(unsigned int );
	inline Node::Type		getType() const;

	inline const Universe*	getUniverse() const;
	inline const Locale*	getLocale() const;

	inline void				attachUniverse(const Universe* );
	inline void				attachLocale(const Locale* );
	inline void				detach();

	inline const Transform3<float>& getTransform() const;
	inline void				setTransform(const Transform3<float>& );

	inline const Vector3<double>& getPosition() const;
	inline void				setPosition(const Vector3<double>& );

	inline const NodePath&	getPath() const;

	inline Node*			operator->() const;
	inline Node&			operator*() const;
	inline					operator Node*() const;

	void					begin();
	void					next();
	inline bool				finished() const;
};

}; };

#endif
