/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_ENUMERATOR_INL
#define __SCENE3D_ENUMERATOR_INL

namespace magnum { namespace scene3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Enumerator::attachUniverse(const Universe* u) {
	ASSERT(u != NULL);
	m_Locale = NULL;
	m_Universe = u;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Enumerator::attachLocale(const Locale* l) {
	ASSERT(l != NULL);
	m_Locale = l;
	m_Universe = l->getUniverse();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Enumerator::detach() {
	m_Locale = NULL;
	m_Universe = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Node::Type Enumerator::getType() const {
	return m_Types;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Enumerator::setType(unsigned int t) {
	m_Types = (Node::Type)t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Locale* Enumerator::getLocale() const {
	return m_Locale;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Universe* Enumerator::getUniverse() const {
	return m_Universe;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const NodePath& Enumerator::getPath() const {
	return m_Path;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Transform3<float>& Enumerator::getTransform() const {
	return m_BaseTransform;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Enumerator::setTransform(const Transform3<float>& t) {
	m_BaseTransform = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Vector3<double>& Enumerator::getPosition() const {
	return m_BasePosition;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	Enumerator::setPosition(const Vector3<double>& p) {
	m_BasePosition = p;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Node* Enumerator::operator->() const {
	return m_CurrentNode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Node& Enumerator::operator*() const {
	return *m_CurrentNode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Enumerator::operator Node*() const {
	return m_CurrentNode;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Enumerator::finished() const {
	return (m_CurrentNode == NULL);
}

}; };

#endif
