/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_GRAPHICSCONTEXT_H
#define __SCENE3D_GRAPHICSCONTEXT_H

#include "geometry.bounds.h"
#include "container.chain.h"
#include "graphics3d.accelerator.h"

namespace magnum { namespace scene3d {
using magnum::object::Object;
using magnum::object::Noncopyable;
using magnum::types::String;
using magnum::container::Chain;
using magnum::geometry::Bounds3;
using magnum::graphics3d::Accelerator;

class Modifier;
class Viewer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class GraphicsContext : public Object, private Noncopyable {
DeclareClass(GraphicsContext, Object);
public:
	GraphicsContext(magnum::graphics3d::Context* );
	
	inline magnum::graphics3d::Context* getContext();
	inline magnum::graphics3d::Accelerator* getAccelerator();
	
	void				clearModifiers();

	inline void			insertModifier(Modifier* );
	inline void			removeModifier(Modifier* );
	inline bool			containsModifier(const Modifier* ) const;

	void				selectModifiers(const Bounds3<float>& );
	inline void			selectModifier(Modifier* );
	inline void			deselectModifier(Modifier* );
	inline void			deselectAllModifiers();

private:
	Chain<Modifier>		m_Modifiers;
	Accelerator			m_Accelerator;
};

}; };

#endif
