/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_GROUP_INL
#define __SCENE3D_GROUP_INL

namespace magnum {namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::Group() : Node(Node::GroupType) {
	m_First = m_Last = NULL;
	m_ChildTypes = 0;
	m_WorldTransform.identity();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::Group(Node::Type t) : Node((Node::Type)(t | Node::GroupType)) {
	m_First = m_Last = NULL;
	m_ChildTypes = 0;
	m_WorldTransform.identity();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::~Group() {
	clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Group::setWorldTransform(const Transform3<float>& t) {
	m_WorldTransform = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Transform3<float>& Group::getWorldTransform() const {
	return m_WorldTransform;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::Iterator Group::begin() const {
	return Iterator(m_First, NULL);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::Iterator Group::end() const {
	return Iterator(NULL, m_Last);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Group::isEmpty() const {
	return (m_First == NULL);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Node* Group::getNext(const Node* node) {
	return node->m_Next;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Node* Group::getPrev(const Node* node) {
	return node->m_Prev;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Group::getAllTypes(const Node* node) {
	int type = node->getType();
	if (type & Node::GroupType)
		return type | ((Group*)node)->m_ChildTypes;
	else
		return type;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::Iterator::Iterator() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::Iterator::operator Node*() const {
	return m_Current;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Node* Group::Iterator::operator->() const {
	return m_Current;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Node& Group::Iterator::operator*() const {
	return *m_Current;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::Iterator& Group::Iterator::operator++() {
	m_Previous = m_Current;
	m_Current = getNext(m_Current);
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::Iterator Group::Iterator::operator++(int ) {
	Node* oc = m_Current;
	Node* op = m_Previous;
	m_Previous = m_Current;
	m_Current = getNext(m_Current);
	return Iterator(oc, op);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::Iterator& Group::Iterator::operator--() {
	m_Current = m_Previous;
	m_Previous = getPrev(m_Current);
	return *this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::Iterator Group::Iterator::operator--(int ) {
	Node* oc = m_Current;
	Node* op = m_Previous;
	m_Current = m_Previous;
	m_Previous = getPrev(m_Current);
	return Iterator(oc, op);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Group::Iterator::operator==(const Iterator& iter) const {
	return m_Current == iter.m_Current;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Group::Iterator::operator!=(const Iterator& iter) const {
	return m_Current != iter.m_Current;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Group::Iterator::Iterator(Node* cur, Node* prev) {
	m_Current = cur;
	m_Previous = prev;
}


}; };

#endif
