/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_LEAF_H
#define __SCENE3D_LEAF_H

#include "scene3d.node.h"
#include "geometry.space.h"
#include "geometry.bounds.h"
#include "geometry.transform.h"

namespace magnum {namespace scene3d {
using magnum::geometry::Placeable;
using magnum::geometry::Sphere;
using magnum::geometry::Bounds;
using magnum::geometry::Transform;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> class Leaf : public Node, private Placeable<float, 3, T> {
DeclareClass(Leaf, Node);
protected:
	inline Leaf(const Leaf& );
	inline explicit Leaf(typename Node::Type );

public:
	inline const Bounds<float,3>& getBounds() const;
	inline void			setBounds(const Bounds<float,3>& );
	
	inline const Bounds<float,3>& getWorldBounds() const;
	inline void			setWorldBounds(const Bounds<float,3>& );
	inline void			transformWorldBounds(const Transform<float,3>& );

protected:
	virtual Leaf*		onClone() const;
	virtual void		onAttachLocale(Locale* );
	virtual void		onDetachLocale(Locale* );
	virtual void		onEnable();
	virtual void		onDisable();
	virtual void		onTransform(const Transform3<float>& );
	
private:
	Bounds<float,3>		m_Bounds;
};

}; };

#endif
