/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_LEAF_INL
#define __SCENE3D_LEAF_INL

namespace magnum {namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Leaf<T>::Leaf(typename Node::Type t) : Node((Node::Type)(t | Node::LeafType))  {
	m_Bounds.setOrigin(Vector3<float>::Null);
	m_Bounds.setAxis(0,Vector3<float>(1.0f,0.0f,0.0f));
	m_Bounds.setAxis(1,Vector3<float>(0.0f,1.0f,0.0f));
	m_Bounds.setAxis(2,Vector3<float>(0.0f,0.0f,1.0f));
	Placeable<float, 3, T>::setBounds(m_Bounds);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline Leaf<T>::Leaf(const Leaf<T>& l) : Node(l)  {
	m_Bounds = l.m_Bounds;
	Placeable<float, 3, T>::setBounds(m_Bounds);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Bounds<float,3>& Leaf<T>::getWorldBounds() const {
	return Placeable<float, 3, T>::getBounds();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void Leaf<T>::setWorldBounds(const Bounds<float,3>& b) {
	Placeable<float, 3, T>::setBounds(b);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void Leaf<T>::transformWorldBounds(const Transform<float,3>& t) {
	Placeable<float, 3, T>::setBounds(Bounds<float,3>(m_Bounds).transform(t));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Bounds<float,3>& Leaf<T>::getBounds() const {
	return m_Bounds;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void Leaf<T>::setBounds(const Bounds<float,3>& b) {
	m_Bounds = b;
	updateTransform();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
Leaf<T>* Leaf<T>::onClone() const {
	return new Leaf<T>(*this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Leaf<T>::onEnable() {
	if(getLocale() != NULL)
		getLocale()->insert((T*)this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Leaf<T>::onDisable() {
	if(getLocale() != NULL)
		getLocale()->remove((T*)this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Leaf<T>::onAttachLocale(Locale* l) {
	Node::onAttachLocale(l);
	if (isEnabled())
		l->insert((T*)this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Leaf<T>::onDetachLocale(Locale* l) {
	getSpace()->remove((T*)this);
	Node::onDetachLocale(l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
void Leaf<T>::onTransform(const Transform3<float>& parent) {
	if (hasParentTransform())
		Placeable<float, 3, T>::setBounds(parent * m_Bounds);
	else
		Placeable<float, 3, T>::setBounds(m_Bounds);
}

}; };

#endif
