/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_LOCALE_H
#define __SCENE3D_LOCALE_H

#include "container.chain.h"
#include "container.visitor.h"
#include "math.vector3.h"
#include "geometry.frustum.h"
#include "geometry.alignedbox.h"
#include "geometry.space.h"
#include "scene3d.leaf.h"

namespace magnum { namespace scene3d {
using magnum::container::Chainable;
using magnum::container::Chain;
using magnum::container::Visitor;
using magnum::math::Vector3;
using magnum::object::Object;
using magnum::object::Noncopyable;
using magnum::types::String;
using magnum::geometry::Frustum3;
using magnum::geometry::AlignedBox3;
using magnum::geometry::Space;

class Universe;
class Root;
class Node;
class Graphics;
class Audio;
class Behaviour;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Locale : public Object, private Chainable<Locale>, public Noncopyable {
DeclareClass(Locale, Object);
friend Universe;
friend Node;
friend Leaf<Audio>;
friend Leaf<Graphics>;
friend Leaf<Behaviour>;
public:
	typedef magnum::container::Chain<Root>::Iterator Iterator;

public:
	Locale(Universe* );
	Locale(Universe* ,const Vector3<double>& );

	inline Universe*		getUniverse() const;

	inline const Vector3<double>& getOrigin() const;
	inline void				setOrigin(const Vector3<double>& );
	
	void					insert(Root* );
	void					remove(Root* );
	bool					contains(Root* ) const;
	void					clear();
							
	inline bool				isEmpty() const;
							
	inline Iterator			begin() const;
	inline Iterator			end() const;
	Iterator				at(const char* ) const;

	bool					optimise();
	inline bool				isOptimised() const;
	
public:
	// Some dirty hack for traversers - we cannot make a traverser a friend
	// of a locale (although this would be very reasonable), beacause
	// the traverser is a template.
	inline void				getSpace(const Space<float, 3, Graphics>* & ) const;
	inline void				getSpace(const Space<float, 3, Audio>* & ) const;
	inline void				getSpace(const Space<float, 3, Behaviour>* & ) const;

private:
	// Some dirty hacks for Leafs
	inline void				insert(Graphics* );
	inline void				remove(Graphics* );
	inline void				insert(Audio* );
	inline void				remove(Audio* );
	inline void				insert(Behaviour* );
	inline void				remove(Behaviour* );

private:
	Universe*				m_Universe;
	Vector3<double>			m_Origin;
	bool					m_Optimised;
	Chain<Root>				m_Roots;
	Space<float, 3, Graphics>	m_GraphicsSpace;
	Space<float, 3, Audio>		m_AudioSpace;
	Space<float, 3, Behaviour>	m_BehaviourSpace;
};

}; };

#endif
