/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_NODE_H
#define __SCENE3D_NODE_H

#include "geometry.transform.h"
#include "object.attachment.h"

namespace magnum { namespace scene3d {
using magnum::object::Object;
using magnum::object::Noncopyable;
using magnum::types::String;
using magnum::geometry::Transform3;
using magnum::object::Attachment;
using magnum::object::AttachmentList;

class Group;
class Root;
class Universe;
class Locale;
class Enumerator;
class Behaviour;
 
/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Node : public Object {
DeclareClass(Node, Object);
friend Group;
friend Enumerator;
friend Universe;
public:
	enum Type {
		Unknown			= 0x00000000,
		TransformType	= 0x00000001,
		LinkType		= 0x00000002,
		GroupType		= 0x00000004,
		LeafType		= 0x00000008,
		MarkerType		= 0x00000010,
		OccluderType	= 0x00000020,
		GraphicsType	= 0x00000100,
		AudioType		= 0x00010000,
		BehaviourType	= 0x01000000,
		AllTypes		= 0xffffffff,
	};

	enum Flag {
		Enabled			= 0x00000001,
		AutoBounds		= 0x00000002,
		DirtyTransform	= 0x00000004,
		ParentTransform = 0x00000008,
		ActiveBehaviour	= 0x00000100,
		ActiveModifier	= 0x00000200,
		ActiveGeometry	= 0x00000400,
		ActiveSound		= 0x00000800,
	};

protected:
	Node(const Node& );
	explicit Node(Type t);
	
public:
	~Node();

	inline Type			getType() const;

	inline String		getName() const;
	inline void			setName(const char* );

	inline Group*		getParent() const;

	inline Universe*	getUniverse() const;
	inline Locale*		getLocale() const;
	Node*				getRoot() const;

	inline void			updateTransform();
	inline bool			hasParentTransform() const;

	inline Node*		clone() const;

	void				enable();
	void				disable();
	inline bool			isEnabled() const;
	
	inline AttachmentList& getAttachments();
	inline const AttachmentList& getAttachments() const;
	template<class T>
	inline Attachment<T>* insertAttachment(const char* ,const T& );
	template<class T>
	inline bool			getAttachment(const char* ,T* ) const;
	inline bool			removeAttachment(const char* );
	inline void			clearAttachments();

	inline unsigned int	getFlags() const;
	inline void			setFlag(Flag );
	inline void			clearFlag(Flag );
	inline bool			testFlag(Flag ) const;
	
protected:
	virtual Node*		onClone() const;
	virtual void		onAttachLocale(Locale* );
	virtual void		onDetachLocale(Locale* );
	virtual void		onEnable();
	virtual void		onDisable();
	virtual void		onTransform(const Transform3<float>& );

protected:
	void				enqueueTransform();
	void				dequeueTransform();
	
private:
	Node&				operator=(const Node& );

private:
	String				m_Name;
	AttachmentList		m_Attachments;
	Locale*				m_Locale;
	Group*				m_Parent;
	Node*				m_Next;
	Node*				m_Prev;
	Type				m_Type;
	unsigned int		m_Flags;
	
	// Chain of transformed nodes
	Node*				m_NextDirty;
	Node*				m_PrevDirty;
};

}; };

#endif
