/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_NODEPATH_INL
#define __SCENE3D_NODEPATH_INL

namespace magnum { namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline NodePath::NodePath() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline NodePath::NodePath(Locale* l) {
	m_Locale = l;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void NodePath::push(Node* n) {
	m_Nodes.push(Reference(n));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void NodePath::push(const Reference& r) {
	m_Nodes.push(r);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void NodePath::push(Node* n, const Transform3<float>& t) {
	m_Nodes.push(Reference(n, t));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Reference NodePath::pop() {
	return m_Nodes.pop();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Reference& NodePath::peek() const {
	return m_Nodes.peek();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Reference& NodePath::peek() {
	return m_Nodes.peek();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Transform3<float>& NodePath::getTransform() const {
	return m_Nodes.peek().getTransform();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Node* NodePath::getNode() const {
	return m_Nodes.peek();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Locale* NodePath::getLocale() const {
	return m_Locale;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void NodePath::setLocale(Locale* l) {
	m_Locale = l;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void NodePath::clear() {
	m_Nodes.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool NodePath::isEmpty() const {
	return m_Nodes.isEmpty();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned int NodePath::getSize() const {
	return m_Nodes.getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline NodePath::Iterator NodePath::begin() const {
	return m_Nodes.begin();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline NodePath::Iterator NodePath::at(unsigned int i) const {
	return m_Nodes.at(i);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline NodePath::Iterator NodePath::end() const {
	return m_Nodes.end();
}
					

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Reference& NodePath::operator[](unsigned int i) const {
	return m_Nodes[i];
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Reference& NodePath::operator[](unsigned int i) {
	return m_Nodes[i];
}

}; };

#endif
