/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_OBSERVER_H
#define __SCENE3D_OBSERVER_H

#include "object.pointer.h"
#include "geometry.frustum.h"
#include "graphics3d.context.h"
#include "audio3d.context.h"
#include "scene3d.graphicscontext.h"
#include "scene3d.audiocontext.h"
#include "scene3d.behaviourscheduler.h"

namespace magnum { namespace scene3d {
using magnum::object::Pointer;

class Viewer;
class Listener;
class Activator;
class Behaviour;
class Graphics;
class Universe;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Observer : public Object {
DeclareClass(Observer, Object);
public:
	Observer(Universe* ,magnum::graphics3d::Context* ,magnum::audio3d::Context* );
	~Observer();
	
	inline Viewer*			getViewer() const;
	inline void				attachViewer(Viewer* );
	inline void				detachViewer();

	inline Listener*		getListener() const;
	inline void				attachListener(Listener* );
	inline void				detachListener();

	inline Activator*		getActivator() const;
	inline void				attachActivator(Activator* );
	inline void				detachActivator();

	inline Universe*		getUniverse() const;
	
	void					process();
	void					listen();
	void					view();
	
private:
	Pointer<Universe>		m_Universe;
	Pointer<Activator>		m_Activator;
	Pointer<Viewer>			m_Viewer;
	Pointer<Listener>		m_Listener;
	
	GraphicsContext			m_GraphicsContext;
	AudioContext			m_AudioContext;
	BehaviourScheduler		m_BehaviourScheduler;
};

}; };

#endif
