/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_PICKTRAVERSER_H
#define __SCENE3D_PICKTRAVERSER_H

#include "geometry.line.h"
#include "geometry.ray.h"
#include "geometry.segment.h"
#include "scene3d.traverser.h"
#include "scene3d.intersection.h"

namespace magnum { namespace scene3d {
using magnum::geometry::Line;
using magnum::geometry::Segment;
using magnum::geometry::Ray;

/*----------------------------------------------------------------------*/
/**
 */
template<class T>
class PickTraverser : public Object {
DeclareClass(PickTraverser, Object);
friend class magnum::geometry::Space<float,3,T>;
public:
	PickTraverser();
	PickTraverser(const Locale* );
	PickTraverser(const Universe* );
	
	inline void				setPreTranslation(const Vector3<double>& );
	inline const Vector3<double>& getPreTranslation() const;

	const T*				traverse(const Ray<float,3>& ,Intersection* = NULL);

	inline const Locale*	getLocale() const;
	inline const Universe*	getUniverse() const;

	inline const Vector3<double>& getTranslation() const;
	
private:
	bool					operator()(T* );
	
private:
	const Universe*			m_Universe;
	const Locale*			m_Locale;
	Vector3<double>			m_PreTranslation;
	Vector3<double>			m_Translation;
	Ray<float,3>			m_Ray;
	Intersection			m_Intersection;
};

}; };

#endif
