/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_PICKTRAVERSER_INL
#define __SCENE3D_PICKTRAVERSER_INL

#include "math.limits.h"

namespace magnum { namespace scene3d {
using magnum::math::Limits;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
PickTraverser<T>::PickTraverser() {
	m_Universe = NULL;
	m_Locale = NULL;
	m_PreTranslation.null();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
PickTraverser<T>::PickTraverser(const Locale* l) {
	ASSERT(l != NULL);
	m_Locale = l
	m_Universe = l->getUniverse();
	m_PreTranslation.null();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
PickTraverser<T>::PickTraverser(const Universe* u) {
	ASSERT(u != NULL);
	m_Universe = u;
	m_Locale = NULL;
	m_PreTranslation.null();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Locale* PickTraverser<T>::getLocale() const {
	return m_Locale;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Universe* PickTraverser<T>::getUniverse() const {
	return m_Universe;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline void PickTraverser<T>::setPreTranslation(const Vector3<double>& t) {
	m_PreTranslation = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Vector3<double>& PickTraverser<T>::getPreTranslation() const {
	return m_PreTranslation;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T> 
inline const Vector3<double>& PickTraverser<T>::getTranslation() const {
	return m_Translation;
}


/*----------------------------------------------------------------------*/
/**
 */
template<class T>
const T* PickTraverser<T>::traverse(const Ray<float,3>& l,Intersection* is) {
	m_Ray = l;
	m_Intersection.setParameter(Limits<float>::infinity());
	m_Intersection.setNode(NULL);
	m_Intersection.setModel(NULL);
	m_Intersection.setMesh(NULL);
	
	Vector3<float> origin = m_Ray.getOrigin();
	const Space<float, 3, T>* space;

	// Check if we only need to traverse a single locale
	if (m_Locale != NULL) {
		// Take pretransformations into account
		m_Translation = m_PreTranslation + m_Locale->getOrigin();

		// Translate volume
		m_Ray.setOrigin(origin - (Vector3<float>)(m_Locale->getOrigin()));

		// Intersect space!
		m_Locale->getSpace(space);
		space->intersect(m_Ray, *this);
		
		if (is != NULL)
			*is = m_Intersection;
		return static_cast<const T*>(m_Intersection.getNode());
	}

	if (m_Universe == NULL)
		return NULL;

	// Traverse complete universe
	Universe::Iterator iter, end;
	end = m_Universe->end();
	for (iter = m_Universe->begin(); iter != end; iter++) {
		m_Locale = (Locale*)iter;

		// Take pretransformations into account
		m_Translation = m_PreTranslation + m_Locale->getOrigin();

		// Translate volume
		m_Ray.setOrigin(origin - (Vector3<float>)(m_Locale->getOrigin()));

		// Intersect space!
		m_Locale->getSpace(space);
		space->intersect(m_Ray, *this);
	}

	m_Locale = NULL;

	if (is != NULL)
		*is = m_Intersection;
	return static_cast<const T*>(m_Intersection.getNode());
}


/*--------------------------------------------------------------------------*/
/** 
 */
template<class T>
bool PickTraverser<T>::operator()(T* item) {
	Intersection is;
	if (item->intersect(m_Ray, &is)) {
		if (is.getParameter() < m_Intersection.getParameter()) {
			m_Intersection = is;
			m_Intersection.setNode(item);
		}
	}
	
	return true;
}


}; };

#endif
