/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_SPOTLIGHTNODE_INL
#define __SCENE3D_SPOTLIGHTNODE_INL

namespace magnum { namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline SpotLightNode::SpotLightNode() {
	attachLight(&m_Light);
	m_Direction = Vector3<float>(1, 0, 0);
	m_Position = Vector3<float>::Null;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline SpotLightNode::~SpotLightNode() {
	detachLight();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Colour<float>& SpotLightNode::getAmbient() const {
	return m_Light.getAmbient();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void SpotLightNode::setAmbient(const Colour<float>& c) {
	m_Light.setAmbient(c);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Colour<float>& SpotLightNode::getDiffuse() const {
	return m_Light.getDiffuse();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void SpotLightNode::setDiffuse(const Colour<float>& c) {
	m_Light.setDiffuse(c);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Colour<float>& SpotLightNode::getSpecular() const {
	return m_Light.getSpecular();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void SpotLightNode::setSpecular(const Colour<float>& c) {
	m_Light.setSpecular(c);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Vector3<float>& SpotLightNode::getPosition() const {
	return m_Position;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	SpotLightNode::setPosition(const Vector3<float>& p) {
	m_Position = p;
	updateTransform();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Vector3<float>& SpotLightNode::getAttenuation() const {
	return m_Light.getAttenuation();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	SpotLightNode::setAttenuation(const Vector3<float>& a) {
	m_Light.setAttenuation(a);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void	SpotLightNode::setCutoff(float f) {
	m_Light.setCutoff(f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float SpotLightNode::getCutoff() const {
	return m_Light.getCutoff();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void SpotLightNode::setDirection(const Vector3<float>& d) {
	m_Direction = d;
	updateTransform();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline const Vector3<float>& SpotLightNode::getDirection() const {
	return m_Direction;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void SpotLightNode::setExponent(float e) {
	m_Light.setExponent(e);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float SpotLightNode::getExponent() const {
	return m_Light.getExponent();
}

}; };

#endif
