/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_TRAVERSER_H
#define __SCENE3D_TRAVERSER_H

#include "math.vector3.h"
#include "scene3d.leaf.h"

namespace magnum { namespace scene3d {
using magnum::math::Vector;
using magnum::math::Vector3;
using magnum::geometry::Sphere;
using magnum::geometry::Cube;

class Universe;
class Locale;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T, class L> class Traverser : public Object {
DeclareClass(Traverser, Object);
friend class magnum::geometry::Space<float,3,L>;
public:
	typedef magnum::geometry::Containment	Visibility;

private:
	const Universe*			m_Universe;
	const Locale*			m_Locale;
	Transform3<float>		m_PreTransform;
	Vector3<double>			m_PreTranslation;
	Transform3<float>		m_Transform;
	Vector3<double>			m_Translation;

public:
	Traverser();
	Traverser(const Locale* );
	Traverser(const Universe* );

	inline void				setPreTransform(const Transform3<float>& );
	inline const Transform3<float>& getPreTransform() const;
	inline void				setPreTranslation(const Vector3<double>& );
	inline const Vector3<double>& getPreTranslation() const;

	template<class V> 
	void					traverse(const V&, const Vector3<double>& = Vector3<double>::Null);
	void					traverse(const Vector3<float>&, const Vector3<double>& = Vector3<double>::Null);
	
	inline const Locale*	getLocale() const;
	inline const Universe*	getUniverse() const;

	inline const Transform3<float>& getTransform() const;
	inline const Vector3<double>& getTranslation() const;

protected:
	inline void				onVisit(L* );

private:
	inline bool				operator()(L* );
};


}; };

#endif
