/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SCENE3D_VIEWER_H
#define __SCENE3D_VIEWER_H

#include "object.pointer.h"
#include "math.vector3.h"
#include "geometry.ray.h"
#include "geometry.frustum.h"
#include "geometry.transform.h"
#include "graphics3d.perspectivecamera.h"
#include "scene3d.graphics.h"

namespace magnum { namespace scene3d {
using magnum::object::Object;
using magnum::object::Pointer;
using magnum::types::String;
using magnum::graphics::Rectangle;
using magnum::graphics3d::Camera;
using magnum::graphics3d::PerspectiveCamera;
using magnum::math::Vector3;
using magnum::math::Vector;
using magnum::geometry::Frustum3;
using magnum::geometry::Transform3;
using magnum::geometry::Ray;

class Locale;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Viewer : public Graphics {
DeclareClass(Viewer, Graphics);
protected:
	Viewer(const Viewer& );
	
public:
	Viewer();
	
	inline Camera*			getCamera();
	inline const Camera*	getCamera() const;

	inline void				setTransform(const Transform3<float>& );
	inline const Transform3<float>& getTransform() const;
	inline const Transform3<float>& getWorldTransform() const;
	
	inline const Frustum3<float>& getFrustum() const;

	inline void				setViewPlane(float );
	inline float			getViewPlane() const;
	inline void				setViewWindow(const Rectangle<float>& );
	inline void				setNearPlane(float );
	inline float			getNearPlane() const;
	inline void				setFarPlane(float );
	inline float			getFarPlane() const;

	Ray<float,3>			createRay(float ,float ) const;

protected:
	virtual Viewer*			onClone() const;
	virtual void			onAttachLocale(Locale* );
	virtual void			onDetachLocale(Locale* );
	virtual void			onEnable();
	virtual void			onDisable();
	virtual void			onTransform(const Transform3<float>& parent);

private:
	void					updateFrustum();	

private:
	Transform3<float>		m_Transform;
	Transform3<float>		m_WorldTransform;
	Frustum3<float>			m_Frustum;
	PerspectiveCamera		m_Camera;
};

}; };

#endif
