/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __STATS_NORMAL_INL
#define __STATS_NORMAL_INL

namespace magnum { namespace stats {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Normal::Normal() {
	m_Mean = 0.0f;
	m_Variance = 1.0f;
	m_VarianceSqrt = 1.0f;
	m_Ready = -1;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Normal::Normal(float m,float v) {
	m_Mean = m;
	m_Variance = v;
	m_VarianceSqrt = sqrt(v);
	m_Ready = -1;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Normal::setMean(float m) {
	m_Mean = m;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Normal::getMean() const {
	return m_Mean;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Normal::setVariance(float v) {
	m_Variance = v;
	m_VarianceSqrt = sqrt(v);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Normal::getVariance() const {
	return m_Variance;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float Normal::operator()() {
	float u,v,s;

	m_Ready = -m_Ready;
	if (m_Ready > 0) 
		return m_NextValue;
	u = randf();
	v = randf();
	s = m_VarianceSqrt * (float)sqrt(-2.0 * log(u));
	m_NextValue = s * (float)sin(2*M_PI * v) + m_Mean;
	return s * (float)cos(2*M_PI * v) + m_Mean;
}


}; };

#endif
