/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SYSTEM_MUTEX_INL
#define __SYSTEM_MUTEX_INL

namespace magnum { namespace system {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Mutex::Mutex() {
#if defined(__FORWINDOWS__)
	InitializeCriticalSection(&m_Mutex);
#elif defined(__FORUNIX__)
	pthread_mutex_init(&m_MutexHandle, NULL); 
#endif
	m_Locked = false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Mutex::~Mutex() {
#if defined(__FORWINDOWS__)
	DeleteCriticalSection(&m_Mutex);
#elif defined(__FORUNIX__)
	pthread_mutex_destroy(&m_MutexHandle);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool	Mutex::lock() {
#if defined(__FORWINDOWS__)
	EnterCriticalSection(&m_Mutex);
	return true;
#elif defined(__FORUNIX__)
	pthread_mutex_lock(&m_MutexHandle);
#endif
	m_Locked = true;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool	Mutex::unlock() {
	m_Locked = false;
#if defined(__FORWINDOWS__)
	LeaveCriticalSection(&m_Mutex);
	return true;
#elif defined(__FORUNIX__)
	return (pthread_mutex_unlock(&m_MutexHandle) == 0);	
#endif
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool	Mutex::isLocked() const {
	return m_Locked;
}

}; };
#endif
