/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SYSTEM_SIGNAL_INL
#define __SYSTEM_SIGNAL_INL

namespace magnum { namespace system {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Signal::Signal() {
	m_Signaled = false;
#if defined(__FORWINDOWS__)
	m_Event = CreateEvent(NULL, TRUE, FALSE, NULL);
#else
	pthread_cond_init(&m_Event, NULL);
	pthread_mutex_init(&m_Mutex, NULL);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Signal::~Signal() {
#if defined(__FORWINDOWS__)
	CloseHandle(m_Event);
#else
	pthread_cond_destroy(&m_Event);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Signal::raise(bool autoreset) {
	m_Signaled = !autoreset;

#if defined(__FORWINDOWS__)
	if (autoreset)
		PulseEvent(m_Event);
	else
		SetEvent(m_Event);
#else
	pthread_cond_broadcast(&m_Event);
#endif

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Signal::reset() {
	if (!m_Signaled)
		return false;

#if defined(__FORWINDOWS__)
	ResetEvent(m_Event);
#endif

	m_Signaled = false;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Signal::wait() {
	if (m_Signaled)
		return true;

#if defined(__FORWINDOWS__)
	WaitForSingleObject(m_Event, INFINITE);
#else
	pthread_mutex_lock(&m_Mutex);
	pthread_cond_wait(&m_Event, &m_Mutex);
	pthread_mutex_unlock(&m_Mutex);
#endif
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Signal::wait(unsigned int timeout) {
	if (m_Signaled)
		return true;

#if defined(__FORWINDOWS__)
	WaitForSingleObject(m_Event, timeout);
#else
	pthread_mutex_lock(&m_Mutex);
	pthread_cond_wait(&m_Event, &m_Mutex);
	pthread_mutex_unlock(&m_Mutex);
#endif
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Signal::isRaised() const {
	return m_Signaled;
}


}; };

#endif
