/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SYSTEM_THREAD_INL
#define __SYSTEM_THREAD_INL

namespace magnum { namespace system {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool Thread::isAlive() const {
	return m_Signal.isRaised();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool	Thread::isInterrupted() const {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline int Thread::getThreadId() const {
	return (int)m_ThreadId;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void* Thread::getTLS(Thread::Storage s) {
#if defined(__FORWINDOWS__)
	return TlsGetValue(s);
#elif defined(__FORUNIX__)
	return pthread_getspecific(s);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void Thread::setTLS(Thread::Storage s, const void* v) {
#if defined(__FORWINDOWS__)
	TlsSetValue(s, (void *)v);
#elif defined(__FORUNIX__)
	pthread_setspecific(s, v);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline Thread* Thread::getCurrent() {
	Thread* t = (Thread *)getTLS(g_Current);
	if (t == NULL)
		return searchCurrent();
	else
		return t;
}


}; };
#endif
