/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#ifndef __SYSTEM_TIMESTAMP_H
#define __SYSTEM_TIMESTAMP_H

#include "object.h"

namespace magnum { namespace system {
using magnum::object::Object;
using magnum::types::String;
using magnum::types::uint64;
using magnum::types::int64;

/*----------------------------------------------------------------------------
This class provides a timestamp that counts the the number of 100-nanosecond 
intervals since January 1, 1601. 
----------------------------------------------------------------------------*/
class Timestamp : public Object {
DeclareClass(Timestamp, Object);
private:
	uint64			m_Time;
	
public:
	enum {
		Microsecond = 1,
		Millisecond = 1000,
		Second		= 1000000,
		Minute		= 60000000,
		Hour		= 3600000000,
	};

public:
	inline Timestamp();
	inline explicit Timestamp(const uint64 time);

	void			getCurrentTime();
	
	String			toString() const;

	inline void		setTime(const uint64 time);
	inline uint64	getTime() const;

	inline bool		isBefore(const Timestamp& time) const;
	inline bool		isAfter(const Timestamp& time) const;

	inline bool		operator< (const Timestamp& time) const;
	inline bool		operator> (const Timestamp& time) const;
	inline bool		operator<= (const Timestamp& time) const;
	inline bool		operator>= (const Timestamp& time) const;
	inline bool		operator== (const Timestamp& time) const;
	inline bool		operator!= (const Timestamp& time) const;

	inline int64	operator- (const Timestamp& time) const;
	inline Timestamp operator+ (const int64& time) const;
};

}; };
#endif
