/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "debug.logger.h"
#include "system.stdio.h"

#include "object.inl"
#include "base.string.inl"
#include "system.stdio.inl"
#include "system.mutex.inl"
#include "system.timestamp.inl"
#include "io.stream.inl"
#include "io.outputstream.inl"
#include "debug.logger.inl"


namespace magnum { namespace debug {
using magnum::system::Err;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Logger::info(const char* str) {
	m_Mutex.lock();
	try {
		m_Timestamp.getCurrentTime();
		onInfo(m_Timestamp, str);
	} catch(...) {
	}
	m_Mutex.unlock();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Logger::warn(const char* str) {
	m_Mutex.lock();
	try {
		m_Timestamp.getCurrentTime();
		onWarn(m_Timestamp, str);
	} catch(...) {
	}
	m_Mutex.unlock();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Logger::error(const char* str) {
	m_Mutex.lock();
	try {
		m_Timestamp.getCurrentTime();
		onError(m_Timestamp, str);
	} catch(...) {
	}
	m_Mutex.unlock();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Logger::onInfo(const Timestamp& t,const char* str) {
	String line = "[Info   :";
	line += t.toString();
	line += "] -- ";
	line += str;
	onPrint(str);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Logger::onWarn(const Timestamp& t,const char* str) {
	String line = "[Warning:";
	line += t.toString();
	line += "] -- ";
	line += str;
	onPrint(str);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Logger::onError(const Timestamp& t,const char* str) {
	String line = "[Error:";
	line += t.toString();
	line += "] -- ";
	line += str;
	onPrint(str);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Logger::onPrint(const String& str) {
	Err << str << "\n";
}

}; };
