/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "debug.profiler.h"
#include "debug.timer.h"
#include "debug.counter.h"
#include <typeinfo.h>

#include "container.chain.inl"
#include "container.iterator.inl"
#include "debug.profiler.inl"
#include "debug.instrument.inl"
#include "debug.timer.inl"
#include "debug.counter.inl"


namespace magnum { namespace debug {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Timer* Profiler::createTimer(const char* n) {
	Instrument* i = findInstrument(n);
	if (i != NULL) {
		if (typeid(*i) != typeid(Timer))
			return NULL;
		i->refer();
		return (Timer*)i;
	}
	
	Timer* t = new Timer(n);
	m_Instruments.insert(t);
	return t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Timer* Profiler::getTimer(const char* n) const {
	Instrument* i = findInstrument(n);
	if (i == NULL)
		return NULL;
	if (typeid(*i) != typeid(Timer))
		return NULL;
	return (Timer*)i;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Counter* Profiler::createCounter(const char* n) {
	Instrument* i = findInstrument(n);
	if (i != NULL) {
		if (typeid(*i) != typeid(Counter))
			return NULL;
		i->refer();
		return (Counter*)i;
	}
	
	Counter* c = new Counter(n);
	m_Instruments.insert(c);
	return c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Counter* Profiler::getCounter(const char* n) const {
	Instrument* i = findInstrument(n);
	if (i == NULL)
		return NULL;
	if (typeid(*i) != typeid(Counter))
		return NULL;
	return (Counter*)i;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Instrument* Profiler::findInstrument(const char* n) const {
	Iterator iter, end;
	end = m_Instruments.end();
	for (iter = m_Instruments.begin(); iter != end; ++iter) {
		if (iter->m_Name == n)
			return (Instrument*)iter;
	}
	
	return NULL;
}

}; };

