/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics.bitmap.h"
#include "graphics.palette.h"

#include "object.inl"
#include "base.string.inl"
#include "graphics.image.inl"
#include "graphics.bitmap.inl"
#include "graphics.rectangle.inl"
#include "graphics.dimension.inl"
#include "graphics.colour.inl"
#include "graphics.palette.inl"


namespace magnum { namespace graphics {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bitmap::Bitmap() {
	m_Pixels = NULL;
	m_Palette = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bitmap::Bitmap(Type type, int w, int h) : Image(type, w, h) {
	m_Pixels = new byte[getSize().getHeight()*getLinesize()];
	if (isIndexed())
		m_Palette = new Palette<byte>();
	else
		m_Palette = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bitmap::Bitmap(Type type, const Dimension<int>& size) : Image(type, size) {
	m_Pixels = new byte[getSize().getHeight()*getLinesize()];
	if (isIndexed())
		m_Palette = new Palette<byte>();
	else
		m_Palette = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bitmap::Bitmap(const Image* img) : Image(img->getType(), img->getSize()) {
	m_Pixels = new byte[getSize().getHeight()*getLinesize()];
	if (isIndexed())
		m_Palette = new Palette<byte>();
	else
		m_Palette = NULL;

	copyPixels(img);
	copyPalette(img);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bitmap::Bitmap(const Image& img) : Image(img.getType(), img.getSize()) {
	m_Pixels = new byte[getSize().getHeight()*getLinesize()];
	if (isIndexed())
		m_Palette = new Palette<byte>();
	else
		m_Palette = NULL;

	copyPixels(&img);
	copyPalette(&img);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bitmap::~Bitmap() {
	if (m_Pixels != NULL)
		delete[] (byte *)m_Pixels;
	m_Pixels = NULL;

	if (m_Palette != NULL)
		m_Palette->release();
	m_Palette = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void* Bitmap::onLock() {
	return m_Pixels;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Bitmap::onUnlock() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Bitmap::onGetPalette(Palette<byte>* pal) const {
	if (m_Palette == NULL)
		return false;
	*pal = *m_Palette;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Bitmap::onSetPalette(const Palette<byte>& pal) {
	if (m_Palette == NULL)
		return false;
	*m_Palette = pal;
	return true;
}

}; };
