/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics.bitmap.h"
#include "graphics.filter.shrink.h"

#include "io.filter.inl"
#include "container.chain.inl"
#include "graphics.filter.inl"
#include "graphics.image.inl"


namespace magnum { namespace graphics {
using magnum::types::byte;
using magnum::types::word;
using magnum::types::dword;
using magnum::types::qword;

struct byte3 {
	byte	_data[3];
};

struct byte6 {
	byte	_data[6];
};

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::onProcess(const Image* in) {
	switch (in->getPixelsize()) {
		case 8:
			return resize<byte>((Image *)in);
		case 16:
			return resize<word>((Image *)in);
		case 24:
			return resize<byte3>((Image *)in);
		case 32:
			return resize<dword>((Image *)in);
		case 48:
			return resize<byte6>((Image *)in);
		case 64:
			return resize<qword>((Image *)in);
		default:
			return NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
template<class T>
Image* FilterShrink::resize(Image* in) {
	int w = in->getWidth()/2;
	int h = in->getHeight()/2;
	if ((w <= 0) || (h <= 0))
		return NULL;

	Image* out = new Bitmap(in->getType(), w, h);
	if (out == NULL)
		return NULL;

	if (in->isIndexed())
		out->copyPalette(in);

	int out_ls = out->getLinesize();
	int in_ls = in->getLinesize() << 1;
	int in_w = in->getWidth();

	T* src = (T *)in->lock();
	T* dst = (T *)out->lock();

	int x,y;
	for (y = 0; y < h; y++) {
		for (x = 0; x < w; x++) {
			dst[x] =  src[x << 1];
		}

		dst = (T *)((byte *)(void *)dst + out_ls);
		src = (T *)((byte *)(void *)src + in_ls);
	}

	in->unlock();
	out->unlock();
	
	return out;	
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resample8(Image* in) {
	return NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resample16(Image* in) {
	return NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resample24(Image* in) {
	return NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Image* FilterShrink::resample32(Image* in) {
	return NULL;
}


}; };
