/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "math.consts.h"
#include "graphics3d.parameter.h"
#include "graphics3d.parameterbuffer.h"
#include "graphics3d.parameterformat.h"
#include "graphics3d.animation.h"

#include "graphics3d.animation.inl"
#include "graphics3d.parameter.inl"
#include "graphics3d.parameterbuffer.inl"
#include "graphics3d.parameterformat.inl"

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Animation::Animation() {
	m_Location = Parameters::AnimationData;
	m_Start = 0.0f;
	m_End = FLT_MAX;
	m_Length = FLT_MAX;
	m_Direction = Forward;
	m_Interpolation = Linear;
	m_Repeat = Single;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Animation::Animation(const char* n) : m_Name(n) {
	m_Location = Parameters::AnimationData;
	m_Start = 0.0f;
	m_End = FLT_MAX;
	m_Length = FLT_MAX;
	m_Direction = Forward;
	m_Interpolation = Linear;
	m_Repeat = Single;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Animation::onAnimate(float ,void* ) const {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
float Animation::normaliseTime(float v) const {
	// If time is before start time, do nothing
	if (v < m_Start) {
		if (m_Direction == Backward)
			return m_End;
		else
			return m_Start;
	}
		
	// Translate time by starting time
	v -= m_Start;
	switch (m_Repeat) {
		case Loop:
			v = fmodf(v,m_Length);
			break;
		case PingPongLoop:
			v = fmodf(v,2.0f*m_Length);
			if (v > m_Length)
				v = 2*m_Length - v;
			break;
		case PingPong:
			if (v > 2*m_Length)
				v = 0.0f;
			else if (v > m_Length)
				v = 2*m_Length - v;
			break;
		default:
			if (v > m_Length)
				v = m_Length;
			break;
	}

	// If we are playing backward, mirror time
	if (m_Direction == Backward)
		v =  m_Length - v;

	// Translate time back
	v += m_Start;

	return v;
}

}; };
