/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.context.h"
#include "graphics3d.blendmode.h"
#include "graphics3d.vertexformat.h"

#include "object.inl"
#include "base.string.inl"
#include "system.thread.inl"
#include "system.assembler.inl"
#include "graphics3d.context.inl"
#include "graphics3d.blendmode.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.vertexformat.inl"


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
---------------------------------------------------------------------------*/
BlendMode::BlendMode() {
	m_AlphaTest = Disable;
	m_AlphaRef = 0.0f;
	m_Source = One;
	m_Destination = Zero;
	m_Equation = Add;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
BlendMode::~BlendMode() {
	// Remove BlendMode from all Contexts!
	Context* context = getContext();
	if (context != NULL) {
		if (context->m_BlendMode == this)
			context->m_BlendMode = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void BlendMode::compileSelect(Assembler& as) const {
	Context* context = getContext();
	int offset = (int)((byte*)&context->m_BlendMode - (byte*)context);
	as.asmLoad(Assembler::T0, (int)(intptr_t)this);
	as.asmStore(Assembler::R0, offset, Assembler::T0);
	if (onCompileSelect(as) == false)
		as.asmCall((void (BlendMode::*)() const)&BlendMode::onSelect, this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void BlendMode::compileDeselect(Assembler& as) const {
	Context* context = getContext();
	int offset = (int)((byte*)&context->m_BlendMode - (byte*)context);
	as.asmLoad(Assembler::T0, NULL);
	as.asmStore(Assembler::R0, offset, Assembler::T0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void BlendMode::onSelect() const {
	// Blending
	if ((m_Source == One) && (m_Destination == Zero)) {
		glDisable(GL_BLEND);
	} else {
		glEnable(GL_BLEND);
		glBlendFunc(m_Source ,m_Destination);
		if (glBlendEquationEXT)
			glBlendEquationEXT(m_Equation);
		if (glBlendColorEXT && (m_Source >= ConstColour || m_Destination >= ConstColour))
			glBlendColorEXT(m_Colour.R, m_Colour.G, m_Colour.B, m_Colour.A);
	}

	// Alpha test related settings
	if (m_AlphaTest == Disable) {
		glDisable(GL_ALPHA_TEST);
	} else {
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(m_AlphaTest, m_AlphaRef);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool BlendMode::onCompileSelect(Assembler& as) const {
	// Blending
	if ((m_Source == One) && (m_Destination == Zero)) {
		as.asmCall(glDisable,(GLenum)GL_BLEND);
	} else {
		as.asmCall(glEnable,(GLenum)GL_BLEND);
		as.asmCall(glBlendFunc,(GLenum)m_Source ,(GLenum)m_Destination);
		if (glBlendEquationEXT)
			as.asmCall(glBlendEquationEXT,(GLenum)m_Equation);
		if (glBlendColorEXT && (m_Source >= ConstColour || m_Destination >= ConstColour))
			as.asmCall(glBlendColorEXT,m_Colour.R, m_Colour.G, m_Colour.B, m_Colour.A);
	}

	// Alpha test related settings
	if (m_AlphaTest == Disable) {
		as.asmCall(glDisable,(GLenum)GL_ALPHA_TEST);
	} else {
		as.asmCall(glEnable,(GLenum)GL_ALPHA_TEST);
		as.asmCall(glAlphaFunc,(GLenum)m_AlphaTest, m_AlphaRef);
	}
	return true;
}


}; };
