/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "algorithm.sort.h"
#include "geometry.transform.h"
#include "graphics3d.parameterbuffer.h"
#include "graphics3d.parameterformat.h"
#include "graphics3d.skeleton.h"
#include "graphics3d.boneanimation.h"

#include "container.array.inl"
#include "container.list.inl"
#include "container.iterator.inl"
#include "algorithm.sort.inl"
#include "algorithm.swap.inl"
#include "math.vector.inl"
#include "math.matrix.inl"
#include "math.quaternion.inl"
#include "geometry.transform.inl"
#include "graphics3d.parameterformat.inl"
#include "graphics3d.parameterbuffer.inl"
#include "graphics3d.keyframe.inl"
#include "graphics3d.skeleton.inl"
#include "graphics3d.animation.inl"
#include "graphics3d.boneanimation.inl"

namespace magnum { namespace graphics3d {
using magnum::algorithm::sort;
using magnum::geometry::Transform3;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void BoneAnimation::clear() {
	m_Frames.clear();
	setStart(0.0f);
	setEnd(0.0f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void BoneAnimation::finish() {
	sortKeyframes(m_Frames);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void BoneAnimation::remove(float f) {
	for (Iterator iter = m_Frames.begin(); iter != m_Frames.end(); ++iter) {
		if (iter->getTime() == f) {
			m_Frames.remove(iter);
			return;
		}
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void BoneAnimation::onAnimate(float t,void* data) const {
	// Normalise time
	t = normaliseTime(t);
	
	// Get keyframe
	const Frame* frame0 = getKeyframe(m_Frames, t);
	const Frame* frame1 = frame0+1;
	ASSERT(frame0 != NULL);
	ASSERT(t >= frame0->getTime());
	ASSERT(t - frame0->getTime() <= frame0->getDuration());
	
	// Get weight from keyframe
	t = (t - frame0->getTime()) / frame0->getDuration();
	Matrix4<float>* dst = reinterpret_cast<Matrix4<float>*>(data);
	for (Skeleton::Iterator iter = m_Skeleton->begin(); iter != m_Skeleton->end(); ++iter) {
		unsigned int idx = iter->getIndex();
		const BoneTransform& bone0 = frame0->getKey(idx);
		const BoneTransform& bone1 = frame1->getKey(idx);
		if (t == 0.0f) {
			Vector3<float> offset =  bone0.offset;
			Quaternion<float> rot = bone0.rotation;
			dst[idx] = Transform3<float>(rot.toMatrix(), offset).toMatrix4();
		} else {
			Vector3<float> offset =  t*(bone1.offset - bone0.offset) + bone0.offset;
			Quaternion<float> rot = nlerp(t,bone0.rotation,bone1.rotation);
			dst[idx] = Transform3<float>(rot.toMatrix(), offset).toMatrix4();
		}
	}
}

}; };
