/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.context.h"
#include "graphics3d.appearance.h"
#include "graphics3d.shader.h"
#include "graphics3d.material.h"
#include "graphics3d.blendmode.h"
#include "graphics3d.stencilmode.h"
#include "graphics3d.depthmode.h"
#include "graphics3d.polygonmode.h"
#include "graphics3d.mesh.h"
#include "graphics3d.deferedmesh.h"

#include "math.matrix.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.context.inl"
#include "graphics3d.appearance.inl"
#include "graphics3d.shader.inl"
#include "graphics3d.material.inl"
#include "graphics3d.blendmode.inl"
#include "graphics3d.stencilmode.inl"
#include "graphics3d.depthmode.inl"
#include "graphics3d.polygonmode.inl"
#include "graphics3d.mesh.inl"
#include "graphics3d.deferedmesh.inl"

namespace magnum { namespace graphics3d {

/*--------------------------------------------------------------------------*/
/**
 */
DeferedMesh::DeferedMesh(const Context* context,const Mesh* mesh,const Matrix<float,4>& mtx) {
	ASSERT(context != NULL);
	ASSERT(mesh != NULL);
	
	m_Mesh = mesh;
	m_ModelView = mtx;
//	m_Bounds = mesh->getBounds();
//	m_Bounds.transform(m_ModelView);

	Vector3<float> center = m_ModelView * mesh->getBounds().getOrigin();
	m_Key = center.z;

	const Appearance* app = m_Mesh->getAppearance();
	if (app == NULL) {
		m_BlendMode = context->getBlendMode();
		m_PolygonMode = context->getPolygonMode();
		m_StencilMode = context->getStencilMode();
		m_DepthMode = context->getDepthMode();
		m_Shader = context->getShader();
		m_Material = context->getMaterial();
		for (unsigned int i = 0; i < Texture::MaxTextures; i++)
			m_Texture[i] = context->getTexture(i);
	} else {
		if (app->getBlendMode() != NULL)
			m_BlendMode = app->getBlendMode();
		else
			m_BlendMode = context->getBlendMode();

		if (app->getPolygonMode() != NULL)
			m_PolygonMode = app->getPolygonMode();
		else
			m_PolygonMode = context->getPolygonMode();

		if (app->getStencilMode() != NULL)
			m_StencilMode = app->getStencilMode();
		else
			m_StencilMode = context->getStencilMode();

		if (app->getDepthMode() != NULL)
			m_DepthMode = app->getDepthMode();
		else
			m_DepthMode = context->getDepthMode();

		if (app->getShader() != NULL)
			m_Shader = app->getShader();
		else
			m_Shader = context->getShader();

		if (app->getMaterial() != NULL)
			m_Material = app->getMaterial();
		else
			m_Material = context->getMaterial();

		for (unsigned int i = 0; i < Texture::MaxTextures; i++) {
			if (app->getTexture(i) != NULL)
				m_Texture[i] = app->getTexture(i);
			else
				m_Texture[i] = context->getTexture(i);
		}
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
void DeferedMesh::render() const {
	Context* context = Context::getCurrent();
	if (m_Shader != NULL)
		m_Shader->select();
	if (m_Material != NULL)
		m_Material->select();
	if (m_BlendMode != NULL)
		m_BlendMode->select();
	if (m_StencilMode != NULL)
		m_StencilMode->select();
	if (m_DepthMode != NULL)
		m_DepthMode->select();
	if (m_PolygonMode != NULL)
		m_PolygonMode->select();
	for (int i = 0; i < Texture::MaxTextures; i++) {
		if (m_Texture[i] != NULL) {
			m_Texture[i]->select(i);
		}
	}

	MatrixStack* ms = context->getMatrixStack();
	ms->setModel(m_ModelView);

	Appearance* app = m_Mesh->getAppearance();
	if (app	!= NULL) {
		app->refer();
		const_cast<Mesh*>(m_Mesh)->detachAppearance();
		m_Mesh->render();
		const_cast<Mesh*>(m_Mesh)->attachAppearance(app);
		app->release();
	} else {
		m_Mesh->render();
	}
}

}; };

