/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.context.h"
#include "graphics3d.depthmode.h"
#include "graphics3d.vertexformat.h"

#include "object.inl"
#include "base.string.inl"
#include "math.range.inl"
#include "system.thread.inl"
#include "system.assembler.inl"
#include "graphics3d.context.inl"
#include "graphics3d.depthmode.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.vertexformat.inl"


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
DepthMode::DepthMode() {
	m_Range.setStart(0.0f);
	m_Range.setEnd(1.0f);
	m_Function = LessEqual;
	m_Write = true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
DepthMode::~DepthMode() {
	// Remove DepthMode from all Contexts!
	Context* context = getContext();
	if (context != NULL) {
		if (context->m_DepthMode == this)
			context->m_DepthMode = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void DepthMode::compileSelect(Assembler& as) const {
	Context* context = getContext();
	int offset = (int)((byte*)&context->m_DepthMode - (byte*)context);
	as.asmLoad(Assembler::T0, (int)(intptr_t)this);
	as.asmStore(Assembler::R0, offset, Assembler::T0);
	if (onCompileSelect(as) == false)
		as.asmCall((void (DepthMode::*)() const)&DepthMode::onSelect, this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void DepthMode::compileDeselect(Assembler& as) const {
	Context* context = getContext();
	int offset = (int)((byte*)&context->m_DepthMode - (byte*)context);
	as.asmLoad(Assembler::T0, NULL);
	as.asmStore(Assembler::R0, offset, Assembler::T0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void DepthMode::onSelect() const {
	// Depth test realted settings
	if (m_Function == Disable) {
		glDisable(GL_DEPTH_TEST);
	} else {
		glEnable(GL_DEPTH_TEST);
		glDepthFunc(m_Function);
	}
	glDepthMask(m_Write);
	glDepthRange(m_Range.getStart(), m_Range.getEnd());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void DepthMode::onDeselect() const {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool DepthMode::onCompileSelect(Assembler& as) const {
	if (m_Function == Disable) {
		as.asmCall(glDisable, (GLenum)GL_DEPTH_TEST);
	} else {
		as.asmCall(glEnable, (GLenum)GL_DEPTH_TEST);
		as.asmCall(glDepthFunc, (GLenum)m_Function);
	}
	as.asmCall(glDepthMask, (GLboolean)m_Write);
	as.asmCall(glDepthRange, (GLclampd)m_Range.getStart(), (GLclampd)m_Range.getEnd());
	return true;
}


}; };
