/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "algorithm.sort.h"
#include "graphics3d.parameterbuffer.h"
#include "graphics3d.parameterformat.h"
#include "graphics3d.frameanimation.h"

#include "container.array.inl"
#include "container.iterator.inl"
#include "algorithm.sort.inl"
#include "algorithm.swap.inl"
#include "graphics3d.keyframe.inl"
#include "graphics3d.animation.inl"
#include "graphics3d.parameterformat.inl"
#include "graphics3d.parameterbuffer.inl"
#include "graphics3d.frameanimation.inl"

namespace magnum { namespace graphics3d {
using magnum::algorithm::sort;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void FrameAnimation::clear() {
	m_Frames.clear();
	setStart(0.0f);
	setEnd(0.0f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void FrameAnimation::finish() {
	sortKeyframes(m_Frames);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void FrameAnimation::remove(float f) {
	for (Iterator iter = m_Frames.begin(); iter != m_Frames.end(); ++iter) {
		if (iter->getTime() == f) {
			m_Frames.remove(iter);
			return;
		}
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void FrameAnimation::onAnimate(float t,void* data) const {
	// Normalise time
	t = normaliseTime(t);
	
	// Get keyframe
	const Frame* frame0 = getKeyframe(m_Frames, t);
	const Frame* frame1 = frame0+1;
	ASSERT(frame0 != NULL);
	ASSERT(t >= frame0->getTime());
	ASSERT(t - frame0->getTime() <= frame0->getDuration());
	
	// Get weight from keyframe
	t = 1.0f - (t - frame0->getTime()) / frame0->getDuration();
	*((unsigned int*)data) = frame0->getKey();
	*((unsigned int*)data+1) = frame1->getKey();
	*((float *)data + 2) = t;
}

}; };
