/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.meshfactory.h"

#include "graphics3d.meshfactory.inl"
#include "graphics3d.meshbuilder.inl"

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Mesh* MeshFactory::createCube(Vector3<float>& v,float f) {
	return createCube(Cube<float,3>(v,f));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Mesh* MeshFactory::createCube(const Cube<float,3>& cube) {
	Vector3<float> o = cube.getOrigin();
	float r = cube.getRadius();
	
	m_Builder.clear();
	m_Builder.setPrimitive(Mesh::QuadList);
	m_Builder.setAttributes(Attributes::Vertex | Attributes::Normal | Attributes::Texture);
	
	// Face 1
	m_Builder.normal(-1.0f, 0.0f, 0.0f);
	m_Builder.vertex(o.x - r, o.y - r, o.z - r);
	m_Builder.vertex(o.x - r, o.y - r, o.z + r);
	m_Builder.vertex(o.x - r, o.y + r, o.z + r);
	m_Builder.vertex(o.x - r, o.y + r, o.z - r);

	// Face 2
	m_Builder.normal(+1.0f, 0.0f, 0.0f);
	m_Builder.vertex(o.x + r, o.y - r, o.z - r);
	m_Builder.vertex(o.x + r, o.y + r, o.z - r);
	m_Builder.vertex(o.x + r, o.y + r, o.z + r);
	m_Builder.vertex(o.x + r, o.y - r, o.z + r);

	// Face	3
	m_Builder.normal(0.0f, -1.0f, 0.0f);
	m_Builder.vertex(o.x - r, o.y - r, o.z - r);
	m_Builder.vertex(o.x + r, o.y - r, o.z - r);
	m_Builder.vertex(o.x + r, o.y - r, o.z + r);
	m_Builder.vertex(o.x - r, o.y - r, o.z + r);

	// Face	4
	m_Builder.normal(0.0f, 1.0f, 0.0f);
	m_Builder.vertex(o.x - r, o.y + r, o.z - r);
	m_Builder.vertex(o.x - r, o.y + r, o.z + r);
	m_Builder.vertex(o.x + r, o.y + r, o.z + r);
	m_Builder.vertex(o.x + r, o.y + r, o.z - r);

	// Face	5
	m_Builder.normal(0.0f, 0.0f, -1.0f);
	m_Builder.vertex(o.x - r, o.y - r, o.z - r);
	m_Builder.vertex(o.x - r, o.y + r, o.z - r);
	m_Builder.vertex(o.x + r, o.y + r, o.z - r);
	m_Builder.vertex(o.x + r, o.y - r, o.z - r);

	// Face	6
	m_Builder.normal(0.0f, 0.0f, 1.0f);
	m_Builder.vertex(o.x - r, o.y - r, o.z + r);
	m_Builder.vertex(o.x + r, o.y - r, o.z + r);
	m_Builder.vertex(o.x + r, o.y + r, o.z + r);
	m_Builder.vertex(o.x - r, o.y + r, o.z + r);

	m_Builder.finish();
	return m_Builder.build();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Mesh* MeshFactory::createBox(const Cube<float,3>& cube) {
	return createCube(cube);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Mesh* MeshFactory::createBox(const AlignedBox<float,3>& box) {
	Vector3<float> mi = box.getMinimum();
	Vector3<float> ma = box.getMaximum();

	m_Builder.clear();
	m_Builder.setPrimitive(Mesh::QuadList);
	m_Builder.setAttributes(Attributes::Vertex | Attributes::Normal | Attributes::Texture);

	// Face 1
	m_Builder.normal(-1.0f, 0.0f, 0.0f);
	m_Builder.vertex(mi.x, mi.y, mi.z);
	m_Builder.vertex(mi.x, mi.y, ma.z);
	m_Builder.vertex(mi.x, ma.y, ma.z);
	m_Builder.vertex(mi.x, ma.y, mi.z);

	// Face 2
	m_Builder.normal(1.0f, 0.0f, 0.0f);
	m_Builder.vertex(ma.x, mi.y, mi.z);
	m_Builder.vertex(ma.x, ma.y, mi.z);
	m_Builder.vertex(ma.x, ma.y, ma.z);
	m_Builder.vertex(ma.x, mi.y, ma.z);

	// Face	3
	m_Builder.normal(0.0f, -1.0f, 0.0f);
	m_Builder.vertex(mi.x, mi.y, mi.z);
	m_Builder.vertex(ma.x, mi.y, mi.z);
	m_Builder.vertex(ma.x, mi.y, ma.z);
	m_Builder.vertex(mi.x, mi.y, ma.z);
	
	// Face	4
	m_Builder.normal(0.0f, 1.0f, 0.0f);
	m_Builder.vertex(mi.x, ma.y, mi.z);
	m_Builder.vertex(mi.x, ma.y, ma.z);
	m_Builder.vertex(ma.x, ma.y, ma.z);
	m_Builder.vertex(ma.x, ma.y, mi.z);

	// Face	5
	m_Builder.normal(0.0f, 0.0f, -1.0f);
	m_Builder.vertex(mi.x, mi.y, mi.z);
	m_Builder.vertex(mi.x, ma.y, mi.z);
	m_Builder.vertex(ma.x, ma.y, mi.z);
	m_Builder.vertex(ma.x, mi.y, mi.z);

	// Face	6
	m_Builder.normal(0.0f, 0.0f, 1.0f);
	m_Builder.vertex(mi.x, mi.y, ma.z);
	m_Builder.vertex(ma.x, mi.y, ma.z);
	m_Builder.vertex(ma.x, ma.y, ma.z);
	m_Builder.vertex(mi.x, ma.y, ma.z);

	m_Builder.finish();
	return m_Builder.build();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Mesh* MeshFactory::createBox(const Box<float,3>& box) {
	Vector3<float> o = box.getOrigin();
	Vector3<float> v[4];
	Vector3<float> n;

	m_Builder.clear();
	m_Builder.setPrimitive(Mesh::QuadList);
	m_Builder.setAttributes(Attributes::Vertex | Attributes::Normal | Attributes::Texture);
	
	// Face 1 OK
	v[0] = o - box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	v[1] = o - box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	v[2] = o + box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	v[3] = o + box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(1)) % ((Vector3<float>)box.getAxis(0));
	n.normalise();
	m_Builder.normal(n.x, n.y, n.z);
	m_Builder.vertex(v[0].x, v[0].y, v[0].z);
	m_Builder.vertex(v[1].x, v[1].y, v[1].z);
	m_Builder.vertex(v[2].x, v[2].y, v[2].z);
	m_Builder.vertex(v[3].x, v[3].y, v[3].z);

	// Face 2 OK!
	v[0] = o - box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	v[1] = o - box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	v[2] = o - box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	v[3] = o - box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(2)) % ((Vector3<float>)box.getAxis(1));
	n.normalise();
	m_Builder.normal(n.x, n.y, n.z);
	m_Builder.vertex(v[0].x, v[0].y, v[0].z);
	m_Builder.vertex(v[1].x, v[1].y, v[1].z);
	m_Builder.vertex(v[2].x, v[2].y, v[2].z);
	m_Builder.vertex(v[3].x, v[3].y, v[3].z);
	
	// Face 3 OK
	v[0] = o - box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	v[1] = o + box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	v[2] = o + box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	v[3] = o - box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(0)) % ((Vector3<float>)box.getAxis(2));
	n.normalise();
	m_Builder.normal(n.x, n.y, n.z);
	m_Builder.vertex(v[0].x, v[0].y, v[0].z);
	m_Builder.vertex(v[1].x, v[1].y, v[1].z);
	m_Builder.vertex(v[2].x, v[2].y, v[2].z);
	m_Builder.vertex(v[3].x, v[3].y, v[3].z);

	// Face 4 OK
	v[0] = o + box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	v[1] = o + box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	v[2] = o + box.getAxis(0) - box.getAxis(1) - box.getAxis(2);
	v[3] = o + box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(1)) % ((Vector3<float>)box.getAxis(2));
	n.normalise();
	m_Builder.normal(n.x, n.y, n.z);
	m_Builder.vertex(v[0].x, v[0].y, v[0].z);
	m_Builder.vertex(v[1].x, v[1].y, v[1].z);
	m_Builder.vertex(v[2].x, v[2].y, v[2].z);
	m_Builder.vertex(v[3].x, v[3].y, v[3].z);

	// Face 5 
	v[0] = o + box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	v[1] = o + box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	v[2] = o - box.getAxis(0) + box.getAxis(1) - box.getAxis(2);
	v[3] = o - box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(2)) % ((Vector3<float>)box.getAxis(0));
	n.normalise();
	m_Builder.normal(n.x, n.y, n.z);
	m_Builder.vertex(v[0].x, v[0].y, v[0].z);
	m_Builder.vertex(v[1].x, v[1].y, v[1].z);
	m_Builder.vertex(v[2].x, v[2].y, v[2].z);
	m_Builder.vertex(v[3].x, v[3].y, v[3].z);

	// Face 6
	v[0] = o + box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	v[1] = o - box.getAxis(0) + box.getAxis(1) + box.getAxis(2);
	v[2] = o - box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	v[3] = o + box.getAxis(0) - box.getAxis(1) + box.getAxis(2);
	n = ((Vector3<float>)box.getAxis(0)) % ((Vector3<float>)box.getAxis(1));
	n.normalise();
	m_Builder.normal(n.x, n.y, n.z);
	m_Builder.vertex(v[0].x, v[0].y, v[0].z);
	m_Builder.vertex(v[1].x, v[1].y, v[1].z);
	m_Builder.vertex(v[2].x, v[2].y, v[2].z);
	m_Builder.vertex(v[3].x, v[3].y, v[3].z);

	m_Builder.finish();
	return m_Builder.build();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Mesh* MeshFactory::createSphere(Vector3<float>& v,float r,unsigned int n) {
	return createSphere(Sphere<float,3>(v,r),n);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Mesh* MeshFactory::createSphere(const Sphere<float,3>& sphere, unsigned int n) {
	unsigned int i,j;
	float theta1, theta2, theta3;
	Vector3<float> e,p;
	Vector3<float> c = sphere.getOrigin();
	float r = sphere.getRadius();

	if (n < 4 || r <= 0) {
		m_Builder.setPrimitive(Mesh::PointList);
		m_Builder.clear();
		m_Builder.vertex(c.x,c.y,c.z);
		m_Builder.finish();
		return m_Builder.build();
	}

	m_Builder.clear();
	m_Builder.setPrimitive(Mesh::TriangleStrip);
	m_Builder.setAttributes(Attributes::Vertex | Attributes::Normal | Attributes::Texture);
	
	for (j=0; j<n/2; j++) {
      	theta1 = 2* j * (float)M_PI / n - (float)M_PI_2;
      	theta2 = 2* (j + 1) * (float)M_PI / n - (float)M_PI_2;
      	
      	float cost1 = cos(theta1);
      	float sint1 = sin(theta1);
      	float cost2 = cos(theta2);
      	float sint2 = sin(theta2);

      	for (i=0; i<=n; i++) {
         	theta3 = 2* i * (float)M_PI / n;
         	float cost3 = cos(theta3);
         	float sint3 = sin(theta3);

         	e.x = cost1 * cost3;
         	e.y = sint1;
         	e.z = cost1 * sint3;
         	p = c + r*e;

         	m_Builder.normal(e.x,e.y,e.z);
         	m_Builder.texCoord(i/(float)n, 2*(j+1)/(float)n);
         	m_Builder.vertex(p.x,p.y,p.z);

         	e.x = cost2 * cost3;
         	e.y = sint2;
         	e.z = cost2 * sint3;
         	p = c + r*e;

         	m_Builder.normal(e.x,e.y,e.z);
         	m_Builder.texCoord(i/(float)n, 2*j/(float)n);
         	m_Builder.vertex(p.x,p.y,p.z);
      	}
      	
      	// Create degenerate triangles
      	m_Builder.vertex(p.x,p.y,p.z);
   		m_Builder.vertex(p.x,p.y,p.z);
   }
   
   m_Builder.finish();
   return m_Builder.build();
}

}; };
