/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.mesh.h"
#include "graphics3d.meshgroup.h"
#include "graphics3d.material.h"
#include "graphics3d.shader.h"
#include "graphics3d.appearance.h"

#include "graphics3d.mesh.inl"
#include "graphics3d.meshgroup.inl"
#include "container.array.inl"
#include "container.iterator.inl"
#include "object.pointer.inl"


namespace magnum {namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool MeshGroup::remove(Mesh* mesh) {
	ASSERT(mesh != NULL);
	for (unsigned int i = 0; i < m_Meshes.getSize(); i++) {
		if (m_Meshes[i] == mesh) {
			m_Meshes.remove(i);
			return true;
		}
	}

	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
MeshGroup::Iterator MeshGroup::at(const char* name) const {
	ASSERT(name != NULL);
	for (unsigned int i = 0; i < m_Meshes.getSize(); i++) {
		if (m_Meshes[i]->getName() == name) {
			return m_Meshes.at(i);
		}
	}

	return m_Meshes.end();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void MeshGroup::render() const {
	if (m_Appearance != NULL)
		m_Appearance->select();
	Iterator miter, mend;
	mend = m_Meshes.end();
	for (miter = m_Meshes.begin(); miter != mend; ++miter) {
		(*miter)->render();
	}
}


/*--------------------------------------------------------------------------*/
/** Updates all bounding volumes of all meshes.
 */
void MeshGroup::updateBounds() {
	for (Iterator iter = m_Meshes.begin(); iter != m_Meshes.end(); ++iter)
		(*iter)->updateBounds();
}


/*--------------------------------------------------------------------------*/
/** Updates all colliders of all meshes.
 * \param force Forces recreation of colliders, even if a mesh already
 *				has a collider attached.
 */
void MeshGroup::updateColliders(bool force) {
	for (Iterator iter = m_Meshes.begin(); iter != m_Meshes.end(); ++iter) {
		Mesh* mesh = *iter;
		if (force || (mesh->getCollider() == NULL))
			mesh->updateCollider();
	}
}

}; };
