/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "math.matrix.h"
#include "graphics3d.context.h"
#include "graphics3d.perspectivecamera.h"

#include "math.matrix.inl"
#include "math.vector.inl"
#include "math.vector3.inl"
#include "system.thread.inl"
#include "geometry.linear.inl"
#include "geometry.ray.inl"
#include "graphics.rectangle.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.camera.inl"
#include "graphics3d.perspectivecamera.inl"


namespace magnum { namespace graphics3d {
using magnum::math::Matrix4;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void PerspectiveCamera::onSelect() const {
	// Calculate Projection Matrix;
	Matrix4<float> mat;
	mat[0][0] = 2*getProjectionPlane()/(m_ViewWindow.getWidth());
	mat[0][1] = 0.0f;
	mat[0][2] = (2*m_ViewWindow.getX() + m_ViewWindow.getWidth()) / m_ViewWindow.getWidth();
	mat[0][3] = 0.0f;
	mat[1][0] = 0.0f;
	mat[1][1] = 2*getProjectionPlane()/(m_ViewWindow.getHeight());
	mat[1][2] = (2*m_ViewWindow.getY() + m_ViewWindow.getHeight()) / m_ViewWindow.getHeight();
	mat[1][3] = 0.0f;
	mat[2][0] = 0.0f;
	mat[2][1] = 0.0f;
	mat[2][2] = - (getFarPlane() + getNearPlane()) / (getFarPlane() - getNearPlane());
	mat[2][3] = - 2* getFarPlane() * getNearPlane() / (getFarPlane() - getNearPlane());
	mat[3][0] = 0.0f;
	mat[3][1] = 0.0f;
	mat[3][2] = -1.0f;
	mat[3][3] = 0.0f;

	// Now set OpenGL Matrix
	int mm;
	glGetIntegerv(GL_MATRIX_MODE, &mm);
	glMatrixMode(GL_PROJECTION);
#if __FOROPENGL__ >= 13
	glLoadTransposeMatrixf(&mat[0][0]);
#else
	if (glLoadTransposeMatrixf)	
		glLoadTransposeMatrixf(&mat[0][0]);
	else
		glLoadMatrixf(&mat.transpose()[0][0]);
#endif

#if 0
	glLoadIdentity();
	glFrustum(m_ViewWindow.getX(), m_ViewWindow.getX() + m_ViewWindow.getWidth(), 
		m_ViewWindow.getY(), m_ViewWindow.getY() + m_ViewWindow.getHeight(), getNearPlane(), getFarPlane());
#endif

	glMatrixMode(mm);
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Ray<float,3> PerspectiveCamera::onCreateRay(float x,float y) const {
	Vector3<float> dir;
	dir.x = 0.5f*(x + 1.0f)*m_ViewWindow.getWidth() + m_ViewWindow.getX(); 
	dir.y = -0.5f*(y + 1.0f)*m_ViewWindow.getHeight() - m_ViewWindow.getY(); 
	dir.z = -getNearPlane();
	return Ray<float,3>(Vector<float,3>::Null, dir);
}

}; };
