/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.context.h"
#include "graphics3d.polygonmode.h"
#include "graphics3d.vertexformat.h"

#include "object.inl"
#include "base.string.inl"
#include "system.thread.inl"
#include "system.assembler.inl"
#include "graphics3d.context.inl"
#include "graphics3d.polygonmode.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.vertexformat.inl"


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
PolygonMode::PolygonMode() {
	m_FrontMode = m_BackMode = Filled;
	m_Offset = m_OffsetFactor = 0.0f;
	m_Culling = CullBack;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
PolygonMode::~PolygonMode() {
	// Remove PolygonMode from all Contexts!
	Context* context = getContext();
	if (context != NULL) {
		if (context->m_PolygonMode == this)
			context->m_PolygonMode = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void PolygonMode::compileSelect(Assembler& as) const {
	Context* context = getContext();
	int offset = (int)((byte*)&context->m_PolygonMode - (byte*)context);
	as.asmLoad(Assembler::T0, (int)(intptr_t)this);
	as.asmStore(Assembler::R0, offset, Assembler::T0);
	if (onCompileSelect(as) == false)
		as.asmCall((void (PolygonMode::*)() const)&PolygonMode::onSelect, this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void PolygonMode::compileDeselect(Assembler& as) const {
	Context* context = getContext();
	int offset = (int)((byte*)&context->m_PolygonMode - (byte*)context);
	as.asmLoad(Assembler::T0, NULL);
	as.asmStore(Assembler::R0, offset, Assembler::T0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void PolygonMode::onSelect() const {
	if (m_FrontMode == m_BackMode) {
		glPolygonMode(GL_FRONT_AND_BACK, m_FrontMode);
	} else {
		glPolygonMode(GL_FRONT, m_FrontMode);
		glPolygonMode(GL_BACK, m_BackMode);
	}
	glPolygonOffset(m_OffsetFactor, m_Offset);

	if (m_Culling == CullFront) {
		glEnable(GL_CULL_FACE);
		glCullFace(GL_FRONT);
	} else if (m_Culling == CullBack) {
		glEnable(GL_CULL_FACE);
		glCullFace(GL_BACK);
	} else {
		glDisable(GL_CULL_FACE);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool PolygonMode::onCompileSelect(Assembler& as) const {
	if (m_FrontMode == m_BackMode) {
		as.asmCall(glPolygonMode, (GLenum)GL_FRONT_AND_BACK, (GLenum)m_FrontMode);
	} else {
		as.asmCall(glPolygonMode, (GLenum)GL_FRONT, (GLenum)m_FrontMode);
		as.asmCall(glPolygonMode, (GLenum)GL_BACK, (GLenum)m_BackMode);
	}
	if (m_Culling == CullFront) {
		as.asmCall(glEnable, (GLenum)GL_CULL_FACE);
		as.asmCall(glCullFace, (GLenum)GL_FRONT);
	} else if (m_Culling == CullBack) {
		as.asmCall(glEnable, (GLenum)GL_CULL_FACE);
		as.asmCall(glCullFace, (GLenum)GL_BACK);
	} else {
		as.asmCall(glDisable, (GLenum)GL_CULL_FACE);
	}
	as.asmCall(glPolygonOffset, m_OffsetFactor, m_Offset);
	return true;
}


}; };
