/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.skeleton.h"

#include "object.inl"
#include "base.string.inl"
#include "container.iterator.inl"
#include "container.allocator.inl"
#include "container.list.inl"
#include "graphics3d.skeleton.inl"

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bone::Bone() {
	m_Name[0] = 0;
	m_Skeleton = NULL;
	m_Parent = NULL;
	m_TorsoWeight = 0.0f;
	m_Distance = 0.0f;
	m_Index = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bone* Skeleton::insert(const char* name, Bone* parent) {
	ASSERT(name != NULL);
	ASSERT(parent != NULL);
	Bone* bone = m_Bones.append();
	bone->setName(name);
	bone->m_Parent = parent;
	return bone;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bone* Skeleton::insert(const char* name) {
	Bone* bone = m_Bones.append();
	bone->setName(name);
	return bone;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Skeleton::remove(Bone* bone) {
	removeChilds(bone);
	for (Iterator iter = m_Bones.begin(); iter != m_Bones.end(); ++iter) {
		if (iter == bone) {
			m_Bones.remove(iter);
			return;
		}
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Skeleton::removeChilds(Bone* bone) {
	for (Iterator iter = m_Bones.begin(); iter != m_Bones.end(); ++iter) {
		if (iter->m_Parent == bone) {
			removeChilds(iter);
			m_Bones.remove(iter);
			iter = m_Bones.begin();
		}
	}
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Skeleton::clear() {
	m_Bones.clear();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Skeleton::Iterator Skeleton::at(const char* n) const {
	for (Iterator iter = m_Bones.begin(); iter != m_Bones.end(); ++iter) {
		if (!strncmp(iter->m_Name, n, sizeof(iter->m_Name)))
			return iter;
	}
	return m_Bones.end();
}

}; };

