/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.context.h"
#include "graphics3d.stencilmode.h"
#include "graphics3d.vertexformat.h"

#include "object.inl"
#include "base.string.inl"
#include "system.thread.inl"
#include "system.assembler.inl"
#include "graphics3d.context.inl"
#include "graphics3d.stencilmode.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.vertexformat.inl"


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
StencilMode::StencilMode() {
	m_Fail = m_ZFail = m_Pass = Keep;
	m_Function = Disable;
	m_Reference = 0;
	m_ReadMask = 0xffffffff;
	m_WriteMask = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
StencilMode::~StencilMode() {
	// Remove StencilMode from all Contexts!
	Context* context = getContext();
	if (context != NULL) {
		if (context->m_StencilMode == this)
			context->m_StencilMode = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void StencilMode::compileSelect(Assembler& as) const {
	Context* context = getContext();
	int offset = (int)((byte*)&context->m_StencilMode - (byte*)context);
	as.asmLoad(Assembler::T0, (int)(intptr_t)this);
	as.asmStore(Assembler::R0, offset, Assembler::T0);
	if (onCompileSelect(as) == false)
		as.asmCall((void (StencilMode::*)() const)&StencilMode::onSelect, this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void StencilMode::compileDeselect(Assembler& as) const {
	Context* context = getContext();
	int offset = (int)((byte*)&context->m_StencilMode - (byte*)context);
	as.asmLoad(Assembler::T0, NULL);
	as.asmStore(Assembler::R0, offset, Assembler::T0);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void StencilMode::onSelect() const {
	// Stencil Buffer related settings
	if (m_Function == Disable) {
		glDisable(GL_STENCIL_TEST);
	} else {
		glEnable(GL_STENCIL_TEST);
		glStencilFunc(m_Function, m_Reference, m_ReadMask);
	}
	glStencilMask(m_WriteMask);
	glStencilOp(m_Fail, m_ZFail, m_Pass);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool StencilMode::onCompileSelect(Assembler& as) const {
	if (m_Function == Disable) {
		as.asmCall(glDisable,(GLenum)GL_STENCIL_TEST);
	} else {
		as.asmCall(glEnable,(GLenum)GL_STENCIL_TEST);
		as.asmCall(glStencilFunc,(GLenum)m_Function, (GLint)m_Reference, m_ReadMask);
	}
	as.asmCall(glStencilMask,m_WriteMask);
	as.asmCall(glStencilOp,(GLenum)m_Fail,(GLenum) m_ZFail,(GLenum) m_Pass);
	return true;
}


}; };
