/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.stencilshadow.h"
#include "graphics3d.indexbuffer.h"
#include "graphics3d.vertexbuffer.h"
#include "graphics3d.vertexformat.h"
#include "graphics3d.pixelprogram.h"
#include "graphics3d.vertexprogram.h"
#include "graphics3d.material.h"
#include "graphics3d.shader.h"
#include "graphics3d.blendmode.h"
#include "graphics3d.stencilmode.h"
#include "graphics3d.appearance.h"
#include "graphics3d.context.h"
#include "graphics3d.indexmesh.h"

#include "object.inl"
#include "object.pointer.inl"
#include "base.string.inl"
#include "math.vector.inl"
#include "math.vector3.inl"
#include "graphics3d.material.inl"
#include "graphics3d.shader.inl"
#include "graphics3d.appearance.inl"
#include "graphics3d.technique.inl"
#include "graphics3d.projectionvolume.inl"
#include "graphics3d.stencilshadow.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.mesh.inl"
#include "graphics3d.context.inl"
#include "graphics3d.stencilmode.inl"
#include "graphics3d.depthmode.inl"
#include "graphics3d.blendmode.inl"
#include "graphics3d.polygonmode.inl"
#include "graphics3d.indexbuffer.inl"
#include "graphics3d.vertexbuffer.inl"
#include "graphics3d.vertexformat.inl"

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
StencilShadow::ShadowShader::ShadowShader() {
	publishAttributes(Attributes::Vertex);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
StencilShadow::StencilShadow() {
	m_DepthMode.setFunction(DepthMode::LessEqual);
	m_DepthMode.setWrite(false);
	m_StencilMode.setWriteMask(~0);
	m_StencilMode.setFunction(StencilMode::Always, 0, ~0);
	m_StencilMode.setAction(StencilMode::Keep, StencilMode::Keep, StencilMode::IncrementWrap);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void StencilShadow::onRender() {
	if (m_Volume.getIndexRange() == 0)
		return;

	Context* context = getContext();

	m_DepthMode.select();
	m_StencilMode.select();
	context->setColourMask(false);
	m_Shader.select();
	m_Volume.getIndexBuffer()->select();
	m_Volume.getVertexBuffer()->selectVertices();

	// Render back faces
	context->setCulling(Context::CullFront);
	context->setPolygonOffset(0.0f, 10.0f);
	context->renderIndices(Mesh::TriangleList, m_Volume.getIndexRange());

	// Render front faces
	context->setStencilAction(StencilMode::Keep, StencilMode::Keep, StencilMode::DecrementWrap);
	context->setCulling(Context::CullBack);
	context->setPolygonOffset(0.0f, -10.0f);
	context->renderIndices(Mesh::TriangleList, m_Volume.getIndexRange());

	context->setPolygonOffset(0.0f, 0.0f);
	context->setColourMask(true);
	DepthMode::selectNone();
	StencilMode::selectNone();
}

}; };
