/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "system.thread.h"
#include "graphics3d.texenvbumpshader.h"
#include "graphics3d.context.h"
#include "graphics3d.vertexformat.h"
#include "graphics3d.vertexprogram.h"
#include "graphics3d.pixelprogram.h"

#include "object.inl"
#include "object.pointer.inl"
#include "base.string.inl"
#include "system.thread.inl"
#include "system.assembler.inl"
#include "container.array.inl"
#include "graphics3d.shader.inl"
#include "graphics3d.context.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.vertexformat.inl"
#include "graphics3d.constant.inl"
#include "graphics3d.constant.inl"


namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
TextureEnvironmentBumpShader::TextureEnvironmentBumpShader(Features f) : FixedShader(f) {
	publishAttributes(
		Attributes::Vertex | 
		Attributes::Normal | 
		Attributes::Texture0 | 
		Attributes::Texture1);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void TextureEnvironmentBumpShader::onSelect() const {
	FixedShader::onSelect();
	glActiveTextureARB(GL_TEXTURE0);
	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_TEXTURE);
	glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_MODULATE);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_PREVIOUS);
	glActiveTextureARB(GL_TEXTURE1);
	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_TEXTURE);
	glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_BUMP_ENVMAP_ATI);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_TEXTURE);
	glTexEnvi(GL_TEXTURE_ENV, GL_BUMP_TARGET_ATI, GL_TEXTURE2);
	glActiveTextureARB(GL_TEXTURE2);
	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_TEXTURE);
	glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_ADD);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_PREVIOUS);
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void TextureEnvironmentBumpShader::onDeselect() const {
	glActiveTextureARB(GL_TEXTURE0);
	glDisable(GL_TEXTURE_2D);
	glActiveTextureARB(GL_TEXTURE1);
	glDisable(GL_TEXTURE_2D);
	glActiveTextureARB(GL_TEXTURE2);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	FixedShader::onDeselect();
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void TextureEnvironmentBumpShader::onCompileSelect(Assembler& as) const {
	FixedShader::onCompileSelect(as);
	as.asmCall(glActiveTextureARB,(GLenum)GL_TEXTURE0);
	as.asmCall(glEnable,(GLenum)GL_TEXTURE_2D);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_TEXTURE_ENV_MODE, (GLint)GL_COMBINE);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_SOURCE0_RGB, (GLint)GL_TEXTURE);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_COMBINE_RGB, (GLint)GL_MODULATE);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_SOURCE1_RGB, (GLint)GL_PREVIOUS);

	as.asmCall(glActiveTextureARB,(GLenum)GL_TEXTURE1);
	as.asmCall(glEnable,(GLenum)GL_TEXTURE_2D);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_TEXTURE_ENV_MODE, (GLint)GL_COMBINE);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_SOURCE0_RGB, (GLint)GL_TEXTURE);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_COMBINE_RGB, (GLint)GL_BUMP_ENVMAP_ATI);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_SOURCE1_RGB, (GLint)GL_TEXTURE);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_BUMP_TARGET_ATI, (GLint)GL_TEXTURE2);

	as.asmCall(glActiveTextureARB,(GLenum)GL_TEXTURE2);
	as.asmCall(glEnable,(GLenum)GL_TEXTURE_2D);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_TEXTURE_ENV_MODE, (GLint)GL_COMBINE);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_SOURCE0_RGB, (GLint)GL_TEXTURE);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_COMBINE_RGB, (GLint)GL_ADD);
	as.asmCall(glTexEnvi,(GLenum)GL_TEXTURE_ENV, (GLenum)GL_SOURCE1_RGB, (GLint)GL_PREVIOUS);

	as.asmCall(glTexGeni,(GLenum)GL_S, (GLenum)GL_TEXTURE_GEN_MODE, (GLint)GL_SPHERE_MAP);
	as.asmCall(glTexGeni,(GLenum)GL_T, (GLenum)GL_TEXTURE_GEN_MODE, (GLint)GL_SPHERE_MAP);
	as.asmCall(glEnable,(GLenum)GL_TEXTURE_GEN_S);
	as.asmCall(glEnable,(GLenum)GL_TEXTURE_GEN_T);
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void TextureEnvironmentBumpShader::onCompileDeselect(Assembler& as) const {
	as.asmCall(glActiveTextureARB,(GLenum)GL_TEXTURE0);
	as.asmCall(glDisable,(GLenum)GL_TEXTURE_2D);
	as.asmCall(glActiveTextureARB,(GLenum)GL_TEXTURE1);
	as.asmCall(glDisable,(GLenum)GL_TEXTURE_2D);
	as.asmCall(glActiveTextureARB,(GLenum)GL_TEXTURE2);
	as.asmCall(glDisable,(GLenum)GL_TEXTURE_2D);
	as.asmCall(glDisable,(GLenum)GL_TEXTURE_GEN_S);
	as.asmCall(glDisable,(GLenum)GL_TEXTURE_GEN_T);
	FixedShader::onCompileDeselect(as);
}
}; };
