/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "io.file.h"
#include "io.sysfile.h"
#include "io.translator.h"
#include "io.exception.h"
#include "graphics3d.texturefactory.h"
#include "graphics3d.texture.h"
#include "graphics3d.texture2d.h"
#include "graphics3d.context.h"
#include "graphics3d.vertexformat.h"

#include "object.inl"
#include "object.exception.inl"
#include "base.string.inl"
#include "container.iterator.inl"
#include "container.chain.inl"
#include "io.translator.inl"
#include "graphics.translator.inl"
#include "graphics3d.texturefactory.inl"
#include "graphics3d.texture.inl"
#include "graphics3d.texture2d.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.context.inl"


namespace magnum { namespace graphics3d {
using magnum::graphics::Image;
using magnum::io::File;
using magnum::io::SystemFile;
using magnum::io::Translator;


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
Texture* TextureFactory::create(const char* fname) {
	Texture* texture;
	Translator<Image>* translator;
	File* file;
	Image* img;

	// Open file
	file = m_FileFactory->open(fname, File::ReadOnly);
	if (file == NULL)
		throw magnum::io::FileNotFoundException();

	// Get translator
	translator = m_TranslatorRoster.identify(file);
	if (translator == NULL) {
		file->release();
		throw magnum::io::TranslatorFormatException();
	}

	// Decode
	img = translator->decode(file->getInputStream());

	// close file again
	file->release();

	// Create Texture
	texture = Texture2D::create(img, Texture::GenerateAll);

	// Release Image
	img->release();

	//
	// TODO: Set Default filters
	//
	texture->setFilter(Texture::LinearMipmapLinear, Texture::Linear);


	return texture;
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void TextureFactory::release(Texture* tex) {
	tex->release();
}

}; };
