/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.translator.h"
#include "graphics3d.texturemanager.h"
#include "graphics3d.shadermanager.h"
#include "graphics3d.texture.h"
#include "graphics3d.context.h"

#include "object.inl"
#include "object.property.inl"
#include "object.pointer.inl"
#include "base.string.inl"
#include "io.translator.inl"
#include "io.sysfile.inl"
#include "io.sysfilefactory.inl"
#include "container.iterator.inl"
#include "container.chain.inl"
#include "container.array.inl"
#include "container.cache.inl"
#include "container.map.inl"
#include "container.allocator.inl"
#include "graphics3d.translator.inl"
#include "graphics3d.texturefactory.inl"
#include "graphics3d.texturemanager.inl"
#include "graphics3d.shadermanager.inl"
#include "graphics3d.texture.inl"
#include "graphics3d.context.inl"
#include "graphics3d.resource.inl"


namespace magnum { namespace graphics3d {
using magnum::object::PropertyDescriptor;
using magnum::object::Property;

static PropertyDescriptor s_Properties[] =
	{{ Property::Pointer, "texturemanager", "", NULL},
	 { Property::Pointer, "shadermanager", "", NULL},
	 { Property::End, "", "", 0}};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
PropertyList* ModelTranslator::onCreateProperties() const {
	PropertyList* list = new PropertyList(s_Properties);
	PropertyList::Iterator iter;

	iter = list->at("texturemanager");
	iter->setPointer(m_TextureManager);
	iter = list->at("shadermanager");
	iter->setPointer(m_ShaderManager);

	return list;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ModelTranslator::onSetProperties(const PropertyList* list) {
	PropertyList::Iterator iter, end;
	end = list->end();

	iter = list->at("texturemanager");
	if (iter != end) {
		TextureManager* tm;
		tm = (TextureManager *)iter->getPointer();
		if (tm != NULL)
			attachTextureManager(tm);
		else
			detachTextureManager();
	}

	iter = list->at("shadermanager");
	if (iter != end) {
		ShaderManager* tm;
		tm = (ShaderManager *)iter->getPointer();
		if (tm != NULL)
			attachShaderManager(tm);
		else
			detachShaderManager();
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Texture* ModelTranslator::createTexture(const char* name) {

	if (m_TextureManager == NULL) {
		m_TextureManager = new TextureManager();
		m_TextureManager->release();
	}

	Texture* tex = NULL;
	try {
		tex = m_TextureManager->get(String(name));
	} catch(...) {
		return NULL;
	}
	if (tex == NULL)
		return NULL;

	tex->refer();
	return tex;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ModelTranslatorRoster::attachTextureManager(TextureManager* m) {
	ASSERT(m != NULL);
	m_TextureManager = m;

	// Attach Texture Manager to all translators
	PropertyList list(s_Properties);
	list["texturemanager"].setPointer(m_TextureManager);
	list["shadermanager"].setPointer(m_ShaderManager);

	Iterator iter = begin();
	Iterator ed = end();

	while(iter != ed) {
		iter->setProperties(&list);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ModelTranslatorRoster::detachTextureManager() {
	m_TextureManager = NULL;

	// Detach Texture Manager from all translators
}

}; };
