/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.layer.h"
#include "graphics3d.viewport.h"
#include "graphics3d.framebuffer.h"

#include "object.pointer.inl"
#include "container.iterator.inl"
#include "graphics.rectangle.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.context.inl"
#include "graphics3d.layer.inl"
#include "graphics3d.layergroup.inl"
#include "graphics3d.viewport.inl"
#include "graphics3d.framebuffer.inl"

namespace magnum { namespace graphics3d {

/*--------------------------------------------------------------------------*/
/**
 */
void Viewport::attachFrameBuffer(FrameBuffer* fb) {
	ASSERT(fb != NULL);
	m_FrameBuffer = fb;
	m_Window.setX(0.0f);
	m_Window.setY(0.0f);
	m_Window.setWidth((float)fb->getWidth());
	m_Window.setHeight((float)fb->getHeight());
}


/*--------------------------------------------------------------------------*/
/**
 */
void Viewport::onRender() {
	Context* context = getContext();
	
	// Select framebuffer (if any is attached)
	if (m_FrameBuffer != NULL)
		m_FrameBuffer->select();
	Rectangle<float> oldvp = context->getViewport();
	
	// Set window
	context->setViewport(m_Window);
	
	// Render layers
	LayerGroup::onRender();
	
	// Disable clipping / restore viewport
	context->setViewport(oldvp);
}

}; };
