/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.weightbuffer.h"

#include "object.inl"
#include "base.string.inl"
#include "math.vector.inl"
#include "math.vector3.inl"
#include "math.matrix.inl"
#include "container.array.inl"
#include "graphics3d.weightbuffer.inl"

namespace magnum { namespace graphics3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
WeightBuffer::WeightBuffer() {
	m_Size = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
WeightBuffer::~WeightBuffer() {
	m_Size = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void WeightBuffer::clear() {
	m_Buffer.clear();
	m_Size = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void WeightBuffer::finish() {
	m_Buffer.shrink();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void WeightBuffer::insert(unsigned int count,const Vector3<float>& n,const Weight* w) {
	ASSERT(count > 0);
	unsigned int dst = m_Buffer.append(4+count*sizeof(Weight)/sizeof(int32));
	m_Buffer[dst] = count;
	*reinterpret_cast<Vector3<float>*>(&m_Buffer[dst+1]) = n;
	memcpy(&m_Buffer[dst+4], w, count*sizeof(Weight));
	m_Size++;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void WeightBuffer::transform(Vector3<float>* dst,const Matrix4<float>* mtx) {
	const int32* src = &m_Buffer[0];
	for (unsigned int i = 0; i < m_Size; ++i) {
		// Get number of weights
		unsigned int numweights = *src++;

		// Get original normal
		Vector3<float> normal = *reinterpret_cast<const Vector3<float>*>(src);
		src += 3;
		
		// Create new normal and final position
		Vector3<float> tnormal(0.0f, 0.0f, 0.0f);
		Vector3<float> tvec(0.0f,0.0f,0.0f);
		while(numweights > 0) {
			const Weight* w = reinterpret_cast<const Weight*>(src);
			tvec += w->weight*(mtx[w->bone] * w->position);
			tnormal.x += w->weight*(mtx[w->bone][0][0]*normal.x + mtx[w->bone][0][1]*normal.y + mtx[w->bone][0][2]*normal.z);
			tnormal.y += w->weight*(mtx[w->bone][1][0]*normal.x + mtx[w->bone][1][1]*normal.y + mtx[w->bone][1][2]*normal.z);
			tnormal.z += w->weight*(mtx[w->bone][2][0]*normal.x + mtx[w->bone][2][1]*normal.y + mtx[w->bone][2][2]*normal.z);
			src += sizeof(Weight) / sizeof(uint32);
			numweights--;
		}
		*dst++ = tvec;
		*dst++ = tnormal;
	}

}


}; };

