/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "gui.bitmap.h"
#include "graphics.filter.byteorder.h"

#include "container.chain.inl"
#include "gui.bitmap.inl"
#include "graphics.point.inl"
#include "graphics.dimension.inl"
#include "graphics.rectangle.inl"
#include "graphics.image.inl"


namespace magnum { namespace gui {
using magnum::graphics::FilterByteorder;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bitmap* Bitmap::create(Type type, const Dimension<int>& size) {
	Bitmap* bmp = new Bitmap(type, size.getWidth(), size.getHeight());
	bmp->createBitmap();
	return bmp;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bitmap* Bitmap::create(Type type, int w, int h) {
	Bitmap* bmp = new Bitmap(type, w, h);
	bmp->createBitmap();
	return bmp;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Bitmap* Bitmap::create(Image* img) {
	Bitmap* bmp;
	FilterByteorder filter;

	bmp = new Bitmap(img->getType(), img->getWidth(), img->getHeight());
	if (bmp == NULL)
		return NULL;

	if (bmp->adjustBitmap() == false) {
		delete bmp;
		return NULL;
	}
	if (bmp->createBitmap() == false) {
		delete bmp;
		return NULL;
	}

	if (bmp->getType() == img->getType()) {
		bmp->copyPixels(img);
	} else if ((bmp->getType() == RGB8) && (img->getType() == BGR8)) {
		filter.filter(bmp, img);
	} else if ((bmp->getType() == BGR8) && (img->getType() == RGB8)) {
		filter.filter(bmp, img);
	} else if ((bmp->getType() == RGBA8) && (img->getType() == BGRA8)) {
		filter.filter(bmp, img);
	} else if ((bmp->getType() == BGRA8) && (img->getType() == RGBA8)) {
		filter.filter(bmp, img);
	} else {
		bmp->release();
		return NULL;
	}

	if (bmp->isIndexed() && img->isIndexed())
		bmp->copyPalette(img);

	return bmp;
}




}; };
