/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "gui.context.h"

#ifndef __FORX11__
#	error "This file is only for X11"
#endif


namespace magnum { namespace gui { 

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Context::Context() {
	m_DisplayHandle = NULL;
	m_WindowHandle = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Context::~Context() {
	m_DisplayHandle = NULL;
	m_WindowHandle = NULL;
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Context::setPalette(const Palette<byte>& ) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Context::getPalette(Palette<byte>& ) const {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Context::realizePalette(void) {
	XStoreColors(m_Display, m_PaletteHandle, m_PaletteColours,256);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Context::unrealizePalette(void) {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Context::setFillColour(const Colour<byte>& col) {
	m_FillColour = col;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Context::setLineColour(const Colour<byte>& col) {
	m_LineColour = col;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Context::setLineWidth(int w) {
	m_LineWidth = w;
	XSetLineAttributes(m_Window->getDisplayHandle(), m_Window->getDeviceHandle(), m_LineWidth, LineSolid, CapButt, JoinMiter); 
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Context::fillRectangle(const Rectangle<int>& rect) {
	Rectangle<int> lrect = convertToWindow(rect);
	if (m_FillColour == m_LineColour) {
		XSetForeground(m_Window->getDisplayHandle(), m_Window->getDeviceHandle(), colourToX11(m_FillColour));
		XFillRectangle(m_Window->getDisplayHandle(), m_Window->getWindowHandle(), m_Window->getDeviceHandle(), lrect.getX(), lrect.getY(), lrect.getWidth(), lrect.getHeight());
	} else {
		XSetForeground(m_Window->getDisplayHandle(), m_Window->getDeviceHandle(), colourToX11(m_FillColour));
		XFillRectangle(m_Window->getDisplayHandle(), m_Window->getWindowHandle(), m_Window->getDeviceHandle(), lrect.getX()+1, lrect.getY()+1, lrect.getWidth()-2, lrect.getHeight()-2);
		XSetForeground(m_Window->getDisplayHandle(), m_Window->getDeviceHandle(), colourToX11(m_LineColour));
		XDrawRectangle(m_Window->getDisplayHandle(), m_Window->getWindowHandle(), m_Window->getDeviceHandle(), lrect.getX(), lrect.getY(), lrect.getWidth()-1, lrect.getHeight()-1);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Context::drawRectangle(const Rectangle<int>& rect) {
	Rectangle<int> lrect = convertToWindow(rect);
	XSetForeground(m_Window->getDisplayHandle(), m_Window->getDeviceHandle(), colourToX11(m_LineColour));
	XDrawRectangle(m_Window->getDisplayHandle(), m_Window->getWindowHandle(), m_Window->getDeviceHandle(), lrect.getX(), lrect.getY(), lrect.getWidth()-1, lrect.getHeight()-1);
}


	XRectangle xrect;
	xrect.x = clipRect.getX();
	xrect.y = clipRect.getY();
	xrect.width = clipRect.getWidth();
	xrect.height = clipRect.getHeight();
	XSetClipRectangles(m_Window->getDisplayHandle(), m_Window->getDeviceHandle(), 0, 0, &xrect, 1, Unsorted);


}; };
