/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "gui3d.button.h"
#include "gui3d.context.h"
#include "gui3d.fontmap.h"
#include "gui3d.root.h"

#include "object.pointer.inl"
#include "graphics.dimension.inl"
#include "freetype.font.inl"
#include "freetype.descriptor.inl"
#include "gui3d.button.inl"
#include "gui3d.control.inl"
#include "gui3d.context.inl"
#include "gui3d.fontmap.inl"
#include "gui3d.view.inl"


namespace magnum { namespace gui3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Button::Button(const Rectangle<float>& r,const Message& msg) : Control(r,msg) {
	init();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Button::Button(const Rectangle<float>& r,const Message& msg,const char* str) : Control(r,msg) {
	m_Text = str;
	init();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Button::init() {
	m_State = NormalState;
	m_Font = NULL;

	m_TextColour = Colour<float>(0.9f, 0.9f, 0.9f);
	m_BackColour = Colour<float>(0.2f, 0.2f, 0.2f);
	m_BorderColour = Colour<float>(0.3f, 0.3f, 0.3f);

	m_HoverTextColour = Colour<float>(0.9f, 0.9f, 0.9f);
	m_HoverBackColour = Colour<float>(0.3f, 0.3f, 0.3f);
	m_HoverBorderColour = Colour<float>(0.4f, 0.4f, 0.4f);

	m_PressedTextColour = Colour<float>(1.0f, 1.0f, 1.0f);
	m_PressedBackColour = Colour<float>(0.8f, 0.6f, 0.2f);
	m_PressedBorderColour = Colour<float>(1.0f, 0.8f, 0.2f);

	m_DisabledTextColour = Colour<float>(0.4f, 0.4f, 0.4f);
	m_DisabledBackColour = Colour<float>(0.2f, 0.2f, 0.2f);
	m_DisabledBorderColour = Colour<float>(0.3f, 0.3f, 0.3f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Button::onPaint() {
	Context* context = getContext();
	if (m_FontMap == NULL) {
		if (m_Font == NULL)
			return true;
		m_FontMap = context->createFont(m_Font);
		context->selectFont(m_FontMap);
		m_TextExtents = context->getStringExtents(m_Text);
	}

	if (m_State == NormalState) {
		context->setColour(m_BackColour);
		context->fillRectangle(getBounds());
		context->setColour(m_BorderColour);
		context->drawRectangle(getBounds());
		
		Point<float> p;
		p.setX((getBounds().getWidth() - m_TextExtents.getWidth()) * 0.5f);
		p.setY(getBounds().getHeight()*0.5f + m_Font->getSize().getHeight()*0.33f);
		context->setColour(m_TextColour);
		context->selectFont(m_FontMap);
		context->drawString(p, m_Text);
	} else if (m_State == PressedState) {
		context->setColour(m_PressedBackColour);
		context->fillRectangle(getBounds());
		context->setColour(m_PressedBorderColour);
		context->drawRectangle(getBounds());
		
		Point<float> p;
		p.setX((getBounds().getWidth() - m_TextExtents.getWidth()) * 0.5f);
		p.setY(getBounds().getHeight()*0.5f + m_Font->getSize().getHeight()*0.33f);
		context->setColour(m_PressedTextColour);
		context->selectFont(m_FontMap);
		context->drawString(p, m_Text);
	} else if (m_State == HoveringState) {
		context->setColour(m_HoverBackColour);
		context->fillRectangle(getBounds());
		context->setColour(m_HoverBorderColour);
		context->drawRectangle(getBounds());
		
		Point<float> p;
		p.setX((getBounds().getWidth() - m_TextExtents.getWidth()) * 0.5f);
		p.setY(getBounds().getHeight()*0.5f + m_Font->getSize().getHeight()*0.33f);
		context->setColour(m_HoverTextColour);
		context->selectFont(m_FontMap);
		context->drawString(p, m_Text);
	} else if (m_State == DisabledState) {
		context->setColour(m_DisabledBackColour);
		context->fillRectangle(getBounds());
		context->setColour(m_DisabledBorderColour);
		context->drawRectangle(getBounds());
		
		Point<float> p;
		p.setX((getBounds().getWidth() - m_TextExtents.getWidth()) * 0.5f);
		p.setY(getBounds().getHeight()*0.5f + m_Font->getSize().getHeight()*0.33f);
		context->setColour(m_DisabledTextColour);
		context->selectFont(m_FontMap);
		context->drawString(p, m_Text);
	}
	
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Button::onEnable() {
	m_State = NormalState;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Button::onDisable() {
	m_State = DisabledState;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Button::onMouseMove(const Point<float>& pos, Mouse::Button button) {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Button::onMouseEnter(const Point<float>&, Mouse::Button buttons) {
	m_State = HoveringState;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Button::onMouseButton(const Point<float>&, Mouse::Button buttons) {
	if (buttons & Mouse::LeftDown) {
		if (m_State != PressedState) {
			m_State = PressedState;
			onPush();
		}
		return true;
	}
	if (buttons & Mouse::LeftUp) {
		if (m_State == PressedState) {
			onRelease();
			onAction();
		}
		m_State = HoveringState;
		return true;
	}
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Button::onMouseLeave() {
	if (m_State == PressedState)
		onRelease();
	m_State = NormalState;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Button::onPush() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Button::onRelease() {
	return true;
}

}; };
