/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "gui3d.root.h"
#include "gui3d.view.h"

#include "gui3d.root.inl"
#include "gui3d.view.inl"

#ifndef __FORWINDOWS__
#error "This file is only for Windows"
#else
#include <commctrl.h>
#endif


namespace magnum { namespace gui3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
LRESULT CALLBACK Root::inputWindowProc(HWND hwnd, UINT Message,WPARAM wParam, LPARAM lParam) {
	Root* root = (Root *)(intptr_t)GetWindowLong(hwnd,GWL_USERDATA);
	RECT rect;
	int buttons;
	int modifiers;

	switch(Message) {
		case WM_KEYDOWN:
PRINT("Root::windowProc: WM_KEYDOWN");
			if ((lParam & 0x40000000) == 0) {
				if (root->m_Focus == NULL)
					root->onKeyDown((Keyboard::Key)wParam);
				else
					root->m_Focus->onKeyDown((Keyboard::Key)wParam);
			}
			return 0;

		case WM_KEYUP:
PRINT("Root::windowProc: WM_KEYUP");
			if (root->m_Focus == NULL)
				root->onKeyUp((Keyboard::Key)wParam);
			else
				root->m_Focus->onKeyUp((Keyboard::Key)wParam);
			return 0;

		case WM_CHAR:
PRINT1("Root::windowProc: WM_CHAR: %c",wParam);
			modifiers = 0;
			if (GetKeyState(VK_LSHIFT))
				modifiers |= Character::ShiftLeft;
			if (GetKeyState(VK_RSHIFT))
				modifiers |= Character::ShiftRight;
			if (GetKeyState(VK_LCONTROL))
				modifiers |= Character::ControlLeft;
			if (GetKeyState(VK_RCONTROL))
				modifiers |= Character::ControlRight;
			if (GetKeyState(VK_LMENU))
				modifiers |= Character::AltLeft;
			if (GetKeyState(VK_RMENU))
				modifiers |= Character::AltRight;
			if (GetKeyState(VK_APPS))
				modifiers |= Character::Meta;
			if (GetKeyState(VK_LWIN))
				modifiers |= Character::MetaLeft;
			if (GetKeyState(VK_RWIN))
				modifiers |= Character::MetaRight;
			if (root->m_Focus == NULL)
				root->onChar((Character::Code)wParam, (Character::Modifier)modifiers);
			else
				root->m_Focus->onChar((Character::Code)wParam, (Character::Modifier)modifiers);
			return 0;

		case WM_MOUSELEAVE:
PRINT("Root::windowProc: WM_MOUSELEAVE");
			root->m_MouseTracking = false;
			root->onMouseLeave();
			return 0;

		case WM_MOUSEMOVE:
PRINT2("Root::windowProc: WM_MOUSEMOVE: (%d,%d)",LOWORD(lParam),HIWORD(lParam));
			buttons = 
				((wParam & MK_LBUTTON) ? Mouse::LeftDown : Mouse::LeftUp) |
				((wParam & MK_MBUTTON) ? Mouse::MiddleDown : Mouse::MiddleUp) |
				((wParam & MK_RBUTTON) ? Mouse::RightDown : Mouse::RightUp) |
				((wParam & MK_SHIFT) ? Mouse::Shift : 0) |
				((wParam & MK_CONTROL) ? Mouse::Control : 0);
			if (!root->m_MouseTracking) {
				PRINT("Root::windowProc: WM_MOUSEMOVE -> onMouseEnter");
				root->m_MouseTracking = true;
				root->onMouseEnter(Point<float>((float)LOWORD(lParam),(float)HIWORD(lParam)), (Mouse::Button)buttons);
				TRACKMOUSEEVENT tme;
				tme.cbSize = sizeof(tme);
				tme.dwFlags = TME_LEAVE;
				tme.dwHoverTime = 0;
				tme.hwndTrack = root->m_WindowHandle;
				_TrackMouseEvent(&tme);
			}
			root->onMouseMove(Point<float>((float)LOWORD(lParam),(float)HIWORD(lParam)),(Mouse::Button)buttons);
			return 0;

		case WM_MBUTTONDOWN:
PRINT("Root::windowProc: WM_MBUTTONDOWN");
			root->onMouseButton(Point<float>((float)LOWORD(lParam),(float)HIWORD(lParam)),Mouse::MiddleDown);
			return 0;

		case WM_MBUTTONUP:
PRINT("Root::windowProc: WM_MBUTTONUP");
			root->onMouseButton(Point<float>((float)LOWORD(lParam),(float)HIWORD(lParam)),Mouse::MiddleUp);
			return 0;

		case WM_LBUTTONDOWN:
PRINT("Root::windowProc: WM_LBUTTONDOWN");
			root->onMouseButton(Point<float>((float)LOWORD(lParam),(float)HIWORD(lParam)),Mouse::LeftDown);
			return 0;

		case WM_LBUTTONUP:
PRINT("Root::windowProc: WM_LBUTTONUP");
			root->onMouseButton(Point<float>((float)LOWORD(lParam),(float)HIWORD(lParam)),Mouse::LeftUp);
			return 0;

		case WM_RBUTTONDOWN:
PRINT("Root::windowProc: WM_RBUTTONDOWN");
			root->onMouseButton(Point<float>((float)LOWORD(lParam),(float)HIWORD(lParam)),Mouse::RightDown);
			return 0;

		case WM_RBUTTONUP:
PRINT("Root::windowProc: WM_RBUTTONUP");
			root->onMouseButton(Point<float>((float)LOWORD(lParam),(float)HIWORD(lParam)),Mouse::RightUp);
			return 0;
			
		case WM_MOUSEWHEEL:
PRINT("Root::windowProc: WM_MOUSEWHEEL");
			buttons = 
				((wParam & MK_LBUTTON) ? Mouse::LeftDown : Mouse::LeftUp) |
				((wParam & MK_MBUTTON) ? Mouse::MiddleDown : Mouse::MiddleUp) |
				((wParam & MK_RBUTTON) ? Mouse::RightDown : Mouse::RightUp) |
				((wParam & MK_SHIFT) ? Mouse::Shift : 0) |
				((wParam & MK_CONTROL) ? Mouse::Control : 0);
			return root->onMouseWheel(HIWORD(wParam), (Mouse::Button)buttons);

		case WM_SIZE:
PRINT("Root::windowProc: WM_SIZE");
			GetClientRect(root->m_WindowHandle, &rect);
			root->m_Size.setWidth((float)(rect.right - rect.left));
			root->m_Size.setHeight((float)(rect.bottom - rect.top));
			root->onResize(root->m_Size);
			return CallWindowProc(root->m_WindowProc,hwnd, Message, wParam, lParam);

		default:
			if (hwnd != NULL) {
				return CallWindowProc(root->m_WindowProc,hwnd, Message, wParam, lParam);
			} else {
				return DefWindowProc(hwnd, Message, wParam, lParam);
			}
	}
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
LRESULT CALLBACK Root::stdWindowProc(HWND hwnd, UINT Message,WPARAM wParam, LPARAM lParam) {
	Root* root = (Root *)(intptr_t)GetWindowLong(hwnd,GWL_USERDATA);
	RECT rect;

	switch(Message) {
		case WM_SIZE:
PRINT("Root::windowProc: WM_SIZE");
			GetClientRect(root->m_WindowHandle, &rect);
			root->m_Size.setWidth((float)(rect.right - rect.left));
			root->m_Size.setHeight((float)(rect.bottom - rect.top));
			root->onResize(root->m_Size);
			return CallWindowProc(root->m_WindowProc,hwnd, Message, wParam, lParam);

		default:
			if (hwnd != NULL) {
				return CallWindowProc(root->m_WindowProc,hwnd, Message, wParam, lParam);
			} else {
				return DefWindowProc(hwnd, Message, wParam, lParam);
			}
	}
}
}; };
