/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "gui3d.view.h"
#include "gui3d.root.h"

#include "object.inl"
#include "base.string.inl"
#include "container.chain.inl"
#include "graphics.rectangle.inl"
#include "graphics.dimension.inl"
#include "graphics.point.inl"
#include "graphics.transform.inl"
#include "gui3d.view.inl"
#include "gui3d.root.inl"


namespace magnum { namespace gui3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
View::View(const Rectangle<float>& frame) {
	m_Root = NULL;
	m_Parent = NULL;
	m_Context = NULL;
	m_Alignement = 0;
	m_Frame = frame;
	m_Enabled = true;
	m_Transform.setIdentity();
	m_Bounds.setOrigin(Point<float>(0,0));
	m_Bounds.setSize(Dimension<float>(m_Frame.getWidth(),m_Frame.getHeight()));
	updateTransform();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
View::~View() {
	if (m_Root != NULL) {
		m_Root->remove(this);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void View::setBounds(const Rectangle<float>& bounds) {
	m_Bounds = bounds;
	m_Transform.setTranslation(m_Bounds.getOrigin());
	m_Transform.setScale(m_Frame.getWidth() / m_Bounds.getWidth(), m_Frame.getHeight() / m_Bounds.getHeight());
	updateTransform();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void View::setScale(const Point<float>& Scale) {
	m_Transform.setScale(Scale);
	m_Bounds = convertFromParent(m_Frame);
	updateTransform();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void View::setOrigin(const Point<float>& origin) {
	m_Transform.setTranslation(origin);
	m_Bounds = convertFromParent(m_Frame);
	updateTransform();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void View::enable() {
	if (m_Enabled)
		return;
	m_Enabled = true;
	onEnable();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void View::disable() {
	if (!m_Enabled)
		return;
	m_Enabled = false;
	if (hasFocus())
		releaseFocus();
	if (m_Root != NULL) {
		if (m_Root->m_MouseView == this)
			m_Root->m_MouseView = NULL;
	}
	onDisable();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onEnable() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onDisable() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onGetFocus() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onReleaseFocus() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onPaint() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onResize(const Rectangle<float>& ) {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Dimension<float> View::onGetPreferredSize() const {
	return m_Frame.getSize();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onSetPreferredSize(void) {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onKeyDown(Keyboard::Key ) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onKeyUp(Keyboard::Key ) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onChar(Character::Code, Character::Modifier ) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onMouseMove(const Point<float>& pos, Mouse::Button button) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onMouseButton(const Point<float>& , Mouse::Button ) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onMouseEnter(const Point<float>& , Mouse::Button ) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onMouseLeave() {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onActivate() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onDeactivate() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onAttach() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool View::onDetach() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void View::updateFrame() {
	m_Bounds = convertFromParent(m_Frame);
	
	if (m_Parent != NULL) {
		m_AbsTransform = ((View*)m_Parent)->m_AbsTransform * (m_Transform + m_Frame.getOrigin());
	} else {
		m_AbsTransform = m_Transform + m_Frame.getOrigin();
	}	

	onResize(m_Frame);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void View::updateTransform() {
	if (m_Parent != NULL) {
		m_AbsTransform = ((View*)m_Parent)->m_AbsTransform * (m_Transform + m_Frame.getOrigin());
	} else {
		m_AbsTransform = m_Transform + m_Frame.getOrigin();
	}	
}


}; };
