/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "gui3d.window.h"
#include "gui3d.root.h"

#include "object.inl"
#include "graphics.rectangle.inl"
#include "graphics.colour.inl"
#include "gui3d.root.inl"
#include "gui3d.window.inl"
#include "gui3d.view.inl"
#include "gui3d.container.inl"
#include "gui3d.context.inl"

namespace magnum { namespace gui3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Window::Window(const Rectangle<float>& r) : Container(r) {
	m_State = NormalState;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Window::onPaint() {
	Context* context = getContext();
	context->setColour(m_BackColour);
	context->fillRectangle(getBounds());
	context->setColour(m_BorderColour);
	context->drawRectangle(getBounds());
	return Container::onPaint();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Window::onMouseMove(const Point<float>& p,Mouse::Button b) {
	if (m_State == NormalState) {
		Container::onMouseMove(p,b);
		return true;
	}
		
	if ((b & Mouse::LeftDown) == 0) {
		m_State = NormalState;
		return true;
	}
	Point<float> loc = getLocation();
	loc.x += p.x - m_MoveReference.x;
	loc.y += p.y - m_MoveReference.y;
	setLocation(loc);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Window::onMouseButton(const Point<float>& p,Mouse::Button b) {
	if (Container::onMouseButton(p,b))
		return true;
	if (b == Mouse::LeftDown) {
		m_State = MovingState;
		m_MoveReference = p;
		return true;
	} else if (b == Mouse::LeftUp) {
		m_State = NormalState;
		return true;
	}
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Window::onMove(const Point<float>& ) {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Window::onResize(const Rectangle<float>& r) {
	return Container::onResize(r);
}


}; };

