/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "input.body.h"
#include "input.glove.h"
#include "input.space.h"

#include "object.inl"
#include "math.vector.inl"
#include "math.matrix.inl"
#include "container.chain.inl"
#include "input.device.inl"
#include "input.marker.inl"
#include "input.body.inl"
#include "input.space.inl"

namespace magnum { namespace input {

/*--------------------------------------------------------------------------*/
/**
 */
Body::Body() {
	m_Calibration.identity();
}


/*--------------------------------------------------------------------------*/
/**
 */
Body::Body(const char* n) : Marker(n) {
	m_Calibration.identity();
}


/*--------------------------------------------------------------------------*/
/**
 */
Body::Body(const String& n) : Marker(n) {
	m_Calibration.identity();
}


/*--------------------------------------------------------------------------*/
/**
 */
Body::Body(Space* s) : Marker(s) {
	m_Calibration.identity();
	getSpace()->m_Bodies.insert(this);
	release();
}


/*--------------------------------------------------------------------------*/
/**
 */
Body::Body(Space* s,const char* n) : Marker(s,n) {
	m_Calibration.identity();
	getSpace()->m_Bodies.insert(this);
	release();
}


/*--------------------------------------------------------------------------*/
/**
 */
Body::Body(Space* s,const String& n) : Marker(s,n) {
	m_Calibration.identity();
	getSpace()->m_Bodies.insert(this);
	release();
}


/*--------------------------------------------------------------------------*/
/**
 */
void Body::updateTransform(const Matrix<float,4>& mtx) {
	if (getSpace() == NULL) {
		// Bestimme Kallibrierung des Bodies im globalen Koordinatensystem
		const Matrix<float,4>& body_calib = getCalibration();
		Matrix<float,4> body_calib_inv = body_calib.getInverse();
		
		// Rechne jetzt die Kallibrierung des Bodies selbst mit ein
		body_calib_inv[0][3] = 
		body_calib_inv[1][3] = 
		body_calib_inv[2][3] = 0.0f;

		// Berechne zunchst Body im globalen Kallibrierungsraum
		m_Transform = mtx * body_calib_inv;
		m_Transform[0][3] -= body_calib[0][3];
		m_Transform[1][3] -= body_calib[1][3];
		m_Transform[2][3] -= body_calib[2][3];
	} else {
		// Hole kallibrierungs-Matrix fr den globalen Raum
		const Matrix<float,4>& space_calib = getSpace()->getCalibration();
		Matrix<float,4> space_inv = space_calib.getInverse();

		// Bestimme Kallibrierung des Bodies im globalen Koordinatensystem
		Matrix<float,4> body_calib = space_inv * getCalibration();
		Matrix<float,4> body_calib_inv = body_calib.getInverse();
		
		// Rechne jetzt die Kallibrierung des Bodies selbst mit ein
		body_calib_inv[0][3] = 
		body_calib_inv[1][3] = 
		body_calib_inv[2][3] = 0.0f;

		// Berechne zunchst Body im globalen Kallibrierungsraum
		m_Transform = space_inv * mtx * body_calib_inv;
		m_Transform[0][3] -= body_calib[0][3];
		m_Transform[1][3] -= body_calib[1][3];
		m_Transform[2][3] -= body_calib[2][3];
	}
	    
    Marker::updatePosition(Vector3<float>(mtx[0][3], mtx[1][3], mtx[2][3]));
}

}; };
