/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "input.marker.h"
#include "input.body.h"
#include "input.glove.h"
#include "input.space.h"

#include "object.inl"
#include "math.vector.inl"
#include "math.matrix.inl"
#include "container.chain.inl"
#include "input.device.inl"
#include "input.marker.inl"
#include "input.space.inl"

namespace magnum { namespace input {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Marker::Marker() : m_Space(NULL) {
	m_Calibration.null();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Marker::Marker(const char* n) : Device(n), m_Space(NULL) {
	m_Calibration.null();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Marker::Marker(const String& n) : Device(n), m_Space(NULL) {
	m_Calibration.null();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Marker::Marker(Space* s) : m_Space(s) {
	m_Calibration.null();
	getSpace()->m_Markers.insert(this);
	release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Marker::Marker(Space* s,const char* n) : Device(n), m_Space(s) {
	m_Calibration.null();
	getSpace()->m_Markers.insert(this);
	release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Marker::Marker(Space* s,const String& n) : Device(n), m_Space(s) {
	m_Calibration.null();
	getSpace()->m_Markers.insert(this);
	release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Marker::updatePosition(const Vector3<float>& pos) {
	if (getSpace() == NULL) {
		// Berechne zunchst Body im globalen Kallibrierungsraum
		m_Position = pos - getCalibration();
	} else {
		// Hole kallibrierungs-Matrix fr den globalen Raum
		const Matrix<float,4>& space_calib = getSpace()->getCalibration();
		Matrix<float,4> space_inv = space_calib.getInverse();

		// Bestimme Kallibrierung des Bodies im globalen Koordinatensystem
		Vector3<float> body_calib = space_inv * getCalibration();
		
		// Berechne zunchst Body im globalen Kallibrierungsraum
		m_Position = space_inv * pos - body_calib;
	}
}

}; };
