/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "io.archive.h"

#include "object.inl"
#include "base.string.inl"
#include "io.file.inl"
#include "io.archive.inl"

namespace magnum { namespace io {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
ArchiveDirectory::ArchiveDirectory(Archive* a) {
	m_Archive = a;
	m_Position = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
ArchiveFile::ArchiveFile(Archive* a) {
	m_Archive = a;
	m_Position = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Archive::Archive(File* file) {
	m_File = file;
	m_Position = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
size_t Archive::rawWrite(const void *buf, int64 where, size_t size) {
	size_t ret = 0;
	m_Mutex.lock();
	try {
		if (m_Position != where)
			m_File->setPosition(where);
		ret = m_File->write(buf, size);
		m_Position += ret;
	} catch(...) {
		m_Mutex.unlock();
		throw;
	}
	m_Mutex.unlock();
	return ret;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
size_t Archive::rawRead(void *buf, int64 where, size_t size) {
	size_t ret = 0;
	m_Mutex.lock();
	try {
		if (m_Position != where)
			m_File->setPosition(where);
		ret = m_File->read(buf, size);
		m_Position += ret;
	} catch(...) {
		m_Mutex.unlock();
		throw;
	}
	m_Mutex.unlock();
	return ret;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int64 Archive::rawGetSize() {
	int64 ret = 0;
	m_Mutex.lock();
	try {
		ret = m_File->getSize();
	} catch(...) {
		m_Mutex.unlock();
		throw;
	}
	m_Mutex.unlock();
	return ret;
}

}; };
