/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "io.datainput.h"
#include "io.inputstream.h"

#include "base.string.inl"
#include "io.datainput.inl"
#include "io.inputstream.inl"

namespace magnum { namespace io {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
DataInput::DataInput(InputStream* in) {
	ASSERT(in != NULL);
	m_Input = in;
	m_Input->refer();
	m_BytesRead = 0;
	m_LastByte = '\r';
	m_Mode = Unknown;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
DataInput::DataInput(InputStream& in) {
	m_Input = &in;
	m_Input->refer();
	m_BytesRead = 0;
	m_LastByte = '\r';
	m_Mode = Unknown;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
DataInput::~DataInput() {
	if (m_Input != NULL) {
		m_Input->release();
		m_Input = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
size_t DataInput::skip(size_t o) {
	ASSERT(m_Input != NULL);
	size_t ret = m_Input->skip(o);
	if (ret == InputStream::EndOfFile)
		return InputStream::EndOfFile;
	m_BytesRead += ret;
	return ret;
}	


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
size_t DataInput::read(void *buffer, size_t size) {
	ASSERT(m_Input != NULL);
	size_t ret = m_Input->read(buffer,size);
	if (ret == InputStream::EndOfFile)
		return InputStream::EndOfFile;
	m_BytesRead += ret;
	return ret;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
String DataInput::readLine() {
	String str;
	char buffer[256];
	char* bufcur = &buffer[0];
	char c;
	while(true) {
		// Try to read next character
		size_t rsize = m_Input->read(&c, sizeof(c));
		if ((int)rsize <= 0)
			break;
		m_BytesRead += sizeof(c);
	
		// Check for newline
		if (c == '\n') {
			if (m_LastByte == '\r')
				continue;
			else 
				break;
		}
		m_LastByte = c;
		if (c == 0)
			break;
		if (c == '\r')
			break;
		*bufcur++ = c;
		if (bufcur == &buffer[255]) {
			*bufcur = 0;
			str += buffer;
			bufcur = &buffer[0];
		}
	}
	
	*bufcur = 0;
	str += buffer;
	return str;
}

}; };

