/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "io.exception.h"
#include "io.sysdirectory.h"

#include "object.inl"
#include "base.string.inl"
#include "container.allocator.inl"
#include "container.list.inl"
#include "io.fileinfo.inl"
#include "io.directory.inl"
#include "io.sysdirectory.inl"

namespace magnum { namespace io {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void SystemDirectory::insert(const WIN32_FIND_DATA& entry) {
	Timestamp ct = Timestamp((((uint64)(entry.ftCreationTime.dwHighDateTime) << 32) + entry.ftCreationTime.dwLowDateTime) / 10);
	Timestamp mt = Timestamp((((uint64)(entry.ftLastWriteTime.dwHighDateTime) << 32) + entry.ftLastWriteTime.dwLowDateTime) / 10);
	Timestamp at = Timestamp((((uint64)(entry.ftLastAccessTime.dwHighDateTime) << 32) + entry.ftLastAccessTime.dwLowDateTime) / 10);
	unsigned int flags = FileInfo::Readable;
	if (entry.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		flags |= FileInfo::Directory;
	if (entry.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN)
		flags |= FileInfo::Hidden;
	if (!(entry.dwFileAttributes & FILE_ATTRIBUTE_READONLY))
		flags |= FileInfo::Writeable;
	if (entry.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT)
		flags |= FileInfo::Link;
		
	Directory::insert(entry.cFileName, entry.nFileSizeLow, ct, mt, at, flags);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void SystemDirectory::onRefresh() {
	HANDLE hnd;
	WIN32_FIND_DATA	entry;
	
	String dir = getDirectory();
	unsigned int len = dir.getLength();
	if (len == 0) {
		dir = ".\\";
	} else if (len > 0) {
		if (dir[len-1] != '\\')
			dir += "\\";
	}
	dir += "*";	
	
	hnd = FindFirstFile(dir, &entry);
	if (hnd == INVALID_HANDLE_VALUE) {
		if (GetLastError() == ERROR_NO_MORE_FILES)	
			return;
		throw DirectoryNotFoundException();
	}
	insert(entry);

	for(;;) {
		if (FindNextFile(hnd, &entry) == 0)
			break;
		insert(entry);
	}
	
	FindClose(hnd);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void SystemDirectory::onOpen(const char* dir) {
	ASSERT(dir != NULL);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool SystemDirectory::create(const char* name) {
#if defined(__FORWINDOWS__)
	return (CreateDirectory(name, NULL) != 0);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool SystemDirectory::remove(const char* name) {
#if defined(__FORWINDOWS__)
	return (RemoveDirectory(name) != 0);
#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
String SystemDirectory::getCurrent() {
	char buffer[MAX_PATH];
	DWORD len;
	len = GetCurrentDirectory(MAX_PATH, buffer);
	return String(buffer);
}

}; };

