/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "media.buffer.h"
#include "media.node.h"

#include "media.buffer.inl"
#include "media.node.inl"

namespace magnum { namespace media {
using magnum::types::byte;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Buffer::Buffer(Node* owner, unsigned int size) {
	m_Buffer = new byte[size];
	m_Size = size;
	m_MaxSize = size;
	m_Consumer = NULL;
	m_Producer = NULL;
	m_Owner = owner;
	m_OwnerData = NULL;
	m_UserData = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Buffer::Buffer(Node* owner, void* data, unsigned int size) {
	m_Buffer = new byte[size];
	m_Size = size;
	m_MaxSize = size;
	m_Consumer = NULL;
	m_Producer = NULL;
	m_Owner = owner;
	m_OwnerData = data;
	m_UserData = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Buffer::~Buffer() {
	if (m_Owner != NULL) 
		m_Owner->onDestroyBuffer(this);
	if (m_Buffer != NULL) 
		delete[] (byte *)m_Buffer;

	m_Buffer = NULL;
	m_Size = 0;
	m_MaxSize = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Buffer::getBuffer(void* dst,unsigned int offset,unsigned int size) const {
	if (m_Buffer == NULL)
		return false;
	if (size == -1)
		size = m_Size - offset;
	else if ((size + offset) > m_Size)
		return false;

	memcpy(dst, ((byte *)m_Buffer)+offset, size);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Buffer::setBuffer(const void* src,unsigned int offset,unsigned int size) {
	if (m_Buffer == NULL)
		return false;
	if (size == -1)
		size = m_Size - offset;
	else if ((size + offset) > m_Size)
		return false;

	memcpy(((byte *)m_Buffer)+offset, src, size);
	return true;
}


}; };

