/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "media.exception.h"
#include "media.device.h"
#include "media.filter.h"
#include "media.audio.h"
#include "media.video.h"

#include "base.string.inl"
#include "object.class.inl"
#include "object.inl"
#include "container.list.inl"
#include "container.allocator.inl"
#include "container.iterator.inl"
#include "media.device.inl"
#include "media.node.inl"
#include "media.filter.inl"
#include "media.audio.inl"
#include "media.video.inl"


namespace magnum { namespace media {

const char Device::DefaultAudio[] = "DefaultAudio";
const char Device::DefaultVideo[] = "DefaultVideo";

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Device* Device::createDevice(const char *which, bool exclusive) {
	Device* dev;

	// Try to create Audio device
	dev = AudioDevice::create(which, exclusive);
	if (dev != NULL)
		return dev;

	// Try to create video device
	dev = VideoDevice::create(which, exclusive);
	if (dev != NULL)
		return dev;

	return NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Node* Device::createNode(const char* which) {
	List<NodeInfo>::Iterator iter;
	NodeInfo *info;

	if (which == NULL) {
		which = Node::DefaultNode;
	}

	for (iter = m_NodeList.begin(); iter != m_NodeList.end(); iter++) {
		info = &(*iter);
		if (info->m_Name == which) {
			try {
				return info->m_Creator(this, info->m_Name.getString());
			} catch (Exception e) {
				return NULL;
			}
		}
	}

	return NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Filter* Device::createFilter(const char* which) {
	if (which == NULL) {
		which = Node::DefaultFilter;
	}

	Filter* filter;
	filter = (Filter *)createNode(which);
	return filter;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Device::publishNode(const char *nodeName, Node::Type t, Device::NodeCreator c) {
	NodeInfo info;
	info.m_Name = nodeName;
	info.m_Type = t;
	info.m_Creator = c;
	m_NodeList.insert(info);
	return true;
}


}; };

