/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "media.looper.h"
#include "media.message.h"

#include "base.string.inl"
#include "object.class.inl"
#include "object.inl"
#include "system.thread.inl"
#include "system.signal.inl"
#include "system.mutex.inl"
#include "container.allocator.inl"
#include "container.list.inl"
#include "container.iterator.inl"
#include "media.looper.inl"
#include "media.message.inl"


namespace magnum { namespace media {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Looper::Looper(Thread* t) {
	m_Thread = t;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Looper::~Looper() {
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Looper::sendMessage(const Message& msg) {
	m_MessageMutex.lock();
	m_Messages.insert(msg);
	m_MessageMutex.unlock();
	m_MessageSignal.raise();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Looper::sendMessage(unsigned int cmd) {
	m_MessageMutex.lock();
	List<Message>::Iterator iter = m_Messages.append();
	Message& msg = *iter;
	msg.setCommand(cmd);
	m_MessageMutex.unlock();
	m_MessageSignal.raise();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Looper::sendMessage(unsigned int cmd,Pin* pin) {
	m_MessageMutex.lock();
	List<Message>::Iterator iter = m_Messages.append();
	Message& msg = *iter;
	msg.setCommand(cmd);
	msg.setPin(pin);
	m_MessageMutex.unlock();
	m_MessageSignal.raise();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Looper::sendMessage(unsigned int cmd,Pin* pin,void* parm) {
	m_MessageMutex.lock();
	List<Message>::Iterator iter = m_Messages.append();
	Message& msg = *iter;
	msg.setCommand(cmd);
	msg.setPin(pin);
	msg.setPointer(parm);
	m_MessageMutex.unlock();
	m_MessageSignal.raise();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Looper::processMessages() {
	for (;;) {
		// Try to get a message from the message queue
		m_MessageMutex.lock();
		List<Message>::Iterator iter = m_Messages.begin();
		if (iter == m_Messages.end()) {
			m_MessageSignal.reset();
			m_MessageMutex.unlock();
			break;
		}
		Message msg = *iter;
		m_Messages.remove(iter);
		m_MessageMutex.unlock();
		
		if (msg.getCommand() == Message::Quit)
			return false;
		onHandleMessage(msg);
	}
	
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Looper::getMessage(Message* msg) {
	m_MessageMutex.lock();
	List<Message>::Iterator iter = m_Messages.begin();
	if (iter == m_Messages.end()) {
		m_MessageSignal.reset();
		m_MessageMutex.unlock();
		return false;
	}
	*msg = *iter;
	m_Messages.remove(iter);
	m_MessageMutex.unlock();
	
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Looper::peekMessage(Message* msg) {
	m_MessageMutex.lock();
	List<Message>::Iterator iter = m_Messages.begin();
	if (iter == m_Messages.end()) {
		m_MessageSignal.reset();
		m_MessageMutex.unlock();
		return false;
	}
	*msg = *iter;
	m_MessageMutex.unlock();
	
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Looper::clearMessages() {
	m_MessageMutex.lock();
	m_Messages.clear();
	m_MessageSignal.reset();
	m_MessageMutex.unlock();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Looper::onLoop() {
	// Process any messages
	return processMessages();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Looper::onHandleMessage(const Message& ) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Looper::onRemoveMessage(const Message& ) {
}

}; };
