/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "media.node.h"
#include "media.pin.h"
#include "media.buffer.h"

#include "object.inl"
#include "object.class.inl"
#include "base.string.inl"
#include "container.chain.inl"
#include "container.iterator.inl"
#include "media.node.inl"
#include "media.pin.inl"
#include "media.buffer.inl"
#include "media.format.inl"

namespace magnum { namespace media {

const char Pin::DefaultInput[] = "DefaultInput";
const char Pin::DefaultOutput[] = "DefaultOutput";


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Pin::adjustFormat(Format& fmt) {
	ASSERT(m_Node != NULL);
	return m_Node->onAdjustFormat(this, fmt);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Pin::requestFormat(const Format& fmt) {
	ASSERT(m_Node != NULL);
	ASSERT(m_Node->getState() == Node::Stopped);
	if (m_Node->getState() != Node::Stopped)
		return false;
		
	Format lfmt = fmt;

	// Request new format
	if (!m_Node->onAdjustFormat(this, lfmt))
		return false;
	if (!m_Node->onSetFormat(this, lfmt))
		return false;

	m_Format = lfmt;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Pin::setFormat(const Format& fmt) {
	ASSERT(m_Node != NULL);
	ASSERT(m_Node->getState() == Node::Stopped);
	if (m_Node->getState() != Node::Stopped)
		return false;

	// Request new format
	if (!m_Node->onSetFormat(this, fmt))
		return false;

	m_Format = fmt;
	return true;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
OutputPin::OutputPin(Node* node,bool def) : Pin(node) {
	if (def)
		node->m_DefaultOutput = this;
	node->m_OutputPins.insert(this);
	release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
OutputPin::~OutputPin() {
	Node* node = getNode();
	if (node->m_DefaultOutput == this)
		node->m_DefaultOutput = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
InputPin::InputPin(Node* node,bool def) : Pin(node) {
	if (def)
		node->m_DefaultInput = this;
	node->m_InputPins.insert(this);
	release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
InputPin::~InputPin() {
	Node* node = getNode();
	if (node->m_DefaultInput == this)
		node->m_DefaultInput = NULL;
}

}; };
