/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "media.node.h"
#include "media.device.h"
#include "media.pin.h"
#include "media.graph.h"
#include "media.buffer.h"
#include "media.processor.h"

#include "base.string.inl"
#include "object.class.inl"
#include "object.inl"
#include "system.thread.inl"
#include "system.signal.inl"
#include "container.list.inl"
#include "media.node.inl"
#include "media.device.inl"
#include "media.pin.inl"
#include "media.graph.inl"
#include "media.buffer.inl"
#include "media.filter.inl"
#include "media.processor.inl"


namespace magnum { namespace media {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
ProcessorNode::ProcessorNode(Device* dev, const char* name) : Filter(dev, name), m_InputPin(this), m_OutputPin(this) {
	m_Thread = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
ProcessorNode::~ProcessorNode() {
	if (m_Thread != NULL) {
		if (m_Thread->isAlive()) {
			quit();
			m_Thread->join();
		}

		m_Thread->release();
		m_Thread = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool ProcessorNode::onStart() {
	if (m_Thread != NULL)
		return false;

	m_Thread = new ProcessorThread(this);
	m_Thread->start(true);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool ProcessorNode::onStop() {
	if (m_Thread == NULL)
		return false;

	if (m_Thread->isAlive()) {
		quit();
		m_Thread->join();
	}

	m_Thread->release();
	m_Thread = NULL;
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool ProcessorNode::onPause() {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool ProcessorNode::onUnpause() {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool ProcessorNode::onReceiveBuffer(InputPin* , Buffer* ) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool ProcessorNode::onSendBuffer(OutputPin* , Buffer* ) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool ProcessorNode::onRequestBuffer(OutputPin* ) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool ProcessorNode::onProcess(const Buffer* , Buffer* ) {
	return false;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
ProcessorNode::ProcessorThread::ProcessorThread(ProcessorNode *node) {
	m_Node = node;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
ProcessorNode::ProcessorThread::~ProcessorThread() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool ProcessorNode::ProcessorThread::run() {
	m_Node->loop();
	return true;
}


}; };

