/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "media.buffer.h"
#include "media.reader.h"

#include "object.inl"
#include "object.pointer.inl"
#include "io.file.inl"
#include "io.stream.inl"
#include "media.reader.inl"
#include "media.pin.inl"

namespace magnum { namespace media {

/*--------------------------------------------------------------------------*/
/**
 */
Reader::Reader() : Node("Reader",Node::Producer),
	m_Pin(this) {
}


/*--------------------------------------------------------------------------*/
/**
 */
Reader::Reader(File* file) : Node("Reader",Node::Producer),
	m_Pin(this),
	m_File(file),
	m_Stream(file->getInputStream()) {
	ASSERT(file != NULL);
}


/*--------------------------------------------------------------------------*/
/**
 */
Reader::Reader(InputStream* stream) : Node("Reader",Node::Producer),
	m_Pin(this),
	m_Stream(stream) {
	ASSERT(stream != NULL);
}


/*--------------------------------------------------------------------------*/
/**
 */
Reader::~Reader() {
	m_Stream.setNull();
	m_File.setNull();
}


/*--------------------------------------------------------------------------*/
/**
 */
bool Reader::onSetFormat(Pin* pin,const Format& fmt) {
	ASSERT(pin->getNode() == this);
	if (pin != getPin())
		return false;
	
	return onSetFormat(fmt);
}


/*--------------------------------------------------------------------------*/
/**
 */
bool Reader::onAdjustFormat(Pin* pin,Format& fmt) {
	ASSERT(pin->getNode() == this);
	if (pin != getPin())
		return false;

	return onAdjustFormat(fmt);
}


/*--------------------------------------------------------------------------*/
/**
 */
bool Reader::onRequestBuffer(OutputPin* pin,Buffer* buf) {
	ASSERT(pin == getPin());
	if (buf == NULL)
		buf = new Buffer(this, getPin()->getFormat().getBufferSize());

	if (!onRead(buf)) {
		buf->release();
		return false;
	}

	return sendBuffer(&m_Pin, buf);
}

	
/*--------------------------------------------------------------------------*/
/** 
 */
bool Reader::onSetFormat(const Format& ) {
	return true;
}


/*--------------------------------------------------------------------------*/
/** 
 */
bool Reader::onAdjustFormat(Format& ) {
	return true;
}


/*--------------------------------------------------------------------------*/
/** 
 */
bool Reader::onRead(Buffer* buf) {
	return false;
}

}; };
