/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "net.mimeheaders.h"

#include "object.inl"
#include "base.string.inl"
#include "container.allocator.inl"
#include "container.iterator.inl"
#include "container.pair.inl"
#include "container.list.inl"
#include "io.stream.inl"
#include "io.outputstream.inl"
#include "net.mimeheaders.inl"

namespace magnum { namespace net {

/*--------------------------------------------------------------------------*/
/**
 */
void MimeHeaders::insert(const char* key,const char* val) {
	ASSERT(key != NULL);
	ASSERT(val != NULL);
	if (strlen(key) == 0)
		return;
		
	Iterator iter = at(key);
	if (iter == m_Headers.end())
		m_Headers.insert(Pair<String, String>(key, val));
	else
		iter->second = val;
}


/*--------------------------------------------------------------------------*/
/**
 */
void MimeHeaders::remove(const char* key) {
	ASSERT(key != NULL);
	Iterator iter = at(key);
	if (iter != m_Headers.end())
		m_Headers.remove(iter);
}


/*--------------------------------------------------------------------------*/
/**
 */
bool MimeHeaders::contains(const char* key) {
	ASSERT(key != NULL);
	return (at(key) != m_Headers.end());
}


/*--------------------------------------------------------------------------*/
/**
 */
const char* MimeHeaders::find(const char* key) const {
	ASSERT(key != NULL);
	Iterator iter = at(key);
	if (iter = m_Headers.end())
		return NULL;
	else
		return iter->second;
}


/*--------------------------------------------------------------------------*/
/**
 */
MimeHeaders::Iterator MimeHeaders::at(const char* key) const {
	ASSERT(key != NULL);
	Iterator iter = m_Headers.begin();
	while (iter != m_Headers.end()) {
		if (iter->first == key)
			break;
		++iter;
	}
	
	return iter;
}


/*--------------------------------------------------------------------------*/
/**
 */
void MimeHeaders::writeTo(OutputStream* os) const {
	ASSERT(os != NULL);
	for (Iterator iter = m_Headers.begin(); iter != m_Headers.end(); ++iter) {
		String str = iter->first + ": " + iter->second + "\r\n";
		os->write((const char*)str, str.getLength());
	}
}

}; };
