/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "net.socket.h"
#include "net.socketstream.h"

#include "object.inl"
#include "base.string.inl"
#include "net.socket.inl"
#include "net.socketstream.inl"

namespace magnum { namespace net {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
SocketInputStream::SocketInputStream(Socket* s) {
	ASSERT(s != NULL);
	m_Socket = s;
	if (m_Socket->getInputStream() != this)
		m_Socket->refer();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
SocketInputStream::~SocketInputStream() {
	if (m_Socket != NULL) {
		if (m_Socket->getInputStream() != this)
			m_Socket->release();
		m_Socket = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
size_t SocketInputStream::onRead(void *buffer, size_t size) {
#if defined(__FORWINDOWS__)
 #if 1
	int received = recv(m_Socket->m_Socket, (char *)buffer, (int)size, 0);
	if (received == SOCKET_ERROR)
		Socket::throwSocketException();
 #else
	DWORD received;
	DWORD flags;
	WSABUF wsabuffer;
	wsabuffer.len = (u_long)size;
	wsabuffer.buf = (char *)buffer;
	if (WSARecv(m_Socket->m_Socket, &wsabuffer, 1, &received, &flags, NULL, NULL) != 0)
		Socket::throwSocketException();
 #endif	
#else
	unsigned long received;
	unsigned long flags = 0;
	received = recv(m_Socket->m_Socket, buffer, size, flags);
	if (received == -1)
		Socket::throwSocketException();
#endif
	return received;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
size_t SocketInputStream::onSkip(size_t size) {
	ASSERT(false);
	return 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int64 SocketInputStream::onAvailable() {
	unsigned long size;
#if defined(__FORWINDOWS__)
	ioctlsocket(m_Socket->m_Socket, FIONREAD, &size);
#else
	ioctl(m_Socket->m_Socket, FIONREAD, &size);
#endif
	return size;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool SocketInputStream::onClose() {
	if (m_Socket != NULL) {
		if (m_Socket->getInputStream() == this)
			return true;
		m_Socket->release();
		m_Socket = NULL;
	}
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
SocketOutputStream::SocketOutputStream(Socket* file) {
	ASSERT(file != NULL);
	m_Socket = file;
	if (m_Socket->getOutputStream() != this)
		m_Socket->refer();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
SocketOutputStream::~SocketOutputStream() {
	if (m_Socket != NULL) {
		if (m_Socket->getOutputStream() != this)
			m_Socket->release();
		m_Socket = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
size_t SocketOutputStream::onWrite(const void *buffer, size_t size) {
#if defined(__FORWINDOWS__)
 #if 1
	int sent = send(m_Socket->m_Socket, (const char*)buffer, (int)size, 0);
	if (sent == SOCKET_ERROR)
		Socket::throwSocketException();
 #else
	DWORD sent;
	DWORD flags = 0;
	WSABUF wsabuffer;
	wsabuffer.len = (u_long)size;
	wsabuffer.buf = (char *)buffer;
	if (WSASend(m_Socket->m_Socket, &wsabuffer, 1, &sent, flags, NULL, NULL) != 0)
		Socket::throwSocketException();
 #endif
#else
	unsigned long sent;
	unsigned long flags = 0;
	sent = send(m_Socket->m_Socket, buffer, size, flags);
	if (sent == -1)
		Socket::throwSocketException();
#endif
	return sent;
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool SocketOutputStream::onFlush() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool SocketOutputStream::onClose() {
	if (m_Socket != NULL) {
		if (m_Socket->getOutputStream() == this)
			return true;
		m_Socket->release();
		m_Socket = NULL;
	}
	return true;
}

}; };

