/* 
  Magnum 
  Copyright (C) 2002-2006 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "net.url.h"

#include "base.string.inl"
#include "object.inl"
#include "net.url.inl"

#include <stdlib.h>

namespace magnum { namespace net {

/*--------------------------------------------------------------------------*/
/**
 */
URL::URL() : m_Port(0), m_Dirty(false) {
}


/*--------------------------------------------------------------------------*/
/**
 */
URL::URL(const char* q) {
	setQuery(q);
}


/*--------------------------------------------------------------------------*/
/**
 */
URL::URL(const URL& ref,const char* file) 
	: m_Dirty(false), m_Protocol(ref.m_Protocol), m_Host(ref.m_Host), m_Port(ref.m_Port) {
	if (file[0] == '/') {
		m_File = file;
	} else {
		int idx = ref.m_Path.lastIndexOf('/');
		if (idx > 0)
			m_File = ref.m_Path.substring(0,idx) + file;
		else
			m_File = file;
	}

	// Extract path information
	int idx = m_File.indexOf('?');
	if (idx > 0)
		m_Path = m_File.substring(0,idx-1);
	else
		m_Path = m_File;

	// Construct query string
	if (m_Port != 0) {
		char sport[10];
		itoa(m_Port, sport, 10);
		m_Query = m_Protocol + "://" + m_Host + ":" + sport + m_File;
	} else {
		m_Query = m_Protocol + "://" + m_Host + m_File;
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
URL::URL(const char* prot, const char* host) 
	: m_Dirty(false), m_Protocol(prot), m_Host(host), m_Port(0), m_File("/"), m_Path("/") {
	m_Protocol.lower();
	m_Host.lower();
	
	// Construct query string
	m_Query = m_Protocol + "://" + m_Host + "/";
}


/*--------------------------------------------------------------------------*/
/**
 */
URL::URL(const char* prot, const char* host, const char* file) 
	: m_Dirty(false), m_Protocol(prot), m_Host(host), m_File(file), m_Port(0) {
	m_Protocol.lower();
	m_Host.lower();
	
	// Prefix a "/" to the file, if necessary
	if ((m_File.getLength() == 0) || (m_File[0] != '/'))
		m_File = "/" + m_File;
		
	// Extract path information
	int idx = m_File.indexOf('?');
	if (idx > 0)
		m_Path = m_File.substring(0,idx-1);
	else
		m_Path = m_File;
		
	// Construct query string
	m_Query = m_Protocol + "://" + m_Host + m_File;
}


/*--------------------------------------------------------------------------*/
/**
 */
URL::URL(const char* prot, const char* host, unsigned int port, const char* file) 
	: m_Dirty(false), m_Protocol(prot), m_Host(host), m_File(file), m_Port(port) {
	m_Protocol.lower();
	m_Host.lower();
	
	// Prefix a "/" to the file, if necessary
	if ((m_File.getLength() == 0) || (m_File[0] != '/'))
		m_File = "/" + m_File;
		
	// Extract path information
	int idx = m_File.indexOf('?');
	if (idx > 0)
		m_Path = m_File.substring(0,idx-1);
	else
		m_Path = m_File;

	// Construct query string
	if (m_Port != 0) {
		char sport[10];
		itoa(m_Port, sport, 10);
		m_Query = m_Protocol + "://" + m_Host + ":" + sport + m_File;
	} else {
		m_Query = m_Protocol + "://" + m_Host + m_File;
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
void URL::setQuery(const char* q) {
	m_Dirty = false;
	m_Query = q;
	
	int protidx = m_Query.indexOf(':');
	if (protidx >= 0) {
		m_Protocol = m_Query.substring(0, protidx-1);
		m_File = m_Query.substring(protidx+1);
	} else {
		m_File = m_Query;
	}
	
	// Remove slashes
	int hstart = m_File.startsWith("//") ? 2 : 0;
	
	// Get hostname
	int hostidx = m_File.indexOf('/',hstart);
	if (hostidx < 0) {
		m_Host = m_File.substring(hstart);
		m_File.clear();
	} else {
		m_Host = m_File.substring(hstart, hostidx-1);
		m_File = m_File.substring(hostidx);
	}
	
	// Get port
	int portidx = m_Host.indexOf(':');
	if (portidx >= 0) {
		m_Port = atoi(m_Host.substring(portidx+1));
		m_Host = m_Host.substring(0,portidx-1);
	} else {
		m_Port = 0;
	}

	// Extract path information
	int idx = m_File.indexOf('?');
	if (idx > 0)
		m_Path = m_File.substring(0,idx-1);
	else
		m_Path = m_File;
}


/*--------------------------------------------------------------------------*/
/**
 */
void URL::setProtocol(const char* p) {
	m_Protocol = p;
	m_Protocol.lower();
	m_Dirty = true;
}


/*--------------------------------------------------------------------------*/
/**
 */
void URL::setHost(const char* p) {
	m_Host = p;
	m_Host.lower();
	m_Dirty = true;
}


/*--------------------------------------------------------------------------*/
/**
 */
void URL::setPath(const char* p) {
	m_File = p;
	m_Path = p;
	m_Dirty = true;
}


/*--------------------------------------------------------------------------*/
/**
 */
void URL::setFile(const char* p) {
	m_File = p;
	m_Dirty = true;

	// Extract path information
	int idx = m_File.indexOf('?');
	if (idx > 0)
		m_Path = m_File.substring(0,idx-1);
	else
		m_Path = m_File;
}


/*--------------------------------------------------------------------------*/
/**
 */
void URL::buildQuery() {
	// Construct query string
	if (m_Port != 0) {
		char sport[10];
		itoa(m_Port, sport, 10);
		m_Query = m_Protocol + "://" + m_Host + ":" + sport + m_File;
	} else {
		m_Query = m_Protocol + "://" + m_Host + m_File;
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
bool URL::isHostNumeric() const {
	if (m_Host.getLength() == 0)
		return false;

	const char* ptr = &m_Host[0];
	while (*ptr != 0) {
		char c = *ptr;
		if ((c < '0' || c > '9') && c != '.')
			return false;
		ptr++;
	}
	return true;
}

}; };
