/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "object.attachment.h"

#include "object.inl"
#include "base.string.inl"
#include "container.chain.inl"
#include "container.iterator.inl"
#include "object.attachment.inl"

namespace magnum { namespace object {

/*--------------------------------------------------------------------------*/
/**
 */
AttachmentList::AttachmentList() {
}


/*--------------------------------------------------------------------------*/
/**
 */
AttachmentList::~AttachmentList() {
	clear();
}


/*--------------------------------------------------------------------------*/
/**
 */
bool AttachmentList::remove(const char* n) {
	ASSERT(n != NULL);
	AttachmentBase* a = find(n);
	if (a == NULL)
		return false;
	m_Attachments.remove(a);
	delete a;
	return true;
}


/*--------------------------------------------------------------------------*/
/**
 */
void AttachmentList::remove(AttachmentBase* a) {
	ASSERT(a != NULL);
	ASSERT(a->getChain() == &m_Attachments);
	m_Attachments.remove(a);
	delete a;
}


/*--------------------------------------------------------------------------*/
/**
 */
void AttachmentList::clear() {
	while (begin() != end()) {
		AttachmentBase* a = begin();
		m_Attachments.remove(a);
		delete a;
	}
}


/*--------------------------------------------------------------------------*/
/**
 */
AttachmentBase* AttachmentList::find(const char* n) {
	for (Iterator iter = begin(); iter != end(); ++iter) {
		if (!strncmp(iter->getName(), n, sizeof(iter->m_Name)))
			return &(*iter);
	}
	return NULL;
}


/*--------------------------------------------------------------------------*/
/**
 */
const AttachmentBase* AttachmentList::find(const char* n) const {
	for (Iterator iter = begin(); iter != end(); ++iter) {
		if (!strncmp(iter->getName(), n, sizeof(iter->m_Name)))
			return &(*iter);
	}
	return NULL;
}

}; };

