/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "object.softpointer.h"

#include "object.inl"
#include "object.softpointer.inl"

namespace magnum { namespace object {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
SoftPointerBase::SoftPointerBase(Object* o) {
	m_Pointer = o;
	if (m_Pointer == NULL) {
		m_Next = m_Prev = NULL;
		return;
	}
	if (o->m_SoftPointer != NULL) {
		o->m_SoftPointer->m_Prev = this;
	}
	m_Next = o->m_SoftPointer;
	m_Prev = NULL;
	o->m_SoftPointer = this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
SoftPointerBase::SoftPointerBase(const SoftPointerBase& ptr) {
	m_Pointer = ptr.m_Pointer;
	if (m_Pointer == NULL) {
		m_Next = m_Prev = NULL;
		return;
	}
	m_Prev = NULL;
	m_Next = m_Pointer->m_SoftPointer;
	m_Next->m_Prev = this;
	m_Pointer->m_SoftPointer = this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
SoftPointerBase::~SoftPointerBase() {
	if (m_Pointer == NULL)
		return;
	if (m_Prev == NULL)
		m_Pointer->m_SoftPointer = m_Next;
	else
		m_Prev->m_Next = m_Next;
	if (m_Next != NULL)
		m_Next->m_Prev = m_Prev;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void SoftPointerBase::setPointer(Object* o) {
	// Did the pointer change?
	if (o == m_Pointer)
		return;
		
	// Remove SoftPointer from previous object
	if (m_Pointer != NULL) {
		if (m_Prev == NULL)
			m_Pointer->m_SoftPointer = m_Next;
		else
			m_Prev->m_Next = m_Next;
		if (m_Next != NULL)
			m_Next->m_Prev = m_Prev;
	}

	// Set new object pointer
	m_Pointer = o;
	if (m_Pointer == NULL) {
		m_Next = m_Prev = NULL;
		return;
	}
	if (o->m_SoftPointer != NULL) {
		o->m_SoftPointer->m_Prev = this;
	}
	m_Next = o->m_SoftPointer;
	m_Prev = NULL;
	o->m_SoftPointer = this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void SoftPointerBase::setNull() {
	if (m_Pointer == NULL)
		return;

	if (m_Prev == NULL)
		m_Pointer->m_SoftPointer = m_Next;
	else
		m_Prev->m_Next = m_Next;
	if (m_Next != NULL)
		m_Next->m_Prev = m_Prev;

	m_Pointer = NULL;		
	m_Next = m_Prev = NULL;
}

}; };

