/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "object.autopointer.h"
#include "graphics3d.colourmaterial.h"
#include "graphics3d.phongshader.h"
#include "graphics3d.context.h"
#include "plots3d.plot.h"

#include "object.inl"
#include "object.pointer.inl"
#include "object.autopointer.inl"
#include "graphics.colour.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.appearance.inl"
#include "graphics3d.shader.inl"
#include "graphics3d.colourmaterial.inl"
#include "graphics3d.material.inl"
#include "graphics3d.blendmode.inl"
#include "graphics3d.depthmode.inl"
#include "graphics3d.stencilmode.inl"
#include "graphics3d.polygonmode.inl"
#include "plots3d.plot.inl"

namespace magnum { namespace plots3d {
using magnum::object::AutoPointer;
using magnum::graphics3d::ColourMaterial;
using magnum::graphics3d::PhongShader;
using magnum::graphics3d::FixedShader;
using magnum::graphics3d::Shader;
using magnum::graphics::Colour;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Plot::Plot() {
	m_Appearance = new Appearance();
	m_Appearance->release();
	AutoPointer<ColourMaterial> material = new ColourMaterial();
	material->setAmbient(Colour<float>(0.1f, 0.1f, 0.1f));
	material->setDiffuse(Colour<float>(0.41f, 0.41f, 0.52f));
	material->setSpecular(Colour<float>(1.0f, 1.0f, 1.0f));
	material->setEmission(Colour<float>(0.0f, 0.0f, 0.0f));
	material->setShininess(20.0f);
	m_Appearance->attachMaterial(material);
	AutoPointer<Shader> shader = new PhongShader((FixedShader::Features)(FixedShader::Lighting | FixedShader::TwoSided | FixedShader::LocalViewer | FixedShader::SeparateSpecular));
	m_Appearance->attachShader(shader);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Plot::onRender(const Matrix4<float>& ) const {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Plot::onClear() {
}

}; };

