/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "object.autopointer.h"
#include "math.vector3.h"
#include "geometry.alignedbox.h"
#include "graphics3d.indexbuffer.h"
#include "graphics3d.vertexbuffer.h"
#include "graphics3d.appearance.h"
#include "graphics3d.context.h"
#include "graphics3d.shader.h"
#include "plots3d.exception.h"
#include "plots3d.volumeplot3d.h"

#include "object.inl"
#include "object.pointer.inl"
#include "object.autopointer.inl"
#include "base.string.inl"
#include "math.vector.inl"
#include "math.vector3.inl"
#include "math.matrix.inl"
#include "geometry.alignedbox.inl"
#include "geometry.bounds.inl"
#include "container.array.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.context.inl"
#include "graphics3d.matrixstack.inl"
#include "graphics3d.mesh.inl"
#include "graphics3d.vertexbuffer.inl"
#include "graphics3d.vertexmesh.inl"
#include "graphics3d.vertexformat.inl"
#include "graphics3d.shader.inl"
#include "plots3d.plot.inl"
#include "plots3d.plot3d.inl"
#include "plots3d.volumeplot3d.inl"

namespace magnum { namespace plots3d {
using magnum::math::Vector3;
using magnum::object::AutoPointer;
using magnum::graphics3d::Context;
using magnum::graphics3d::MatrixStack;
using magnum::graphics3d::VertexFormat;
using magnum::graphics3d::Types;
using magnum::graphics3d::Attributes;
using magnum::graphics3d::Mesh;
using magnum::geometry::AlignedBox3;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
VolumePlot3d::VolumePlot3d() {
	setResolution(100);
	Vector3<float> b(1.0f,1.0f,1.0f);
	publishBounds(Bounds3<float>(AlignedBox3<float>(-b,b)));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void VolumePlot3d::setResolution(unsigned int r) {
	m_Resolution = r;
	
	VertexFormat fmt;
	fmt.insert(Types::Vector3f, Attributes::Vertex);
	
	AutoPointer<VertexBuffer> vb = VertexBuffer::create(fmt, m_Resolution*4, VertexBuffer::Static);
	if (vb == NULL)
		throw MemoryException();

	m_Mesh.setPrimitive(Mesh::QuadList);
	m_Mesh.attachVertexBuffer(vb);
	Vector3<float>* data = (Vector3<float>*)vb->lock(VertexBuffer::Discard);
	for (unsigned int i = 0; i < m_Resolution; i++) {
		*data++ = Vector3<float>(-1.0f, -1.0f, 2.0f*i/m_Resolution -1.0f);
		*data++ = Vector3<float>(1.0f, -1.0f, 2.0f*i/m_Resolution -1.0f);
		*data++ = Vector3<float>(1.0f, 1.0f, 2.0f*i/m_Resolution -1.0f);
		*data++ = Vector3<float>(-1.0f, 1.0f, 2.0f*i/m_Resolution -1.0f);
	}
	vb->unlock();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void VolumePlot3d::onRender(const Matrix4<float>& mtx) const {
	Context* c = getContext();
	ASSERT(c != NULL);
	c->getMatrixStack()->setModel(mtx);

	if (getAppearance() != NULL)
		getAppearance()->select();
	m_Mesh.render();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void VolumePlot3d::onClear() {
}

}; };

