/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "scene3d.behaviour.h"
#include "scene3d.locale.h"
#include "scene3d.universe.h"
#include "scene3d.audio.h"
#include "scene3d.graphics.h"

#include "object.inl"
#include "object.softpointer.inl"
#include "base.string.inl"
#include "container.allocator.inl"
#include "container.scheduler.inl"
#include "geometry.space.inl"
#include "scene3d.node.inl"
#include "scene3d.leaf.inl"
#include "scene3d.behaviour.inl"
#include "scene3d.locale.inl"
#include "scene3d.universe.inl"

namespace magnum { namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Behaviour* Behaviour::onClone() const {
	return new Behaviour(*this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Behaviour::onAttachLocale(Locale* l) {
	Leaf<Behaviour>::onAttachLocale(l);
	clearFlag(ActiveBehaviour);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Behaviour::onDetachLocale(Locale* l) {
	Leaf<Behaviour>::onDetachLocale(l);
	if (getScheduler() != NULL)
		getScheduler()->remove(this);
	clearFlag(ActiveBehaviour);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Behaviour::onEnable() {
	Leaf<Behaviour>::onEnable();
	clearFlag(ActiveBehaviour);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Behaviour::onDisable() {
	Leaf<Behaviour>::onDisable();
	if (getScheduler() == NULL)
		return;
	getScheduler()->remove(this);
	clearFlag(ActiveBehaviour);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Behaviour::onProcess(const Timer& ) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Behaviour::onActivate() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Behaviour::onDeactivate() {
}

}; };
