/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/
#include "graphics3d.context.h"
#include "scene3d.marker.h"
#include "scene3d.universe.h"
#include "scene3d.cubemapgenerator.h"
#include "scene3d.rendertraverser.h"
#include "scene3d.graphics.h"
#include "scene3d.locale.h"

#include "object.pointer.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.camera.inl"
#include "graphics3d.perspectivecamera.inl"
#include "graphics3d.framebuffer.inl"
#include "scene3d.marker.inl"
#include "scene3d.universe.inl"
#include "scene3d.cubemapgenerator.inl"
#include "scene3d.rendertraverser.inl"
#include "scene3d.locale.inl"
#include "scene3d.graphics.inl"

namespace magnum { namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
CubemapGenerator::CubemapGenerator(Universe* u,magnum::graphics3d::Context* c) :
	m_Universe(u), m_GraphicsContext(c) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void CubemapGenerator::render() {
	if (m_Marker == NULL)
		return;
	if (getTexture() == NULL)
		return;

	ASSERT(m_Marker->getUniverse() == getUniverse());

	// Make context current
	m_GraphicsContext.getContext()->makeCurrent();

	// Setup traverser
	magnum::graphics3d::CubemapGenerator::render(Transform3<float>(m_Marker->getWorldTransform().getInverse()));
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void CubemapGenerator::onRender(const Matrix4<float>& mtx,CubeTexture::Face ) {
	RenderTraverser traverser(getUniverse(), &m_GraphicsContext);

	// Build frustum
	Frustum3<float> frustum;
	
	// Get Camera transformation
	traverser.setPreTransform(Transform3<float>(mtx));
	traverser.setPreTranslation(-m_Marker->getLocale()->getOrigin());
	((Traverser<RenderTraverser, Graphics>*)&traverser)->traverse(frustum, -traverser.getPreTranslation());
}

}; };
