/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.context.h"
#include "scene3d.viewer.h"
#include "scene3d.universe.h"
#include "scene3d.imagegenerator.h"
#include "scene3d.rendertraverser.h"

#include "object.pointer.inl"
#include "graphics.rectangle.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.camera.inl"
#include "graphics3d.framebuffer.inl"
#include "scene3d.viewer.inl"
#include "scene3d.universe.inl"
#include "scene3d.imagegenerator.inl"
#include "scene3d.rendertraverser.inl"

namespace magnum { namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
ImageGenerator::ImageGenerator(Universe* u,magnum::graphics3d::Context* c) :
	m_Universe(u), m_GraphicsContext(c) {
	FrameBuffer* fb = FrameBuffer::getDefault();
	m_Viewport.setX(0.0f);
	m_Viewport.setY(0.0f);
	m_Viewport.setWidth((float)fb->getWidth());
	m_Viewport.setHeight((float)fb->getHeight());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ImageGenerator::attachFrameBuffer(FrameBuffer* fb) {
	ASSERT(fb != NULL);
	m_FrameBuffer = fb;
	m_Viewport.setX(0.0f);
	m_Viewport.setY(0.0f);
	m_Viewport.setWidth((float)fb->getWidth());
	m_Viewport.setHeight((float)fb->getHeight());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void ImageGenerator::render() {
	if (m_Viewer == NULL)
		return;
	if (m_FrameBuffer == NULL)
		return;

	ASSERT(m_Viewer->getUniverse() == getUniverse());

	// Make context current
	m_GraphicsContext.getContext()->makeCurrent();

	// Select viewer
	if (m_FrameBuffer != NULL)
		m_FrameBuffer->select();
	else
		FrameBuffer::selectDefault();
	m_Viewer->getCamera()->select();

	// Setup traverser
	RenderTraverser traverser(getUniverse(), &m_GraphicsContext);
	traverser.traverse(m_Viewer);
}

}; };
