/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "scene3d.locale.h"
#include "scene3d.universe.h"
#include "scene3d.root.h"
#include "scene3d.graphics.h"
#include "scene3d.audio.h"
#include "scene3d.behaviour.h"

#include "container.chain.inl"
#include "container.allocator.inl"
#include "geometry.space.inl"
#include "scene3d.root.inl"
#include "scene3d.locale.inl"
#include "scene3d.universe.inl"


namespace magnum { namespace scene3d {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Locale::Locale(Universe* u) {
	ASSERT(u != NULL);
	m_Universe = u;
	m_Optimised = false;
	m_Origin = Vector3<double>(0.0, 0.0, 0.0);
	m_Universe->m_Locales.insert(this);
	release();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Locale::Locale(Universe* u, const Vector3<double>& vec) {
	ASSERT(u != NULL);
	m_Universe = u;
	m_Optimised = false;
	m_Origin = vec;
	m_Universe->m_Locales.insert(this);
	release();
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Locale::insert(Root* root) {
	m_Roots.insert(root);
	root->m_Locale = this;
	root->onAttachLocale(this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Locale::remove(Root* root) {
	root->onDetachLocale(this);
	root->m_Locale = NULL;
	m_Roots.remove(root);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Locale::contains(Root* root) const {
	return m_Roots.contains(root);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Locale::clear() {
	while(!m_Roots.isEmpty()) {
		Root* root = m_Roots.begin();
		root->onDetachLocale(this);
		root->m_Locale = NULL;
		m_Roots.remove(root);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool Locale::optimise() {
	m_Optimised = false;
	return m_Optimised;
}
					

}; };
