/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.framebuffer.h"
#include "graphics3d.appearance.h"
#include "graphics3d.shader.h"
#include "graphics3d.model.h"
#include "audio3d.source.h"
#include "scene3d.observer.h"
#include "scene3d.universe.h"
#include "scene3d.locale.h"
#include "scene3d.viewer.h"
#include "scene3d.listener.h"
#include "scene3d.activator.h"
#include "scene3d.modifier.h"
#include "scene3d.root.h"
#include "scene3d.rendertraverser.h"
#include "scene3d.listentraverser.h"
#include "scene3d.scheduletraverser.h"

#include "object.pointer.inl"
#include "object.softpointer.inl"
#include "container.iterator.inl"
#include "container.chain.inl"
#include "container.scheduler.inl"
#include "graphics3d.context.inl"
#include "graphics3d.framebuffer.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.accelerator.inl"
#include "graphics3d.material.inl"
#include "graphics3d.light.inl"
#include "graphics3d.texture.inl"
#include "graphics3d.shader.inl"
#include "audio3d.context.inl"
#include "scene3d.observer.inl"
#include "scene3d.universe.inl"
#include "scene3d.node.inl"
#include "scene3d.leaf.inl"
#include "scene3d.group.inl"
#include "scene3d.root.inl"
#include "scene3d.viewer.inl"
#include "scene3d.listener.inl"
#include "scene3d.activator.inl"
#include "scene3d.modifier.inl"
#include "scene3d.behaviourscheduler.inl"
#include "scene3d.rendertraverser.inl"
#include "scene3d.listentraverser.inl"
#include "scene3d.scheduletraverser.inl"
#include "scene3d.audiocontext.inl"
#include "scene3d.graphicscontext.inl"

namespace magnum { namespace scene3d {
using magnum::graphics3d::FrameBuffer;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Observer::Observer(Universe* u, magnum::graphics3d::Context* g, magnum::audio3d::Context* a) :
	m_GraphicsContext(g), m_AudioContext(a), m_Universe(u) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Observer::~Observer() {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Observer::process() {
	if (m_Activator == NULL)
		return;

	// Setup traverser
	ScheduleTraverser traverser(getUniverse(), &m_BehaviourScheduler);
	traverser.traverse(m_Activator);
	m_BehaviourScheduler.process(*getUniverse()->getTimer());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Observer::listen() {
	if (m_Listener == NULL)
		return;

	ASSERT(m_Listener ->getUniverse() == getUniverse());

	// Make context current
	m_AudioContext.getContext()->makeCurrent();

	// Get Camera transformation
	ListenTraverser traverser(getUniverse(), &m_AudioContext);
	traverser.traverse(m_Listener);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Observer::view() {
	if (m_Viewer == NULL)
		return;

	ASSERT(m_Viewer->getUniverse() == getUniverse());

	// Make context current
	m_GraphicsContext.getContext()->makeCurrent();

	// Select framebuffer
	FrameBuffer::selectDefault();

	// Setup traverser
	RenderTraverser traverser(getUniverse(), &m_GraphicsContext);
	traverser.traverse(m_Viewer);
}

}; };
