/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "graphics3d.context.h"
#include "scene3d.rendertraverser.h"
#include "scene3d.universe.h"
#include "scene3d.root.h"
#include "scene3d.modifier.h"
#include "scene3d.viewer.h"
#include "scene3d.marker.h"
#include "scene3d.locale.h"
#include "scene3d.audio.h"
#include "scene3d.graphics.h"
#include "scene3d.behaviour.h"

#include "object.inl"
#include "object.pointer.inl"
#include "base.string.inl"
#include "math.vector.inl"
#include "math.vector3.inl"
#include "math.matrix.inl"
#include "geometry.space.inl"
#include "geometry.transform.inl"
#include "geometry.box.inl"
#include "geometry.cube.inl"
#include "geometry.frustum.inl"
#include "container.iterator.inl"
#include "container.chain.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.camera.inl"
#include "scene3d.universe.inl"
#include "scene3d.locale.inl"
#include "scene3d.rendertraverser.inl"
#include "scene3d.graphicscontext.inl"
#include "scene3d.node.inl"
#include "scene3d.leaf.inl"
#include "scene3d.root.inl"
#include "scene3d.graphics.inl"
#include "scene3d.modifier.inl"
#include "scene3d.viewer.inl"
#include "scene3d.marker.inl"
#include "scene3d.traverser.inl"

namespace magnum { namespace scene3d {


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
RenderTraverser::RenderTraverser(Universe* u,GraphicsContext* c) : Traverser<RenderTraverser,Graphics>(u) {
	m_Context = c;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void RenderTraverser::traverse(const Viewer* viewer) {
	ASSERT(viewer != NULL);
	ASSERT(viewer->getUniverse() == getUniverse());

	// Select viewer
	viewer->getCamera()->select();

	// Get Camera transformation
	setPreTransform(Transform3<float>(viewer->getWorldTransform()).invert());
	setPreTranslation(-viewer->getLocale()->getOrigin());
	Traverser<RenderTraverser, Graphics>::traverse(viewer->getFrustum(), -getPreTranslation());
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void RenderTraverser::traverse(const Marker* node, const Frustum3<float>& frustum) {
	ASSERT(node != NULL);
	ASSERT(node->getUniverse() == getUniverse());

	// Get Camera transformation
	setPreTransform(Transform3<float>(node->getWorldTransform()).invert());
	setPreTranslation(-node->getLocale()->getOrigin());
	Traverser<RenderTraverser, Graphics>::traverse(frustum, -getPreTranslation());
}

}; };

