/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "math.vector.h"
#include "math.linearsolver.h"
#include "graphics.point.h"
#include "graphics3d.context.h"
#include "scene3d.viewer.h"
#include "scene3d.locale.h"

#include "object.inl"
#include "object.pointer.inl"
#include "base.string.inl"
#include "math.matrix.inl"
#include "math.vector.inl"
#include "math.vector2.inl"
#include "math.vector3.inl"
#include "math.linearsolver.inl"
#include "geometry.frustum.inl"
#include "geometry.transform.inl"
#include "geometry.linear.inl"
#include "geometry.ray.inl"
#include "graphics.point.inl"
#include "graphics.dimension.inl"
#include "graphics.rectangle.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.camera.inl"
#include "graphics3d.perspectivecamera.inl"
#include "graphics3d.framebuffer.inl"
#include "scene3d.node.inl"
#include "scene3d.graphics.inl"
#include "scene3d.viewer.inl"
#include "scene3d.locale.inl"


namespace magnum { namespace scene3d {
using magnum::math::Vector;
using magnum::graphics::Point;
using magnum::graphics::Dimension;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Viewer::Viewer() {
	m_Transform = Transform3<float>::Identity;
	setNearPlane(100.0f);
	setFarPlane(1000.0f);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Viewer::Viewer(const Viewer& v) 
	: Graphics (v), 
	m_Frustum(v.m_Frustum), 
	m_Transform(v.m_Transform), 
	m_WorldTransform(v.m_WorldTransform),
	m_Camera(v.m_Camera) {
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Ray<float,3> Viewer::createRay(float x,float y) const {
	Ray<float,3> ray = m_Camera.createRay(x,y);
	ray.transform(m_WorldTransform);
	return ray;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Viewer* Viewer::onClone() const {
	return new Viewer(*this);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Viewer::onAttachLocale(Locale* l) {
	Node::onAttachLocale(l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Viewer::onDetachLocale(Locale* l) {
	Node::onDetachLocale(l);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Viewer::onEnable() {
	Node::onEnable();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Viewer::onDisable() {
	Node::onDisable();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Viewer::onTransform(const Transform3<float>& parent) {
	m_WorldTransform = parent * m_Transform;
	updateFrustum();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Viewer::updateFrustum() {
	Vector<float,3> dir = m_Transform.getAxis(2);
	Vector<float,3> right = m_Transform.getAxis(0);
	Vector<float,3> up = m_Transform.getAxis(1);
	Point<float> ll = m_Camera.getViewWindow().getOrigin();
	Point<float> ur = m_Camera.getViewWindow().getOrigin() + m_Camera.getViewWindow().getSize();
	Dimension<float> dim = m_Camera.getViewWindow().getSize();

	m_Frustum.setOrigin(m_Transform.getTranslation());
	// m_Frustum.setNear(m_Camera.getNearPlane());
	// m_Frustum.setFar(m_Camera.getFarPlane());
	m_Frustum.setDirection(-dir);
	m_Frustum.setAxis(0, 0.5f * dim.getWidth() * right);
	m_Frustum.setAxis(1, 0.5f * dim.getHeight() * up);
	m_Frustum.setOffset(0.5f * (ll + ur));
}

}; };
