/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Kupferschmidt. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Kupferschmidt  (k.kupferschmidt@dimajix.de)
*/

#include "system.timestamp.h"

#include "object.inl"
#include "base.string.inl"
#include "system.timestamp.inl"

#ifdef __FORWINDOWS__
#	undef NONLS
#	undef _WINNLS_
#	include <winnls.h>
#endif


namespace magnum { namespace system {

/*----------------------------------------------------------------------------
Description:
	Sets the Timestamp object to the current system time.
----------------------------------------------------------------------------*/
void Timestamp::getCurrentTime() {
	#ifdef WIN32
		SYSTEMTIME stime;
		FILETIME ftime;
		GetSystemTime(&stime);
		SystemTimeToFileTime(&stime,&ftime);
		m_Time = ((((uint64)ftime.dwHighDateTime) << 32 ) + (uint64)ftime.dwLowDateTime) / 10;
	#else
		m_Time = 0;
	#endif
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
String Timestamp::toString() const {
	#ifdef WIN32
		SYSTEMTIME stime,ltime;
		FILETIME ftime;
		ftime.dwHighDateTime = (DWORD)((10*m_Time) >> 32);
		ftime.dwLowDateTime = (DWORD)(10*m_Time);
		FileTimeToSystemTime(&ftime, &stime);
		SystemTimeToTzSpecificLocalTime(NULL, &stime, &ltime);
		char tstr[64], dstr[64];
		GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &ltime, NULL, dstr, 64);
		GetTimeFormat(LOCALE_USER_DEFAULT, TIME_FORCE24HOURFORMAT, &ltime, NULL, tstr, 64);
		return String(dstr) + " " + tstr;
	#else
		return String();
	#endif
}

}; };
