#include "magnum.h"

using namespace magnum::container;
using namespace magnum::algorithm;


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
class ChainTestClass : public Chainable<ChainTestClass>, public Object {
public:
	int			data;
};


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void ChainTest(void)  {
	Chain<ChainTestClass> chain;
	unsigned int i;
	unsigned int size;
	ChainTestClass items[1000];

	printf("\nStarting chain test...\n");

	size = 0;
	for(i = 1; i < 1000; i++) {
		items[i].data = i;
		chain.insert(&items[i]);
		if (chain.getSize() != i)
			printf(" ChainSize error! ");
		size++;
	}

	printf("Checking chain size: ");
	if (chain.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking iterator: ");
	Chain<ChainTestClass>::Iterator iter, end, begin;
	end = chain.end();
	iter = chain.begin();
	size = 0;
	while(iter != end) {
		size++;
		if (iter->data != (int)size)
			printf ("Chain content error. ");
		iter++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (chain.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking backward iterator: ");
	begin = chain.begin();
	iter = chain.end();
	size = 0;
	while(iter != begin) {
		iter--;
		size++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (chain.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking remove: ");
	i = 0;
	while(chain.begin() != chain.end()) {
		chain.remove(chain.begin());
		i++;
	}
	if (i != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking insertFirst: ");
	size = 0;
	for(i = 1; i < 1000; i++) {
		chain.insertFirst(&items[i]);
		if (chain.getSize() != i)
			printf(" ChainSize error! ");
		size++;
	}
	printf("Passed.\n");

	printf("Checking iterator: ");
	end = chain.end();
	iter = chain.begin();
	size = 0;
	while(iter != end) {
		if (iter->data != (int)(chain.getSize() - size))
			printf ("Chain content error. ");
		size++;
		iter++;
	}
	printf("Passed.\n");

	chain.clear();

}



/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void ArrayTest(void)  {
	Array<int> list;
	unsigned int i;
	unsigned int size;
	int item;

	printf("\nStarting list test...\n");

	size = 0;
	for(i = 1; i < 1000; i++) {
		list.insert(i);
		if (list.getSize() != i)
			printf(" ArraySize error! ");
		size++;
	}

	printf("Checking list size: ");
	if (list.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking iterator: ");
	Array<int>::Iterator iter, end, begin;
	end = list.end();
	iter = list.begin();
	size = 0;
	while(iter != end) {
		size++;
		if (*iter != (int)size)
			printf ("Array content error. ");
		iter++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (list.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking backward iterator: ");
	begin = list.begin();
	iter = list.end();
	size = 0;
	while(iter != begin) {
		iter--;
		item = *iter;
		size++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (list.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void ListTest(void)  {
	List<int> list;
	unsigned int i;
	unsigned int size;
	int item;

	printf("\nStarting list test...\n");

	size = 0;
	for(i = 1; i < 1000; i++) {
		list.insert(i);
		if (list.getSize() != i)
			printf(" ListSize error! ");
		size++;
	}

	printf("Checking list size: ");
	if (list.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking iterator: ");
	List<int>::Iterator iter, end, begin;
	end = list.end();
	iter = list.begin();
	size = 0;
	while(iter != end) {
		size++;
		if (*iter != (int)size)
			printf ("List content error. ");
		iter++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (list.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking backward iterator: ");
	iter = list.rbegin();
	end = list.rend();
	size = 0;
	while(iter != end) {
		item = *iter;
		if (item != list.getSize() - size)
			printf ("List content error. ");
		iter--;
		size++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (list.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking remove: ");
	i = 0;
	while(list.begin() != list.end()) {
		list.remove(list.begin());
		i++;
	}
	if ((i != size) || (list.getSize() != 0))
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking insertFirst: ");
	size = 0;
	for(i = 1; i < 1000; i++) {
		list.insertFirst(i);
		if (list.getSize() != i)
			printf(" ListSize error! ");
		size++;
	}
	printf("Passed.\n");

	printf("Checking iterator: ");
	end = list.end();
	iter = list.begin();
	size = 0;
	while(iter != end) {
		if (*iter != (int)(list.getSize() - size))
			printf ("List content error. ");
		size++;
		iter++;
	}
	printf("Passed.\n");

}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void SortTest(void)  {
	Array<int> list;
	unsigned int i;
	int item;

	printf("\nStarting sort test...\n");

	printf("Creating random list: ");
	for(i = 0; i < 1000; i++) {
		list.insert(rand() % 200);
	}
	printf("Ready.\n");

	printf("Sorting: ");
	sort(list.begin(), list.end());
	printf("Ready.\n");

	printf("Checking order: ");
	item = -10;
	for(i = 0; i < 1000; i++) {
		if (list[i] < item)
			printf("Sort order error! ");
		item = list[i];
	}
	printf("Ready.\n");
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void FindTest(void)  {
	Array<int> list;
	unsigned int i;

	printf("\nStarting find test...\n");

	printf("Creating sorted list: ");
	for(i = 0; i < 1000; i++) {
		list.insert(i);
	}
	printf("Ready.\n");

	printf("Checking find: ");
	for(i = 0; i < 1000; i++) {
		Array<int>::Iterator iter;
		iter = find(list.begin(), list.end(), (int)i);
		if (iter == list.end())
			printf("Error: Key not found! ");
		else if (*iter != (int)i)
			printf("Error: Returned wrong key! ");
	}
	printf("Ready.\n");
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void SetTest(void)  {
	Set<int> set;
	unsigned int i;
	unsigned int size;
	int item;

	printf("\nStarting set test...\n");

	size = 0;
	for(i = 0; i < 1000; i++) {
		item = rand() % 200;
		if (!set.contains(item))
			size++;
		set.insert(item);
	}

	printf("Checking set size: ");
	if (set.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking iterator: ");
	Set<int>::Iterator iter, end, begin;
	end = set.end();
	iter = set.begin();
	size = 0;
	item = -1;
	while(iter != end) {
		size++;
		if (*iter <= item)
			printf("Iterator error! ");
		item = *iter;
		iter++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (set.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

/*
	printf("Checking backward iterator: ");
	begin = set.begin();
	iter = set.end();
	item = 5000;
	size = 0;
	while(iter != begin) {
		iter--;
		if (*iter >= item)
			printf("Iterator error! ");
		item = *iter;
		size++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (set.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");
*/
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void MultisetTest(void) {
	Multiset<int> set;
	unsigned int i;
	unsigned int size;
	int item;

	printf("\nStarting multiset test...\n");

	size = 0;
	for(i = 0; i < 1000; i++) {
		item = rand() % 200;
		size++;
		set.insert(item);
	}

	printf("Checking set size: ");
	if (set.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking iterator: ");
	Set<int>::Iterator iter, end, begin;
	end = set.end();
	iter = set.begin();
	size = 0;
	item = -1;
	while(iter != end) {
		size++;
		if (*iter < item)
			printf("Iterator error! ");
		item = *iter;
		iter++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (set.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");


/*	printf("Checking backward iterator: ");
	begin = set.begin();
	iter = set.end();
	item = 5000;
	size = 0;
	while(iter != begin) {
		iter--;
		if (*iter > item)
			printf("Iterator error! ");
		item = *iter;
		size++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (set.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");
*/
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void MapTest(void) {
	Map<int, int> map;
	unsigned int i;
	unsigned int size;
	int item;

	printf("\nStarting map test...\n");

	size = 0;
	for(i = 0; i < 1000; i++) {
		item = rand() % 200;
		if (!map.contains(item))
			size++;
		map.insert(item, item);
	}

	printf("Checking map size: ");
	if (map.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking iterator: ");
	Map<int, int>::Iterator iter, end, begin;
	end = map.end();
	iter = map.begin();
	size = 0;
	item = -1;
	while(iter != end) {
		size++;
		if (*iter <= item)
			printf("Iterator error! ");
		item = *iter;
		iter++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (map.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

/*
	printf("Checking backward iterator: ");
	begin = map.begin();
	iter = map.end();
	item = 5000;
	size = 0;
	while(iter != begin) {
		iter--;
		if (*iter >= item)
			printf("Iterator error! ");
		item = *iter;
		size++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (map.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");
*/
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void MultimapTest(void) {
	Multimap<int, int> map;
	unsigned int i;
	unsigned int size;
	int item;

	printf("\nStarting multimap test...\n");

	size = 0;
	for(i = 0; i < 1000; i++) {
		item = rand() % 200;
		size++;
		map.insert(item, item);
	}

	printf("Checking multimap size: ");
	if (map.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking iterator: ");
	Multimap<int, int>::Iterator iter, end, begin;
	end = map.end();
	iter = map.begin();
	size = 0;
	item = -1;
	while(iter != end) {
		size++;
		if (*iter < item)
			printf("Iterator error! ");
		item = *iter;
		iter++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (map.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

/*
	printf("Checking backward iterator: ");
	begin = map.begin();
	iter = map.end();
	item = 5000;
	size = 0;
	while(iter != begin) {
		iter--;
		if (*iter >= item)
			printf("Iterator error! ");
		item = *iter;
		size++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (map.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");
*/
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void HeapTest(void) {
	Heap<int> heap;
	unsigned int i;
	unsigned int size;
	int item, r;

	printf("\nStarting heap test...\n");

	size = 0;
	for(i = 0; i < 1000; i++) {
		item = rand() % 200;
		heap.insert(item);
		size++;
	}

	printf("Checking heap size: ");
	if (heap.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking iterator: ");
	Heap<int>::Iterator iter, end, begin;
	end = heap.end();
	iter = heap.begin();
	size = 0;
	item = -1;
	while(iter != end) {
		size++;
//		if (*iter <= item)
//			printf("Iterator error! ");
		iter++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (heap.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking backward iterator: ");
	begin = heap.begin();
	iter = heap.end();
	size = 0;
	while(iter != begin) {
		iter--;
		size++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (heap.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");
		
	printf("Checking remove: ");
	i = 0;
	item = 300;
	while(heap.getSize() > 0) {
		r = heap.remove();
		if (r > item)
			printf("Remove error! ");
		item = r;
		i++;
	}

	printf("Checking remove count: ");
	if (i != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
void PriorityHeapTest(void) {
	PriorityHeap<int> heap;
	PriorityHeap<int>::Iterator ref[1000];
	unsigned int i;
	unsigned int size;
	int item, r;

	printf("\nStarting priorityheap test...\n");

	size = 0;
	for(i = 0; i < 1000; i++) {
		item = rand() % 200;
		ref[i] = heap.insert(item, item);
		size++;
	}

	printf("Checking heap size: ");
	if (heap.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");


	printf("Checking change: ");
	for(i = 0; i < 1000; i++) {
		item = rand() % 200;
		heap.change(ref[i], item);
	}
	for(i = 0; i < 1000; i++) {
		heap.change(ref[i], *ref[i]);
	}
	printf("Passed.\n");


	printf("Checking iterator: ");
	PriorityHeap<int>::Iterator iter, end, begin;
	end = heap.end();
	iter = heap.begin();
	size = 0;
	item = -1;
	while(iter != end) {
		size++;
		iter++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (heap.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking backward iterator: ");
	begin = heap.begin();
	iter = heap.end();
	size = 0;
	while(iter != begin) {
		iter--;
		size++;
	}
	printf("Passed.\n");

	printf("Checking iterator count: ");
	if (heap.getSize() != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");

	printf("Checking remove: ");
	i = 0;
	item = 300;
	while(heap.getSize() > 0) {
		r = heap.remove();
		if (r > item)
			printf("Remove error! ");
		item = r;
		i++;
	}
	if (i != size)
		printf("Mismtach! \n");
	else
		printf("Passed.\n");
}


/*----------------------------------------------------------------------------------
----------------------------------------------------------------------------------*/
int main(void) {
	ListTest();
	ChainTest();
	ArrayTest();
	SortTest();
	FindTest();
	HeapTest();
	PriorityHeapTest();
	SetTest();
	MultisetTest();
	MapTest();
	MultimapTest();
	return 0;
}
