/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/


#include "Gui3dTest.h"
#include "Ticker.h"
#include "magnum.io.h"
#include "magnum.math.h"
#include "magnum.freetype.h"

namespace magnum { namespace sample {
using magnum::object::AutoPointer;
using magnum::application::Message;
using magnum::graphics3d::Context;
using magnum::graphics::Image;
using magnum::graphics::TranslatorPNG;
using magnum::graphics::TranslatorPSD;
using magnum::freetype::Font;
using magnum::freetype::FontFactory;
using magnum::freetype::Descriptor;
using magnum::io::SystemFile;
using magnum::io::SystemFileFactory;
using magnum::math::Range;
using magnum::gui3d::Root;
using magnum::gui3d::Window;
using magnum::gui3d::Box;
using magnum::gui3d::Bitmap;
using magnum::gui3d::Button;
using magnum::gui3d::Slider;
using magnum::gui3d::Checkbox;
using magnum::gui3d::ImageView;
using magnum::gui3d::StringView;
using magnum::gui3d::Container;
using magnum::input::Mouse;



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
OpenGLView::OpenGLView(Rectangle<float> rect) : OpenGL(rect) {
	m_trans[0] = m_trans[1] = m_trans[2] = 0.0f;
	m_rot[0] = m_rot[1] = 0.0f;
}



/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool OpenGLView::onMouseMove(const Point<float>& pt, int but) {
	m_omouse = m_mouse;
	m_mouse = pt;
//	if(mx & 1 << 15) mx -= (1 << 16);
//	if(my & 1 << 15) my -= (1 << 16);

	float dx = m_omouse.getX() - m_mouse.getX();
    float dy = m_mouse.getY() - m_omouse.getY();

	if (but & Mouse::LeftDown) {
		m_trans[0] -= dx / 10.0f;
		m_trans[1] -= dy / 10.0f;
	}
	if (but & Mouse::MiddleDown) {
		m_rot[0] += (dy * 180.0f) / 500.0f;
		m_rot[1] -= (dx * 180.0f) / 500.0f;
	#define clamp(x) x = x > 360.0f ? x-360.0f : x < -360.0f ? x+=360.0f : x
		clamp(m_rot[0]);
		clamp(m_rot[1]);
	}
	if (but & Mouse::RightDown) {
		m_trans[2] -= (dx+dy) / 10.0f;
    }

	if (but & Mouse::AnyDown)
		invalidate();

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool OpenGLView::onAttachWindow() {
	OpenGL::onAttachWindow();

	m_Context = magnum::graphics3d::Context::getCurrent();
	m_Context->setCulling(magnum::graphics3d::Context::CullBack);

	m_Context->setClearColour(Colour<float>(0,0,0));
	m_Context->setViewport(getBounds());

	// Create GUI3D Root
	m_Root = new Root(m_Context);
	m_Root->attachDirectInput();

	// Create buttons
	SystemFileFactory filefactory("data");
	FontFactory ffactory(&filefactory);
	Descriptor desc;
	desc.setFamily("Accidental Presidency");
	desc.setSize(32);
	AutoPointer<Font> font = ffactory.create(desc);

	AutoPointer<Button> button = new Button(Rectangle<float>(10,10,170,40), Message(), "Button 1!");
	button->setFont(font);
	m_Root->insert(button);

	button = new Button(Rectangle<float>(10,60,170,40), Message(), "Button 2!");
	button->setFont(font);
	m_Root->insert(button);

	button = new Button(Rectangle<float>(10,110,170,40), Message(), "Button 3!");
	button->setFont(font);
	m_Root->insert(button);
	
	
	// Create Checkboxes
	Checkbox* checkbox = new Checkbox(Rectangle<float>(10,160,170,30), Message(), "Checkbox");
	checkbox->setFont(font);
	m_Root->insert(checkbox);
	
	
	// Create Slider
	AutoPointer<Slider> slider = new Slider(Rectangle<float>(10,200,170,20), Message());
	m_Root->insert(slider);

	// Create Window
	AutoPointer<magnum::gui3d::Window> win = new magnum::gui3d::Window(Rectangle<float>(190,10,256,256));
	m_Root->insert(win);

	// Create Bitmap view
	TranslatorPNG reader;
	AutoPointer<File> imgFile = SystemFile::open("data/test.png");
	AutoPointer<Image> img = reader.decode(imgFile->getInputStream());
	AutoPointer<Bitmap> bmp = Bitmap::create(img);

	AutoPointer<ImageView> iview = new ImageView(Rectangle<float>(0,0,200,200), bmp);
	iview->setSize((Dimension<float>)bmp->getSize());
	win->insert(iview);

	button = new Button(Rectangle<float>(10,10,170,40), Message(), "Button 4!");
	button->setFont(font);
	win->insert(button);
	
		
	// Create different stringviews
	AutoPointer<StringView> sview;
	desc.setFamily("Larson");
	font = ffactory.create(desc);
	sview = new StringView(Rectangle<float>(10,300,400,10), "abcABC123456789");
	sview->setColour(Colour<float>(1.0f, 0.7f, 1.0f));
	sview->setFont(font);
	m_Root->insert(sview);

	desc.setFamily("Acquaintance");
	font = ffactory.create(desc);
	sview = new StringView(Rectangle<float>(10,330,400,10), "abcABC123456789");
	sview->setColour(Colour<float>(1.0f, 1.0f, 0.7f));
	sview->setFont(font);
	m_Root->insert(sview);

	desc.setFamily("JFJungleRock");
	font = ffactory.create(desc);
	sview = new StringView(Rectangle<float>(10,368,400,10), "abcABC123456789");
	sview->setColour(Colour<float>(0.7f, 1.0f, 1.0f));
	sview->setFont(font);
	m_Root->insert(sview);

	desc.setFamily("Lantern");
	font = ffactory.create(desc);
	sview = new StringView(Rectangle<float>(10,406,400,10), "abcABC123456789");
	sview->setColour(Colour<float>(1.0f, 0.7f, 0.4f));
	sview->setFont(font);
	m_Root->insert(sview);

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool OpenGLView::onDetachWindow() {
	OpenGL::onDetachWindow();
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void OpenGLView::onRender() {
	m_Context->clear(Context::ClearColour | Context::ClearDepth | Context::ClearStencil);
	
	m_Root->paint();
	m_Root->process();
}


void loadfont(void) {
	Descriptor desc;
	FontFactory factory;
	SystemFile* file;
	Font* font;
	
	desc.setSize(16.0f);
	file = SystemFile::open("arial.ttf");
	font = factory.create(desc, file);
	file->release();
}

}; };


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int main(void) {
	using namespace magnum::application;
	using namespace magnum::gui;
	using namespace magnum::sample;
	using magnum::types::byte;

	magnum::gui::Window* window = new magnum::gui::Window();
	magnum::gui::OpenGL* opengl = new OpenGLView(magnum::graphics::Rectangle<float>(0,0,460,420));
	Thread* thread;

	opengl->setColourBits(24);
	opengl->setAlphaBits(8);
	opengl->setDepthBits(24);
	opengl->setStencilBits(8);
	opengl->setAlignement(magnum::gui::View::FollowAll);

	window->setName("Magnum");
	window->setSize(Dimension<float>(460,420));
	window->insert(opengl);
	window->setMinimumSize(Dimension<float>(256,256));
	window->open();
	
	Ticker* ticker = new RedrawTicker(opengl);
	ticker->setPeriod(50);
	ticker->start();	
	
	thread = window->getThread();
	thread->join();
	
	ticker->kill();
	ticker->release();

	return true;
}
