#include "magnum.h"

using namespace magnum::geometry;
using namespace magnum::math;
using namespace magnum::stats;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void SphereTest(void) {
	Uniform u(0.0f, 100.0f);
	Vector3<float> vec[1000];
	
	unsigned int i;
	for (i = 0; i < 1000; i++) {
		vec[i].x = u();
		vec[i].y = u();
		vec[i].z = u();
	}
	
	printf("Creating sphere...");
	Sphere3<float> sphere;
	sphere.bound(vec, 1000);
	printf("ready.\n");
	
	printf("Sphere.origin = (%f, %f, %f)  Sphere.radius = %f\n",
		sphere.getOrigin()[0], sphere.getOrigin()[1], sphere.getOrigin()[2], sphere.getRadius());
	
	printf("Verifying sphere...\n");
	for (i = 0; i < 1000; i++) {
		if (!sphere.isInside(vec[i])) {
			Vector3<float> tmp = vec[i] - (Vector3<float>)sphere.getOrigin();
			float l = tmp.length();
			printf("Error at point %d = (%f, %f, %f) dist = %f!\n", i, vec[i].x, vec[i].y, vec[i].z,l);
		}
	}
	printf("ready.\n");
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void MatrixInvertTest() {
	unsigned int i,j,r;
	Uniform U;

	printf("Inverting 100x100 matrices...\n");
	Matrix<float,100> mtx, inv;
	for (r = 0; r < 100; r++) {
		for (i = 0; i < 100; i++)
			for (j = 0; j < 100; j++)
				mtx[i][j] = U();
		bool ret = LinearSolver<float>::invert(100, &mtx[0][0], &inv[0][0]);
		if (!ret) {
			printf("Singular Matrix\n");
			continue;
		}
		mtx *= inv;
		float error = 0.0f;
		for (i = 0; i < 100; i++)
			for (j = 0; j < 100; j++)
				error += (i==j) ? abs(1.0f - mtx[i][i]) : abs(mtx[i][j]);
		printf("Error: %f \n", error);
	}

	printf("Inverting 3x3 matrices...\n");
	Matrix3<float> mtx3, inv3;
	for (r = 0; r < 100; r++) {
		for (i = 0; i < 3; i++)
			for (j = 0; j < 3; j++)
				mtx3[i][j] = U();
		inv3 = mtx3.getInverse();
		mtx3 *= inv3;
		float error = 0.0f;
		for (i = 0; i < 3; i++)
			for (j = 0; j < 3; j++)
				error += (i==j) ? abs(1.0f - mtx3[i][i]) : abs(mtx3[i][j]);
		printf("Error: %f \n", error);
	}

	printf("Inverting 4x4 matrices...\n");
	Matrix4<float> mtx4, inv4;
	for (r = 0; r < 100; r++) {
		for (i = 0; i < 4; i++)
			for (j = 0; j < 4; j++)
				mtx4[i][j] = U();
		inv4 = mtx4.getInverse();
		mtx4 *= inv4;
		float error = 0.0f;
		for (i = 0; i < 4; i++)
			for (j = 0; j < 4; j++)
				error += (i==j) ? abs(1.0f - mtx4[i][i]) : abs(mtx4[i][j]);
		printf("Error: %f \n", error);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int main(void) {
	MatrixInvertTest();
	SphereTest();
	return 1;
}