/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics.h"
#include "gui.h"
#include "plots.h"

#include "graphics.inl"
#include "gui.inl"


namespace magnum { namespace sample {
using namespace magnum::application;
using namespace magnum::graphics;
using namespace magnum::graphics3d;
using namespace magnum::plots3d;
using namespace magnum::io;
using magnum::graphics3d::Context;

static PointLight				g_Light;
static Plot3d*					g_Plot;

enum Messages {
	FunctionPlot = 1,
	ParametricSurfacePlot = 2,
	ParametricPlot = 3,
};

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onInit() {
	getGui()->setTranslation(Vector3<float>(0.0f,0.0f,20.0f));
	getGui()->setMotionSpeed(0.1f);
	
	m_Camera.setNearPlane(2);
	m_Camera.setFarPlane(200);

	g_Light.setPosition(Vector3<float>(00.0f,0.0f,400.0f));
	g_Light.setAmbient(Colour<float>(0.0f,0.0f,0.0f));
	g_Light.setDiffuse(Colour<float>(1.0f,1.0f,1.0f));
	g_Light.setSpecular(Colour<float>(0.7f,0.7f,0.7f));

	g_Plot = createFunctionPlot();

	// Create GUI
	Button* button;
	button = new Button(Rectangle<float>(10,10,170,40), Message(this, ParametricPlot), "Parametric Plot");
	button->setFont(m_Gui->getFont());
	getGui()->insert(button);

	button = new Button(Rectangle<float>(10,60,170,40), Message(this, FunctionPlot), "Function Plot");
	button->setFont(m_Gui->getFont());
	getGui()->insert(button);

	button = new Button(Rectangle<float>(10,110,170,40), Message(this, ParametricSurfacePlot), "Parametric Srfc");
	button->setFont(m_Gui->getFont());
	getGui()->insert(button);

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onExit() {
	g_Light.deselect(0);
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onHandleMessage(const Message& msg) {
	switch (msg.getCommand()) {
		case FunctionPlot:
			g_Plot->release();
			g_Plot = createFunctionPlot();
			return true;
		case ParametricSurfacePlot:
			g_Plot->release();
			g_Plot = createParametricSurfacePlot();
			return true;
		case ParametricPlot:
			g_Plot->release();
			g_Plot = createParametricPlot();
			return true;
		default:
			return OpenGL::onHandleMessage(msg);
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GraphicsView::onRender() {
	DepthMode::selectDefault();
	BlendMode::selectDefault();

	m_Timer.update();
	m_Camera.select();
	m_Context->setClearColour(Colour<float>(0.59f, 0.59f, 1.0f));
	m_Context->clear(Context::ClearColour | Context::ClearDepth | Context::ClearStencil);

	m_MatrixStack->selectMode(MatrixStack::ModelView);
	m_MatrixStack->identity();

	g_Light.select(0);

	m_Context->setCulling(Context::CullNone);
	
	g_Plot->render((Matrix4<float>)getTransform().getInverse());

	m_Gui->process();
	m_Gui->paint();
}

}; };

