/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "plots.h"

namespace magnum { namespace sample {
using namespace magnum::plots3d;
using namespace magnum::math;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Function_SinCos : public Function<Vector2<float>,float> {
public:
	inline float operator()(const Vector2<float>& vec) const {
		return (float)(sin(vec.x)*cos(vec.y));
	};
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Function_Spiral : public Function<float,Vector3<float> > {
public:
	inline Vector3<float> operator()(float t) const {
		return Vector3<float>(9.0f*cos(t)*sin(3.0f*t),7.0f*cos(7.0f*t),5.0f*sin(5.0f*t));
	};
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
class Function_SinCosX2 : public Function<Vector2<float>,Vector3<float> > {
public:
	inline Vector3<float> operator()(const Vector2<float>& vec) const {
		return Vector3<float>(vec.y*sin(vec.x),vec.x*cos(vec.y),vec.x*vec.y);
	};
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Plot3d* createFunctionPlot() {
	FunctionPlot3d* plot = new FunctionPlot3d();
	plot->plot(Function_SinCos(),Vector2<float>(-3.0f,-3.0f),Vector2<float>(3.0f,3.0f));
	return plot;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Plot3d* createParametricPlot() {
	ParametricPlot3d* plot = new ParametricPlot3d();
	plot->setResolution(1000);
	plot->plot(Function_Spiral(),0.0f,6.5f);
	return plot;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Plot3d* createParametricSurfacePlot() {
	ParametricSurfacePlot3d* plot = new ParametricSurfacePlot3d();
	plot->plot(Function_SinCosX2(),Vector2<float>(-3.0f,-3.0f),Vector2<float>(3.0f,3.0f));
	return plot;
}

}; };

