/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics.h"
#include "gui.h"

#include "graphics.inl"
#include "gui.inl"


namespace magnum { namespace sample {
using namespace magnum::graphics;
using namespace magnum::graphics3d;
using namespace magnum::physics;
using magnum::graphics3d::Context;


static SpringModel*			g_Model;
static PointLight*			g_Light;
static ParameterBuffer*		g_Parameters;
static Relaxator<float,3>*	g_Springs;


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onInit() {
	m_Camera.setNearPlane(10);
	m_Camera.setFarPlane(16000);

	g_Light = new PointLight();
	g_Light->setPosition(Vector3<float>(0.0f,0.0f,400.0f));
	g_Light->setAmbient(Colour<float>(0.2f,0.2f,0.2f));
	g_Light->setDiffuse(Colour<float>(1.0f,1.0f,1.0f));
	g_Light->setSpecular(Colour<float>(0.3f,0.3f,0.3f));
	
	g_Model = new SpringModel(SpringModel::Rectangle,50,10.0f);
	g_Springs = g_Model->getSprings();
	(*g_Springs)[0].setFixed(true);
	(*g_Springs)[49].setFixed(true);
	(*g_Springs)[49*50].setFixed(true);
	(*g_Springs)[49*50 + 49].setFixed(true);
	g_Springs->setGravity(Vector3<float>(0.0f, 0.0f, -9.8f));
	g_Springs->setConstant(60.0f);
	g_Springs->setDamping(0.1f);

	g_Parameters = ParameterBuffer::create(g_Model->getParameters());

	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onExit() {
	g_Model->release();
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onHandleMessage(const Message& msg) {
	return OpenGL::onHandleMessage(msg);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GraphicsView::onRender() {
	DepthMode::selectDefault();
	BlendMode::selectDefault();

	m_Camera.select();
	m_Context->setClearColour(Colour<float>(0.59f, 0.59f, 1.0f));
	m_Context->clear(Context::ClearColour | Context::ClearDepth);

	m_MatrixStack->selectMode(MatrixStack::ModelView);
	m_MatrixStack->identity();

	g_Light->select(0);

	m_Context->setCulling(Context::CullNone);
	*(Matrix4<float>*)g_Parameters->getBuffer() = (Matrix4<float>)getTransform().getInverse();
	g_Model->update();
	g_Model->render(g_Parameters);
	g_Model->relax(1.0f/20.0f);
	
	float width = getBounds().getWidth();
	float height = getBounds().getHeight();
	GeometryRenderer::renderAxes(getTransform(), Vector2<float>(width - 32, height - 32));
}

}; };
